package ch.unizh.ifi.adora.constraints.compiler;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.ArrayList;

import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.osgi.framework.Bundle;

import ch.unizh.ifi.adora.constraints.ConstraintsPlugin;
import ch.unizh.ifi.adora.constraints.ui.preferences.PreferencesConstants;

/**
 * A wrapper to ease the use of com.sun.tools.javac.Main.
 * 
 * @author liyang
 */
public final class Javac {

	/**
	 * returns the location where the constraintplugin is installed
	 * 
	 * @return
	 */
	public static String getPath(String bundlename) {
		try {
			String s = getResource("/", bundlename);
			if (s == null) {
				s = "";
			}
			s = (!s.endsWith("/")) ? s += "/" : s;
			return s;
		} catch (Exception e) {
		}
		return "";
	}

	public static String getResource(String path, String bundlename) {
		try {
			Bundle bundle = Platform.getBundle(bundlename);
			return new Path(Platform.resolve(Platform.find(bundle, new Path(path))).getFile()).toFile().toString();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return null;
	}

	String bootclasspath;

	String classpath;

	String encoding;

	String extdirs;

	String outputdir;

	String sourcepath;

	String target;

	public Javac(String classpathin, String outputdir) {
		this.classpath = classpathin;
		String adoramodel = getPath("ch.unizh.ifi.adora.model");
		String util = getPath("ch.unizh.ifi.adora.icl");
		String pathsep = System.getProperty("path.separator");
		this.classpath = classpath.concat(pathsep.concat(adoramodel + "adoramodel.jar"));
		this.classpath = classpath.concat(pathsep.concat(util + "util.jar"));
		this.classpath = classpath.concat(pathsep.concat(outputdir));
		this.outputdir = outputdir;
		;

	}

	private String[] buildJavacArgs(String srcFiles[]) {
		ArrayList<String> args = new ArrayList<String>();

		if (classpath != null) {
			args.add("-classpath");
			args.add(classpath);
		}
		if (outputdir != null) {
			args.add("-d");
			args.add(outputdir);
		}
		if (sourcepath != null) {
			args.add("-sourcepath");
			args.add(sourcepath);
		}
		if (bootclasspath != null) {
			args.add("-bootclasspath");
			args.add(bootclasspath);
		}
		if (extdirs != null) {
			args.add("-extdirs");
			args.add(extdirs);
		}
		if (encoding != null) {
			args.add("-encoding");
			args.add(encoding);
		}
		if (target != null) {
			args.add("-target");
			args.add(target);
		}

		for (int i = 0; i < srcFiles.length; i++) {
			args.add(srcFiles[i]);
		}

		return args.toArray(new String[args.size()]);
	}

	public String compile(File srcFiles[]) {
		String paths[] = new String[srcFiles.length];

		for (int i = 0; i < paths.length; i++) {
			paths[i] = srcFiles[i].getAbsolutePath();
		}
		return compile(paths);
	}

	/**
	 * Compile the given source files.
	 * 
	 * @param srcFiles
	 * @return null if success; or compilation errors
	 */
	public String compile(String srcFiles[]) {

		StringWriter err = new StringWriter();
		PrintWriter errPrinter = new PrintWriter(err);

		String args[] = buildJavacArgs(srcFiles);
		int resultCode = 0;
		String temp = "";

		for (int i = 0; i < args.length; i++) {
			temp += " \"" + args[i] + "\"";
		}
		String file = ConstraintsPlugin.getDefault().getPreferenceStore().getString(PreferencesConstants.SRC_FILES);
		temp = "-warn:none " + temp;
		temp = "-log " + file + "/errors.txt " + temp;
		temp = "-proceedOnError " + temp;
		temp = "-source 1.5 " + temp; 

		org.eclipse.jdt.internal.compiler.batch.Main.compile(temp);
		errPrinter.close();
		return (resultCode == 0) ? null : err.toString();
	}

	public String getBootclasspath() {
		return bootclasspath;
	}

	public String getClasspath() {
		return classpath;
	}

	public String getEncoding() {
		return encoding;
	}

	public String getExtdirs() {
		return extdirs;
	}

	public String getOutputdir() {
		return outputdir;
	}

	public String getSourcepath() {
		return sourcepath;
	}

	public String getTarget() {
		return target;
	}

	public void setBootclasspath(String bootclasspath) {
		this.bootclasspath = bootclasspath;
	}

	public void setClasspath(String classpath) {
		this.classpath = classpath;
	}

	public void setEncoding(String encoding) {
		this.encoding = encoding;
	}

	public void setExtdirs(String extdirs) {
		this.extdirs = extdirs;
	}

	public void setOutputdir(String outputdir) {
		this.outputdir = outputdir;
	}

	public void setSourcepath(String sourcepath) {
		this.sourcepath = sourcepath;
	}

	public void setTarget(String target) {
		this.target = target;
	}

}
