package ch.unizh.ifi.adora.constraints.compiler;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.HashMap;

public class PackageClassLoader extends ClassLoader {
	private HashMap store = new HashMap();

	public PackageClassLoader(final ClassLoader parentClassLoader) {
		super(parentClassLoader);
	}

	public Class fastFindClass(final String name) {
		Class clazz = findLoadedClass(name);

		if (clazz == null) {
			final byte[] clazzBytes = read(name);
			if (clazzBytes != null) {
				java.lang.reflect.Method method = null;

				try {
					Class cls = Class.forName("java.lang.ClassLoader");
					method = cls.getDeclaredMethod("defineClass", new Class[] { String.class, byte[].class, int.class,
							int.class });

					// protected method invocaton
					method.setAccessible(true);
					Object[] args = new Object[] { name, clazzBytes, new Integer(0), new Integer(clazzBytes.length) };
					clazz = (Class) method.invoke(this.getParent(), args);
				} catch (Exception e) {
					// not possible to load the class
				} finally {
					method.setAccessible(false);
				}
			}
		}

		return clazz;
	}

	@Override
	protected Class findClass(final String name) throws ClassNotFoundException {
		final Class clazz = fastFindClass(name);
		if (clazz == null) {
			throw new ClassNotFoundException(name);
		}
		return clazz;
	}

	@Override
	public InputStream getResourceAsStream(final String name) {
		final byte[] bytes = (byte[]) store.get(name);
		if (bytes != null) {
			return new ByteArrayInputStream(bytes);
		} else {
			return super.getResourceAsStream(name);
		}
	}

	/**
	 * Javadocs recommend that this method not be overloaded. We overload this
	 * so that we can prioritise the fastFindClass over method calls to
	 * parent.loadClass(name, false); and c = findBootstrapClass0(name); which
	 * the default implementation would first - hence why we call it
	 * "fastFindClass" instead of standard findClass, this indicates that we
	 * give it a higher priority than normal.
	 * 
	 */
	@Override
	protected synchronized Class loadClass(final String name, final boolean resolve) throws ClassNotFoundException {
		Class clazz = fastFindClass(name);

		if (clazz == null) {
			final ClassLoader parent = getParent();
			if (parent != null) {
				clazz = parent.loadClass(name);
			} else {
				throw new ClassNotFoundException(name);
			}
		}

		if (resolve) {
			resolveClass(clazz);
		}

		return clazz;
	}

	public byte[] read(final String resourceName) {
		final byte[] bytes = null;

		if (this.store != null) {
			return (byte[]) this.store.get(resourceName.replace('.', '/') + ".class");
		}
		return bytes;
	}

	public void write(final String resourceName, final byte[] clazzData) throws RuntimeException {
		this.store.put(resourceName.replace('.', '/') + ".class", clazzData);
	}
}