package ch.unizh.ifi.adora.constraints.constraints.util;

import java.util.Enumeration;
import java.util.NoSuchElementException;
import java.util.Vector;
import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;


/**
 * 	THIS IS NEW
 * This class is a helper class which implements the {@link java.util.Enumeration}
 * interface. It provides the possibility to retrieve a set sub elements which
 * belong to a to the same type of class. Therefore, the user has to specify
 * a type of class when retrieving the set of elements.
 *
 * @author		Silvio Meier
 * @copyright   Requirements Engineering Research Group
 *              Department of Computer Science
 *              University of Zurich
 * @history
 *   2003-12-02 SM First Version
 *   2004-01-05 SM Corrected a pre condition violation.
 *   2004-01-09 CS overtaken from the adorafsl project to adora
 *   2004-01-10 JK changed internal representation from SimpleHierarchyNode[] to
 *                 Object[]. Added Constructors for Enumerations..
 *   2004-01-10 JK taken back from project layoutalgorithm to ibadora :)
 *
 * @version $Date: 2008-10-27 10:12:16 +0100 (Mon, 27 Oct 2008) $, $Author: jehle $, $Revision: 3853 $
 * @responsibilities Provides a class with the functionality of a
 *                   {@link java.util.Enumeration} which can be used to retrieve
 *                   a set of sub nodes of the same type.
 */
public class TypeEnumerator implements Enumeration {

    /**
     * The array of Objects over which will be enumerated
     */
    protected Object[] container;

    /**
     * Current index points to the current object. The object's type is specifyed
     * by the variable {@link #instanceType}.
     *
     */
    protected int index;

    /**
     * Describes the type which is used to filter the objects
     * in the given set of objects (see {@link #container}).
     */
    protected Class instanceType;


    /**
     * Constructor creates a new enumerator object which does not filter objects
     * belonging to a specific type, i.e. all objects within the array
     * {@link #container}.
     *
     * @pre true
     * @post true
     * @param container Array of {@link java.lang.Object} objects which are returned
     *        by this enumerator object.
     */
    public TypeEnumerator(Object[] container) {
        this.container = container;
        this.instanceType = Object.class;
        index = 0;
    }
    
   


    /**
     * Constructor creates a new enumerator object which does not filter objects
     * belonging to a specific type, i.e. all objects within the array
     * {@link #container}.
     *
     * @pre true
     * @post true
     * @param container Vector of {@link Object} objects which are returned
     *        by this enumerator object.
     */
    public TypeEnumerator(Vector container) {
        this.container = container.toArray();
        this.instanceType = Object.class;
        index = 0;
    }
    
    public TypeEnumerator(List container) {
        this.container = container.toArray();
        this.instanceType = Object.class;
        index = 0;
    }
    
    
    
     /**
     * Constructor creates a new enumerator object which does not filter objects
     * belonging to a specific type, i.e. all objects within the array
     * {@link #container}.
     *
     * @pre true
     * @post true
     * @param container Iterator of {@link Object} objects which are returned
     *        by this enumerator object.
     */
    public TypeEnumerator(Iterator container) {
    	Vector v=new Vector();
    	while(container.hasNext()){
    		v.add(container.next());
    	}
        this.container = v.toArray();
        this.instanceType = Object.class;
        index = 0;
    }
    
    /**
     * Constructor creates a new enumerator object which does not filter objects
     * belonging to a specific type, i.e. all objects within the array
     * {@link #container}.
     *
     * @pre true
     * @post true
     * @param container Enumerator of {@link Object} objects which are returned
     *        by this enumerator object.
     */
    public TypeEnumerator(Enumeration container) {
        Vector v = new Vector();
        while (container.hasMoreElements()) {
            v.addElement(container.nextElement());
        }
        this.container = v.toArray();
        this.instanceType = Object.class;
        index = 0;
    }


    /**
     * Constructor creates a new enumerator object which filters objects
     * belonging to a specific type, i.e. all objects within the array
     * {@link #container} that belong to instance of the parameter <i>instanceType</i>.
     *
     * @pre container != null && instanceType != null
     * @post true
     * @param container Array of {@link Object} objects which are returned
     *        by this enumerator object.
     * @param instanceType {@link java.lang.Class} object which represents
     *        the type which will be used to filter the set of given nodes in
     *        the {@link #container}.
     */
    public TypeEnumerator(Object[] container, Class instanceType) {
        this(container);
        this.instanceType = instanceType;
    }
    
    
   
     /**
     * Constructor creates a new enumerator object which does not filter objects
     * belonging to a specific type, i.e. all objects within the array
     * {@link #container}.
     *
     * @pre true
     * @post true
     * @param container Iterator of {@link Object} objects which are returned
     *        by this enumerator object.
     */
    public TypeEnumerator(ArrayList container) {
        this.container = container.toArray();
        this.instanceType = Object.class;
        index = 0;
    }


    /**
     * Constructor creates a new enumerator object which filters objects
     * belonging to a specific type, i.e. all objects within the array
     * {@link #container} that belong to instance of the parameter <i>instanceType</i>.
     *
     * @pre container != null && instanceType != null
     * @post true
     * @param container Vector of {@link Object} objects which are returned
     *        by this enumerator object.
     * @param instanceType {@link java.lang.Class} object which represents
     *        the type which will be used to filter the set of given nodes in
     *        the {@link #container}.
     */
    public TypeEnumerator(Vector container, Class instanceType) {
        this(container);
        this.instanceType = instanceType;
    }
    
    public TypeEnumerator(List container, Class instanceType) {
        this(container);
        this.instanceType = instanceType;
    }
    
    /**
     * Constructor creates a new enumerator object which filters objects
     * belonging to a specific type, i.e. all objects within the array
     * {@link #container} that belong to instance of the parameter <i>instanceType</i>.
     *
     * @pre container != null && instanceType != null
     * @post true
     * @param container Enumartion of {@link Object} objects which are returned
     *        by this enumerator object.
     * @param instanceType {@link java.lang.Class} object which represents
     *        the type which will be used to filter the set of given nodes in
     *        the {@link #container}.
     */
    public TypeEnumerator(Enumeration container, Class instanceType) {
        this(container);
        this.instanceType = instanceType;
    }
  
  
  
   public TypeEnumerator(Iterator container, Class instanceType) {
        this(container);
        this.instanceType = instanceType;
    }
    
     public TypeEnumerator(ArrayList container, Class instanceType) {
        this(container);
        this.instanceType = instanceType;
    }


    /**
     * This method returns true, if there are further objects in
     * {@link #container} of the type {@link #instanceType}, otherwise
     * the method returns false.
     *
     * @pre true
     * @post true
     * @return Returns true if there
     */
    public boolean hasMoreElements() {
        boolean result = false;
        int findIndex = index;

        if (container == null) {
            return false;
        }

        while ((findIndex < container.length) &&
                !(instanceType.isInstance(container[findIndex]))) {
            findIndex++;
        }

        if ((findIndex < container.length) && (instanceType.isInstance(container[findIndex]))) {
            result = true;
        }
        return result;
    }


    /**
     * Returns the next element in the set of nodes which is of the type
     * {@link #instanceType}. If there is no element left which is of
     * the given type, the method throws a {@link java.util.NoSuchElementException}.
     *
     * @pre true
     * @post return != null
     * @return Returns the next object from the set of nodes. The returned object
     *         will be of a sub type of the class {@link hierarchy.SimpleHierarchyNode}.
     * @throws java.util.NoSuchElementException Exception which will be
     *         thrown, if no element is left. To avoid this exception call
     *         the method {@link #hasMoreElements()} first.
     */
    public Object nextElement() {
        try {
            while ((index < container.length) &&
                    !(instanceType.isInstance(container[index]))) {
                index++;
            }
        } catch (Exception ex) {
            throw new NoSuchElementException("No element found");
        }
        if (index == container.length) {
            throw new NoSuchElementException("No element found");
        }
        Object retval = container[index];
        index++;
        return retval;
    }
}
