package ch.unizh.ifi.adora.constraints.editor;

import icl.compiler.java.CompilerCaller;
import icl.errors.ErrorWarningManager;

import java.util.Enumeration;
import java.util.Properties;

import ch.unizh.ifi.adora.constraints.ConstraintsPlugin;
import ch.unizh.ifi.adora.constraints.compiler.DynamicCodeLoader;
import ch.unizh.ifi.adora.constraints.ui.preferences.PreferencesConstants;

/**
 * contains all data needed for ILC-Compilation and offers methods to compile
 * strings or files
 * 
 * @author Christina Cramer
 * 
 */
public class CompileManager {

	private static final String CONSTRAINT_LIB = "ch.unizh.ifi.adora.constraints.constraints.util";

	private static final String CONSTRAINT_PKG = "ch.unizh.ifi.adora.constraints.constraints";

	/**
	 * properties for the compilation
	 */
	private Properties compProps;

	/**
	 * location where constraints will be saved
	 */
	private String CONSTRAINT_SRC = ConstraintsPlugin.getDefault().getPreferenceStore().getString(
			PreferencesConstants.SRC_FILES);

	private Properties genProps;

	/**
	 * calls the ICL-Compilation
	 */
	private CompilerCaller iclCompiler;

	/**
	 * constructor
	 * 
	 */
	public CompileManager() {
		initProperties();
	}

	/**
	 * compiles a constraint that is given as a string
	 * 
	 * @param constraintstring
	 * @return null if it was possible to compile the constraint, Enumeration
	 *         containing the errors (String) if compilation was not successful
	 */
	public Enumeration compile(String constraintstring) {
		iclCompiler = new CompilerCaller(genProps);
		compProps.setProperty(CompilerCaller.ICL_INPUT_STRING_KEY_PREFIX, constraintstring);
		try {
			iclCompiler.iclStringCompile(compProps);
		} catch (Exception e) {
			//e.printStackTrace();
			ErrorWarningManager errorWarnings = iclCompiler.getErrorsAndWarnings();
			Enumeration errors = errorWarnings.getErrors();
			return errors;
		}

		return null;
	}

	/**
	 * initializes the properties needed for (ICL-) Compilation
	 */
	public void initProperties() {
		genProps = new Properties();
		compProps = new Properties();
		compProps.setProperty(CompilerCaller.ICL_DESTINATION_DIR_KEY, CONSTRAINT_SRC);
		compProps.setProperty(CompilerCaller.ICL_JAVA_COMPILE_CLASSPATH_KEY, DynamicCodeLoader.extractClasspath(Thread
				.currentThread().getContextClassLoader()));
		compProps.setProperty(CompilerCaller.ICL_COMPILE_STRINGS_KEY, "true");
		genProps.setProperty(CompilerCaller.ICL_LIBRARY_PACKAGE_KEY, CONSTRAINT_LIB);
		genProps.setProperty(CompilerCaller.ICL_CONSTRAINTS_IMPORT_KEY, "ch.unizh.ifi.adora.model.representation.*:java.util.*");
		genProps.setProperty(CompilerCaller.ICL_CONSTRAINT_PACKAGE_KEY, CONSTRAINT_PKG);
		genProps.setProperty(CompilerCaller.ICL_QUIET_COMPILATION_KEY, "iclQuietCompile");

	}

}
