package ch.unizh.ifi.adora.constraints.editor;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.operation.IRunnableContext;
import org.eclipse.jface.text.Document;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.source.AnnotationModel;
import org.eclipse.jface.text.source.IAnnotationModel;
import org.eclipse.ui.texteditor.AbstractDocumentProvider;

import ch.unizh.ifi.adora.model.properties.CommentableElementPropNames;

/**
 * This class provides a ConstraintDocument. The special responsibilities of
 * this object, is to create a new document,
 * 
 * 
 * @author ccramer
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 06.03.2006 SM First Version (as CommentDocumentProvider) 27.01.2007
 *          CR Changed to ConstraintDocumentProvider
 * @version $Date: 2008-01-24 10:07:03 +0100 (Thu, 24 Jan 2008) $, $Author: cramer $, $Revision: 3493 $
 */

public class ConstraintDocumentProvider extends AbstractDocumentProvider implements CommentableElementPropNames {

	/**
	 * PropertyChangeSupport used to update the component model's Comment field.
	 */
	private PropertyChangeSupport pcsDelegate;

	/**
	 * Instantiates a CommentDialogCellEditor.
	 */
	public ConstraintDocumentProvider() {
		pcsDelegate = new PropertyChangeSupport(this);
	}

	/**
	 * Adds a PropertyChangeListener. This Listener is used to update the
	 * component model's Comment field. Since the edit part doesn't know this
	 * object, the properties are fired in a chain.
	 */
	public synchronized void addPropertyChangeListener(PropertyChangeListener l) {
		pcsDelegate.addPropertyChangeListener(l);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.texteditor.AbstractDocumentProvider#getOperationRunner(org.eclipse.core.runtime.IProgressMonitor)
	 */
	@Override
	protected IAnnotationModel createAnnotationModel(Object element) throws CoreException {
		return new AnnotationModel();
	}

	/**
	 * Creates a new document containing the Comment stored in the model.
	 */
	@Override
	protected IDocument createDocument(Object element) throws CoreException {
		Document document = new Document();
		return document;

	}

	/**
	 * 
	 */
	@Override
	protected void doSaveDocument(IProgressMonitor monitor, Object element, IDocument document, boolean overwrite)
			throws CoreException {
		pcsDelegate.firePropertyChange(COMMENT_PROP, null, document.get());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.texteditor.AbstractDocumentProvider#getOperationRunner(org.eclipse.core.runtime.IProgressMonitor)
	 */
	@Override
	protected IRunnableContext getOperationRunner(IProgressMonitor monitor) {
	
		return null;
	}

	
	
	/**
	 * Document is editable.
	 * 
	 * @return
	 * <li>true</li>;
	 */
	@Override
	public boolean isModifiable(Object element) {
		return true;
	}

	/**
	 * Document is editable.
	 * 
	 * @return
	 * <li>false</li>;
	 */
	@Override
	public boolean isReadOnly(Object element) {
		return false;
	}
}