package ch.unizh.ifi.adora.constraints.editor;

import java.util.Enumeration;
import java.util.Vector;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.text.Assert;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IPersistableElement;

/**
 * This ConstraintEditorInput is a light weight descriptor of editor input, like
 * a file name but more abstract. It is not a model. It is a description of the
 * model source for an IEditorPart.
 * 
 * @author Christina Cramer
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 06.03.2006 SM First Version (as CommentEditorInput) 24.01.2007 CR
 *          changed to needs of ConstraintEditorInput
 * @version $Date: 2007-10-11 10:49:01 +0200 (Thu, 11 Oct 2007) $, $Author: cramer $, $Revision: 3123 $
 */
public class ConstraintEditorInput implements IEditorInput {

	/**
	 * Stores the metaData of this editor.
	 */
	private EditorMetaData myMetaData;

	/**
	 * Creates a test editor input on the given myMetaData.
	 */
	public ConstraintEditorInput(EditorMetaData myMetaData) {
		Assert.isNotNull(myMetaData);
		this.myMetaData = myMetaData;
	}

	/**
	 * Answers true for two inputs that are the same. The
	 * IWorbenchPage.openEditor APIs are dependent on this to find an editor
	 * with the same input.
	 */
	@Override
	public boolean equals(Object obj) {
		if (!(obj instanceof ConstraintEditorInput)) {
			return false;
		}
		ConstraintEditorInput other = (ConstraintEditorInput) obj;
		return this.myMetaData.equals(other.myMetaData);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IEditorInput#exists()
	 */
	public boolean exists() {
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	public Object getAdapter(Class adapter) {
		return null;
	}

	public Vector getErrors() {
		return myMetaData.getErrors();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IEditorInput#getImageDescriptor()
	 */
	public ImageDescriptor getImageDescriptor() {
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IEditorInput#getName()
	 */
	public String getName() {
		return myMetaData.getArtifactName();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IEditorInput#getPersistable()
	 */
	public IPersistableElement getPersistable() {
		// Not persistable between sessions
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IEditorInput#getToolTipText()
	 */
	public String getToolTipText() {
		return "";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		return myMetaData.hashCode();
	}

	/**
	 * checks if the input is the same
	 * 
	 * @param other
	 *            ConstraintEditorInput to compare to
	 * @return true if it is the same
	 */
	public boolean sameConstraint(ConstraintEditorInput other) {
		return (other.myMetaData.getArtifactName().equals(this.myMetaData.getArtifactName()));
	}

	public void setErrors(Enumeration errors) {
		this.myMetaData.setErrors(errors);
	}

	public void setName(String s) {
		this.myMetaData.setName(s);
	}
}