package ch.unizh.ifi.adora.constraints.editor;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.text.IDocument;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.editors.text.TextEditor;





/**
 * This editor acts as the standard editor for component Constraints
 * 
 * 
 * @author Christina Cramer
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 06.03.2006 SM First Version (as CommentTextEditor) 27.01.2007 CR
 *          changed to ConstraintTextEditor
 * @version $Date: 2009-05-06 10:51:39 +0200 (Wed, 06 May 2009) $, $Author: treinhard $, $Revision: 4182 $
 */

public class ConstraintTextEditor extends TextEditor implements PropertyChangeListener {

	
	/**
	 * ID, used for detecting the help of this {@link TextEditor} when only
	 * knowing the {@link org.eclipse.ui.IWorkbench}.
	 */
	public static final String HELPID = "ch.unizh.ifi.adora.constraints.editor.ConstraintTextEditorHelp";

	/**
	 * ID, used for detecting this {@link TextEditor} when only knowing the
	 * {@link org.eclipse.ui.IWorkbench}. needs to be specified in plugin.xml
	 */
	public static final String ID = "ch.unizh.ifi.adora.constraints.editor.ConstraintTextEditor";

	public ch.unizh.ifi.adora.constraints.editor.EditorManager editormanager;

	/**
	 * input of the ConstraintTextEditor
	 */
	private ConstraintEditorInput input;

	/**
	 * PropertyChangeSupport used to update the component model's Comment field.
	 */
	private PropertyChangeSupport pcsDelegate;

	/**
	 * Instantiates a CommentTextEditor.
	 */
	public ConstraintTextEditor() {
		super();
		setSourceViewerConfiguration(new ConstraintViewerConfig());
		setDocumentProvider(new ConstraintDocumentProvider());
		pcsDelegate = new PropertyChangeSupport(this);
		
	}
	
public ConstraintViewerConfig getConstraintViewer(){
	return (ConstraintViewerConfig)this.getSourceViewerConfiguration();
}
	
	
	
	public void addParams(String param, String cls){
		((ConstraintsCompletionProcessor)this.getSourceViewerConfiguration().getContentAssistant(this.getSourceViewer()).getContentAssistProcessor(IDocument.DEFAULT_CONTENT_TYPE)).addParams(param,cls);
	}

	/**
	 * adds one EditorManager to the ConstraintTextEditor the EditorManager will
	 * have to be used to save a constraint
	 * 
	 * @param editorManager2
	 */
	public void addEditorManager(ch.unizh.ifi.adora.constraints.editor.EditorManager editorManager2) {
		editormanager = editorManager2;
	}

	/**
	 * Adds a PropertyChangeListener. This Listener is used to update the
	 * component model's Constraint field. Since the edit part doesn't know this
	 * object, the properties are fired in a chain.
	 */
	public synchronized void addPropertyChangeListener(PropertyChangeListener l) {
		pcsDelegate.addPropertyChangeListener(l);
	}

	@Override
	protected void createActions() {

		super.createActions();
	}

	/**
	 * will be called when TextEditor is closed. inform manager about closing of
	 * TextEditor
	 */
	@Override
	public void dispose() {
		editormanager.disposeEditor(this);
		super.dispose();
	}

	/*
	 * overwrite save so the the specific saving action for constraints
	 * (=writing it to a file with filename=constraintname) is done
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.texteditor.AbstractTextEditor#doSave(org.eclipse.core.runtime.IProgressMonitor)
	 */
	@Override
	public void doSave(IProgressMonitor monitor) {
		if (!editormanager.saveConstraint()) {
			monitor.setCanceled(true);
		} else {
			super.doSave(monitor);
		}
	}

	/**
	 * returns the input (description of source) of the constraintTextEditor
	 * 
	 * @return input
	 */
	public ConstraintEditorInput getInput() {
		return input;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IEditorPart#init(org.eclipse.ui.IEditorSite,
	 *      org.eclipse.ui.IEditorInput)
	 */
	@Override
	public void init(IEditorSite site, IEditorInput input) throws PartInitException {
		setSite(site);
		setInput(input);
		this.input = (ConstraintEditorInput) input;
		

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.texteditor.AbstractDecoratedTextEditor#initializeEditor()
	 */
	@Override
	protected void initializeEditor() {
		super.initializeEditor();
		ConstraintDocumentProvider txtProv = new ConstraintDocumentProvider();
		txtProv.addPropertyChangeListener(this);
		setDocumentProvider(txtProv);
		
	}

	/**
	 * save as is not allowed
	 */
	@Override
	public boolean isSaveAsAllowed() {
		return false;
	}

	/**
	 * Fires a propertyChange message to notify the ConstraintDialogCellEditor
	 * object of the new Comment.
	 */
	public void propertyChange(PropertyChangeEvent arg0) {
		pcsDelegate.firePropertyChange(arg0);
	}

	@Override
	public void setPartName(String name) {
		super.setPartName(name);

	}

}
