package ch.unizh.ifi.adora.constraints.editor;

import java.io.IOException;
import java.lang.reflect.Method;
import java.util.Hashtable;
import java.util.StringTokenizer;
import java.util.Vector;

import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.ITextViewer;
import org.eclipse.jface.text.contentassist.CompletionProposal;
import org.eclipse.jface.text.contentassist.ICompletionProposal;
import org.eclipse.jface.text.contentassist.IContentAssistProcessor;
import org.eclipse.jface.text.contentassist.IContextInformation;
import org.eclipse.jface.text.contentassist.IContextInformationValidator;
import org.osgi.framework.Bundle;

/**
 * Help to write Constraint with AutoCompletion
 * 
 * @author Christina Cramer
 * 
 */
public class ConstraintsCompletionProcessor implements IContentAssistProcessor {

	/**
	 * saves the class of each parameter
	 */
	public Hashtable<String, String> parameters = new Hashtable<String, String>();

	/**
	 * constraint keywords for auto completion
	 */
	public String[] keywords = new String[] { "constraint", "on", "is",
			"for-all", "there-exists", "ifTrue", "ifFalse", "ifError" };
	/**
	 * Base Classes to be used to write constraints. 
	 */
	String[] classes = new String[] { "Component", "State", "Actor",
			"Scenario", "AspectContainer", "Association","Transition","Node","StartStateContainer"};

	/**
	 * all Classes that can be used in Constraints
	 */
	Vector<Class> allClasses;
	/**
	 * package where the classes are found.
	 */
	private static final String classLocation = "ch.unizh.ifi.adora.model.representation";

	/**
	 * Last cause of computing CompletionProposals. 
	 */
	private char last = ' ';
	
	/**
	 * Methods for a class. 
	 */
	private Hashtable<Class, Vector<Method>> methodsForClass=new Hashtable<Class, Vector<Method>>();
	


	public ConstraintsCompletionProcessor() {
	}

	/*
	 * (non-Javadoc) Method declared on IContentAssistProcessor Compution of the
	 * Proposals list depending on last character that was typed
	 */
	public ICompletionProposal[] computeCompletionProposals(ITextViewer viewer,
			int documentOffset) {
		IDocument doc=viewer.getDocument();
		try {
			char cause = doc.getChar(documentOffset - 1);
			if (cause == '.' || cause == ':' || cause == ' ') {
				last = cause;
			}
			if (cause == '(') {
				doc.replace(documentOffset, 0, ")");
			} else if (last == ':') {
				String word = getWord(doc, documentOffset);
				return this.findClasses(word, documentOffset);
			} else {
		
				String testword = getWholeString(doc, documentOffset + 1);
				if ((testword.indexOf('.') != -1)) {
					return getMethodList(doc, documentOffset);

				} else if (!testword.equals("")&&!(testword.trim().length()==0)) {// no dots on name --> could							// be constraint keyword!
					testword=cleancls(testword);
					Vector<String> words = new Vector<String>();
					for (String s : keywords) {
						if (s.startsWith(testword)) {
							words.add(s);
						}
					}
					ICompletionProposal[] result = new ICompletionProposal[words
							.size()];

					for (int i = 0; i < words.size(); i++) {
						String toshow = (String) words.elementAt(i);
						result[i] = new CompletionProposal(toshow,
								documentOffset - testword.length(), testword
										.length(), toshow.length(), null,
								toshow, null, null);
					}
					return result;

				}

			}
		} catch (BadLocationException e) {
		}
		return null;

	}

	
	/**
	 * Return the list of possible Methods that fit at the given location. 
	 * It is assumed, that the first element of a string separated with dots (.) is 
	 * the parameter. If the class of the parameter is not known yet, a search for it is done 
	 * and the found class is added to the parameters Hashtable. 
	 * @param doc
	 * @param documentOffset
	 * @return
	 */
	private ICompletionProposal[] getMethodList(IDocument doc,
			int documentOffset) {
		Class c;
		String word = "";
		String[] wordList = getWordList(doc, documentOffset);
		String cls = wordList[0];
		cls = cleancls(cls);
		int length = cls.length() + 1;

		if (parameters.get(cls) == null) {
			int index = doc.get().indexOf(cls + ":");
			if (index == -1) {
				index = doc.get().indexOf(cls + " :");
				length = length + 1;
			}
			String nextword = getNextWord(doc, index + length);
			parameters.put(cls, nextword);
		}

		try {
			c = Class.forName(classLocation+"."+ parameters.get(cls));
			if (wordList.length == 1) {
				word = getWord(doc, documentOffset);

				return this.getProposalList(c, word, documentOffset);
			} else {
				for (int i = 1; i < wordList.length; i++) {
					String method = wordList[i];
					if (method.startsWith("asType(")) {
						if (method.endsWith(")")) {
							String clsString = method.substring(method
									.indexOf('(') + 1, method.indexOf(')'));
							c = Class.forName(classLocation+"."+ clsString);
						}
					}
								//c = Class.forName(classLocation+"."+ clsString);
								Method[] methods = c.getMethods();
								
								for (Method m : methods) {
									if (method.startsWith(m.getName())) {
										c = m.getReturnType();
									}
								}
							
						
					
				}
				word = getWord(doc, documentOffset);
				return this.getProposalList(c, word, documentOffset);
			}
		} catch (ClassNotFoundException e1) {
			return null;
		}

	}

	/**
	 * remove all non-letter characters from the string. 
	 * @param cls
	 * @return
	 */
	private String cleancls(String cls) {
		String word = "";
		char[] chars = cls.toCharArray();
		for (int i = 0; i < chars.length; i++) {
			char c = chars[i];
			if (java.lang.Character.isLetter(c)) {
				word += c;
			}
		}
		return word;
	}

	
	/**
	 * Returns the word right after the the cursor. 
	 * @param document
	 * @param documentOffset
	 * @return
	 */
	private String getNextWord(IDocument document, int documentOffset) {
		int offset = documentOffset;
		String word = "";
		try {
			while (java.lang.Character.isSpaceChar(document.getChar(offset))) {
				offset++;
			}
			while (java.lang.Character.isJavaIdentifierStart(document
					.getChar(offset))) {
				word += document.getChar(offset);
				offset++;
			}
		} catch (BadLocationException e) {
			// TODO Auto-generated catch block
			return word;
		}
		return word;
	}

	
	/**
	 * Get the whole String before the cursor and split it into tokens separated by the dots. 
	 * @param doc
	 * @param documentOffset
	 * @return
	 */
	private String[] getWordList(IDocument doc, int documentOffset) {
		int offset = documentOffset;
		String wholeWord = this.getWholeString(doc, offset);
		StringTokenizer tokenizer = new StringTokenizer(wholeWord, ".");
		String[] words = new String[tokenizer.countTokens()];
		int i = 0;
		while (tokenizer.hasMoreTokens()) {
			words[i] = tokenizer.nextToken();
			i++;
		}
		return words;
	}

	
	
	
	/**
	 * Get the last String typed. 
	 * @param doc
	 * @param documentOffset
	 * @return
	 */
	private String getWholeString(IDocument doc, int documentOffset) {
		String word = "";
		int offset = documentOffset - 1;
		char actualchar;
		try {
			actualchar = doc.getChar(offset);
			while(java.lang.Character.isSpaceChar(actualchar)){
				offset--;
				actualchar=doc.getChar(offset);
			}
			while (!java.lang.Character.isSpaceChar(actualchar)) {
				word = actualchar + word;
				offset--;
				actualchar = doc.getChar(offset);

			}
		} catch (BadLocationException e) {
			return "";
		}
		return word;
	}

	

	/**
	 * Finds all classes that can be used at this place. 
	 * @param word already typen start of word. 
	 * @param pos current position of cursor. 
	 * @return
	 */
	public ICompletionProposal[] findClasses(String word, int pos) {
		Vector<Class> myClasses = getClasses();
		//Class[] myClasses=getClasses();
		Vector<String> resultClasses=new Vector<String>();
	

		for (int i = 0; i < myClasses.size(); i++) {
			
			String toshow = (String) myClasses.elementAt(i).getSimpleName();
			if(toshow.startsWith(word)){
				resultClasses.add(toshow);
			}
		}
		ICompletionProposal[] result = new ICompletionProposal[resultClasses.size()];
		for(int i=0;i<resultClasses.size();i++){
			String toshow=resultClasses.elementAt(i);
			result[i] = new CompletionProposal(toshow, pos - word.length(), word.length(),toshow.length(), null, toshow, null, null);}
		
		return result;
	}

	
	/*
	 * (non-Javadoc) Method declared on IContentAssistProcessor
	 */
	public IContextInformation[] computeContextInformation(ITextViewer viewer,
			int documentOffset) {
		
		return null;
	}

	/**
	 * The chars that cause the completionProposal to become active. 
	 */
	String activationChars = " .(:abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";

	/**
	 * Returns the characters that shall cause the completionProposal to get active. 
	 */
	public char[] getCompletionProposalAutoActivationCharacters() {
		return activationChars.toCharArray();

	}


	/*
	 * (non-Javadoc) Method declared on IContentAssistProcessor
	 */
	public char[] getContextInformationAutoActivationCharacters() {
		return null;

	}

	// For Context information
	/*
	 * (non-Javadoc) Method declared on IContentAssistProcessor
	 */
	public IContextInformationValidator getContextInformationValidator() {
		return null;
	}

	/*
	 * (non-Javadoc) Method declared on IContentAssistProcessor
	 */
	public String getErrorMessage() {
		return null;
	}

	
	
	/**
	 * Design the CompletionProposals of all Methods of that class starting with word. 
	 * @param cls
	 * @param word
	 * @param pos
	 * @return
	 */
	public ICompletionProposal[] getProposalList(Class cls, String word, int pos) {
		Vector<Method>	myInputs = getMethods(cls);
		Vector<String> outputs = new Vector<String>();
		Vector<String> inserts = new Vector<String>();
		for (Method m : myInputs) {
			if (m.getName().startsWith(word)||word.equals("")) {
				String element = m.getName() + "(";
				String insertelement = m.getName() + "(";
				Class<?>[] params = m.getParameterTypes();
				for (int i = 0; i < params.length; i++) {
					Class c = params[i];
					element += c.getSimpleName();
					insertelement += c.getSimpleName();
					if (i < params.length - 1) {
						element += ",";
						insertelement += ",";
					}
				}
				element += ")";
				element += " " + m.getReturnType().getSimpleName() + " ";
				insertelement += ")";
				outputs.add(element);
				inserts.add(insertelement);
		}
		}
		ICompletionProposal[] result = new ICompletionProposal[outputs.size()];

		for (int i = 0; i < outputs.size(); i++) {
			String toshow = outputs.elementAt(i);
			String insert = inserts.elementAt(i);
			result[i] = new CompletionProposal(insert, pos - word.length(),
					word.length(), insert.length(), null, toshow, null, null);
		}
		return result;
	}

	/**
	 * Get all methods of a given class. If no methods for this class are saved in
	 * methodsForClass, it will be saved there. 
	 * @param cls
	 * @return
	 */
	private Vector<Method> getMethods(Class cls) {
		if(methodsForClass.get(cls)!=null){
			return methodsForClass.get(cls);
		}
		Vector<Method> inputs = new Vector<Method>();
		Method[] methods = cls.getMethods();
		for (Method m : methods) {
			inputs.add(m);
		}
		methodsForClass.put(cls, inputs);
		return inputs;
	}

	
	/**
	 * Returns all classed that can be used in constraints. 
	 * First time retrieving all Superclasses of classes. 
	 * TODO add automatic finding of classes in adoramodel. 
	 * @return
	 */
	private Vector<Class> getClasses() {
		try {
			
		
			if(allClasses==null){
			allClasses=new Vector();
		//Vector<Class> classesVector = new Vector<Class>();
			Class c;
		
			for (String cls : classes) {
				c = Class.forName(classLocation + "." + cls);
				allClasses.add(c);
				//c=c.getSuperclass();
				//while((c!=null)&&isConstraintCheckable(c)){			
					//if(!allClasses.contains(c)){
						//allClasses.add(c);				}
					//c=c.getSuperclass();
				//}
			
			}

		}
	
		} catch (ClassNotFoundException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		
		return allClasses;
	}

	
	private boolean isConstraintCheckable(Class c) {
		if(c.getSimpleName().equals("VisualizableModelElement")){
			return true;
		}
		if(c.getSimpleName().equals("Object")){return false;}
		return isConstraintCheckable(c.getSuperclass());
	}

	/**
	 * Get word before cursor position
	 * @param doc
	 * @param pos
	 * @return
	 */
	public String getWord(IDocument doc, int pos) {
		String word = "";
		int offset= pos-1;
		try {
			char actualchar=doc.getChar(offset);
			while(Character.isLetter(actualchar)){
				word = actualchar + word;
				offset--;
				actualchar=doc.getChar(offset);
			}
		} catch (BadLocationException e) {
			return word;
		}
		return word;
	}

	public void addParams(String param, String cls) {
		// TODO Auto-generated method stub
		
	}

	public static String getPath(String bundlename) {
		try {
			String s = getResource("/", bundlename);
			if (s == null) {
				s = "";
			}
			s = (!s.endsWith("/")) ? s += "/" : s;
			return s;
		} catch (Exception e) {
		}
		return "";
	}

	public static String getResource(String path, String bundlename) {
		try {
			Bundle bundle = Platform.getBundle(bundlename);
			return new Path(Platform.resolve(Platform.find(bundle, new Path(path))).getFile()).toFile().toString();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return null;

}
}
