package ch.unizh.ifi.adora.constraints.editor;

import java.io.File;
import java.util.Iterator;

import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;

import ch.unizh.ifi.adora.constraints.ui.ConstraintManageView;
import ch.unizh.ifi.adora.constraints.ui.table.ConstraintConverter;
import ch.unizh.ifi.adora.constraints.ui.table.ConstraintTableEntry;

/**
 * handles deletion of constraints
 * 
 * @author Christina Cramer
 * 
 */
public class DeleteManager {
	private static DeleteManager instance;

	public static DeleteManager getInstance(ConstraintManageView newview) {
		if (instance == null) {
			instance = new DeleteManager(newview);
		}
		return instance;
	}

	/**
	 * used for operations (constraintname-constraintfile)
	 */
	private ConstraintConverter converter = new ConstraintConverter();

	/**
	 * the view which will call the deletio
	 */
	private ConstraintManageView view;

	/**
	 * returns the (only) instance of the DeleteManager for the given View
	 * 
	 * @return
	 */

	private DeleteManager(ConstraintManageView newview) {
		this.view = newview;
	}

	public void deleteConstraints(ISelection elements) {
		Iterator iter = ((StructuredSelection) elements).iterator();
		while (iter.hasNext()) {
			ConstraintTableEntry entry = (ConstraintTableEntry) iter.next();
			if (!view.getMediator().isShowing(entry.getName())) {
				view.getConstraintList().removeEntry(entry);
				// delete it from the filesystem
				File file = converter.getConstraintFile(entry.getName());
				file.delete();
			}
		}

	}

	/**
	 * deletes constraints that are checked in the view only if the constraints
	 * are not open in an editor removes from view and deletes them from the
	 * filesystem.
	 * 
	 */
	public void deleteConstraints(Object[] elements) {
		for (int i = 0; i < elements.length; i++) {
			ConstraintTableEntry entry = (ConstraintTableEntry) elements[i];
			if (!view.getMediator().isShowing(entry.getName())) {
				view.getConstraintList().removeEntry(entry);
				// delete it from the filesystem
				File file = converter.getConstraintFile(entry.getName());
				file.delete();
			}
		}
	}

}
