package ch.unizh.ifi.adora.constraints.editor;

import icl.parser.IclParserTokenManager;
import icl.parser.Token;

import java.util.HashMap;

import org.eclipse.jface.text.Assert;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.rules.ICharacterScanner;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.ITokenScanner;



import ch.unizh.ifi.adora.properties.specification.scanner.LexerInputStream;
import ch.unizh.ifi.adora.properties.specification.scanner.SpecificationToken;

public class JavaCCSpecificationScanner implements ICharacterScanner, ITokenScanner {
    public final static int STATE_READ_NEXT = 0;
    
    public final static int STATE_CONSUMED_SPECIAL_TOKEN = 1;
    
    public final static int STATE_CONSUME_NORMAL_TOKEN = 2;
    
    private int tokenstate = STATE_READ_NEXT;
    
    private IclParserTokenManager lexer;
    
    private LexerInputStream is;
    
    private icl.parser.SimpleCharStream stream;
 
    private Token currToken;
    
    private HashMap<Integer, IToken> dataMappings;
    
    private int tokenLength=0;
    
    private IDocument myDocument;
    
    private int internalOffset=0;
 
    
    /** The list of rules of this scanner */
//    protected IRule[] fRules;
    /** The token to be returned by default if no rule fires */
    protected IToken fDefaultReturnToken;
    /** The document to be scanned */
    protected IDocument fDocument;
    /** The cached legal line delimiters of the document */
    protected char[][] fDelimiters;
    /** The offset of the last read token */
    protected int fTokenOffset;
 
    /** Internal setting for the un-initialized column cache. */
    protected static final int UNDEFINED= -1;

    public JavaCCSpecificationScanner() {
        is = new LexerInputStream();
        stream = new icl.parser.SimpleCharStream(is);
        lexer = new IclParserTokenManager(stream);
        dataMappings = new HashMap<Integer, IToken>();
    }
    

    /**
     * Configures the scanner with the given sequence of rules.
     *
     * @param rules the sequence of rules controlling this scanner
     */
    /*public void setRules(IRule[] rules) {
        if (rules != null) {
            fRules= new IRule[rules.length];
            System.arraycopy(rules, 0, fRules, 0, rules.length);
        } else
            fRules= null;
    }*/

    /**
     * Configures the scanner's default return token. This is the token
     * which is returned when non of the rules fired and EOF has not been
     * reached.
     *
     * @param defaultReturnToken the default return token
     * @since 2.0
     */
    public void setDefaultReturnToken(IToken defaultReturnToken) {
        Assert.isNotNull(defaultReturnToken.getData());
        fDefaultReturnToken= defaultReturnToken;
    }

    /*
     * @see ITokenScanner#setRange(IDocument, int, int)
     */
    public void setRange(IDocument document, int offset, int length) {
    	//System.out.println(offset);
    	//System.out.println("setRange");

    	//int myOffset=offset;
//    	try {
//    		String word="";
//    		while(!java.lang.Character.isLetter(document.getChar(myOffset))){
//    			myOffset++;
//    		}
//    		while(java.lang.Character.isLetter(document.getChar(myOffset))){
//    			word+=document.getChar(myOffset);
//    			myOffset++;
//    		}
//			//System.out.println(word+"   "+word.length());
//		} catch (BadLocationException e) {
//			// TODO Auto-generated catch blockSyste
//			e.printStackTrace();
//		}
    	fDocument= document;
        is = new LexerInputStream(fDocument);
        is.setFOffset(offset);
        is.setFColumn(UNDEFINED);
        int total=offset+length;
        is.setFRangeEnd(Math.max(fDocument.getLength(), offset + length));

        String[] delimiters= fDocument.getLegalLineDelimiters();

        fDelimiters= new char[delimiters.length][];
        for (int i= 0; i < delimiters.length; i++)
            fDelimiters[i]= delimiters[i].toCharArray();

        if (fDefaultReturnToken == null)
            fDefaultReturnToken= new SpecificationToken(null);
        
        stream.ReInit(is);
        lexer.ReInit(stream);
        tokenstate = STATE_READ_NEXT;
    }

    /*
     * @see ITokenScanner#getTokenOffset()
     */
    public int getTokenOffset() {
        return fTokenOffset;
    }

    /*
     * @see ITokenScanner#getTokenLength()
     */
    public int getTokenLength() {
    	int length;
    
    	//System.out.println("getTokenLength()");
    	
        //if (is.getFOffset() < is.getFRangeEnd()){
        	//System.out.println(1);
            length= is.getFOffset() - getTokenOffset();//}
        //else
        //{System.out.println(2);
        	//length= is.getFRangeEnd() - getTokenOffset();}
       
        //return length;
            
            return this.tokenLength;
    }

    public void setTokenMappingTupel(int tokenType, IToken t) {
        dataMappings.put(tokenType, t);
    }

    public void setTokenMappingTupel(int startTokenType, int endTokenType, IToken t) {
        for (int index = startTokenType; index <= endTokenType; index++) {
            dataMappings.put(index, t);
        }
        
    }
    
    public IToken getMappedToken(int tokenType) {
        IToken t = dataMappings.get(tokenType);
        if (t == null) {
            t = fDefaultReturnToken; 
        } 
        return t;
    }
  
    protected IToken getSpecificationToken(Token t) {
        return getMappedToken(t.kind);
    }
    
    
    /*
     * @see ICharacterScanner#getColumn()
     */
    public int getColumn() {
        if (is.getFColumn() == UNDEFINED) {
            try {
                int line= fDocument.getLineOfOffset(is.getFOffset());
                int start= fDocument.getLineOffset(line);

                is.setFColumn(is.getFColumn() - start);

            } catch (BadLocationException ex) {
            }
        }
        return is.getFColumn();
    }

    /*
     * @see ICharacterScanner#getLegalLineDelimiters()
     */
    public char[][] getLegalLineDelimiters() {
        return fDelimiters;
    }

    /*
     * @see ITokenScanner#nextToken()
     */
    public IToken nextToken() {
   
        if (is.isEof())
            return SpecificationToken.EOF;
        fTokenOffset= is.getFOffset();
        //System.out.println(fTokenOffset);
       // fTokenOffset=40;
        //System.out.println(is.getFOffset());
        is.setFColumn(UNDEFINED);
        Token t = getNextToken();
       // System.out.println(t);
        this.tokenLength=t.toString().length();
        //System.out.println(t);
        IToken tokenType = getMappedToken(t.kind);
        return tokenType;
        //
        //    
        //return fDefaultReturnToken;
    }

    
 
  
    
    protected Token getNextToken() {
        Token temp = null; 
        
        if (tokenstate == STATE_READ_NEXT) {
            if (currToken == null) {
                currToken = lexer.getNextToken();
            } else {
                if (currToken.next != null) currToken = currToken.next;
                else currToken = currToken.next = lexer.getNextToken();
            }
            if (currToken.specialToken != null) {
                temp = currToken.specialToken;
                tokenstate = STATE_CONSUMED_SPECIAL_TOKEN;
            } else {
                temp = currToken;
                tokenstate = STATE_READ_NEXT;
            }
        } else {
            if (tokenstate == STATE_CONSUMED_SPECIAL_TOKEN) {
        
                temp = currToken;
                tokenstate = STATE_READ_NEXT;
            }
        }
        return temp;
      }
    
    /*
     * @see ICharacterScanner#read()
     */
    public int read() {
        
        int ret = UNDEFINED;
        try {
            ret = is.read();
        } catch (Exception ex) {
            
        }
        return ret;
        
    }

    /*
     * @see ICharacterScanner#unread()
     */
    public void unread() {
            is.unread();
    }
    
    
}
