package ch.unizh.ifi.adora.constraints.engine;

import java.io.File;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchPage;
import ch.unizh.ifi.adora.constraints.ConstraintsPlugin;
import ch.unizh.ifi.adora.constraints.compiler.DynamicCodeLoader;
import ch.unizh.ifi.adora.constraints.ui.ConstraintManageView;
import ch.unizh.ifi.adora.constraints.ui.preferences.PreferencesConstants;
import ch.unizh.ifi.adora.constraints.ui.table.ConstraintTableEntry;
import ch.unizh.ifi.adora.model.representation.*;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;

/**
 * initial configuration of the whole Constraint Checking Mechanism. getting all
 * the modelelements of the model for checking getting all the constraints for
 * checking
 * 
 * @author cramer
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history
 * @version Oct 11, 2007 cramer
 */
public class ConstraintManageLoader {
	
	private Hashtable<IEditorPart,RuleManager> editors=new Hashtable<IEditorPart,RuleManager>();

	public static final String CONSTRAINT_PKG = "ch.unizh.ifi.adora.constraints.constraints";
	
	
	private DynamicCodeLoader loader;
	
	/**
	 * the view that contains the information, which constraints shall be checked
	 */
	private ConstraintManageView view;
	
	Vector constraintclasses=new Vector();

	public ConstraintManageLoader(ConstraintManageView constraintManageView) {
		this.view = constraintManageView;
	}

	
	/**
	 * add all modelelements to the rulemanager
	 * @param string 
	 */
	public boolean checkModel() {
		
		boolean success=true;
		IWorkbenchPage page = ch.unizh.ifi.adora.constraints.ConstraintsPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getActivePage();
		IEditorPart editor = page.getActiveEditor();
		if(editor==null){return true;}
		if(view.constraintsChanged()||constraintclasses.size()==0){
			constraintclasses = getConstraintClasses(view.getConstraintListAsArray());
			editors=new Hashtable<IEditorPart,RuleManager>();
		}
		
		if(!editors.containsKey(editor)){
			RuleManager rulemanager = init(constraintclasses);		
			Vector<ConstraintCheckable> objects = getObjects();
			for (int i = 0; i < objects.size(); i++) {
			rulemanager.newModelelement(objects.elementAt(i), "   ");
			}
			editors.put(editor, rulemanager);
			success=rulemanager.getSuccess();
		}
		else{
			editors.get(editor).reshow();	
			success=editors.get(editor).getSuccess();
		}
		
		return success;
	}

	/**
	 * get all constraints that are saved
	 * @param constraintListAsArray
	 * @return
	 */
	private Vector getConstraintClasses(Object[] constraintListAsArray) {
		if (loader == null) {
			loader = new DynamicCodeLoader();
		}

		String CONSTRAINT_SRC = ConstraintsPlugin.getDefault().getPreferenceStore().getString(
				PreferencesConstants.SRC_FILES);
		loader.addSourceDir(new File(CONSTRAINT_SRC));
		Vector constraintclasses = new Vector();
		Object[] constraints = view.getConstraintListAsArray();
		for (int i = 0; i < constraints.length; i++) {
			Object actualc = constraints[i];
			String constraintname = ((ConstraintTableEntry) actualc).getName();
			Vector<String> constraintcategory = ((ConstraintTableEntry) actualc).getCategory();
			try {
				Object o = loader.loadClass(CONSTRAINT_PKG + "." + constraintname).newInstance();
				constraintclasses.add(o);
			} catch (Exception e) {
				e.printStackTrace();
			}
			
		}
		return constraintclasses;
	}

	/**
	 * get all the modelelements of the open adoraeditor
	 * @return
	 */
	private Vector<ConstraintCheckable> getObjects() {
		Vector<ConstraintCheckable> objects = new Vector<ConstraintCheckable>();
		IWorkbenchPage page = ch.unizh.ifi.adora.constraints.ConstraintsPlugin.getDefault().getWorkbench()
				.getActiveWorkbenchWindow().getActivePage();
		IEditorPart editor = page.getActiveEditor();
		
		if (editor instanceof AdoraModelEditor) {
			AdoraModel am = ((AdoraModelEditor) editor).getModel();
			Iterator modelelements = am.getRecursiveChildrenIterator();
			while (modelelements.hasNext()) {
				Object nextelement = modelelements.next();
				
				if (nextelement instanceof ConstraintCheckable) {
			
					objects.add((ConstraintCheckable) nextelement);
				if(nextelement instanceof ConnectableElement){
					List<Connection> connections=((ConnectableElement)nextelement).getSourceConnections();
					for(Connection connection:connections){
						objects.add((ConstraintCheckable) connection);
					}
				}
				}
			}
		}
		return objects;
	}
	
	
	
/**
 * make the constraint checking components ready
 * @param constraintclasses
 * @return
 */
	private RuleManager init(Vector constraintclasses) {
		ConstraintTypeLoader loader = new ConstraintTypeLoader(constraintclasses);
		ScopeManager scopemanager = new ScopeManager();
		ConstraintCheckManager checkmanager = new ConstraintCheckManager();
		RuleManager rulemanager = new RuleManager();
		ConstraintsViewManager viewmanager = new ConstraintsViewManager();
		rulemanager.addViewManager(viewmanager);
		rulemanager.addLoader(loader);
		rulemanager.addScopeManager(scopemanager);
		rulemanager.addConstraintCheckManager(checkmanager);
		IEditorPart editor= ch.unizh.ifi.adora.constraints.ConstraintsPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getActivePage().getActiveEditor();
		if (editor instanceof AdoraModelEditor) {
			AdoraModel am = ((AdoraModelEditor) editor).getModel();
		am.addPropertyChangeListener(rulemanager);
		}
		return rulemanager;
	}

/**
 * Closing ConstraintCHecking. Removing PropertyChangeListener RuleManager from ALL ModelElements. 
 */
public void informClosed() {
	Enumeration<IEditorPart> keys=editors.keys();
	while(keys.hasMoreElements()){
		IEditorPart editor=keys.nextElement();
		RuleManager rm=editors.get(editor);
		AdoraModel am = ((AdoraModelEditor) editor).getModel();
		Iterator modelelements = am.getRecursiveChildrenIterator();
		while (modelelements.hasNext()) {
			ConstraintCheckable nextelement = (ConstraintCheckable)modelelements.next();
			nextelement.removePropertyChangeListener(rm);
				
			if(nextelement instanceof ConnectableElement){
				List<Connection> connections=((ConnectableElement)nextelement).getSourceConnections();
				for(Connection connection:connections){
					connection.removePropertyChangeListener(rm);
				}
			}
			}
		}
		
	}
	




}
