package ch.unizh.ifi.adora.constraints.engine;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;
import java.util.Vector;

import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;

import ch.unizh.ifi.adora.constraints.ui.ConstraintsTreeNode;
import ch.unizh.ifi.adora.constraints.ui.ConstraintsTreeNodeConstraint;
import ch.unizh.ifi.adora.constraints.ui.ConstraintsTreeNodeObject;
import ch.unizh.ifi.adora.constraints.ui.ResultTree;
import ch.unizh.ifi.adora.model.representation.ConstraintCheckable;

/**
 * @author cramer
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history Oct 10, 2007 cramer refactoring
 * @version Oct 10, 2007 cramer
 */
public class ConstraintsViewManager {
	/**
	 * connection from ConstraintCheckable (rootelement) to the node showing
	 * this elemenet
	 */
	private Hashtable<ConstraintCheckable, ConstraintsTreeNodeObject> objecttonode = new Hashtable<ConstraintCheckable, ConstraintsTreeNodeObject>();

	/**
	 * root of tree
	 */
	private ConstraintsTreeNode root;

	/**
	 * tree showing in the view
	 */
	private ResultTree tree = new ResultTree();

	/**
	 * Constructor. Tries to open a new ResultTreeView.
	 */
	public ConstraintsViewManager() {
		try {
			tree = (ResultTree) PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().showView(
					"ch.unizh.ifi.adora.constraints.ui.ResultTree");
			root = new ConstraintsTreeNode();
		} catch (PartInitException e) {
			e.printStackTrace();
		}
		root = new ConstraintsTreeNode();
		tree.setTree(root);
		
	}

	/**
	 * inserts a new node into the tree for the first time. all re-inserting is
	 * handed by informchanged
	 * 
	 * @param element
	 *            ConstraintCheckable, root element
	 * @param constraint,
	 *            the constraint that was violated
	 * @param result,
	 *            Constraint Result from the check, needed for calculating the
	 *            incidentobjects
	 */
	// public void informNew(ConstraintCheckable element, ConstraintInstance
	// constraint) {
	// makeNewObjectNode(element,constraint);
	// }

	/**
	 * to handle changes of inconsisenty in model
	 * 
	 * @param object
	 *            ruleinstance (=constraint)
	 * @param element
	 *            (rootelement)
	 * @param cause 
	 * @param result
	 *            (constraintresult, for logged elements)
	 */
	public void informchanged(ConstraintInstance constraint, ConstraintCheckable element, String cause) {
		boolean result = constraint.getResult().isSuccessful();
		ConstraintsTreeNodeObject treenode = objecttonode.get(element);
		if (objecttonode.get(element) == null) {
			if (!result) {
				makeNewObjectNode(element, constraint, cause);
			}
		} else {
			if (result) {
				removeObjectNode(element, constraint);
			} else {
				redrawObjectNode(element, constraint,cause);
			}
		}

	}

	/**
	 * makes a new first level node with all children
	 * 
	 * @param element
	 *            to be shown in first level
	 * @param cause 
	 * @param result
	 *            constraintresult (for incident objects)
	 * @param constraint
	 *            (for constraintmessage)
	 */
	private void makeNewObjectNode(ConstraintCheckable element, ConstraintInstance constraintinst, String cause) {
		ConstraintsTreeNodeObject objectnode = new ConstraintsTreeNodeObject(root, element);
		ConstraintsTreeNodeConstraint constraintnode = new ConstraintsTreeNodeConstraint(objectnode, constraintinst,cause);
		constraintnode.setCategory(constraintinst.getResult().getCategory());
		Enumeration elements = constraintinst.getResult().getIncidentObjects(false);
		Vector<ConstraintCheckable> elementsNoDouble = removeDouble(elements);
		for (int i = 0; i < elementsNoDouble.size(); i++) {
			ConstraintsTreeNodeObject childnode = new ConstraintsTreeNodeObject(constraintnode, elementsNoDouble
					.elementAt(i));
			
		}

		tree.getContentProvider().addNode(root, objectnode);
		try {
			objecttonode.put(element, objectnode);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * redraw the third level elements
	 * 
	 * @param element
	 *            object of first level
	 * @param cause 
	 * @param result
	 *            constraintresult (getIncidentObjects(false)) gives incident
	 *            eleemnts
	 * @param constraint
	 */
	private void redrawObjectNode(ConstraintCheckable element, ConstraintInstance constraintinst, String cause) {
		ConstraintsTreeNodeObject objectnode = objecttonode.get(element);
		String oldName="";;
		if (objectnode.hasConstraint(constraintinst.getRuleinstance()) != null) {
			ConstraintsTreeNodeConstraint constraintnode = objectnode.hasConstraint(constraintinst.getRuleinstance());
			objectnode.removeChild(constraintnode);
			oldName=constraintnode.getName();
			tree.getContentProvider().removeNode(constraintnode);
			tree.getContentProvider().refreshNode(objectnode);
		}
		ConstraintsTreeNodeConstraint constraintnode = new ConstraintsTreeNodeConstraint(objectnode, constraintinst,cause);
		constraintnode.setCategory(constraintinst.getResult().getCategory());
		if(!oldName.equals("")){constraintnode.setName(oldName);}
		Enumeration<ConstraintCheckable> elements = constraintinst.getResult().getIncidentObjects(false);
		Vector<ConstraintCheckable> elementsNoDouble = removeDouble(elements);
		for (int i = 0; i < elementsNoDouble.size(); i++) {
			ConstraintsTreeNodeObject childnode = new ConstraintsTreeNodeObject(constraintnode, elementsNoDouble
					.elementAt(i));
		}
		tree.getContentProvider().addNode(objectnode, constraintnode);
		tree.getContentProvider().refreshNode(objectnode);
	}

	/**
	 * remove a node from the tree
	 * 
	 * @param source
	 */
	public void remove(Object source) {
		if (objecttonode.get(source) != null) {
			ConstraintsTreeNodeObject objectnode = objecttonode.get(source);
			tree.getContentProvider().removeNode(objectnode);
			objecttonode.remove(source);
			root.removeChild(objectnode);
		}

	}

	/**
	 * removes double elements from Enumeration
	 * 
	 * @param elements
	 * @return Vector
	 */
	private Vector<ConstraintCheckable> removeDouble(Enumeration<ConstraintCheckable> elements) {
		Vector<ConstraintCheckable> noDouble = new Vector<ConstraintCheckable>();
		while (elements.hasMoreElements()) {
			ConstraintCheckable element = elements.nextElement();
			if (!noDouble.contains(element)) {
				noDouble.add(element);
			}
		}
		return noDouble;
	}

	/**
	 * remove a object node from tree
	 * 
	 * @param element
	 *            object node
	 * @param result
	 * @param constraint
	 */
	private void removeObjectNode(ConstraintCheckable element, ConstraintInstance constraintinst) {
		ConstraintsTreeNodeObject objectnode = objecttonode.get(element);
		List children = objectnode.children;
		for (int i = 0; i < children.size(); i++) {
			ConstraintsTreeNodeConstraint con = (ConstraintsTreeNodeConstraint) children.get(i);
			if (con.getconstraint().equals(constraintinst.getRuleinstance())) {
				try {
					tree.getContentProvider().removeNode(con);
					objectnode.removeChild(con);
					if (!objectnode.hasChildren()) {
						objecttonode.remove(element);
						tree.getContentProvider().removeNode(objectnode);
						root.removeChild(objectnode);
						tree.getContentProvider().refreshNode(root);

					}
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}

	}

	
	
	
	
	public void reshow(){
		try {
			tree = (ResultTree) PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().showView(
					"ch.unizh.ifi.adora.constraints.ui.ResultTree");
		} catch (PartInitException e) {
			e.printStackTrace();
		}

		tree.setTree(root);
	}

	public void informNameChanged(ConstraintCheckable source, String newValue) {
		ConstraintsTreeNodeObject node = objecttonode.get(source);
		if(node!=null){node.setName(newValue);
		tree.getContentProvider().refreshNode(node);
		}
		
	}
	
	
}
