package ch.unizh.ifi.adora.constraints.preferences;

import java.io.*;

import org.eclipse.core.runtime.preferences.AbstractPreferenceInitializer;
import org.eclipse.jface.preference.IPreferenceStore;

import ch.unizh.ifi.adora.constraints.ConstraintsPlugin;
import ch.unizh.ifi.adora.constraints.editor.CompileManager;

/**
 * 
 * Initializes the preferences storage with default values the constraints
 * plugin
 * 
 * @author ccramer
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history Feb16, 2006 CR
 * @version $Date: 2007-11-30 10:20:56 +0100 (Fri, 30 Nov 2007) $, $Author: cramer $, $Revision: 3341 $
 * 
 */
public class PreferencesInitializer extends AbstractPreferenceInitializer {

	/**
	 * Read read constraint from textfile. 
	 * 
	 * @param constraintfile
	 * @return text of Constraint
	 */
	public String getConstraintText(File constraintfile) {
		String next = "";
		BufferedReader br;
		try {
			FileReader fr = new FileReader(constraintfile);
			br = new BufferedReader(fr);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			return next;
		}
		String constraintstring = "";
		try {
			next = br.readLine();
			while(next!=null){
				String newline = System.getProperty("line.separator");
				constraintstring = constraintstring.concat(next + "" + newline);
				next=br.readLine();
				
			}
			return constraintstring;
		} catch (IOException e) {
		}

		return null;

	}

	public boolean initConstraints() {
		try {
			CompileManager compiler = new CompileManager();
			// get all constraints in folder ch.unizh.ifi.adora.constraints
			String locString = ConstraintsPlugin.getPath() + "doc";
			File txtFile = new File(locString);
			File[] files = txtFile.listFiles();
			for (int i = 0; i < files.length; i++) {
				String filename = files[i].getName();
				if (filename.endsWith("txt")) {
					String constraintText = getConstraintText(files[i]);
					compiler.compile(constraintText);
				}
			}
		return true;	
		} catch (Exception e) {
			e.printStackTrace();
		return false;
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.core.runtime.preferences.AbstractPreferenceInitializer#initializeDefaultPreferences()
	 */
	@Override
	public void initializeDefaultPreferences() {
		IPreferenceStore store = ch.unizh.ifi.adora.constraints.ConstraintsPlugin.getDefault().getPreferenceStore();
		store.setDefault(PreferencesConstants.SRC_FILES, PreferencesConstants.SRC_FILES);
		store.setDefault(PreferencesConstants.modelconstraint, PreferencesConstants.modelconstraint);
		String subfolder = ConstraintsPlugin.getDefault().getPreferenceStore()
				.getString(PreferencesConstants.SRC_FILES)
				+ "/ch";
		File f = new File(subfolder);
		if (!f.exists()) {
			initConstraints();
		}

	}

}
