package ch.unizh.ifi.adora.constraints.ui;

import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.PlatformUI;

import ch.unizh.ifi.adora.AdoraPlugin;

/**
 * This class is responsible for showing constraint messages in the status bar
 * of eclipse. These messages occure if a user performes a constraint violating
 * action.
 * 
 * @author Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 20.09.2006 IV First Version
 * @version $Date: 2007-10-11 10:49:01 +0200 (Thu, 11 Oct 2007) $, $Author: cramer $, $Revision: 3123 $
 */
public class ConstraintHint {

	/**
	 * Stores whether a constraint message is currently shown or not
	 */
	private static boolean constraintShown = false;

	/**
	 * Shows a constraint message in the status bar of eclipse.
	 * 
	 * @param msg
	 *            The message which is shown in the eclipse status bar.
	 */
	public static void displayConstraintHint(String msg) {

		IStatusLineManager statusbar = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage()
				.getActiveEditor().getEditorSite().getActionBars().getStatusLineManager();

		Image img = ImageDescriptor.createFromFile(AdoraPlugin.class, "icons/alert_obj.gif").createImage();

		statusbar.setErrorMessage(img, msg);
		constraintShown = true;
	}

	/**
	 * Used to determine if the consraint message has to be reset in the
	 * {@link ch.unizh.ifi.adora.tracker.ConstraintNodeTracker#handleButtonUp(int)}
	 * method.
	 * 
	 * @return <code>true</code> if a constraint message is currently shown.
	 */
	public static boolean isConstraintShown() {
		return constraintShown;
	}

	/**
	 * Clears the status bar of any message.
	 */
	public static void resetConstraintHint() {

		IStatusLineManager statusbar = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage()
				.getActiveEditor().getEditorSite().getActionBars().getStatusLineManager();

		statusbar.setErrorMessage(null, "");
		constraintShown = false;
	}
}