package ch.unizh.ifi.adora.constraints.ui;

import java.io.File;
import java.util.Arrays;
import java.util.Vector;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.CheckboxCellEditor;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import ch.unizh.ifi.adora.constraints.ConstraintsPlugin;
import ch.unizh.ifi.adora.constraints.editor.ConstraintTextEditor;
import ch.unizh.ifi.adora.constraints.editor.DeleteManager;
import ch.unizh.ifi.adora.constraints.editor.TableEditorMediator;
import ch.unizh.ifi.adora.constraints.engine.ConstraintManageLoader;
import ch.unizh.ifi.adora.constraints.preferences.PreferencesInitializer;
import ch.unizh.ifi.adora.constraints.ui.preferences.PreferencesConstants;
import ch.unizh.ifi.adora.constraints.ui.table.ConstraintTableEntriesList;
import ch.unizh.ifi.adora.constraints.ui.table.ConstraintTableEntry;
import ch.unizh.ifi.adora.constraints.ui.table.ConstraintTableFiller;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;

/**
 * 
 * @author Christina Cramer 
 * 
 * 			View that is used to get user input concerning the
 *         management of constraints also shows all saved constraints that the
 *         user can change and errors that occured when making a new
 *         constraint/changeing a constraint
 */
public class ConstraintManageView extends ViewPart {

	

	/**
	 * InnerClass that acts as a proxy for the Table providing content
	 * for the Table. 
	 */
	public class ConstraintTableContentProvider implements IStructuredContentProvider {

		

		/**
		 * Adding a new ConstraintTableEntry to the ConstraintTable and adds the
		 * Category of the entry to the list of Categories.
		 */
		public void addConstraintTableEntry(ConstraintTableEntry newEntry) {
			tableViewer.add(newEntry);
			tableViewer.setChecked(newEntry, true);			
		}

		

		/**
		 * Returns the Elements that are shown.
		 */
		public Object[] getElements(Object parent) {
			return tableEntries.getConstraintTableEntries().toArray();

		}

		/**
		 * Changes the ConstraintTableEntries list by adding or removing an
		 * element.
		 */
		public void inputChanged(Viewer v, Object oldInput, Object newInput) {
			if (newInput != null) {
				((ConstraintTableEntriesList) newInput).addChangeListener(this);
			}
			if (oldInput != null) {
				((ConstraintTableEntriesList) oldInput).removeChangeListener(this);
			}
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see ITaskListViewer#removeTask(ExampleTask)
		 */
		/**
		 * Removes one entry from the Table.
		 */
		public void removeTask(ConstraintTableEntry newEntry) {
			constraintschanged = true;
			tableViewer.remove(newEntry);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see ITaskListViewer#updateTask(ExampleTask)
		 */
		/**
		 * Resfreshes one Entry of the table.
		 */
		public void updateConstraint(ConstraintTableEntry newEntry) {

			tableViewer.update(newEntry, null);
			constraintschanged = true;
		}



		public void dispose() {
			// TODO Auto-generated method stub
			
		}
	}

	/**
	 * the active EditorPage
	 */
	private Object activePage = "DEFAULT";

	/**
	 * Actions in toolbar
	 */
	Action addItemAction, deleteItemAction, refreshAction;

	/**
	 * String for check all constraints.
	 */
	public static final String allConstraints = "ALL";

	/**
	 * Categories to select from.
	 */
	private Vector<String> categories = new Vector<String>();

	
	/**
	 * Manager that organizes the constant validating of the model. 
	 */
	private ConstraintManageLoader checkManager;

	/**
	 * Names of the Table Columns. 
	 */
	private final String[] colNames = { "", "Name", "Descripton" };

	/**
	 * Table showing the constraints that are available for changing
	 */
	private Table constraints;

	/**
	 * True if constraints changed since last time checking.
	 */
	boolean constraintschanged = true;

	/**
	 * Buttons
	 */
	//private Button discard, makeNew;

	/**
	 * Textfield for error message.
	 */
	private StyledText errormsg;

	/**
	 * Listens for changes in the View.
	 */
	private ConstraintManageViewListener listener;;

	/**
	 * The mediator used to communicate between view and editor.
	 */
	private TableEditorMediator mediator;

	/**
	 * Composite holding all the items.
	 */
	private Composite parent;


	/**
	 * Current Selection of Category.
	 */
	String showingConstraints = allConstraints;

	/**
	 * Make a new ConstraintTableEntry with init values.
	 */
	private ConstraintTableEntriesList tableEntries = new ConstraintTableEntriesList();

	/**
	 * viewer for the constraints table
	 */
	private CheckboxTableViewer tableViewer;

	/**
	 * Combo Box for selection of Categories.
	 */
	private Combo comb;

	
	/**
	 * ID of the view
	 */
	public static final String VIEW_ID = "ch.unizh.ifi.adora.constraints.ui.ConstraintManageView";
	
	/**
	 * weaving category
	 */
	public static final String CATEGORY_WEAVING = "\"weaving\"";
	
	public ConstraintManageView() {
	}



	/**
	 * adds a new Constraint to the list of constraint
	 * 
	 * @param constraintname
	 */
	public void addConstraint(String constraintname) {
		constraintschanged = true;
		tableEntries.addConstraint(constraintname);
		ConstraintTableEntry entry=tableEntries.getEntry(constraintname);
		for(String s:entry.getCategory()){
			if(!this.categories.contains(s)){
				categories.add(s);
				this.comb.add(s);
			}
		}
	}

	/**
	 * If the new showing editor is an AdoraEditor: check the model in the
	 * editor. If the new showing edtor is a ContraintTextEditor: show the
	 * ConstraintManageView.
	 * 
	 * @param newer
	 */
	public void changeEditor(Object newer) {
		this.activePage = newer;
		if (newer instanceof AdoraModelEditor) {
			if (checkManager == null) {
				checkManager = new ConstraintManageLoader(this);
			}
			boolean b = checkManager.checkModel();
			PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().activate(
					PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getActiveEditor());
		} else if (newer instanceof ConstraintTextEditor) {
			try {
				PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().showView(
						"ch.unizh.ifi.adora.constraints.ui.ConstraintManageView");
			} catch (PartInitException e) {
			}
		}
	}

	// }

	/**
	 * checks if the selected column is the name columd
	 * 
	 * @param e
	 *            MouseEvent
	 * @return true if the selected column was the name column, false otherwize.
	 */
	public boolean checkNameSelected(MouseEvent e) {
		Point point = new Point(e.x, e.y);
		TableItem selected = constraints.getItem(point);
		int column = -1;
		for (int i = 0, n = constraints.getColumnCount(); i < n; i++) {
			Rectangle rect = selected.getBounds(i);
			if (rect.contains(point)) {
				column = i;
				break;
			}
		}
		if (column == 0) {
			return true;
		}
		return false;
	}

	/**
	 * True if constraints changed since last calling
	 * 
	 */
	public boolean constraintsChanged() {
		return constraintschanged;
	}

	/**
	 * Create the action of the ToolBar. 
	 */
	public void createToolbarActions() {

		refreshAction = new Action("Refresh...") {
			@Override
			public void run() {
				refreshAction();
			}
		};
		refreshAction .setImageDescriptor(ImageDescriptor.createFromFile(ConstraintsPlugin.class, "icons/refresh.gif"));
		
		
		addItemAction = new Action("Add...") {
			@Override
			public void run() {
				mediator.makeNewAction();
			}

		};
		addItemAction.setImageDescriptor(ImageDescriptor.createFromFile(ConstraintsPlugin.class, "icons/alert_obj.gif"));

		deleteItemAction = new Action("Delete...") {
			@Override
			public void run() {
				deleteItem(tableViewer.getSelection());
			}
		};
		deleteItemAction.setImageDescriptor(ImageDescriptor.createFromFile(ConstraintsPlugin.class, "icons/delete.gif"));
	}

	
	/**
	 * Refreshes all the constraints. Recompiling them from ICL. 
	 */
	private void refreshAction() {

		String foldername = ConstraintsPlugin.getDefault().getPreferenceStore().getString(PreferencesConstants.SRC_FILES)+System.getProperty("file.separator")+"ch";
		File file=new File(foldername);
		file.delete();
		
		for(ConstraintTableEntry entry: tableEntries.getConstraintTableEntries()){
			this.tableViewer.remove(entry);
		}

		PreferencesInitializer pref=new PreferencesInitializer();
		boolean success=pref.initConstraints();
		if(success){
		this.setTableEntries();
			}

		
	}
	
	
	/**
	 * Initializing the layout.
	 */
	@Override
	public void createPartControl(Composite parent) {
		mediator = new TableEditorMediator(this);
		this.parent = parent;
		GridLayout layout = new GridLayout(2, false);
		parent.setLayout(layout);
		layoutTable();
		
		comb = new Combo(parent, SWT.PUSH);

		//makeNew = new Button(parent, SWT.PUSH);
		//makeNew.setText("New constraint");

		//discard = new Button(parent, SWT.PUSH);
		//discard.setText("Delete");
		

		errormsg = new StyledText(parent, SWT.PUSH | SWT.MULTI | SWT.READ_ONLY);
		
		layoutErrorMsg();
		
		
		createTableViewer();
		//addButtonActions();
		makeComboActions();
		createToolbarActions();
		createToolbar();

	}

	/**
	 * Create the Actions for the Combo. 
	 */
	private void makeComboActions() {
		comb.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent e) {
			}
			public void widgetSelected(SelectionEvent e) {
				setConstraints(comb.getItem(comb.getSelectionIndex()));
			}
		});
		comb.add(this.allConstraints);

		for (int i = 0; i < categories.size(); i++) {
			comb.add(categories.elementAt(i));
		}
		comb.select(0);
	}

	/**
	 * Layout the tableViewer.
	 * 
	 */
	public void createTableViewer() {
		tableViewer = new CheckboxTableViewer(constraints);
		tableViewer.setColumnProperties(colNames);
		CellEditor[] editors = new CellEditor[colNames.length];
		CheckboxCellEditor check = new CheckboxCellEditor(constraints);
		editors[0] = new TextCellEditor(constraints);
		editors[1] = new TextCellEditor(constraints);
		tableViewer.setCellEditors(editors);
		//tableViewer.setCellModifier(new ConstraintTableModifier(this));
		tableViewer.setContentProvider(new ConstraintTableContentProvider());
		tableViewer.setLabelProvider(new ConstraintTableFiller());
		setTableEntries();
		tableViewer.setAllChecked(true);
	}

	
	/**
	 * Enter the values into the Table. 
	 */
	private void setTableEntries() {
		tableEntries = new ConstraintTableEntriesList();
		Vector<ConstraintTableEntry> tableentries = tableEntries.getConstraintTableEntries();
		for (int i = 0; i < tableentries.size(); i++) {
			Vector<String> cat=tableentries.elementAt(i).getCategory();
			for(String s: cat){
				if(!categories.contains(s)){
					categories.add(s);
				}
			}
		}
		tableViewer.setInput(tableEntries);
		
	}

	/**
	 * Put the entries into the toolbar. 
	 */
	private void createToolbar() {
		IToolBarManager mgr = getViewSite().getActionBars().getToolBarManager();
		mgr.add(addItemAction);
		mgr.add(deleteItemAction);
		mgr.add(refreshAction);
	}

	private void deleteItem(ISelection selection) {
		DeleteManager de = DeleteManager.getInstance(this);
		de.deleteConstraints(selection);
	}


	/**
	 * returns the columnnames as a list
	 * 
	 * @return
	 */
	public java.util.List getColumnNames() {
		return Arrays.asList(colNames);
	}

	/**
	 * 
	 * @return the current list of ConstraintTableEntries
	 */
	public ConstraintTableEntriesList getConstraintList() {
		return tableEntries;
	}

	/**
	 * 
	 * @return the current list of ConstraintTableEntries
	 */
	public Object[] getConstraintListAsArray() {
		constraintschanged = false;
		return tableViewer.getCheckedElements();

	}

	/**
	 * Returns the TextField for the error-message. 
	 * @return
	 */
	public StyledText getErrorTextField() {
		return errormsg;
	}

	/**
	 * Returns the mediator to the (Constraint)Editor. 
	 * @return
	 */
	public TableEditorMediator getMediator() {
		return mediator;
	}

	@Override
	/**
	 * Initialize the View. 
	 */
	public void init(IViewSite site) throws PartInitException {
		super.init(site);
		final IWorkbench workbench = PlatformUI.getWorkbench();
		workbench.getDisplay().asyncExec(new Runnable() {
			public void run() {
				IWorkbenchWindow window = workbench.getActiveWorkbenchWindow();
				if (window != null) {
					initListener();

				}
			}
		});

	}

	/**
	 * Called when closing the ConstraintManageView. 
	 * ConstraintTreeView needs to be closed as well. 
	 * Remove PropertyChangeListener from ModelElements. 
	 * Remove PropertyChangeListeners from all EditParts. 
	 */
	public void dispose(){
		try {
			if (PlatformUI.getWorkbench() != null) {
				if (PlatformUI.getWorkbench().getActiveWorkbenchWindow() != null) {
					if (PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage() != null) {
						PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().removePartListener(listener);
					}
				}
			}
			IViewPart view=null;
		if(PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage()!=null){
			view=PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().findView("ch.unizh.ifi.adora.constraints.ui.ResultTree");
		}
		if(view!=null){PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().hideView(view);
		view.dispose();
		}
		if(checkManager!=null){checkManager.informClosed();}
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		super.dispose();
		
	}
	/**
	 * Add a listener to the active Page. As Well as first check of the showing model. 
	 */
	void initListener() {
		try {
			listener = new ConstraintManageViewListener(this);
			if (PlatformUI.getWorkbench() != null) {
				if (PlatformUI.getWorkbench().getActiveWorkbenchWindow() != null) {
					if (PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage() != null) {
						PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().addPartListener(listener);
				}
			}
		}
		}catch (Exception ex) {
			ex.printStackTrace();
		}
		if (checkManager == null) {
			checkManager = new ConstraintManageLoader(this);
		}
		boolean b = checkManager.checkModel();
		PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().activate(
				PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getActiveEditor());

	}

	/**
	 * Layout the Error Message. 
	 */
	public void layoutErrorMsg() {	
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.horizontalSpan = 2;
		errormsg.setLayoutData(gridData);
		errormsg.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				StyleRange range = new StyleRange();
				range.start=0;
				Display display = parent.getDisplay();
				range.foreground = display.getSystemColor(SWT.COLOR_RED);
				range.length = errormsg.getText().length();
				errormsg.setStyleRange(range);
				reLayout();
			}
		});
	}

	/**
	 * Sets the general layout of the table that shows all constraints.
	 * 
	 */
	public void layoutTable() {
		GridData data = new GridData(GridData.FILL_BOTH);
		data.minimumHeight=20;
		data.horizontalIndent = 10;
		data.verticalSpan = 3;
		constraints = new Table(parent, SWT.CHECK | SWT.FULL_SELECTION | SWT.MULTI|SWT.BORDER);
		constraints.setLayoutData(data);
		constraints.setLinesVisible(true);
		constraints.setHeaderVisible(true);
		constraints.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseDoubleClick(MouseEvent e) {
				if (checkNameSelected(e)) {
					TableItem selection = constraints.getSelection()[0];
					ConstraintTableEntry entry = (ConstraintTableEntry) selection.getData();
					mediator.showConstraint(entry.getName());
				}
			}

			@Override
			public void mouseDown(MouseEvent e) {
			}
		});

		
		
	
		TableColumn name = new TableColumn(constraints, SWT.LEFT);
		name.setText("Name");
		name.setWidth(300);
		TableColumn desc=new TableColumn(constraints, SWT.LEFT, 1);
		desc.setText("Description");
		desc.setWidth(500);
	}

	/**
	 * refreshes the layout of the whole composite
	 * 
	 */
	public void reLayout() {
		parent.layout();
	}

	/**
	 * now a new Model is showing --> check it
	 * 
	 * @param part
	 */
	public void setActiveEditor(IWorkbenchPart part) {
		Object newactivePage = ch.unizh.ifi.adora.constraints.ConstraintsPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow()
				.getActivePage().getActiveEditor();
		final IWorkbench workbench = PlatformUI.getWorkbench();
		workbench.getDisplay().asyncExec(new Myrun(this.activePage, this) {

		});

	}

	/**
	 * updates a constraint in the table
	 * 
	 * @param constraint
	 */
	public void setChanged(String constraintname) {
		ConstraintTableEntry entry=tableEntries.entryChanged(constraintname);
		for(String s:entry.getCategory()){
			if(!this.categories.contains(s)){
				categories.add(s);
				this.comb.add(s);
			}
		}
		
	}

	/**
	 * Shows a certain category of constraints (only).
	 * 
	 * @param s
	 *            category of constraints to be shown
	 * @return true if there are no violations of constraints, false if one or
	 *         more constraints of that category fail.
	 */
	public boolean setConstraints(String s) {
		if (s.equals(this.allConstraints)) {// show all constraints
			if(!this.showingConstraints.equals(this.allConstraints)){
			for (int i = 0; i < this.tableEntries.getConstraintTableEntries().size(); i++) {
				ConstraintTableEntry entry = (ConstraintTableEntry) this.tableEntries.getConstraintTableEntries().elementAt(i);
				Vector<String> cat = entry.getCategory();
				if (!cat.contains(this.showingConstraints)) {
					this.tableViewer.add(entry);
				}
			}
			}
		} else {
			if (this.showingConstraints.equals(this.allConstraints)) {// all
				for (int i = 0; i < this.tableEntries.getConstraintTableEntries().size(); i++) {
					ConstraintTableEntry entry = (ConstraintTableEntry) this.tableEntries.getConstraintTableEntries().elementAt(i);
					Vector<String> cat = entry.getCategory();
					if (!cat.contains(s)) {
						this.tableViewer.remove(entry);
					}
				}
			} else {// one category is showing, show another one now
				for (int i = 0; i < this.tableEntries.getConstraintTableEntries().size(); i++) {
					ConstraintTableEntry entry = (ConstraintTableEntry) this.tableEntries.getConstraintTableEntries().elementAt(i);
					Vector cat = entry.getCategory();
					if (cat.contains(this.showingConstraints)) {
						this.tableViewer.remove(entry);
					}
					if (cat.contains(s)) {
						this.tableViewer.add(entry);
					}
				}

			}
		}

		this.showingConstraints = s;
		this.constraintschanged = true;
		this.tableViewer.setAllChecked(true);
		if (checkManager == null) {
			checkManager = new ConstraintManageLoader(this);
		}
		boolean b = checkManager.checkModel();
		PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().activate(
				PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getActiveEditor());
		return b;
	}

	@Override
	public void setFocus() {
	}

	public void setMediator(TableEditorMediator mediator) {
		this.mediator = mediator;
	}

}

/**
 * Something to do with initialization of the ConstraintsMangeView. 
 * Initializtion must twait until the editor is built. 
 * @author cramer
 *
 */
class Myrun implements Runnable {
	private Object old = "DEFAULT";
	private ConstraintManageView runview;

	public Myrun(Object activePage, ConstraintManageView view) {
		this.runview = view;
		this.old = activePage;
	}

	public void changeView(Object newer) {
		runview.changeEditor(newer);
	}

	public void run() {
		final IWorkbench workbench = PlatformUI.getWorkbench();
		Object newer = workbench.getActiveWorkbenchWindow().getActivePage().getActiveEditor();
		if (old != newer) {
			runview.changeEditor(newer);
		}
	}
	



}
