package ch.unizh.ifi.adora.constraints.ui;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;

import ch.unizh.ifi.adora.constraints.ConstraintsPlugin;

//representing a Treenode
public class ConstraintsTreeNode {

	/**
	 * object that is represented by this node
	 */
	public Object asObject;

	/**
	 * children of this node
	 */
	public List children = new ArrayList();

	public Image image;

	/**
	 * name shown on node
	 */
	public String name;

	/**
	 * parent of this node
	 */
	public ConstraintsTreeNode parent;

	/**
	 * empty constructor, will be root of the tree
	 * 
	 */
	public ConstraintsTreeNode() {
		this.name = "ROOT";
		this.parent = null;
	}

	/**
	 * constructor initializing a node with a given parent and a given name
	 * 
	 * @param parent
	 *            parent (ResultTreeNode) of this node
	 * @param name
	 *            name of the node (name that will be shown in tree)
	 */
	public ConstraintsTreeNode(ConstraintsTreeNode parent, String name) {
		this.image = ImageDescriptor.createFromFile(ConstraintsPlugin.class, "icons/alert_obj.gif").createImage();
		this.parent = parent;
		parent.addChild(this);
		this.name = name;
	}

	/**
	 * constructor initializing a node with a given parent and a given objecr
	 * 
	 * @param parent
	 *            parent (ResultTreeNode) of this node
	 * @param object
	 *            that is represented by this node
	 */
	/*
	 * public ConstraintsTreeNode(ConstraintsTreeNode parent, Object object) {
	 * String pic = object.getClass().getSimpleName(); this.image =
	 * ImageDescriptor.createFromFile(ConstraintsPlugin.class, "icons/" + pic +
	 * ".gif").createImage(); this.parent = parent; parent.addChild(this);
	 * this.name = object.toString(); if ((this.name == null) ||
	 * (this.name.indexOf(" ") != -1)) { this.name = "Unnamed"; } this.asObject =
	 * object; }
	 */

	/**
	 * adds a child to this node
	 * 
	 * @param node
	 */
	public void addChild(ConstraintsTreeNode node) {
		children.add(node);
	}

	/**
	 * @return all children of this node
	 */
	public List getChildren() {
		return children;
	}

	/**
	 * @return image of this node
	 */
	public Image getImage() {
		return image;
	}

	/**
	 * @return name to be shown in tree of this node
	 */
	public String getName() {
		if (this.getObject() != null) {
			return this.getObject().getClass().getSimpleName() + ": " + name;
		}
		return name;
	}

	/**
	 * @return the object behind this node
	 */
	public Object getObject() {
		return this.asObject;
	}

	public ConstraintsTreeNode getParent() {
		return parent;
	}

	/**
	 * @return true if this node has children, false if no children
	 */
	public boolean hasChildren() {
		return children.size() > 0;
	}

	public void removeChild(ConstraintsTreeNode child) {
		for (int i = 0; i < children.size(); i++) {
			ConstraintsTreeNode c = (ConstraintsTreeNode) children.get(i);
			if (child.equals(c)) {
				children.remove(c);
			}
		}
	}

	public void removeChildren() {
		children.clear();

	}

}
