package ch.unizh.ifi.adora.constraints.ui;

import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;

import ch.unizh.ifi.adora.constraints.ConstraintsPlugin;
import ch.unizh.ifi.adora.constraints.constraints.util.ConstraintResult;
import ch.unizh.ifi.adora.constraints.engine.ConstraintInstance;

/**
 * representing a treenode
 * 
 * @author cramer
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history Oct 10, 2007 cramer refactoring
 * @version Oct 10, 2007 cramer
 */
public class ConstraintsTreeNodeConstraint extends ConstraintsTreeNode {

	/**
	 * object that is represented by this node
	 */
	private Object asObject;

	/**
	 * children of this node
	 */
	private List children = new ArrayList();

	private Object constraint;

	private Image image;

	/**
	 * name shown on node
	 */
	private String name;

	/**
	 * parent of this node
	 */
	private ConstraintsTreeNode parent;

	private Vector category;

	private ConstraintInstance constraintInstance;

	/**
	 * empty constructor, will be root of the tree
	 * 
	 * @param result
	 * 
	 */
	public ConstraintsTreeNodeConstraint(ConstraintResult result) {
		this.name = result.getMessage();
		this.parent = null;
	}

	/**
	 * constructor initializing a node with a given parent and a given name
	 * 
	 * @param o
	 *            parent (ResultTreeNode) of this node
	 * @param cause 
	 * @param result.getMessage
	 *            name of the node (name that will be shown in tree)
	 */
	public ConstraintsTreeNodeConstraint(ConstraintsTreeNode o, ConstraintInstance constraintinst, String cause) {
		this.constraintInstance=constraintinst;
		this.constraint = constraintinst.getRuleinstance();
		this.image = ImageDescriptor.createFromFile(ConstraintsPlugin.class, "icons/alert_obj.gif").createImage();
		this.parent = o;
		o.addChild(this);
		this.name = constraintinst.getResult().getMessage()+"   "+cause;
	}

	

	/**
	 * adds a child to this node
	 * 
	 * @param node
	 */
	@Override
	public void addChild(ConstraintsTreeNode node) {
		children.add(node);
	}

	/**
	 * @return all children of this node
	 */
	@Override
	public List getChildren() {
		return children;
	}

	public Object getconstraint() {
		return constraint;
	}

	/**
	 * @return image of this node
	 */
	@Override
	public Image getImage() {
		return image;
	}

	/**
	 * @return name to be shown in tree of this node
	 */
	@Override
	public String getName() {
		if (this.getObject() != null) {
			return this.getObject().getClass().getSimpleName() + ": " + name;
		}
		return name;
	}

	/**
	 * @return the object behind this node
	 */
	@Override
	public Object getObject() {
		return this.asObject;
	}

	@Override
	public ConstraintsTreeNode getParent() {
		return parent;
	}

	/**
	 * @return true if this node has children, false if no children
	 */
	@Override
	public boolean hasChildren() {
		return children.size() > 0;
	}

	public void setCategory(Vector<String> category) {
		this.category=category;
		
	}
	
	public Vector getCategory(){
		return category;
	}

	public ConstraintInstance getConstraintInstance() {
		return constraintInstance;
	}

	public void setName(String oldName) {
		this.name=oldName;
		
	}

}
