package ch.unizh.ifi.adora.constraints.ui;

import java.util.Vector;

import org.eclipse.jface.viewers.*;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.*;
import org.eclipse.ui.part.ViewPart;

import ch.unizh.ifi.adora.constraints.constraints.util.ConstraintResult;
import ch.unizh.ifi.adora.constraints.engine.ConstraintInstance;
import ch.unizh.ifi.adora.constraints.engine.TableModelMediator;
import ch.unizh.ifi.adora.constraints.ui.resulttree.TreeLabelProvider;

/**
 * tree showing constraintviolations
 * 
 * @author Christina Cramer
 * 
 */
public class ResultTree extends ViewPart {
	/**
	 * inner class, contentProvider for the Tree
	 * 
	 * @author Christina Cramer
	 * 
	 */
	public class TreeContentProvider implements ITreeContentProvider {

		public void addNode(ConstraintsTreeNode root, ConstraintsTreeNode node) {
			treeViewer.add(root, node);
			treeViewer.expandToLevel(node, AbstractTreeViewer.ALL_LEVELS);
			
		}

		public void dispose() {
			// TODO Auto-generated method stub
		}

		/**
		 * returns the children of a Node of the Tree
		 */
		public Object[] getChildren(Object parentElement) {
			return ((ConstraintsTreeNode) parentElement).getChildren().toArray();
		}

		public Object[] getElements(Object inputElement) {
			return getChildren(inputElement);
		}

		/**
		 * returns the parent of a Node of the Tree
		 */
		public Object getParent(Object element) {
			return ((ConstraintsTreeNode) element).getParent();
		}

		/**
		 * @return true if Node has children
		 */
		public boolean hasChildren(Object element) {
			return ((ConstraintsTreeNode) element).hasChildren();
		}

		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
			treeViewer.refresh();

		}

		public void refreshNode(ConstraintsTreeNode node) {
			treeViewer.refresh(node);
			treeViewer.refresh();
		}

		public void removeNode(ConstraintsTreeNode node) {
			treeViewer.remove(node);
		}
	}

	/**
	 * mediator to make changes in editor
	 */
	private TableModelMediator mediator;

	// private ConstraintsManager manager;

	/**
	 * the objects that are represented by the node that was selected
	 */
	private Vector selected;

	/**
	 * treeViewer to show tree
	 */
	public TreeViewer treeViewer;

	@Override
	/**
	 * creates the view of the tree. first tree has no elements adds a
	 * SelectionChangeListener to the tree
	 */
	public void createPartControl(Composite parent) {
	
		mediator = new TableModelMediator();
		treeViewer = new TreeViewer(parent);

		treeViewer.setContentProvider(new TreeContentProvider());
		treeViewer.setLabelProvider(new TreeLabelProvider());
		treeViewer.addTreeListener(new ITreeViewerListener() {

			public void treeCollapsed(TreeExpansionEvent event) {
				// TODO Auto-generated method stub

			}

			public void treeExpanded(TreeExpansionEvent event) {
				ConstraintsTreeNode element = (ConstraintsTreeNode) event.getElement();
				treeViewer.expandToLevel(event.getElement(), -1);

			}

		});
		/**
		 * menu shown on right button selection
		 */
		final Menu menu=new Menu(parent);
		treeViewer.getTree().setMenu(menu);
		menu.setVisible(false);
		MenuItem menuItem=new MenuItem(menu,SWT.None);
		menuItem.setText(":-)");
		menuItem.addSelectionListener(new SelectionListener(){

			public void widgetDefaultSelected(SelectionEvent e) {
				// TODO Auto-generated method stub
				
			}

			public void widgetSelected(SelectionEvent e) {
			if(treeViewer.getSelection() instanceof ConstraintsTreeNodeConstraint){
				ConstraintsTreeNodeConstraint constraintNode=(ConstraintsTreeNodeConstraint )treeViewer.getSelection();
				makeAction(constraintNode.getConstraintInstance(),constraintNode.getParent());
		}

				
			}

			});
		treeViewer.getTree().addMouseListener(new MouseListener(){

			public void mouseDoubleClick(MouseEvent e) {
				// TODO Auto-generated method stub
				
			}

			public void mouseDown(MouseEvent e) {
				
				if(e.button==3){

					TreeItem selectedItem=treeViewer.getTree().getSelection()[0];
					Rectangle r=selectedItem.getBounds();
					Point p=new Point(e.x,e.y);
					if(r.contains(p)){
						menu.setVisible(true);
					}
					else{menu.setVisible(false);}
					
				}
				
			}

			public void mouseUp(MouseEvent e) {
				// TODO Auto-generated method stub
				
			}});
		treeViewer.addSelectionChangedListener(new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent event) {
				if (event.getSelection().isEmpty()) {
					return;
				}
				if (event.getSelection() instanceof IStructuredSelection) {
					try {
						selected = new Vector();
						IStructuredSelection selection = (IStructuredSelection) event.getSelection();
						ConstraintsTreeNode treenode = (ConstraintsTreeNode) selection.getFirstElement();
						if (treenode instanceof ConstraintsTreeNodeObject) {
							selected.add(treenode.getObject());
							mediator.showElements(selected);
						}

					} catch (Exception e) {
						e.printStackTrace();
					}
				}

			}

		});

	}


	public TreeContentProvider getContentProvider() {
		return (TreeContentProvider) this.treeViewer.getContentProvider();
	}

	@Override
	public void setFocus() {
	}

	/**
	 * initializes the view with a tree
	 * 
	 * @param root
	 */
	public void setTree(ConstraintsTreeNode root) {
		try {
			treeViewer.setInput(root);
			treeViewer.expandAll();
		} catch (Exception e) {
			e.printStackTrace();
		}

	}
	
	private void makeAction(ConstraintInstance constraint, ConstraintsTreeNode constraintsTreeNode) {
	ConstraintResult result=constraint.getResult();
	
		
	}
	
	
}
