package ch.unizh.ifi.adora.constraints.ui.preferences;

import java.io.*;

import org.eclipse.jface.preference.FieldEditor;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.preference.StringFieldEditor;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

import ch.unizh.ifi.adora.constraints.ConstraintsPlugin;
import ch.unizh.ifi.adora.constraints.editor.CompileManager;

/**
 * Creates the preferences dialog page for the empirical test plugin.
 * 
 * @author smeier
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history May 31, 2006 SM First Version Oct 13, 2006 SM Implemented the access
 *          for the content of an inquery from a http server.
 * @version $Date: 2007-10-30 10:36:11 +0100 (Tue, 30 Oct 2007) $, $Author: cramer $, $Revision: 3201 $
 * 
 */
public class UIPreferences extends FieldEditorPreferencePage implements IWorkbenchPreferencePage {

	public StringFieldEditor srcFilesLoc;

	public UIPreferences() {
		super(GRID);
		setPreferenceStore(ch.unizh.ifi.adora.constraints.ConstraintsPlugin.getDefault().getPreferenceStore());
		setDescription("ADORA Constraints");
	}

	@Override
	protected void checkState() {
		super.checkState();
		if (!isValid()) {
			return;
		}
		try {
			File file = new File(srcFilesLoc.getStringValue());
			if(file.getAbsolutePath().contains(" ")){
				setErrorMessage("folder name must not contain spaces!");
				setValid(false);
				return;
			}
			if (!file.exists()) {
				setErrorMessage("not a valid folder, pls select an existing folder!");
				setValid(false);
				return;
			}
		} catch (Exception e) {
			// should not get there
			setValid(false);
			return;
		}
		setErrorMessage(null);
		setValid(true);
	}

	@Override
	protected void createFieldEditors() {

		srcFilesLoc = new StringFieldEditor(PreferencesConstants.SRC_FILES, "folder to save constraints:",
				getFieldEditorParent());
		addField(srcFilesLoc);

	}

	public String getConstraintText(File constraintfile) {
		String next = "";
		BufferedReader br;
		try {
			FileReader fr = new FileReader(constraintfile);
			br = new BufferedReader(fr);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			return next;
		}
		String constraintstring = "";
		try {
			do {
				next = br.readLine();
				String newline = System.getProperty("line.separator");
				constraintstring = constraintstring.concat(next + "" + newline);
			} while (!next.equals("end"));
			return constraintstring;
		} catch (IOException e) {
		}

		return null;

	}

	public void init(IWorkbench workbench) {

	}

	/**
	 * icl-compile all given constraints into this folder
	 */
	@Override
	public boolean performOk() {
		// do compilation
		super.performOk();
		try {
			// get all constraints in folder ch.unizh.ifi.adora.constraints
			String locString = ConstraintsPlugin.getPath() + "doc";
			File txtFile = new File(locString);
			File[] files = txtFile.listFiles();
			for (int i = 0; i < files.length; i++) {
				String filename = files[i].getName();
				if (filename.endsWith("txt")) {
					String constraintText = getConstraintText(files[i]);
					CompileManager compiler = new CompileManager();
					compiler.compile(constraintText);
				}
			}
		} catch (Exception e) {
			return false;
		}
		return true;
	}

	/**
	 * Calls the validation
	 */
	@Override
	public void propertyChange(PropertyChangeEvent evt) {
		super.propertyChange(evt);
		if (evt.getProperty().equals(FieldEditor.VALUE)) {
			if (evt.getSource() == this.srcFilesLoc) {
				checkState();
			}
		}
	}
}
