package ch.unizh.ifi.adora.constraints.ui.resulttree;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;

import ch.unizh.ifi.adora.constraints.ConstraintsPlugin;

/**
 * represents one entry in the resulttree
 * 
 * @author Christina Cramer
 * 
 */
public class ResultTreeNode {

	/**
	 * object that is represented by this node
	 */
	private Object asObject;

	/**
	 * children of this node
	 */
	private List children = new ArrayList();

	private Image image;

	/**
	 * name shown on node
	 */
	private String name;

	/**
	 * parent of this node
	 */
	private ResultTreeNode parent;

	/**
	 * empty constructor, will be root of the tree
	 * 
	 */
	public ResultTreeNode() {
		this.name = "ROOT";
		this.parent = null;
	}

	/**
	 * constructor initializing a node with a given parent and a given objecr
	 * 
	 * @param parent
	 *            parent (ResultTreeNode) of this node
	 * @param object
	 *            that is represented by this node
	 */
	public ResultTreeNode(ResultTreeNode parent, Object object) {
		String pic = object.getClass().getSimpleName();
		this.image = ImageDescriptor.createFromFile(ConstraintsPlugin.class, "icons/" + pic + ".gif").createImage();
		this.parent = parent;
		parent.addChild(this);
		this.name = object.toString();
		if ((this.name == null) || (this.name.indexOf(" ") != -1)) {
			this.name = "Unnamed";
		}
		this.asObject = object;
	}

	/**
	 * constructor initializing a node with a given parent and a given name
	 * 
	 * @param parent
	 *            parent (ResultTreeNode) of this node
	 * @param name
	 *            name of the node (name that will be shown in tree)
	 */
	public ResultTreeNode(ResultTreeNode parent, String name) {
		this.image = ImageDescriptor.createFromFile(ConstraintsPlugin.class, "icons/alert_obj.gif").createImage();
		this.parent = parent;
		parent.addChild(this);
		this.name = name;
	}

	/**
	 * adds a child to this node
	 * 
	 * @param child
	 */
	public void addChild(ResultTreeNode child) {
		children.add(child);
	}

	/**
	 * @return all children of this node
	 */
	public List getChildren() {
		return children;
	}

	/**
	 * @return image of this node
	 */
	public Image getImage() {
		return image;
	}

	/**
	 * @return name to be shown in tree of this node
	 */
	public String getName() {
		if (this.getObject() != null) {
			return this.getObject().getClass().getSimpleName() + ": " + name;
		}
		return name;
	}

	/**
	 * @return the object behind this node
	 */
	public Object getObject() {
		return this.asObject;
	}

	public ResultTreeNode getParent() {
		return parent;
	}

	/**
	 * @return true if this node has children, false if no children
	 */
	public boolean hasChildren() {
		return children.size() > 0;
	}

}
