package ch.unizh.ifi.adora.constraints.ui.table;

import icl.parser.IclConstraint;
import icl.parser.ParseException;

import java.io.*;
import java.util.Vector;

import ch.unizh.ifi.adora.constraints.ConstraintsPlugin;

/**
 * handles the changes from constraint (as .txt file) to entries of the
 * ConstraintTable as well as the change of the saving of a constraint that is
 * written in the editor to a file
 * 
 * @author Christina Cramer
 * 
 */
public class ConstraintConverter {

	private static final String descexpression = "**desc**";

	/**
	 * contains all the Constraints in format ConstraintTableEntry
	 */
	//private Vector allConstraintEntries;

	/**
	 * Vector with all the .txt files (not the file itself, only the filename)
	 * that are in the folder constraintlocation
	 */
	private Vector<String> allFiles;

	/**
	 * folder where .txt files of constraints will be saved
	 */

	private String constraintlocation = "";

	/**
	 * constructor
	 * 
	 */
	public ConstraintConverter() {
		this.constraintlocation = ConstraintsPlugin.getPath() + "doc";

	}

	/**
	 * searches for String descexpression within the string text
	 * 
	 * @param text
	 *            to be searched for description
	 * @return description of Constraint if found, emptry String if not found
	 */
	public String findDescription(String text) {
		String desc = "";
		try {
			int first = text.indexOf(descexpression);
			int second = text.indexOf(descexpression, first + 1);
			desc = text.substring(first + descexpression.length(), second);
		} catch (StringIndexOutOfBoundsException e) {
		}
		return desc;
	}

	/**
	 * @return Vector containing all the Constraints that are in the folder
	 *         constraintlocation as ConstraintTableEntry elements
	 */
	public Vector<ConstraintTableEntry> getAllConstraintsAsEntries() {
		getAllFiles();
		String constraintname;
		FileReader r;
		icl.parser.IclParser p=null;
		String filename;
		Vector<ConstraintTableEntry> allConstraintEntries=new Vector<ConstraintTableEntry>();
		try {
			for (int i = 0; i < allFiles.size(); i++) {
			filename=constraintlocation.concat(System.getProperty("file.separator") + allFiles.elementAt(i));
			r = new FileReader(filename);
			if(p==null){
				p=new icl.parser.IclParser(r);}
			else{
				p.ReInit(r);
			}
			IclConstraint c=p.Constraint();
			constraintname=c.getName();
			
			//File file = new File(constraintlocation.concat(System.getProperty("file.separator") + constraintname));
			ConstraintTableEntry entry = new ConstraintTableEntry(constraintname);
			entry.setFile(new File(filename));
 			entry.setDescription(c.getDescription());
 			entry.setCategory(c.getCategory());
			
 			allConstraintEntries.add(entry);
			r.close();
			}
		} catch (FileNotFoundException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		

		return allConstraintEntries;
	}
	
	public ConstraintTableEntry getConstraintAsEntry(String constraintname) {
		String filename=constraintlocation.concat(System.getProperty("file.separator") + constraintname+".txt");
		try {
			FileReader r = new FileReader(filename);
			icl.parser.IclParser p=new icl.parser.IclParser(r);
			IclConstraint c=p.Constraint();
			constraintname=c.getName();
			//File file = new File(constraintlocation.concat(System.getProperty("file.separator") + constraintname));
			ConstraintTableEntry entry = new ConstraintTableEntry(constraintname);
			entry.setFile(new File(filename));
 			entry.setDescription(c.getDescription());
 			entry.setCategory(c.getCategory());
			return entry;
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
		
	}

	/**
	 * get all .txt files that are in the folder constraintlocation
	 * 
	 * @throws NullPointerException
	 *             if folder does not exist
	 */
	public void getAllFiles() {
		allFiles = new Vector<String>();
		try {
			File dir = new File(constraintlocation);
			String[] children = dir.list();
			if (children == null) {
			} else {
				String filename = "";
				for (int i = 0; i < children.length; i++) {
					// Get filename of file or directory
					filename = children[i];
					if (isTextFile(filename)) {
						allFiles.add(filename);
					}
				}
			}
		} catch (NullPointerException e) {
		}
	}

	/**
	 * gets the File that has the name constraintname in folder
	 * constraintlocatio
	 * 
	 * @param constraintname
	 * @return
	 */
	public File getConstraintFile(String constraintname) {
		try {
			File file = new File(constraintlocation.concat(System.getProperty("file.separator") + constraintname
					+ ".txt"));
			return file;
		} catch (NullPointerException e) {
			return null;
		}
	}

	/**
	 * returns the constraintname from the string witht he whole constraint text
	 * 
	 * @param text
	 *            whole constrainttext
	 * @return text of the constraint
	 */
	public String getConstraintName(String text) {
		String[] result = text.split("\\s");
		return result[1];
	}

	/**
	 * returns the text (String) of a given file
	 * 
	 * @pre constraintfile is a textfile (.txt)
	 * @param constraintfile
	 * @return String with text of constraint including newlines
	 */
	public String getConstraintText(File constraintfile) {
		String next = "";
		BufferedReader br;
		try {
			FileReader fr = new FileReader(constraintfile);
			br = new BufferedReader(fr);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			return next;
		}
		String constraintstring = "";
		try {
			String newline = System.getProperty("line.separator");

			while (br.ready()) { 			
				next = br.readLine();
				constraintstring = constraintstring.concat(next + "" + newline);
			} 
			return constraintstring;
		} catch (IOException e) {
		}

		return null;

	}

	/**
	 * reads the Description of the Constraint from the Constraint that is
	 * written in the file with given filename description of constraint has to
	 * be written in comment and start/end with the expression specified as
	 * descexpression
	 * 
	 * @return description of constraint if there is one
	 * @param File
	 *            where description is in the text
	 */
	public String getDescription(File file) {
		String desc = "";
		// open the file
		try {
			String constrainttext = getConstraintText(file);
			desc = findDescription(constrainttext);
		} catch (NullPointerException e) {
		}

		return desc;
	}

	/**
	 * checks if a file is a textfile
	 * 
	 * @param file
	 *            filename (full)
	 * @return true if the filename represents a .txt file, false otherwize
	 */
	public boolean isTextFile(String file) {
		return file.endsWith(".txt");

	}

	

}
