package ch.unizh.ifi.adora.constraints.ui.table;

import java.util.*;

import ch.unizh.ifi.adora.constraints.ui.ConstraintManageView.ConstraintTableContentProvider;

/**
 * this class contains all the entries of the ConstraintTable shown in the
 * ConstraintManage View
 * 
 * @author Christina Cramer
 * 
 */
public class ConstraintTableEntriesList {



	private Hashtable<String,ConstraintTableEntry> constraintEntriesTable = new Hashtable<String, ConstraintTableEntry>();
	
	/**
	 * class used to convert constraints (as files) to ConstraintTableEntries
	 */
	ConstraintConverter converter = new ConstraintConverter();

	/**
	 * all Listeners of the ConstraintTable
	 */
	private Set Listeners = new HashSet();

	/**
	 * Constructor, calling the initalization of the ConstraintTable
	 * 
	 */
	public ConstraintTableEntriesList() {
		getEntries();
	}

	/**
	 * add a listener to the List of ConstraintTableEntries. Listener will be
	 * informed of changes in the LIst
	 * 
	 * @param viewer
	 */
	public void addChangeListener(ConstraintTableContentProvider viewer) {
		Listeners.add(viewer);
	}

	/**
	 * makes a new ConstraintTableEntry from a constraint and adds it to the
	 * List of ConstraintTableEntries
	 * 
	 * @param constraintname
	 *            name of the constraint
	 * @pre file exists
	 *      ConstraintConverter.constraintlocation.constraintname.txt
	 */
	public void addConstraint(String constraintname) {
		ConstraintTableEntry entry =converter.getConstraintAsEntry(constraintname);
		constraintEntriesTable.put(constraintname, entry);
		Iterator iter = Listeners.iterator();
		while (iter.hasNext()) {
			((ConstraintTableContentProvider) iter.next()).addConstraintTableEntry(entry);
		}
	}

	/**
	 * removes and entry from the list. also informs listeners about the change
	 * 
	 * @param entry
	 */
	public void addEntry(ConstraintTableEntry entry) {
		constraintEntriesTable.put(entry.getName(), entry);
		Iterator iter = Listeners.iterator();
		while (iter.hasNext()) {
			((ConstraintTableContentProvider) iter.next()).updateConstraint(entry);
		}
	}

	/**
	 * checks if the list of constraintEntries already contains an entry with
	 * that name
	 * 
	 * @return true if contains an entry with same name
	 */
	public boolean containsConstraintEntry(String constraintname) {
		return constraintEntriesTable.get(constraintname)!=null;
	}

	/**
	 * informing listeners that an entry was changed
	 * 
	 * @param entry
	 */
	public boolean entryChanged(ConstraintTableEntry entry) {
		Iterator iterator = Listeners.iterator();
		while (iterator.hasNext()) {
			((ConstraintTableContentProvider) iterator.next()).updateConstraint(entry);
		}
	return true;
		
	}

	/**
	 * changing an constraintentry of a given name only thing that can change
	 * (at the moment) is the description if the constraint is new (there is no
	 * ContraintTableEntry to that name) make new ConstraintTableEntry
	 * 
	 * @param entry
	 *            (String)
	 * @pre getEntry(entryname)!=null
	 */
	public ConstraintTableEntry entryChanged(String entryname) {
		ConstraintTableEntry oldEntry=constraintEntriesTable.get(entryname);
		ConstraintTableEntry entry =converter.getConstraintAsEntry(entryname);
		oldEntry.setCategory(entry.getCategory());
		oldEntry.setDescription(entry.getDescription());
		oldEntry.setFile(entry.getFile());
		constraintEntriesTable.put(entryname, oldEntry);
		if(entryChanged(oldEntry));
		return oldEntry;
	}

	/**
	 * returns all the ConstraintTableEntry that are available
	 * 
	 * @return Vector with ConstraintTableEntry Elements
	 */
	public Vector<ConstraintTableEntry> getConstraintTableEntries() {
		Vector<ConstraintTableEntry> values=new Vector<ConstraintTableEntry>();
		Enumeration keys=constraintEntriesTable.keys();
		while(keys.hasMoreElements()){
			values.add(constraintEntriesTable.get(keys.nextElement()));
		}
		return values;
	}

	/**
	 * get the data to fill the ConstraintTable with ConstraintTableEntry
	 * Elements gets the Constraints from a given folder and the information
	 * about status from storage
	 */
	public void getEntries() {
		ConstraintConverter converter = new ConstraintConverter();
		Vector<ConstraintTableEntry> constraintEntries = converter.getAllConstraintsAsEntries();
		for(ConstraintTableEntry entry:constraintEntries){
			constraintEntriesTable.put(entry.getName(), entry);
		}
	}

	public ConstraintTableEntry getEntry(String entryname) {
		return constraintEntriesTable.get(entryname);
	}

	
	/**
	 * removes an Listener to the List
	 * 
	 * @param viewer
	 */
	public void removeChangeListener(ConstraintTableContentProvider viewer) {
		Listeners.remove(viewer);
	}

	/**
	 * removes and entry from the list. also informs listeners about the change
	 * 
	 * @param entry
	 */
	public void removeEntry(ConstraintTableEntry entry) {
		constraintEntriesTable.remove(entry.getName());
		Iterator iter = Listeners.iterator();
		while (iter.hasNext()) {
			((ConstraintTableContentProvider) iter.next()).removeTask(entry);
		}
		

	}

}
