package ch.unizh.ifi.adora.constraints.ui.table;

import java.io.File;
import java.util.Vector;

/**
 * class representing a single entry of the constrainttable in the table the
 * name of the constraint, a description and the status of the constraint will
 * be shown
 * 
 * @author Christina Cramer
 * 
 */
public class ConstraintTableEntry {

	/**
	 * the file which constraints the constraint.
	 */
	private File constraintfile;

	private String description = "";

	private String name = "";

	private Vector<String> categories=new Vector<String>();
	
	public ConstraintTableEntry() {
	}

	/**
	 * Create a ConstraintTableEntry
	 * 
	 * @param string
	 *            name of the Constraint
	 */
	public ConstraintTableEntry(String name) {
		setName(name);
	}

	/**
	 * checks if other ConstraintTableEntry is same. two ConstrainTableEntry are
	 * same if they have the same name
	 * 
	 * @param ce
	 * @return true if same name, false otherwize
	 */
	public boolean equals(ConstraintTableEntry ce) {
		return ce.getName().equals(name);
	}

	/**
	 * returns the description of the ConstraintTableEntry
	 * 
	 * @return
	 */
	public String getDescription() {
		return description;
	}

	/**
	 * returns the File of the ConstraintEntry
	 * 
	 * @return File constraintfile
	 */
	public File getFile() {
		return constraintfile;
	}

	/**
	 * returns the name of the ConstraintTableEntry
	 * 
	 * @return String name
	 */
	public String getName() {
		return name;
	}

	/**
	 * sets the description that will be shown in the table. description can
	 * only be set once
	 * 
	 * @param desc
	 *            descripion of the ConstraintTableEntry
	 */
	public void setDescription(String desc) {
		this.description = desc;

	}

	/**
	 * sets the file of this ConstraintTableEntry to the given file
	 * 
	 * @param file
	 */
	public void setFile(File file) {
		constraintfile = file;
	}

	/**
	 * sets the name of the ConstraintTableEntry. Only allowed for a new Entry!
	 * Name of Constraint cannot be changed once it is set.
	 * 
	 * @param setname
	 */
	public void setName(String setname) {
		if (name.equals("")) {
			this.name = setname;
		}
	}

	public Vector<String> getCategory() {
		return categories;
	}

	public void setCategory(Vector<String> category) {
		this.categories = category;
	}

}
