package ch.unizh.ifi.adora.constraints.ui.table;

import org.eclipse.jface.viewers.ICellModifier;
import org.eclipse.swt.widgets.TableItem;

import ch.unizh.ifi.adora.constraints.ui.ConstraintManageView;

/**
 * will be called when a cell of the ConstraintTable is modifie
 * 
 * @author Christina Cramer
 * 
 */
public class ConstraintTableModifier implements ICellModifier {

	/**
	 * the view that is watched by this modifier
	 */
	private ConstraintManageView view;

	public ConstraintTableModifier() {
	}

	/**
	 * Constructor that sets the view of the Modifier
	 * 
	 * @param view
	 */
	public ConstraintTableModifier(ConstraintManageView view) {
		this.view = view;
	}

	/**
	 * only status column can be modified
	 * 
	 * @see org.eclipse.jface.viewers.ICellModifier#canModify(java.lang.Object,
	 *      java.lang.String)
	 */
	public boolean canModify(Object element, String property) {
		return true;
	}

	/**
	 * get the value of a cell
	 * 
	 * @see org.eclipse.jface.viewers.ICellModifier#getValue(java.lang.Object,
	 *      java.lang.String)
	 */
	public Object getValue(Object element, String property) {

		// Find the index of the column
		int columnIndex = view.getColumnNames().indexOf(property);

		Object result = null;
		ConstraintTableEntry entry = (ConstraintTableEntry) element;

		switch (columnIndex) {
		case 0: // checkbox column
			result = true;
			break;
		case 1: // name
			result = entry.getName();
			break;
		case 2: // description
			result = entry.getDescription();
			break;
		default:
			result = "";
		}
		return result;
	}

	/**
	 * modify the value of a cell note that only the status can be modified
	 * directly in the table
	 * 
	 * @see org.eclipse.jface.viewers.ICellModifier#modify(java.lang.Object,
	 *      java.lang.String, java.lang.Object)
	 */
	public void modify(Object element, String property, Object value) {
		// Find the index of the column
		int index = view.getColumnNames().indexOf(property);
		TableItem item = (TableItem) element;
		ConstraintTableEntry entry = (ConstraintTableEntry) item.getData();
	}

}
