package ch.unizh.ifi.adora;

import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.EditorActionBarContributor;
import org.eclipse.ui.plugin.AbstractUIPlugin;

import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;

/**
 * The ADORA plugin class which is the entry point for the ADORA plugin.
 *      
 * @author				gygax
 *						smeier
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				30.08.2005 AG First Version
 *						18.09.2005 SM Comments
 *						24.10.2007 SM Some auxiliary methods implemented
 *                                    which help to retrieve the currently active editor
 *                                    as well as the currently active model. 
 *                                    
 * @version			$Date: 2010-02-11 20:38:24 +0100 (Thu, 11 Feb 2010) $, $Author: vigan $, $Revision: 5115 $
 *
 */
public class AdoraPlugin extends AbstractUIPlugin {
	public static final String PLUGIN_ID = "ch.unizh.ifi.adora.editor";
	
	/**
	 * The default instance of the plugin.
	 */
	private static AdoraPlugin plugin;

	/**
	 * The constructor for the plugin class.
	 */
    public AdoraPlugin() {
		plugin = this;
 	}

	/**
	 * Returns the shared instance.
	 * @return Returns the default plugin instance.
	 */
	public static AdoraPlugin getDefault() {
		return plugin;
	}
	
	/**
	 * Returns the currently active ADORA editor. This method returns null, if 
	 * there is none.
	 * 
	 * @return Returns the currently active editor or null.
	 */
	public static AdoraModelEditor getActiveEditor() {
		AdoraModelEditor activeEditor = null;
		IWorkbench o = PlatformUI.getWorkbench();
		if (o != null) {
			IWorkbenchWindow iw = o.getActiveWorkbenchWindow();
			if (iw != null) {
				IWorkbenchPage page = iw.getActivePage();
				if (page != null) {
					Object editor = page.getActiveEditor();

					if (editor instanceof AdoraModelEditor
							&& editor != null) {
						activeEditor = (AdoraModelEditor) editor;
					}
				}
			}
		}
		return activeEditor;
	}
	
	/**
	 * Returns the ADORA model of the currently active model.
	 * @return    Returns the model of the currently active ADORA editor.
	 */
	public static AdoraModel getActiveModel() {
		AdoraModelEditor ame = getActiveEditor();
		AdoraModel retVal    =  null;
		if (ame != null) {
			retVal = ame.getModel();
		}
		return retVal;
	}
	
	/**
	 * Sets the focus
	 */
	public static void setFocus() {
		getActiveEditor().setFocus();
	}
}