package ch.unizh.ifi.adora.actions;

import java.util.Iterator;
import java.util.List;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionFactory;

import ch.unizh.ifi.adora.actions.resources.MenuNames;
import ch.unizh.ifi.adora.model.commands.NodeDelete;
import ch.unizh.ifi.adora.model.commands.RemoveNodeVisually;
import ch.unizh.ifi.adora.model.representation.Node;

/**
 * Provides the cut functionality for a model.
 * The cutted elements are put into the clipboard.
 * Furthermore, a resizing of the surrounding takes place. 
 * 
 * @author		        vigan
 * @author				smeier
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			04.02.2006 IV First Version
 *						29.02.2006 SM Comments and fixed bug #527
 *						18.07.2006 SM Comments, dead code removed.
 * @version			$Date: 2009-12-23 14:25:55 +0100 (Wed, 23 Dec 2009) $, $Author: jeanneret $, $Revision: 5005 $
 *
 */

public class AdoraCutAction extends AdoraCopyAction {

	/**
	 * Constructs a new AdoraCutAction on a given WorkbenchPart 
	 * 
	 * @pre       part!=null
	 * @post      true
	 * @param part The workbench part which is referred by this action.
	 */
	public AdoraCutAction(IWorkbenchPart part) {
        super(part);        
	}

	/**
	 * Deletes all selected elemenets, and resizes the surroundings.
	 * The cut element is done put into the clipboard and removed from
	 * the canvas.
	 * @param selection The selection for which the removal will be
	 *					 executed.
	 * @see ch.unizh.ifi.adora.actions.AdoraCopyAction#executeCommandsOnSelection(java.util.List)
	 */
	protected void executeCommandsOnSelection(List selection)  {
		NodeDelete deleteCommand;
        RemoveNodeVisually shrinkCommand;
        Command  combinedCommad;
		Node currentElement;
		// iterate over the selection
		Iterator iter = selection.iterator();
		while(iter.hasNext()){
			// remove every element and shrink the freed space.
			currentElement = (Node)iter.next();
            deleteCommand = new NodeDelete(currentElement.getParent(),currentElement);    
			shrinkCommand = new RemoveNodeVisually(currentElement);
            combinedCommad = new CompoundCommand("Cut");
            combinedCommad = shrinkCommand.chain(deleteCommand);
            getCommandStack().execute(combinedCommad);
		}
	}
		
	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#init()
	 */
    protected void init() {
		setId(ActionFactory.CUT.getId());
		setText(MenuNames.CUT);
		
		ISharedImages sharedImages = PlatformUI.getWorkbench().getSharedImages();
		setImageDescriptor(sharedImages.getImageDescriptor(ISharedImages.IMG_TOOL_CUT));
		setDisabledImageDescriptor(sharedImages.getImageDescriptor(
				ISharedImages.IMG_TOOL_CUT_DISABLED));
	}
    
	@Override
	public void dispose() {
		setWorkbenchPart(null);
		super.dispose();
	}
}