package ch.unizh.ifi.adora.actions.representation;

import org.eclipse.jface.action.IAction;
import org.eclipse.swt.widgets.Event;

import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.ExitPoint;

/**
 * This class' run method executes the AspectView algorithm.
 * 
 * @author Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 02.05.2006 IV First Version<br>
 *          18.07.2006 SM Comments & Refactoring<br>
 *          23.01.2008 PS Refactoring & Use of "Observers"
 * @version $Date: 2009-12-23 14:25:55 +0100 (Wed, 23 Dec 2009) $, $Author: jeanneret $, $Revision: 5005 $
 */
public class AspectViewAction extends AbstractViewAction {
	/**
     * Describes the icon name for the activated view.
     */
	private final static String ICON_NAME = "icons/aspectview.gif";

	/**
     * Describes the icon name for the deactivated view.
     */
	private final static String ICON_NAME_DEACTIVATED = "icons/aspectview_deactivated.gif";

	/**
     * Specifies which Nodetypes are filtered by this view manager.
     */
	public AspectViewAction() {
		type = AspectContainer.class;
	}

	/**
     * Overwritten since the AspectViewAction has to hide 2 palette entries.
     * 
     * @param type
     *            the {@link ch.unizh.ifi.adora.model.representation.Node}-Class
     *            which gets filtered.
     * @param filterApplied
     *            hide or show.
     */
	protected void updatePalette(boolean filterApplied) {
		super.updatePalette(type, filterApplied);
		super.updatePalette(ExitPoint.class, filterApplied);
	}

	/*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.actions.representation.AbstractViewAction#getIconDeactivatedName()
     */
	@Override
	public String getIconDeactivatedName() {
		return ICON_NAME_DEACTIVATED;
	}

	/*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.actions.representation.AbstractViewAction#getIconName()
     */
	@Override
	public String getIconName() {
		return ICON_NAME;
	}



}