package ch.unizh.ifi.adora.actions.representation;

import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.ui.IWorkbenchPart;

import ch.unizh.ifi.adora.model.commands.NodeCompact;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.parts.ZoomableNodeContainerEditPart;

/**
 * Action to reduce the size of a node by reducing the border around
 * the children.
 * 
 * @author 		Tobias Reinhard
 * @copyright 	Requirements Engineering Research Group<br>
 *            	Department of Informatics<br>
 *            	University of Zurich<br>
 * @history 	2008-06-11 TR First Version
 * @version 	$Date: 2009-12-23 14:25:55 +0100 (Wed, 23 Dec 2009) $, $Author: jeanneret $, $Revision: 5005 $
 * 
 */
public class CompactNodeAction extends SelectionAction {

	/**
	 * Id of the action.
	 */
	public static final String COMPACT_NODE = "CompactNode";
	
	/**
	 * Constructor.
	 * 
	 * @param part The workbench part associated with this action
	 */
	public CompactNodeAction(IWorkbenchPart part) {
		super(part);
	}
	
	
	/**
	 * Invokes the action.
	 */
	public void run() {
		
		ZoomableNodeContainerEditPart node = (ZoomableNodeContainerEditPart)getSelectedObjects().get(0);
		NodeCompact cmd = new NodeCompact();
		cmd.setTarget((ZoomableNodeContainer)node.getModel());
		execute(cmd);
		
	}
	
	
	protected boolean calculateEnabled() {
		
		if (getSelectedObjects().isEmpty() || getSelectedObjects().size() > 1) {
			return false;
		}
		
		Object selectedNode = getSelectedObjects().get(0);
		if(selectedNode instanceof ZoomableNodeContainerEditPart) {
			return true;
		}
		
		return false;
	}
	
	
    protected void init() {
    	setId(COMPACT_NODE);
		setText("Compact node");
		
		//setImageDescriptor(ImageDescriptor
		//		.createFromFile(AdoraPlugin.class, "icons/magnifyingglass24.gif"));
    }
	@Override
	public void dispose() {
		setWorkbenchPart(null);
		super.dispose();
	}
}
