package ch.unizh.ifi.adora.actions.representation;

import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.IWorkbenchPart;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.commands.RecursiveFisheyeZoom;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.parts.ZoomableNodeContainerEditPart;

/**
 * Action to perform a fisheye zoom on a node and all its direct and indirect 
 * children.
 *
 * @author				Tobias Reinhard
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				2007-10-30 TR First Version
 * @version				$Date: 2009-12-23 14:25:55 +0100 (Wed, 23 Dec 2009) $, $Author: jeanneret $, $Revision: 5005 $
 *
 */
public class RecursiveFisheyeZoomAction extends SelectionAction {

	/**
	 * Id of the action.
	 */
	public static final String RECURSIVE_ZOOM = "RecursiveFisheyeZoom";
	
	private boolean zoomIn;
	
	
	/**
	 * Constructor.
	 * 
	 * @param part The workbench part associated with this action
	 */
	public RecursiveFisheyeZoomAction(IWorkbenchPart part) {
		super(part);
	}

	
	/**
	 * Invokes the action.
	 */
	public void run() {
		
		ZoomableNodeContainerEditPart node = (ZoomableNodeContainerEditPart)getSelectedObjects().get(0);
		RecursiveFisheyeZoom cmd = new RecursiveFisheyeZoom(zoomIn);
		cmd.setTarget((ZoomableNodeContainer)node.getModel());
		execute(cmd);
		
	}

	
	/**
	 * Returns true if this action can be invoked, false otherwise.
	 * 
	 * @return true if the action is available, false otherwise
	 */
	protected boolean calculateEnabled() {
		
		if (getSelectedObjects().isEmpty()) {
			return false;
		}
		
		if(getSelectedObjects().size() > 1) {
			return false;
		}
		
		Object selectedNode = getSelectedObjects().get(0);
		if(selectedNode instanceof ZoomableNodeContainerEditPart) {
			ZoomableNodeContainerEditPart node = (ZoomableNodeContainerEditPart)selectedNode;
			ZoomableNodeContainer model = (ZoomableNodeContainer)node.getModel();
			if(!model.isZoomedIn()) {
				setText("Zoom recursively in");
				zoomIn = true;
				return true;
			}
			if(model.getChildCount() > 0) {
				setText("Zoom recursively out");
				zoomIn = false;
				return true;
			}
		}
		
		return false;
		
	}
	
	
    /*
     *  (non-Javadoc)
     * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#init()
     */
    protected void init() {
    	setId(RECURSIVE_ZOOM);
		setText("Zoom recursively");
		
		setImageDescriptor(ImageDescriptor
				.createFromFile(AdoraPlugin.class, "icons/magnifyingglass24.gif"));
    }
	@Override
	public void dispose() {
		setWorkbenchPart(null);
		super.dispose();
	}
}
