package ch.unizh.ifi.adora.actions.representation;

import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;

import ch.unizh.ifi.adora.parts.ConnectionEditPart;
import ch.unizh.ifi.adora.view.figures.ConnectionFigure;

/**
 * Action to route a single line.
 *
 * @author				Tobias Reinhard
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				2008-03-12 TR First Version
 * @version				$Date: 2009-12-23 14:25:55 +0100 (Wed, 23 Dec 2009) $, $Author: jeanneret $, $Revision: 5005 $
 *
 */
public class RerouteConnectionAction extends SelectionAction {

	/**
	 * Id of the action.
	 */
	public static final String REROUTE_LINE = "RerouteLine";
	
	
	/**
	 * Constructor.
	 * 
	 * @param part The workbench part associated with this action
	 */
	public RerouteConnectionAction(IWorkbenchPart part) {
		super(part);
	}
	
	
	/**
	 * Invokes the action.
	 */
	public void run() {
		
		Object selectedNode = getSelectedObjects().get(0);
		ConnectionEditPart part = (ConnectionEditPart)selectedNode;

		((ConnectionFigure)part.getFigure()).layout();
		
	}
	
	
	/**
	 * Returns true if exactly one line is selected.
	 * 
	 * @return true is exactly one line is selected
	 */
	protected boolean calculateEnabled() {
		
		if (getSelectedObjects().isEmpty()) {
			return false;
		}
		
		if(getSelectedObjects().size() > 1) {
			return false;
		}
		
		Object selectedNode = getSelectedObjects().get(0);
		if(selectedNode instanceof ConnectionEditPart) {
			return true;
		}
		
		return false;
	}

	
    protected void init() {
    	setId(REROUTE_LINE);
		setText("Reroute line");
		
		ISharedImages sharedImages = PlatformUI.getWorkbench().getSharedImages();
		setImageDescriptor(sharedImages.getImageDescriptor(ISharedImages.IMG_TOOL_REDO));
    }
    
	@Override
	public void dispose() {
		setWorkbenchPart(null);
		super.dispose();
	}
}
