package ch.unizh.ifi.adora.actions.representation;

import java.util.Iterator;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionFactory;

import ch.unizh.ifi.adora.model.commands.representation.RepresentationStateChangeTree;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.parts.tree.NodeTreeEditPart;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;

/**
 * Shows/Hides all nodes, which are selected in the Outline-Tree and
 * whose visibility change 
 *
 * @author				Ivo Vigan
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			22.03.2006 IV First Version
 * 						18.07.2006 SM Comments & Refactoring
 * @version			$Date: 2009-12-23 14:25:55 +0100 (Wed, 23 Dec 2009) $, $Author: jeanneret $, $Revision: 5005 $
 */
public class TreeFilterAction extends SelectionAction {
   
    /**
     * Instantiates a new TreeFilterAction
     * @param part the IWorkbenchPart where this action is appended.
     */
    public TreeFilterAction(IWorkbenchPart part) {
        super(part);
    }

    /**
     * Filteres out a node visually if 
     * the user clicks the "invert view" 
     * menuitem in the outline tree pane.
     */
    public void run() {
        Object o;
        Node n;
        RepresentationStateChangeTree cmd; 
        
        Iterator iter = getSelectedObjects().iterator();
        //For all selected Objects
        while (iter.hasNext()){
            o = iter.next();
            if (o instanceof NodeTreeEditPart){
                n = ((Node)((NodeTreeEditPart)o).getModel());
                if(n instanceof Scenario){
                    for(Scenario s: ((Scenario)n).getAllScenarioChildren()){
                        cmd = new RepresentationStateChangeTree(s);
                        executeCommandOnStack(cmd);
                    }
                }
                cmd = new RepresentationStateChangeTree(n);
                executeCommandOnStack(cmd);
            }
        }
    }

    /*
     * (non-Javadoc)
     * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#calculateEnabled()
     */
    protected boolean calculateEnabled() {
    	// TODO Workaround for GEF-Bug 82622/39369.  Should be removed when 39369 is fixed.
		// should always return true as workaround.
        return true;
    }
    
    /*
     *  (non-Javadoc)
     * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#init()
     */
    protected void init() {
        setId(ActionFactory.HELP_CONTENTS.getId());
        setText("Invert Visibility");
    }
    
    /*
     * (non-Javadoc)
     * @see org.eclipse.jface.action.IAction#isEnabled()
     */
    public boolean isEnabled(){
        Object o;
        Node n;
        
        Iterator iter = getSelectedObjects().iterator();
        //For all selected Objects
        while (iter.hasNext()){
            o = iter.next();
            if (o instanceof NodeTreeEditPart){
                n = ((Node)((NodeTreeEditPart)o).getModel());
                
                //If fisheye-zoom has been applied on parent: return false
                if(n.getParent() instanceof ZoomableNodeContainer){
                    if(((ZoomableNodeContainer)n.getParent()).isAnchestorZoomedOut()){
                        return false;
                    }
                }
                //If scenraio-zoom has been applied on scenario-parent: return false                
                if(n instanceof Scenario){
                   if(((Scenario)n).getScenarioParent()!=null){
                       if(((Scenario)n).getScenarioParent().isAnchestorZoomedOut()){
                           return false;
                       }
                   }
                }
                //If view manager has been applied which affects 
                //the current node: return false
                if(((AdoraModel)n.getParent().getRootContainer()).
                        getViewFilterState(n.getClass())){
                    return false;
                }
                
                //If parent isn't visible, child can't set it's 
                //visible state to false.
                if (!(n.getParent() instanceof AdoraModel)&&
                        !((Node)n.getParent()).isVisible()){
                    return false;
                }
            }
        }
        return true;
    }
    
	 /**
     * Executes a given {@link org.eclipse.gef.commands.Command} on the {@link org.eclipse.gef.commands.CommandStack} 
     * of the editor.
     *
     * @param cmd The {@link org.eclipse.gef.commands.Command} which gets executed on the 
     * {@link org.eclipse.gef.commands.CommandStack}.
     */
    protected void executeCommandOnStack(Command cmd){
    	IWorkbenchPage page = PlatformUI.getWorkbench()
		.getActiveWorkbenchWindow().getActivePage();
		
    	try {
			IEditorPart editor = page.getActiveEditor();
			if (editor instanceof AdoraModelEditor) {
				((AdoraModelEditor)editor).getCommandStack().execute(cmd);
			}
		} catch (Exception ex) {
			ex.printStackTrace();
		}
    }
    
	@Override
	public void dispose() {
		setWorkbenchPart(null);
		super.dispose();
	}
}