package ch.unizh.ifi.adora.actions.representation;

import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.model.representation.ScenarioConnection;

/**
 * This class' run method executes the UserView algorithm.
 * 
 * @author Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 16.03.2006 SM First Version<br>
 *          23.01.2008 PS Refactoring & Use of "Observers"
 * @version $Date: 2008-01-23 16:17:45 +0100 (Wed, 23 Jan 2008) $, $Author: schoeni $, $Revision: 3491 $
 */
public class UserViewAction extends AbstractViewAction {
	/**
     * Describes the icon name for the activated view.
     */
	private final static String ICON_NAME = "icons/userview.gif";

	/**
     * Describes the icon name for the deactivated view.
     */
	private final static String ICON_NAME_DEACTIVATED = "icons/userview_deactivated.gif";

	/**
     * Specifies which Nodetypes are filtered by this view manager.
     */
	public UserViewAction() {
		type = Scenario.class;
	}

	/**
     * Overwritten since the ScenarioView has to hide 2 palette entries.
     * 
     * @param type
     *            Whose ToolEntry's visibility should be updated
     * @param filterApplied
     *            hide/show
     */
	protected void updatePalette(Class type, boolean filterApplied) {
		super.updatePalette(type, filterApplied);
		super.updatePalette(ScenarioConnection.class, filterApplied);
	}

	/*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.actions.representation.AbstractViewAction#getIconDeactivatedName()
     */
	@Override
	public String getIconDeactivatedName() {
		return ICON_NAME_DEACTIVATED;
	}

	/*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.actions.representation.AbstractViewAction#getIconName()
     */
	@Override
	public String getIconName() {
		return ICON_NAME;
	}
}