package ch.unizh.ifi.adora.model.commands;

import ch.unizh.ifi.adora.model.geometry.Point;
import ch.unizh.ifi.adora.model.geometry.Rectangle;
import ch.unizh.ifi.adora.model.representation.zoom.IZoomNode;
import ch.unizh.ifi.adora.utils.ZoomAlgorithmsManager;

/**
 * Command to allocate additional white space inside a node.
 *
 * @author				Tobias Reinhard
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				2007-10-23 TR First Version
 * @version				$Date: 2007-11-05 10:41:57 +0100 (Mon, 05 Nov 2007) $, $Author: schoeni $, $Revision: 3231 $
 *
 */
public class AllocateWhiteSpace extends SavableCommand {

	/**
	 * The node the white space should be allocated in.
	 */
	private IZoomNode parentNode;
	
	/**
	 * The bounds of the additional white space.
	 */
	private Rectangle requestedWhiteSpace;
	
	
	/**
	 * Constructor. The coordinates of the bounds that represent the additional
	 * white space have to be relative to the node <i>parent</i>.
	 * 
	 * @param parent the node the white space should be allocated in.
	 * @param requestedSpace the bounds of the requested space.
	 */
	public AllocateWhiteSpace(IZoomNode parent, Rectangle requestedSpace) {
		super("Allocate white space");
		parentNode = parent;
		requestedWhiteSpace = requestedSpace;
		
	}
	
	
	/**
	 * Executes the command (i.e. adds additional white space to the model).
	 * The moethod doesn't change the layout of the model if the requested
	 * white space is already available.
	 */
	public void execute() {
		
		saveModel();
		
		ZoomAlgorithmsManager.getInstance().getInsertAlgorithm().allocateWhiteSpace(parentNode, requestedWhiteSpace);
		
	}
	
	@Override
	public String toString() {
		return "Allocate whitespace (" + requestedWhiteSpace + ")";
	}
	
	public Point getLocation() {
		return requestedWhiteSpace.getLocation();
	}
}
