package ch.unizh.ifi.adora.model.commands;

import ch.unizh.ifi.adora.model.representation.Association;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.Role;
import ch.unizh.ifi.adora.representation.abstractions.algorithm.ConnectionAbstractionAlgorithm;

/**
 * Represents a create command for an association in 
 * an ADORA model. 
 *
 * @author				smeier
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			31.10.2005 SM First Version
 * @version			$Date: 2007-07-04 16:38:32 +0200 (Wed, 04 Jul 2007) $, $Author: vontobel $, $Revision: 2853 $
 * @responsibilities	Represents the command which creates an 
 *                      association.
 *
 */
public class AssociationCreate extends NodeConnectionCreate {
	private Role roleFrom;
	private Role roleTo;
	/**
	 * Instantiate a command that can create a connection between two shapes.
	 * 
	 * @param source    The source endpoint 
	 *                  (a non-null Shape instance)
	 * @throws          IllegalArgumentException
	 *                  if source is null
	 */
	public AssociationCreate(Node source) {
		super(source);
		this.setLabel("Association Create");
	}

	/**
	 * The execute command for creating a new association.
	 * 
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() {
        Association connection = new Association(getSource(), getTarget());
		connection.setSourceClickPoint(getSourceClickPoint());
		
		/*
		 * Set roles (if defined)
		 */
		if (this.roleFrom != null)
			connection.setRoleFrom(this.roleFrom);
		if (this.roleTo != null)
			connection.setRoleTo(this.roleTo);
		
		// TODO Check if this is correct???
		// The target click point does not exists at the 
		// time the command is executed.
		connection.setTargetClickPoint(getTargetClickPoint());
        setConnection(connection);
        ((ConnectableElement)getSource()).connectionCreated(connection);
        ((ConnectableElement)getTarget()).connectionCreated(connection);
        
        ConnectionAbstractionAlgorithm.getInstance().
        	recalculateInterRelations(connection);
   }
	public Role getRoleFrom() {
		return roleFrom;
	}
	public void setRoleFrom(Role roleFrom) {
		this.roleFrom = roleFrom;
	}
	public Role getRoleTo() {
		return roleTo;
	}
	public void setRoleTo(Role roleTo) {
		this.roleTo = roleTo;
	}
}