package ch.unizh.ifi.adora.model.commands;

import ch.unizh.ifi.adora.model.representation.Association;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.representation.abstractions.algorithm.ConnectionAbstractionAlgorithm;

/**
 * 
 * Represents the delete command for an association.
 *
 * @author				smeier
 * 
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * 
 * @history			31.10.2005 SM First Version
 * 					08.08.2006 JS Replaced association attribute by calls 
 * 					              to getConnection()
 * @version			$Date: 2006-08-10 15:05:44 +0200 (Thu, 10 Aug 2006) $, $Author: vigan $, $Revision: 2047 $
 * 
 * @responsibilities	Represents the delete command for an association.	
 */
public class AssociationDelete extends ConnectionDelete {

	/**
	 * Creates a command that will disconnect a 
	 * Association from its endpoints.
	 * 
	 * @param conn The Association instance to disconnect 
	 *             (non-null)
	 *             
	 * @throws IllegalArgumentException
	 *         If conn is null.
	 */
	public AssociationDelete(Association association) {
		super(association);
		setLabel("Association Delete");
	}

	/**
	 * Reorganises all Interrelationships.
	 */
	public void execute() {
		Node target = (Node) getConnection().getTarget();
		Node source = (Node) getConnection().getSource();

		ConnectionAbstractionAlgorithm.getInstance().recalculateInterRelations(source,target);
		
		super.execute();
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	public void undo() {
		super.undo();
		ConnectionAbstractionAlgorithm.getInstance().recalculateInterRelations(
				(Node)getConnection().getSource(),
				(Node)getConnection().getTarget());
	}
}