package ch.unizh.ifi.adora.model.commands;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.model.representation.Association;
import ch.unizh.ifi.adora.representation.abstractions.algorithm.ConnectionAbstractionAlgorithm;

/**
 * This {@link org.eclipse.gef.commands.Command} is used to 
 * calculate the {@link ch.unizh.ifi.adora.model.representation.Interrelationship}
 * of child-{@link ch.unizh.ifi.adora.model.representation.Node}s which 
 * are connected by {@link Association}s after an {@link Association} of a parent
 * {link ch.unizh.ifi.adora.model.representation.Node} has been manually abstracted.
 *
 * @author				Ivo Vigan
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			20.07.2006 IV First Version
 * 						21.07.2006 SM Comments reformatted and completed.
 * @version			$Date: 2006-08-21 14:04:43 +0200 (Mon, 21 Aug 2006) $, $Author: vigan $, $Revision: 2087 $
 */
public class CalculateInterRelations extends Command {
	
	/**
	 * {@link Association}, which got manually abstracted, and therefore need
	 * to be considered for recalculating the 
	 * {@link ch.unizh.ifi.adora.model.representation.Interrelationship}s
	 * of {@link Association}s of child-{@link ch.unizh.ifi.adora.model.representation.Node}s.
	 */
	private Association association;
	
	/**
	 * Creates a new CalculateInterRelations command.
	 * 
	 * @param {@link Association}, which got manually abstracted, and therefore need
	 * to be considered for recalculating the
	 *  {@link ch.unizh.ifi.adora.model.representation.Interrelationship}s
	 * of {@link Association}s of children.
	 */
    public CalculateInterRelations(Association association){
    	this.association = association;
        setLabel("Calculates the InterRelations");
    }
    
    /*
     *  (non-Javadoc)
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute() {
    	ConnectionAbstractionAlgorithm.getInstance().
    		recalculateInterRelations(association);
    }
    
    /**
     *No undoing needed.
     *
     *@retun false 
     */
    public boolean canUndo() {
        return false;
    }
}