package ch.unizh.ifi.adora.model.commands;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.model.representation.ScenarioConnection;

public class ComponentScenarioConnectionCopyCommand extends Command {
	private ArrayList<AspectContainer> aspectList;

	private HashMap<String, String> scenarioSourceAndTarget;

	public ComponentScenarioConnectionCopyCommand(
			ArrayList<AspectContainer> aspectList) {
		scenarioSourceAndTarget = new HashMap<String, String>();
		this.aspectList = aspectList;
		for (AspectContainer a : aspectList) {
			searchForComponentScenarioConnections(a);
		}
	}

	private void searchForComponentScenarioConnections(AspectContainer a) {
		Iterator<ModelElement> iterator = a.getRecursiveChildrenIterator();
		while (iterator.hasNext()) {
			ModelElement modelElement = iterator.next();
			if (modelElement instanceof Scenario) {
				Scenario scenario = (Scenario) modelElement;
				for (Connection con : scenario.getSourceConnections()) {
					if (con instanceof ScenarioConnection) {
						ScenarioConnection scenarioCon = (ScenarioConnection) con;
						if (scenarioCon.getTarget().getParent() != scenario
								.getParent()) {
							scenarioSourceAndTarget.put(scenarioCon.getTarget()
									.getName(), scenario.getName());
						}
					}
				}
			}
		}
	}

	public void execute() {
		createScenario();
	}

	private void createScenario() {
		ScenarioConnectionCreate command = null;
		Iterator<ModelElement> iterator = AdoraPlugin.getActiveModel()
				.getRecursiveChildrenIterator();
		while (iterator.hasNext()) {
			ModelElement modelElement = iterator.next();
			if (modelElement instanceof Scenario) {
				Scenario scenario = (Scenario) modelElement;
				if (scenarioSourceAndTarget.get(scenario.getName()) != null
						&& scenario.isVisible()
						&& scenario.getTargetConnections().size() == 0) {
					command = new ScenarioConnectionCreate(
							getScenario(scenarioSourceAndTarget.get(scenario
									.getName())));
					command.setTarget(scenario);
					scenarioSourceAndTarget.remove(scenario.getName());
					if (command != null) {
						command.execute();
					}
				}

			}
		}
	}

	private Scenario getScenario(String scenarioName) {
		Iterator<ModelElement> iterator = AdoraPlugin.getActiveModel()
				.getRecursiveChildrenIterator();
		while (iterator.hasNext()) {
			ModelElement modelElement = iterator.next();
			if (modelElement instanceof Scenario) {
				Scenario scenario = (Scenario) modelElement;
				if (scenario.getName().equals(scenarioName)) {
					return scenario;
				}
			}
		}
		return null;
	}
}
