package ch.unizh.ifi.adora.model.commands;

import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.ui.views.explosiveZoom.ExplosiveZoomHandler;

/**
 * Performs an explosive instead of a fisheye zoom. This is used for the validation
 * of the zoom algorithm.
 * The explosive zoom is handled in the <code>ExplosiveZoomHandler</code>.
 *
 * @author			Tobias Reinhard
 * @copyright		Requirements Engineering Research Group<br>
 *					Department of Informatics<br>
 *					University of Zurich<br>
 * @history			2008-11-07 TR First Version
 * 
 * @version			$Date: 2008-12-06 13:30:59 +0100 (Sat, 06 Dec 2008) $, $Author: jehle $, $Revision: 3948 $
 */
public class ExplosiveZoom extends SavableCommand {

	/**
     * Target on which zooming is performed.
     */
	transient private ZoomableNodeContainer zoomTarget;
	
	/**
	 * Creates the explosive zoom command.
	 */
	public ExplosiveZoom()
	{
		super("Explosive Zoom");
	}

	/**
	 * Sets the target container for the zoom operation.
	 * 
	 * @param zoomTarget != null.
	 */
	public void setTarget(ZoomableNodeContainer zoomTarget)
	{
		this.zoomTarget = zoomTarget;
	}
	

	/**
	 * When explosive zoom is applied on a model element, this command is
	 * executed and handled by the <code>ExplosiveZoomHanlder</code>.
	 */
	public void execute()
	{
		ExplosiveZoomHandler exHandler = new ExplosiveZoomHandler(this.zoomTarget);
		exHandler.openEditor();
	}	
}