package ch.unizh.ifi.adora.model.commands;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.ConstraintConnection;
import ch.unizh.ifi.adora.model.representation.ConstraintImplicationConnection;
import ch.unizh.ifi.adora.model.representation.DecisionPointFamilyConnection;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.VariationPointConnection;
import ch.unizh.ifi.adora.model.representation.variability.Constraint;
import ch.unizh.ifi.adora.model.representation.variability.Decision;
import ch.unizh.ifi.adora.model.representation.variability.VariationPoint;

/**
 * This {@link Command}  deletes all {@link ConstraintConnection}s 
 * aswell as all {@link VariationPointConnection}s in the model.
 * 
 * @author    Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history   2008-11-19 IV First Version
 * @version 
 */
public class HideAllConstraintConnections extends Command {
	
	/**
	 * The AdoraModel.
	 */
	private AdoraModel adoraModel;
	
	/**
	 * Generates this {@link Command} the hide all the {@link VariationPoint}s
	 * in the Editor.
	 * 
	 * @param adoraModel The AdoraModel.
	 */
	public HideAllConstraintConnections(AdoraModel adoraModel) {
		this.adoraModel = adoraModel;
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() {
		
    	for (Decision d : adoraModel.getDecisions()) {
    		
    		boolean contained = false;
    		//Is d contained in some visible vp => we couldn't delete the DecisionPointFamilyConnection
    		for (VariationPoint vp : adoraModel.getVariationPoints()) {
    			if (vp.isVisible() && vp.getDecisions().contains(d)) {
    				contained = true;
    				continue;
    			}
    		}
    		if (!contained){
        		for (DecisionPointFamilyConnection dpfc : d.getFamilyLines()) {
    				ConnectionDelete conDel = new ConnectionDelete(dpfc);
    				conDel.execute();
    			}
				d.removeFamilyLines();
    		}

    		
    		
    		for (JoinRelation jr : d.getJoinRelations()) {
    			if (!jr.isVisible() && jr.abstrectedIn() != null) {
    				jr = jr.abstrectedIn();
    			}
    			for (Connection c : jr.getConnections()) {
    				if (c instanceof ConstraintConnection ||
    						c instanceof ConstraintImplicationConnection) {
    					ConnectionDelete conDel = new ConnectionDelete(c);
    					conDel.execute();
    				}
    			}
    		}
    	}
    	for (Constraint con : adoraModel.getConstraints()) 
        	con.setVisible(false);
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#redo()
	 */
	public void redo() {
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	public void undo() {
	}
}