package ch.unizh.ifi.adora.model.commands;

import org.eclipse.gef.commands.Command;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.DecisionPointFamilyConnection;
import ch.unizh.ifi.adora.model.representation.VariationPointConnection;
import ch.unizh.ifi.adora.model.representation.variability.Constraint;
import ch.unizh.ifi.adora.model.representation.variability.Decision;
import ch.unizh.ifi.adora.model.representation.variability.VariationPoint;

/**
 * This {@link Command}  deletes all visible {@link VariationPoint}s 
 * in the model.
 * 
 * @author    Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history   2008-11-19 IV First Version
 * @version 
 */
public class HideAllVariationPoints extends Command {
	
	/**
	 * The AdoraModel.
	 */
	private AdoraModel adoraModel;
	
	
	/**
	 * Generates this {@link Command} the hide all the {@link VariationPoint}s
	 * in the Editor.
	 * 
	 * @param adoraModel The AdoraModel.
	 */
	public HideAllVariationPoints(AdoraModel adoraModel) {
		this.adoraModel = adoraModel;
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() {
		for (VariationPoint vp : adoraModel.getVariationPoints()) {
			for (Decision d : vp.getDecisions()) {
				
				//delete DecisionFamily if D only appears in constr
				boolean deletetFamilyConnection = true;
				for (Constraint ct : adoraModel.getConstraints()) {
					if (ct.containsDecision(d) && ct.isVisible()) {
						deletetFamilyConnection = false;
						break;
					}
				}
				if (deletetFamilyConnection) {
    				for (DecisionPointFamilyConnection dpfc : d.getFamilyLines()) {
    					ConnectionDelete conDel = new ConnectionDelete(dpfc);
    					conDel.execute();
    				}
				d.removeFamilyLines();
				}
			}
		
			for (VariationPointConnection vpc : vp.getVPConnections()) {
				ConnectionDelete conDel = new ConnectionDelete(vpc);
				conDel.execute();
			}
			vp.removeVPConnections();
			vp.setVisible(false);
		}
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#redo()
	 */
	public void redo() {
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	public void undo() {
	}
}