package ch.unizh.ifi.adora.model.commands;


import java.util.ArrayList;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.ConstraintConnection;
import ch.unizh.ifi.adora.model.representation.ConstraintImplicationConnection;
import ch.unizh.ifi.adora.model.representation.DecisionPointFamilyConnection;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.VariationPointConnection;
import ch.unizh.ifi.adora.model.representation.variability.Constraint;
import ch.unizh.ifi.adora.model.representation.variability.Decision;
import ch.unizh.ifi.adora.model.representation.variability.VariationPoint;

/**
 * This {@link Command} deletes all selected {@link VariationPoint}s 
 * and {@link Constraint}s in the model.
 * 
 * @author    Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history   2008-11-19 IV First Version
 * @version 
 */
public class HideSelectedConstraints extends Command {
	
	/**
	 * The AdoraModel.
	 */
	private AdoraModel adoraModel;
	
	/**
	 * Stores the selected {@link Decision}s
	 */
	private ArrayList<Decision> selectedDecision;
	
	/**
	 * Generates this {@link Command} the hide all the {@link VariationPoint}s
	 * and {@link Constraint}s in the Editor.
	 * 
	 * @param adoraModel The AdoraModel.
	 * @param selectedDecision The selected {@link Decision}s.
	 */
	public HideSelectedConstraints(AdoraModel adoraModel, ArrayList<Decision> selectedDecision) {
		this.adoraModel = adoraModel;
		this.selectedDecision = selectedDecision;
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() {
		
		for (Decision dec : selectedDecision) {
			//delete VariationPoints
			for (VariationPoint vp : adoraModel.getVariationPoints()) {
    			if (vp.getDecisions().contains(dec)) {
        			for (Decision d : vp.getDecisions()) {
        				for (DecisionPointFamilyConnection dpfc : d.getFamilyLines()) {
        					ConnectionDelete conDel = new ConnectionDelete(dpfc);
        					conDel.execute();
        				}
        			}
		
        			for (VariationPointConnection vpc : vp.getVPConnections()) {
        				ConnectionDelete conDel = new ConnectionDelete(vpc);
        				conDel.execute();
        			}
        			vp.removeVPConnections();
        			vp.setVisible(false);
    			}
    		}
			//delete Constraints
			for (Constraint cnstr : adoraModel.getConstraints()) {
				if (cnstr.containsDecision(dec)) {
					
					for (Decision d : adoraModel.getDecisions()) {
						if (cnstr.containsDecision(d)) {
							
							
							//delete DecisionFamily if D only appears in constr
							boolean deletetFamilyConnection = true;
							for (Constraint ct : adoraModel.getConstraints()) {
								if (ct.containsDecision(dec) && ct != cnstr && ct.isVisible()) {
									deletetFamilyConnection = false;
									break;
								}
							}
							for (VariationPoint vp : adoraModel.getVariationPoints()) {
								if (vp.getDecisions().contains(d) && vp.isVisible()) {
									deletetFamilyConnection = false;
									break;
								}
								
							}
							if (deletetFamilyConnection) {
    							for (DecisionPointFamilyConnection dpfc : d.getFamilyLines()) {
    								ConnectionDelete conDel = new ConnectionDelete(dpfc);
    								conDel.execute();
    							}
							}
				    			
        					for (JoinRelation jr : d.getJoinRelations()) {
        		    			if (!jr.isVisible() && jr.abstrectedIn() != null) {
        		    				jr = jr.abstrectedIn();
        		    			}
        		    			for (Connection c : jr.getConnections()) {
        		    				if (c instanceof ConstraintConnection ||
        		    						c instanceof ConstraintImplicationConnection) {
        		    					ConnectionDelete conDel = new ConnectionDelete(c);
        		    					conDel.execute();
        		    				}
        		    			}
        		    		}
						}
					}
				}
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#redo()
	 */
	public void redo() {
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	public void undo() {
	}
}