package ch.unizh.ifi.adora.model.commands;

import java.util.List;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.geometry.Dimension;
import ch.unizh.ifi.adora.model.representation.INodeContainer;
import ch.unizh.ifi.adora.model.representation.VisualizableModelElement;
/**
 * This command shrinks a INodeContainer so that there is no unused space
 * on any side of the container.
 * 
 * @author 		Marc Vontobel
 * @copyright 	Requirements Engineering Research Group<br>
 *            	Department of Informatics<br>
 *            	University of Zurich<br>
 * @history 	04.07.2007 IV First Version 02.07.2007 
 * 				04.07.2007 SM Comments reformatted andcompleted.
 * @version 	$Date: 2007-07-04 10:13:21 +0200 (Wed, 04 Jul 2007) $, $Author: vontobel $, $Revision: 2845 $
 * 
 */
public class INodeContainerShrink extends Command {
	private static final int GAP = 20;
	private INodeContainer container;
	
	public INodeContainerShrink(INodeContainer container) {
		this.container = container;
	}
	/**
	 * Shrinks the container
	 */
	public void execute() {
		VisualizableModelElement visualContainer = (VisualizableModelElement) this.container;
		/*
		 * Defining the actual properties
		 */
		int containerWidth = (int) visualContainer.getBounds().getWidth();
		int containerHeight = (int) visualContainer.getBounds().getHeight();
		int containerX = (int) visualContainer.getBounds().getX();
		int containerY = (int) visualContainer.getBounds().getY();
		
		//System.out.println("Container (BEGINNING): (X: " + containerX + ", Y: " + containerY + ", W: " + containerWidth + ", H: " + containerHeight + ");");
		
		/*
		 * Variables to save the extrema of the containers child objects
		 */
		int leftestX = (int) visualContainer.getBounds().getWidth();
		int rightestX = 0;
		int highestY = (int) visualContainer.getBounds().getHeight();
		int lowestY = 0;
		
		/*
		 * Iterate over all childs of the container to define the extremas
		 */
		List<ModelElement> children = this.container.getChildren();
		for (ModelElement child : children) {
			if (child instanceof VisualizableModelElement) {
				int x = (int) ((VisualizableModelElement) child).getBounds().getX();
				int y = (int) ((VisualizableModelElement) child).getBounds().getY();
				int width = (int) ((VisualizableModelElement) child).getBounds().getWidth();
				int height = (int) ((VisualizableModelElement) child).getBounds().getHeight();
				leftestX = (leftestX > x) ? x : leftestX;
				rightestX = (rightestX < (x+width)) ? (x+width) : rightestX;
				highestY = (highestY > y) ? y : highestY;
				lowestY = (lowestY < (y+height)) ? (y+height) : lowestY;
				//System.out.println(child.getClass().toString() + ": (X: " + x + ", Y: " + y + ", W: " + width + ", H: " + height + ");");
			}
		}
		
		/*
		 * Adding the GAP to each side
		 */
		leftestX -= GAP;
		rightestX += GAP;
		highestY -= GAP;
		lowestY += GAP;
		
		
		/*
		 * Moving all the elements to the top left corner
		 */
		for (ModelElement child : children) {
			if (child instanceof VisualizableModelElement) {
				int x = (int) ((VisualizableModelElement) child).getBounds().getX();
				int y = (int) ((VisualizableModelElement) child).getBounds().getY();
				((VisualizableModelElement) child).setLocation(x-leftestX, y-highestY);
			}
		}
		
		//System.out.println("Extrema: (LX: " + leftestX + ", RX: " + rightestX + ", HY: " + highestY + ", LY: " + lowestY + ");");
		
		/*
		 * Resizing the container properties to fit it's children extrema
		 */
		containerWidth -= (leftestX + (containerWidth-rightestX));
		containerHeight -= (highestY + (containerHeight-lowestY));
		containerX += leftestX;
		containerY += highestY;
		
		//System.out.println("Container (END): (X: " + containerX + ", Y: " + containerY + ", W: " + containerWidth + ", H: " + containerHeight + ");");
		
		/*
		 * Adding the properties to the container
		 */
		visualContainer.setLocation(containerX, containerY);
		visualContainer.setSize(new Dimension(containerWidth, containerHeight));
	}
}
