package ch.unizh.ifi.adora.model.commands;

import java.util.List;

import ch.unizh.ifi.adora.model.geometry.Point;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;

/**
 * Command for inserting a nodegroup into a parent by orientation.
 * 
 * @author Pascal Schoeni
 */
public class InsertNodesVisuallyByOrientation extends InsertNodesVisually {
	/**
     * Orientation NORTH_EAST in the parent
     */
	public static final int NORTH_EAST = 0;

	/**
     * Orientation NORTH_WEST in the parent
     */
	public static final int NORTH_WEST = 1;

	/**
     * Orientation SOUTH_EAST in the parent
     */
	public static final int SOUTH_EAST = 2;

	/**
     * Orientation SOUTH_WEST in the parent
     */
	public static final int SOUTH_WEST = 3;

	/**
     * the orientation of the nodelist which will be inserted
     */
	private int orientation;

	public InsertNodesVisuallyByOrientation(ZoomableNodeContainer parent,
			List<Node> nodes, int orientation) {
		super(parent, nodes);

		this.orientation = orientation;
	}

	@Override
	protected Point getLocation() {
		if (orientation == NORTH_WEST) {
			return new Point(0, 0);
		}

		if (orientation == SOUTH_WEST) {
			return new Point(0, (int) getParent().getBounds().getHeight()
					- (int) getDimension().getHeight());
		}

		if (orientation == NORTH_EAST) {
			return new Point((int) getParent().getBounds().getWidth()
					- (int) getDimension().getWidth(), 0);
		}

		if (orientation == SOUTH_EAST) {
			return new Point((int) getParent().getBounds().getWidth()
					- (int) getDimension().getWidth(), (int) getParent()
					.getBounds().getHeight()
					- (int) getDimension().getHeight());
		}

		throw new IllegalStateException("orientation not clear");
	}
}
