package ch.unizh.ifi.adora.model.commands;

import java.util.List;

import ch.unizh.ifi.adora.model.geometry.Point;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.Transition;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;

/**
 * Command for inserting a nodegroup into a parent by the position of a
 * transition.
 * 
 * @author Pascal Schoeni
 */
public class InsertNodesVisuallyByTransition extends InsertNodesVisually {
	/**
     * the transition, from which the location is calculated
     */
	private Transition transition;

	public InsertNodesVisuallyByTransition(ZoomableNodeContainer parent,
			List<Node> nodes, Transition transition) {
		super(parent, nodes);

		this.transition = transition;
	}

	@Override
	protected Point getLocation() {
		/*
         * calculate the location, first get the two points source / target
         */
		Point source = addPoints(transition.getSource().getLocation(),
				transition.getSourceClickPoint());
		Point target = addPoints(transition.getTarget().getLocation(),
				transition.getTargetClickPoint());

		/*
         * calculate deltas of the two points and a first x / y combination
         */
		int dx = (int) Math.abs(source.getX() - target.getX());
		int dy = (int) Math.abs(source.getY() - target.getY());

		int x = (int) (source.getX() + target.getX()) / 2;
		int y = (int) (source.getY() + target.getY()) / 2;

		if (dx > dy) {
			// the horizontal offset is bigger than the vertical offset
			y -= getDimension().getHeight() / 2;
			if (y < 0) {
				y = 0;
			}
		} else {
			// the vertical offset is bigger than the horizontal offset
			x -= getDimension().getWidth() / 2;
			if (x < 0) {
				x = 0;
			}
		}

		return new Point(x, y);
	}

	/**
     * Sums the two points
     * 
     * @param a
     * @param b
     * @return
     */
	private Point addPoints(Point a, Point b) {
		return new Point(a.x + b.x, a.y + b.y);
	}
}
