package ch.unizh.ifi.adora.model.commands;

import ch.unizh.ifi.adora.model.IContainer;
import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.variability.Decision;

/**
 * Represents a create command for an {@link JoinRelation}.
 * 
 * @author Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 19.07.2006 IV First Version
 * @version $Date: 2010-01-28 04:43:33 +0100 (Thu, 28 Jan 2010) $, $Author: vigan $, $Revision: 5098 $
 */
public class JoinRelationCreate extends ConnectionCreate {

	/**
     * Instantiate a command that can create a connection between a
     * {@link ch.unizh.ifi.adora.model.representation.Transition} and a
     * {@link ch.unizh.ifi.adora.model.representation.State}.
     * 
     * @param source
     *            The source endpoint (a non-null Shape instance)
     * @throws IllegalArgumentException
     *             if source is null
     */
	public JoinRelationCreate(Node source) {
		super(source);
		setLabel("Join Relationship Creation");
	}

	/*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.model.commands.ConnectionCreate#getSource()
     */
	public Node getSource() {
		return (Node) super.getSource();
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.gef.commands.Command#execute()
     */
	public void execute() {
		JoinRelation connection = new JoinRelation(getSource(), getTarget());
		connection.setSourceClickPoint(getSourceClickPoint());
		connection.setTargetClickPoint(getTargetClickPoint());
		setConnection(connection);
		((ConnectableElement) getSource()).connectionCreated(connection);
		((ConnectableElement) getTarget()).connectionCreated(connection);

		connection.setManuallyAbstracted(JoinRelationHelper.isAbstract(
				connection.getSource(), connection.getTarget()));
		
		//generate Decision for this JR
		boolean inAC = false;
		
		if (getSource() instanceof AspectContainer) {
			inAC = ((AspectContainer)getSource()).isVariability();
		} else {
    		IContainer parent = getSource().getParent();
    		while (parent != null) {
    			if (parent instanceof AspectContainer) {
    				inAC = (((AspectContainer)parent).isVariability());
    				break;
    			}
    			parent = parent.getParent();
    		}
		}
		
		if (inAC || getSource() instanceof AspectContainer && ((AspectContainer)getSource()).isVariability()) {
    		Decision dec = new Decision(connection);
    		
    		//1) Check if the Decision has to be set to false 
   		 	if (!dec.hasTruePathToCommonality())
    			dec.setDecisionValueAutomatically(false);
		}
	}
}