package ch.unizh.ifi.adora.model.commands;

import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.JoinRelation;

/**
 * Command for reconnecting a JoinRelation.
 * 
 * @author Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 18.08.2006 IV First Version
 * @version $Date: 2008-01-25 14:45:24 +0100 (Fri, 25 Jan 2008) $, $Author: schoeni $, $Revision: 3504 $
 */
public class JoinRelationReconnect extends ConnectionReconnect {

	/**
     * Stores the JoinRelation to be reconnected.
     */
	private JoinRelation conn;

	/**
     * Constructs a new JoinRelationReconnect command
     * 
     * @param conn
     *            The JoinRelation.
     */
	public JoinRelationReconnect(JoinRelation conn) {
		super(conn);
		this.conn = conn;
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.gef.commands.Command#execute()
     */
	public void execute() {
		super.execute();

		if (newSource != null && newSource instanceof AspectContainer) {
			conn.setWeavingType(JoinRelation.noType);
		}

		if (newSource != null) {
			conn.setManuallyAbstracted(JoinRelationHelper.isAbstract(newSource,
					conn.getTarget()));
		} else {
			conn.setManuallyAbstracted(JoinRelationHelper.isAbstract(conn
					.getSource(), newTarget));
		}
	}
}