package ch.unizh.ifi.adora.model.commands;

import java.util.ArrayList;
import java.util.Iterator;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.geometry.Dimension;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.INodeContainer;
import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.model.representation.State;
import ch.unizh.ifi.adora.model.representation.ZoomableNode;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.model.util.NodeGroup;
import ch.unizh.ifi.adora.model.util.NodeGroups;

/**
 * This class is responsible for reordering the layout and adjusting it after a
 * weaving sequence.
 * 
 * @author Anil Kandrical
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 2008-06-25 PS First Version
 * @version $Date: 2009-06-25 10:45:03 +0200 (Do, 25 Jun 2009) $, $Author: anil
 *          $, $Revision: 4403 $
 */
public class LayoutOrderingCommand extends Command {
	public LayoutOrderingCommand() {
	}

	/**
	 * Makes layout more readable, especially for large state charts.
	 * 
	 * @param model
	 */
	public void execute() {
		//TODO: IV: COMMENTED OUT, DESTROYS LAYOUT COMPLETELY
		//setInitialArrangementOfGraphicalElements();
		//this.optimizeGraphicalLayout();
	}

	/**
	 * Layout optimizing starts here by selecting element which is capsulated
	 * the deepest. From these deepest element we order the layout until the
	 * parent looks human friendly.
	 */
	private void optimizeGraphicalLayout() {
		arrrangeElementsBottomUp(this.getLowestHierarchyNode());
	}

	/**
	 * Arrange Elements in each container, so that they look pleasing and do not
	 * use too much space.
	 * 
	 * @param lowestHierarchyNode
	 */
	private void arrrangeElementsBottomUp(
			ArrayList<ModelElement> lowestHierarchyNode) {
		for(int i=0;i<lowestHierarchyNode.size();i++){
		ZoomableNodeContainer znc = (ZoomableNodeContainer) lowestHierarchyNode
				.get(i);
		if (znc.getChildCount() != 0) {
			// there are some scenarios inside the zoomableNodecontainer, which
			// need ordering
			// TODO
			NodeGroups s = znc.getNodeGroups();
		} else {
			INodeContainer parent = znc.getParent();
			if (parent instanceof ZoomableNodeContainer) {
				optimizeContainerLayoutWithMentalMap(parent);
				bottomUpLayoutOptimizer((ZoomableNodeContainer)parent);
			}
		}
		}
	}

	private void bottomUpLayoutOptimizer(ZoomableNodeContainer node) {
		if(node.getParent() instanceof AdoraModel){
			return;
		}
		for(ModelElement m:node.getParent().getChildren()){
			if(m instanceof ZoomableNodeContainer){
				ZoomableNodeContainer zoomable=(ZoomableNodeContainer)m;
				this.optimizeContainerLayoutWithMentalMap(zoomable);
			}
		}
		this.bottomUpLayoutOptimizer((ZoomableNodeContainer)node.getParent());
	}

	private void optimizeContainerLayoutWithMentalMap(INodeContainer parent) {
		if (parent instanceof ZoomableNodeContainer) {
			ZoomableNodeContainer znc = (ZoomableNodeContainer) parent;
			NodeGroups ng = znc.getNodeGroups();
			for (NodeGroup nodeGroup : ng.getNodeGroups()) {
				if (nodeGroup.isComponentGroup()) {
			
				}
				if (nodeGroup.isScenarioGroup()) {
				//ScenarioLayouter command=new ScenarioLayouter(nodeGroup);
				//	command.execute();
				
					
				}
				if (nodeGroup.isStateGroup()) {
					StateLayouter command=new StateLayouter(nodeGroup);
					command.execute();
				}
			}
		}
	}

	/**
	 * Fetch the elements which are the lowest in the total hierachy.
	 */
	private ArrayList<ModelElement> getLowestHierarchyNode() {
		ArrayList<ModelElement> nodes = new ArrayList<ModelElement>();
		ArrayList<HierarchyClass> lowestHierarchyNodes = new ArrayList<HierarchyClass>();
		Iterator<ModelElement> iterator = AdoraPlugin.getActiveModel()
				.getChildrenIterator();
		while (iterator.hasNext()) {
			ModelElement modelElement = iterator.next();
			if (modelElement instanceof ZoomableNodeContainer) {
				if (((ZoomableNodeContainer) modelElement).isVisible()) {
					lowestHierarchyNodes.add((getDeepestNode(modelElement, 0)));
				}
			}
			
		}
		for (HierarchyClass hc : lowestHierarchyNodes) {
			if (hc != null) {
				nodes.add(hc.getModelElement());
			}
		}
		return nodes;
	}

	/**
	 * Method to recursively search for the deepest nodes in the adoramodel.
	 * 
	 * @param hierarchyNode
	 * @param height
	 * @return
	 */
	private HierarchyClass getDeepestNode(ModelElement hierarchyNode, int height) {
		HierarchyClass lowestHierarchyNode = null;
		ZoomableNodeContainer znc = (ZoomableNodeContainer) hierarchyNode;
		ArrayList<HierarchyClass> children = new ArrayList<HierarchyClass>();
		Iterator<ModelElement> iterator = znc.getChildrenIterator();
		if (isNoZoomableNodeContainerInside(znc)) {
			return new HierarchyClass(hierarchyNode, height);
		}
		while (iterator.hasNext()) {
			ModelElement modelElement = iterator.next();
			if (modelElement instanceof ZoomableNodeContainer ) {
				if (((ZoomableNodeContainer) modelElement).isVisible()) {
					children.add(getDeepestNode(modelElement, (height + 1)));
				}
			}
			
		}
		for (int i = 0; i < children.size(); i++) {
			if (i == 0) {
				lowestHierarchyNode = children.get(i);
			}
			if (lowestHierarchyNode.getHeight() < children.get(i).getHeight()) {
				lowestHierarchyNode = children.get(i);
			}
		}
		return lowestHierarchyNode;
	}

	private boolean isNoZoomableNodeContainerInside(ZoomableNodeContainer znc) {
		for (ModelElement modelElement : znc.getChildren()) {
			if (modelElement instanceof ZoomableNodeContainer) {
				return false;
			}
		}
		return true;
	}

	/**
	 * Rearrange minimal size of the elements and set base visibility of the
	 * elements.
	 */
	private void setInitialArrangementOfGraphicalElements() {
		Iterator<ModelElement> iterator = AdoraPlugin.getActiveModel()
				.getRecursiveChildrenIterator();

		while (iterator.hasNext()) {
			ModelElement modelElement = iterator.next();

			if (modelElement instanceof Component) {
				Component c = (Component) modelElement;
				NodeCompact cd = new NodeCompact();
				cd.setTarget(c);
				if (c.getChildCount() != 0) {
					cd.execute();
				} else {
					c.setSize(new Dimension(100, 70));
				}

			}
			if (modelElement instanceof AspectContainer) {
				AspectContainer c = (AspectContainer) modelElement;
				NodeCompact cd = new NodeCompact();
				cd.setTarget(c);
				if (c.getChildCount() != 0) {
					cd.execute();
				} else {
					c.setSize(new Dimension(100, 70));
				}

			}
			if (modelElement instanceof Scenario) {
				Scenario s = (Scenario) modelElement;
				s.setSize(new Dimension(100, 70));
			}
			if (modelElement instanceof State) {
				State s = (State) modelElement;
				NodeCompact cd = new NodeCompact();
				cd.setTarget(s);
				if (s.getChildCount() != 0) {
					cd.execute();
				} else {
					s.setSize(new Dimension(100, 70));
				}
			}
			if (modelElement instanceof ZoomableNode) {
				ZoomableNode znc = (ZoomableNode) modelElement;
				for (Connection con : znc.getConnections()) {
					//Profiler Performance optimization
					//con.disconnect();
					if(con.getSource().isVisible()&&con.getTarget().isVisible()){
					con.reconnect();
					}
					else{
						con.disconnect();
					}
				}

			}
		}
	}

	public class HierarchyClass {
		private ModelElement modelElement;

		private int height;

		public HierarchyClass(ModelElement modelElement, int height) {
			this.modelElement = modelElement;
			this.height = height;
		}

		public int getHeight() {
			return height;
		}

		public ModelElement getModelElement() {
			return modelElement;
		}
	}
}
