package ch.unizh.ifi.adora.model.commands;

import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.utils.ZoomAlgorithmsManager;

/**
 * This class is the super class for the commands 
 * {@link ch.unizh.ifi.adora.model.commands.InsertNodeVisually} and 
 * {@link ch.unizh.ifi.adora.model.commands.RemoveNodeVisually}. This avoids
 * duplication of the code for removing and inserting, which is used several
 * times by the sub classes for executing/undoing/redoing the command.
 *
 * @author				schlaefli
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			02.08.2006 JS First Version
 * 					11.08.2006 JS Adapted to new signature of LayoutUtils.calcShrunkSize
 * @version			$Date: 2007-07-02 16:59:43 +0200 (Mon, 02 Jul 2007) $, $Author: vontobel $, $Revision: 2835 $
 */
public abstract class MoveNodeVisually  extends SavableCommand {

	
	/*
	 * (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#Command(String)
	 */
	public MoveNodeVisually(String description) {
		super(description);
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public abstract void execute();
	
	/*
	 * (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	public abstract void undo();
	
	/*
	 * (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#redo()
	 */
	public abstract void redo();
	
	/**
	 * Moves the siblings of the node away to create sufficient space.
	 * 
	 * @param node the node that is inserted
	 * @pre node != null;
	 */
	protected void insert(Node node) {
	    
	    ZoomAlgorithmsManager.getInstance().getInsertAlgorithm().insertNode(node);
	    
	}
	
	/**
	 * Sets the size of the removed node to (1,1) so that the space can be
	 * used by the sibling of the node, or shrinks its parent node 
	 * proportionally if node was the only child.
	 * 
	 * @param node the node that is removed from its parent
	 * @pre node != null
	 */
	protected void remove(Node node) {
		
		ZoomAlgorithmsManager.getInstance().getRemoveAlgorithm().removeNode(node);
        
	}
}
