package ch.unizh.ifi.adora.model.commands;

import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.Node;

/**
 * A command for creating Connections between two
 * {@link ch.unizh.ifi.adora.model.representation.Node}s.
 *      
 * @author				Adrian Gygax
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			06.12.2005 AG First Version
 * 						20.07.2006 SM Comments reformatted, checked and completed.
 * @version			$Date: 2006-07-22 17:49:18 +0200 (Sat, 22 Jul 2006) $, $Author: smeier $, $Revision: 1886 $
 *
 */
public abstract class NodeConnectionCreate extends ConnectionCreate {

	/**
	 * Constructs a new NodeConnectionCreate.
	 * 
	 * @param source Node, where the connection starts.
	 */
    public NodeConnectionCreate(Node source) {
        super(source);
		setLabel("Creates a Node Connection");
    }

    /*
     *  (non-Javadoc)
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public abstract void execute();

    /*
     *  (non-Javadoc)
     * @see org.eclipse.gef.commands.Command#canExecute()
     */
    public boolean canExecute() {
    	return super.canExecute();
    }

    /*
     *  (non-Javadoc)
     * @see ch.unizh.ifi.adora.model.commands.ConnectionCreate#getSource()
     */
    public Node getSource() {
        return (Node) super.getSource();
    }

    /*
     *  (non-Javadoc)
     * @see ch.unizh.ifi.adora.model.commands.ConnectionCreate#getTarget()
     */
    public Node getTarget() {
        return (Node) super.getTarget();
    }

    /**
     * Sets the target node for this command.
     * 
     * @param target
     *            must be of type {@link Node}.
     */
    @Override
    public void setTarget(ConnectableElement target) {
        if (!(target instanceof Node))
            throw new IllegalArgumentException("Target must be of type Node");
        super.setTarget(target);
    }
}