package ch.unizh.ifi.adora.model.commands;

import ch.unizh.ifi.adora.model.representation.Association;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.ObjectConnection;
import ch.unizh.ifi.adora.representation.abstractions.algorithm.ConnectionAbstractionAlgorithm;

/**
 * Creates a command that can reconnect a Connection instance to a
 * different source or target endpoint.
 *
 * @author				smeier
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			31.10.2005 SM First Version
 * 						17.07.2006 SM Comments, Reformatted.
 * @version			$Date: 2006-08-21 14:04:43 +0200 (Mon, 21 Aug 2006) $, $Author: vigan $, $Revision: 2087 $
 */
public class ObjectConnectionReconnect extends ConnectionReconnect {
	/**
	 * @param conn
	 *            the connection instance to reconnect (non-null)
	 * @throws IllegalArgumentException
	 *             if conn is null
	 */
	public ObjectConnectionReconnect(ObjectConnection conn) {
		super(conn);
		setLabel("Reconnects Object Connection");
	}

	/**
	 * Updates all InterrelationShips in case of a manually abstracted
	 * Association being moved.
	 */
	public void execute() {
		super.execute();
		if(connection instanceof Association &&
				((Association)connection).isManuallyAbstracted()){
			ConnectionAbstractionAlgorithm.getInstance().createInterRelations(
					(Node)connection.getTarget());
		}
	}
}