package ch.unizh.ifi.adora.model.commands;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.model.geometry.Point;
import ch.unizh.ifi.adora.model.representation.INodeContainer;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.representation.abstractions.algorithm.ConnectionAbstractionAlgorithm;

/**
 * OrphanChildCommand removes a Node from its parent.
 * @author				smeier
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			31.10.2005 SM First Version
 * 						18.07.2006 SM Comments, Reformatting
 * 						08.08.2006 JS Comments
 * @version			$Date: 2006-08-09 10:35:47 +0200 (Wed, 09 Aug 2006) $, $Author: schlaefli $, $Revision: 2038 $
 */
public class OrphanChild extends Command {

    /**
     * Needed for placing a figure after undoing a moving action 
     * which results in an orphanChildCommand.
     */
	private Point oldLocation;

    /**
     * Parent before the execution of the 
     * OrphanChild command.
     */
	private INodeContainer parent;

	/**
	 * The child which is orphaned.
	 */
	private Node child;

	/**
	 * Initializes the orphan child command.
	 * 
	 */
	public OrphanChild() {
		super("Orphan Child Command");
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() {
		oldLocation = child.getLocation();
		parent.removeChild(child);
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#redo()
	 */
	public void redo() {
		parent.removeChild(child);
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	public void undo() {
		child.setLocation(oldLocation);
		parent.addChild(child);
		ConnectionAbstractionAlgorithm.getInstance().createInterRelations(child);
	}
	
	/**
	 * Sets the child to orphan.
	 * @pre       child != null
	 * @post      true
	 * @param child The child which will be 
	 *               orphaned.
	 */
	public void setChild(Node child) {
		this.child = child;
	}

	/**
	 * Sets the parent from which the 
	 * child is orphaned.
	 * 
	 * @pre       parent != null
	 * @param parent 	The parent to orphan the 
	 *					child from.
	 */
	public void setParent(INodeContainer parent) {
		this.parent = (INodeContainer) parent;
	}


}
