package ch.unizh.ifi.adora.model.commands;

import ch.unizh.ifi.adora.model.geometry.Dimension;
import ch.unizh.ifi.adora.model.geometry.Point;
import ch.unizh.ifi.adora.model.representation.Node;

/**
 * This class represents a command which relayouts the parent
 * and the siblings of a given node before the removal of it.
 * This is done by shifting the siblings radially together so that
 * the freed space is occupied by them.
 * 
 * @author		        smeier
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             Dec 2, 2005 SM First Version
 * @version             $Date: 2007-10-31 10:46:16 +0100 (Wed, 31 Oct 2007) $, $Author: schoeni $, $Revision: 3218 $
 */
public class RemoveNodeVisually extends MoveNodeVisually {

	/**
	 * The node for which the radial remove comman is executed.
	 */
	private final Node node;
	
	/**
	 * Stored size. Used for undo/redo purposes.
	 */
	private Dimension size;
	
	/**
	 * Stored location. Used for undo/redo purposes.
	 */
	private Point location;
	
	/**
	 * Constructor 
	 * @param nc Node which will be removed.
	 */
	public RemoveNodeVisually(Node nc) {
		super("Remove Node Visually");
		this.node = nc;		
	}
	
	/**
	 * Executes this command by adding the given child node to the new parent
	 * node.
	 * 
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() {
		size = node.getSize();
		location = node.getLocation();
		
		remove(node);
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#redo()
	 */
	public void redo() {
		execute();
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	public void undo() {
		node.setLocation(location);
		node.setSize(size);

		insert(node);
	}
	
	@Override
	public String toString() {
		return "Remove node " + node + " visually";
	}
}