package ch.unizh.ifi.adora.model.commands;

import org.eclipse.gef.commands.Command;
import org.eclipse.jface.preference.IPreferenceStore;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.ui.preferences.IPreferenceNameConstants;
import ch.unizh.ifi.adora.utils.ZoomAlgorithmsManager;
import ch.unizh.ifi.adora.zoom.berner.algorithm.LayoutAlgorithm;

/**
 * This {@link Command} is used to reroute the connections after a weaving operations has been performed.
 * To avoid redundant code the rerouting is performed by the {@link LayoutAlgorithm} by just zooming the 
 * parent of the weaved elements in and out.
 * 
 * @author ivovigan
 */
public class RerouteChildConnections extends Command {

	/**
	 * The parent of the weaved elements.
	 */
	private ZoomableNodeContainer zoomTarget;
	
	/**
	 * Creates a new {@link RerouteChildConnections} command for a given parent.
	 * 
	 * @param zoomTarget the parent of the weaved elements.
	 */
	public RerouteChildConnections(ZoomableNodeContainer zoomTarget) {
		
		this.zoomTarget = zoomTarget;
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() {
		
		IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
		boolean oldAnimate = store.getBoolean(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_ANIMATE_ZOOM);
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_ANIMATE_ZOOM, false);
		
		
		ZoomAlgorithmsManager.getInstance().getZoomAlgorithm().zoom(zoomTarget, false);
		ZoomAlgorithmsManager.getInstance().getZoomAlgorithm().zoom(zoomTarget, true);
		
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_ANIMATE_ZOOM, oldAnimate);
		
	}
}