package ch.unizh.ifi.adora.model.commands;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.OutputStreamWriter;

import org.eclipse.gef.commands.Command;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PlatformUI;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.util.ModelFileProperties;
import ch.unizh.ifi.adora.model.util.ModelFileProperty;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;
import ch.unizh.ifi.adora.ui.preferences.IPreferenceNameConstants;

import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.io.xml.DomDriver;

/**
 * Superclass for all adora commands.
 * 
 * @author		        Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             10.08.2006 IV First Version
 * 						23.12.2009 CJ SavableCommands do not listen to Preferences anymore (simply check them before saving)
 * @version             $Date: 2009-12-23 14:25:55 +0100 (Wed, 23 Dec 2009) $, $Author: jeanneret $, $Revision: 5005 $
 */
public abstract class SavableCommand extends Command {

	
	/**
	 * Constructs a SavableCommand with a given text.
	 * 
	 * @param label
	 */
	public SavableCommand(String label){
		super(label);
	}
	
	/**
	 * Constructs a unlabeled comamnd.
	 */
	public SavableCommand(){
	}

	/**
	 * Saves the model and the current command name persistently.
	 */
	protected void saveModel(){
		IPreferenceStore store  = AdoraPlugin.getDefault().getPreferenceStore();
		boolean save = store.getBoolean(IPreferenceNameConstants.ADORA_SAVE_MODEL_BEFORE_COMMAND);
		
		if (!save) {
			return;
		}

		ByteArrayOutputStream out = null;
        try {
        	
        	File adoraFoler = new File(System.getProperty("user.home") + "/adorasave");
        	if (!adoraFoler.exists());
        		adoraFoler.mkdir();
        	//File savedModelFoler = new File(System.getProperty("user.home") +"/adora");
         //   if (!savedModelFoler.exists());
         //   	savedModelFoler.mkdir();
        		
        	File adoraFile = new File(System.getProperty("user.home") + "/adorasave//adora.adora");
        	FileOutputStream fo = new FileOutputStream(adoraFile);
        	
        	File commandFile = new File(System.getProperty("user.home") + "/adorasave/command.txt");
        	FileOutputStream commandfo = new FileOutputStream(commandFile);
        	
        	//-------
        	OutputStreamWriter osw = new OutputStreamWriter(commandfo);
    		XStream xstream = new XStream(new DomDriver()); 
    		ObjectOutputStream oos = xstream.createObjectOutputStream(osw);
    		oos.writeObject(this);
    		oos.close();
    		osw.close();
    		//-------------
        	
        	ModelFileProperty property = ModelFileProperties.getInstance()
        		.getProperty(adoraFile.getAbsolutePath());
        	// set the current model.
        	property.setAdoraModel(getModel());
        	property.setFileFormat(ModelFileProperty.FILE_FORMAT_XML);
        	
        	// get the output stream
        	out = property.getModelOutputStream();
        	byte[] content = new byte[out.size()];
        	content = out.toByteArray();
       	 	for (int i = 0; i < content.length; i++) {
       	 		fo.write(content[i]);
            }


        	out.close();
        	fo.close();
         } catch (IOException ioe) {
            ioe.printStackTrace();
        } catch (IllegalArgumentException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} 
    }
	
	/**
	 * @return The adora model on which the filteraction is applied.
	 */
	private AdoraModel getModel() {
		IWorkbenchPage page = PlatformUI.getWorkbench()
				.getActiveWorkbenchWindow().getActivePage();
		try {
			IEditorPart editor = page.getActiveEditor();
			if (editor instanceof AdoraModelEditor) {
				return ((AdoraModelEditor) editor).getModel();
			}
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		return null;
	}
}