package ch.unizh.ifi.adora.model.commands;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.model.geometry.Rectangle;
import ch.unizh.ifi.adora.model.representation.INodeContainer;
import ch.unizh.ifi.adora.model.representation.Scenario;

public class ScenarioAdd extends Command{

	 /**
     * Stores the the child node
     */
	private Scenario visibleScenario;

	/**
     * Stores the parent node.
	 */
	private INodeContainer parent;

	/** 
     * Stores the new size and location. 
     */
	protected final Rectangle newBounds;

	/**
	 * Creates the add command with a label for undo history
	 * 
	 * @param child The child to add to the parent.
	 * @param newParent The parent to which child is added.
	 * 
	 * @pre child != null && newParent != null
	 */
	public ScenarioAdd(Scenario child, INodeContainer newParent, Rectangle rect) {
		super("Add Scenario Node");
		this.visibleScenario = child;
		this.parent = newParent;
		this.newBounds =  rect.getCopy();
	}

	/**
	 * Executes this command by adding the given child node to the new parent
	 * node.
	 * 
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() {
		addChild(visibleScenario);
		for (Scenario currentScenario : visibleScenario.getScenarioChildren()){
			if (!currentScenario.isVisible()) {
				addChild(currentScenario);
				for (Scenario hiddenScenario : currentScenario.getAllScenarioChildren()){
					addChild(hiddenScenario);
				}
			}
		}
	}
	
	private void addChild(Scenario child){
		
		new Add(child, parent).execute();	
		
		Rectangle newChildBounds = newBounds.getCopy();
		newChildBounds.x += child.getLocation().x - visibleScenario.getLocation().x;
		newChildBounds.y += child.getLocation().y - visibleScenario.getLocation().y;
		
		new VisualizableModelElementSetConstraint(child, newChildBounds).execute();
		
		new InsertNodeVisually(child, false).execute();
		
	}
}