package ch.unizh.ifi.adora.model.commands;

import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.model.representation.ScenarioConnection;

/**
 * Creates a command that can create a connection between two Scenarios.
 *
 * @author				smeier
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			31.10.2005 SM First Version
 * 						18.07.2006 SM Comments, Reformatting.
 * 						08.08.2006 JS Added getTarget()
 * @version			$Date: 2007-06-28 16:03:26 +0200 (Thu, 28 Jun 2007) $, $Author: vontobel $, $Revision: 2808 $
 */
public class ScenarioConnectionCreate extends NodeConnectionCreate {
	
    /**
	 * @param source
	 *            the source endpoint (a non-null Shape instance)
	 * @throws IllegalArgumentException
	 *             if source is null
	 */
	public ScenarioConnectionCreate(Scenario source) {
		super(source);	
		setLabel("Scenario Connection Creation");		
	}
    
	/*
	 * (non-Javadoc)
	 * @see ch.unizh.ifi.adora.model.commands.NodeConnectionCreate#getSource()
	 */
    @Override
    public Scenario getSource() {
        return (Scenario)super.getSource();
    }
    
	/*
	 * (non-Javadoc)
	 * @see ch.unizh.ifi.adora.model.commands.NodeConnectionCreate#getTarget()
	 */
    @Override
    public Scenario getTarget() {
        return (Scenario)super.getTarget();
    }

    /* (non-Javadoc)
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute() {
        ScenarioConnection connection = new ScenarioConnection(getSource(), getTarget());
		if(getSource() == null) { 
			connection.setSourceClickPoint(getSourceClickPoint());
		}
		if(getTarget() == null) { 
			connection.setTargetClickPoint(getTargetClickPoint());
		}
        setConnection(connection);
        ((ConnectableElement)getSource()).connectionCreated(connection);
        ((ConnectableElement)getTarget()).connectionCreated(connection);
  	}
}
