package ch.unizh.ifi.adora.model.commands;


import java.util.ArrayList;
import java.util.List;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.model.geometry.Dimension;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.model.representation.ScenarioConnection;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.model.representation.zoom.IZoomNode;
import ch.unizh.ifi.adora.model.util.NodeGroup;
import ch.unizh.ifi.adora.model.geometry.Rectangle;
import ch.unizh.ifi.adora.zoom.continuous.algorithm.ContinuousZoom;

public class ScenarioLayouter extends Command {
	NodeGroup nodeGroup;
	public ScenarioLayouter(NodeGroup nodeGroup) {
		this.nodeGroup=nodeGroup;
	}

	public void execute() {
		handleScenarioGroup();

	}
	private void handleScenarioGroup() {
		double x = 0, y = 0;
		for (int i = 0; i < nodeGroup.getNodes().size(); i++) {
			if (i == 0) {
				x = nodeGroup.getNodes().get(i).getLocation().getX();
				y = nodeGroup.getNodes().get(i).getLocation().getY();
			}
			if (x <= nodeGroup.getNodes().get(i).getLocation().getX()) {
				x = nodeGroup.getNodes().get(i).getLocation().getX();
			}
			if (y <= nodeGroup.getNodes().get(i).getLocation().getY()) {
				y = nodeGroup.getNodes().get(i).getLocation().getY();
			}
		}
		int xmax = (int) x + 100;
		int ymax = (int) y + 70;
		y = 0;
		x = 0;
		for (int i = 0; i < nodeGroup.getNodes().size(); i++) {
			if (i == 0) {
				x = nodeGroup.getNodes().get(i).getLocation().getX();
				y = nodeGroup.getNodes().get(i).getLocation().getY();
			}
			if (x >= nodeGroup.getNodes().get(i).getLocation().getX()) {
				x = nodeGroup.getNodes().get(i).getLocation().getX();
			}
			if (y >= nodeGroup.getNodes().get(i).getLocation().getY()) {
				y = nodeGroup.getNodes().get(i).getLocation().getY();
			}
		}
		int xmin = (int) x;
		int ymin = (int) y;
		Rectangle r = new Rectangle();
		r.setBounds((int) x, (int) y, xmax - xmin, ymax - ymin);
		/*AllocateWhiteSpace awc = new AllocateWhiteSpace(nodeGroup.getNodes()
				.get(0).getZoomParent(), r);
		awc.execute();*/
		this.handleNormalScenarioTree((ZoomableNodeContainer)(nodeGroup.getNodes().get(0).getParent()),r);
		NodeCompact cd = new NodeCompact();
		cd.setTarget((ZoomableNodeContainer) nodeGroup.getNodes().get(0)
				.getZoomParent());
		cd.execute();
	}
	
	public void handleNormalScenarioTree(ZoomableNodeContainer comp, Rectangle bound) {
		ArrayList<NodeGroup> nodeGroup = comp.getNodeGroups()
				.getScenarioGroups();
		ArrayList<Node> nodeOrdering = new ArrayList<Node>();
		setUniqueStateNodeSize(nodeGroup, nodeOrdering);
		// place the nodes
		placeScenarioNode(nodeOrdering, bound);
	}

	private List<Scenario> scenariosPerHeight = new ArrayList<Scenario>();

	private void setUniqueStateNodeSize(ArrayList<NodeGroup> nodeGroup,
			ArrayList<Node> nodeOrdering) {
		for (int i = 0; i < nodeGroup.size(); i++) {
			for (Node node : nodeGroup.get(i).getNodes()) {
				nodeOrdering.add(node);
				// set Size of node
				node.setSize(new Dimension(100, 70));
			}
		}
	}

	private void placeScenarioNode(ArrayList<Node> nodeOrdering,
			Rectangle nodeParentBound) {
		int nodeHeight = (int) nodeOrdering.get(0).getBounds().getHeight();
		Scenario root = fetchScenarioRoot(nodeOrdering.get(0));
		int nHeight = calculateScenarioTreeHeight(root);

		int nodeHeightDistance = (int) (Math.round(nodeParentBound.getHeight()
				- nHeight * nodeHeight) / (nHeight + 1));
		// calculate nodes per tree layer
		buildTree(root, nodeParentBound.width, nodeHeightDistance,
				calculateScenarioTreeHeight(root));

	}

	private void buildTree(Scenario root, int width, int height, int layerHeight) {
		for (int i = 1; i <= layerHeight; i++) {
			List<Scenario> scenariosPerLayer = getScenarioNodesPerHeight(root,
					i);
			int layerSpecificWidth = (width - scenariosPerLayer.size() * 100)
					/ (scenariosPerLayer.size() + 1);
			// int layerSpecificWidth=(width-())/(+);
			for (int j = 0; j < scenariosPerLayer.size(); j++) {
				ContinuousZoom.getInstance().changeNodeBounds(scenariosPerLayer.get(j),new Rectangle(layerSpecificWidth + (j * layerSpecificWidth)+ (j * 100), i * height + (i - 1) * 70, 100, 70));
		/*		scenariosPerLayer.get(j).setLocation(
						layerSpecificWidth + (j * layerSpecificWidth)
								+ (j * 100), i * height + (i - 1) * 70);*/
				// optimize connections
				for (Connection con : scenariosPerLayer.get(j).getConnections()) {
					if (con instanceof ScenarioConnection) {
						con.disconnect();
						con.reconnect();
					}
				}

			}
		}
	}

	private List<Scenario> getScenarioNodesPerHeight(Scenario root, int height) {
		scenariosPerHeight = new ArrayList<Scenario>();
		getScenarioNodesPerHeightHelper(root, height);
		return scenariosPerHeight;
	}

	private void getScenarioNodesPerHeightHelper(Scenario root, int height) {
		if (height == 1) {
			scenariosPerHeight.add(root);
			return;
		}
		if (root.getScenarioChildren().size() != 0) {
			for (int i = 0; i < root.getScenarioChildren().size(); i++) {
				getScenarioNodesPerHeightHelper(root.getScenarioChildren().get(
						i), height - 1);
			}
		}
	}


	private int calculateScenarioTreeHeight(Scenario scenarioNode) {
		if (scenarioNode.getScenarioChildren().size() == 0) {
			return 1;
		}
		int height = 0;
		for (int i = 0; i < scenarioNode.getScenarioChildren().size(); i++) {
			int branchHeight = calculateScenarioTreeHeight(scenarioNode
					.getScenarioChildren().get(i));
			if (height < branchHeight) {
				height = branchHeight;
			}

		}
		return 1 + height;

	}

	private Scenario fetchScenarioRoot(Node node) {
		Scenario scenarioNode = (Scenario) node;
		if (scenarioNode.getScenarioParent() == null) {
			return scenarioNode;
		}
		return fetchScenarioRoot(scenarioNode.getScenarioParent());
	}
}
