package ch.unizh.ifi.adora.model.commands;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.Scenario;

/**
 * A command to increment sequence Numbers of child nodes of a node
 * 
 * @author				Marc Vontobel
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				29.06.2007 IV First Version
 * 						02.07.2007 SM Comments reformatted and completed.
 * @version				$Date: 2007-10-10 14:07:16 +0200 (Wed, 10 Oct 2007) $, $Author: vontobel $, $Revision: 3114 $
 *
 */
public class ScenarioSeqNrIncrement extends Command {
	private Scenario parentScenario;
	private Scenario seqNumberScenario;
	
	public ScenarioSeqNrIncrement(Scenario parentNode, Scenario seqNumberScenario) {
		this.parentScenario = parentNode;
		this.seqNumberScenario = seqNumberScenario;
	}
	/**
	 * Finds all the children of the defined parent node. Iterates over them and
	 * increments every sequence number which is equal or bigger to the reference node
	 */
	public void execute() {
		ArrayList<Scenario> neighbours = new ArrayList<Scenario>();
		List<Connection> connections = this.parentScenario.getSourceConnections();
		for (Connection connection : connections) {
			if (connection.getTarget() instanceof Scenario && !neighbours.contains(connection.getTarget()) && !connection.getTarget().equals(this.seqNumberScenario)) {
				Scenario neighbour = (Scenario) connection.getTarget();
				neighbours.add(neighbour);
				if (neighbour.getSeqNo() >= this.seqNumberScenario.getSeqNo()) {
					neighbour.setSeqNr(neighbour.getSeqNo() + 1);
				}
			}
		}
	}
}
