package ch.unizh.ifi.adora.model.commands;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.model.representation.Scenario;
/**
 * A command to set the sequence number of a scenario node. this command
 * is used because manually changing the sequence number in the weaving process
 * results in different errors caused by the sequential execution of the weaving commands.
 * 
 * @author				Marc Vontobel
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				29.06.2007 MV First Version
 * 						05.11.2007 MV Comments reformatted and completed.
 * @version				$Date: 2007-12-17 10:04:07 +0100 (Mon, 17 Dec 2007) $, $Author: vontobel $, $Revision: 3412 $
 *
 */
public class ScenarioSeqNrSet extends Command {
	public static final int HARD = 0, REFERENCE = 1, AFTER = 0, BEFORE = 1, EQUAL = 2;
	private Scenario targetNode;
	private Scenario referenceNode;
	private int settingType;
	private int seqNr;
	private int executingType;
	private int undoSeqNr;
	
	public ScenarioSeqNrSet(Scenario targetNode, Scenario referenceNode, int settingType) {
		this.undoSeqNr = targetNode.getSeqNo();
		this.targetNode = targetNode;
		this.referenceNode = referenceNode;
		this.settingType = settingType;		
		this.executingType = REFERENCE;
	}
	
	public ScenarioSeqNrSet(Scenario targetNode, int seqNr) {
		this.undoSeqNr = targetNode.getSeqNo();
		this.targetNode = targetNode;
		this.seqNr = seqNr;
		this.executingType = HARD;
	}
	
	public void execute() {
		switch (this.executingType) {
		case HARD:
			this.targetNode.setSeqNr(this.seqNr);
			break;
		case REFERENCE:
			switch(settingType) {
			case AFTER:
				this.targetNode.setSeqNr(this.referenceNode.getSeqNo() + 1);
				break;
			case BEFORE:
				this.targetNode.setSeqNr(this.referenceNode.getSeqNo());
				break;
			case EQUAL:
				this.targetNode.setSeqNr(this.referenceNode.getSeqNo());
				break;
			}
			break;
		}
	}
	
	public void undo() { 
		int temp = this.targetNode.getSeqNo();
		this.targetNode.setSeqNr(undoSeqNr);
		this.undoSeqNr = temp;
	}
	
	public void redo() { 
		int temp = this.targetNode.getSeqNo();
		this.targetNode.setSeqNr(undoSeqNr);
		this.undoSeqNr = temp;
	}

}
