package ch.unizh.ifi.adora.model.commands;

import java.util.ArrayList;
import java.util.List;

import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.representation.abstractions.algorithm.ConnectionAbstractionAlgorithm;
import ch.unizh.ifi.adora.utils.ZoomAlgorithmsManager;
import ch.unizh.ifi.adora.zoom.berner.algorithm.NewBerner;

/**
 * Resonsible for updating the zoom attribute of the scenario, 
 * on which the zooming action takes place.   
 * 
 * @author				Ivo Vigan
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			19.04.2006 SM First Version
 * @version			$Date: 2007-08-17 10:56:16 +0200 (Fri, 17 Aug 2007) $, $Author: treinhard $, $Revision: 2912 $
 */
public class ScenarioZoom extends SavableCommand {
    
    /**
     * Stores the scenario on which the zooming action takes place. 
     */
	transient private Scenario zoomTarget;
    
    /**
     * Creates a Scenario zoom command for a given
     * scenario which should be zoomed in or out.
     * @param zoomTarget Scenario on which the zooming action takes place.
     */
    public ScenarioZoom(Scenario zoomTarget) {
        super("Scenario Zoom");
        this.zoomTarget = zoomTarget;
    }

    /*
     *  (non-Javadoc)
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute() {
        saveModel();
              
        //zoom out operation
        zoomTarget.setZoomedIn(!zoomTarget.isZoomedIn());
       
        List<ZoomableNodeContainer> parents = new ArrayList<ZoomableNodeContainer>();
		
        for(Scenario child : zoomTarget.getAllScenarioChildren()){
        	
        	// TODO: Remove
        	if(ZoomAlgorithmsManager.getInstance().getFilterAlgorithm() instanceof NewBerner) {
        		
          
            	NewBerner.getInstance().
                   calculateStoredChildrenBounds(
                		   (ZoomableNodeContainer)child.getParent());
         
                child.updateVisibility();
          
                NewBerner.getInstance().adjustNode(
                		(ZoomableNodeContainer) child.getParent());
        		
        	}
        	else {
        		
        		child.updateVisibility();
        		ZoomAlgorithmsManager.getInstance().getFilterAlgorithm().filterNode(child);
        		
        	}
            
            if (!parents.contains(child.getParent())) {
            	parents.add((ZoomableNodeContainer)child.getParent());
            }
        }
        
		for (ZoomableNodeContainer container : parents) {
			ConnectionAbstractionAlgorithm.getInstance().run(container);
		}
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.gef.commands.Command#canExecute()
     */
    public boolean canExecute() {
        if(zoomTarget != null) {
            return true;
        }
        return false;
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo() {
        execute();
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo() {
        execute();
    }

}