package ch.unizh.ifi.adora.model.commands;

import ch.unizh.ifi.adora.model.geometry.Point;
import ch.unizh.ifi.adora.model.geometry.Rectangle;
import ch.unizh.ifi.adora.model.representation.ConnectionLabel;


/**
 * This {@link org.eclipse.gef.commands.Command} is responsible
 * for updating the size/location of the label.
 * 
 * This {@link org.eclipse.gef.commands.Command} differs 
 * from the  {@link VisualizableModelElementSetConstraint}-command 
 * forasmtch as location of the label is only specified by the its 
 * displacement to the {@link ch.unizh.ifi.adora.view.figures.ConnectionFigure} 
 * endpoint. 
 *      
 *
 * @author		        Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             27.07.2006 IV First Version
 *                      09.08.2006 JS Comments
 * @version             $Date: 2006-08-22 11:47:05 +0200 (Tue, 22 Aug 2006) $, $Author: vigan $, $Revision: 2088 $
 */
public class SetConnectionLabelConstraint extends VisualizableModelElementSetConstraint {
	
	/**
	 * The {@link ConnectionLabel} for which the bounds are changed.
	 */
	transient private ConnectionLabel label;

	/**
	 * The distance between the {@link ConnectionLabel} and the 
	 * {@link ch.unizh.ifi.adora.view.figures.LabeledConnection}.
	 */
	private Point connectionOffset;

	/**
	 * Initiates the SetConnectionLabelConstraint.
	 * 
	 * @param label the label the constraint of which is changed
	 * @param connectionOffset the offset of the label
	 * @param rect the bounds to be set
	 */
	public SetConnectionLabelConstraint(ConnectionLabel label, Point connectionOffset, Rectangle rect) {
    	super(label, rect); 
    	this.label = label;
    	this.connectionOffset = connectionOffset;
    }
    
    /*
     * (non-Javadoc)
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute() {
    	
    	if (!connectionOffset.equals(new Point(0, 0))) {
    		label.changeConnectionOffset(connectionOffset);
    	}
    	super.execute();
    	
    }
	 
    /*
     *  (non-Javadoc)
     * @see org.eclipse.gef.commands.Command#undo()
     */
	public void undo() {
		label.changeConnectionOffset(connectionOffset.getNegated());
    	super.undo();
	}

}
