package ch.unizh.ifi.adora.model.commands;

import org.eclipse.gef.commands.Command;
import org.eclipse.jface.viewers.ICellModifier;

import ch.unizh.ifi.adora.model.representation.variability.Constraint;

/**
 * This class is the {@link ICellModifier} for the ConstraintTable.
 * It returns, if a column can be modified, what the current value is and it
 * modifies the entry.
 * 
 * @author Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 2008-10-31 IV First Version
 * @version 
 */
public class SetConstraintsTableListAntecedent extends Command {
	
	/**
     * The given value, from which the chosen decisions are extracted.
     */
	private String value;

	/**
     * The variation point to update.
     */
	protected Constraint constraint;

	/**
     * Here the information for undo / redo get stored. 
     */
	private String oldAntecedent;

	/**
     * The constructor.
     * 
     * @param value
     *            The user input.
     * @param constraint
     *            The constraint to update.
     */
	public SetConstraintsTableListAntecedent(String value, Constraint constraint) {
		this.value = value;
		this.constraint = constraint;

		oldAntecedent = constraint.getAntecedent();
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.gef.commands.Command#execute()
     */
	@Override
	public void execute() {
		
		if (isLegalAntecedent()) 
			constraint.setAntecedent(value);
	}
	
	/**
	 * Checks whether all the decisions used in this antecedent exist in the model.
	 * 
	 * @return True, if all the decisions used in this antecedent exist in the model.
	 */
	private boolean isLegalAntecedent() {
		
		//TODO: Add Parser check , brackets
		
		
		// parse input string and set the things..
/*	AdoraModel adoraModel = ModelObserverMediator.getInstance()
				.getAdoraModelEditor().getModel();

		String decisionsString = value;
		String[] decisions = decisionsString.toLowerCase().split("and | or | xor");
	

		List<Decision> newLines = new ArrayList<Decision>();
		for (String search : decisions) {
			boolean found = false;
			for (Decision toCheck : adoraModel.getDecisions()) {
				if (toCheck.toString().equals(search.trim())) {
					newLines.add(toCheck);
					found = true;
					break;
				}
			}
			if (!found) 
				return false;
		}*/
		return true;
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.gef.commands.Command#undo()
     */
	@Override
	public void undo() {
		constraint.setAntecedent(oldAntecedent);
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.gef.commands.Command#redo()
     */
	@Override
	public void redo() {
		execute();
	}
}