package ch.unizh.ifi.adora.model.commands;

import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Stack;
import java.util.Map.Entry;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.variability.Decision;
import ch.unizh.ifi.adora.ui.views.ModelObserverMediator;

/**
 * Property to set the range. Supports undo / redo.
 * 
 * @author Pascal Schoeni
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 17.03.2008 PS First Version
 * @version $Date: 2010-03-26 18:20:14 +0100 (Fri, 26 Mar 2010) $, $Author: vigan $, $Revision: 5173 $
 */
public class SetDecision extends Command {
	
	/**
     * The corresponding decision.
     */
	private Decision decision;

	/**
     * The new decisionValue.
     */
	private Boolean decisionValue;

	/**
     * The old value.
     */
	private Boolean oldValue;

	/**
     * The list of included decisions.
     */
	private HashMap<Decision, Boolean> includes = new HashMap<Decision, Boolean>();

	/**
     * The list of excluded decisions.
     */
	private HashMap<Decision, Boolean> excludes = new HashMap<Decision, Boolean>();
	
	/**
     * The constructor.
     * 
     * @param decisionValue
     *            The decision.
     * @param decisionValue
     *            The boolean.
     */
	public SetDecision(Decision decision, Boolean decisionValue) {
		this.decision = decision;
		this.oldValue = decision.getDecisionValue();
		this.decisionValue = decisionValue;
	}

	@Override
	public void execute() {
		
		decision.setDecisionValueManually(decisionValue);

		Stack<Decision> stack = new Stack<Decision>();
		stack.addAll(decision.getInclude());
		while (!stack.isEmpty()) {
			Decision current = stack.pop();

			if (!includes.containsKey(current)) {
				includes.put(current, current.getDecisionValue());
				stack.addAll(current.getInclude());
			}
		}

		for (Entry<Decision, Boolean> entry : includes.entrySet()) {
			entry.getKey().setDecisionValue(decisionValue);
		}

		stack.addAll(decision.getExclude());

		while (!stack.isEmpty()) {
			Decision current = stack.pop();

			if (!excludes.containsKey(current)) {
				excludes.put(current, current.getDecisionValue());
				stack.addAll(current.getExclude());
			}
		}

		for (Entry<Decision, Boolean> entry : excludes.entrySet()) {
			if (entry.getValue() == null)
				continue;
			entry.getKey().setDecisionValue(!decisionValue);
		}
	}
	
	

	@Override
	public void redo() {
		includes.clear();
		excludes.clear();

		execute();
	}

	@Override
	public void undo() {
		decision.setDecisionValue(oldValue);

		for (Entry<Decision, Boolean> entry : includes.entrySet()) {
			entry.getKey().setDecisionValue(entry.getValue());
		}

		for (Entry<Decision, Boolean> entry : excludes.entrySet()) {
			entry.getKey().setDecisionValue(entry.getValue());
		}
	}
}