package ch.unizh.ifi.adora.model.commands;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.variability.Decision;
import ch.unizh.ifi.adora.ui.views.ModelObserverMediator;

/**
 * This command updates a list of decisions on a decision. This could be
 * for example the list of included decisions or the list of excluded
 * decisions.
 * 
 * @author Pascal Schoeni
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 18.03.2008 PS First Version
 * @version $Date: 2008-07-03 10:45:00 +0200 (Thu, 03 Jul 2008) $, $Author: stoiber $, $Revision: 3683 $
 */
public abstract class SetDecisionList extends Command {
	/**
     * The given value, from which the chosen decisions are extracted.
     */
	protected String value;

	/**
     * The decision to update.
     */
	protected Decision decision;

	/**
     * Here we store the information for undo / redo.
     */
	protected List<Decision> oldList;

	/**
     * The constructor.
     * 
     * @param value
     *            The user input.
     * @param decision
     *            The decision to update.
     */
	public SetDecisionList(String value, Decision decision) {
		this.value = value;
		this.decision = decision;

		oldList = new ArrayList<Decision>(getList());
	}
	
	public SetDecisionList(){
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.gef.commands.Command#execute()
     */
	@Override
	public void execute() {
		// parse input string and set the things..
		AdoraModel adoraModel = ModelObserverMediator.getInstance()
				.getAdoraModelEditor().getModel();

		String decisionsString = value;
		String[] decisions = decisionsString.split(",");

		List<Decision> newLines = new ArrayList<Decision>();
		for (String search : decisions) {
			boolean found = false;
			for (Decision toCheck : adoraModel.getDecisions()) {
				if (toCheck.toString().equals(search.trim())) {
					newLines.add(toCheck);
					found = true;
					break;
				}
			}

			if (!found) {
				// TODO what to do here? the decision was not found!
			}
		}

		updateList(newLines);
	}

	private void updateList(List<Decision> newLines) {
		// toArray() to preserve ConcurrentModificationException
		for (Object toCheck : getList().toArray()) {
			if (!newLines.contains(toCheck)) {
				removeDecision((Decision) toCheck);
			}
		}

		for (Decision toCheck : newLines) {
			if (!getList().contains(toCheck)) {
				addDecision(toCheck);
			}
		}
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.gef.commands.Command#undo()
     */
	@Override
	public void undo() {
		updateList(oldList);
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.gef.commands.Command#redo()
     */
	@Override
	public void redo() {
		execute();
	}

	/**
     * Returns the current list, which is edited.
     * 
     * @return The current list to edit.
     */
	abstract protected List<Decision> getList();

	/**
     * Adds a decision to the list.
     * 
     * @param decision
     *            The decision to add.
     */
	abstract protected void addDecision(Decision decision);

	/**
     * Removes a decision on the list.
     * 
     * @param decision
     *            The decision to remove.
     */
	abstract protected void removeDecision(Decision decision);
}
