package ch.unizh.ifi.adora.model.commands;

import java.util.ArrayList;
import java.util.List;

import ch.unizh.ifi.adora.model.representation.variability.Decision;
import ch.unizh.ifi.adora.model.representation.variability.VariationPoint;

/**
 * This is the concrete command for setting the list of involved decisions 
 * for a variation point (cardinality column in the decision model).
 * 
 * @author Reinhard Stoiber
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 24.06.2008 RS First Version
 * @version $Date: 2008-07-03 10:45:00 +0200 (Thu, 03 Jul 2008) $, $Author: stoiber $, $Revision: 3683 $
 */
public class SetDecisionListVariationPoint extends SetDecisionList {
	
	/**
     * The variation point to update.
     */
	VariationPoint variationPoint;
	
	/**
	 * the constructor
	 * @param value
	 * @param variationPoint
	 */
	public SetDecisionListVariationPoint (String value, VariationPoint variationPoint) {
		this.value = value;
		this.variationPoint = variationPoint;

		oldList = new ArrayList<Decision>(getList());
	}

	/*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.model.commands.SetDecisionList#addProductLine(ch.unizh.ifi.adora.model.representation.productline.ProductLine)
     */
	@Override
	protected void addDecision(Decision decision) {
		this.decision.addIncluded(decision);
	}

	/*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.model.commands.SetDecisionList#getList()
     */
	@Override
	protected List<Decision> getList() {
		return this.decision.getInclude();
	}

	/*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.model.commands.SetDecisionList#removeProductLine(ch.unizh.ifi.adora.model.representation.productline.ProductLine)
     */
	@Override
	protected void removeDecision(Decision decision) {
		this.decision.removeIncluded(decision);
	}

}
