package ch.unizh.ifi.adora.model.commands;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.variability.Decision;
import ch.unizh.ifi.adora.model.representation.variability.VariationPoint;
import ch.unizh.ifi.adora.ui.views.ModelObserverMediator;

/**
 * This command updates a list of decisions in a variation point. The affected
 * column in the variation point table is called "Decisions Involved".
 * 
 * @author Reinhard Stoiber
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 23.04.2008 RS First Version
 * @version 
 */
public class SetVariationPointListDecisions extends Command {
	
	/**
     * The given value, from which the chosen decisions are extracted.
     */
	private String value;

	/**
     * The variation point to update.
     */
	protected VariationPoint variationPoint;

	/**
     * Here we store the information for undo / redo.
     */
	private List<Decision> oldList;

	/**
     * The constructor.
     * 
     * @param value
     *            The user input.
     * @param variationPoint
     *            The decision to update.
     */
	public SetVariationPointListDecisions(String value, VariationPoint variationPoint) {
		this.value = value;
		this.variationPoint = variationPoint;

		oldList = new ArrayList<Decision>(getList());
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.gef.commands.Command#execute()
     */
	@Override
	public void execute() {
		// parse input string and set the things..
		AdoraModel adoraModel = ModelObserverMediator.getInstance()
				.getAdoraModelEditor().getModel();

		String decisionsString = value;
		String[] decisions = decisionsString.split(",");

		List<Decision> newLines = new ArrayList<Decision>();
		for (String search : decisions) {
			boolean found = false;
			for (Decision toCheck : adoraModel.getDecisions()) {
				if (toCheck.toString().equals(search.trim())) {
					newLines.add(toCheck);
					found = true;
					break;
				}
			}

			if (!found) {
				// TODO what to do here? the decision was not found!
			}
		}

		updateList(newLines);
	}

	private void updateList(List<Decision> newLines) {
		// toArray() to preserve ConcurrentModificationException
		for (Object toCheck : getList().toArray()) {
			if (!newLines.contains(toCheck)) {
				removeDecision((Decision) toCheck);
			}
		}

		for (Decision toCheck : newLines) {
			if (!getList().contains(toCheck)) {
				addDecision(toCheck);
			}
		}
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.gef.commands.Command#undo()
     */
	@Override
	public void undo() {
		updateList(oldList);
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.gef.commands.Command#redo()
     */
	@Override
	public void redo() {
		execute();
	}

	/**
     * Returns the current list, which is edited.
     * 
     * @return The current list to edit.
     */
	protected List<Decision> getList() {
		return this.variationPoint.getDecisions();
	}

	/**
     * Adds a Decision to the list.
     * 
     * @param decision
     *            The Decision to add.
     */
	protected void addDecision(Decision decision) {
		this.variationPoint.addDecision(decision);
	}

	/**
     * Removes a Decision on the list.
     * 
     * @param decision
     *            The Decision to remove.
     */
	protected void removeDecision(Decision decision) {
		this.variationPoint.removeDecision(decision);
	}
	
}
