package ch.unizh.ifi.adora.model.commands;

import java.util.ArrayList;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.model.geometry.Rectangle;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.model.util.NodeGroup;
import ch.unizh.ifi.adora.zoom.continuous.algorithm.ContinuousZoom;

public class StateLayouter extends Command {
	NodeGroup nodeGroup;

	public StateLayouter(NodeGroup nodeGroup) {
		this.nodeGroup = nodeGroup;
	}

	public void execute() {
		handleStateGroup();
	}

	private void handleStateGroup() {
		double x = 0, y = 0;
		for (int i = 0; i < nodeGroup.getNodes().size(); i++) {
			if (i == 0) {
				x = nodeGroup.getNodes().get(i).getLocation().getX();
				y = nodeGroup.getNodes().get(i).getLocation().getY();
			}
			if (x <= nodeGroup.getNodes().get(i).getLocation().getX()) {
				x = nodeGroup.getNodes().get(i).getLocation().getX();
			}
			if (y <= nodeGroup.getNodes().get(i).getLocation().getY()) {
				y = nodeGroup.getNodes().get(i).getLocation().getY();
			}
		}
		int xmax = (int) x + 100;
		int ymax = (int) y + 70;
		y = 0;
		x = 0;
		for (int i = 0; i < nodeGroup.getNodes().size(); i++) {
			if (i == 0) {
				x = nodeGroup.getNodes().get(i).getLocation().getX();
				y = nodeGroup.getNodes().get(i).getLocation().getY();
			}
			if (x >= nodeGroup.getNodes().get(i).getLocation().getX()) {
				x = nodeGroup.getNodes().get(i).getLocation().getX();
			}
			if (y >= nodeGroup.getNodes().get(i).getLocation().getY()) {
				y = nodeGroup.getNodes().get(i).getLocation().getY();
			}
		}
		int xmin = (int) x;
		int ymin = (int) y;
		Rectangle r = new Rectangle();
		r.setBounds((int) x, (int) y, xmax - xmin, ymax - ymin);
		AllocateWhiteSpace awc = new AllocateWhiteSpace(nodeGroup.getNodes()
				.get(0).getZoomParent(), r);
		awc.execute();
		this.placeStateNode(nodeGroup.getNodes(), xmax - xmin, ymax - ymin,
				xmin, ymin);
		NodeCompact cd = new NodeCompact();
		cd.setTarget((ZoomableNodeContainer) nodeGroup.getNodes().get(0)
				.getZoomParent());
		cd.execute();
	}

	/**
	 * Place nodes within its parent node.
	 * 
	 * @param nodeOrdering
	 * @param nodeParentBound
	 */

	private void placeStateNode(ArrayList<Node> nodeOrdering, int width,
			int height, int x, int y) {
		int nodeWidth = 100;
		int nodeHeight = 70;
		int nWidth = Math
				.round(((width) / (nodeOrdering.get(0).getBounds().width)));
		int nHeight;
		// handles case where there are fewer nodes than width e.g. 2 nodes but
		// a nWidth of 3
		if (nWidth > nodeOrdering.size()) {
			nHeight = 1;
			nWidth = nodeOrdering.size();
		} else {
			nHeight = Math.round(nodeOrdering.size() / nWidth);
		}
		int nodeWidthDistance = 20;
		int nodeHeightDistance = 20;
		// distance too small, recalculate node distances
		while (nodeWidthDistance < 10) {
			nWidth = nWidth - 1;
			if (0 != (nodeOrdering.size() % nWidth)) {
				nHeight = nHeight + 1;
			}
			nodeWidthDistance = (int) Math.round((width - nWidth * nodeWidth)
					/ (nWidth + 3));
			nodeHeightDistance = (int) (Math.round(height - nHeight
					* nodeHeight) / (nHeight + 3));
			if (nodeWidthDistance >= 10)
				break;
		}
		for (int i = 0; i < nodeOrdering.size(); i++) {
			ContinuousZoom.getInstance().changeNodeBounds(
					nodeOrdering.get(i),
					new Rectangle(x + i % nWidth
							* (nodeWidth + nodeWidthDistance), y
							+ (int) (i / nWidth)
							* (nodeHeight + nodeHeightDistance), 100, 70));

		}
	}

}
