package ch.unizh.ifi.adora.model.commands;

import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.Transition;

/**
 * Represents the create command of a transition.
 * 
 * @author smeier
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 10.10.2005 SM First version 04.12.2005 SM Comments 09.08.2006 JS
 *          Removed incorrect comment
 * @version $Date: 2007-10-01 10:24:16 +0200 (Mon, 01 Oct 2007) $, $Author: schoeni $, $Revision: 3039 $
 * @responsibilities Represents the create command of a transition.
 */
public class TransitionCreate extends NodeConnectionCreate {

	private String transitionDescription;

	/**
     * Instantiate a command that can create a connection between two shapes.
     * 
     * @param source
     *            the source endpoint (a non-null Shape instance)
     * @throws IllegalArgumentException
     *             if source is null
     */
	public TransitionCreate(Node source) {
		super(source);
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.gef.commands.Command#execute()
     */
	public void execute() {
		Transition connection = new Transition(getSource(), getTarget());
		connection.setSourceClickPoint(getSourceClickPoint());
		connection.setTargetClickPoint(getTargetClickPoint());
		
		if (getTransitionDescription() != null) {
			connection.getLabel().setName(getTransitionDescription());
		}

		setConnection(connection);
		((ConnectableElement) getSource()).connectionCreated(connection);
		((ConnectableElement) getTarget()).connectionCreated(connection);
	}

	/**
     * @return the transitionDescription
     */
	public String getTransitionDescription() {
		return transitionDescription;
	}

	/**
     * @param transitionDescription
     *            the transitionDescription to set
     */
	public void setTransitionDescription(String transitionDescription) {
		this.transitionDescription = transitionDescription;
	}

	@Override
	public String toString() {
		return "Creating a connection from " + getSource() + " to "
				+ getTarget();
	}
}
