package ch.unizh.ifi.adora.model.commands;

import ch.unizh.ifi.adora.model.representation.Transition;

/**
 * Represents a transition reconnect command.
 * 
 * @author smeier
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 10.10.2005 SM First Version 04.12.2005 SM Comments 19.07.2006 SM
 *          Comments, Reformatting
 * @version $Date: 2007-10-31 10:46:16 +0100 (Wed, 31 Oct 2007) $, $Author: schoeni $, $Revision: 3218 $
 */
public class TransitionReconnect extends ConnectionReconnect {
	/**
     * Instantiate a command that can reconnect a Connection instance to a
     * different source or target endpoint.
     * 
     * @param conn
     *            the connection instance to reconnect (non-null)
     * @throws IllegalArgumentException
     *             if conn is null
     */
	public TransitionReconnect(Transition conn) {
		super(conn);
		setLabel("Reconnect Transition");
	}

	@Override
	public String toString() {
		String text = "Reconnect transition " + connection;

		if (getNewSource() != null) {
			text += " source to " + getNewSource();
		} else {
			text += " target to " + getNewTarget();
		}

		return text;
	}
}