package ch.unizh.ifi.adora.model.commands;

import java.util.ArrayList;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.variability.Decision;
import ch.unizh.ifi.adora.model.representation.variability.VariationPoint;

/**
 * Represents a variation point add command.
 * 
 * @author Reinhard Stoiber
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 03.07.2008 RS First Version
 *          
 * @version $Date: 2010-01-28 04:43:33 +0100 (Thu, 28 Jan 2010) $, $Author: vigan $, $Revision: 5098 $
 */

public class VariationPointAdd extends Command {
	
	private VariationPoint variationPoint;
	private AdoraModel adoraModel;
	
	public VariationPointAdd(AdoraModel adoraModel, ArrayList<Decision> decisions) {
		
		if (decisions != null && !decisions.isEmpty()) {
			variationPoint = new VariationPoint(decisions);
		}
		else{
			//then add an empty ArrayList (not null)
			variationPoint = new VariationPoint(new ArrayList<Decision> ());
		}
		this.adoraModel = adoraModel;
	}
	
	public void execute() {
		adoraModel.addVariationPoint(variationPoint);
	}

	
	public void redo() {

	}

	
	public void undo() {
		ArrayList<VariationPoint> x = new ArrayList<VariationPoint>();
		x.clear();
		x.add(variationPoint);
		adoraModel.removeVariationPoints(x);
	}
	
	public VariationPoint getVariationPoint() {
		return variationPoint;
	}
}