package ch.unizh.ifi.adora.model.commands;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.variability.Decision;
import ch.unizh.ifi.adora.model.representation.variability.VariationPoint;

/**
 * This {@link Command} first deletes all visible {@link VariationPoint}s 
 * in the model and then shows the {@link VariationPoint}s either by the selected {@link Decision}s 
 * or by the selected {@link VariationPoint}s in the 
 * {@link DecisionTableViewPart} respectively in the {@link VariationPointViewPart}.
 * 
 * @author    Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history   2008-11-19 IV First Version
 * @version 
 */
public class VariationPointConnectionCreate  extends Command {
	
	/**
	 * The AdoraModel.
	 */
	private AdoraModel adoraModel;
	
	/**
	 * The {@link List} of the selected {@link VariationPoint}s.
	 */
	private List<VariationPoint> selectedVPs;
	
	/**
	 * Generates this {@link Command} out of selected {@link Decision}s 
     * in the {@link DecisionTableViewPart}.
	 * 
	 * @param adoraModel The AdoraModel.
	 * @param checkVPlist Contains a list of selected {@link Decision}
	 */
	public VariationPointConnectionCreate(AdoraModel adoraModel, ArrayList<Decision> checkVPlist) {
		this.adoraModel = adoraModel;
		if (checkVPlist != null && !checkVPlist.isEmpty()) {
			selectedVPs = getSelectedVPs(checkVPlist);
		}
	}
	
	/**
	 * Generates this {@link Command} out of selected {@link VariationPoint}s 
     * in the {@link VariationPointViewPart}.
     * 
	 * @param adoraModel The AdoraModel. 
	 * @param selectedVPlist Contains a list of {@link VariationPoint}s
	 */
	public VariationPointConnectionCreate(AdoraModel adoraModel, List<VariationPoint> selectedVPlist) {
		this.adoraModel = adoraModel;
		this.selectedVPs = selectedVPlist; 
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() {
		
		HideAllVariationPoints hideCmd = new HideAllVariationPoints(adoraModel);
		hideCmd.execute();
		
		if (selectedVPs == null)
			return;
		for (VariationPoint vp : selectedVPs) {
			//we don't need to display non-relevant or already satisfied vps 
			if (!vp.isRelevant() || vp.isSatisfied()) 
				continue;
			vp.generateVPconnections();
		}
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#redo()
	 */
	public void redo() {

	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	public void undo() {
		for (VariationPoint vp : selectedVPs) {
			vp.removeVPConnections();
		}
	}
	
	/**
	 * Extracts the {@link VariationPoint}s from the {@link Decision}s stored in the {@link List}
	 * @param decisions {@link List} storing the selected {@link Decision}s. 
	 * @return {@link List} of the corresponding {@link VariationPoint}s.
	 */
	private List<VariationPoint> getSelectedVPs(ArrayList<Decision> decisions) {
		List<VariationPoint> selectedVps = new ArrayList<VariationPoint>();
		
		List<VariationPoint> vps = adoraModel.getVariationPoints();
		
		for (VariationPoint vp : vps) {
			for (Decision dec : decisions) {
				if (vp.getDecisions().contains(dec) && !selectedVps.contains(vp)) {
					selectedVps.add(vp);
				}
				//add the VariationPoints in which the selected Decisions are involved.
				/*for (Decision vpDec : vp.getDecisions()) {
					if (dec.getNr().intValue() == vpDec.getNr().intValue() &&  !selectedVps.contains(vp)) {
						selectedVps.add(vp);
						break;
					}
				}*/
			/*	if (vp.getDecisions().contains(dec) && !selectedVps.contains(vp)) {
					selectedVps.add(vp);
				}*/
			}
		}
		return selectedVps;
	}
}