package ch.unizh.ifi.adora.model.commands;

import java.util.ArrayList;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.DecisionPointFamilyConnection;
import ch.unizh.ifi.adora.model.representation.VariationPointConnection;
import ch.unizh.ifi.adora.model.representation.variability.Decision;
import ch.unizh.ifi.adora.model.representation.variability.VariationPoint;

/**
 * Represents a variation point remove command.
 * 
 * @author Reinhard Stoiber
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 09.07.2008 RS First Version
 *          
 * @version $Date: 2008-11-12 17:21:18 +0100 (Wed, 12 Nov 2008) $, $Author: vigan $, $Revision: 3884 $
 */

public class VariationPointRemove extends Command {
	
	private ArrayList<VariationPoint> variationPoints;
	private AdoraModel adoraModel;
	
	public VariationPointRemove(AdoraModel adoraModel, ArrayList<VariationPoint> vPToRemove) {
		variationPoints = vPToRemove;
		this.adoraModel = adoraModel;
	}
	
	public void execute() {
		for (VariationPoint vp : variationPoints) {
			for (Decision d : vp.getDecisions()) {
				for (DecisionPointFamilyConnection dpfc : d.getFamilyLines()) {
					ConnectionDelete conDel = new ConnectionDelete(dpfc);
					conDel.execute();
				}
				
			}
			for (VariationPointConnection vpc : vp.getVPConnections()) {
				ConnectionDelete conDel = new ConnectionDelete(vpc);
				conDel.execute();
			}
			
			vp.removeVPConnections();
		}
		boolean x = adoraModel.removeVariationPoints(variationPoints);
	}

	
	public void redo() {

	}

	
	public void undo() {
		//TODO
		for (VariationPoint vp : variationPoints){
			adoraModel.addVariationPoint(vp);
		}
	}
	
	public ArrayList<VariationPoint> getVariationPoints() {
		return variationPoints;
	}

}
