package ch.unizh.ifi.adora.model.commands;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.model.geometry.Rectangle;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.VisualizableModelElement;
import ch.unizh.ifi.adora.utils.ZoomAlgorithmsManager;

/**
 * A command to resize and/or move a shape. The command can be undone or redone.
 *
 * @author				smeier
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				20.04.2006 SM First Version
 * 						29.08.2006 JS Use of reshape in undo (Bug 747, part 1)
 * 						30.08.2006 JS Use of adjustNode in undo (Bug 747, part 2)
 * @version				$Date: 2007-08-09 12:56:37 +0200 (Thu, 09 Aug 2007) $, $Author: treinhard $, $Revision: 2893 $
 */
public class VisualizableModelElementSetConstraint extends Command {

	/** 
     * Stores the new size and location. 
     */
	protected final Rectangle newBounds;

	/** 
	 * Stores the old size and location. 
	 */
	protected Rectangle oldBounds;

	/** 
	 * Shape to manipulate. 
	 */
	transient private final VisualizableModelElement shape;

	/**
	 * Create a command that can resize and/or move a shape.
	 * 
	 * @param part the shape to manipulate
	 * @param rect the move and resize request
	 */
	public VisualizableModelElementSetConstraint(VisualizableModelElement part, 
			Rectangle rect) {
		this.shape = part;
		this.newBounds =  rect.getCopy();
		setLabel("Change Bounds");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() {
		oldBounds = new Rectangle(shape.getLocation(), shape.getSize());
		redo();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.commands.Command#redo()
	 */
	public void redo() {	
		
		if(shape instanceof Node) {

			java.awt.Rectangle rect = new java.awt.Rectangle(newBounds.x, newBounds.y, newBounds.width, newBounds.height);
			
			ZoomAlgorithmsManager.getInstance().getMoveAlgorithm().changeNodeBounds((Node)shape, rect);

		} else {

			if (!(oldBounds.getSize().equals(newBounds.getSize()))){
				shape.setSize(newBounds.getSize());
			}
			if (!(oldBounds.getLocation().equals(newBounds.getLocation()))){
				shape.setLocation(newBounds.getLocation());
			}

		}
		
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	public void undo() {
		
		if(shape instanceof Node) {
			
			java.awt.Rectangle rect = new java.awt.Rectangle(oldBounds.x, oldBounds.y, oldBounds.width, oldBounds.height);

			ZoomAlgorithmsManager.getInstance().getMoveAlgorithm().changeNodeBounds((Node)shape, rect);
			
		} else {
		
			if (!(oldBounds.getSize().equals(newBounds.getSize()))){
				shape.setSize(oldBounds.getSize());
			}
			if (!(oldBounds.getLocation().equals(newBounds.getLocation()))){
				shape.setLocation(oldBounds.getLocation());
			}
		
		}

	}
}