package ch.unizh.ifi.adora.model.commands;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.model.representation.VisualizableModelElement;

/**
 * Command used for naming nodes.      
 *
 * @author				Ivo Vigan
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				20.04.2006 IV First Version
 * 						19.07.2006 SM Comments, Reformatting
 * 						21.07.2006 SM Class renaming.
 * @version				$Date: 2006-08-21 14:04:43 +0200 (Mon, 21 Aug 2006) $, $Author: vigan $, $Revision: 2087 $
 */
public class VisualizableModelElementSetName extends Command {

    /**
     * Name after renaming.
     */
    private String newName;
    
    /**
     * Name before renaming.
     */
    private String oldName;

    /**
     * Label which contains the name.
     */
	private VisualizableModelElement label;

	/**
	 * Initializes the command with a label.
	 */
	public VisualizableModelElementSetName() {
		super("Set Name");
	}
	
    /**
     * Instantiates a new VisualizableModelElementNameCommand.
     * 
     * @param l ModelElement to be named.
     * @param s Name
     */
	public VisualizableModelElementSetName(VisualizableModelElement l, String s) {
		label = l;
		if (s != null)
			newName = s;
		else
			newName = "";
		setLabel("set the name of an VisualizableModelElement");
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() {
		oldName = label.getName();
		label.setName(newName);
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	public void undo() {
		label.setName(oldName);
	}
}
