package ch.unizh.ifi.adora.model.commands.representation;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.utils.ZoomAlgorithmsManager;

/**
 * Shrinks a node after a view manager is applied.     
 * 
 * @author				Ivo Vigan
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			20.04.2006 SM First Version
 *						06.06.2006 JS All nodes simply shrink to 0,0 now,
 *									  resulting in the parent nodes resizing
 *									  when a node is hidden
 *						19.07.2006 SM Comments reformatting, correction
 * @version			$Date: 2009-07-01 15:32:13 +0200 (Wed, 01 Jul 2009) $, $Author: jehle $, $Revision: 4419 $
 */
public class RepresentationNodeShrink extends Command {

    /**
     * {@link Node} which gets shrinked.
     */
    private Node node;
    
    /**
     * Constructs a new RepresentationNodeShrink.
     * @param n {@link Node} which gets shrinked.
     */
    public RepresentationNodeShrink(Node n){
        this.node = n;
        setLabel("Shrinks a Node");
    }

    /**
     * Shrinks the node to its saved size.
     */
    public void execute()
    {
    	ZoomAlgorithmsManager.getInstance().getFilterAlgorithm().filterNode(node);

    	if (node instanceof AspectContainer)
    	{
    		node.setManuallyHidden(node.isVisible());
    	}
    }
}