package ch.unizh.ifi.adora.model.commands.representation;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.variability.Decision;
import ch.unizh.ifi.adora.representation.management.ViewManager;

/**
 * This command changes the view states on the {@link AdoraModel}. These
 * view-states store the information, which filters are applied.
 * 
 * Changing a filter's view state results in a visibility invertion of all
 * correspondent {@link ch.unizh.ifi.adora.model.representation.Node}s.
 * 
 * @author Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 31.03.2006 IV First Version 19.07.2006 SM Reformatting
 * @version $Date: 2010-01-14 04:48:21 +0100 (Thu, 14 Jan 2010) $, $Author: anil
 *          $, $Revision: 5079 $
 */
public class RepresentationStateChange extends Command
{

    /**
     * Type in form of a {@link java.lang.Class} which will be filtered out
     * visually from the model.
     */
    private Class type;

    /**
     * {@link AdoraModel}, at which the viewfilters are applied
     */
    private AdoraModel model;

    /**
     * Creates a new ViewCommand for a given type.
     * 
     * @param type
     *            Type in form of a {@link java.lang.Class} which will be
     *            filtered out visually from the model.
     * @param model
     *            {@link AdoraModel}, at which the viewfilters are applied
     */
    public RepresentationStateChange(Class type, AdoraModel model)
    {
        this.type = type;
        this.model = model;
        setLabel("Changes the Representation State");
    }

    /**
     * Executes the filter operation on the model.
     * 
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute() {
    	
        if (model != null) {
        
        	model.changeViewFilterState(type);
            ViewManager viewmanager = ViewManager.getInstance();
            viewmanager.applyViewFilter(type);
            
        }
    }

    /**
     * Returns true if the given representation filter is currently applied.
     * 
     * @return true, if this filter is currently applied.
     */
    public boolean isApplied()
    {
        if (model != null)
        {
            return model.getViewFilterState(type);
        }
        return false;
    }

    /**
     * Since a view filter can easily be undone by a single buttons click,
     * undoing this command isn't allowed.
     */
    public boolean canUndo()
    {
        return false;
    }
}