package ch.unizh.ifi.adora.model.factories;

import org.eclipse.gef.requests.CreationFactory;

import ch.unizh.ifi.adora.model.ModelElement;

/**
 * This factory is used to create pasted elements.
 *
 * @author				smeier
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			04.02.2006 SM First Version
 *						29.02.2006 SM Comments and fixed bug #527
 *						19.07.2006 SM Comments, Reformatting.
 * @version            $Date: 2006-07-19 15:21:49 +0200 (Wed, 19 Jul 2006) $, $Author: smeier $, $Revision: 1857 $
 * @responsibilities Factory for creating a clone of a given model element 
 *                      which will be pasted. 
 *
 */
public class AdoraCloneElementFactory implements CreationFactory {

    /**
     * The element of which a clone will be pasted into the model.
     */
	private ModelElement elementToClone;
	
    /**
     * The constructor which is used to create a clone of a given 
     * model element to copy.
     * 
     * @param o The model element that will be cloned.
     */
    public AdoraCloneElementFactory(ModelElement o) {
		elementToClone = o;
	}
	
    /**
     * The factory method which clones the given element.
     * 
     * @return Returns the newly created object.
     */
	public Object getNewObject() {
		Object o = null;
		try {
            ModelObjectMapper mop = ModelObjectMapper.getInstance();
			o = mop.getClone(elementToClone);
		} catch (CloneNotSupportedException e) {
			e.printStackTrace();
		}
		return o;
	}
	
    /**
     * Returns the element type of the model element to clone.
     * @return Returns the type (Class object) of the element
     *          which will be cloned.
     */
	public Object getObjectType() {
		return elementToClone.getClass();
	}
}

