package ch.unizh.ifi.adora.model.syntaxerrors;

import java.beans.PropertyChangeEvent;
import ch.unizh.ifi.adora.model.ModelElement;

/** 
 * A context that will be given to {@link ISyntaxVerificationRule}
 * for validation.
 * 
 * @author Cedric Jeanneret
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history   Apr 24, 2008 CJ First Version
 * @version   $Date: 2008-10-27 10:12:16 +0100 (Mon, 27 Oct 2008) $, $Author: jehle $, $Revision: 3853 $
 */
public interface ISyntaxVerificationContext {

	/**
	 * Remove all annotations from the current target
	 */
	public void clearAnnotationsFromTarget();
	
	/**
	 * Add an error message to the current target
	 * @param errorMessage
	 */
	public void addErrorToTarget(String errorMessage);
	
	/**
	 * Add a warning message to the current target
	 * @param warningMessage
	 */
	public void addWarningToTarget(String warningMessage);
	
	/**
	 * Get the particular model element the rule should validate
	 * @return the current target
	 */
	public ModelElement getTarget();
	
	/**
	 * The {@link PropertyChangeEvent} that triggered the verification
	 * @return
	 */
	public PropertyChangeEvent getEvent();
	

}
