package ch.unizh.ifi.adora.model.syntaxerrors;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.Platform;

import ch.unizh.ifi.adora.model.ModelElement;

/** 
 * These elements represents a syntax problem.
 * 
 * @author Cedric Jeanneret
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history   24.08.2008 CJ First Version
 * @version   $Date: 2008-10-27 10:12:16 +0100 (Mon, 27 Oct 2008) $, $Author: jehle $, $Revision: 3853 $
 *
 */
public class SyntaxProblem implements IAdaptable{
	
	/** 
	 * This enum enumerates the level of a problem's severity
	 *
	 */
	public static enum Severity {Warning, Error};
	
	/**
	 * The {@link ModelElement} concerned by this problem.
	 */
	private ModelElement modelElement;
	
	/**
	 * The error related to this problem
	 */
	private String errorMessage;
	
	/**
	 * The name of the syntax rule that
	 * spawned this problem
	 */
	private String syntaxRuleName;
	
	/**
	 * The severity level of this problem
	 */
	private Severity severity;
	
	
	/**
	 * Constructor for this class
	 * @param modelElement
	 * @param errorMessage
	 * @param syntaxRuleName
	 * @param severity
	 */
	SyntaxProblem(ModelElement modelElement, String errorMessage, String syntaxRuleName, Severity severity)
	{
		this.modelElement = modelElement;
		this.errorMessage = errorMessage;
		this.syntaxRuleName = syntaxRuleName;
		this.severity = severity;
	}

	/**
	 * Getter for the modelElement property
	 * @return the modelElement
	 */
	public ModelElement getModelElement() 
	{
		return modelElement;
	}

	/**
	 * Setter for the modelElement property
	 * @param modelElement the modelElement to set
	 */
	public void setModelElement(ModelElement modelElement)
	{
		this.modelElement = modelElement;
	}

	/**
	 * Getter for the errorMessage property
	 * @return the errorMessage
	 */
	public String getErrorMessage()
	{
		return errorMessage;
	}

	/**
	 * Setter for the errorMessage property
	 * @param errorMessage the errorMessage to set
	 */
	public void setErrorMessage(String errorMessage)
	{
		this.errorMessage = errorMessage;
	}

	/**
	 * Getter for the syntaxRuleName property
	 * @return the syntaxRuleName
	 */
	public String getSyntaxRuleName()
	{
		return syntaxRuleName;
	}

	/**
	 * Setter for the syntaxRuleName property
	 * @param syntaxRuleName the syntaxRuleName to set
	 */
	public void setSyntaxRuleName(String syntaxRuleName)
	{
		this.syntaxRuleName = syntaxRuleName;
	}

	/**
	 * Getter for the severity property
	 * @return the severity
	 */
	public Severity getSeverity()
	{
		return severity;
	}

	/**
	 * Setter for the severity property
	 * @param severity the severity to set
	 */
	public void setSeverity(Severity severity) 
	{
		this.severity = severity;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	@SuppressWarnings("unchecked")
	public Object getAdapter(Class adapter)
	{
		return Platform.getAdapterManager().getAdapter(this, adapter);
	}
}
