package ch.unizh.ifi.adora.model.syntaxerrors.rules;

import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.model.syntaxerrors.ISyntaxVerificationContext;
import ch.unizh.ifi.adora.model.syntaxerrors.ISyntaxVerificationRule;


/** 
 * Checks the syntax of the transform of a {@link Scenario}
 * 
 * @author Cedric Jeanneret
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history   24.04.2008 CJ First Version
 * @version   $Date: 2008-10-27 10:12:16 +0100 (Mon, 27 Oct 2008) $, $Author: jehle $, $Revision: 3853 $
 *
 */
public class ScenarioTransformSyntaxRule implements ISyntaxVerificationRule
{
	/**
	 * Name of this {@link ISyntaxVerificationRule}
	 */
	public static final String RULE_NAME = "Transform of Scenario";

	
	/* (non-Javadoc)
	 * @see ch.unizh.ifi.adora.model.syntaxerrors.rules.ISyntaxVerificationRule#verify(ch.unizh.ifi.adora.model.syntaxerrors.ISyntaxVerificationContext)
	 */
	public void verify(ISyntaxVerificationContext context)
	{
		ModelElement target = context.getTarget();
		if (target instanceof Scenario)
		{
			String answer = null;
			answer = ((Scenario) target).parseTransformExpression();
			if(answer == null || answer.equals(""))
			{
				context.clearAnnotationsFromTarget();
			}
			else
			{
				context.addErrorToTarget(answer);
			}
		}
	}

	/* (non-Javadoc)
	 * @see ch.unizh.ifi.adora.model.syntaxerrors.rules.ISyntaxVerificationRule#getRuleName()
	 */
	public String getRuleName()
	{
		return RULE_NAME;
	}
	

}
