package ch.unizh.ifi.adora.model.syntaxerrors.rules;

import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.syntaxerrors.ISyntaxVerificationContext;
import ch.unizh.ifi.adora.model.syntaxerrors.ISyntaxVerificationRule;

/** 
 * Checks the syntax of the specification of a {@link Component}
 * or an {@link AspectContainer}
 * 
 * @author Cedric Jeanneret
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history   24.04.2008 CJ First Version
 * @version   $Date: 2008-10-27 10:12:16 +0100 (Mon, 27 Oct 2008) $, $Author: jehle $, $Revision: 3853 $
 *
 */
public class SpecificationSyntaxRule implements ISyntaxVerificationRule {

	/**
	 * Name of this {@link ISyntaxVerificationRule}
	 */
	public static final String RULE_NAME = "Functional Specification";
	
	/* (non-Javadoc)
	 * @see ch.unizh.ifi.adora.model.syntaxerrors.rules.ISyntaxVerificationRule#verify(ch.unizh.ifi.adora.model.syntaxerrors.ISyntaxVerificationContext)
	 */
	public void verify(ISyntaxVerificationContext context) {
		ModelElement target = context.getTarget();
		if(target instanceof Component || target instanceof AspectContainer)
		{
    		String answer = null;
    		if (target instanceof Component)
    		{
    			Component component = (Component) target;
    			answer = component.parseFunctionalSpecification();
    		}
    		if (target instanceof AspectContainer)
    		{
    			AspectContainer aspectContainer = (AspectContainer) target;
    			answer = aspectContainer.parseFunctionalSpecification();
    		}
    		if(answer == null || answer.equals(""))
    		{
    			context.clearAnnotationsFromTarget();
    		}
    		else
    		{
    			context.addErrorToTarget(answer);
    		}
		}
		
	}

	/* (non-Javadoc)
	 * @see ch.unizh.ifi.adora.model.syntaxerrors.rules.ISyntaxVerificationRule#getRuleName()
	 */
	public String getRuleName() {
		return RULE_NAME;
	}

}
