package ch.unizh.ifi.adora.parts;

import java.beans.PropertyChangeEvent;

import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.gef.commands.Command;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PlatformUI;

import ch.unizh.ifi.adora.model.properties.AbstractableConnectionPropNames;
import ch.unizh.ifi.adora.model.representation.IAbstractableConnection;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;
import ch.unizh.ifi.adora.view.figures.IAbstractableConnectionFigure;

/**
 * This Class acts as a superclass for all abstractable connection edit parts. 
 * 
 * @author		        Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             21.09.2006 IV First Version
 * @version             $Date: 2006-09-26 10:52:33 +0200 (Tue, 26 Sep 2006) $, $Author: vigan $, $Revision: 2267 $
 */
public abstract class AbstractableConnectionEditPart extends LabeledConnectionEditPart{
	
	/**
	 * The width of the figure if abstracted.
	 */
	protected static final int ABSTRACTED_WIDTH = 2;

	/**
	 * The width of the figure if not abstracted.
	 */
	protected static final int NORMAL_WIDTH = 1;

	/**
	 * Return the figure, associated with this edit part.
	 * @return {@link IAbstractableConnectionFigure}
	 */
	public abstract IAbstractableConnectionFigure getCastedFigure();
	
	/**
	 * Returns the underlying model. 
	 * @return Returns the model which belongs to this edit part.
	 */
	public abstract IAbstractableConnection getCastedModel();
	
	
	/**
	 * Refresh Code used due to initialisation Faults caused by GEF.
	 * GEF doesn't set the tickness of a line correctly when a model is loaded
	 * so we have to override this method to visualize abstract associations
	 * correctely.
	 */
	protected void refreshVisuals() {
		boolean isAbstracted = getCastedModel().isAbstracted();
		int lineWidth = isAbstracted ? ABSTRACTED_WIDTH : NORMAL_WIDTH;
		PolylineConnection f = (PolylineConnection)getCastedFigure();
		f.setLineWidth(lineWidth);
		f.repaint();
		super.refreshVisuals();
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent event) {
		String property = event.getPropertyName();
		if (AbstractableConnectionPropNames.MAN_ABSTRACTED_PROP.equals(property)
				|| AbstractableConnectionPropNames.CALC_ABSTRACTED_PROP.equals(property)) {
			
			getCastedFigure().setAbstracted(getCastedModel().isAbstracted());
			refreshVisuals();
		}
		super.propertyChange(event);
	}
	
	/**
    * Executes a given {@link org.eclipse.gef.commands.Command} on the {@link org.eclipse.gef.commands.CommandStack} 
    * of the editor.
    *
    * @param cmd The {@link org.eclipse.gef.commands.Command} which gets executed on the 
    * {@link org.eclipse.gef.commands.CommandStack}.
    */
	protected void executeCommandOnStack(Command cmd){
		IWorkbenchPage page = PlatformUI.getWorkbench().
			getActiveWorkbenchWindow().getActivePage();
		
		try {
			IEditorPart editor = page.getActiveEditor();
			if (editor instanceof AdoraModelEditor) {
				((AdoraModelEditor)editor).getCommandStack().execute(cmd);
			}
		} catch (Exception ex) {
			ex.printStackTrace();
		}
   }
}