package ch.unizh.ifi.adora.parts;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartFactory;

import ch.unizh.ifi.adora.model.representation.Actor;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.Association;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.ConstraintConnection;
import ch.unizh.ifi.adora.model.representation.ConstraintImplicationConnection;
import ch.unizh.ifi.adora.model.representation.DecisionPointFamilyConnection;
import ch.unizh.ifi.adora.model.representation.ExitPoint;
import ch.unizh.ifi.adora.model.representation.Interrelationship;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.model.representation.ScenarioConnection;
import ch.unizh.ifi.adora.model.representation.State;
import ch.unizh.ifi.adora.model.representation.Transition;
import ch.unizh.ifi.adora.model.representation.VariationPointConnection;


/**
 * 
 * This is a factory that provides a specific edit part for a given model element.
 *      
 * @author				Adrian Gygax
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			31.08.2006 AG First Version
 * 						20.07.2006 SM Comments reformatting, completion and checking
 * 
 * @version			$Date: 2009-05-06 10:51:39 +0200 (Wed, 06 May 2009) $, $Author: treinhard $, $Revision: 4182 $
 */
public class AdoraEditPartFactory implements EditPartFactory {

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.EditPartFactory#createEditPart(org.eclipse.gef.EditPart,
	 *      java.lang.Object)
	 */
	public EditPart createEditPart(EditPart context, Object modelElement) {
		// get EditPart for model element
		EditPart part = getPartForElement(modelElement);
		// store model element in EditPart
		part.setModel(modelElement);
		return part;
	}

	/**
	 * Maps an object to an EditPart.
	 * 
	 * @throws RuntimeException
	 *             if no match was found (programming error)
	 */
	private EditPart getPartForElement(Object modelElement) {
		if (modelElement instanceof AdoraModel) {
			return new AdoraModelEditPart();
		}
		if (modelElement instanceof Actor) {
			return new ActorEditPart();
		}
		if (modelElement instanceof State) {
			return new StateEditPart();
		}
		if (modelElement instanceof Scenario) {
			return new ScenarioEditPart();
		}
		if (modelElement instanceof Transition) {
			return new TransitionEditPart();
		}
		if (modelElement instanceof Association) {
			return new AssociationEditPart();
		}
		if (modelElement instanceof ScenarioConnection) {
			return new ScenarioConnectionEditPart();
		}
        if (modelElement instanceof Interrelationship) {
            return new InterrelationshipEditPart();
        }
        if (modelElement instanceof JoinRelation) {
            return new JoinRelationEditPart();
        }
		if (modelElement instanceof Component) {
			return new ComponentEditPart();
		}
        if (modelElement instanceof AspectContainer) {
            return new AspectContainerEditPart();
        }
        if (modelElement instanceof ExitPoint) {
            return new ExitPointEditPart();
        }
        if (modelElement instanceof VariationPointConnection) {
            return new VariationPointConnectionEditPart();
        }
        if (modelElement instanceof DecisionPointFamilyConnection) {
            return new DecisionPointFamilyConnectionEditPart();
        }
        if (modelElement instanceof ConstraintConnection) {
            return new ConstraintConnectionEditPart();
        }
        if (modelElement instanceof ConstraintImplicationConnection) {
            return new ConstraintImplicationEditPart();
        }
        
		throw new RuntimeException("Can't create part for model element: "
				+ ((modelElement != null) ? modelElement.getClass().getName()
						: "null"));
	}
}