package ch.unizh.ifi.adora.parts;

import org.eclipse.draw2d.FreeformLayer;
import org.eclipse.draw2d.FreeformLayeredPane;
import org.eclipse.draw2d.LayeredPane;
import org.eclipse.gef.editparts.ScalableFreeformRootEditPart;

import ch.unizh.ifi.adora.view.layers.LabelPlacementVisualizationLayer;
import ch.unizh.ifi.adora.view.layers.LayoutVisualizationLayer;
import ch.unizh.ifi.adora.view.layers.MultipleRoutersConnectionLayer;
import ch.unizh.ifi.adora.view.layers.RoutingVisualizationLayer;

/**
 * This is the topmost edit part of the adora editor that manages the different
 * layers (primary layer, connection layer etc.).<br>
 * <br>
 * The class {@link org.eclipse.gef.editparts.ScalableFreeformRootEditPart} is
 * subclassed so that the multiple routers connection layer can be used instead of
 * the standard gef connection layer.
 *
 * @author				Tobias Reinhard
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				2006-04-25 TR First Version
 * 						2006-07-20 SM Comments reformatted and checked.
 * 						2006-08-04 JS Put the LayoutVisualizationLayer in front of
 * 									  the other layers
 * @version			$ Date: 2006/04/20 11:59:51 $, $ Author: vigan $, $ Revision: 1.34 $
 * @responsibilities	Manages the different layers of the editor
 *
 */
public class AdoraRootEditPart extends ScalableFreeformRootEditPart {
 
    /**
     * Creates the layers that are visible on the screen and are printed. The
     * method adds the multiple routers connection layer instead of the standard
     * gef connection layer. 
     */
    protected LayeredPane createPrintableLayers() {
        FreeformLayeredPane layeredPane = new FreeformLayeredPane();
        layeredPane.add(new FreeformLayer(), PRIMARY_LAYER);
        layeredPane.add(new LayoutVisualizationLayer(), "LAYOUT_VISUALIZATION_LAYER");
        layeredPane.add(new RoutingVisualizationLayer(), "ROUTING_VISUALIZATION_LAYER");
        layeredPane.add(new LabelPlacementVisualizationLayer(), "LABEL_PLACEMENT_VISUALIZATION_LAYER");
        layeredPane.add(new MultipleRoutersConnectionLayer(), CONNECTION_LAYER);
        return layeredPane;
    }

	@Override
	/**
	 * Since the root figure can't be GCed, make the memory foot print as small as possible after deactivation
	 * See #11.
	 * TODO: Find a way to have the FigureCanvas GCed.
	 */
	public void deactivate() {
		getPrintableLayers().removeAll();
		super.deactivate();
	}

    
}
