package ch.unizh.ifi.adora.parts;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.LayoutAnimator;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.ui.views.properties.IPropertySource;

import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.parts.directedit.NodeCellEditorLocator;
import ch.unizh.ifi.adora.parts.directedit.NodeEditManager;
import ch.unizh.ifi.adora.parts.policies.containers.AspectContainerPolicy;
import ch.unizh.ifi.adora.parts.policies.nodes.AspectContainerNodePolicy;
import ch.unizh.ifi.adora.properties.AspectContainerProperties;
import ch.unizh.ifi.adora.utils.ModelElementVisibilityFilter;
import ch.unizh.ifi.adora.utils.filteredlist.FilteredArrayList;
import ch.unizh.ifi.adora.view.anchors.AspectContainerClickPointAnchor;
import ch.unizh.ifi.adora.view.figures.AspectContainerFigure;

/**
 * Edipart representing an AspectContainer.      
 *
 * @author				Ivo Vigan
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			28.04.2006 IV First Version
 * 						20.07.2006 SM Comments checked and corrected.
 * 						15.08.2006 JS getCastedObjectModel() renamed to 
 * 									  getCastedModel(), unnecessary casts removed
 * @version			$Date: 2008-11-25 14:32:54 +0100 (Tue, 25 Nov 2008) $, $Author: treinhard $, $Revision: 3896 $
 */
public class AspectContainerEditPart extends ZoomableNodeContainerEditPart implements IAdaptable{

    /**
     * IPropertySource for this ModelElement.
     */
    private AspectContainerProperties aspectContainerProps;
    
   /*
    *  (non-Javadoc)
    * @see ch.unizh.ifi.adora.parts.VisualizableModelElementEditPart#createFigureForModel()
    */
    protected IFigure createFigureForModel() {
        AspectContainerFigure f = new AspectContainerFigure(
        		getCastedModel().getName(),
        		getCastedModel().isPartial());
        
        f.setZoomed(getCastedModel().isZoomedIn());
        f.setComment(getCastedModel().getComment());
        f.setCalculatedPartial(getCastedModel().isCalculatedPartial());
        f.addLayoutListener(LayoutAnimator.getDefault());
		return f; 
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.gef.editparts.AbstractEditPart#getModelChildren()
     */
    protected List getModelChildren() {
        FilteredArrayList list = new FilteredArrayList(
                (ArrayList) getCastedModel().getChildren());
            list.setFilter(new ModelElementVisibilityFilter());
        return list;
    }

    /**
     * Performs the direct editing capabilities.
     */
    private void performDirectEdit() {
        if (manager == null)
            manager = new NodeEditManager(this, new NodeCellEditorLocator(
                    ((AspectContainerFigure) getFigure()).getNameLabel()));
        manager.setLabel(((AspectContainerFigure)getFigure()).getNameLabel());
        manager.show();
    }

    /*
     *  (non-Javadoc)
     * @see org.eclipse.gef.EditPart#performRequest(org.eclipse.gef.Request)
     */
    public void performRequest(Request request) {
        if (request.getType() == RequestConstants.REQ_DIRECT_EDIT)
            performDirectEdit();
    }
    
    /*
     *  (non-Javadoc)
     * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
     */
    protected void createEditPolicies() {
        super.createEditPolicies();

        installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE,
                new AspectContainerNodePolicy());
        
        installEditPolicy(EditPolicy.LAYOUT_ROLE,
                new AspectContainerPolicy());
    }
    
    /*
     *  (non-Javadoc)
     * @see ch.unizh.ifi.adora.parts.ZoomableNodeContainerEditPart#getCastedModel()
     */
    protected AspectContainer getCastedModel() {
        return (AspectContainer) getModel();
    }
    
    /*
     * (non-Javadoc)
     * @see ch.unizh.ifi.adora.parts.ZoomableNodeContainerEditPart#createConnectionAnchor()
     */
    protected ConnectionAnchor createConnectionAnchor() {
        // gets this.getFigure() and sets it as 
        // figure for which the anchor has to be computed
        //return new AspectContainerClickPointAnchor(getFigure());
        return new AspectContainerClickPointAnchor(getFigure());
    }
    
    /**
	 * Returns the IPropertySource for this Modelelement.
	 * 
	 * @param adapter The class for which an object should be returned.
	 * @return Returns the object for the given adapter.
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getAdapter(java.lang.Class)
	 */	
    public Object getAdapter(Class adapter) {
         if (adapter == IPropertySource.class) {
            if (aspectContainerProps == null) {
                aspectContainerProps = new AspectContainerProperties(getCastedModel());
            }
            return aspectContainerProps;
         }  
         
         return super.getAdapter(adapter);
         
    }
}