package ch.unizh.ifi.adora.parts;

import java.beans.PropertyChangeEvent;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.LayoutAnimator;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.ui.views.properties.IPropertySource;

import ch.unizh.ifi.adora.model.properties.CardinalityPropNames;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.parts.directedit.NodeCellEditorLocator;
import ch.unizh.ifi.adora.parts.directedit.NodeEditManager;
import ch.unizh.ifi.adora.parts.policies.containers.ComponentContainerPolicy;
import ch.unizh.ifi.adora.parts.policies.nodes.ComponentPolicy;
import ch.unizh.ifi.adora.properties.ComponentProperties;
import ch.unizh.ifi.adora.view.figures.ComponentFigure;

/**
 * Provides the EditPart for a Component.
 *      
 * @author				Silvio Meier
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			Dec 5,  2005 SM First Version
 * 						Jul 20, 2006 SM Comments check and correct.
 * 										Some refactoring.
 * 						Aug 15, 2006 JS Renamed getCastedObjectModel() to 
 * 										getCastedModel(), unnecessary casts
 * 										removed
 * @version			$Date: 2008-11-25 14:32:54 +0100 (Tue, 25 Nov 2008) $, $Author: treinhard $, $Revision: 3896 $
 *
 */
public class ComponentEditPart extends StartStateEditPart implements IAdaptable {

	/**
	 * IPropertySource for this ModelElement.
	 */
	private ComponentProperties componentProps;

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.parts.NodeEditPart#createFigureForModel()
	 */
	protected IFigure createFigureForModel() {
		Component model = getCastedModel();
		ComponentFigure compFig = new ComponentFigure(
				model.getMinCardinality(), model.getMaxCardinality(), model
						.isStartState(), model.isPartial(), model.isExternal());
		compFig.setText(model.getName());
		compFig.setZoomed(model.isZoomedIn());
		compFig.setComment(getCastedModel().getComment());
		compFig.setCalculatedPartial(getCastedModel().isCalculatedPartial());
		
		compFig.addLayoutListener(LayoutAnimator.getDefault());
		
		return compFig;
	}

	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		super.createEditPolicies();

		installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE, new ComponentPolicy());
		// allow removal of the associated model element
		installEditPolicy(EditPolicy.LAYOUT_ROLE,
				new ComponentContainerPolicy());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent evt) {
		String prop = evt.getPropertyName();
		ComponentFigure f = (ComponentFigure) getFigure();
		if (CardinalityPropNames.MINCARD_PROP.equals(prop)) {
			 
			f.setMinCardinality( getCastedModel()
					.getMinCardinality());
			f.relayout();
			f.repaint();

		} else if (CardinalityPropNames.MAXCARD_PROP.equals(prop)) {
			f.setMaxCardinality(getCastedModel()
					.getMaxCardinality());
			f.relayout();
			f.repaint();
		} else if (Component.ISEXTERNAL_PROP.equals(prop)) {
			
			f.setExternal(((Component) getCastedModel()).isExternal());
			f.repaint();
		} else if (Component.ADORA_TYPE_PROP.equals(prop)) {			
			if (getCastedModel().getType() == null){
				f.setType("");
			}
			else{
				f.setType(getCastedModel().getType().getName());
			}
			f.repaint();
		} else {
			super.propertyChange(evt);
		}
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.parts.ZoomableNodeContainerEditPart#getCastedModel()
	 */
	protected Component getCastedModel() {
		return (Component) getModel();
	}

	/**
	 * performs the direct edit by creating a new NodeEditManager.
	 */
	private void performDirectEdit() {
		ComponentFigure f = (ComponentFigure) getFigure();
	//	if (manager == null)
			manager = new NodeEditManager(this, 
					new NodeCellEditorLocator(f.getNameLabel()));
		manager.setLabel(f.getNameLabel());
		manager.show();
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractEditPart#performRequest(org.eclipse.gef.Request)
	 */
	public void performRequest(Request request) {
		if (request.getType() == RequestConstants.REQ_DIRECT_EDIT) {
			performDirectEdit();
		}
	}

	/**
	 * Returns the IPropertySource for this ModelElement.
	 * @pre       true
	 * @post      return != null
	 * @param adapter The adapter class object.
	 * @return Returns the IPropertySource for this ModelElement
	 * @see org.eclipse.gef.editparts.AbstractConnectionEditPart#getAdapter(java.lang.Class)
	 */
	public Object getAdapter(Class adapter) {
		if (adapter == IPropertySource.class) {
			if (componentProps == null) {
				componentProps = new ComponentProperties(getCastedModel());
			}
			return componentProps;
		}
		
		return super.getAdapter(adapter);
		
	}
}