package ch.unizh.ifi.adora.parts;

import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.NodeEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.requests.DropRequest;

import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.view.anchors.ClickPointAnchor;
import ch.unizh.ifi.adora.view.anchors.ConstrainableConnectionAnchor;
import ch.unizh.ifi.adora.view.figures.ConnectionFigure;

/**
 * Superclass for all {@link ConnectionEditPart}s which can 
 * be crosscutted by {@link ch.unizh.ifi.adora.model.representation.JoinRelation}s.
 * 
 * @author		        Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             21.09.2006 IV First Version
 * @version             $Date: 2007-04-20 09:42:41 +0200 (Fri, 20 Apr 2007) $, $Author: treinhard $, $Revision: 2607 $
 */
public class ConstrainableConnectionEditPart extends ConnectionEditPart
		implements NodeEditPart {

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
     */
    public ConnectionAnchor getTargetConnectionAnchor(
            org.eclipse.gef.ConnectionEditPart connection) {
    	
    	ConnectionAnchor anchor = new ConstrainableConnectionAnchor((PolylineConnection)getFigure());
		
    	float xRatio = ((Connection)connection.getModel()).getTargetXRatio();
		float yRatio = ((Connection)connection.getModel()).getTargetYRatio();
		
		((ClickPointAnchor)anchor).setClickPoint(xRatio, yRatio);

		return anchor;
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.Request)
     */
    public ConnectionAnchor getTargetConnectionAnchor(Request request) {
    	
		ConnectionAnchor anchor = new ConstrainableConnectionAnchor((PolylineConnection)getConnectionFigure());

        org.eclipse.draw2d.geometry.Point clickPoint = ((DropRequest) request).getLocation().getCopy();
        IFigure f = getFigure();
        f.translateToRelative(clickPoint);
        f.translateFromParent(clickPoint);
        Rectangle realBounds = ((ConnectionFigure)getFigure()).getRealBounds();
        clickPoint.translate(realBounds.getLocation().getNegated());
            
        float xRatio = clickPoint.x / (float)realBounds.width;
    	float yRatio = clickPoint.y / (float)realBounds.height;
    
        ((ClickPointAnchor) anchor).setClickPoint(xRatio, yRatio);

        return anchor;
    }

	public ConnectionAnchor getSourceConnectionAnchor(
			org.eclipse.gef.ConnectionEditPart connection) {
		ConnectionAnchor anchor = new ConstrainableConnectionAnchor(
				(PolylineConnection) getFigure());

		float xRatio = ((Connection) connection.getModel()).getTargetXRatio();
		float yRatio = ((Connection) connection.getModel()).getTargetYRatio();
		((ClickPointAnchor) anchor).setClickPoint(xRatio, yRatio);

		return anchor;
	}

	public ConnectionAnchor getSourceConnectionAnchor(Request request) {
		ConnectionAnchor anchor = new ConstrainableConnectionAnchor(
				(PolylineConnection) getConnectionFigure());

		org.eclipse.draw2d.geometry.Point clickPoint = ((DropRequest) request)
		.getLocation().getCopy();
		IFigure f = getFigure();
		f.translateToRelative(clickPoint);
		f.translateFromParent(clickPoint);
		Rectangle realBounds = ((ConnectionFigure)getFigure()).getRealBounds();
		clickPoint.translate(realBounds.getLocation().getNegated());

		float xRatio = clickPoint.x / (float)realBounds.width;
		float yRatio = clickPoint.y / (float)realBounds.height;

		((ClickPointAnchor)anchor).setClickPoint(xRatio, yRatio);

		return anchor;
	}
}