package ch.unizh.ifi.adora.parts;

import java.beans.PropertyChangeEvent;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.NodeEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.requests.DropRequest;

import ch.unizh.ifi.adora.model.properties.ConstraintConnectionPropNames;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.ConstraintConnection;
import ch.unizh.ifi.adora.view.anchors.ClickPointAnchor;
import ch.unizh.ifi.adora.view.anchors.ConstraintConnectionAnchor;
import ch.unizh.ifi.adora.view.figures.ConnectionFigure;
import ch.unizh.ifi.adora.view.figures.ConstraintConnectionFigure;

/**
 * Editpart for the {@link ConstraintConnection}.
 * 
 * @author		        Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             2.12.2008 IV First Version
 */
public class ConstraintConnectionEditPart  extends ConnectionEditPart
		implements NodeEditPart {

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
     */
    public ConnectionAnchor getTargetConnectionAnchor(
            org.eclipse.gef.ConnectionEditPart connection) {
    	
    	ConnectionAnchor anchor = new ConstraintConnectionAnchor((PolylineConnection)getFigure());
		
    	float xRatio = ((Connection)connection.getModel()).getTargetXRatio();
		float yRatio = ((Connection)connection.getModel()).getTargetYRatio();
		
		((ClickPointAnchor)anchor).setClickPoint(xRatio, yRatio);

		return anchor;
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.Request)
     */
    public ConnectionAnchor getTargetConnectionAnchor(Request request) {
    	
		ConnectionAnchor anchor = new ConstraintConnectionAnchor((PolylineConnection)getConnectionFigure());

        org.eclipse.draw2d.geometry.Point clickPoint = ((DropRequest) request).getLocation().getCopy();
        IFigure f = getFigure();
        f.translateToRelative(clickPoint);
        f.translateFromParent(clickPoint);
        Rectangle realBounds = ((ConnectionFigure)getFigure()).getRealBounds();
        clickPoint.translate(realBounds.getLocation().getNegated());
            
        float xRatio = clickPoint.x / (float)realBounds.width;
    	float yRatio = clickPoint.y / (float)realBounds.height;
    
        ((ClickPointAnchor) anchor).setClickPoint(xRatio, yRatio);

        return anchor;
    }

	public ConnectionAnchor getSourceConnectionAnchor(
			org.eclipse.gef.ConnectionEditPart connection) {
		ConnectionAnchor anchor = new ConstraintConnectionAnchor(
				(PolylineConnection) getFigure());

		float xRatio = ((Connection) connection.getModel()).getTargetXRatio();
		float yRatio = ((Connection) connection.getModel()).getTargetYRatio();
		((ClickPointAnchor) anchor).setClickPoint(xRatio, yRatio);

		return anchor;
	}

	public ConnectionAnchor getSourceConnectionAnchor(Request request) {
		ConnectionAnchor anchor = new ConstraintConnectionAnchor(
				(PolylineConnection) getConnectionFigure());

		org.eclipse.draw2d.geometry.Point clickPoint = ((DropRequest) request)
		.getLocation().getCopy();
		IFigure f = getFigure();
		f.translateToRelative(clickPoint);
		f.translateFromParent(clickPoint);
		Rectangle realBounds = ((ConnectionFigure)getFigure()).getRealBounds();
		clickPoint.translate(realBounds.getLocation().getNegated());

		float xRatio = clickPoint.x / (float)realBounds.width;
		float yRatio = clickPoint.y / (float)realBounds.height;

		((ClickPointAnchor)anchor).setClickPoint(xRatio, yRatio);

		return anchor;
	}
	/*
	 * (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractConnectionEditPart#createFigure()
	 */
    @Override
    protected IFigure createFigure() {
        ConstraintConnectionFigure connection = new ConstraintConnectionFigure(
        		getCastedModel().getOperation(), 
        		getCastedModel().isNegFrom(), 
        		getCastedModel().isNegTo());
        
    	connection.setForegroundColor(ColorConstants.green);
        
    	return connection;
    }
    
    /**
     * ConstraintConnections don't have any policies, they're not editable.
     * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
     */
    @Override
    protected void createEditPolicies() {}
    
    
    public void propertyChange(PropertyChangeEvent event) {
		String property = event.getPropertyName();
		if (property.equals(ConstraintConnectionPropNames.BUILT)) {
			((ConstraintConnectionFigure)getFigure()).setOpCode(getCastedModel().getOperation());
			((ConstraintConnectionFigure)getFigure()).setNegFrom(getCastedModel().isNegFrom());
			((ConstraintConnectionFigure)getFigure()).setNegTo(getCastedModel().isNegTo());
			((ConstraintConnectionFigure)getFigure()).repaint();
        } super.propertyChange(event);
	}
    
	
	/**
	 * Returns the {@link ConstraintConnection} model element
	 * @return  the {@link ConstraintConnection} 
	 */
	private ConstraintConnection getCastedModel(){
		return (ConstraintConnection)getModel();
	}
}