package ch.unizh.ifi.adora.parts;

import java.beans.PropertyChangeEvent;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.IFigure;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.NodeEditPart;

import ch.unizh.ifi.adora.model.properties.ConstraintConnectionPropNames;
import ch.unizh.ifi.adora.model.properties.VisualModElemPropNames;
import ch.unizh.ifi.adora.model.representation.ConstraintConnection;
import ch.unizh.ifi.adora.model.representation.ConstraintConnectionLabel;
import ch.unizh.ifi.adora.model.representation.ConstraintImplicationConnection;
import ch.unizh.ifi.adora.model.representation.VariationPointConnectionLabel;
import ch.unizh.ifi.adora.view.figures.ConstraintImplicationFigure;


public class ConstraintImplicationEditPart extends LabeledConnectionEditPart
		implements NodeEditPart {

	

	
	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.gef.EditPart#activate()
	 */
	public void activate() {
		if (!isActive()) {
			ConstraintConnectionLabel child = getCastedModel().getLabel();
			child.addPropertyChangeListener(this);
			super.activate();
		}
	}

	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.gef.EditPart#deactivate()
	 */
	public void deactivate() {
		if (isActive()) {
			ConstraintConnectionLabel child = getCastedModel().getLabel();
			child.removePropertyChangeListener(this);
			super.deactivate();
		}
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractConnectionEditPart#createFigure()
	 */
    @Override
    protected IFigure createFigure() {
    	ConstraintImplicationFigure f = new ConstraintImplicationFigure(
    			getCastedModel().isImplication());
    	return f;
    }
    
	/*
	 * (non-Javadoc)
	 * @see ch.unizh.ifi.adora.parts.ConnectionEditPart#propertyChange(java.beans.PropertyChangeEvent)
	 */
    public void propertyChange(PropertyChangeEvent event) {
		String property = event.getPropertyName();
		if (property.equals(ConstraintConnectionPropNames.BUILT)) {
			((ConstraintImplicationFigure)getFigure()).setImplication(getCastedModel().isImplication());
			((ConstraintImplicationFigure)getFigure()).repaint();
        } else if (VisualModElemPropNames.NAME_PROP.equals(property)) {
        	refresh();
        } super.propertyChange(event);
	}
	/**
	 * Returns the {@link ConstraintConnection} model element
	 * @return  the {@link ConstraintConnection} 
	 */
	private ConstraintImplicationConnection getCastedModel(){
		return (ConstraintImplicationConnection)getModel();
	}
	
	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createChild(java.lang.Object)
	 */
	public EditPart createChild(Object model) {
		ConstraintImplicationLabelEditPart part = 
				new ConstraintImplicationLabelEditPart();
		part.setModel(model);
		return part;
	}
 	
	/**
	 * Returns the {@link VariationPointConnectionLabel} if its content is unequal the default content.
	 * @return Returns a {@link java.util.List} containing the model children of this
	 *          Join Relationship.
	 * @see org.eclipse.gef.editparts.AbstractEditPart#getModelChildren()
	 */
	public List<ConstraintConnectionLabel> getModelChildren() {
		List<ConstraintConnectionLabel> children = new ArrayList<ConstraintConnectionLabel>();
		ConstraintConnectionLabel lbl = getCastedModel().getLabel();
		
		children.add(lbl);
		
		return children;
	}
}