package ch.unizh.ifi.adora.parts;

import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.tools.DirectEditManager;

import ch.unizh.ifi.adora.parts.directedit.NodeCellEditorLocator;
import ch.unizh.ifi.adora.parts.directedit.NodeEditManager;
import ch.unizh.ifi.adora.parts.policies.LabelDirectEditPolicy;
import ch.unizh.ifi.adora.view.figures.label.ConnectionLabel;


/**
 * Superclass for {@link RoleEditPart} and
 * {@link ch.unizh.ifi.adora.parts.TransitionLabelEditPart}. 
 * 
 * @author				Ivo Vigan
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			19.07.2006 IV First Version
 * 						20.07.2006 SM Comments check.
 * @version			$Date: 2006-08-24 13:29:45 +0200 (Thu, 24 Aug 2006) $, $Author: vigan $, $Revision: 2101 $
 */
public abstract class DirectEditableLabelEditPart extends ConnectionLabelEditPart{

	/**
	 * Manages the direct editing of the label.
	 */
	private DirectEditManager manager;

	/**
	 * Creates all the necessary policies. 
	 * Only the policie for direct editing is specialy needed.
	 */
	protected void createEditPolicies() {
		super.createEditPolicies();
		installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE,
				new LabelDirectEditPolicy());
	}
	
	/**
	 * Handles the directediting of the Label.
	 */
	private void performDirectEdit() {
		if (manager == null)
			manager = new NodeEditManager(this, new NodeCellEditorLocator(
					((ConnectionLabel)getFigure()).getWrappingLabel()));
		manager.show();
	}

	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.gef.EditPart#performRequest(org.eclipse.gef.Request)
	 */
	public void performRequest(Request request) {
		if (request.getType() == RequestConstants.REQ_DIRECT_EDIT) {
			performDirectEdit();
		}
	}
}
