package ch.unizh.ifi.adora.parts;

import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;

/**
 * This interface is used for all Nodes which can be connected by Tranisitons.
 * These nodes which build a graph where the transition is the edge have to be checked
 * if they are selected or not. This is done when moving multiple nodes into an other parent.
 * 
 * @author		        Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             31.08.2006 IV First Version
 * @version             $Date: 2006-09-27 17:02:22 +0200 (Wed, 27 Sep 2006) $, $Author: vigan $, $Revision: 2287 $
 */
public interface ITransitionGraph {

	/**
     * Checks if {@link ZoomableNodeContainer}s can be moved.
     *  
     * @return <code>true, </code> if the whole graph in which the currently draged 
     * {@link ZoomableNodeContainer} is contained is selected.
     */
	boolean depthFirstSearch();
	
	/**
     * Resets the {@link ZoomableNodeContainerEditPart#visited} flag 
     * in all {@link ZoomableNodeContainerEditPart} which were tested for 
     * selection.
     */
    void resetVisited();
    
    /**
     * Used to determine if the node was allready checked.
     * @return <code>true, </code> if the node was allready checked.
     */
    boolean wasGraphNodeChecked();
}