package ch.unizh.ifi.adora.parts;

import java.beans.PropertyChangeEvent;

import org.eclipse.draw2d.IFigure;
import org.eclipse.ui.views.properties.IPropertySource;

import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.parts.figureinterfaces.INodeProperties;
import ch.unizh.ifi.adora.properties.JoinRelationProperties;
import ch.unizh.ifi.adora.view.figures.JoinRelationFigure;
import ch.unizh.ifi.adora.view.figures.label.ConnectionLabel;
import ch.unizh.ifi.adora.view.figures.label.algorithm.placing.ConnectionMidpointLocator;

/**
 * Editpart used to direct edit, 
 * resize and relocate labels associated with an JoinRelation.
 * 
 * @author				Ivo Vigan
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			19.07.2006 IV First Version
 * 						20.07.2006 SM Comments checked and corrected.
 * 									  Small Refactoring
 * @version			$Date: 2008-12-12 13:24:44 +0100 (Fri, 12 Dec 2008) $, $Author: treinhard $, $Revision: 3961 $
 */
public class JoinRelationLabelEditPart extends ConnectionLabelEditPart {

	/**
	 * The connection on which this label is appended.
	 */
	private JoinRelationFigure lineFigure = null;

	/**
	 * IPropertySource for this ModelElement. This property source
	 * is responsible for handling the getting and setting properties.
	 */
	private JoinRelationProperties JoinRelationProps;

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.parts.VisualizableModelElementEditPart#createFigureForModel()
	 */
	protected IFigure createFigureForModel() {

		if (getParent() == null || lineFigure != null)
			return lineFigure;

		lineFigure = ((JoinRelationEditPart) getParent()).getCastedFigure();
		
		decorateLocator(new ConnectionMidpointLocator(lineFigure));
		
		ConnectionLabel label = new ConnectionLabel(generateLabelText());
	
		setSize(label);
  		  	
		return label;
	}
	
	/**
	 * Adds this editpart to the propertyListener of the transition.
	 * @see ch.unizh.ifi.adora.parts.ConnectionLabelEditPart#activate()
	 */
	public void activate() {
		if (!isActive()) {
			JoinRelation JoinRelation =
					(JoinRelation) ((JoinRelationEditPart) 
						getParent()).getModel();
			JoinRelation.addPropertyChangeListener(this);
			super.activate();
		}
	}

	/**
	 * Removes this editpart from the propertyListener of the transition.
	 * @see ch.unizh.ifi.adora.parts.ConnectionLabelEditPart#deactivate()
	 */
	public void deactivate() {
		if (isActive()) {
			JoinRelation JoinRelation =
					(JoinRelation) ((JoinRelationEditPart) 
							getParent()).getModel();
			JoinRelation.removePropertyChangeListener(this);
			super.deactivate();
		}
	}
	
	 /* (non-Javadoc)
	 * 
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent event) {
		String property = event.getPropertyName();
		if (JoinRelation.WEAVING_PRIORITY_PROP.equals(property) || 
				JoinRelation.DECISION_SET_PROP.equals(property) ||
				JoinRelation.WEAVING_TYPE_PROP.equals(property)) {
			
			INodeProperties f = (INodeProperties) getFigure();
			
			f.setText(generateLabelText());
			refresh();
		} else if (JoinRelation.DECISION_ABSTRACTED.equals(property)) {
			INodeProperties f = (INodeProperties) getFigure();
			String s = ((JoinRelation)getParent().getModel()).getAbstractedDecNrs();
			f.setText(s);
		} else			
			super.propertyChange(event);
	}
	

	/**
	 * Generates the displayed label text. This method is called each
	 * time the label property changes.
	 * 
	 * @return The text which will be displayed on the Join Relationship label.
	 */
	private String generateLabelText() {

		String labelName = "";

		if (getParent() == null) {
			return labelName;
		}

		JoinRelation con = (JoinRelation) getParent().getModel();

		// Display the DecisionNr if a Decision is associated with the JR.
		if (con.getDecision() != null) {
			labelName += con.getDecisionNr()+" ";
		}

		/*
		 * Stores the priority values shown in the combo box.
		 */
		String[] priorityValues = { "none", "1", "2", "3", "4", "5", "6", "7",
				"8", "9", "10" };

		/*
		 * Stores the weaving type values shown in the combo box. 
		 */
		String[] weavingTypeValues = { "none", "before", "instead", "after" };

		if (con.getWeavingType() != JoinRelation.noType) {
			labelName += weavingTypeValues[con.getWeavingType()] + " ";
		}
		if (con.getPriority() != JoinRelation.noPriority) {
			labelName += priorityValues[con.getPriority()];
		}

		return labelName;
	}
	
	/**
	 * Returns the IPropertySource for this Modelelement.
	 * 
	 * @param adapter The class for which an object should be returned.
	 * @return Returns the object for the given adapter.
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getAdapter(java.lang.Class)
	 */	
	public Object getAdapter(Class adapter) {
		if (adapter == IPropertySource.class) {
			if (JoinRelationProps == null) {
				JoinRelationProps = new JoinRelationProperties(
						(JoinRelation) getParent().getModel());
			}
			return JoinRelationProps;
		}
		return null;
	}
}