package ch.unizh.ifi.adora.parts;

import java.beans.PropertyChangeEvent;

import org.eclipse.draw2d.ConnectionEndpointLocator;
import org.eclipse.draw2d.IFigure;
import org.eclipse.ui.views.properties.IPropertySource;

import ch.unizh.ifi.adora.model.representation.Association;
import ch.unizh.ifi.adora.model.representation.Role;
import ch.unizh.ifi.adora.parts.callback.ConnectionLabelConstraintCallBackImpl;
import ch.unizh.ifi.adora.properties.RoleProperties;
import ch.unizh.ifi.adora.view.figures.AssociationFigure;
import ch.unizh.ifi.adora.view.figures.label.associationlabel.AssociationLabel;

/** 
 * Editpart used to direct edit, resize and relocate Rolelabels 
 * associated with a Association.
 * 
 * @author				vigan
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			Feb 20, 2006 SM First Version
 * 						Jul 20, 2006 SM Comments added, checked and corrected.
 * 						Aug 15, 2006 JS getCastedModel() overridden and used
 * 										where applicable
 * 						25.09.2007 CR changes to allow resizing of Label
 * @version			$Date: 2009-05-06 10:51:39 +0200 (Wed, 06 May 2009) $, $Author: treinhard $, $Revision: 4182 $
 *
 */
public class RoleEditPart extends DirectEditableLabelEditPart{

    /**
     * IPropertySource for this ModelElement. This IPropertySource
     * has to handle the setting and getting of property values in 
     * the corresponding model element.
     */
    private RoleProperties roleProps;
    
    /**
     * Label, which contains an arrow, a cardinality and 
     * a textual desciption.
     */
    private AssociationLabel label = null;
   
    /**
     * Creates the Label Figure, specifies if the Label belongs to 
     * the source or target object and set the initial cardinality.
     * @return The figure which is associated with this 
     *          edit part.
     */
    protected IFigure createFigureForModel() {
	
    	Role role = getCastedModel();
		
    	if (getParent() == null || label != null) {
			return label;
    	}
            
        AssociationFigure lineFigure = ((AssociationEditPart)getParent()).
        	getCastedFigure();
 
        
        decorateLocator(new ConnectionEndpointLocator(lineFigure, role.isTo()));
        ConnectionLabelConstraintCallBackImpl callback=new ConnectionLabelConstraintCallBackImpl(getCastedModel());
		//ConnectionLabel label = new ConnectionLabel(callback);
        
       	label = new AssociationLabel(lineFigure,
		  					role.getMinCard(), 
			  				role.getMaxCard(), 
			  				role.getName(), 
			  				role.isTo(),callback);
       	
  		setSize(label);
  		return label;
	}

	/**
	 * Returns the casted model element which is associated with
	 * this edit part.
	 * @return Returns the casted model element.
	 */
	protected Role getCastedModel() {
		return (Role) getModel();
	}

    /**
     * Adds this editpart to the propertyListener of the association.
     * @see ch.unizh.ifi.adora.parts.VisualizableModelElementEditPart#activate()
     */
	public void activate() {
		if (!isActive()) {
			Association association =  (Association)
				((AssociationEditPart)getParent()).getModel();
			association.addPropertyChangeListener(this);
			super.activate();
		}
	}

	/**
	 * Removes this editpart from the propertyListener of 
	 * the association.
	 * 
	 * @see ch.unizh.ifi.adora.parts.VisualizableModelElementEditPart#deactivate()
	 */
	public void deactivate() {
		if (isActive()) {
			Association association =  (Association)
				((AssociationEditPart)getParent()).getModel();
			association.removePropertyChangeListener(this);
			super.deactivate();
		}
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent event) {
		String prop = event.getPropertyName();
		
		if (Role.MINCARD_PROP.equals(prop)) {
			((AssociationLabel)getFigure()).setMinCard(
					getCastedModel().getMinCard());
			
             refresh();
		} else if (Role.MAXCARD_PROP.equals(prop)) {
         	((AssociationLabel)getFigure()).setMaxCard(
         			getCastedModel().getMaxCard());
         	
            refresh();
        } else{
            super.propertyChange(event);
        }
	}
    
	/**
	 * Returnes the IPropertySource for this Modelelement.
	 */
    public Object getAdapter(Class adapter) {
         if (adapter == IPropertySource.class) {
            if (roleProps == null) {
                roleProps = new RoleProperties(getCastedModel());
            }
            return roleProps;
         }
         return null;
    }
 }