package ch.unizh.ifi.adora.parts;

import org.eclipse.draw2d.IFigure;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.editpolicies.ConnectionEndpointEditPolicy;
import org.eclipse.ui.views.properties.IPropertySource;

import ch.unizh.ifi.adora.model.representation.CommentableElement;
import ch.unizh.ifi.adora.model.representation.ScenarioConnection;
import ch.unizh.ifi.adora.parts.policies.connections.ScenarioConnectionEditPolicy;
import ch.unizh.ifi.adora.properties.CommentableElementProperties;
import ch.unizh.ifi.adora.properties.ConnectionProperties;
import ch.unizh.ifi.adora.view.figures.ScenarioConnectionFigure;

/**
 * 
 * The EditPart for a ScenarioConnection.   
 *
 * @author				smeier
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			27.09.2005 SM First Version
 * 						20.09.2006 SM Comments added, checked and corrected.
 * @version			$Date: 2009-05-06 10:51:39 +0200 (Wed, 06 May 2009) $, $Author: treinhard $, $Revision: 4182 $
 *
 */
public class ScenarioConnectionEditPart extends ConnectionEditPart{
	
	/**
	 * IPropertySource for this ModelElement.
	 */
	private CommentableElementProperties commentProps; 
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		super.createEditPolicies();
		// Selection handle edit policy.
		// Makes the connection show a feedback, when selected by the user.
		installEditPolicy(EditPolicy.CONNECTION_ENDPOINTS_ROLE,
				new ConnectionEndpointEditPolicy());
		// Allows the removal of the connection model element
		installEditPolicy(EditPolicy.CONNECTION_ROLE, new ScenarioConnectionEditPolicy(this));
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	protected IFigure createFigure() {
		ScenarioConnectionFigure connection = new ScenarioConnectionFigure();
        connection.setComment(getCastedModel().getComment());
		return connection;
	}

	/**
	 * Returns the casted model which corresponds to this
	 * edit part.
	 */
	public ScenarioConnection getCastedModel() {
		return (ScenarioConnection) getModel();
	}
	
	/**
	 * Returns the IPropertySource for this Modelelement.
	 * 
	 * @param adapter The class for which an object should be returned.
	 * @return Returns the object for the given adapter.
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getAdapter(java.lang.Class)
	 */	
	public Object getAdapter(Class adapter) {
		if (adapter == IPropertySource.class) {
			if (commentProps == null) {
				commentProps = new ConnectionProperties(
						(CommentableElement) getCastedModel());
			}
			return commentProps;
		}
		return null;
	}
}