package ch.unizh.ifi.adora.parts;

import java.beans.PropertyChangeEvent;
import java.util.Iterator;

import org.eclipse.draw2d.ConnectionAnchor;

import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.StartStateContainer;
import ch.unizh.ifi.adora.view.anchors.ContainerNodeClickPointAnchor;
import ch.unizh.ifi.adora.view.figures.IZoomableFigure;

/**
 * 
 * The EditPart of nodes that can be a start state.
 *
 * @author				Ivo Vigan
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			24.02.2006 IV First Version
 * 						14.08.2006 JS Comments added
 * 						15.08.2006 JS getCastedModel() added, unnecessary cast
 * 									  removed
 * @version			$Date: 2006-08-15 17:23:44 +0200 (Tue, 15 Aug 2006) $, $Author: schlaefli $, $Revision: 2064 $
 *
 */
public abstract class StartStateEditPart  extends ZoomableNodeContainerEditPart{
  
    /*
     *  (non-Javadoc)
     * @see ch.unizh.ifi.adora.parts.NodeEditPart#createConnectionAnchor()
     */
    protected ConnectionAnchor createConnectionAnchor(){
        // gets this.getFigure() and sets it as 
        // figure for which the anchor has to be computed
        return new ContainerNodeClickPointAnchor(getFigure());
    }
    
    /**
     * Returns the correspondent casted model object.
     * 
     * @pre       true
     * @post      return != null
     * @return Returns the casted model object.
     */
    protected abstract StartStateContainer getCastedModel();
    
    /*
     * (non-Javadoc)
     * 
     * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
     */
    public void propertyChange(PropertyChangeEvent evt) {
        String prop = evt.getPropertyName();
        if(StartStateContainer.STARTSTATE_PROP.equals(prop)){
            IZoomableFigure f = (IZoomableFigure)getFigure();
            f.setStartState(getCastedModel().isStartState());
            
            f.relayout();
            f.repaint();
            refresh();
            refreshAllConnections();
            getParent().refresh();
            
            //update connections, because of new anchor
            Iterator iter = getCastedModel().getConnectionsIterator();
            Connection con;
            while(iter.hasNext()){
                con = (Connection)iter.next();
                con.setVisible(con.isVisible());
            }
        }
        else{
            super.propertyChange(evt);
        }
    }
}
