package ch.unizh.ifi.adora.parts;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.LayoutAnimator;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.ui.views.properties.IPropertySource;

import ch.unizh.ifi.adora.model.representation.State;
import ch.unizh.ifi.adora.parts.directedit.NodeCellEditorLocator;
import ch.unizh.ifi.adora.parts.directedit.NodeEditManager;
import ch.unizh.ifi.adora.parts.policies.containers.StateContainerPolicy;
import ch.unizh.ifi.adora.parts.policies.nodes.StateNodeEditPolicy;
import ch.unizh.ifi.adora.properties.StartStateContainerProperties;
import ch.unizh.ifi.adora.properties.ZoomableNodeContainerProperties;
import ch.unizh.ifi.adora.view.figures.StateFigure;

/**
 * Represents the edit part of an Adora state.
 *      
 * @author				smeier
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			Sep 29, 2005 SM First Version
 *						Dec  5, 2005 SM Comments, using specific XY layout policy.
 *						Jul 20, 2006 SM Comments reformatting, checking and correcting.
 * 						Aug 15, 2006 JS Renamed getCastedObjectModel() to getCastedModel(), 
 * 										unnecessary casts removed
 * @version			$Date: 2008-11-25 14:32:54 +0100 (Tue, 25 Nov 2008) $, $Author: treinhard $, $Revision: 3896 $
 * @responsibilities	Represents the edit part of an Adora state.	
 *
 */
public class StateEditPart extends StartStateEditPart implements IAdaptable {

	/**
	 * IPropertySource for this ModelElement. This 
	 * IPropertySource is responsible for the setting and 
	 * getting of property values of the corresponding model
	 * element. 
	 */
	private ZoomableNodeContainerProperties zoomProps;
	
	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.parts.NodeEditPart#createFigureForModel()
	 */
	protected IFigure createFigureForModel() {
        State model = getCastedModel();
        
        StateFigure p = new StateFigure(model.isStartState(),model.isPartial());
		p.setText(model.getName());
		p.setComment(model.getComment());
		p.setZoomed(getCastedModel().isZoomedIn());
		p.addLayoutListener(LayoutAnimator.getDefault());
		
		return p;
	}

	/**
	 * Returns the corresponding casted model element for
	 * this edit part.
	 */
    protected State getCastedModel() {
		return (State) getModel();
	}
	
    /**
	 * Performs the direct editing by instantiating a
	 * new node edit manager.
	 */
	private void performDirectEdit() {
		if (manager == null)
			manager = new NodeEditManager(this, new NodeCellEditorLocator(
					((StateFigure) getFigure()).getNameLabel()));
        manager.setLabel(((StateFigure)getFigure()).getNameLabel());
		manager.show();
	}

	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.gef.EditPart#performRequest(org.eclipse.gef.Request)
	 */
	public void performRequest(Request request) {
		if (request.getType() == RequestConstants.REQ_DIRECT_EDIT)
			performDirectEdit();
	}
    
    /*
     *  (non-Javadoc)
     * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
     */
    protected void createEditPolicies() {
        super.createEditPolicies();

        installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE,
                new StateNodeEditPolicy());
        // allow removal of the associated model element
        installEditPolicy(EditPolicy.LAYOUT_ROLE,
                new StateContainerPolicy());

    }
    
	/**
	 * Returns the IPropertySource for this Modelelement.
	 * 
	 * @param adapter The class for which an object should be returned.
	 * @return Returns the object for the given adapter.
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getAdapter(java.lang.Class)
	 */	
	public Object getAdapter(Class adapter) {
		 if (adapter == IPropertySource.class) {
			if (zoomProps == null) {
				zoomProps = new StartStateContainerProperties(getCastedModel());
			}
			return zoomProps;
		 }
		 
		 return super.getAdapter(adapter);
		 
	}
}