package ch.unizh.ifi.adora.parts;

import org.eclipse.draw2d.IFigure;
import org.eclipse.ui.views.properties.IPropertySource;

import ch.unizh.ifi.adora.model.representation.Transition;
import ch.unizh.ifi.adora.model.representation.TransitionDescription;
import ch.unizh.ifi.adora.parts.callback.ConnectionLabelConstraintCallBackImpl;
import ch.unizh.ifi.adora.properties.TransitionDescriptionProperties;
import ch.unizh.ifi.adora.view.figures.TransitionFigure;
import ch.unizh.ifi.adora.view.figures.label.ConnectionLabel;
import ch.unizh.ifi.adora.view.figures.label.algorithm.placing.ConnectionMidpointLocator;

/** 
 * Editpart used to direct edit, 
 * resize and relocate Rolelabels associated with a Transition.
 * 
 * @author				vigan
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			Feb 20, 2006 SM First Version
 * 						Aug 14, 2006 JS Use of getCastedModel instead of casts
 * 						Aug 15, 2006 JS getCastedModel() overriden
 * 						21.09.2007 CR changes to allow setting size according to length of text
 * @version			$Date: 2007-09-25 16:03:24 +0200 (Tue, 25 Sep 2007) $, $Author: cramer $, $Revision: 3009 $
 */
public class TransitionLabelEditPart extends DirectEditableLabelEditPart {

	/**
	 * IPropertySource for this ModelElement. This 
	 * IPropertySource is responsible for the setting and 
	 * getting of property values of the corresponding model
	 * element. 
	 */
	private TransitionDescriptionProperties transitionProps;

	/**
	 * The connection on which this label is appended.
	 */
	private TransitionFigure lineFigure = null;

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.parts.VisualizableModelElementEditPart#createFigureForModel()
	 */
	protected IFigure createFigureForModel() {

		if (getParent() == null || lineFigure != null){
			return lineFigure;
		}

		lineFigure = ((TransitionEditPart) getParent()).getCastedFigure();
		
		decorateLocator(new ConnectionMidpointLocator(lineFigure));
		ConnectionLabelConstraintCallBackImpl callback=new ConnectionLabelConstraintCallBackImpl(getCastedModel());
		ConnectionLabel label = new ConnectionLabel(callback);
		setSize(label);
		label.setInitialText(getCastedModel().getName());
		return label;
	}
	
	/**
	 * Returns the casted model element which is associated with
	 * this edit part.
	 * @return Returns the casted model element.
	 */
	protected TransitionDescription getCastedModel() {
		return (TransitionDescription) getModel();
	}

	/**
	 * Adds this editpart to the propertyListener of the transition.
	 */
	public void activate() {
		if (!isActive()) {
			Transition transition = ((TransitionEditPart) getParent()).getCastedModel();
			transition.addPropertyChangeListener(this);
			super.activate();
		}
	}

	/**
	 * Removes this editpart from the propertyListener of the transition.
	 */
	public void deactivate() {
		if (isActive()) {
			Transition transition = ((TransitionEditPart) getParent()).getCastedModel();
			transition.removePropertyChangeListener(this);
			super.deactivate();
		}
	}

	/**
	 * Returns the IPropertySource for this Modelelement.
	 * 
	 * @param adapter The class for which an object should be returned.
	 * @return Returns the object for the given adapter.
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getAdapter(java.lang.Class)
	 */	
	public Object getAdapter(Class adapter) {
		if (adapter == IPropertySource.class) {
			if (transitionProps == null) {
				transitionProps = new TransitionDescriptionProperties(getCastedModel());
			}
			return transitionProps;
		}
		return null;
	}
}