package ch.unizh.ifi.adora.parts;

import java.beans.PropertyChangeEvent;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.IFigure;
import org.eclipse.gef.EditPart;

import ch.unizh.ifi.adora.model.properties.ConnectableElementPropNames;
import ch.unizh.ifi.adora.model.properties.VisualModElemPropNames;
import ch.unizh.ifi.adora.model.representation.VariationPointConnection;
import ch.unizh.ifi.adora.model.representation.VariationPointConnectionLabel;
import ch.unizh.ifi.adora.view.figures.VariationPointConnectionFigure;


/**
 * This is the EditPart for a {@link VariationPointConnection}. 
 * 
 * @author				Ivo Vigan
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			19.07.2006 IV First Version
 * 						20.07.2006 SM Comments reformatting, checking and correcting.
 * 						14.08.2006 JS Removed unnecessary cast
 * @version			$Date: 2009-05-06 10:51:39 +0200 (Wed, 06 May 2009) $, $Author: treinhard $, $Revision: 4182 $
 */
public class VariationPointConnectionEditPart extends LabeledConnectionEditPart {

	
	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.gef.EditPart#activate()
	 */
	public void activate() {
		if (!isActive()) {
			VariationPointConnectionLabel child = ((VariationPointConnectionLabel) ((VariationPointConnection)getModel()).getLabel());
			child.addPropertyChangeListener(this);
			super.activate();
		}
	}

	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.gef.EditPart#deactivate()
	 */
	public void deactivate() {
		if (isActive()) {
			VariationPointConnectionLabel child = ((VariationPointConnectionLabel) ((VariationPointConnection)getModel()).getLabel());
			child.removePropertyChangeListener(this);
			super.deactivate();
		}
	}
	
	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createChild(java.lang.Object)
	 */
	public EditPart createChild(Object model) {
		VariationPointConnectionLabelEditPart part = 
				new VariationPointConnectionLabelEditPart();
		part.setModel(model);
		return part;
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractConnectionEditPart#createFigure()
	 */
    @Override
    protected IFigure createFigure() {
       VariationPointConnectionFigure  connection = new VariationPointConnectionFigure();
        
        return connection;
    }
    
    
    /*
     *  (non-Javadoc)
     * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
     */
	public void propertyChange(PropertyChangeEvent event) {
		String property = event.getPropertyName();
		if (ConnectableElementPropNames.SOURCE_CONNECTIONS_PROP.equals(property)) {
            getSource().refresh();
        } else if (ConnectableElementPropNames.TARGET_CONNECTIONS_PROP.equals(property)) {
        	getTarget().refresh();
        } else if (VisualModElemPropNames.NAME_PROP.equals(property)) {
        	refresh();
        }
		super.propertyChange(event);
	}
	
	/**
	 * Returns the {@link VariationPointConnectionLabel} if its content is unequal the default content.
	 * @return Returns a {@link java.util.List} containing the model children of this
	 *          Join Relationship.
	 * @see org.eclipse.gef.editparts.AbstractEditPart#getModelChildren()
	 */
	public List<VariationPointConnectionLabel> getModelChildren() {
		List<VariationPointConnectionLabel> children = new ArrayList<VariationPointConnectionLabel>();
		VariationPointConnectionLabel lbl = ((VariationPointConnectionLabel) ((VariationPointConnection)getModel()).getLabel());
		
//		if (!lbl.getName().equals(""))
			children.add(lbl);
		
		return children;
	}
}