package ch.unizh.ifi.adora.parts;

import java.beans.PropertyChangeEvent;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.gef.CompoundSnapToHelper;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.SnapToGeometry;
import org.eclipse.gef.SnapToGrid;
import org.eclipse.gef.SnapToGuides;
import org.eclipse.gef.SnapToHelper;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.SnapFeedbackPolicy;
import org.eclipse.gef.rulers.RulerProvider;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PlatformUI;

import ch.unizh.ifi.adora.model.commands.SetBorderDimension;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.INodeContainer;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.Transition;
import ch.unizh.ifi.adora.model.representation.VisualizableModelElement;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.parts.directedit.NodeEditManager;
import ch.unizh.ifi.adora.parts.policies.AdoraContainerEditPolicy;
import ch.unizh.ifi.adora.parts.policies.FisheyeZoomEditPolicy;
import ch.unizh.ifi.adora.parts.policies.LabelDirectEditPolicy;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;
import ch.unizh.ifi.adora.utils.ModelElementVisibilityFilter;
import ch.unizh.ifi.adora.utils.filteredlist.FilteredArrayList;
import ch.unizh.ifi.adora.view.figures.IZoomableFigure;

/**
 * Abstract class for all zoomable EditParts.
 *      
 * @author				Ivo Vigan
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 * 						University of Zurich<br>
 * @history			24.03.2006 SM First Version
 * 						20.07.2006 SM Comments reformatted, added and checked.
 * 						15.08.2006 JS getCastedObjectModel() renamed to getCastedModel(),
 * 									  unnecessary casts removed
 * @version			$Date: 2008-04-29 11:08:17 +0200 (Tue, 29 Apr 2008) $, $Author: treinhard $, $Revision: 3625 $
 */
public abstract class ZoomableNodeContainerEditPart extends NodeEditPart implements ITransitionGraph{
      
    /**
     * Handles the direct editing capabilites for a node edit part.
     */
    protected NodeEditManager manager;
  
    /**
     * Used for iteration of the IGraph.
     */
    private boolean visited = false;
    
    /*
     * (non-Javadoc)
     * @see ch.unizh.ifi.adora.parts.NodeEditPart#activate()
     */
    public void activate() {
        if (!isActive()) {
            super.activate();
            ((IZoomableFigure)getFigure()).addPropertyChangeListener(this);
            
        }
    }
  
    /**
     * Removes the PropertyChangeListener from the LabelBorder.
     * 
     * @see ch.unizh.ifi.adora.parts.NodeEditPart#deactivate()
     */
    public void deactivate() {
        if (isActive()) {
            super.deactivate();
            ((IZoomableFigure)getFigure()).removePropertyChangeListener(this);
        }
    }   

    /*
     * (non-Javadoc)
     * 
     * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
     */
    public void propertyChange(PropertyChangeEvent evt) {
        String prop = evt.getPropertyName();
        IZoomableFigure zf = (IZoomableFigure)getFigure();
        if (INodeContainer.CHILD_ADDED_PROP.equals(prop)
                || INodeContainer.CHILD_REMOVED_PROP.equals(prop)){
            refreshChildren();
            refresh();
            zf.repaint();
        } else if (IZoomableFigure.INSETS_CHANGED_PROP.equals(prop)) {
            
            executeCommandOnStack(new SetBorderDimension(
            		getCastedModel(), 
                    (Insets)evt.getNewValue()));

            refresh();
            zf.repaint();
            zf.layout();
        } else if(ZoomableNodeContainer.ISPARTIAL_PROP.equals(prop)){         
            zf.setPartial(getCastedModel().isPartial());
            zf.relayout();
            zf.repaint();
        } else if (Node.NAME_PROP.equals(prop)) {
           zf.setText(getCastedModel().getName());
           zf.relayout();
           zf.repaint();
        } else if (VisualizableModelElement.VISIBILITY_PROP.equals(prop)) {

			zf.setCalculatedPartial(getCastedModel().isCalculatedPartial());
			super.propertyChange(evt);
           
        } else{
            super.propertyChange(evt);
        } 
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.gef.editparts.AbstractEditPart#getModelChildren()
     */
    protected List getModelChildren() {
        FilteredArrayList list = new FilteredArrayList((ArrayList)getCastedModel().getChildren());
        list.setFilter(new ModelElementVisibilityFilter());
        return list;
    }
    
    /*
     *  (non-Javadoc)
     * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
     */
    protected void createEditPolicies() {
        super.createEditPolicies();
        installEditPolicy(EditPolicy.CONTAINER_ROLE,
                new AdoraContainerEditPolicy());
        installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE,
                new LabelDirectEditPolicy());
        installEditPolicy(FisheyeZoomEditPolicy.FISHEYE_ZOOM_ROLE,
                new FisheyeZoomEditPolicy());
        
        installEditPolicy("Snap Feedback", new SnapFeedbackPolicy());
    }

    
    /**
     * Returns the correspondent casted model object.
     * 
     * @pre       true
     * @post      return != null
     * @return Returns the casted model object.
     */
    protected abstract ZoomableNodeContainer getCastedModel();
        
    /**
     * Creates a connection anchor the the figures.
     * @return Returns a connection anchor.
     */
    protected abstract ConnectionAnchor createConnectionAnchor();

    /*
     * (non-Javadoc)
     * @see ch.unizh.ifi.adora.parts.VisualizableModelElementEditPart#createFigureForModel()
     */
    @Override
    protected abstract IFigure createFigureForModel();
    
    /*
     *  (non-Javadoc)
     * @see ch.unizh.ifi.adora.parts.IGraph#resetVisited()
     */
    public void resetVisited(){
    	if (!visited)
    		return;
    	visited = false;
    	ArrayList<ConnectionEditPart> list = new ArrayList(getSourceConnections());
    	Iterator conIter = getTargetConnections().iterator();
    	while(conIter.hasNext()){
    		list.add((ConnectionEditPart)conIter.next());
    	}
		Iterator iter = list.iterator();
		ConnectionEditPart connectionEP;
		NodeEditPart otherNodePart;
		while (iter.hasNext()){
			connectionEP = (ConnectionEditPart)iter.next();
			if (!(connectionEP instanceof TransitionEditPart))
				continue;
			otherNodePart = connectionEP.getSource() == this ? 
						(NodeEditPart)connectionEP.getTarget() :
						(NodeEditPart)connectionEP.getSource();
						
			((ITransitionGraph)otherNodePart).resetVisited();
		}
    }
	
    /*
     *  (non-Javadoc)
     * @see ch.unizh.ifi.adora.parts.IGraph#depthFirstSearch()
     */
    public boolean depthFirstSearch() {
		
		Node otherNode;

		boolean movable = true;
		
		if (visited)
			return getSelected() > 0;
	
		visited = true;
		
		if (getSelected() == 0)
			return false;
		
		//check for hidden nodes --> no moving allowed
		for (Connection c : getCastedModel().getConnections()) {
			if (!(c instanceof Transition)) 
				continue;
		
			otherNode = c.getSource() == getModel() ? (Node)c.getTarget() :
				(Node)c.getSource();
			
			if(!otherNode.isVisible()){
				return false;
			}
		}
		
		ArrayList<ConnectionEditPart> list = new ArrayList(getSourceConnections());
    	Iterator conIter = getTargetConnections().iterator();
    	while(conIter.hasNext()){
    		list.add((ConnectionEditPart) conIter.next());
    	}
		Iterator iter = list.iterator();
		ConnectionEditPart connectionEP;
		NodeEditPart otherNodePart;
		while (iter.hasNext()){
			connectionEP = (ConnectionEditPart)iter.next();
			if (!(connectionEP instanceof TransitionEditPart))
				continue;
			otherNodePart = connectionEP.getSource() == this ? 
						(NodeEditPart)connectionEP.getTarget() :
						(NodeEditPart)connectionEP.getSource();
						
			movable = ((ITransitionGraph)otherNodePart).depthFirstSearch();
			if (!movable){
				break;
			}
		}
		return movable;
	}
    
    /*
     * (non-Javadoc)
     * @see ch.unizh.ifi.adora.parts.IGraph#isVisited()
     */
	public boolean wasGraphNodeChecked() {
		return visited;
	}
	
	 /**
     * Executes a given {@link org.eclipse.gef.commands.Command} on the {@link org.eclipse.gef.commands.CommandStack} 
     * of the editor.
     *
     * @param cmd The {@link org.eclipse.gef.commands.Command} which gets executed on the 
     * {@link org.eclipse.gef.commands.CommandStack}.
     */
    protected void executeCommandOnStack(Command cmd){
    	IWorkbenchPage page = PlatformUI.getWorkbench()
		.getActiveWorkbenchWindow().getActivePage();
		
    	try {
			IEditorPart editor = page.getActiveEditor();
			if (editor instanceof AdoraModelEditor) {
				((AdoraModelEditor)editor).getCommandStack().execute(cmd);
			}
		} catch (Exception ex) {
			ex.printStackTrace();
		}
    }
    
    
	/**
	 * Overriden to add the Snap (snap to grid, snap to geometry) helpers.
	 * 
	 */
	public Object getAdapter(Class adapter) {
		
		// Snap to grid or layout
		if (adapter == SnapToHelper.class) {
			List<SnapToHelper> snapStrategies = new ArrayList<SnapToHelper>();
			Boolean val = (Boolean)getViewer().getProperty(RulerProvider.PROPERTY_RULER_VISIBILITY);
			if (val != null && val.booleanValue())
				snapStrategies.add(new SnapToGuides(this));
			val = (Boolean)getViewer().getProperty(SnapToGeometry.PROPERTY_SNAP_ENABLED);
			if (val != null && val.booleanValue())
				snapStrategies.add(new SnapToGeometry(this));
			val = (Boolean)getViewer().getProperty(SnapToGrid.PROPERTY_GRID_ENABLED);
			if (val != null && val.booleanValue())
				snapStrategies.add(new SnapToGrid(this));
			
			if (snapStrategies.size() == 0)
				return null;
			if (snapStrategies.size() == 1)
				return snapStrategies.get(0);

			SnapToHelper ss[] = new SnapToHelper[snapStrategies.size()];
			for (int i = 0; i < snapStrategies.size(); i++)
				ss[i] = (SnapToHelper)snapStrategies.get(i);
			return new CompoundSnapToHelper(ss);
		}
		
		return super.getAdapter(adapter);
	}
    
}