package ch.unizh.ifi.adora.parts.directedit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.tools.CellEditorLocator;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Text;

import ch.unizh.ifi.adora.view.figures.IZoomableFigure;

/**
* Determines the location of the direct editing component on a Node divers to a ZoomableNodeContainer.
*
* @author				Ivo Vigan
* @copyright			Requirements Engineering Research Group<br>
*						Department of Informatics<br>
*						University of Zurich<br>
* @history				16.03.2006 SM First Version
* 						22.07.2006 SM Comments reformatting.
* 						14.08.2006 JS Added some comments.
* @version				$Date: 2006-08-15 14:11:37 +0200 (Tue, 15 Aug 2006) $, $Author: schlaefli $, $Revision: 2061 $
*/
public class NodeCellEditorLocator implements CellEditorLocator{

    /**
     * Node on which the direct editing is performed.
     */
	private IFigure label;

    /**
     * Instantiates a new NodeCellEditorLocator.
     * 
     * @param node Node on which the Direct Editing Component appears.
     */
    public NodeCellEditorLocator(IFigure node) {
		setLabel(node);
	}

    /*
     *  (non-Javadoc)
     * @see org.eclipse.gef.tools.CellEditorLocator#relocate(org.eclipse.jface.viewers.CellEditor)
     */
	public void relocate(CellEditor celleditor) {
        Text text = (Text) celleditor.getControl();
        
        Rectangle rect = label.getBounds().getCopy();
        label.translateToAbsolute(rect);

        int x = rect.x;
        int y = rect.y;
        
        int width = rect.width;
        int height = text.computeSize(width, SWT.DEFAULT).y;
        
        int labelHeight = rect.height;
        // if there's a border, shrink the new bounds by the insets of the label:
        if(label.getBorder() != null) {
            Insets insets = label.getBorder().getInsets(label);
            //If label situated in IZoomableFigure, the parentborder has to be regarded too.
            if(label.getParent() instanceof IZoomableFigure){
                x += label.getParent().getInsets().left;
                width -= label.getParent().getInsets().left + label.getParent().getInsets().right;
                
            }
            x += insets.left;
            y += insets.top;
            width -= insets.left + insets.right;
            labelHeight -= insets.top + insets.bottom;
        }
        // center the new bounds vertically
        if (label.getBounds().height > height) {
            y += (labelHeight - height) / 2;
        }

        text.setBounds(x, y, width, height);

    }

    /*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.parts.directedit.AdoraCellEditorLocator#getLabel()
     */
	public IFigure getLabel() {
		return label;
	}

    /*
     *  (non-Javadoc)
     * @see ch.unizh.ifi.adora.parts.directedit.AdoraCellEditorLocator#setLabel(org.eclipse.draw2d.IFigure)
     */
	public void setLabel(IFigure node) {
		this.label = node;
	}

}