package ch.unizh.ifi.adora.parts.figureinterfaces;

/**
 * Used to treat all zoomable {@link org.eclipse.draw2d.IFigure}s 
 * the same way. This generalisation is used in the 
 * {@link ch.unizh.ifi.adora.parts.NodeEditPart#propertyChange(PropertyChangeEvent)}  
 * method.
 * 
 * @author				Ivo Vigan
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				24.07.2006 IV First Version
 * 						25.07.2006 SM Comments checked, reformatted.
 * 						15.08.2006 JS Comment corrected
 * @version				$Date: 2006-08-15 17:23:44 +0200 (Tue, 15 Aug 2006) $, $Author: schlaefli $, $Revision: 2064 $
 */
public interface INodeContainerProperties extends INodeProperties {
	
	/**
	 * Stores the Property Key for a zooming property.
	 */
	public static final String ZOOMED_PROP = "Node.Zoomed";
	
	/**
	 * Sets whether this node is zoomed in (true) or zoomed out (false).
	 * @pre true
	 * @post true
	 * @param z Parameter indicating the zoom state of this node.
	 */
	void setZoomed(boolean z);
	
	/**
	 * Returns true, if this node is zomed in, otherwise false.
	 * @pre true
	 * @post true 
	 * @return Returns true in the case of a zoomed in node.
	 */
	boolean isZoomed();
	
}