package ch.unizh.ifi.adora.parts.policies;

import java.util.Iterator;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.ComponentEditPolicy;
import org.eclipse.gef.requests.GroupRequest;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.ui.PlatformUI;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.commands.NodeDelete;
import ch.unizh.ifi.adora.model.commands.RemoveNodeVisually;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.INodeContainer;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.variability.Constraint;
import ch.unizh.ifi.adora.model.representation.variability.Decision;
import ch.unizh.ifi.adora.model.representation.variability.VariationPoint;

/**
 * Policy for handling the the deletion of a sub node.
 * 
 * @author				smeier
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			Oct 10, 2005 SM First Version
 *						Dec 2,  2005 SM Handling of visual removing of the node.
 *						Jul 22, 2006 SM Removed unnecessary code.
 *										Comments reformatting.
 * @version			$Date: 2009-02-09 17:47:30 +0100 (Mon, 09 Feb 2009) $, $Author: vigan $, $Revision: 4000 $
 * @responsibilities	Provides the policy for deleting components.
 */
public class AdoraComponentEditPolicy extends ComponentEditPolicy {

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editpolicies.ComponentEditPolicy#createDeleteCommand(org.eclipse.gef.requests.GroupRequest)
	 */
	protected Command createDeleteCommand(GroupRequest deleteRequest) {

		Command tempCommand;
		
		if (!isDecisionOk(getHost().getModel()))
			return null;
		
		if (getHost().getParent().getModel() instanceof INodeContainer 
				&& getHost().getModel() instanceof Node) {
			INodeContainer parent = (INodeContainer)getHost().getParent().getModel();
			Node child = (Node)getHost().getModel();
			
            NodeDelete deleteCommand;
            deleteCommand = new NodeDelete(parent, child);    
            
			// first: resize child to delete to a small size
			RemoveNodeVisually radialRemove = new RemoveNodeVisually(child);
			// second: chain it with the delete command
			tempCommand = radialRemove.chain(deleteCommand);
			// Return the delete command 
			return tempCommand;
			
		}
		return super.createDeleteCommand(deleteRequest);
	}
	
	
	/**
	 * Check if a decision would be deleted which is contained in some constraint.
	 * 
	 * @return If the component can be deleted. 
	 */
	private boolean isDecisionOk(Object ele) {
		
		String confConstr = "";
		String decStr = "";
		
		if (!(ele instanceof ConnectableElement))
			return false;
		
		
		for (Connection con : ((ConnectableElement)ele).getConnections()) {
			if (con instanceof JoinRelation ) {
				Decision decision = ((JoinRelation)con).getDecision();
				
				if (decision != null) {
					for (VariationPoint vp : AdoraPlugin.getActiveModel().getVariationPoints()) {
						if (vp.getDecisions().contains(decision)) {
							if (!confConstr.contains("VP"+vp.getNr()+" "))
								confConstr += "VP"+vp.getNr()+" ";
							if (!decStr.contains("D"+decision.getNr()+" "))
								decStr +="D"+ decision.getNr()+" ";
						}
					}
					for (Constraint c : AdoraPlugin.getActiveModel().getConstraints()) {
						if (c.containsDecision(decision)) {
							if (!confConstr.contains("C"+c.getNr()+" "))
								confConstr += "C"+c.getNr()+" ";
							if (!decStr.contains("D"+decision.getNr()+" "))
								decStr +="D"+ decision.getNr()+" ";
						}
					}
				}
			}
		}			
		
		if (ele instanceof INodeContainer) {
			Iterator<ModelElement> itr  = ((INodeContainer)ele).getRecursiveChildrenIterator();
			while (itr.hasNext()) {
				ModelElement e = itr.next();
				if (e instanceof ConnectableElement) {
					for (Connection con : ((ConnectableElement)e).getConnections()) {
						if (con instanceof JoinRelation ) {
							Decision decision = ((JoinRelation)con).getDecision();
							
							if (decision != null) {
								for (VariationPoint vp : AdoraPlugin.getActiveModel().getVariationPoints()) {
									if (vp.getDecisions().contains(decision)) {
										if (!confConstr.contains("VP"+vp.getNr()+" "))
											confConstr += "VP"+vp.getNr()+" ";
										if (!decStr.contains("D"+decision.getNr()+" "))
											decStr +="D"+ decision.getNr()+" ";
									}
								}
								for (Constraint c : AdoraPlugin.getActiveModel().getConstraints()) {
									if (c.containsDecision(decision)) {
										if (!confConstr.contains("C"+c.getNr()+" "))
											confConstr += "C"+c.getNr()+" ";
										if (!decStr.contains("D"+decision.getNr()+" "))
											decStr +="D"+ decision.getNr()+" ";
									}
								}
							}
						}
					}
				}
			}
			if (!confConstr.equals("")) {
		//		showWarning(decStr, confConstr);
				return false;
			}
		}
		return true;
	}
	
	private void showWarning(String decStr, String conf) {
		MessageBox mb = new MessageBox(PlatformUI.getWorkbench().
				getActiveWorkbenchWindow().getShell(), 
						SWT.ICON_INFORMATION | SWT.OK);
		mb.setText("Decision Conflict");
		mb.setMessage("Decisions:"+ decStr + " appear in: "+conf);
	    mb.open();
	}
}