package ch.unizh.ifi.adora.parts.policies;

import java.util.List;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.gef.editpolicies.ContainerEditPolicy;
import org.eclipse.gef.requests.CreateRequest;
import org.eclipse.gef.requests.GroupRequest;

import ch.unizh.ifi.adora.model.commands.OrphanChild;
import ch.unizh.ifi.adora.model.commands.RemoveNodeVisually;
import ch.unizh.ifi.adora.model.representation.INodeContainer;
import ch.unizh.ifi.adora.model.representation.Node;

/**
 * This class provides the edit policy for container nodes.
 * 
 * 
 * @author 				smeier
 * @copyright 				Requirements Engineering Research Group<br>
 *            				Department of Informatics<br>
 *							University of Zurich<br>
 * @history 				Oct 10, 2005 SM First Version 
 * 							Dec 2,  2005 SM Comments and visual
 *											handling of orphaning a child node.
 *							Jul 22, 2006 SM Comments reformatted
 * @version 				$Date 2006/04/22 16:57:01 $, $Author: vigan $, $Revision: 2104 $
 * @responsibilities 		Provides the policy for deleting commands.
 * 
 */
public class AdoraContainerEditPolicy extends ContainerEditPolicy {

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editpolicies.ContainerEditPolicy#getCreateCommand(org.eclipse.gef.requests.CreateRequest)
	 */
	protected Command getCreateCommand(CreateRequest request) {
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editpolicies.ContainerEditPolicy#getOrphanChildrenCommand(org.eclipse.gef.requests.GroupRequest)
	 */
	public Command getOrphanChildrenCommand(GroupRequest request) {
		List parts = request.getEditParts();
		Command tempCommand = null;
		CompoundCommand result = new CompoundCommand(
				"Orphan all selected childs");
		for (int i = 0; i < parts.size(); i++) {
			OrphanChild orphan = new OrphanChild();
			orphan.setChild((Node) ((EditPart) parts.get(i))
					.getModel());
			orphan.setParent((INodeContainer) getHost().getModel());
			orphan.setLabel("Orphan child command");
			// first make child to orphan 
			RemoveNodeVisually radialRemove = new RemoveNodeVisually(
					(Node) ((EditPart) parts.get(i)).getModel());
			// second chain it with the orphan command
			tempCommand = radialRemove.chain(orphan);
			// add the chained command to the
			result.add(tempCommand);
		}
		return result.unwrap();
	}

}
