package ch.unizh.ifi.adora.parts.policies;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.DirectEditPolicy;
import org.eclipse.gef.requests.DirectEditRequest;

import ch.unizh.ifi.adora.model.commands.VisualizableModelElementSetName;
import ch.unizh.ifi.adora.model.representation.VisualizableModelElement;
import ch.unizh.ifi.adora.parts.VisualizableModelElementEditPart;
import ch.unizh.ifi.adora.parts.figureinterfaces.INodeProperties;

/**
 * Shows the DirectEdit feedback and creates the command to perform a 
 * "direct edit". The direct Editing capabilities are needed when the 
 * User is editing a property of an edit part directly 
 * (as opposed to in the properties view) on the canvas 
 * using a org.eclipse.jface.viewers.CellEditor. 
 * This EditPolicy is typically installed using 
 * {@link org.eclipse.gef.EditPolicy.DIRECT_EDIT_ROLE}. 
 * 
 * @author				Ivo Vigan
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				24.07.2006 IV First Version
 * 						25.07.2006 SM Comments checked, corrected.
 * @version				$Date: 2006-08-24 13:29:45 +0200 (Thu, 24 Aug 2006) $, $Author: vigan $, $Revision: 2101 $
 */
public class LabelDirectEditPolicy extends DirectEditPolicy {

	/**
	 * Returns a new {@link VisualizableModelElementSetName} Command.
	 * 
	 * @see DirectEditPolicy#getDirectEditCommand(DirectEditRequest)
	 */
	protected Command getDirectEditCommand(DirectEditRequest edit) {
		VisualizableModelElementEditPart label = (VisualizableModelElementEditPart) getHost();
		VisualizableModelElementSetName command = 
					new VisualizableModelElementSetName(
							(VisualizableModelElement) label.getModel(),
							(String) edit.getCellEditor().getValue()
					);
		return command;
	}

	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.gef.editpolicies.DirectEditPolicy#showCurrentEditValue(org.eclipse.gef.requests.DirectEditRequest)
	 */
	protected void showCurrentEditValue(DirectEditRequest request) {
		String value = (String) request.getCellEditor().getValue();
		((INodeProperties) getHostFigure()).setText(value);
		// hack to prevent async layout from placing the cell editor twice.
		getHostFigure().getUpdateManager().performUpdate();

	}
}