package ch.unizh.ifi.adora.parts.policies;


import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy;
import org.eclipse.gef.editpolicies.LayoutEditPolicy;
import org.eclipse.gef.requests.ChangeBoundsRequest;
import org.eclipse.gef.requests.CreateRequest;

import ch.unizh.ifi.adora.model.commands.SetConnectionLabelConstraint;
import ch.unizh.ifi.adora.model.commands.VisualizableModelElementSetConstraint;
import ch.unizh.ifi.adora.model.representation.ConnectionLabel;
import ch.unizh.ifi.adora.parts.geometryfactory.GeometryFactory;

/** 
* This policy handles the incomming move and resize requests
* concerning a Connection label.
* 
* @author				vigan
* @copyright			Requirements Engineering Research Group<br>
*						Department of Informatics<br>
*						University of Zurich<br>
* @history				Feb 15, 2006 IV First Version
* 						Jul 23, 2006 SM Some reformatting.
* 						Jul 25, 2006 SM Completing comments.
* 	
* @version				$Date: 2006-08-24 13:29:45 +0200 (Thu, 24 Aug 2006) $, $Author: vigan $, $Revision: 2101 $
*
*/
public class MovableLabelLayoutEditPolicy extends ConstrainedLayoutEditPolicy {

	/**
	 * Returns the layout's origin relative to the container
	 * returned by {@link LayoutEditPolicy#getLayoutContainer()}.
	 * 
	 * In case of a connection label, this is allways the origin of the canvas (0/0).
	 * 
	 * @return The canvas' origin (0/0).
	 */
	public Point getLayoutOrigin(){
		return new Point(0,0);
	}
	
	/**
	 * Return a new {@link VisualizableModelElementSetConstraint}-Command, 
	 * which is used to set the bounds of Connection labels.
	 * 
	 * @param	request		the ChangeBoundsRequest
	 * @param	child		the EditPart of the child being changed
	 * @param	constraint	the new constraint, after being {@link
	 * 						#translateToModelConstraint(Object) translated}
	 * 
	 * @return Command A {@link VisualizableModelElementSetConstraint}-Command to change the 
	 * 				constraints of the given child as specified in the  
	 * 				given <li>request</li>.
	 */
	protected Command createChangeConstraintCommand(
			ChangeBoundsRequest request, EditPart child, Object constraint) {
		
		return new SetConnectionLabelConstraint(
				(ConnectionLabel) child.getModel(),
				(ch.unizh.ifi.adora.model.geometry.Point) 
				GeometryFactory.getAdoraElement(request.getMoveDelta()),
				(ch.unizh.ifi.adora.model.geometry.Rectangle) 
						GeometryFactory.getAdoraElement(constraint));
	}
	
	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#createAddCommand(org.eclipse.gef.EditPart, java.lang.Object)
	 */
	protected Command createAddCommand(EditPart child, Object constraint) {
		return null;
	}

	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#createChangeConstraintCommand(org.eclipse.gef.EditPart, java.lang.Object)
	 */
	protected Command createChangeConstraintCommand(EditPart child, Object constraint) {
		return null;
	}
	
	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#getCreateCommand(org.eclipse.gef.requests.CreateRequest)
	 */
	protected Command getCreateCommand(CreateRequest request) {
			return null;
	}

	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#getDeleteDependantCommand(org.eclipse.gef.Request)
	 */
	protected Command getDeleteDependantCommand(Request request) {
		return null;
	}

	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#getConstraintFor(org.eclipse.draw2d.geometry.Point)
	 */
	protected Object getConstraintFor(Point point) {
		return new Rectangle(point, new Dimension(-1, -1));
	}

	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#getConstraintFor(org.eclipse.draw2d.geometry.Rectangle)
	 */
	protected Object getConstraintFor(Rectangle rect) {
		return new Rectangle(rect);
	}
}
