package ch.unizh.ifi.adora.parts.policies.connections;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.ConnectionEditPolicy;
import org.eclipse.gef.requests.GroupRequest;

import ch.unizh.ifi.adora.model.commands.AssociationDelete;
import ch.unizh.ifi.adora.model.representation.Association;
import ch.unizh.ifi.adora.parts.AssociationEditPart;

/**
 * This {@link ConnectionEditPolicy} is used to delete, 
 * an {@link ch.unizh.ifi.adora.model.representation.Association} 
 * if permitted.
 * 
 * @author			smeier
 * @copyright		Requirements Engineering Research Group<br>
 *					Department of Informatics<br>
 *					University of Zurich<br>
 * @history			27.09.2005 SM First Version 23.07.2006 SM Some reformatting
 * 					25.07.2006 SM Comment checked, corrected, reformatted.
 * @version			$Date: 2006-08-24 13:29:45 +0200 (Thu, 24 Aug 2006) $, $Author: vigan $, $Revision: 2101 $
 * 
 */
public class AssociationEditPolicy extends ConnectionEditPolicy {

	/**
     * {@link AssociationEditPart} for which this policy is used.
     */
	private AssociationEditPart associationEditPart;

	/**
     * Creates a new AssociationEditPolicy for a given
     * {@link AssociationEditPart}.
     * 
     * @param aep
     *            {@link AssociationEditPart} for which this policy is used.
     */
	public AssociationEditPolicy(AssociationEditPart aep) {
		associationEditPart = aep;
	}

	/**
	 * Creates a delete command, if the 
     * {@link ch.unizh.ifi.adora.model.representation.Association} 
     * isn't (calculated) abstracted.
     * 
	 * @param request The request for deleting a particular 
	 * {@link ch.unizh.ifi.adora.model.representation.Association}.
	 * 
	 * @return Reutrns the {@link AssociationDelete} command or null.
	 */
	protected Command getDeleteCommand(GroupRequest request) {
		if (!((Association) associationEditPart.getCastedModel())
				.isCalculatedAbstracted()) {
			return new AssociationDelete(associationEditPart.getCastedModel());
		}
		return null;
	}
}