package ch.unizh.ifi.adora.parts.policies.connections;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.ConnectionEditPolicy;
import org.eclipse.gef.requests.GroupRequest;

import ch.unizh.ifi.adora.model.commands.ConnectionDelete;
import ch.unizh.ifi.adora.parts.JoinRelationEditPart;

/**
 * This {@link ConnectionEditPolicy} is mainly used to delete 
 * an {@link ch.unizh.ifi.adora.model.representation.JoinRelation}.
 * 
 * @author				Ivo Vigan
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 * 						University of Zurich<br>
 * @history				24.07.2006 IV First Version
 * 						25.07.2006 SM Comments completed.
 * @version				$Date: 2006-08-24 13:29:45 +0200 (Thu, 24 Aug 2006) $, $Author: vigan $, $Revision: 2101 $
 */
public class JoinRelationEditPolicy extends ConnectionEditPolicy {

	/**
	 * {@link JoinRelationEditPart} for which this policy is used.
	 */
	private final JoinRelationEditPart part;
    
	/**
	 * Creates a new JoinRelationEditPolicy for a given 
     * {@link JoinRelationEditPart}.
	 * 
	 * @param tep {@link JoinRelationEditPart} for which this policy is used.
	 */
	public JoinRelationEditPolicy(JoinRelationEditPart part) {
        this.part = part;
    }
    
	/**
	 * Creates a delete command.
	 *  
	 * @param request Request for deleting an 
	 * {@link ch.unizh.ifi.adora.model.representation.JoinRelation}.
	 * 
	 * @return Returns a {@link ConnectionDelete} command or null.
	 */
	protected Command getDeleteCommand(GroupRequest request) {
        return new ConnectionDelete(part.getModel());
    }
}