package ch.unizh.ifi.adora.parts.policies.connections;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.ConnectionEditPolicy;
import org.eclipse.gef.requests.GroupRequest;

import ch.unizh.ifi.adora.model.commands.ScenarioConnectionDelete;
import ch.unizh.ifi.adora.parts.ScenarioConnectionEditPart;


/**
 * This {@link ConnectionEditPolicy} is used to delete 
 * a {@link ch.unizh.ifi.adora.model.representation.ScenarioConnection}.
 *      
 * @author				smeier
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				27.09.2005 SM First Version
 * 						23.07.2006 SM Some reformatting
 * @version				$Date: 2006-08-24 13:29:45 +0200 (Thu, 24 Aug 2006) $, $Author: vigan $, $Revision: 2101 $
 *
 */
public class ScenarioConnectionEditPolicy extends ConnectionEditPolicy {

	/**
     * {@link ScenarioConnectionEditPart} for which this policy is used.
     */
	private ScenarioConnectionEditPart scenarioConnectionEditPart;

	/**
     * Creates a new ScenarioConnectionEditPolicy for a given
     * {@link ScenarioConnectionEditPart}.
     * 
     * @param scep
     *            {@link ScenarioConnectionEditPart} for which this policy is used.
     */
	public ScenarioConnectionEditPolicy(ScenarioConnectionEditPart scep) {
		scenarioConnectionEditPart = scep;
	}
	

	/**
	 * Creates a delete command.
	 * 
	 * @param request Request for deleting an 
	 * {@link ch.unizh.ifi.adora.model.representation.ScenarioConnection}.
	 * 
	 * @return {@link ScenarioConnectionDelete} command.
	 */
	protected Command getDeleteCommand(GroupRequest request) {
		return new ScenarioConnectionDelete(scenarioConnectionEditPart.
				getCastedModel());
	}
}
