package ch.unizh.ifi.adora.parts.policies.connections;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.ConnectionEditPolicy;
import org.eclipse.gef.requests.GroupRequest;

import ch.unizh.ifi.adora.model.commands.TransitionDelete;
import ch.unizh.ifi.adora.parts.TransitionEditPart;

/**
 * This {@link ConnectionEditPolicy} is used to delete, 
 * a {@link ch.unizh.ifi.adora.model.representation.Transition}.
 *      
 * @author				smeier
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				27.09.2005 SM First Version
 * 						23.07.2006 SM Reformatting of comments.
 * @version				$Date: 2006-08-24 13:29:45 +0200 (Thu, 24 Aug 2006) $, $Author: vigan $, $Revision: 2101 $
 *
 */
public class TransitionEditPolicy extends ConnectionEditPolicy {

	/**
	 * {@link TransitionEditPart} for which this policy is used.
	 */
	private TransitionEditPart transitionEditPart;

	/**
	 * Creates a new TransitionEditPolicy for a given 
     * {@link TransitionEditPart}.
	 * 
	 * @param tep {@link TransitionEditPart} for which this policy is used.
	 */
	public TransitionEditPolicy(TransitionEditPart tep) {
		transitionEditPart = tep;
	}
	
	/**
	 * Creates a delete command.
	 *  
	 * @param request Request for deleting an 
	 * {@link ch.unizh.ifi.adora.model.representation.Transition}.
	 * 
	 * @return {@link TransitionDelete} command.
	 */
	protected Command getDeleteCommand(GroupRequest request) {
		return new TransitionDelete(transitionEditPart.getCastedModel());
	}
}