package ch.unizh.ifi.adora.parts.policies.containers;

import java.util.Iterator;

import org.eclipse.gef.EditPart;

import ch.unizh.ifi.adora.model.ScenarioHierarchyConstraints;
import ch.unizh.ifi.adora.model.representation.Actor;
import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.INodeContainer;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.ui.constrainthint.ConstraintHint;
import ch.unizh.ifi.adora.ui.constrainthint.IConstraintMessages;

/**
 * Provides the container XY layout policy for the root of an 
 * Adora model.
 *      
 * @author				smeier
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			Dec 5, 2005 SM First Version
 * @version			$Date: 2009-02-06 17:48:19 +0100 (Fri, 06 Feb 2009) $, $Author: vigan $, $Revision: 3996 $
 * @responsibilities	Container XY edit policy of a root node.
 *
 */
public class AdoraModelContainerPolicy extends ContainerPolicy {

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.parts.policy.ContainerXYLayoutEditPolicy#addCommandAllowed(ch.unizh.ifi.adora.model.INodeContainer, ch.unizh.ifi.adora.model.NodeContainer)
	 */
	protected boolean addCommandAllowed(INodeContainer host, EditPart part) {
		// Constraint: if the root node is an adora model
		// the part must be ei ther a component or an actor.
		Object model = part.getModel();

		if (model instanceof Component) {
			Iterator iter = ((Component) model).getRecursiveChildrenIterator();
			while (iter.hasNext()) {

				Node n = (Node) iter.next();

				//ScenarioConstraints
				if (n instanceof Scenario) {
					if (((Scenario) n).getScenarioParent() != null) {
						//part has outgoing scenario connection
						if (!ScenarioHierarchyConstraints
								.isContainerParentOfChildContainer(
										((Scenario) n).getScenarioParent(),
										(Component) model)) {
							
							ConstraintHint.displayConstraintHint(
									IConstraintMessages.SCENARIO_COMPONENT_HIERARCHY);
							return false;
						}
					}
				}
			}
		}
 		if (!(model instanceof Component || 
				model instanceof AspectContainer || 
				model instanceof Actor)){
 			ConstraintHint.displayConstraintHint(
					IConstraintMessages.ROOT_CHILDREN);
 			return false;
 		}
		return super.addCommandAllowed(host, part);
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.parts.policy.ContainerXYLayoutEditPolicy#createCommandAllowed(ch.unizh.ifi.adora.model.INodeContainer, ch.unizh.ifi.adora.model.NodeContainer)
	 */
	protected boolean createCommandAllowed(INodeContainer host, Node part) {
		if (!(part instanceof Component || 
				part instanceof AspectContainer || 
				part instanceof Actor)){
 			ConstraintHint.displayConstraintHint(
					IConstraintMessages.ROOT_CHILDREN);
 			return false;
		}
 		return super.createCommandAllowed(host, part);
	}
}