package ch.unizh.ifi.adora.parts.policies.containers;

import org.eclipse.gef.EditPart;

import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.INodeContainer;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.State;
import ch.unizh.ifi.adora.parts.ScenarioEditPart;
import ch.unizh.ifi.adora.parts.StateEditPart;
import ch.unizh.ifi.adora.ui.constrainthint.ConstraintHint;
import ch.unizh.ifi.adora.ui.constrainthint.IConstraintMessages;

/**
 * Provides the container XY layout policy for states in an 
 * Adora model.
 *      
 * @author				smeier
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			Dec 5, 2005  SM First Version
 * 						Jul 23, 2006 SM Reformatting comments.
 * @version			$Date: 2007-01-25 18:06:03 +0100 (Thu, 25 Jan 2007) $, $Author: smeier $, $Revision: 2403 $
 * @responsibilities	Container XY edit policy of a root node.
 */
public class StateContainerPolicy extends ZoomableNodeContainerPolicy {

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.parts.policy.ContainerXYLayoutEditPolicy#addCommandAllowed(ch.unizh.ifi.adora.model.INodeContainer, ch.unizh.ifi.adora.model.NodeContainer)
	 */
	protected boolean addCommandAllowed(INodeContainer host, EditPart part) {
		if (!(part.getModel() instanceof State)) {
			ConstraintHint.displayConstraintHint(IConstraintMessages.STATE_CHILDREN);
			return false;
		}
		return super.addCommandAllowed(host, part);
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.parts.policy.ContainerXYLayoutEditPolicy#createCommandAllowed(ch.unizh.ifi.adora.model.INodeContainer, ch.unizh.ifi.adora.model.NodeContainer)
	 */
	protected boolean createCommandAllowed(INodeContainer host, Node part) {
		if (!(part instanceof State)) {
			ConstraintHint.displayConstraintHint(IConstraintMessages.STATE_CHILDREN);
			return false;
		}
		return super.createCommandAllowed(host, part);
	}
	
	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.parts.policies.containers.ZoomableNodeContainerPolicy#hasJoinRelationConflict(ch.unizh.ifi.adora.model.representation.INodeContainer, org.eclipse.gef.EditPart)
	 */
	protected boolean hasJoinRelationConflict(INodeContainer host, EditPart part){
		
		if (part instanceof StateEditPart || part instanceof ScenarioEditPart){
			for (Connection c: ((Node)part.getModel()).getConnections()) {
				if (c instanceof JoinRelation) {
					return true;
				}
			}
		}
		return false;
	}
}