package ch.unizh.ifi.adora.parts.policies.containers;

import org.eclipse.gef.EditPart;

import ch.unizh.ifi.adora.model.representation.INodeContainer;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.ui.constrainthint.ConstraintHint;
import ch.unizh.ifi.adora.ui.constrainthint.IConstraintMessages;

/**
 * Policy used for {@link ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer}s.     
 *
 * @author		        Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             25.08.2006 IV First Version
 * @version             $Date: 2006-09-28 16:05:58 +0200 (Thu, 28 Sep 2006) $, $Author: vigan $, $Revision: 2297 $
 */
public abstract class ZoomableNodeContainerPolicy extends ContainerPolicy{

	/**
	 *  Checks if the given <i>part</i> has JoinRelationShips.
	 *  
	 * @param   host The host container to which the given part should be added.
	 * @param   part The part which should be added to the given host container.
	 * @return  Returns true, if the given <i>part</i> has no JoinRelationShips.
	 */
	protected abstract boolean hasJoinRelationConflict(INodeContainer host, EditPart part);
	
	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.parts.policies.containers.ContainerPolicy#createCommandAllowed(ch.unizh.ifi.adora.model.representation.INodeContainer, ch.unizh.ifi.adora.model.representation.Node)
	 */
	protected boolean createCommandAllowed(INodeContainer host, Node part) {
		if (!((ZoomableNodeContainer)host).isZoomedIn()){
			ConstraintHint.displayConstraintHint(IConstraintMessages.ZOOMED_CONTAINER);
			return false;
		}
		return super.createCommandAllowed(host, part);
	}
	
	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.parts.policies.containers.ContainerPolicy#addCommandAllowed(ch.unizh.ifi.adora.model.representation.INodeContainer, org.eclipse.gef.EditPart)
	 */
	protected boolean addCommandAllowed(INodeContainer host, EditPart part){
		
		if (hasJoinRelationConflict(host, part)) {
			ConstraintHint.displayConstraintHint(IConstraintMessages.CROSSCUTTING_ELEMENT);
			return false;
		}
		if (!((ZoomableNodeContainer)host).isZoomedIn()){
			ConstraintHint.displayConstraintHint(IConstraintMessages.ZOOMED_CONTAINER);
			return false;
		}
		return super.addCommandAllowed(host, part);
	}
}