package ch.unizh.ifi.adora.parts.policies.nodes;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.model.commands.AssociationCreate;
import ch.unizh.ifi.adora.model.commands.JoinRelationCreate;
import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.Association;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.model.representation.State;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.parts.policies.JoinRelationPolicyHelper;
import ch.unizh.ifi.adora.ui.constrainthint.ConstraintHint;
import ch.unizh.ifi.adora.ui.constrainthint.IConstraintMessages;

/**
 * Determines the {@link ch.unizh.ifi.adora.model.representation.Connection}-Type,  
 * allowed to be attached to an {@link ch.unizh.ifi.adora.model.representation.AspectContainer}.     
 *
 * @author				Ivo Vigan
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				02.05.2006 SM First Version<br>
 * 						22.07.2006 SM Comments reformatted.<br>
 * 						25.07.2006 SM Comments reformatted, checked.<br>
 * 						25.01.2007 PS Constraints updated (Bug #806).
 * @version			$Date: 2008-01-25 14:17:46 +0100 (Fri, 25 Jan 2008) $, $Author: schoeni $, $Revision: 3503 $
 */
public class AspectContainerNodePolicy extends ConstrainablePolicy {

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.parts.policies.nodes.ConnectableElementPolicy#connectionAllowed(ch.unizh.ifi.adora.model.representation.Node, ch.unizh.ifi.adora.model.representation.ConnectableElement, org.eclipse.gef.commands.Command)
	 */
	protected boolean connectingTargetAllowed(Node sourceNode, ConnectableElement targetComponent, Command command){
		if (command instanceof AssociationCreate) {
			if (!(sourceNode instanceof Component)) {
				ConstraintHint.displayConstraintHint(IConstraintMessages.ASSOCIATION_TO_NODES);
				return false;
			}
			if (isRecusiveChildOfAspectContainer((AspectContainer)getHost().getModel())) {
				ConstraintHint.displayConstraintHint(IConstraintMessages.ASSOCIATION_TO_INNER_ASPECT);
				return false;
			}
			return super.connectingTargetAllowed(sourceNode, targetComponent, command);
		} else if (command instanceof JoinRelationCreate) {
			if (sourceNode instanceof AspectContainer) {
				return super.connectingTargetAllowed(sourceNode, targetComponent, command);
			} else if (sourceNode instanceof State || sourceNode instanceof Component || sourceNode instanceof Scenario) {
				return JoinRelationPolicyHelper.checkChunkCompletely(sourceNode, targetComponent);
			}
		}

		return false;
	}
	
	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.parts.policies.nodes.ConnectableElementPolicy#reconnectionAllowed(ch.unizh.ifi.adora.model.representation.ConnectableElement, ch.unizh.ifi.adora.model.representation.ConnectableElement, ch.unizh.ifi.adora.model.representation.Connection)
	 */
	protected boolean reconnectingAllowed(ConnectableElement source,
			ConnectableElement target, Connection con) {
		if (con instanceof Association) {
			return !((Association) con).isCalculatedAbstracted() &&
				getOtherEnd(source, target) instanceof Component  && 
					!isRecusiveChildOfAspectContainer((AspectContainer)getHost().getModel());
		} else if (con instanceof JoinRelation) {
			if (source instanceof AspectContainer){
				return super.reconnectingAllowed(source, target, con);
			} else if (source instanceof State || source instanceof Component || source instanceof Scenario) {
				return JoinRelationPolicyHelper.checkChunkCompletely(source, target);
			}
			return getOtherEnd(source, target) instanceof AspectContainer;
		}
		return false;
	}
	
	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.parts.policies.nodes.ConnectableElementPolicy#connectionCreateAllowedd(java.lang.Class)
	 */
	protected boolean creatingConnectionAllowed(Class connectionType) {
		return connectionType == Association.class && 
			!isRecusiveChildOfAspectContainer((AspectContainer)getHost().getModel()) ||
					connectionType  == JoinRelation.class;
	}
	
	/**
	 * Used to determine if constraints are violated.
	 *  
	 * @return <code>true</code> if the model is a Recusive Child Of an AspectContainer.
	 */
	private boolean isRecusiveChildOfAspectContainer(ZoomableNodeContainer model){
		if(model.getParent() instanceof AspectContainer) {
            return true;
        }
        if(model.getParent() instanceof ZoomableNodeContainer) {
            return isRecusiveChildOfAspectContainer((ZoomableNodeContainer)model.getParent());
        }
        return false;
	}
}