package ch.unizh.ifi.adora.parts.policies.nodes;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.requests.CreateConnectionRequest;

import ch.unizh.ifi.adora.model.commands.JoinRelationCreate;
import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.model.representation.State;
import ch.unizh.ifi.adora.parts.policies.JoinRelationPolicyHelper;
import ch.unizh.ifi.adora.ui.constrainthint.ConstraintHint;
import ch.unizh.ifi.adora.ui.constrainthint.IConstraintMessages;

/**
 * Policy for all {@link ch.unizh.ifi.adora.model.representation.Connection}s
 * which can be crosscutted by
 * {@link ch.unizh.ifi.adora.model.representation.JoinRelation}s originating
 * from {@link ch.unizh.ifi.adora.model.representation.AspectContainer}s.
 * 
 * @author Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 21.09.2006 IV First Version<br>
 *          25.01.2007 PS Constraints updated (Bug #806).
 * @version $Date: 2008-01-25 14:17:46 +0100 (Fri, 25 Jan 2008) $, $Author: schoeni $, $Revision: 3503 $
 */
public class ConstrainableConnectionPolicy extends ConnectableElementPolicy {

	/*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.parts.policies.nodes.ConnectableElementPolicy#connectingTargetAllowed(ch.unizh.ifi.adora.model.representation.Node,
     *      ch.unizh.ifi.adora.model.representation.ConnectableElement,
     *      org.eclipse.gef.commands.Command)
     */
	protected boolean connectingTargetAllowed(Node sourceNode,
			ConnectableElement targetElement, Command command) {
		if (command instanceof JoinRelationCreate) {
			if (sourceNode instanceof AspectContainer) {

				Connection connection = (Connection) getHost().getModel();

				if ((((AspectContainer) sourceNode).getParent()
						.hasChildRecursively(connection.getSource()) || ((AspectContainer) sourceNode)
						.getParent()
						.hasChildRecursively(connection.getTarget()))
						&& !(((AspectContainer) sourceNode)
								.hasChildRecursively(connection.getSource()) || ((AspectContainer) sourceNode)
								.hasChildRecursively(connection.getTarget()))
						&& sourceNode != connection.getTarget()
						&& sourceNode != connection.getSource()) {

					return true;
				} else {
					ConstraintHint
							.displayConstraintHint(IConstraintMessages.NON_FUNCTIONAL_CONNECTION);
					return false;
				}
			} else if (sourceNode instanceof State
					|| sourceNode instanceof Component
					|| sourceNode instanceof Scenario) {
				return JoinRelationPolicyHelper.checkChunkCompletely(sourceNode,
						targetElement);
			}
		}
		ConstraintHint
				.displayConstraintHint(IConstraintMessages.CONNECTION_CONNECTING);
		return false;
	}

	/*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.parts.policies.nodes.ConnectableElementPolicy#reconnectingAllowed(ch.unizh.ifi.adora.model.representation.ConnectableElement,
     *      ch.unizh.ifi.adora.model.representation.ConnectableElement,
     *      ch.unizh.ifi.adora.model.representation.Connection)
     */
	protected boolean reconnectingAllowed(ConnectableElement source,
			ConnectableElement target, Connection con) {
		if (con instanceof JoinRelation) {
			if (source instanceof AspectContainer) {

				Connection targetCon = (Connection) getHost().getModel();

				if ((((AspectContainer) source).getParent()
						.hasChildRecursively(con.getSource()) || ((AspectContainer) source)
						.getParent().hasChildRecursively(con.getTarget()))
						&& !(((AspectContainer) source).hasChildRecursively(con
								.getSource()) || ((AspectContainer) source)
								.hasChildRecursively(con.getTarget()))
						&& source != targetCon.getTarget()
						&& source != targetCon.getSource()) {

					return true;
				} else {
					ConstraintHint
							.displayConstraintHint(IConstraintMessages.NON_FUNCTIONAL_CONNECTION);
					return false;
				}
			} else if (source instanceof State || source instanceof Component
					|| source instanceof Scenario) {
				return JoinRelationPolicyHelper.checkChunkCompletely(source, target);
			}
		}
		ConstraintHint
				.displayConstraintHint(IConstraintMessages.CONNECTION_CONNECTING);
		return false;
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCreateCommand(org.eclipse.gef.requests.CreateConnectionRequest)
     */
	protected Command getConnectionCreateCommand(CreateConnectionRequest request) {
		return null;
	}

	/*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.parts.policies.nodes.ConnectableElementPolicy#connectionCreateAllowedd(java.lang.Class)
     */
	protected boolean creatingConnectionAllowed(Class connectionType) {
		return false;
	}
}