package ch.unizh.ifi.adora.parts.policies.nodes;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.model.commands.JoinRelationCreate;
import ch.unizh.ifi.adora.model.commands.TransitionCreate;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.ExitPoint;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.State;
import ch.unizh.ifi.adora.model.representation.Transition;
import ch.unizh.ifi.adora.ui.constrainthint.ConstraintHint;
import ch.unizh.ifi.adora.ui.constrainthint.IConstraintMessages;

/**
 * This policy determins, which elements, in what way, can be connected to an
 * ExitPoint.
 * 
 * @author Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 02.05.2006 IV First Version 23.07.2006 SM Some reformatting.
 * @version $Date: 2007-10-29 13:24:52 +0100 (Mon, 29 Oct 2007) $, $Author: schoeni $, $Revision: 3189 $
 */
public class ExitPointPolicy extends StateNodeEditPolicy {
	/*
     * Only a state from the same parent as the ExitPoint parent can be
     * connected.
     * 
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.parts.policy.StateNodeEditPolicy#connectionAllowed(ch.unizh.ifi.adora.model.representation.Node,
     *      org.eclipse.gef.commands.Command)
     */
	protected boolean connectingTargetAllowed(Node sourceNode,
			ConnectableElement targetComponent, Command command) {
		ExitPoint exitPoint = (ExitPoint) getHost().getModel();

		if (command instanceof TransitionCreate) {
			/*
             * Transitions may be connected to exit points
             */
			if (sourceNode instanceof State || sourceNode instanceof Component) {
				/*
                 * Only a state from the same parent as the ExitPoint parent can
                 * be connected.
                 */
				if (sourceNode.getParent() != exitPoint.getParent()) {
					ConstraintHint
							.displayConstraintHint(IConstraintMessages.TRANSITION_GRAPH);
					return false;
				}

				return true;
			}

			ConstraintHint
					.displayConstraintHint(IConstraintMessages.EXIT_POINT_CONNECTION);
			return false;
		}

		if (command instanceof JoinRelationCreate) {
			return false;
		}

		return false;
	}

	/*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.parts.policies.nodes.ConnectableElementPolicy#reconnectionAllowed(ch.unizh.ifi.adora.model.representation.ConnectableElement,
     *      ch.unizh.ifi.adora.model.representation.ConnectableElement,
     *      ch.unizh.ifi.adora.model.representation.Connection)
     */
	protected boolean reconnectingAllowed(ConnectableElement source,
			ConnectableElement target, Connection con) {
		if (con instanceof JoinRelation) {
			return false;
		}

		if (con instanceof Transition) {
			if (source.getParent() != target.getParent()) {
				ConstraintHint
						.displayConstraintHint(IConstraintMessages.TRANSITION_GRAPH);
				return false;
			}

			ConnectableElement otherEnd = getOtherEnd(source, target);

			if (otherEnd instanceof State || otherEnd instanceof Component) {
				return true;
			}
		}

		ConstraintHint
				.displayConstraintHint(IConstraintMessages.EXIT_POINT_CONNECTION);
		return false;
	}

	/*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.parts.policies.nodes.ConnectableElementPolicy#connectionCreateAllowedd(java.lang.Class)
     */
	protected boolean creatingConnectionAllowed(Class connectionType) {
		return false;
	}
}