package ch.unizh.ifi.adora.parts.tree;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartFactory;

import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.Node;

/**
 * 
 * Factory that maps model elements to TreeEditParts. TreeEditParts are used in
 * the outline view of the ShapesEditor.
 *
 * @author				smeier
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			23.08.2005 SM First Version
 * 						23.07.2006 SM Comments reformatting.
 * @version			$Date: 2006-07-24 12:29:04 +0200 (Mon, 24 Jul 2006) $, $Author: vigan $, $Revision: 1891 $
 *
 */
public class AdoraTreeEditPartFactory implements EditPartFactory {

	/*
	 * (non-Javadoc)
	 * 
	 * @@see org.eclipse.gef.EditPartFactory#createEditPart(org.eclipse.gef.EditPart,
	 *      java.lang.Object)
	 */
	public EditPart createEditPart(EditPart context, Object model) {
		if (model instanceof Node) {
			return new NodeTreeEditPart((Node) model);
		}
		if (model instanceof AdoraModel) {
			return new AdoraModelTreeEditPart((AdoraModel) model);
		}
		return null; // will not show an entry for the corresponding model
						// instance
	}
}