package ch.unizh.ifi.adora.properties;

import org.eclipse.gef.commands.CommandStackEvent;
import org.eclipse.gef.commands.CommandStackEventListener;
import org.eclipse.ui.views.properties.PropertySheetPage;

import ch.unizh.ifi.adora.model.commands.SetProperty;

/**
 * A {@link org.eclipse.ui.views.properties.PropertySheetPage} where a  
 * {@link org.eclipse.ui.views.properties.PropertySheetSorter} can be set.
 * Eclipse only provides an alphabetic {@link org.eclipse.ui.views.properties.PropertySheetSorter} 
 * which isn't suitable for our {@link org.eclipse.ui.views.properties.PropertySheetPage}.
 * 
 * This class is necessary because a 
 * {@link org.eclipse.ui.views.properties.PropertySheetSorter} can 
 * only be set from a subclass of {@link org.eclipse.ui.views.properties.PropertySheetPage}.
 * 
 * @author		        Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             21.08.2006 IV First Version
 * @version             $Date: 2006-09-26 10:52:33 +0200 (Tue, 26 Sep 2006) $, $Author: vigan $, $Revision: 2267 $
 */
public class AdoraPropertySheet extends PropertySheetPage implements CommandStackEventListener{
	
	/**
	 * Instantiates a new AdoraPropertySheet.
	 */
	public AdoraPropertySheet(){
		setSorter(new AdoraPropertySheetSorter());
	}

	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.gef.commands.CommandStackEventListener#stackChanged(org.eclipse.gef.commands.CommandStackEvent)
	 */
	public void stackChanged(CommandStackEvent event) {
		if (!(event.getCommand() instanceof SetProperty)){
			refresh();
		}
	}
}