package ch.unizh.ifi.adora.properties;

import java.util.Iterator;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.views.properties.ComboBoxPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertyDescriptor;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.properties.AspectContainerPropNames;
import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.variability.Constraint;
import ch.unizh.ifi.adora.model.representation.variability.Decision;
import ch.unizh.ifi.adora.model.representation.variability.VariationPoint;
import ch.unizh.ifi.adora.properties.longtextproperty.LongTextPropertyDescriptor;
import ch.unizh.ifi.adora.properties.specification.AspectContainerSpecificationPropertyAccess;
import ch.unizh.ifi.adora.properties.specification.ExtendedSpecificationPropNames;

/**
 * The property source for the aspect container.
 * 
 * @author IV
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history April 26, 2006 IV First Version May 12, 2006 SM Help context ids Aug
 *          8, 2006 SM Refactoring. 2008-03-05 PS Added property.
 *          Apr 09, 2008 CJ Address Bug 855
 * @version $Date: 2009-05-06 10:51:39 +0200 (Wed, 06 May 2009) $, $Author: treinhard $, $Revision: 4182 $
 * 
 */
public class AspectContainerProperties extends ZoomableNodeContainerProperties
		implements 
		AspectContainerPropNames, ExtendedSpecificationPropNames {

	/**
     * Initializes the aspect container properties.
     * 
     * @param aspectContainer
     *            The host aspect container (modell element).
     */
	public AspectContainerProperties(AspectContainer aspectContainer) {
		super(aspectContainer);
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
     */
	protected IPropertyDescriptor[] createProperties() {
		LongTextPropertyDescriptor spec = new LongTextPropertyDescriptor(
				SPECIFICATION_ACCESS_PROP, SPECIFICATION_PROP_NAME);
		IPropertyDescriptor[] desc = new IPropertyDescriptor[] {
				spec,
				new ComboBoxPropertyDescriptor(ASPECT_VARIBILITY_PROP,
						"Variability", boolValues) };

		setHelpId(desc, PROPERTIES_HELP_CONTEXT);

		return this
				.getJoinedPropertyDescriptors(super.createProperties(), desc);
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java.lang.Object)
     */
	public Object getPropertyValue(Object propertyId) {
		if (ASPECT_VARIBILITY_PROP.equals(propertyId)) {
			return new Integer(getModel().isVariability() ? 1 : 0);
		}
		if(SPECIFICATION_PROP.equals(propertyId))
		{
			return getModel().getSpecification();
		}
		//Bug Fix 855: This property's value is "wrapped" in a ILongTextPropertyAccessor
		if (SPECIFICATION_ACCESS_PROP.equals(propertyId))
		{
			return new AspectContainerSpecificationPropertyAccess(getModel(),this);
		}

		return super.getPropertyValue(propertyId);
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object,
     *      java.lang.Object)
     */
	public void setPropertyValue(Object propertyId, Object value) {
		if (ASPECT_VARIBILITY_PROP.equals(propertyId)) {
			
			if (((Integer) value).intValue() == 0)
				if (!isDecisionOk())
					return;
			
			putNewCommandOnStack(getModel(), ((Integer) value).intValue() > 0,
					boolean.class, "setVariability", "isVariability");
		}
		if (SPECIFICATION_PROP.equals(propertyId)) {
			putNewCommandOnStack(getModel(), value, String.class,
					"setSpecification", "getSpecification", SPECIFICATION_PROP_NAME);
		}
		if(SPECIFICATION_ACCESS_PROP.equals(propertyId))
		{
			// do nothing: specificationAccess property is readonly
		}
		super.setPropertyValue(propertyId, value);
	}

	/*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.properties.VisualizableModelElementProperties#getModel()
     */
	protected AspectContainer getModel() {
		return (AspectContainer) super.getModel();
	}
	
	/**
	 * Check if a decision would be deleted which is contained in some constraint.
	 * 
	 * @return If the component can be deleted. 
	 */
	private boolean isDecisionOk() {
		
		String confConstr = "";
		String decStr = "";
		
		
		for (Connection con : getModel().getConnections()) {
			if (con instanceof JoinRelation ) {
				Decision decision = ((JoinRelation)con).getDecision();
				
				if (decision != null) {
					for (VariationPoint vp : AdoraPlugin.getActiveModel().getVariationPoints()) {
						if (vp.getDecisions().contains(decision)) {
							if (!confConstr.contains("VP"+vp.getNr()+" "))
								confConstr += "VP"+vp.getNr()+" ";
							if (!decStr.contains("D"+decision.getNr()+" "))
								decStr +="D"+ decision.getNr()+" ";
						}
					}
					for (Constraint c : AdoraPlugin.getActiveModel().getConstraints()) {
						if (c.containsDecision(decision)) {
							if (!confConstr.contains("C"+c.getNr()+" "))
								confConstr += "C"+c.getNr()+" ";
							if (!decStr.contains("D"+decision.getNr()+" "))
								decStr +="D"+ decision.getNr()+" ";
						}
					}
				}
			}
		}
		
			
		
		Iterator<ModelElement> itr  = getModel().getRecursiveChildrenIterator();
		while (itr.hasNext()) {
			ModelElement ele = itr.next();
			if (ele instanceof ConnectableElement) {
				for (Connection con : ((ConnectableElement)ele).getConnections()) {
					if (con instanceof JoinRelation ) {
						Decision decision = ((JoinRelation)con).getDecision();
						
						if (decision != null) {
							for (VariationPoint vp : AdoraPlugin.getActiveModel().getVariationPoints()) {
								if (vp.getDecisions().contains(decision)) {
									if (!confConstr.contains("VP"+vp.getNr()+" "))
										confConstr += "VP"+vp.getNr()+" ";
									if (!decStr.contains("D"+decision.getNr()+" "))
										decStr +="D"+ decision.getNr()+" ";
								}
							}
							for (Constraint c : AdoraPlugin.getActiveModel().getConstraints()) {
								if (c.containsDecision(decision)) {
									if (!confConstr.contains("C"+c.getNr()+" "))
										confConstr += "C"+c.getNr()+" ";
									if (!decStr.contains("D"+decision.getNr()+" "))
										decStr +="D"+ decision.getNr()+" ";
								}
							}
						}
					}
				}
			}
		}
		if (!confConstr.equals("")) {
			showWarning(decStr, confConstr);
			return false;
		}

		return true;
	}	

	
	private void showWarning(String decStr, String conf) {
		MessageBox mb = new MessageBox(PlatformUI.getWorkbench().
				getActiveWorkbenchWindow().getShell(), 
						SWT.ICON_INFORMATION | SWT.OK);
		mb.setText("Decision Conflict");
		mb.setMessage("Decisions:"+ decStr + " appear in: "+conf);
	    mb.open();
	}
}
	
