package ch.unizh.ifi.adora.properties;

import org.eclipse.ui.views.properties.IPropertyDescriptor;

import ch.unizh.ifi.adora.model.representation.CommentableElement;
import ch.unizh.ifi.adora.properties.comment.CommentableCommentPropertyAccess;
import ch.unizh.ifi.adora.properties.comment.ExtendedCommentPropNames;
import ch.unizh.ifi.adora.properties.longtextproperty.LongTextPropertyDescriptor;

/**
 * This PropertySource object handles all property request (get & set)
 * for an association object. Also all necessary PropertyDescriptors are instantiated
 * here. 
 *
 * @author		        Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             21.09.2006 IV First Version
 * 						15.04.2008 CJ Fix Bug 855 (and refactoring of long text property editors)
 * @version             $Date: 2008-06-23 14:28:48 +0200 (Mon, 23 Jun 2008) $, $Author: stoiber $, $Revision: 3650 $
 */
public class CommentableElementProperties extends VisualizableModelElementProperties implements ExtendedCommentPropNames {

    /**
	 * Instantiates a new NodeProperties object.
	 * @param node
	 */
	public CommentableElementProperties(CommentableElement element){
		super(element);
	}
		
	 /**
     * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
     */
    protected IPropertyDescriptor[] createProperties() {
    	LongTextPropertyDescriptor com = new LongTextPropertyDescriptor(
				COMMENT_ACCESS_PROP, COMMENT_PROP_NAME);
		IPropertyDescriptor[] desc = new IPropertyDescriptor[] {com };
		setHelpId(desc, PROPERTIES_HELP_CONTEXT);
		return desc;
    }
	
 	
	/**
	 * Return the property value for the given propertyId, or null.
	 * <p>
	 * The property view uses the IDs from the IPropertyDescriptors array to
	 * obtain the value of the corresponding properties.
	 * </p>
	 * 
	 * @see #descriptors
	 * @see #getPropertyDescriptors()
	 */
	public Object getPropertyValue(Object propertyId) {
		if (COMMENT_PROP.equals(propertyId)) {
            return getModel().getComment();
        }
		if (COMMENT_ACCESS_PROP.equals(propertyId)) {
            return new CommentableCommentPropertyAccess(getModel(),this);
        }
		return null;
	}

	/**
	 * Set the property value for the given property id. If no matching id is
	 * found, the call is forwarded to the superclass.
	 * <p>
	 * The property view uses the IDs from the IPropertyDescriptors array to set
	 * the values of the corresponding properties.
	 * </p>
	 * 
	 * @see #descriptors
	 * @see #getPropertyDescriptors()
	 */
	public void setPropertyValue(Object propertyId, Object value) {
		if (COMMENT_PROP.equals(propertyId)) {
			putNewCommandOnStack(getModel(), value, 
					String.class, "setComment", "getComment", COMMENT_PROP_NAME);
		} 
	}	
	
	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#getEditableValue()
	 */
	public Object getEditableValue() {
		return null;
	}

	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#isPropertySet(java.lang.Object)
	 */
	public boolean isPropertySet(Object id) {
		return false;
	}

	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#resetPropertyValue(java.lang.Object)
	 */
	public void resetPropertyValue(Object id) {
	}
	
	
	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.properties.VisualizableModelElementProperties#getModel()
	 */
	protected CommentableElement getModel(){
		return (CommentableElement)super.getModel();
	}
}