package ch.unizh.ifi.adora.properties;


import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import ch.unizh.ifi.adora.model.properties.AbstractableConnectionPropNames;
import ch.unizh.ifi.adora.model.properties.ConnectableElementPropNames;
import ch.unizh.ifi.adora.model.properties.VisualModElemPropNames;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.Node;


/**
 * This PropertySource object handels all property request (get & set)
 * for a Node object. Also all necessary PropertyDescriptors are instantiated
 * here.  This class implements the generic method pattern. All subclasses call
 * the getPropertyDescriptors method which then calls the getLocalDescriptors- and  
 * getSuperDescriptors-method of the current object.
 *
 * @author		       Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br> 
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             06.03.2006 IV First Version
 *                      12.05.2006 SM Context help ids
 *                      08.08.2006 SM Refactoring.
 * @version             $Date: 2009-05-06 10:51:39 +0200 (Wed, 06 May 2009) $, $Author: treinhard $, $Revision: 4182 $
 */
public abstract class NodeProperties extends CommentableElementProperties 
implements ConnectableElementPropNames, AbstractableConnectionPropNames, VisualModElemPropNames {

    /**
     * Stores the logical pair of variates; 
     */
    protected static final String[] boolValues = {"false","true"};
    
    /**
     * Displays the values in the Design Decision combobox.
     */
    protected static final String[] designDecisions = { "", "Existence", "Properties", "Structure", "Texture" };
    
    /**
     * Displays the values in the Requirements ID combobox.
     */
    
    /**
     * Stores the current adora model
     */
    private AdoraModel adoraModel;
    
    /**
     * Displays the values in the RequirementID combobox.
     */
    protected String[] requirementIDs;
      
    /**
	 * Instantiates a new NodeProperties object.
	 * @param node
	 */
	public NodeProperties(Node node){
		super(node);
	}
		
	 /**
     * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
     */
    protected IPropertyDescriptor[] createProperties() {
    	IPropertyDescriptor[] desc = new IPropertyDescriptor[]{
	   			new TextPropertyDescriptor(NAME_PROP, "Name"),
				new PropertyDescriptor(XPOS_PROP, "X"), 
	   			new PropertyDescriptor(YPOS_PROP, "Y"),
	   			new PropertyDescriptor(WIDTH_PROP, "Width"),
	   			new PropertyDescriptor(HEIGHT_PROP, "Height")
			};
	            		
			for(int i = 1; i < desc.length; i++) {
			    PropertyDescriptor pd = ((PropertyDescriptor)desc[i]); 
			    pd.setCategory("Bounds");
			}
	        setHelpId(desc, PROPERTIES_HELP_CONTEXT);
	        return this.getJoinedPropertyDescriptors(super.createProperties(), desc);
    }
	
    
    /*
     *  (non-Javadoc)
     * @see org.eclipse.ui.views.properties.IPropertySource#getEditableValue()
     */
	public Object getEditableValue() {
		return null;
	}
	
	/**
	 * Return the property value for the given propertyId, or null.
	 * <p>
	 * The property view uses the IDs from the IPropertyDescriptors array to
	 * obtain the value of the corresponding properties.
	 * </p>
	 * 
	 * @see #descriptors
	 * @see #getPropertyDescriptors()
	 */
	public Object getPropertyValue(Object propertyId) {
		
		if (XPOS_PROP.equals(propertyId)) {
			return Integer.toString(getModel().getLocation().x);
		}
		if (YPOS_PROP.equals(propertyId)) {
			return Integer.toString(getModel().getLocation().y);
		}
		if (HEIGHT_PROP.equals(propertyId)) {
			return Integer.toString(getModel().getSize().height);
		}
		if (WIDTH_PROP.equals(propertyId)) {
			return Integer.toString(getModel().getSize().width);
		}
		if (NAME_PROP.equals(propertyId)) {
			return getModel().getName();
		}

		return super.getPropertyValue(propertyId);
	}

	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#isPropertySet(java.lang.Object)
	 */
	public boolean isPropertySet(Object id) {
		return false;
	}

	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#resetPropertyValue(java.lang.Object)
	 */
	public void resetPropertyValue(Object id) {
	}

	/**
	 * Set the property value for the given property id. If no matching id is
	 * found, the call is forwarded to the superclass.
	 * <p>
	 * The property view uses the IDs from the IPropertyDescriptors array to set
	 * the values of the corresponding properties.
	 * </p>
	 * 
	 * @see #descriptors
	 * @see #getPropertyDescriptors()
	 */
	public void setPropertyValue(Object propertyId, Object value) {
		if (NAME_PROP.equals(propertyId)) {
			putNewCommandOnStack(getModel(), value, 
					String.class, "setName", "getName");
		}
		if (REQUIREMENT_ID_PROP.equals(propertyId)) {
			putNewCommandOnStack(getModel(), ((Integer) value).intValue(), 
					int.class, "setRequirementID", "getRequirementID");
		}
		if (DESIGN_DECISION_PROP.equals(propertyId)) {
			putNewCommandOnStack(getModel(), ((Integer) value).intValue(), 
					int.class, "setDesignDecision", "getDesignDecision");
		}
		
		super.setPropertyValue(propertyId, value);
	}	
	
	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.properties.VisualizableModelElementProperties#getModel()
	 */
	protected Node getModel(){
		return (Node)super.getModel();
	}
}