package ch.unizh.ifi.adora.properties;

import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import ch.unizh.ifi.adora.model.properties.CardinalityPropNames;
import ch.unizh.ifi.adora.model.properties.VisualModElemPropNames;
import ch.unizh.ifi.adora.model.representation.Role;
import ch.unizh.ifi.adora.properties.validators.cardinality.MaxCardinalityValidator;
import ch.unizh.ifi.adora.properties.validators.cardinality.MinCardinalityValidator;


/**
 * This PropertySource object handels all property request (get & set)
 * for a role object. Also all necessary PropertyDescriptors are instantiated
 * here. 
 *      
 * @author              Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             18.04.2006 SM First Version
 *                      08.08.2006 SM Refactoring.
 * @version             $Date: 2006-09-28 16:05:58 +0200 (Thu, 28 Sep 2006) $, $Author: vigan $, $Revision: 2297 $
 */
public class RoleProperties extends VisualizableModelElementProperties  implements CardinalityPropNames, VisualModElemPropNames, IPropertySource {

     /**
     * Creates the role property source for the given role model element.
     * @param role
     */
    public RoleProperties(Role role) {
    	super(role);
    }
    
    /*
     *  (non-Javadoc)
     * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
     */
    protected IPropertyDescriptor[] createProperties() {
        IPropertyDescriptor[] returnDesc = new IPropertyDescriptor[3];


            returnDesc[0] = new TextPropertyDescriptor(NAME_PROP, "Label");        

            returnDesc[1] = new TextPropertyDescriptor(MINCARD_PROP,
                    "Min Cardinality");

            returnDesc[2] = new TextPropertyDescriptor(MAXCARD_PROP,
                    "Max Cardinality");

            ((TextPropertyDescriptor)returnDesc[1]).setValidator(
            		new MinCardinalityValidator(getModel().getCardinality()));
            ((TextPropertyDescriptor)returnDesc[1]).setValidator(
            		new MaxCardinalityValidator(getModel().getCardinality()));
            
            setHelpId(returnDesc, PROPERTIES_HELP_CONTEXT);           

        return returnDesc;
    }
    
    /**
     * Return the property value for the given propertyId, or null.
     * <p>
     * The property view uses the IDs from the IPropertyDescriptors array to
     * obtain the value of the corresponding properties.
     * </p>
     * 
     * @see #descriptors
     * @see #getPropertyDescriptors()
     */
    public Object getPropertyValue(Object propertyId) {
        if (MAXCARD_PROP.equals(propertyId)) {
            return getModel().getMaxCard();
        }
        if (MINCARD_PROP.equals(propertyId)) {
            return getModel().getMinCard();
        }
        if (NAME_PROP.equals(propertyId)) {
            return getModel().getName();
        }
        return null;
    }

    /**
     * Set the property value for the given property id. If no matching id is
     * found, the call is forwarded to the superclass.
     * <p>
     * The property view uses the IDs from the IPropertyDescriptors array to set
     * the values of the corresponding properties.
     * </p>
     * 
     * @see #descriptors
     * @see #getPropertyDescriptors()
     */
    public void setPropertyValue(Object propertyId, Object value) {
        if (MINCARD_PROP.equals(propertyId)) {
        	putNewCommandOnStack(getModel(), value, 
					String.class, "setMinCard", "getMinCard");
	    } 
        else if (MAXCARD_PROP.equals(propertyId)) {
    		putNewCommandOnStack(getModel(), value, 
					String.class, "setMaxCard", "getMaxCard");
        }
        else if (NAME_PROP.equals(propertyId)) {
       		putNewCommandOnStack(getModel(), value, 
					String.class, "setName", "getName");
        }
        //ConstraintHint.resetConstraintHint();
    }   

    /*
     *  (non-Javadoc)
     * @see org.eclipse.ui.views.properties.IPropertySource#getEditableValue()
     */
    public Object getEditableValue() {
        return null;
    }

    /*
     *  (non-Javadoc)
     * @see org.eclipse.ui.views.properties.IPropertySource#isPropertySet(java.lang.Object)
     */
    public boolean isPropertySet(Object id) {
        return false;
    }

    /*
     *  (non-Javadoc)
     * @see org.eclipse.ui.views.properties.IPropertySource#resetPropertyValue(java.lang.Object)
     */
    public void resetPropertyValue(Object id) {
    }
    
	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.properties.VisualizableModelElementProperties#getModel()
	 */
	protected Role getModel(){
		return (Role)super.getModel();
	}
}