package ch.unizh.ifi.adora.properties;

import org.eclipse.ui.views.properties.ComboBoxPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.PropertyDescriptor;

import ch.unizh.ifi.adora.model.properties.StartStatePropNames;
import ch.unizh.ifi.adora.model.representation.StartStateContainer;
import ch.unizh.ifi.adora.properties.validators.StartStateValidator;

/**
 * 
 * Represents the properties for containers that can be a start state (i.e.
 * states and components).
 * 
 * @author smeier
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history Apr 28, 2006 IV First Version Aug 8, 2006 SM Refactoring
 * @version $Date: 2007-05-21 10:48:21 +0200 (Mon, 21 May 2007) $, $Author: schoeni $, $Revision: 2699 $
 * 
 */
public class StartStateContainerProperties extends
		ZoomableNodeContainerProperties implements StartStatePropNames {

	/**
     * Constructs a new ActorProperties object for a given actor object.
     * 
     * @param actor,
     *            for which this IPropertySource object is been instantiated.
     */
	public StartStateContainerProperties(StartStateContainer startState) {
		super(startState);
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
     */
	protected IPropertyDescriptor[] createProperties() {
		IPropertyDescriptor[] desc = new IPropertyDescriptor[] { new ComboBoxPropertyDescriptor(
				STARTSTATE_PROP, "Start State", boolValues) };
		setHelpId(desc, PROPERTIES_HELP_CONTEXT);

		((PropertyDescriptor) desc[0]).setValidator(new StartStateValidator(
				getModel()));

		return this
				.getJoinedPropertyDescriptors(super.createProperties(), desc);
	}

	/**
     * Returns the property value for the given propertyId, or null.
     * <p>
     * The property view uses the IDs from the IPropertyDescriptors array to
     * obtain the value of the corresponding properties.
     * </p>
     * 
     * @see #descriptors
     * @see #getPropertyDescriptors()
     */
	public Object getPropertyValue(Object propertyId) {
		if (STARTSTATE_PROP.equals(propertyId)) {
			return new Integer(getModel().isStartState() ? 1 : 0);
		}
		return super.getPropertyValue(propertyId);
	}

	/**
     * Sets the property value for the given property id. If no matching id is
     * found, the call is forwarded to the superclass.
     * <p>
     * The property view uses the IDs from the IPropertyDescriptors array to set
     * the values of the corresponding properties.
     * </p>
     * 
     * @see #descriptors
     * @see #getPropertyDescriptors()
     */
	public void setPropertyValue(Object propertyId, Object value) {
		if (STARTSTATE_PROP.equals(propertyId)) {
			putNewCommandOnStack(getModel(), ((Integer) value).intValue() > 0,
					boolean.class, "setStartState", "isStartState");
		}
		super.setPropertyValue(propertyId, value);
	}

	/*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.properties.VisualizableModelElementProperties#getModel()
     */
	protected StartStateContainer getModel() {
		return (StartStateContainer) super.getModel();
	}
}