package ch.unizh.ifi.adora.properties;

import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import ch.unizh.ifi.adora.model.properties.ConnectionPropNames;
import ch.unizh.ifi.adora.model.representation.TransitionDescription;

/**
 * This PropertySource object handels all property request (get & set) for an
 * transitionlabel object. Also all necessary PropertyDescriptors are
 * instantiated here.
 * 
 * @author 		Ivo Vigan
 * @copyright 	Requirements Engineering Research Group<br>
 *            	Department of Informatics<br>
 *            	University of Zurich<br>
 * @history 	06.03.2006 SM First Version 
 * 				08.08.2006 SM Refactoring. 
 * 				22.09.2006 SM Bug 772 fixed.
 *              20.09.2007 SM Refactoring call to the syntax problem view.
 * @version $Date: 2009-05-06 10:51:39 +0200 (Wed, 06 May 2009) $, $Author: treinhard $, $Revision: 4182 $
 */
public class TransitionDescriptionProperties extends
		VisualizableModelElementProperties implements IPropertySource,
		ConnectionPropNames {

	/**
     * Constructs a new TransitionProperties object for a given transition
     * object.
     * 
     * @param transition
     */
	public TransitionDescriptionProperties(TransitionDescription transitionLabel) {
		super(transitionLabel);
	}

	/**
     * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
     */
	protected IPropertyDescriptor[] createProperties() {
		IPropertyDescriptor[] desc;

		desc = new IPropertyDescriptor[] { new TextPropertyDescriptor(
				NAME_PROP, "Transition Description") };

		setHelpId(desc, "ch.unizh.ifi.adora.Properties");
		return desc;
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.views.properties.IPropertySource#getEditableValue()
     */
	public Object getEditableValue() {
		return null;
	}

	/**
     * Return the property value for the given propertyId, or null.
     * <p>
     * The property view uses the IDs from the IPropertyDescriptors array to
     * obtain the value of the corresponding properties.
     * </p>
     * 
     * @see #descriptors
     * @see #getPropertyDescriptors()
     */
	public Object getPropertyValue(Object propertyId) {
		if (NAME_PROP.equals(propertyId)) {
			return getModel().getName();
		}
		return null;
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.views.properties.IPropertySource#isPropertySet(java.lang.Object)
     */
	public boolean isPropertySet(Object id) {
		return false;
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.views.properties.IPropertySource#resetPropertyValue(java.lang.Object)
     */
	public void resetPropertyValue(Object id) {
	}

	/**
     * Set the property value for the given property id. If no matching id is
     * found, the call is forwarded to the superclass.
     * <p>
     * The property view uses the IDs from the IPropertyDescriptors array to set
     * the values of the corresponding properties.
     * </p>
     * 
     * @see #descriptors
     * @see #getPropertyDescriptors()
     */
	public void setPropertyValue(Object propertyId, Object value) {
		if (NAME_PROP.equals(propertyId)
				|| TransitionProperties.NAME_PROP.equals(propertyId)) {

			putNewCommandOnStack(getModel(), value, String.class, "setName",
					"getName");

			//SyntaxProblemsView.getInstance().checkForSyntaxErrors(getModel());
		}
	}

	/*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.properties.VisualizableModelElementProperties#getModel()
     */
	protected TransitionDescription getModel() {
		return (TransitionDescription) super.getModel();
	}
}