package ch.unizh.ifi.adora.properties;

import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import ch.unizh.ifi.adora.model.representation.Transition;

/**
 * This PropertySource object handels all property request (get & set) for an
 * transitionlabel object. Also all necessary PropertyDescriptors are
 * instantiated here.
 * 
 * @author Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 21.09.2006 IV First Version
 * @version $Date: 2007-05-02 09:39:20 +0200 (Wed, 02 May 2007) $, $Author: schoeni $, $Revision: 2618 $
 */
public class TransitionProperties extends ConnectionProperties {

	/**
     * Constructs a new TransitionProperties object for a given transition
     * object.
     * 
     * @param transition
     */
	public TransitionProperties(Transition trans) {
		super(trans);
	}

	/**
     * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
     */
	protected IPropertyDescriptor[] createProperties() {
		IPropertyDescriptor[] desc;

		desc = new IPropertyDescriptor[] { new TextPropertyDescriptor(
				NAME_PROP, "Transition Description") };

		setHelpId(desc, "ch.unizh.ifi.adora.Properties");
		return this
				.getJoinedPropertyDescriptors(super.createProperties(), desc);
	}

	/**
     * Set the property value for the given property id. If no matching id is
     * found, the call is forwarded to the superclass.
     * <p>
     * The property view uses the IDs from the IPropertyDescriptors array to set
     * the values of the corresponding properties.
     * </p>
     * 
     * @see #descriptors
     * @see #getPropertyDescriptors()
     */
	public void setPropertyValue(Object propertyId, Object value) {
		if (NAME_PROP.equals(propertyId)) {
			TransitionDescriptionProperties tdp = new TransitionDescriptionProperties(
					getModel().getLabel());
			tdp.setPropertyValue(propertyId, value);
		}

		super.setPropertyValue(propertyId, value);
	}

	/**
     * Return the property value for the given propertyId, or null.
     * <p>
     * The property view uses the IDs from the IPropertyDescriptors array to
     * obtain the value of the corresponding properties.
     * </p>
     * 
     * @see #descriptors
     * @see #getPropertyDescriptors()
     */
	public Object getPropertyValue(Object propertyId) {
		if (NAME_PROP.equals(propertyId)) {
			return getModel().getLabel().getName();
		}

		return super.getPropertyValue(propertyId);
	}

	/*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.properties.VisualizableModelElementProperties#getModel()
     */
	protected Transition getModel() {
		return (Transition) super.getModel();
	}
}