package ch.unizh.ifi.adora.properties;

import org.eclipse.ui.views.properties.ComboBoxPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.PropertyDescriptor;

import ch.unizh.ifi.adora.model.properties.ZoomablePropNames;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.properties.validators.component.PartialValidator;

/**
 * This PropertySource object handels all property request (get & set)
 * for a ZoomableNodeContainer object. Concret it handles the properties for
 * actors and components. 
 * Also all necessary PropertyDescriptors are instantiated here. 
 *    
 * @author		       Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             06.03.2006 IV First Version
 *                      12.05.2006 SM Help context IDs
 *                      08.08.2006 SM Refactoring.
 * @version             $Date: 2006-10-27 15:02:09 +0200 (Fri, 27 Oct 2006) $, $Author: smeier $, $Revision: 2338 $
 *
 */
public class ZoomableNodeContainerProperties extends NodeProperties implements ZoomablePropNames{

	/**
	 * Constructs a new ZoomableNodeContainerProperties object for a given ZoomableNodeContainer object.
	 * @param zoomContainer ZoomableNodeContainer, for which this IPropertySource object is been
	 * instantiated. 
	 */
	public ZoomableNodeContainerProperties(ZoomableNodeContainer zoomContainer) {
		super(zoomContainer);
	}
	
	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
	 */
	protected IPropertyDescriptor[] createProperties(){
		IPropertyDescriptor[] desc = new IPropertyDescriptor[]{
		   		new PropertyDescriptor(ZOOMED_PROP, "Zoomed"),
		   		new ComboBoxPropertyDescriptor(ISPARTIAL_PROP, "Partial", boolValues)
		};
		
		if (this instanceof ComponentProperties) {
			((PropertyDescriptor) desc[1]).setValidator(
					new PartialValidator((Component)getModel()));
		}
		
        setHelpId(desc, PROPERTIES_HELP_CONTEXT);
	
    	return this.getJoinedPropertyDescriptors(super.createProperties(), desc);
    }
	
	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java.lang.Object)
	 */
	public Object getPropertyValue(Object propertyId) {
		if (ZOOMED_PROP.equals(propertyId)) {
			return Boolean.toString(getModel().isZoomedIn());
		}
		if (ISPARTIAL_PROP.equals(propertyId)) {
            return new Integer(getModel().isPartial() ? 1:0);
		}
		return super.getPropertyValue(propertyId);
	}

	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object, java.lang.Object)
	 */
	public void setPropertyValue(Object propertyId, Object value) {
		if (ISPARTIAL_PROP.equals(propertyId)) {
			putNewCommandOnStack(getModel(), ((Integer)value).intValue() > 0, 
		        	boolean.class, "setPartial", "isPartial");
		}
		super.setPropertyValue(propertyId, value);
	}
	
	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.properties.VisualizableModelElementProperties#getModel()
	 */
	protected ZoomableNodeContainer getModel(){
		return (ZoomableNodeContainer)super.getModel();
	}
}