package ch.unizh.ifi.adora.properties.longtextproperty;

import org.eclipse.core.runtime.PlatformObject;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.IPersistableElement;

/**
 * An {@link AbstractLongTextEditorInput} is an editor input
 * for {@link LongTextPropertyEditor}. 
 * It keeps a {@link ILongTextPropertyAccess} - 
 * an accessor to a long text property to a model element
 *
 * @author		        Cedric Jeanneret
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             15.04.2008 CJ First Version (Fix Bug 855 and refactoring of long text property editors)
 * @version             $Date: 2008-10-27 10:12:16 +0100 (Mon, 27 Oct 2008) $, $Author: jehle $, $Revision: 3853 $
 */
public abstract class AbstractLongTextEditorInput extends PlatformObject implements ILongTextEditorInput{

	/**
	 * Stores the specification property of this editor input.
	 */
	private ILongTextPropertyAccess propertyAccess;

	/** 
	 *  Creates a long text editor input on the given property accessor. 
	 **/
	public AbstractLongTextEditorInput(ILongTextPropertyAccess propertyAccess) {
		this.propertyAccess = propertyAccess;
	}

	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.ui.IEditorInput#exists()
	 */
	public boolean exists() {
		return true;
	}

	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.ui.IEditorInput#getImageDescriptor()
	 */
	public ImageDescriptor getImageDescriptor() {
		return null;
	}

	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.ui.IEditorInput#getName()
	 */
	public String getName() {
		return getPropertyAccess().getPropertyName()
				+ ": " + getPropertyAccess().getArtifactName();
	}

	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.ui.IEditorInput#getPersistable()
	 */
	/**
	 * The editor should not be persisted through sessions
	 */
	public IPersistableElement getPersistable() {
		return null;
	}

	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.ui.IEditorInput#getToolTipText()
	 */
	public String getToolTipText() {
		return getPropertyAccess().getPropertyName()
				+ " of " + getPropertyAccess().getArtifactName();
	}


	/**
	 * Answers true for two inputs that are the same. 
	 * The IWorbenchPage.openEditor APIs are dependent on this 
	 * to find an editor with the same input.
	 * In this case, two inputs are the same if their {@link ILongTextPropertyAccess}
	 * refers to the same model element and access to the same property
	 */
	public boolean equals(Object obj) {
		if (obj instanceof AbstractLongTextEditorInput)
		{
			AbstractLongTextEditorInput other = (AbstractLongTextEditorInput) obj;
			return this.getPropertyAccess().refersToSameValue(other.getPropertyAccess());
		}
		return false;
	}

	/**
	 * Getter for the property accessor
	 * @return the property accessor
	 */
	public ILongTextPropertyAccess getPropertyAccess() {
		return propertyAccess;
	}
}