package ch.unizh.ifi.adora.properties.longtextproperty;

import org.eclipse.core.runtime.IAdaptable;

/**
 * Interface for long text properties (such as comment, context map and specification) accessors.
 * Objects conforming to this interface will be given to 
 * {@link LongTextPropertyCellEditor} and {@link AbstractLongTextEditorInput} as values to be edited.
 * 
 * It is important that implementation classes can be adapted (via the {@link IAdaptable} mechanism)
 * to {@link AbstractLongTextEditorInput}.
 *      
 * @author              Cedric Jeanneret
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             15.04.2008 CJ First Version (Fix Bug 855 and refactoring of long text property editors)
 * @version             $Date: 2008-10-27 10:12:16 +0100 (Mon, 27 Oct 2008) $, $Author: jehle $, $Revision: 3853 $
 */
public interface ILongTextPropertyAccess extends IAdaptable {
	
    /**
     * Returns a printed name of the model element this object gives access to
     * @return the model element's printed name.
     */
    public String getArtifactName();
    
    /**
     * Returns a printed name of the property this object gives access to
     * @return the property's printed name.
     */
    public String getPropertyName();
	
	
    /**
     * Returns the long text as a string.
     * 
     * @return long text as string.
     */
    public String getLongText();
    
    /**
     * Sets a new value for this long text property
     * 
     * @param newLongText the new long text
     */
    public void setLongtext(String newLongText);
       
    /**
     * This method determines whether 2 ILongTextPropertyAccess refers
     * to the same model element and to the same property
     * 
     * @param other an {@link ILongTextPropertyAccess} object to compare to
     * @return true if "this" and "other" refers to the same model element and the same property
     */
    public boolean refersToSameValue(ILongTextPropertyAccess other);

}
