package ch.unizh.ifi.adora.properties.longtextproperty;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.jface.viewers.DialogCellEditor;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;

/**
 * This {@link DialogCellEditor} has a label control on the left and a button on the right. 
 * Pressing the button opens a LongTextEditor. 
 *      
 * This cell editor is planned to edit {@link ILongTextPropertyAccess} objects.
 *
 * @author		        Cedric Jeanneret
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             15.04.2008 CJ First Version (Fix Bug 855 and refactoring of long text property editors)
 * @version             $Date: 2008-10-27 10:12:16 +0100 (Mon, 27 Oct 2008) $, $Author: jehle $, $Revision: 3853 $
 */

public class LongTextPropertyCellEditor extends DialogCellEditor {
	
	/**
	 * Instantiates a {@link LongTextPropertyCellEditor}.
	 *
	 * @param parent Composite, for which this {@link LongTextPropertyCellEditor}
	 * is being instantiated.
	 */
    public LongTextPropertyCellEditor(Composite parent) {
       super(parent);
     }


    /* (non-Javadoc)
     * @see org.eclipse.jface.viewers.DialogCellEditor#openDialogBox(org.eclipse.swt.widgets.Control)
     */
    @Override
    protected Object openDialogBox(Control cellEditorWindow) {
    	try {
            AbstractLongTextEditorInput input = buildEditorInput();
            IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
            page.openEditor(input, input.getLongTextEditorID(), true);
        }
        catch (PartInitException e) {
            e.printStackTrace();
      }
      return null;
    }
	
	/**
	 * @return an Editor input for the current value
	 */
	protected AbstractLongTextEditorInput buildEditorInput() {
		if (getValue() instanceof IAdaptable) {
			IAdaptable adaptee = (IAdaptable) getValue();
			return (AbstractLongTextEditorInput) adaptee.getAdapter(AbstractLongTextEditorInput.class);
		}
		return null;
	}
    
    
    
 }