package ch.unizh.ifi.adora.properties.longtextproperty;

import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.views.properties.PropertyDescriptor;

/**
 * Implements a PropertyDescriptor for the long text property field.
 * The editor for property it describes is a {@link LongTextPropertyCellEditor}.
 *
 * @author		        Cedric Jeanneret
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             15.04.2008 CJ First Version (Fix Bug 855 and refactoring of long text property editors)
 * @version             $Date: 2008-10-27 10:12:16 +0100 (Mon, 27 Oct 2008) $, $Author: jehle $, $Revision: 3853 $
 */

public class LongTextPropertyDescriptor extends PropertyDescriptor {

    /**
     * The constructor for a long text property. 
     * Initializes the long text property
     * with an id and a displayable name.
     * 
     * @param id The id of the property.
     * @param displayName The displayable name of the property.
     */
	public LongTextPropertyDescriptor(Object id, String displayName) {
		super(id, displayName);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.PropertyDescriptor#createPropertyEditor(org.eclipse.swt.widgets.Composite)
	 */
	@Override
    public CellEditor createPropertyEditor(Composite parent) {
        CellEditor editor = new LongTextPropertyCellEditor(parent);
        
        if (getValidator() != null)
            editor.setValidator(getValidator());
        return editor;
    }
    
}