package ch.unizh.ifi.adora.properties.longtextproperty;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.editors.text.TextEditor;

import ch.unizh.ifi.adora.AdoraPlugin;

/**
 * This editor acts as the standard editor for long text properties. It gets
 * called as soon as a user presses the button in a {@link LongTextPropertyCellEditor}.
 * 
 * This class should be subclassed to set help context IDs and to
 * provide an unique Editor ID.
 * 
 * @author Cedric Jeanneret
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history   15.04.2008 CJ First Version (Fix Bug 855 and refactoring of long text property editors)
 * @version   $Date: 2008-10-27 10:12:16 +0100 (Mon, 27 Oct 2008) $, $Author: jehle $, $Revision: 3853 $
 */

public class LongTextPropertyEditor extends TextEditor {

	/**
     * Instantiates a LongTextPropertyEditor.
     */
	public LongTextPropertyEditor() {
		super();
		setDocumentProvider(new LongTextPropertyDocumentProvider());
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.IEditorPart#init(org.eclipse.ui.IEditorSite,
     *      org.eclipse.ui.IEditorInput)
     */
	/**
	 * We override this method to make sure that the input is an
	 * {@link AbstractLongTextEditorInput}.
	 */
	@Override
	public void init(IEditorSite site, IEditorInput input)
			throws PartInitException {
		if(input instanceof ILongTextEditorInput)
		{
			super.init(site, input);
		}
		else throw new PartInitException(new Status(IStatus.ERROR,AdoraPlugin.PLUGIN_ID, IStatus.OK, "Input cannot be understood.",null));
	}


}
