package ch.unizh.ifi.adora.properties.specification;

import org.eclipse.ui.views.properties.IPropertySource;

import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.properties.AspectContainerProperties;
import ch.unizh.ifi.adora.properties.longtextproperty.AbstractLongTextPropertyAccess;
import ch.unizh.ifi.adora.properties.longtextproperty.ILongTextPropertyAccess;

/**
 * {@link AspectContainerSpecificationPropertyAccess} objects are 
 * {@link ILongTextPropertyAccess} for the specification property
 * of {@link AspectContainer} model elements
 * 
 * @author				Cedric Jeanneret
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				15.04.2008 CJ First Version (Fix Bug 855 and refactoring of long text property editors)
 * @version				$Date: 2008-10-27 10:12:16 +0100 (Mon, 27 Oct 2008) $, $Author: jehle $, $Revision: 3853 $
 */

public class AspectContainerSpecificationPropertyAccess extends AbstractLongTextPropertyAccess implements ExtendedSpecificationPropNames {
	


	/**
	 * @param modelElement
	 */
	public AspectContainerSpecificationPropertyAccess(AspectContainer modelElement) {
		super(SPECIFICATION_PROP, SPECIFICATION_PROP_NAME, modelElement, new AspectContainerProperties(modelElement));
	}

	/**
	 * @param modelElement
	 * @param propertySource
	 */
	public AspectContainerSpecificationPropertyAccess(AspectContainer modelElement,
			IPropertySource propertySource) {
		super(SPECIFICATION_PROP, SPECIFICATION_PROP_NAME, modelElement, propertySource);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	@SuppressWarnings("unchecked")
	public Object getAdapter(Class adapter) {
		if(adapter.isAssignableFrom(SpecificationEditorInput.class))
		{
			return new SpecificationEditorInput(this);
		}
		return null;
	}


	
	

}
