package ch.unizh.ifi.adora.properties.specification.scanner;

import org.eclipse.jface.text.TextAttribute;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.Display;

import ch.unizh.ifi.adora.model.parser.AdoraParserConstants;

public class FSLSpecificationScanner extends JavaCCSpecificationScanner implements AdoraParserConstants {
    private static Color KEYWORD_COLOR =
        new Color(Display.getCurrent(), new RGB(0, 0, 200));
    
    private static Color COMMENT_COLOR =
        new Color(Display.getCurrent(), new RGB(0, 200, 0));

    private static Color LITERAL_COLOR =
        new Color(Display.getCurrent(), new RGB(200, 0, 0));

    
    private static Color DEFAULT_COLOR =
        new Color(Display.getCurrent(), new RGB(0, 0, 0));

    private static SpecificationToken commentToken = new SpecificationToken(new TextAttribute(COMMENT_COLOR));
    
    private static SpecificationToken defaultToken = new SpecificationToken(new TextAttribute(DEFAULT_COLOR));
    
    private static SpecificationToken keywordToken = new SpecificationToken(new TextAttribute(KEYWORD_COLOR));

    private static SpecificationToken literalToken = new SpecificationToken(new TextAttribute(LITERAL_COLOR));

    
    public FSLSpecificationScanner() {
        this.setDefaultReturnToken(defaultToken);
        this.setTokenMappingTupel(LOCAL, EXISTS, keywordToken);
        this.setTokenMappingTupel(CONDITIONALOR, keywordToken);
        this.setTokenMappingTupel(CONDITIONALAND, keywordToken);
        this.setTokenMappingTupel(IN, keywordToken);
        this.setTokenMappingTupel(RECEIVE, CONSTRAINTS_UNIQUE, keywordToken);
//        this.setTokenMappingTupel(BOOLEAN_LITERAL,CARDINALITY_LETTER_LITERAL, literalToken);
        this.setTokenMappingTupel(LETTER, literalToken);
        this.setTokenMappingTupel(DIGIT, literalToken);
        this.setTokenMappingTupel(COMMENT, commentToken);
        this.setTokenMappingTupel(INFORMAL_COMMENT, commentToken);
        
        
    

        /*
        int EOF = 0;
        int LOCAL = 5;
        int EXTENSION = 6;
        int OPERATION = 7;
        int INVARIANT = 8;
        int SYNCOPERATION = 9;
        int VARIABLE = 10;
        int PROVIDES = 11;
        int REQUIRES = 12;
        int PRECONDITION = 13;
        int POSTCONDITION = 14;
        int BEGIN_STATEMENTS = 15;
        int IN_PARAM = 16;
        int OUT_PARAM = 17;
        int DATA = 18;
        int LIST = 19;
        int OF = 20;
        int STRUCTURE = 21;
        int INHERITS = 22;
        int STEREOTYPE = 23;
        int DESCRIPTION = 24;
        int CONDITION = 25;
        int OBJECT = 26;
        int COMPONENT = 27;
        int PARTIAL = 28;
        int END = 29;
        int SPECIFICATION = 30;
        int MODEL = 31;
        int SET = 32;
        int ATTRIBUTE = 33;
        int TYPE = 34;
        int PUBLIC = 35;
        int IS = 36;
        int ELEMENT = 37;
        int LINEELEMENT = 38;
        int ENVIRONMENT = 39;
        int SCENARIO = 40;
        int ASSOCIATION = 41;
        int ROLE = 42;
        int SCENARIOCONNECTION = 43;
        int TRANSITION = 44;
        int CONNECTIONS = 45;
        int EXTERNAL = 46;
        int STATE = 47;
        int CONSISTS = 48;
        int ORTHOGONAL = 49;
        int START = 50;
        int REPRESENTATION = 51;
        int POSITION = 52;
        int SIZE = 53;
        int MINIMUM = 54;
        int MANUALLY = 55;
        int HIDDEN = 56;
        int STORED = 57;
        int VISIBLE = 58;
        int ZOOMED = 59;
        int ASPECT = 60;
        int BEFORE = 61;
        int AFTER = 62;
        int INSTEAD = 63;
        int JOINRELATION = 64;
        int JOIN = 65;
        int EXIT = 66;
        int DATATYPEDEF = 67;
        int STRING = 68;
        int INTEGER = 69;
        int TIME = 70;
        int CHAR = 71;
        int BOOLEAN = 72;
        int FLOAT = 73;
        int ID = 74;
        int ALTERNATIVE = 75;
        int SEQUENCE = 76;
        int ITERATION = 77;
        int ROOT = 78;
        int PARALLEL = 79;
        int COMPOSITION_OF = 80;
        int COMPONENTS_OF = 81;
        int IS_ACTIVE_STATE = 82;
        int FIND_OBJECTS = 83;
        int MAX_OBJECT_NUMBER = 84;
        int MIN_OBJECT_NUMBER = 85;
        int ACT_OBJECT_NUMBER = 86;
        int CREATE = 87;
        int DISPOSE = 88;
        int SYSTEMTIME = 89;
        int FOR = 90;
        int ALL = 91;
        int EXISTS = 92;
        int IMPLICATION = 93;
        int EQUIVALENCE = 94;
        int CONDITIONALOR = 95;
        int CONDITIONALAND = 96;
        int NEGATION = 97;
        int SMALLER_THAN = 98;
        int GREATER_THAN = 99;
        int GREATER_EQUALS = 100;
        int SMALLER_EQUALS = 101;
        int UNEQUALS = 102;
        int EQUALS = 103;
        int ASSIGNMENT = 104;
        int IN = 105;
        int PLUS = 106;
        int MINUS = 107;
        int DIVIDE = 108;
        int MULTIPLY = 109;
        int RECEIVE = 110;
        int FROM = 111;
        int HIGH_PRIO = 112;
        int DEFAULT_PRIO = 113;
        int LOW_PRIO = 114;
        int SEND = 115;
        int MULTICAST = 116;
        int OVER = 117;
        int TO = 118;
        int CALL = 119;
        int SIMULATION_START = 120;
        int STATE_ENTERED = 121;
        int OBJECT_INITIALIZED = 122;
        int STEPS = 123;
        int SECS = 124;
        int DO = 125;
        int TABLE = 126;
        int ON = 127;
        int TRANSFORM = 128;
        int OUTPUT = 129;
        int INPUT = 130;
        int PROPERTYDEF = 131;
        int PROPERTY = 132;
        int NUMBERED = 133;
        int CONSTRAINTS_UNIQUE = 134;
        int BOOLEAN_LITERAL = 135;
        int STRING_LITERAL = 136;
        int REFERENCE_STRING_LITERAL = 137;
        int INTEGER_LITERAL = 138;
        int DECIMAL_LITERAL = 139;
        int HEX_LITERAL = 140;
        int OCTAL_LITERAL = 141;
        int FLOATING_POINT_LITERAL = 142;
        int EXPONENT = 143;
        int TIME_LITERAL = 144;
        int DATE_LITERAL = 145;
        int CARDINALITY_LETTER_LITERAL = 146;
        int IDENTIFIER = 147;
        int LETTER = 148;
        int DIGIT = 149;
        int PRECON = 150;
        int LPAREN = 151;
        int RPAREN = 152;
        int SEMICOLON = 153;
        int COLON = 154;
        int COMMA = 155;
        int DOT = 156;
        int LBRACKET = 157;
        int RBRACKET = 158;
        int LBRACE = 159;
        int RBRACE = 160;
        int MSG_SEPARATOR = 161;
        int COMMENT = 162;
        int COMMENT_STATE = 163;
        int ANNOTATION = 165;
        int ANNOTATION_STATE = 166;
        int ANNOTATION_SINGLE = 168;
        int ANNOTATION_SINGLE_STATE = 169;

        int DEFAULT = 0;
        int IN_COMMENT_STATE = 1;
        int IN_ANNOTATION_STATE = 2;
        int IN_ANNOTATION_SINGLE_STATE = 3;
        */
    }    
}
