package ch.unizh.ifi.adora.properties.specification.scanner;

import org.eclipse.jface.text.rules.IToken;

public class SpecificationToken implements IToken {

    /** Internal token type: Undefined */
    private static final int T_UNDEFINED= 0;
    /** Internal token type: EOF */
    private static final int T_EOF= 1;
    /** Internal token type: Whitespace */
    private static final int T_WHITESPACE= 2;
    /** Internal token type: Others */
    private static final int T_OTHER=   3;

    /** The type of this token */
    private int fType;
    /** The data associated with this token */
    private Object fData;

    
    /**
     * Standard token: Undefined.
     */
    public static final IToken UNDEFINED= new SpecificationToken(T_UNDEFINED);
    /**
     * Standard token: End Of File.
     */
    public static final IToken EOF= new SpecificationToken(T_EOF);
    /**
     * Standard token: Whitespace.
     */
    public static final IToken WHITESPACE= new SpecificationToken(T_WHITESPACE);

    
    public SpecificationToken(int type) {
       fType = type;
       fData = null;
    }
       
    public SpecificationToken(Object data) {
        fData = data;
     
        fType= T_OTHER;
    }
    
    /*
     * @see IToken#getData()
     */
    public Object getData() {
        return fData;
    }

    /*
     * @see IToken#isOther()
     */
    public boolean isOther() {
        return (fType == T_OTHER);
    }

    /*
     * @see IToken#isEOF()
     */
    public boolean isEOF() {
        return (fType == T_EOF);
    }

    /*
     * @see IToken#isWhitespace()
     */
    public boolean isWhitespace() {
        return (fType == T_WHITESPACE);
    }

    /*
     * @see IToken#isUndefined()
     */
    public boolean isUndefined() {
        return (fType == T_UNDEFINED);
    }

}
