package ch.unizh.ifi.adora.properties.validators.cardinality;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.jface.viewers.ICellEditorValidator;

import ch.unizh.ifi.adora.model.representation.Cardinality;

/**
 * Responsible that only numerical values and 
 * at most one letter are typed into a property field.     
 *
 * @author              Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             20.04.2006 SM First Version
 * @version             $Date: 2007-01-25 18:06:03 +0100 (Thu, 25 Jan 2007) $, $Author: smeier $, $Revision: 2403 $
 */
public abstract class CardinalityValidator  implements ICellEditorValidator {

	/**
	 * Stores the other value of the cardinality.
	 */
	protected Cardinality card; 
	
	/**
	 * Constructor
	 * 
	 * @param card The Cardinality.
	 */
	public CardinalityValidator(Cardinality card){
		this.card = card;
	}
    
	/**
     * Checks the newly entered cardinality.
     *
     * @param card The newly entered cardinality to check.
     * 
     * @return true</code> if the cardinality can be set.
     */
    protected abstract String checkCard(String card); 

	/**
     * Checks, if a cast between sting and cardinality can be performed.
     */
    public String isValid(Object value) {
        boolean b =false;
        Pattern pNum = Pattern.compile("\\d*");
        Pattern pAlpha = Pattern.compile("[a-zA-Z]");
        
        Matcher mNum = pNum.matcher((String) value);
        Matcher mAlpha = pAlpha.matcher((String) value);
        b = mNum.matches() || mAlpha.matches();
        
        if (!b) {
        	return new String("Only alphanumeric values are allowed.");
        } 
        return checkCard((String)value);
    }
	
    /**
     * Return <code>true</code> if the String represents an integer value. 
     * 
     * @param s String to be checked
     * 
     * @return true</code> if the String represents an integer value.
     */
    protected boolean isInteger(String s){
    	try {
    		Integer.parseInt(s);
    	} catch (NumberFormatException e) {
    		return false;
    	}
    	return true;
    }
}