package ch.unizh.ifi.adora.properties.validators.cardinality;

import ch.unizh.ifi.adora.model.representation.Cardinality;
import ch.unizh.ifi.adora.ui.constrainthint.IPropertyConstraintMessages;

/**
 * Assures that the newly entered cardinality isn't smaller
 * than the minimum cardinality.
 *      
 * @author		        Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             28.09.2006 IV First Version
 * @version             $Date: 2007-01-25 18:06:03 +0100 (Thu, 25 Jan 2007) $, $Author: smeier $, $Revision: 2403 $
 */
public class MaxCardinalityValidator extends CardinalityValidator{
	
	/**
	 * Constructor
	 * 
	 * @param card The Cardinality.
	 */
	public MaxCardinalityValidator(Cardinality card){
		super(card);
	}
    
	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.properties.validators.CardinalityValidator#checkCard(java.lang.String)
	 */
	protected String checkCard(String maxCard) {
    	if (isInteger(card.getMinCard())) {
    		if (isInteger(maxCard)){
    			if (Integer.parseInt(maxCard) < 
    					Integer.parseInt(card.getMinCard())) {
    				return IPropertyConstraintMessages.MAX_CARD;
    			}
    		}
    	} else {
    		if (isInteger(maxCard)){
    			// variable > int
				return IPropertyConstraintMessages.MAX_CARD;
    		}
    	}
    	return null;
    }
}