package ch.unizh.ifi.adora.properties.validators.cardinality;

import ch.unizh.ifi.adora.model.representation.Cardinality;
import ch.unizh.ifi.adora.ui.constrainthint.IPropertyConstraintMessages;

/**
* Assures that the newly entered cardinality isn't bigger
* than the maximum cardinality.
*      
* @author		        Ivo Vigan
* @copyright           Requirements Engineering Research Group<br>
*                      Department of Informatics<br>
*                      University of Zurich<br>
* @history             28.09.2006 IV First Version
* @version             $Date: 2007-01-25 18:06:03 +0100 (Thu, 25 Jan 2007) $, $Author: smeier $, $Revision: 2403 $
*/

public class MinCardinalityValidator extends CardinalityValidator{

	/**
	 * Constructor
	 * 
	 * @param card The Cardinality.
	 */
	public MinCardinalityValidator(Cardinality card){
		super(card);
	}
	
	/**
     * Checks the minimal cardinality.
     *
     * @param minCard The minCard to check.
     * 
     * @return true</code> if the cardinality can be set.
     */
    protected String checkCard(String minCard) {
    	if (isInteger(card.getMaxCard())) {
    		if (isInteger(minCard)) {
    			if (Integer.parseInt(minCard) > 
    					Integer.parseInt(card.getMaxCard())) {
    				return IPropertyConstraintMessages.MIN_CARD;
    			}
    		} else {
    			// variable > int
				return IPropertyConstraintMessages.MIN_CARD;
    		}
    	}
    	return null;
    }
}