package ch.unizh.ifi.adora.properties.validators.component;

import org.eclipse.jface.viewers.ICellEditorValidator;

import ch.unizh.ifi.adora.model.representation.Actor;
import ch.unizh.ifi.adora.model.representation.Association;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.ui.constrainthint.IPropertyConstraintMessages;

/**
 * Validates the external attribute.
 * 
 * @author		        Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             28.09.2006 IV First Version
 * @version             $Date: 2007-05-21 10:48:21 +0200 (Mon, 21 May 2007) $, $Author: schoeni $, $Revision: 2699 $
 */
public class ExternalValidator implements ICellEditorValidator {

	/**
	 * Stores the modelelement.
	 */
	private Component component; 
	
	/**
	 * Constructor
	 * 
	 * @param otherCard The modelelement.
	 */
	public ExternalValidator(Component component){
		this.component = component;
	}
	
	/*  (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ICellEditorValidator#isValid(java.lang.Object)
	 */
	public String isValid(Object value) {
		boolean on = ((Integer)value).intValue() > 0; 
		
		if (on) {
			if (component.getType() != null)
				return IPropertyConstraintMessages.EXTERNAL_TYPE;
			if (component.getChildCount() > 0) 
				return IPropertyConstraintMessages.EXTERNAL_CHILD;
			if (component.isPartial())
				return IPropertyConstraintMessages.EXTERNAL_PARTIAL;
			if (component.isStartState())
				return IPropertyConstraintMessages.EXTERNAL_STARTSTATE;
		} else{		
			for (Connection c : component.getConnections()) {
				if (c instanceof Association) {
					ConnectableElement otherEnd = c.getSource() == component ? c.getTarget() : c.getSource();
					if (otherEnd instanceof Actor || otherEnd instanceof Scenario) {
						return IPropertyConstraintMessages.EXTERNAL_ASSO;
					}
				}
			}
		}
		return null;
	}
}