/**
 * 
 */
package ch.unizh.ifi.adora.representation.routing.algorithm;

import org.eclipse.draw2d.ChopboxAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;

public class CornerAnchor extends ChopboxAnchor {
	boolean isStart = true;
	Point refCorner = null;
	
	public CornerAnchor(IFigure source, boolean isStart) {
		super(source);
		this.isStart = isStart;
	}

	public Point getLocation(Point reference) {
		Point loc = super.getLocation(reference);
		Point corner = getCorner(loc);
		Rectangle r = getOwner().getBounds().getCopy();
		getOwner().translateToAbsolute(corner);
		refCorner = corner.getCopy();
		
		int yOffset = r.height / 3;
		int xOffset = r.width / 3;
		if (isStart) {	
			if (corner.y == r.y)
				corner.translate(0, yOffset);
			else corner.translate(0, -yOffset);					
		}
		else {
			if (corner.x == r.x)
				corner.translate(xOffset, 0);
			else corner.translate(-xOffset, 0);
		}
		return corner;
	}	
	
	private Point getCorner(Point loc) {
		Rectangle r = getOwner().getBounds().getCopy();
		getOwner().translateToAbsolute(r);
		float centerX = r.x + 0.5f * r.width;
		float centerY = r.y + 0.5f * r.height;
		
		int quadrant = 1;
		if (loc.x <= centerX && loc.y <= centerY) quadrant = 1;
		if (loc.x > centerX && loc.y < centerY) quadrant = 2;
		if (loc.x < centerX && loc.y > centerY) quadrant = 3;
		if (loc.x > centerX && loc.y > centerY) quadrant = 4;
		
		switch(quadrant) {
			case 1: 
				return r.getTopLeft();
			case 2: 
				return r.getTopRight();
			case 3: 
				return r.getBottomLeft();
			case 4: 
				return r.getBottomRight();
		}
		return r.getTopLeft();
	}
	
	public Point getRefCorner() {
		return refCorner;
	}
	
}