package ch.unizh.ifi.adora.representation.routing.algorithm;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.draw2d.AbstractRouter;
import org.eclipse.draw2d.BendpointConnectionRouter;
import org.eclipse.draw2d.Connection;
import org.eclipse.draw2d.FigureListener;
import org.eclipse.draw2d.FreeformLayeredPane;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Layer;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.linerouting.algorithm.RoutingConstants;
import ch.unizh.ifi.adora.linerouting.interfaces.IRoutingLine;
import ch.unizh.ifi.adora.linerouting.interfaces.IRoutingNode;
import ch.unizh.ifi.adora.ui.preferences.IPreferenceNameConstants;
import ch.unizh.ifi.adora.view.layers.LabelPlacementVisualizationLayer;
import ch.unizh.ifi.adora.view.layers.RoutingVisualizationLayer;

public class OrthogonalRouter extends LayoutObservingRouter implements IPropertyChangeListener {

	private Layer connectionLayer;

	private AbstractRouter temporaryRouter;
	
	/** 
	 * Delegate used to implement property-change-support. 
	 */
	private transient PropertyChangeSupport pcsDelegate;
	
	private IPreferenceStore store;
	

	public OrthogonalRouter(IFigure root, Layer connectionLayer) {	

		super(root, connectionLayer);
		
		pcsDelegate = new PropertyChangeSupport(this);
		
		this.connectionLayer = connectionLayer;
		temporaryRouter = new BendpointConnectionRouter();
		
		store = AdoraPlugin.getDefault().getPreferenceStore();
		store.addPropertyChangeListener(this);
		
    	RoutingConstants.ROUTING_NODE_COSTFACTOR = store.getFloat(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_NODE_COSTFACTOR);
    	RoutingConstants.ROUTING_ABSTRACT_NODE_COSTFACTOR = store.getFloat(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_ABSTRACT_NODE_COSTFACTOR);
    	RoutingConstants.ROUTING_LINE_COSTFACTOR = store.getFloat(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_LINE_COSTFACTOR);
    	RoutingConstants.SHOW_ROUTING_SOLUTIONS = store.getBoolean(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_PRINT_SOLUTIONS);
    	RoutingConstants.PLACE_LABELS = store.getBoolean(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_PLACE_LABELS);
		
		// TODO: This should be the other way round. The layer registers
		// itself as listeners
		FreeformLayeredPane layeredPane = (FreeformLayeredPane)connectionLayer.getParent();
		RoutingVisualizationLayer routingLayer = (RoutingVisualizationLayer)layeredPane.getLayer("ROUTING_VISUALIZATION_LAYER");
		LabelPlacementVisualizationLayer labelLayer = (LabelPlacementVisualizationLayer)layeredPane.getLayer("LABEL_PLACEMENT_VISUALIZATION_LAYER");
		addPropertyChangeListener(routingLayer);
		addPropertyChangeListener(labelLayer);

	}

	public void route(Connection line) {
		
		ch.unizh.ifi.adora.linerouting.algorithm.OrthogonalTileMazeRouter otmr =
			ch.unizh.ifi.adora.linerouting.algorithm.OrthogonalTileMazeRouter.getInstance();
		
		// Don't route the line if either the source or the target of the
		// line is null (during the creation of a new connection)
		if (line.getSourceAnchor().getOwner() == null
				|| line.getTargetAnchor().getOwner() == null) {
			return;
		}
		
		
		// Don't route the line if it connects a node with one of its ancestors
		IFigure parent = line.getSourceAnchor().getOwner();
		while (parent != null) {
			if (parent == line.getTargetAnchor().getOwner()
					&& line.getSourceAnchor().getOwner() != line
							.getTargetAnchor().getOwner()) {
				temporaryRouter.route(line);
				return;
			}
			parent = parent.getParent();
		}
		
		
		// Don't route the line if it connects a node with one of its ancestors
        parent = line.getTargetAnchor().getOwner();
        if(parent instanceof Connection) {
        		parent = ((Connection)parent).getTargetAnchor().getOwner();
        }
        
        while (parent != null) {
            if (parent == line.getSourceAnchor().getOwner()
                    && line.getTargetAnchor().getOwner() != line
                            .getSourceAnchor().getOwner()) {
                temporaryRouter.route(line);
                return;
            }
            parent = parent.getParent();
        }
        
        if(!(line.getSourceAnchor().getOwner() instanceof IRoutingNode)
        		|| !(line.getTargetAnchor().getOwner() instanceof IRoutingNode)) {
        	
        	temporaryRouter.route(line);
            return;
        	
        }
        
        
        // Don't route the line if the start and end point of a reflexive line
        // are too close together, because we can't split the node if the two
        // points are too close.
        if(((IRoutingLine)line).getRoutingSource() == ((IRoutingLine)line).getRoutingTarget() &&
        		(Math.abs(((IRoutingLine)line).getRoutingStartPoint().x - ((IRoutingLine)line).getRoutingEndPoint().x) < 2
        		&& Math.abs(((IRoutingLine)line).getRoutingStartPoint().y - ((IRoutingLine)line).getRoutingEndPoint().y) < 2)) {
        	
        	temporaryRouter.route(line);
            return;
        	
        }

        
        if(line.getTargetAnchor().getOwner() instanceof Connection) {
        	
        	Connection target = (Connection)line.getTargetAnchor().getOwner();
        	
        	// Don't route the line if its target (which is itself a line)
        	// has not yet been routed.
        	if(target.getBounds().x == 0 && target.getBounds().y == 0) {
        		return;
        	}

        }
        
        // Do the actual routing
		if(line instanceof IRoutingLine) {
			
			List<IRoutingLine> lines = new ArrayList<IRoutingLine>();
        	
        	Iterator<IFigure> children = connectionLayer.getChildren().iterator();
        	while(children.hasNext()) {
        		IFigure child = children.next();
        		if(child == line) {
        			break;
        		}
        		if(child instanceof IRoutingLine && ((IRoutingLine)child).getRoutingTarget() != line) {
        			lines.add((IRoutingLine)child);
        		}
        	}
        	
        	if(line.getTargetAnchor().getOwner() instanceof Connection
        			&& !lines.contains(line.getTargetAnchor().getOwner())) {
        		
        		lines.add((IRoutingLine)line.getTargetAnchor().getOwner());
        		
        	}
        	
			otmr.route((IRoutingLine)line, lines);
			
			this.firePropertyChange(IRoutingEventConstants.TILESTRUCTURE, null, null);
		}
		
		
	}
	
	
	void removeChild(IFigure child) {
		child.removeFigureListener(figureListener);
	}

	
	void addChild(IFigure child) {
		child.addFigureListener(figureListener);
	}


	private FigureListener figureListener = new FigureListener() {
		public void figureMoved(IFigure source) {
			Iterator iter = connectionLayer.getChildren().iterator();
			while (iter.hasNext()) {
				Connection line = (Connection) iter.next();
				route(line);
			}

		}
	};
	
	
	/**
	 * Attach a non-null PropertyChangeListener to this object.
	 * 
	 * @param l
	 *            a non-null PropertyChangeListener instance
	 * @throws IllegalArgumentException
	 *             if the parameter is null
	 */
	public synchronized void addPropertyChangeListener(PropertyChangeListener l) {
		if (l == null) {
			throw new IllegalArgumentException();
		}
		pcsDelegate.addPropertyChangeListener(l);
	}
	
	
	/**
	 * Attach a non-null PropertyChangeListener to this object that listens only to a given property
	 * 
	 * @param l a non-null PropertyChangeListener instance
	 * @param property the property that the listener shall be informed about
	 * @throws IllegalArgumentException
	 *             if the parameter is null
	 */
	public synchronized void addPropertyChangeListener(String property,PropertyChangeListener l) {
		if (l == null) {
			throw new IllegalArgumentException();
		}
		pcsDelegate.addPropertyChangeListener(property,l);
	}

	/**
 	 * 
	 * Remove a PropertyChangeListener from this component.
	 * 
	 * @param l A PropertyChangeListener instance
	 */
	public synchronized void removePropertyChangeListener(
			PropertyChangeListener l) {
		if (l != null) {
			pcsDelegate.removePropertyChangeListener(l);
		}
	}
	

	/**
	 * Children should override this. The default implementation does nothing.
	 */
	public void resetPropertyValue(Object id) {
		// do nothing
	}

	
	/**
	 * Report a property change to registered listeners (for example edit
	 * parts).
	 * 
	 * @param property The programmatic name of the property that changed
	 * @param oldValue The old value of this property
	 * @param newValue The new value of this property
	 */
	protected void firePropertyChange(String property, Object oldValue,
			Object newValue) {
		if (pcsDelegate.hasListeners(property)) {
			pcsDelegate.firePropertyChange(property, oldValue, newValue);
		}
	}

	public void propertyChange(PropertyChangeEvent event) {
		
		String eventType = event.getProperty();
        if(eventType.equals(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_NODE_COSTFACTOR)) {
        	RoutingConstants.ROUTING_NODE_COSTFACTOR = store.getFloat(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_NODE_COSTFACTOR);
        	connectionLayer.invalidateTree();
        	connectionLayer.revalidate();
        }
        else if(eventType.equals(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_ABSTRACT_NODE_COSTFACTOR)) {
        	RoutingConstants.ROUTING_ABSTRACT_NODE_COSTFACTOR = store.getFloat(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_ABSTRACT_NODE_COSTFACTOR);
        	connectionLayer.invalidateTree();
        	connectionLayer.revalidate();
        }
        else if(eventType.equals(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_LINE_COSTFACTOR)) {
        	RoutingConstants.ROUTING_LINE_COSTFACTOR = store.getFloat(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_LINE_COSTFACTOR);
        	connectionLayer.invalidateTree();
        	connectionLayer.revalidate();
        }
        else if(eventType.equals(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_PRINT_SOLUTIONS)) {
        	RoutingConstants.SHOW_ROUTING_SOLUTIONS = store.getBoolean(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_PRINT_SOLUTIONS);
        	connectionLayer.invalidate();
        }
		
	}
	

}
