package ch.unizh.ifi.adora.representation.routing.algorithm.directline;

import org.eclipse.draw2d.Connection;
import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Layer;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PointList;

import ch.unizh.ifi.adora.representation.routing.algorithm.LayoutObservingRouter;
import ch.unizh.ifi.adora.view.layers.MultipleRoutersConnectionLayer;

/**
 * Routes a connection in the shortest possible way.
 * The resulting distance of the connection always equals
 * the beeline.      
 *
 * @author		        Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             03.08.2006 IV First Version
 * @version             $Date: 2008-04-04 12:01:27 +0200 (Fri, 04 Apr 2008) $, $Author: treinhard $, $Revision: 3583 $
 */
public class DirectLineRouter extends LayoutObservingRouter {
    
    /**
     * Constructor for the tile maze router. Registers the layout listener at all
     * existing figures in the model.
     * 
     * @pre root != null && connectionLayer != null
     * @post true
     * 
     * @param root the root element in the model
     * @param connectionLayer the layer that conatins all the connections
     */
    public DirectLineRouter(IFigure root, Layer connectionLayer) {
  		
    		super(root, connectionLayer);
    }

    /*
	 *  (non-Javadoc)
	 * @see org.eclipse.draw2d.ConnectionRouter#route(org.eclipse.draw2d.Connection)
	 */
	public void route(Connection connection) {
		
		if(connection.getSourceAnchor().getOwner() ==
			connection.getTargetAnchor().getOwner()){
			
			((MultipleRoutersConnectionLayer)connectionLayer).
				getSpecificRouter(MultipleRoutersConnectionLayer.OrthogonalRouter).
					route(connection);
			return;
		}
		
		PointList points = new PointList();
				
		ConnectionAnchor sourceAnchor = connection.getSourceAnchor();
		ConnectionAnchor targetAnchor = connection.getTargetAnchor();
			
		Point start = sourceAnchor.getLocation(targetAnchor.getReferencePoint());
		Point end = targetAnchor.getLocation(sourceAnchor.getReferencePoint());
	
		connection.translateToRelative(start);
		connection.translateToRelative(end);
		
		points.addPoint(start);
		points.addPoint(end);

		connection.setPoints(points);
	}
    
 }
