package ch.unizh.ifi.adora.tools;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.Request;
import org.eclipse.gef.requests.DirectEditRequest;
import org.eclipse.gef.tools.CreationTool;

/**
 * This class extends {@link org.eclipse.gef.tools.CreationTool}
 * in order to send a DirectEdit request to the freshly created Editpart.
 * It is a solution for the bug ID 798 (direct edit on creation)
 *
 * @author		        Cdric Jeanneret
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             08.04.2008 CJ First Version
 * @version             $Date: 2008-04-08 11:25:54 +0200 (Tue, 08 Apr 2008) $, $Author: jeanneret $, $Revision: 3587 $
 */
public class DirectEditNodeCreationTool extends CreationTool {

	/**
	 * Override the performCreation method such that a direct edit
	 * request is sent to the freshly created object
	 * 
	 * @see org.eclipse.gef.tools.CreationTool#performCreation(int)
	 */
	@Override
	protected void performCreation(int button) {
		super.performCreation(button);
		sendDirectEditRequest();
	}

	/**
	 * Find the created Edit Part and send it a Direct Edit
	 * request (if it understands it).
	 * 
	 * @pre a model object has been created
	 * @post a direct edit request is sent to its corresponding edit part
	 */
	protected void sendDirectEditRequest()
	{
		final Object model = getCreateRequest().getNewObject();
		EditPartViewer viewer = getCurrentViewer();
		if (model == null || viewer == null)
			return;
		Object control = viewer.getEditPartRegistry().get(model);
		if (control instanceof EditPart) {
			EditPart editpart = (EditPart) control;
			Request directEditRequest = new DirectEditRequest();
			if(editpart.understandsRequest(directEditRequest))
			{
				editpart.performRequest(directEditRequest);
			}
		}
	}

}
