package ch.unizh.ifi.adora.tools;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.SharedCursors;
import org.eclipse.gef.tools.TargetingTool;

import ch.unizh.ifi.adora.utils.AdoraCursors;
import ch.unizh.ifi.adora.utils.RequestConstants;

/**
 * Tool to perform fisheye zooming.
 * 
 * @author    Adrian Gygax
 * 			  Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 07.12.05 AG First version
 * @version $Date: 2008-04-11 11:35:51 +0200 (Fri, 11 Apr 2008) $, $Author: treinhard $, $Revision: 3597 $
 * @see ch.unizh.ifi.adora.model.commands.FisheyeZoom
 * @see ch.unizh.ifi.adora.parts.policies.FisheyeZoomEditPolicy
 */
public class FisheyeZoomTool extends TargetingTool {

	
	/**
	 * Creates a new instance with default cursors.
	 */
	public FisheyeZoomTool() {
		setDefaultCursor(AdoraCursors.CURSOR_MAGNIFY);
		setDisabledCursor(SharedCursors.NO);

	}
    
	/* (non-Javadoc)
	 * @see org.eclipse.gef.tools.AbstractTool#getCommandName()
	 */
	protected String getCommandName() {
		return RequestConstants.REQ_FISHEYEZOOM;
	}
	
	
	/**
	 * The zoom tool only works by clicking mouse button 1 (the left mouse
	 * button in a right-handed world).
	 */
	protected boolean handleButtonDown(int button) {
		if (getCurrentViewer() != null && button == 1) {
			executeCurrentCommand();
	
			updateTargetUnderMouse();
			EditPart editpart = getTargetEditPart();
			if (editpart != null) {
				return true;
			}
		}
		return false;
	}
	/**
	 * Tracks the node under the mouse cursor so the cursor can be switched and the current EditPart 
	 * can be marked as selected, to show that the zoom operation can be performed.
	 * 
	 * @return <li>true</li>.
	 */
	protected boolean handleMove() {

		updateTargetUnderMouse();
		setCurrentCommand(getCommand());
		
		return true;
	}

}