package ch.unizh.ifi.adora.tools;

import org.eclipse.gef.palette.ToolEntry;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.jface.resource.ImageDescriptor;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.ui.preferences.IPreferenceNameConstants;

/**
 * ToolEntry for {@link ch.unizh.ifi.adora.tools.FisheyeZoomTool}.
 * 
 * @author    smeier
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history   20.09.2005 SM First Version
 *            07.12.2005 AG Changing name of returned tool
 *            
 * @version $Date: 2008-11-27 11:56:32 +0100 (Thu, 27 Nov 2008) $, $Author: treinhard $, $Revision: 3898 $
 */
public final class FisheyeZoomToolEntry extends ToolEntry implements IPropertyChangeListener {

	/**
	 * Stores the small icon for the FisheyeZoomTool.
	 */
	private static final ImageDescriptor smallIcon = ImageDescriptor
			.createFromFile(AdoraPlugin.class, "icons/magnifyingglass16.gif");

	/**
	 * Stores the large icon for the FisheyeZoomTool.
	 */
	private static final ImageDescriptor largeIcon = ImageDescriptor
			.createFromFile(AdoraPlugin.class, "icons/magnifyingglass24.gif");

	/**
	 * Instantiates a new FisheyeZoomToolEntry.
	 */
	public FisheyeZoomToolEntry() {
		super("Fisheye Zoom", "Fisheye zooming", smallIcon, largeIcon,
				FisheyeZoomTool.class);
		setUserModificationPermission(PERMISSION_NO_MODIFICATION);
		
		IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
		store.addPropertyChangeListener(this);
		setVisible(!store.getBoolean(IPreferenceNameConstants.ADORA_REPRESENTATION_FISHEYE_DISABLED));
	}

	
	/**
	 * Hides the tool entry if the fisheye zoom is disabled in the preferences.
	 */
	public void propertyChange(PropertyChangeEvent event) {
		
		if(event.getProperty().equals(IPreferenceNameConstants.ADORA_REPRESENTATION_FISHEYE_DISABLED)) {
			
			IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
			setVisible(!store.getBoolean(IPreferenceNameConstants.ADORA_REPRESENTATION_FISHEYE_DISABLED));
			
		}
		
	}
	
	
}