package ch.unizh.ifi.adora.tools;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.tools.ConnectionCreationTool;

import ch.unizh.ifi.adora.model.commands.ConnectionCreate;

/**
 * Extends the standard GEF ConnectionCreationTool so that a connection is
 * automatically selected after it has been added.
 *
 * @author				Tobias Reinhard
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				2007-10-24 TR First Version
 * @version				$Date: 2007-10-26 17:11:49 +0200 (Fri, 26 Oct 2007) $, $Author: treinhard $, $Revision: 3180 $
 */
public class SelectingConnectionCreationTool extends ConnectionCreationTool {

	
	/**
	 * Extend method to select the connection after is has been created.
	 */
	protected boolean handleCreateConnection() {
		
		Command cmd = getCommand();
		if(cmd == null) {
			return false;
		}
		
		super.handleCreateConnection();
		
		// Select the connection
		EditPartViewer viewer = getCurrentViewer();
		selectAddedObject(viewer, cmd);
		
		return true;
		
	}
	
	
	/**
	 * Add the newly created connection to the viewer's selected objects.
	 */
	private void selectAddedObject(EditPartViewer viewer, Command cmd) {
		final Object model = ((ConnectionCreate)cmd).getConnection();
		if (model == null || viewer == null)
			return;
		Object editpart = viewer.getEditPartRegistry().get(model);
		if (editpart instanceof EditPart) {
			//Force the new object to get positioned in the viewer. 
			viewer.flush();
			viewer.select((EditPart)editpart);
		}
	}
	
	
}
