package ch.unizh.ifi.adora.tracker;


import org.eclipse.gef.EditPart;
import org.eclipse.gef.tools.DragEditPartsTracker;

import ch.unizh.ifi.adora.parts.ConnectionLabelEditPart;
import ch.unizh.ifi.adora.parts.RoleEditPart;

/**
 * This tracker has to be adapted, since  
 * {@link DragEditPartsTracker#isMove()} 
 * has to return true if a {@link RoleEditPart} is selected.
 * 
 * @author		        vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             Feb 15, 2006 SM First Version
 * @version             $Date: 2006-09-26 10:52:33 +0200 (Tue, 26 Sep 2006) $, $Author: vigan $, $Revision: 2267 $
 */

public class AdoraDragTracker extends DragEditPartsTracker {

	/**
	 * Instantiates a new AdoraDragTracker.
	 * 
	 * @param sourceEditPart The EditPart for which this Tracker
	 * has been called.
	 */
	public AdoraDragTracker(EditPart sourceEditPart){
		super(sourceEditPart);
	}
	
	/**
	 *  Overwritten since a RoleEditPart has to be treated specialy.
	 *  @see {@link DragEditPartsTracker#isMove()
	 */
	protected boolean isMove() {
		EditPart part = getSourceEditPart();
		while (part != getTargetEditPart() && part != null) {
			//RoleEditParts are always movable
			if (part instanceof ConnectionLabelEditPart || 
					part.getParent() == getTargetEditPart() 
					&& part.getSelected() != EditPart.SELECTED_NONE)
				return true;
			part = part.getParent();
		}
		return false;
	}
}