package ch.unizh.ifi.adora.ui;

import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;

/**
 * A wizard for Adora models which have the default suffix ".adora". Those files
 * can be used with the {@link ch.unizh.ifi.adora.ui.AdoraModelEditor} see
 * plugin.xml. The wizard will be disposed automatically, if the user has
 * finished his entry.
 * 
 * @author Adrian Gygax
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 23.08.2005 AG First Version 21.07.2006 SM Comments
 * @version $Date: 2007-06-05 14:07:40 +0200 (Tue, 05 Jun 2007) $, $Author: schoeni $, $Revision: 2749 $
 */
public class AdoraModelCreationWizard extends Wizard implements INewWizard {

	/**
     * Determins the ending of the proposed default filename.
     */
	private static int fileCount = 1;

	/**
     * Returns the file counter value for uniquely naming the file.
     */
	public static int getfileCount() {
		return fileCount;
	}

	/**
     * Increases the file counter for uniquely nameing the file.
     */
	public static void increaseFileCount() {
		fileCount++;
	}

	/**
     * The file creation page used for creating a new ADORA model.
     */
	private AdoraFileCreationPage page1;

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.wizard.IWizard#addPages()
     */
	public void addPages() {
		// add pages to this wizard
		addPage(page1);
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.IWorkbenchWizard#init(org.eclipse.ui.IWorkbench,
     *      org.eclipse.jface.viewers.IStructuredSelection)
     */
	public void init(IWorkbench workbench, IStructuredSelection selection) {
		// create pages for this wizard
		page1 = new AdoraFileCreationPage(workbench, selection);
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.wizard.IWizard#performFinish()
     */
	public boolean performFinish() {
		return page1.finish();
	}
}
