package ch.unizh.ifi.adora.ui;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.InvocationTargetException;
import java.util.EventObject;
import java.util.List;
import java.util.logging.Logger;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IStorage;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.draw2d.FreeformLayeredPane;
import org.eclipse.gef.ContextMenuProvider;
import org.eclipse.gef.DefaultEditDomain;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.LayerConstants;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.gef.dnd.TemplateTransferDragSourceListener;
import org.eclipse.gef.dnd.TemplateTransferDropTargetListener;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.gef.editparts.ZoomManager;
import org.eclipse.gef.palette.PaletteRoot;
import org.eclipse.gef.requests.CreationFactory;
import org.eclipse.gef.requests.DirectEditRequest;
import org.eclipse.gef.requests.SimpleFactory;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.gef.ui.actions.ToggleGridAction;
import org.eclipse.gef.ui.actions.ToggleSnapToGeometryAction;
import org.eclipse.gef.ui.palette.PaletteViewer;
import org.eclipse.gef.ui.palette.PaletteViewerProvider;
import org.eclipse.gef.ui.palette.FlyoutPaletteComposite.FlyoutPreferences;
import org.eclipse.gef.ui.parts.GraphicalEditorWithFlyoutPalette;
import org.eclipse.gef.ui.parts.GraphicalViewerKeyHandler;
import org.eclipse.gef.ui.parts.TreeViewer;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.util.TransferDropTargetListener;
import org.eclipse.swt.events.MenuDetectEvent;
import org.eclipse.swt.events.MenuDetectListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.IStorageEditorInput;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.eclipse.ui.help.IWorkbenchHelpSystem;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.views.contentoutline.IContentOutlinePage;
import org.eclipse.ui.views.properties.IPropertySheetPage;

import ch.unizh.ifi.adora.actions.AdoraCopyAction;
import ch.unizh.ifi.adora.actions.AdoraCutAction;
import ch.unizh.ifi.adora.actions.AdoraDeleteAction;
import ch.unizh.ifi.adora.actions.AdoraPasteAction;
import ch.unizh.ifi.adora.actions.representation.CompactNodeAction;
import ch.unizh.ifi.adora.actions.representation.RecursiveFisheyeZoomAction;
import ch.unizh.ifi.adora.actions.representation.RerouteConnectionAction;
import ch.unizh.ifi.adora.actions.representation.TreeFilterAction;
import ch.unizh.ifi.adora.actions.representation.UnweaveAction;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.syntaxerrors.SyntaxProblemsSorter;
import ch.unizh.ifi.adora.model.syntaxerrors.SyntaxVerifier;
import ch.unizh.ifi.adora.model.util.ModelFileProperties;
import ch.unizh.ifi.adora.model.util.ModelFileProperty;
import ch.unizh.ifi.adora.model.util.ModelSerializer;
import ch.unizh.ifi.adora.parts.AdoraEditPartFactory;
import ch.unizh.ifi.adora.parts.AdoraModelEditPart;
import ch.unizh.ifi.adora.parts.AdoraRootEditPart;
import ch.unizh.ifi.adora.properties.AdoraPropertySheet;
import ch.unizh.ifi.adora.ui.views.syntaxerrors.SyntaxChecker;
import ch.unizh.ifi.adora.view.layers.MultipleRoutersConnectionLayer;

/**
 * A graphical editor with flyout palette that can edit .adora files. The
 * binding between the .adora file extension and this editor is done in
 * plugin.xml
 * 
 * @author smeier
 * @author gygax
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history Aug 31, 2005 AG First Version 
 *          June 15, 2006 SM Comments and xml storing of model. 
 *          June 18, 2006 SM Three different file formats can
 *                            be stored. 
 *          Oct 28, 2006 SM Added slider for zoom and zoom levels.
 *          May 25, 2007 SM Small refactoring, enabling the editor to change the
 *                          model on the fly. The view is updated when changing the model. This
 *                          feature is used for weaving.
 *          June 30, 2007 SM Corrected bug, the model in the treeview was not 
 *                           replaced properly which was causing exceptions when 
 *                           working on the replaced model.
 *          Sept 20, 2007 SM Inserted methods for selecting elements in the editor. Added methods
 *                           {@link #selectElements(List<Object> modelElements)} and 
 *                           {@link #sselectElement(Object modelElement)}.
 *          25.01.2008 PS PROP_MODEL_CHANGE added.
 *          08.04.2008 CJ Send direct edit request to elements created via DND
 *          24.04.2008 CJ Refactoring of syntax error view and now contains a {@link SyntaxVerifier}
 *			28.04.2008 CJ Addressing bug 783 (paste location)
 *			23.12.2009 CJ Fixing Memory Leaks
 * @version $Date: 2009-12-23 14:25:55 +0100 (Wed, 23 Dec 2009) $, $Author: jeanneret $, $Revision: 5005 $
 * 
 */
public class AdoraModelEditor extends GraphicalEditorWithFlyoutPalette implements MenuDetectListener {

	/**
	 * This is the code which indicated that the loaded model has changed.
	 * TODO please check, which value should be used here, so that we not 
	 * run into a problem with the values GEF uses.
	 */
	public static final int PROP_MODEL_CHANGE = 15800;

	/**
	 * true if the currently shown model is weaved
	 */
	private boolean isWeaved=false;
	
	private boolean requirementsVisible = false;
	
	/**
	 * The blackboard where the editor pushes unweave commands.
	 * Null, if the unweaver is not used.
	 */
	private Object unweaverBlackboard;
	
	/**
	 * Reference to the unweave action. This is needed to reset the
	 * action in case of an invalid user selection.
	 */
	private UnweaveAction unweaveAction;
	
	/**
	 * Keep track of the DND listener, so it can be removed at dispose
	 */
	private TransferDropTargetListener dndTargetListener;
	
	private boolean disposed = false;
	/**
	 * Sets the state of the model to weaved. Saves the model. 
	 * @param neworiginalModel
	 */
	public void setWeaved(boolean weaved)
	{
	    isWeaved = weaved;
	
//	    this.doSave(null);
	}
	
	//public AdoraModel getOriginalModel(){
	//	return this.originalModel;
	//}
	
	/**
	 * Makes the requirement IDs visible
	 */
	public void setRequirementsVisible(){
		//add requirement ID label to all model elements
		
		requirementsVisible = true;
	}
	
	public  void setNormalModel(){
		isWeaved=false;
		IEditorInput input=(IFileEditorInput) getEditorInput();
		this.setInput(input);
	}
	
	/**
	 * Makes the requirement IDs invisible
	 */
	public void setRequirementsInvisible(){
		requirementsVisible = false;
	}
	
	
	/**
	 * returns true if currently shown model is the weaved model
	 * @return
	 */
	public boolean isWeaved(){
		return isWeaved;
	}
	
	/**
	 * Returns a reference to the blackboard of the unweaver.
	 * Null, if the unewaver is not used.
	 * 
	 * @return Object
	 */
	public Object getUnweaverBlackboard()
	{
		return this.unweaverBlackboard;
	}
	
	/**
	 * Use this method to reset the unweave action in case of
	 * an in valid user selection.
	 */
	public void resetUnweaveAction()
	{
		this.unweaveAction.resetAction();
	}
	
	/**
	 * returns true if requirements are visible in the current model
	 * @return Boolean requirementsVisible
	 */
	public boolean requirementsVisible() {
		return requirementsVisible;
	}
	
	
	
	public static final double[] ZOOM_LEVELS = { .1, .15, .25, .5,
			.75, 1.0, 1.5, 2.0 };

	/**
     * Logger for this class.
     */
	private static final Logger logger = Logger
			.getLogger(AdoraModelEditor.class.getName());

	/**
     * This is the root of the editor's model.
     */
	private AdoraModel adoraModel;

	/**
     * This is a reference to the palette factory.
     */
	private AdoraModelEditorPaletteFactory paletteFactory;

	/**
     * Palette component, holding the tools and shapes.
     */
	private PaletteRoot paletteRoot;

	/**
     * The PropertySheet used for displaying attributes.
     */
	private AdoraPropertySheet propertySheet;

	/**
	 * Returns the outline page which belongs to this editor.
	 */
	private AdoraModelOutlinePage outline = null;
	
	/**
	 * The syntax checker for this editor.
	 */
	private SyntaxVerifier syntaxChecker;

	/**
	 * Keeps track where the context menu is enabled (essentially for
	 * the {@link AdoraPasteAction}.
	 */
	private Point lastContextMenuLocation;
	
	/**
     * Create a new AdoraModelEditor instance. This is called by the Workspace.
     */
	public AdoraModelEditor() {
		paletteFactory = new AdoraModelEditorPaletteFactory();
		setEditDomain(new AdoraEditDomain(this));
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.gef.ui.parts.GraphicalEditor#commandStackChanged(java.util.EventObject)
     */
	public void commandStackChanged(EventObject event) {
		firePropertyChange(IEditorPart.PROP_DIRTY);
		super.commandStackChanged(event);
	}

	
	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.ISaveablePart#doSave(org.eclipse.core.runtime.IProgressMonitor)
     */
	public void doSave(IProgressMonitor monitor) {
		ByteArrayOutputStream out = null;
		try {

			IFile file = ((IFileEditorInput) getEditorInput()).getFile();
			// get the file name
			String absFileName = file.getLocation().makeAbsolute().toString();

			ModelFileProperty property = ModelFileProperties.getInstance()
					.getProperty(absFileName);
			// set the unweaved model if model is weaved
			//if(this.hasOriginalModel()){
			//	property.setAdoraModel(this.getOriginalModel());
			//}
			
			//else{
			property.setAdoraModel(getModel());
			//}
			// get the output stream
			out = property.getModelOutputStream();

			// OutputStreamWriter writer = enw OutputStreamWriter(new
			// ByteArrayInputStream(out.toByteArray()), textEncoding)

			file.setContents(new ByteArrayInputStream(out.toByteArray()), true,
					false, monitor);
			out.close();
			getCommandStack().markSaveLocation();
		} catch (CoreException ce) {
			ce.printStackTrace();
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.ISaveablePart#doSaveAs()
     */
	public void doSaveAs() {
		// Show a SaveAs dialog
		Shell shell = getSite().getWorkbenchWindow().getShell();
		String filename = ((IFileEditorInput) getEditorInput()).getFile()
				.getLocation().toString();
		ModelFileProperty property = ModelFileProperties.getInstance()
				.getProperty(filename);
		AdoraModelSaveAsDialog dialog = new AdoraModelSaveAsDialog(shell,
				property.getFileFormat());
		dialog.setOriginalFile(((IFileEditorInput) getEditorInput()).getFile());
		dialog.open();

		IPath path = dialog.getResult();

		if (path == null) {
			return;
		}

		// try to save the editor's contents under a different file name
		final IFile file = ResourcesPlugin.getWorkspace().getRoot().getFile(
				path);
		// read the file format
		int fileFormat = dialog.getFileFormat();
		// set the property for the file.
		final String absFileName = file.getLocation().toString();
		// set the current model.
		//if(this.hasOriginalModel()){
		//	ModelFileProperties.getInstance().setProperty(absFileName,
			//		this.getOriginalModel(), fileFormat);
		//}
		if(this.isWeaved){
			return;
		}
		else{
		ModelFileProperties.getInstance().setProperty(absFileName,
				this.getModel(), fileFormat);
		}

		try {
			new ProgressMonitorDialog(shell).run(false, false,
					new WorkspaceModifyOperation() {
						public void execute(final IProgressMonitor monitor) {
							try {
								ByteArrayOutputStream out = null;
								// get the output stream containing the file
								out = ModelFileProperties.getInstance()
										.getProperty(absFileName)
										.getModelOutputStream();
								out.flush();
								if (file.exists()) {
									file.setContents(new ByteArrayInputStream(
											out.toByteArray()), true, false,
											monitor);
								} else {
									file.create(new ByteArrayInputStream(out
											.toByteArray()), true, monitor);
								}
								out.close();
							} catch (CoreException ce) {
								ce.printStackTrace();
							} catch (IOException ioe) {
								ioe.printStackTrace();
							}
						}
					});
			// set input to the new file
			setInput(new FileEditorInput(file));
			getCommandStack().markSaveLocation();
		} catch (InterruptedException ie) {
			// should not happen, since the monitor dialog is not cancelable
			ie.printStackTrace();
		} catch (InvocationTargetException ite) {
			ite.printStackTrace();
		}
	}

	/**
     * Defines the view to be shown in the outline page. (Here it is a tree of
     * the model).
     */
	public Object getAdapter(Class type) {
		if (type == IContentOutlinePage.class) {
			return getAdoraOutlinePage();
		}

		if (type == IPropertySheetPage.class) {
			if (propertySheet == null) {
				propertySheet = new AdoraPropertySheet();
				getCommandStack().addCommandStackEventListener(propertySheet);
			}
			return propertySheet;
		}

		if (type == ZoomManager.class) {
			return getGraphicalViewer().getProperty(
					ZoomManager.class.toString());
		}

		return super.getAdapter(type);
	}

	/**
	 * Returns the outline page of this ADORA editor.
	 * @return Returns the outline page.
	 */
	public AdoraModelOutlinePage getAdoraOutlinePage() {
		if (outline==null) {
			outline = new AdoraModelOutlinePage(new TreeViewer(), getEditDomain(),
					getSelectionSynchronizer(), getActionRegistry(), getModel());
		}
		return outline;
	}
	
	/**
     * Returns the model associated with this editor.
     * 
     * @return !=null
     */
	public AdoraModel getModel() {
		return adoraModel;
	}

	/**
     * Sets the model and sets the new content in the viewer, if the viewer
     * already exits.
     * 
     * @param model
     *            The new model to set.
     */
	public void setModel(AdoraModel model) {
		adoraModel = model;

		if (getViewer() != null) {
			getViewer().setContents(model);
			
			// Hack to register the line routers as listeners at the figures
			// of the newly set model.
			AdoraRootEditPart rootEditPart = (AdoraRootEditPart)getViewer().getRootEditPart();
			AdoraModelEditPart modelEditPart = (AdoraModelEditPart)rootEditPart.getChildren().get(0);
			FreeformLayeredPane pane = (FreeformLayeredPane)modelEditPart.getFigure().getParent().getParent();
			MultipleRoutersConnectionLayer layer = (MultipleRoutersConnectionLayer)pane.getLayer(LayerConstants.CONNECTION_LAYER);
			layer.initializeRouters();
			
		}
		
		if (getAdoraOutlinePage() != null) {
			getAdoraOutlinePage().setModel(model);
		}
		if(this.syntaxChecker != null)
		{
			this.syntaxChecker.dispose();
		}
		this.syntaxChecker = SyntaxVerifier.createSyntaxVerifier(model);
		
		firePropertyChange(PROP_MODEL_CHANGE);
	}

	/**
     * Returns a factory for creating this editor's palette.
     * 
     * @return
     */
	public AdoraModelEditorPaletteFactory getPaletteFactory() {
		return paletteFactory;
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.gef.ui.parts.GraphicalEditorWithFlyoutPalette#getPaletteRoot()
     */
	public PaletteRoot getPaletteRoot() {
		if (paletteRoot == null) {
			paletteRoot = paletteFactory.createPalette();
		}

		return paletteRoot;
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.ISaveablePart#isSaveAsAllowed()
     */
	public boolean isSaveAsAllowed() {
		return true;
	}

	/**
     * Create a transfer drop target listener. When using a
     * CombinedTemplateCreationEntry tool in the palette, this will enable model
     * element creation by dragging from the palette.
     * 
     * @see #createPaletteViewerProvider()
     */
	private TransferDropTargetListener createTransferDropTargetListener() {
		return new TemplateTransferDropTargetListener(getGraphicalViewer()) {
			protected CreationFactory getFactory(Object template) {
				return new SimpleFactory((Class) template);
			}
			
			/**
			 * Override the handleDrop method such that a direct edit
			 * request is sent to the freshly created object
			 * 
			 * @see org.eclipse.gef.dnd.TemplateTransferDropTargetListener#handleDrop()
			 */
			@Override
			protected void handleDrop() {
				super.handleDrop();
				sendDirectEditRequest();
			}

			/**
			 * Find the created Edit Part and send it a Direct Edit
			 * request (if it understands it).
			 * 
			 * @post a direct edit request is sent to its corresponding edit part
			 */
			protected void sendDirectEditRequest()
			{
				final Object model = getCreateRequest().getNewObject();
				EditPartViewer viewer = getViewer();
				if (model == null || viewer == null)
					return;
				Object control = viewer.getEditPartRegistry().get(model);
				if (control instanceof EditPart) {
					EditPart editpart = (EditPart) control;
					Request directEditRequest = new DirectEditRequest();
					if(editpart.understandsRequest(directEditRequest))
					{
						editpart.performRequest(directEditRequest);
					}
				}
			}
		};
	}

	/**
     * Configure the graphical viewer before it receives contents.
     * <p>
     * This is the place to choose an appropriate RootEditPart and
     * EditPartFactory for your editor. The RootEditPart determines the behavior
     * of the editor's "work-area". For example, GEF includes zoomable and
     * scrollable root edit parts. The EditPartFactory maps model elements to
     * edit parts (controllers).
     * </p>
     * 
     * @see org.eclipse.gef.ui.parts.GraphicalEditor#configureGraphicalViewer()
     */
	protected void configureGraphicalViewer() {
		super.configureGraphicalViewer();

		final GraphicalViewer viewer = getGraphicalViewer();
		viewer.setEditPartFactory(new AdoraEditPartFactory());
		AdoraRootEditPart arep = new AdoraRootEditPart();
		viewer.setRootEditPart(arep);
		arep.getZoomManager().setZoomLevels(ZOOM_LEVELS);
		viewer.setKeyHandler(new GraphicalViewerKeyHandler(viewer));

		// configure the context menu provider
		final ContextMenuProvider cmProvider = new AdoraModelEditorContextMenuProvider(
				viewer, getActionRegistry());
		viewer.setContextMenu(cmProvider);
		
		// Register this editor as a menu detect listener (to obtain
		// the location of the context menu popup, see bug 783)
		viewer.getControl().addMenuDetectListener(this);
		
		// Don't register with the EditorSite to hide the standard actions 
		// (run as etc.)
		// getSite().registerContextMenu(cmProvider, viewer);
		registerHelp(viewer);
		
		IAction showGrid = new ToggleGridAction(getGraphicalViewer());
		getActionRegistry().registerAction(showGrid);
		
		IAction snapAction = new ToggleSnapToGeometryAction(getGraphicalViewer());
		getActionRegistry().registerAction(snapAction);
		
		

	}

	/**
     * Temporary method for testing the zooming. Remove it, if not needed any
     * more.
     * 
     * @return
     */
	public GraphicalViewer getViewer() {
		return getGraphicalViewer();
	}

	/**
     * Registers the editor canvas and its palette (the editor) at the help.
     */
	private void registerHelp(GraphicalViewer viewer) {
		// Register help
		IWorkbenchHelpSystem wh = PlatformUI.getWorkbench().getHelpSystem();
		wh
				.setHelp(viewer.getControl(),
						"ch.unizh.ifi.adora.editor.AdoraEditor");
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.gef.ui.parts.GraphicalEditor#createActions()
     */
	@SuppressWarnings("unchecked")
	protected void createActions() {
		super.createActions();
		ActionRegistry registry = getActionRegistry();
		IAction action;

		// register delete action
		action = new AdoraDeleteAction(this);
		registry.registerAction(action);
		getSelectionActions().add(action.getId());

		// register cut action
		action = new AdoraCutAction(this);
		registry.registerAction(action);
		getSelectionActions().add(action.getId());

		// register copy action
		action = new AdoraCopyAction(this);
		registry.registerAction(action);
		getSelectionActions().add(action.getId());

		// register paste action
		action = new AdoraPasteAction(this);
		registry.registerAction(action);
		getSelectionActions().add(action.getId());

		// register the node filter action for the outline view
		action = new TreeFilterAction(this);
		registry.registerAction(action);
		getSelectionActions().add(action.getId());
		
		// register the recursive fisheye zoom action
		action = new RecursiveFisheyeZoomAction(this);
		registry.registerAction(action);
		getSelectionActions().add(action.getId());
		
		// register the compact node action
		action = new CompactNodeAction(this);
		registry.registerAction(action);
		getSelectionActions().add(action.getId());
		
		// register the action to reroute connections
		action = new RerouteConnectionAction(this);
		registry.registerAction(action);
		getSelectionActions().add(action.getId());
		
		// register unweaving action
		action = new UnweaveAction(this);
		this.unweaverBlackboard = ((UnweaveAction) action).getBlackboard();
		registry.registerAction(action);
		getSelectionActions().add(action.getId());
		this.unweaveAction = (UnweaveAction) action;
	}
	
	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.gef.ui.parts.GraphicalEditorWithFlyoutPalette#createPaletteViewerProvider()
     */
	protected PaletteViewerProvider createPaletteViewerProvider() {
		return new AdoraPaletteViewerProvider(getEditDomain());
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.gef.ui.parts.GraphicalEditorWithFlyoutPalette#getPalettePreferences()
     */
	protected FlyoutPreferences getPalettePreferences() {
		return paletteFactory.createPalettePreferences();
	}

	/**
     * Set up the editor's inital content (after creation).
     * 
     * @see org.eclipse.gef.ui.parts.GraphicalEditorWithFlyoutPalette#initializeGraphicalViewer()
     */
	protected void initializeGraphicalViewer() {
		super.initializeGraphicalViewer();
		final GraphicalViewer viewer = getGraphicalViewer();
		viewer.setContents(getModel()); // set the contents of this editor
		

		// listen for dropped parts
		dndTargetListener = createTransferDropTargetListener();
		viewer.addDropTargetListener(dndTargetListener);
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.part.EditorPart#setInput(org.eclipse.ui.IEditorInput)
     */
	public void setInput(IEditorInput input) {
		AdoraModel model = null;
		String absFileName = "";

		super.setInput(input);

		int fileFormat = ModelFileProperty.FILE_FORMAT_XML_GZIPPED;
		IStorage file = null;
		try {
			file = ((IStorageEditorInput) input).getStorage();
		} catch (CoreException e1) {
			e1.printStackTrace();
		}

		// Not empirical --> model is savable
		if (input instanceof IFileEditorInput) {
			absFileName = ((IFileEditorInput) input).getFile().getLocation()
					.toString();
		}

		try {
			// input stream for getting the content of the file.
			InputStream is = ((IStorageEditorInput) input).getStorage()
					.getContents();
			try {
				boolean isZipped = ModelSerializer.checkIfValidZipStream(is);
				is.close();
				if (isZipped) {
					is = ((IStorageEditorInput) input).getStorage()
							.getContents();
					model = ModelSerializer.deserializeModelFromXMLInputStream(
							is, true);
					fileFormat = ModelFileProperty.FILE_FORMAT_XML_GZIPPED;
				} else {
					is = ((IStorageEditorInput) input).getStorage()
							.getContents();
					model = ModelSerializer.deserializeModelFromXMLInputStream(
							is, false);
					fileFormat = ModelFileProperty.FILE_FORMAT_XML;
				}
			} catch (Exception ex) {
				// if the file is neither a compressed xml file,
				// nor a uncompressed xml file, try to deserialize it.
				ex.printStackTrace();
				if (is != null) {
					is.close();
				}
				is = file.getContents();
				model = ModelSerializer.deserializeModelFromStream(is);
				fileFormat = ModelFileProperty.FILE_FORMAT_SERIALIZED;
			}
			setPartName(file.getName());
		} catch (Exception e) {
			// if nothing of the above
			// format were correct, log an error and
			// create a new model.
			logger
					.warning("** Unknown file format or the file may be corrupted. Using empty model. **"
							+ e);
			model = new AdoraModel();
			// use default file format.
		}

		setModel(model);
		
		// sets the property for the storage of changes.
		if (input instanceof IFileEditorInput) {
			ModelFileProperties.getInstance().setProperty(absFileName, model,
					fileFormat);
		}
	}

	/**
	 * Selects the edit part <i>modelPart</i>, which should be of a sub-type of the class {@link ch.unizh.ifi.adora.model.ModelElement}.
	 * The current selection will be unselected. The element is visible in the viewer of the editor after executing 
	 * the method (viewer scrolls if necessary).
	 * @param modelElements A list of model elements. The selection only occurs if the given 
	 * element has a corresponding element in the edit parts of the editor.
	 */
	public void selectElement(Object modelPart) {
		AbstractGraphicalEditPart part = (AbstractGraphicalEditPart)this.getViewer().getEditPartRegistry().get(modelPart);
		if (part != null) {
			//	Initially inserted connections should be selected.
			getViewer().deselectAll();
			// Make the inserted connection the newly selected one.
			getViewer().appendSelection(part);
			getViewer().reveal(part);
		}
	}
	
	/**
	 * Selects the edit parts of the {@link ch.unizh.ifi.adora.model.ModelElement} objects
	 * given in the list <i>modelElements</i>. The current selection will be unselected. The
	 * first element in the list is visible in the viewer of the editor after executing 
	 * the method (viewer scrolls if necessary).
	 * @param modelElements A list of model elements. All of them which have a corresponding edit 
	 * part will be selected in the editor view.
	 */
	public void selectElements(List<Object> modelElements) {
		//	Intially inserted connections should be selected.
		getViewer().deselectAll();
		AbstractGraphicalEditPart agep = null;
		for (Object o : modelElements) {
			AbstractGraphicalEditPart part = (AbstractGraphicalEditPart)this.getViewer().getEditPartRegistry().get(o);
			if (part != null) {				
				// Make the inserted connection the newly selected one.
				getViewer().appendSelection(part);
				if (agep == null)  { 
					agep = part;
					getViewer().reveal(part);
				}
			}
		}
	}
	
	

	/**
     * Returns the command stack.
     * 
     * @return the command stack
     */
	public CommandStack getCommandStack() {
		return super.getCommandStack();
	}

	/**
	 * Return the errors gathered by the {@link SyntaxChecker}.
	 * @return a {@link SyntaxProblemsSorter} containing the errors.
	 */
	public SyntaxProblemsSorter getErrors() {
		return syntaxChecker.getSyntaxErrors();
		
	}
	
	/**
	 * Getter for the lastContextMenuLocation property
	 * @return the lastContextMenuLocation
	 */
	public Point getLastContextMenuLocation() {
		return lastContextMenuLocation;
	}

	/*
	 * Implementation of {@link MenuDetectListener} Interface
	 */

	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.MenuDetectListener#menuDetected(org.eclipse.swt.events.MenuDetectEvent)
	 */
	public void menuDetected(MenuDetectEvent e) {
		lastContextMenuLocation = new Point(e.x,e.y);
		
	}

	@Override
	
	/**
	 * Addressing issue #11
	 */
	public void dispose()
	{
		getGraphicalViewer().deselectAll();
				
		if(dndTargetListener != null)
		{
			getGraphicalViewer().removeDropTargetListener(dndTargetListener);
			dndTargetListener = null;
		}
		
		getGraphicalViewer().getContextMenu().dispose();
		getGraphicalViewer().setContextMenu(null);
	
		
		((AdoraEditDomain)getEditDomain()).dispose();
		
		if(syntaxChecker != null)
		{
			syntaxChecker.dispose();
			syntaxChecker = null;
		}
	
		
		this.outline = null;
		this.adoraModel = null;
		this.paletteRoot = null;
		this.paletteFactory = null;
		this.unweaveAction = null;		
		
		super.dispose();
		
	}



}

/**
 * Refactored as a "standalone class", so that AdoraModelEditor objects can be GCed
 * (See #11)
 * 
 * @author Cdric Jeanneret
 */
class AdoraPaletteViewerProvider extends PaletteViewerProvider
{
	public AdoraPaletteViewerProvider(EditDomain graphicalViewerDomain) {
		super(graphicalViewerDomain);
	}

	protected void configurePaletteViewer(PaletteViewer viewer) {
		super.configurePaletteViewer(viewer);
		// create a drag source listener for this palette viewer
		// together with an appropriate transfer drop target listener,
		// this will enable
		// model element creation by dragging a
		// CombinatedTemplateCreationEntries
		// from the palette into the editor
		// @see ShapesEditor#createTransferDropTargetListener()
		viewer.addDragSourceListener(new TemplateTransferDragSourceListener(viewer));
	}
}

/**
 * Added disposability to {@link EditDomain}, so that AdoraModelEditor objects can be GCed
 * (See #11)
 * 
 * @author Cdric Jeanneret
 */
class AdoraEditDomain extends DefaultEditDomain
{
	public AdoraEditDomain(IEditorPart editorPart) {
		super(editorPart);
	}

	public void dispose()
	{
		setEditorPart(null);
	}
}