package ch.unizh.ifi.adora.ui;

import org.eclipse.gef.ContextMenuProvider;
import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.gef.ui.actions.GEFActionConstants;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.ui.actions.ActionFactory;

import ch.unizh.ifi.adora.actions.representation.CompactNodeAction;
import ch.unizh.ifi.adora.actions.representation.RecursiveFisheyeZoomAction;
import ch.unizh.ifi.adora.actions.representation.RerouteConnectionAction;
import ch.unizh.ifi.adora.actions.representation.UnweaveAction;

/**
 * Provides context menu actions for the ShapesEditor.
 * 
 * @author smeier
 */
class AdoraModelEditorContextMenuProvider extends ContextMenuProvider {

    /** The editor's action registry. */
    private ActionRegistry actionRegistry;
    
    /**
     * Instantiate a new menu context provider for the specified EditPartViewer
     * and ActionRegistry.
     * 
     * @param viewer
     *            the editor's graphical viewer
     * @param registry
     *            the editor's action registry
     * @throws NullPointerException
     *             if registry is null.
     */
    public AdoraModelEditorContextMenuProvider(EditPartViewer viewer,
            ActionRegistry registry) {
        super(viewer);
        if (registry == null) {
            throw new NullPointerException("Parameter must not be null: " + registry);
        }
        actionRegistry = registry;
    }

    /**
     * Called when the context menu is about to show. Actions, whose state is
     * enabled, will appear in the context menu.
     * 
     * @see org.eclipse.gef.ContextMenuProvider#buildContextMenu(org.eclipse.jface.action.IMenuManager)
     */
    public void buildContextMenu(IMenuManager menu) {
        // Add standard action groups to the menu
        GEFActionConstants.addStandardActionGroups(menu);
        IAction action;
        
        // Add actions to the menu
        menu.appendToGroup(GEFActionConstants.GROUP_UNDO, // target group id
                getAction(ActionFactory.UNDO.getId())); // action to add
        
        menu.appendToGroup(GEFActionConstants.GROUP_UNDO,
                getAction(ActionFactory.REDO.getId()));
        
        action = getAction(ActionFactory.DELETE.getId());
        if (action.isEnabled())
            menu.appendToGroup(GEFActionConstants.GROUP_EDIT,action);
        
        action = getAction(ActionFactory.PASTE.getId());
        if (action.isEnabled())
            menu.appendToGroup(GEFActionConstants.GROUP_EDIT, action);
        
        action = getAction(ActionFactory.COPY.getId()); 
        if (action.isEnabled()) 
            menu.appendToGroup(GEFActionConstants.GROUP_COPY, action);
        
        action = getAction(ActionFactory.CUT.getId());
        if (action.isEnabled())
            menu.appendToGroup(GEFActionConstants.GROUP_EDIT,   action);
        
    	action = getAction(RecursiveFisheyeZoomAction.RECURSIVE_ZOOM);
    	if (action.isEnabled()) {
    		menu.appendToGroup(GEFActionConstants.GROUP_EDIT,   action);
    	}
    	
    	action = getAction(CompactNodeAction.COMPACT_NODE);
    	if (action.isEnabled()) {
    		menu.appendToGroup(GEFActionConstants.GROUP_EDIT,   action);
    	}
    	
    	action = getAction(RerouteConnectionAction.REROUTE_LINE);
    	if (action.isEnabled()) {
    		menu.appendToGroup(GEFActionConstants.GROUP_EDIT,   action);
    	}
    	
    	action = getAction(UnweaveAction.UNWEAVE_ACTION);
    	if (action.isEnabled())
    	{
    		menu.appendToGroup(GEFActionConstants.GROUP_EDIT, action);
    	}
    }

    protected IAction getAction(String actionId) {
    	return actionRegistry.getAction(actionId);
    }

	@Override
	public void dispose() {
		setViewer(null);
		actionRegistry = null;
		super.dispose();
	}
    
    
}