package ch.unizh.ifi.adora.ui;

import org.eclipse.gef.palette.MarqueeToolEntry;
import org.eclipse.gef.palette.PaletteContainer;
import org.eclipse.gef.palette.PaletteDrawer;
import org.eclipse.gef.palette.PaletteGroup;
import org.eclipse.gef.palette.PaletteRoot;
import org.eclipse.gef.palette.PaletteSeparator;
import org.eclipse.gef.palette.PanningSelectionToolEntry;
import org.eclipse.gef.palette.ToolEntry;
import org.eclipse.gef.requests.CreationFactory;
import org.eclipse.gef.requests.SimpleFactory;
import org.eclipse.gef.ui.palette.FlyoutPaletteComposite.FlyoutPreferences;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.resource.ImageDescriptor;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.factories.ObjectSetFactory;
import ch.unizh.ifi.adora.model.representation.Actor;
import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.Association;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.ExitPoint;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.model.representation.ScenarioConnection;
import ch.unizh.ifi.adora.model.representation.State;
import ch.unizh.ifi.adora.model.representation.Transition;
import ch.unizh.ifi.adora.tools.ConstraintTool;
import ch.unizh.ifi.adora.tools.FisheyeZoomToolEntry;
import ch.unizh.ifi.adora.tools.SelectingConnectionCreationToolEntry;

/**
 * Utility class that can create a GEF Palette.
 * 
 * @see #createPalette()
 * @author Adrian Gygax
 * @author Silvio Meier
 */
public class AdoraModelEditorPaletteFactory {

	/** Preference ID used to persist the palette location. */
	private static final String PALETTE_DOCK_LOCATION = "AdoraModelEditorPaletteFactory.Location";

	/** Preference ID used to persist the palette size. */
	private static  final String PALETTE_SIZE = "AdoraModelEditorPaletteFactory.Size";

	/** Preference ID used to persist the flyout palette's state. */
	private static final String PALETTE_STATE = "AdoraModelEditorPaletteFactory.State";

    /*
     * The following attributes are used for storing the palette entries. 
     * The attributes are used for showing and hiding the palette entries, 
     * by the view management.  
     */
    
    /**
     * Stores the associationEntry, whose visibility state must be alterable. 
     */
    private   ToolEntry joinRelationEntry;
    
    /**
	 * Stores the associationEntry, whose visibility state must be alterable. 
	 */
	private   ToolEntry associationEntry;
	
	/**
	 * Stores the transitionEntry, whose visibility state must be alterable. 
	 */
	private   ToolEntry transitionEntry;
	
    /**
     * Stores the scenarioConnectionEntry, whose visibility state must be alterable. 
     */
    private   ToolEntry scenarioConnectionEntry;
    
	/**
	 * Stores the scenarioEntry, whose visibility state must be alterable. 
	 */
	private   StickyTemplateCreationEntry scenarioEntry;
	
	/**
	 * Stores the actorEntry, whose visibility state must be alterable. 
	 */
	private   StickyTemplateCreationEntry actorEntry;
	
	/**
	 * Stores the stateEntry, whose visibility state must be alterable. 
	 */
    private   StickyTemplateCreationEntry stateEntry;

    /**
     * Stores the abstractEntry, whose visibility state must be alterable. 
     */
    private   StickyTemplateCreationEntry abstractEntry;
    
    /**
     * Stores the component, whose visibility state must be alterable. 
     */
    private   StickyTemplateCreationEntry component;

    /**
     * Stores the aspectContainerEntry, whose visibility state can be alterable. 
     */
    private   StickyTemplateCreationEntry aspectContainerEntry;

    /**
     * Stores the exitPointEntry, whose visibility state can be alterable. 
     */
    private   StickyTemplateCreationEntry exitPointEntry;

    /**
     * Stores the adoraTool.
     */
    private   ConstraintTool adoraTool;
    
    /**
     * Default tool
     */
    private ToolEntry defaultTool;
    
	/** Create the "Adora" drawer. */
	private   PaletteContainer createShapesDrawer() {
		PaletteDrawer componentsDrawer = new PaletteDrawer("Nodes");

		component = new StickyTemplateCreationEntry(
				"Object Set", "Creates an Object Set", Component.class,
				new ObjectSetFactory(), ImageDescriptor
						.createFromFile(AdoraPlugin.class,
								"icons/stack16.gif"), ImageDescriptor
						.createFromFile(AdoraPlugin.class,
								"icons/stack24.gif"));
		componentsDrawer.add(component);
		
        abstractEntry = new StickyTemplateCreationEntry("Abstract Object",
				"Creates an Abstract Object", Component.class,
				new SimpleFactory(Component.class), ImageDescriptor
						.createFromFile(AdoraPlugin.class,
								"icons/rectangle16.gif"), ImageDescriptor
						.createFromFile(AdoraPlugin.class,
								"icons/rectangle24.gif"));
		componentsDrawer.add(abstractEntry);

		stateEntry = new StickyTemplateCreationEntry(
				"State", "Creates a State", State.class,
				new SimpleFactory(State.class), ImageDescriptor
						.createFromFile(AdoraPlugin.class,
								"icons/roundedrectangle16.gif"), ImageDescriptor
						.createFromFile(AdoraPlugin.class,
								"icons/roundedrectangle24.gif"));
		componentsDrawer.add(stateEntry);

		actorEntry = new StickyTemplateCreationEntry(
				"Actor", "Creates an Environmental Object", Actor.class,
				new SimpleFactory(Actor.class), ImageDescriptor
						.createFromFile(AdoraPlugin.class,
								"icons/sexangle16.gif"), ImageDescriptor
						.createFromFile(AdoraPlugin.class,
								"icons/sexangle24.gif"));
		componentsDrawer.add(actorEntry);

		
		scenarioEntry = new StickyTemplateCreationEntry(
				"Scenario", "Creates a Scenario", Scenario.class,
				new SimpleFactory(Scenario.class), ImageDescriptor
						.createFromFile(AdoraPlugin.class,
								"icons/ellipse16.gif"), ImageDescriptor
						.createFromFile(AdoraPlugin.class,
								"icons/ellipse24.gif"));
		componentsDrawer.add(scenarioEntry);

        aspectContainerEntry = new StickyTemplateCreationEntry(
                "AspectContainer", "Creates an ExitPoint", AspectContainer.class,
                new SimpleFactory(AspectContainer.class), ImageDescriptor
                        .createFromFile(AdoraPlugin.class,
                                "icons/beveledRectangle16.gif"), ImageDescriptor
                        .createFromFile(AdoraPlugin.class,
                                "icons/beveledRectangle24.gif"));
        componentsDrawer.add(aspectContainerEntry);
        
        exitPointEntry = new StickyTemplateCreationEntry(
                "ExitPoint", "Creates an ExitPoint", ExitPoint.class,
                new SimpleFactory(ExitPoint.class), ImageDescriptor
                        .createFromFile(AdoraPlugin.class,
                                "icons/exitPoint16.gif"), ImageDescriptor
                        .createFromFile(AdoraPlugin.class,
                                "icons/exitPoint24.gif"));
        componentsDrawer.add(exitPointEntry);
        
		return componentsDrawer;
	}

	/**
	 * Creates the PaletteRoot and adds all palette elements. Use this factory
	 * method to create a new palette for your graphical editor.
	 * 
	 * @return a new PaletteRoot
	 */
	  PaletteRoot createPalette() {
		PaletteRoot palette = new PaletteRoot();
		palette.add(createToolsGroup(palette));
		palette.add(createShapesDrawer());
		return palette;
	}

	/**
	 * Return a FlyoutPreferences instance used to save/load the preferences of
	 * a flyout palette.
	 */
	  FlyoutPreferences createPalettePreferences() {
		return new FlyoutPreferences() {
			private IPreferenceStore getPreferenceStore() {
				return AdoraPlugin.getDefault().getPreferenceStore();
			}

			public int getDockLocation() {
				return getPreferenceStore().getInt(PALETTE_DOCK_LOCATION);
			}

			public int getPaletteState() {
				return getPreferenceStore().getInt(PALETTE_STATE);
			}

			public int getPaletteWidth() {
				return getPreferenceStore().getInt(PALETTE_SIZE);
			}

			public void setDockLocation(int location) {
				getPreferenceStore().setValue(PALETTE_DOCK_LOCATION, location);
			}

			public void setPaletteState(int state) {
				getPreferenceStore().setValue(PALETTE_STATE, state);
			}

			public void setPaletteWidth(int width) {
				getPreferenceStore().setValue(PALETTE_SIZE, width);
			}
		};
	}
	  
  /**
   *  Used to return the selectioning tool.
   * @return the marker tool
   */
	public ToolEntry getDefaultTool(){
		return defaultTool;
	}

	/** Create the "Tools" group. */
	private PaletteContainer createToolsGroup(PaletteRoot palette) {
		PaletteGroup toolGroup = new PaletteGroup("Tools");

		// Add a selection tool to the group
		ToolEntry defaultTool = new PanningSelectionToolEntry();
		//adoraTool = new AdoraPanningSelectionTool();
		
		adoraTool = new ConstraintTool();
		
		defaultTool.setToolClass(adoraTool.getClass());
		toolGroup.add(defaultTool);
		palette.setDefaultEntry(defaultTool);

		// Add a marquee tool to the group
		toolGroup.add(new MarqueeToolEntry());

		// Add a (unnamed) separator to the group
		toolGroup.add(new PaletteSeparator());

		//Fisheye zoom
		ToolEntry tool = new FisheyeZoomToolEntry();
		toolGroup.add(tool);
		
		
		// Add (solid-line) connection tool
		associationEntry = new SelectingConnectionCreationToolEntry("Association",
				"Creates an Association", new CreationFactory() {
					public Object getNewObject() {
						return null;
					}

					// see ShapeEditPart#createEditPolicies()
					// this is abused to transmit the desired line style
					public Object getObjectType() {
						return Association.class;
					}
				}, ImageDescriptor.createFromFile(AdoraPlugin.class,
						"icons/association16.gif"), ImageDescriptor
						.createFromFile(AdoraPlugin.class,
								"icons/association24.gif"));

		toolGroup.add(associationEntry);

		// Add (solid-line) connection tool
		transitionEntry = new SelectingConnectionCreationToolEntry("Transition",
				"Creates a Transition", new CreationFactory() {
					public Object getNewObject() {
						return null;
					}

					// see ShapeEditPart#createEditPolicies()
					// this is abused to transmit the desired line style
					public Object getObjectType() {
						return Transition.class;
					}
				}, ImageDescriptor.createFromFile(AdoraPlugin.class,
						"icons/transition16.gif"), ImageDescriptor
						.createFromFile(AdoraPlugin.class,
								"icons/transition24.gif"));
		toolGroup.add(transitionEntry);

		// Add (solid-line) connection tool
        scenarioConnectionEntry = new SelectingConnectionCreationToolEntry("Scenario Connection",
				"Creates a Scenario Connection", new CreationFactory() {
					public Object getNewObject() {
						return null;
					}

					// see ShapeEditPart#createEditPolicies()
					// this is abused to transmit the desired line style
					public Object getObjectType() {
						return ScenarioConnection.class;
					}
				}, ImageDescriptor.createFromFile(AdoraPlugin.class,
						"icons/scenarioconnection16.gif"), ImageDescriptor
						.createFromFile(AdoraPlugin.class,
								"icons/scenarioconnection24.gif"));
		toolGroup.add(scenarioConnectionEntry);
        
        // Add a JoinRelation tool
        joinRelationEntry = new SelectingConnectionCreationToolEntry("JoinRelation",
                "Creates an JoinRelation", new CreationFactory() {
                    public Object getNewObject() {
                        return null;
                    }

                    // see ShapeEditPart#createEditPolicies()
                    // this is abused to transmit the desired line style
                    public Object getObjectType() {
                        return JoinRelation.class;
                    }
                }, ImageDescriptor.createFromFile(AdoraPlugin.class,
                        "icons/joinRelation16.gif"), ImageDescriptor
                        .createFromFile(AdoraPlugin.class,
                                "icons/joinRelation24.gif"));
        toolGroup.add(joinRelationEntry);

		return toolGroup;
	}

    /**
     * Returns a ToolEntry to the view Actions.
     * This is used for hiding ToolEntries which would create
     * components which are hidden at the moment.
     * 
     * @pre       true
     * @post      true
     * @obligation 
     * @param type Determins which ToolEntry shall be returned.
     * @return ToolEntry to hide.
     */
	public ToolEntry getToolEntry(Class type){
        
        if (type.equals(State.class)){
            return stateEntry;
        }if (type.equals(Scenario.class)){
            return scenarioEntry;
        }if (type.equals(Actor.class)){
            return actorEntry;
        }if (type.equals(Association.class)){
            return associationEntry;
        }if (type.equals(AspectContainer.class)){
            return aspectContainerEntry;
        }if (type.equals(ExitPoint.class)){
            return exitPointEntry;
        }if (type.equals(ScenarioConnection.class)){
            return scenarioConnectionEntry;
        }if (type.equals(JoinRelation.class)){
            return joinRelationEntry;
        }
        return null;
    }
	
	/**
	 * This method is responsible for hiding / showing the 
	 * {@link AdoraModelEditorPaletteFactory}. This is important since
	 * during the product derivation, no new elements should be added because
	 * once the derivation is stopped, these elements are deleted.
	 * 
	 * @param hide  true  => palette-elements get hidden
	 * 				false => palette-elements get shown 
	 */
	public void updatePaletteVisisbility(boolean hide) {
        stateEntry.setVisible(!hide);
        scenarioEntry.setVisible(!hide);
        actorEntry.setVisible(!hide);
        associationEntry.setVisible(!hide);
        aspectContainerEntry.setVisible(!hide);
        exitPointEntry.setVisible(!hide);
        scenarioConnectionEntry.setVisible(!hide);
        joinRelationEntry.setVisible(!hide);
        transitionEntry.setVisible(!hide);
        abstractEntry.setVisible(!hide);
        component.setVisible(!hide);
	}
}