package ch.unizh.ifi.adora.ui;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.dialogs.SaveAsDialog;

import ch.unizh.ifi.adora.model.util.ModelFileProperty;

/**
 * A save dialog for ADORA models. It extends the default
 * save dialog by the option to choose the file format.
 * 
 * @author		        smeier
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			Jun 18, 2006 SM First Version
 * @version            $Date: 2006-06-18 18:27:41 +0200 (Sun, 18 Jun 2006) $, $Author: smeier $, $Revision: 1679 $
 */
public class AdoraModelSaveAsDialog extends SaveAsDialog {
	
	/**
	 * holds all the ui elements for choosing the file type.
	 */
	private Composite fileTypeArea;
	
	/**
	 * Combo box describing the type to choose for file format.
	 */
	private Combo fileTypeCombo;
	
	/**
	 * Describes the file format.
	 */
	private int fileFormat = ModelFileProperty.FILE_FORMAT_XML_GZIPPED;
	
	/**
	 * The constructor of the ADORA model save dialog.
	 * @param parentShell The shell for which the 
	 *         dialog is shown.
	 * @param The fileformat which is set initially.
	 */
	public AdoraModelSaveAsDialog(Shell parentShell, int format) {
		super(parentShell);
		this.fileFormat = ModelFileProperty.FILE_FORMAT_XML_GZIPPED;
		this.setFileFormat(format);
	}

	/**
	 * The constructor of the ADORA model save dialog.
	 * @param parentShell The shell for which the 
	 *         dialog is shown.
	 */
	public AdoraModelSaveAsDialog(Shell parentShell) {
		super(parentShell);
		this.fileFormat = ModelFileProperty.FILE_FORMAT_XML_GZIPPED;
	}

	
	/* (non-Javadoc)
     * Method declared on Dialog.
     */
    protected Control createDialogArea(Composite parent) {
        // top level composite
        Composite parentComposite = (Composite) super.createDialogArea(parent);
		fileTypeArea = new Composite(parentComposite, SWT.NONE);
		GridLayout gl = new GridLayout();
		gl.numColumns = 2;
		fileTypeArea.setLayout(gl);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		// grid data settings for the fiel type area --> use the horizontal space
		gd.horizontalAlignment = SWT.FILL;
		gd.grabExcessHorizontalSpace = true;
		
		fileTypeArea.setLayoutData(gd);
		fileTypeArea.setFont(parent.getFont());
		
		Label fileTypeComboLabel = new Label(fileTypeArea, SWT.LEFT);
		fileTypeComboLabel.setText("File Format: ");
		gd = new GridData();
		gd.horizontalAlignment = SWT.FILL;
		//gd.grabExcessHorizontalSpace = true;
		fileTypeComboLabel.setLayoutData(gd);
		
		fileTypeCombo = new Combo(fileTypeArea, SWT.DROP_DOWN | SWT.READ_ONLY);
		gd = new GridData();
		gd.horizontalAlignment = GridData.FILL;
		gd.grabExcessHorizontalSpace = true;
		
		fileTypeCombo.setLayoutData(gd);
		fileTypeCombo.add("XML GZipped",0);
		fileTypeCombo.add("XML Uncompressed", 1);
		fileTypeCombo.add("Java Serialized", 2);
		fileTypeCombo.select(this.fileFormat);

    	return parentComposite;
    }
    
    /**
     * Returns the file format which has been selected
     * for doing the save of the file.
     * 
     * @return Returns the file format.
     */
    public int getFileFormat() {
    	return this.fileFormat;
    }
    
    /**
     * Sets the file format for the file to save.
     * @param format The file format descriptor.
     */
    protected void setFileFormat(int format) {
    	this.fileFormat = format;
    }
    
    /* (non-Javadoc)
     * Method declared on Dialog.
     */
    protected void okPressed() {
    	this.fileFormat = fileTypeCombo.getSelectionIndex();
    	super.okPressed();
    }
}
