/**
 * 
 */
package ch.unizh.ifi.adora.ui;

import org.eclipse.ui.IPageLayout;
import org.eclipse.ui.IPerspectiveFactory;

/**
 * Creates the default perspective for Adora model editing.
 * 
 * @author Adrian Gygax
 */
public class AdoraPerspectiveFactory implements IPerspectiveFactory {

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IPerspectiveFactory#createInitialLayout(org.eclipse.ui.IPageLayout)
	 */
	public void createInitialLayout(IPageLayout layout) {
		defineActions(layout);
		defineLayout(layout);
	}
	
	/**
	 * Defines the layout for the created perspective. 
	 * @param  the page layout inerface for the perspective.
	 */
	public void defineLayout(IPageLayout layout) {
		final String editorArea = layout.getEditorArea();

		layout.addView(IPageLayout.ID_OUTLINE, IPageLayout.RIGHT, 0.85f,
				editorArea);
		layout.addView(IPageLayout.ID_RES_NAV, IPageLayout.LEFT, 0.2f,
				editorArea);
		layout.addView(IPageLayout.ID_PROP_SHEET, IPageLayout.BOTTOM, 0.5f,
				IPageLayout.ID_RES_NAV);
		layout.addView(IPageLayout.ID_PROBLEM_VIEW, IPageLayout.BOTTOM, 0.8f,
				editorArea);	
	}

	/**
	 * Defines various short cuts to the created perspectives.
	 * @param the page layout inerface for the perspective.
	 */
	public void defineActions(IPageLayout layout) {
        // Add "new wizards".
		layout.addNewWizardShortcut("ch.unizh.ifi.adora.AdoraModelCreationWizard");
        layout.addNewWizardShortcut("org.eclipse.ui.wizards.new.folder");
        layout.addNewWizardShortcut("org.eclipse.ui.wizards.new.file");
        layout.addPerspectiveShortcut("org.eclipse.ui.resourcePerspective");
        
        // Add "show views".
        layout.addShowViewShortcut(IPageLayout.ID_RES_NAV);
        layout.addShowViewShortcut(IPageLayout.ID_BOOKMARKS);
        layout.addShowViewShortcut(IPageLayout.ID_OUTLINE);
        layout.addShowViewShortcut(IPageLayout.ID_PROP_SHEET);
        layout.addShowViewShortcut(IPageLayout.ID_TASK_LIST);
	}
	
}
