package ch.unizh.ifi.adora.ui;

import org.eclipse.gef.palette.CombinedTemplateCreationEntry;
import org.eclipse.gef.requests.CreationFactory;
import org.eclipse.gef.tools.AbstractTool;
import org.eclipse.jface.resource.ImageDescriptor;

import ch.unizh.ifi.adora.tools.DirectEditNodeCreationTool;

/**
 * Creates a new {@link org.eclipse.gef.palette.ToolEntry} which 
 * is selected until a new {@link org.eclipse.gef.palette.ToolEntry} 
 * is chosen by the user.     
 *
 * @author		        Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             21.08.2006 IV First Version
 * 						08.04.2008 CJ Bug Fix ID 798
 * @version             $Date: 2008-04-08 11:25:54 +0200 (Tue, 08 Apr 2008) $, $Author: jeanneret $, $Revision: 3587 $
 */
public class StickyTemplateCreationEntry extends CombinedTemplateCreationEntry {

	/**
	 * Constructs a new StickyTemplateCreationEntry with the given parameters.
	 * This creation entry will rely on an extended creation tool.
	 * 
	 * @see DirectEditNodeCreationTool
	 * 
	 * @param label the label
	 * @param shortDesc the description
	 * @param template the template object
	 * @param factory the CreationFactory
	 * @param iconSmall the small icon
	 * @param iconLarge the large icon
	 */
	public StickyTemplateCreationEntry(String label, 
			String shortDesc, 
			Object template, 
			CreationFactory factory, 
			ImageDescriptor iconSmall, 
			ImageDescriptor iconLarge) {
		super(label, shortDesc, template, factory, iconSmall, iconLarge);
		setToolProperty(AbstractTool.PROPERTY_UNLOAD_WHEN_FINISHED,          
				Boolean.FALSE);
		setToolClass(DirectEditNodeCreationTool.class);
	}
}
