package ch.unizh.ifi.adora.ui.constrainthint;

import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.PlatformUI;

import ch.unizh.ifi.adora.AdoraPlugin;

/**
 * This class is responsible for showing constraint messages 
 * in the status bar of eclipse. These messages occure if a user performes a 
 * constraint violating action.   
 * 
 * @author		        Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             20.09.2006 IV First Version
 * @version             $Date: 2006-10-27 22:37:43 +0200 (Fri, 27 Oct 2006) $, $Author: smeier $, $Revision: 2340 $
 */
public class ConstraintHint {

	/**
	 * Stores whether a constraint message is currently shown or not
	 */
	private static boolean constraintShown = false;
	
	/**
	 * Shows a constraint message in the status bar of eclipse. 
	 *  
	 * @param msg The message which is shown in the eclipse status bar. 
	 */
	public static void displayConstraintHint(String msg){
		
		IStatusLineManager statusbar = PlatformUI.getWorkbench().
				getActiveWorkbenchWindow().
				getActivePage().getActiveEditor().
				getEditorSite().getActionBars().
				getStatusLineManager();
		
		Image img = ImageDescriptor.createFromFile(
				AdoraPlugin.class, 
				"icons/alert_obj.gif").createImage();
		
		statusbar.setErrorMessage(img, msg);
		constraintShown = true;
	}
	
	/**
	 * Clears the status bar of any message. 
	 */
	public static void resetConstraintHint(){
		
		IStatusLineManager statusbar = PlatformUI.getWorkbench().
				getActiveWorkbenchWindow().
				getActivePage().getActiveEditor().
				getEditorSite().getActionBars().
				getStatusLineManager();
		
		statusbar.setErrorMessage(null, "");
		constraintShown = false;
	}

	/**
	 * Used to determine if the consraint message has to be reset in the {@link ch.unizh.ifi.adora.tracker.ConstraintNodeTracker#handleButtonUp(int)} 
	 * method.
	 * 
	 * @return <code>true</code> if a constraint message is currently shown.
	 */
	public static boolean isConstraintShown() {
		return constraintShown;
	}
}