package ch.unizh.ifi.adora.ui.preferences;

import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.help.IWorkbenchHelpSystem;

import ch.unizh.ifi.adora.AdoraPlugin;

/**
 * This class represents a preference page that the adora plugin contributes to the
 * Preferences dialog.
 * <p>
 * The superclass FieldEditorPreferencePage provides a field support build into
 * JFace and does all the storing and loading of the preferences itself.
 * <p>
 * This page is used to modify preferences only. They are stored in the preference 
 * store that belongs to the main plug-in class (AdoraPlugin). That way, preferences can
 * be accessed directly via the preference store.
 *
 * @author		         treinhard
 * @copyright           Requirements Engineering Research Group<br>
 *                       Department of Informatics<br>
 *                       University of Zurich<br>
 * @history             Feb 13, 2006 TR First Version
 *                       May 20, 2006 SM Added help context.
 * @version             $Date: 2006-06-26 16:52:43 +0200 (Mon, 26 Jun 2006) $, $Author: vigan $, $Revision: 1740 $
 * @responsibilities	 Preference page of the ADORA plugin
 *
 */
public class AdoraPreferencePage extends FieldEditorPreferencePage
	implements IWorkbenchPreferencePage {
	

	/**
	 * Constructor for the preference page.
	 * 
	 * @pre true
	 * @post true
	 */
	public AdoraPreferencePage() {
		super(GRID);
		setPreferenceStore(AdoraPlugin.getDefault().getPreferenceStore());
		setDescription("ADORA preferences");
	}
	
	/**
	 * Creates the field editors. Field editors are abstractions of the common GUI 
	 * blocks needed to manipulate various types of preferences. Each field editor 
	 * knows how to save and restore itself.
	 * 
	 * @pre true
	 * @post true
	 */
	public void createFieldEditors() {		
        registerHelp();
	}

	/**
	 * @see org.eclipse.jface.preference.FieldEditorPreferencePage#init(org.eclipse.ui.IWorkbench)
	 */
	public void init(IWorkbench workbench) {
      
	}
	
    /**
     * Registers the editor canvas and its palette (the editor) at the help.
     */
    private void registerHelp() {
        // Register help
        IWorkbenchHelpSystem wh = PlatformUI.getWorkbench().getHelpSystem();
        wh.setHelp(getControl(), "ch.unizh.ifi.adora.editor.AdoraPreferencePage");   
    }
}