package ch.unizh.ifi.adora.ui.preferences;

import org.eclipse.jface.preference.BooleanFieldEditor;
import org.eclipse.jface.preference.ColorFieldEditor;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.preference.IntegerFieldEditor;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

import ch.unizh.ifi.adora.AdoraPlugin;

/**
 * Preference page for the continuous zoom.
 *
 * @author				Tobias Reinhard
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				2007-05-31 TR First Version
 * @version				$Date: 2009-07-20 15:44:11 +0200 (Mon, 20 Jul 2009) $, $Author: treinhard $, $Revision: 4528 $
 *
 */
public class FisheyeZoomPreferencePage extends FieldEditorPreferencePage implements IWorkbenchPreferencePage {
	
	public FisheyeZoomPreferencePage() {
		super(GRID);
		setPreferenceStore(AdoraPlugin.getDefault().getPreferenceStore());
		setDescription("Zoom preferences");
	}
	
	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.jface.preference.FieldEditorPreferencePage#createFieldEditors()
	 */
	protected void createFieldEditors() {
		
		addField(
				new BooleanFieldEditor(
					IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_HORIZONTAL_INTERVALS,
					"Show the horizontal intervals of the continuous zoom",
					getFieldEditorParent()));
		
		addField(
				new ColorFieldEditor(
					IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_HORIZONTAL_INTERVALS_COLOR,
					"Color of the horizontal intervals",
					getFieldEditorParent()));
		
		addField(
				new BooleanFieldEditor(
					IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_VERTICAL_INTERVALS,
					"Show the vertical intervals of the continuous zoom",
					getFieldEditorParent()));
		
		addField(
				new ColorFieldEditor(
					IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_VERTICAL_INTERVALS_COLOR,
					"Color of the vertical intervals",
					getFieldEditorParent()));
		
		addField(
				new BooleanFieldEditor(
					IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_ZOOM_HOLES,
					"Show the zoom holes of the continuous zoom",
					getFieldEditorParent()));
		
		addField(
				new ColorFieldEditor(
					IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_ZOOM_HOLES_COLOR,
					"Color of the zoom holes",
					getFieldEditorParent()));
		
		addField(
				new BooleanFieldEditor(
					IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_ANIMATE_ZOOM,
					"Animate the fisheye zooming",
					getFieldEditorParent()));
		
		addField(
				new IntegerFieldEditor(
					IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_ZOOM_ANIMATION_DELAY,
					"Fisheye zoom animation delay [ms]",
					getFieldEditorParent()));
		addField(
				new BooleanFieldEditor(
					IPreferenceNameConstants.ADORA_REPRESENTATION_EXPLOSIVE_ZOOM,
					"Explosive zoom (EXPERIMENTAL)",
					getFieldEditorParent()));
		
		addField(
				new BooleanFieldEditor(
					IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_SCALE_GAPS,
					"Scale gap intervals (EXPERIMENTAL)",
					getFieldEditorParent()));
	}
	
	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
	 */
	public void init(IWorkbench workbench) {
	}
}