package ch.unizh.ifi.adora.ui.preferences;

/**
 * Constant definitions for the ADORA plugin preferences.
 * <p>   
 *
 * @author		         treinhard
 * @copyright           Requirements Engineering Research Group<br>
 *                       Department of Informatics<br>
 *                       University of Zurich<br>
 * @history             Feb 13, 2006 TR First Version
 *                       May 20, 2006 SM Included constant for join relationship router
 *                       May 31, 2006 JS added constants for layout visualization
 * @version             $Date: 2009-11-03 10:14:59 +0100 (Tue, 03 Nov 2009) $, $Author: treinhard $, $Revision: 4930 $
 * @responsibilities	 Container for the preference constants of the adora plugin.
 *
 */
public interface IPreferenceNameConstants {

	/**
	 * The zoom algorithm
	 */
	public static final String ADORA_REPRESENTATION_ZOOM_ALGORITHM =
		"adora.representation.zoom.algorithm";
	
	/**
	 * The layout algorithm that is invoked if a node is inserted
	 */
	public static final String ADORA_REPRESENTATION_INSERT_LAYOUT_ALGORITHM =
		"adora.representation.insert.layoutAlgorithm";
	
	/**
	 * The layout algorithm that is invoked if a node is removed
	 */
	public static final String ADORA_REPRESENTATION_REMOVE_LAYOUT_ALGORITHM =
		"adora.representation.remove.layoutAlgorithm";
	
	/**
	 * The layout algorithm that is invoked if a node is moved
	 */
	public static final String ADORA_REPRESENTATION_MOVE_LAYOUT_ALGORITHM =
		"adora.representation.move.layoutAlgorithm";
	
	/**
	 * The layout algorithm that is invoked if some model elements are filtered
	 * out.
	 */
	public static final String ADORA_REPRESENTATION_FILTER_LAYOUT_ALGORITHM =
		"adora.representation.filter.layoutAlgorithm";
	
	/**
	 * Imitation of an explosive zoom (for the experimental validation).
	 */
	public static final String ADORA_REPRESENTATION_EXPLOSIVE_ZOOM =
		"adora.representation.zoom.explosive";
	
	/**
	 * Disables the fisheye zoom
	 */
	public static final String ADORA_REPRESENTATION_FISHEYE_DISABLED =
		"adora.representation.fisheyeZoom.disabled";
	
	/**
	 * Disables the view mechanism
	 */
	public static final String ADORA_REPRESENTATION_VIEWS_DISABLED =
		"adora.representation.views.disabled";
	
	/**
	 * The line router for the associations.
	 */
	public static final String ADORA_REPRESENTATION_ROUTER_ASSOCIATION = 
		"adora.representation.routing.association";
    
    /**
     * The line router for the transitions.
     */
    public static final String ADORA_REPRESENTATION_ROUTER_TRANSITION = 
        "adora.representation.routing.transition";
    
    /**
     * The line router for the scenario connections.
     */
    public static final String ADORA_REPRESENTATION_ROUTER_SCENARIOCONNECTION = 
        "adora.representation.routing.scenarioconnection";

    /**
     * The line router for the scenario connections.
     */
    public static final String ADORA_REPRESENTATION_ROUTER_JOIN_RELATIONSHIP = 
        "adora.representation.routing.joinrelationship";
    
    
	/**
	 * The factor the costs (i.e. distance) are multiplied by if the line
	 * goes through a node.
	 */
	public static final String ADORA_REPRESENTATION_ROUTING_NODE_COSTFACTOR =
		"adora.representation.routing.nodeCostFactor";
	
	/**
	 * The factor the costs (i.e. distance) are multiplied by if the line
	 * goes through an abstracted node.
	 */
	public static final String ADORA_REPRESENTATION_ROUTING_ABSTRACT_NODE_COSTFACTOR =
		"adora.representation.routing.abstractNodeCostFactor";
	
	/**
	 * The factor the costs (i.e. distance) are multiplied by if the line goes
	 * through a tile that is occupied by a node. This is used for the calculation
	 * of the start and end point of the line.
	 */
	public static final String ADORA_REPRESENTATION_ROUTING_LINE_COSTFACTOR =
		"adora.representation.routing.lineCostFactor";

	/**
	 * Constant for the preference to paint the tile structure.
	 */
	public static final String ADORA_REPRESENTATION_ROUTING_SHOW_TILES = 
		"adora.representation.tilestructure.showTiles";
	
	/**
	 * Constant to color the costs in the tile structure.
	 */
	public static final String ADORA_REPRESENTATION_ROUTING_COLOR_TILES =
		"adora.representation.tilestructure.colorTiles";
	
	/**
	 * Constant for the preference to print the line paths on the console.
	 */
	public static final String ADORA_REPRESENTATION_ROUTING_PRINT_SOLUTIONS =
		"adora.representation.routing.printSolutions";
	
	/**
	 * Constant for preference option to enable automatic label placement
	 */
	public static final String ADORA_REPRESENTATION_ROUTING_PLACE_LABELS =
		"adora.representation.routing.placeLabels";
	
	/**
	 * Constant to color show possible line label positions.
	 */
	public static final String ADORA_REPRESENTATION_ROUTING_SHOW_LABEL_POSITIONS =
		"adora.representation.routing.showLabelPositions";
	
	/**
	 * The font of the node labels.
	 */
	public static final String ADORA_VIEW_NODE_LABEL_FONT =
		"adora.view.nodeLabelFont";
	
	/**
	 * The font of the connection labels.
	 */
	public static final String ADORA_VIEW_CONNECTION_LABEL_FONT =
		"adora.view.connectionLabelFont";
	
	/**
	 * The size of the transitions' endpoints' arrow
	 */
	public static final String ADORA_VIEW_TRANSITION_ARROW_SIZE =
		"adora.view.transitionArrowSize";
	
	/**
	 * Shade the hierarchy
	 */
	public static final String ADORA_VIEW_SHADE_HIERARCHY =
		"adora.view.shadeHierarchy";
	
	/**
	 * The color to shade the hierarchy with
	 */
	public static final String ADORA_VIEW_SHADE_COLOR =
		"adora.view.shadeColor";
	
	/**
	 * Should movement vectors be drawn when a layout algorithm moves an element?
	 */
	public static final String ADORA_REPRESENTATION_LAYOUT_SHOW_ARROWS = 
		"adora.representation.layout.showArrows";
	
	/**
	 * Should the original positions be visualized when a layout algorithm moves 
	 * a component?
	 */
	public static final String ADORA_REPRESENTATION_LAYOUT_SHOW_ORIGINAL_POSITIONS = 
		"adora.representation.layout.originalPositions";
	
	/**
	 * Should the original positions be visualized when a layout algorithm moves 
	 * all children of a component?
	 */
	public static final String ADORA_REPRESENTATION_LAYOUT_SHOW_ORIGINAL_CHILD_POSITIONS = 
		"adora.representation.layout.originalChildPositions";
	
	/**
	 * Should occlusions detected by layout algorithms be visualized?
	 */
	public static final String ADORA_REPRESENTATION_LAYOUT_SHOW_OCCLUSIONS = 
		"adora.representation.layout.showOcclusions";
	
	
	/**
	 * If true, models get saved before executing critical commands.
	 */
	public static final String ADORA_SAVE_MODEL_BEFORE_COMMAND = 
		"adora.command.savablecommand.save";
	
	/**
	 * Show the horizontal intervals of the continuous zoom
	 */
	public static final String ADORA_REPRESENTATION_LAYOUT_HORIZONTAL_INTERVALS =
		"adora.representation.layout.horizontalIntervals";
	
	/**
	 * The color of the horizontal intervals visualization
	 */
	public static final String ADORA_REPRESENTATION_LAYOUT_HORIZONTAL_INTERVALS_COLOR =
		"adora.representation.layout.horizontalIntervals.color";
	
	/**
	 * Show the vertical intervals of the continuous zoom
	 */
	public static final String ADORA_REPRESENTATION_LAYOUT_VERTICAL_INTERVALS =
		"adora.representation.layout.verticalIntervals";
	
	/**
	 * The color of the vertical intervals visualization
	 */
	public static final String ADORA_REPRESENTATION_LAYOUT_VERTICAL_INTERVALS_COLOR =
		"adora.representation.layout.verticalIntervals.color";
	
	/**
	 * Show the zoom holes of the continuous zoom
	 */
	public static final String ADORA_REPRESENTATION_LAYOUT_ZOOM_HOLES =
		"adora.representation.layout.zoomHoles";
	
	/**
	 * The color of the zoom holes visualization
	 */
	public static final String ADORA_REPRESENTATION_LAYOUT_ZOOM_HOLES_COLOR =
		"adora.representation.layout.zoomHoles.color";
	
	/**
	 * Insert new nodes in a centered way
	 */
	public static final String ADORA_REPRESENTATION_LAYOUT_INSERT_CENTERED =
		"adora.representation.layout.insertCentered";
	
	/**
	 * Animate the fisheye zooming
	 */
	public static final String ADORA_REPRESENTATION_LAYOUT_ANIMATE_ZOOM =
		"adora.representation.layout.animateZoom";
	
	/**
	 * Animate the fisheye zooming
	 */
	public static final String ADORA_REPRESENTATION_LAYOUT_ZOOM_ANIMATION_DELAY =
		"adora.representation.layout.zoomAnimationDelay";
	
	/**
	 * Scale gap intervals during filter operations.
	 */
	public static final String ADORA_REPRESENTATION_LAYOUT_SCALE_GAPS =
		"adora.representation.layout.scaleGaps";
	
}
