package ch.unizh.ifi.adora.ui.preferences;

import org.eclipse.core.runtime.preferences.AbstractPreferenceInitializer;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferenceConverter;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.widgets.Shell;

import ch.unizh.ifi.adora.AdoraPlugin;

/**
 * This class initializes the default values for the ADORA plugin preferences.
 *
 * @author		          treinhard
 * @copyright           Requirements Engineering Research Group<br>
 *                        Department of Informatics<br>
 *                        University of Zurich<br>
 * @history              Feb 13, 2006 TR First Version
 *                        May 20, 2006 SM Included code for join relationship router.
 * @version              $Date $, $Author $, $Revision $
 * @responsibilities	  Initializes the default values for the preferences
 *
 */
public class PreferenceInitializer extends AbstractPreferenceInitializer {

	/**
	 * @see org.eclipse.core.runtime.preferences.AbstractPreferenceInitializer#initializeDefaultPreferences()
	 */
	public void initializeDefaultPreferences() {
		IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
		
		// The default layout algorithms
		store.setDefault(IPreferenceNameConstants.ADORA_REPRESENTATION_ZOOM_ALGORITHM, IPreferenceValueConstants.ADORA_REPRESENTATION_ZOOM_CONTINUOUS);
		store.setDefault(IPreferenceNameConstants.ADORA_REPRESENTATION_INSERT_LAYOUT_ALGORITHM, IPreferenceValueConstants.ADORA_REPRESENTATION_ZOOM_CONTINUOUS);
		store.setDefault(IPreferenceNameConstants.ADORA_REPRESENTATION_REMOVE_LAYOUT_ALGORITHM, IPreferenceValueConstants.ADORA_REPRESENTATION_ZOOM_CONTINUOUS);
		store.setDefault(IPreferenceNameConstants.ADORA_REPRESENTATION_MOVE_LAYOUT_ALGORITHM, IPreferenceValueConstants.ADORA_REPRESENTATION_ZOOM_CONTINUOUS);
		store.setDefault(IPreferenceNameConstants.ADORA_REPRESENTATION_FILTER_LAYOUT_ALGORITHM, IPreferenceValueConstants.ADORA_REPRESENTATION_ZOOM_CONTINUOUS);
		store.setDefault(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_INSERT_CENTERED, true);
		
        // The default line routers for the different connection types.
        store.setDefault(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTER_ASSOCIATION, IPreferenceValueConstants.ADORA_REPRESENTATION_ROUTER_NEWORTHOGONAL);
        store.setDefault(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTER_TRANSITION, IPreferenceValueConstants.ADORA_REPRESENTATION_ROUTER_NEWORTHOGONAL);
        store.setDefault(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTER_SCENARIOCONNECTION, IPreferenceValueConstants.ADORA_REPRESENTATION_ROUTER_SHORTESTPATH);
        store.setDefault(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTER_JOIN_RELATIONSHIP, IPreferenceValueConstants.ADORA_REPRESENTATION_ROUTER_DIRECTLINE);
        
        // Additional properties of the orthogonal tile maze router
        store.setDefault(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_NODE_COSTFACTOR, "6.0");
        store.setDefault(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_ABSTRACT_NODE_COSTFACTOR, "1.0");
		store.setDefault(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_LINE_COSTFACTOR, "60.0");
		store.setDefault(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_SHOW_TILES, false);
		
        // Size of the arrowheads of the transitions
		store.setDefault(IPreferenceNameConstants.ADORA_VIEW_TRANSITION_ARROW_SIZE, 8);
        
        // Default fonts for the labels
        GC gc = new GC(new Shell());
        FontData fontData = gc.getFont().getFontData()[0];
        PreferenceConverter.setDefault(store, IPreferenceNameConstants.ADORA_VIEW_CONNECTION_LABEL_FONT, fontData);
        PreferenceConverter.setDefault(store, IPreferenceNameConstants.ADORA_VIEW_NODE_LABEL_FONT, fontData);
        
        // Default values for the shading of the hierarchy
        store.setDefault(IPreferenceNameConstants.ADORA_VIEW_SHADE_HIERARCHY, true);
        PreferenceConverter.setDefault(store, IPreferenceNameConstants.ADORA_VIEW_SHADE_COLOR, ColorConstants.menuBackground.getRGB());
        
        // Default values for the visualization of the interval structure
        PreferenceConverter.setDefault(store, IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_HORIZONTAL_INTERVALS_COLOR, ColorConstants.lightGray.getRGB());
        PreferenceConverter.setDefault(store, IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_VERTICAL_INTERVALS_COLOR, ColorConstants.lightGray.getRGB());
        PreferenceConverter.setDefault(store, IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_ZOOM_HOLES_COLOR, ColorConstants.red.getRGB());
        
        // Zoom animation
        store.setDefault(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_ANIMATE_ZOOM, true);
        store.setDefault(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_ZOOM_ANIMATION_DELAY, 250);
        
        // The fisheye zoom is enabled by default
        store.setDefault(IPreferenceNameConstants.ADORA_REPRESENTATION_FISHEYE_DISABLED, false);
        // The view mechanism is enabled by default
        store.setDefault(IPreferenceNameConstants.ADORA_REPRESENTATION_VIEWS_DISABLED, false);
        // Scaling of gap intervals during filter operations is disabled by default
        store.setDefault(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_SCALE_GAPS, false);
        
	}
}
