
package ch.unizh.ifi.adora.ui.preferences;

import org.eclipse.jface.preference.BooleanFieldEditor;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.preference.RadioGroupFieldEditor;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

import ch.unizh.ifi.adora.AdoraPlugin;

/**
 * This class creates the preferences page for the layout algorithms.
 * 
 * @author              schlaefli
 * @copyright          Requirements Engineering Research Group<br>
 *                       Department of Informatics<br>
 *                       University of Zurich<br>
 * @history             May 24, 2006 JS First Version
 *                      May 31, 2006 JS added ADORA_REPRESENTATION_LAYOUT_SHOW_ORIGINAL_CHILD_POSITIONS
 * @version             $Date: 2007-09-06 12:27:36 +0200 (Thu, 06 Sep 2007) $, $Author: treinhard $, $Revision: 2941 $
 * @responsibilities   Preference page of the ADORA plugin
 *
 */
public class ZoomAlgorithmPreferencePage extends FieldEditorPreferencePage implements IWorkbenchPreferencePage {
	
	public ZoomAlgorithmPreferencePage() {
		super(GRID);
		setPreferenceStore(AdoraPlugin.getDefault().getPreferenceStore());
		setDescription("Zoom preferences");
	}
	
	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.jface.preference.FieldEditorPreferencePage#createFieldEditors()
	 */
	protected void createFieldEditors() {
		
		addField(
				new RadioGroupFieldEditor(
					IPreferenceNameConstants.ADORA_REPRESENTATION_ZOOM_ALGORITHM,
					"&Zooming algorithm",
					1,
					new String[][] {
				     				{"New Berner", IPreferenceValueConstants.ADORA_REPRESENTATION_ZOOM_BERNER},
				     				{"Continuous Zoom", IPreferenceValueConstants.ADORA_REPRESENTATION_ZOOM_CONTINUOUS}
				     			},
				    getFieldEditorParent()));
		
		addField(
				new RadioGroupFieldEditor(
					IPreferenceNameConstants.ADORA_REPRESENTATION_INSERT_LAYOUT_ALGORITHM,
					"Node &insert layout algorithm",
					1,
					new String[][] {
				     				{"Minimal Radial", IPreferenceValueConstants.ADORA_REPRESENTATION_ZOOM_MINRADIAL },
				     				{"Continuous Zoom", IPreferenceValueConstants.ADORA_REPRESENTATION_ZOOM_CONTINUOUS }
				     			},
				    getFieldEditorParent()));
		
		addField(
				new RadioGroupFieldEditor(
					IPreferenceNameConstants.ADORA_REPRESENTATION_REMOVE_LAYOUT_ALGORITHM,
					"Node &remove layout algorithm",
					1,
					new String[][] {
									{"New Berner", IPreferenceValueConstants.ADORA_REPRESENTATION_ZOOM_BERNER },
									{"Continuous Zoom", IPreferenceValueConstants.ADORA_REPRESENTATION_ZOOM_CONTINUOUS }
				     			},
				    getFieldEditorParent()));
		
		addField(
				new RadioGroupFieldEditor(
					IPreferenceNameConstants.ADORA_REPRESENTATION_MOVE_LAYOUT_ALGORITHM,
					"Node &move layout algorithm",
					1,
					new String[][] {
									{"Minimal Radial", IPreferenceValueConstants.ADORA_REPRESENTATION_ZOOM_MINRADIAL },
									{"Continuous Zoom", IPreferenceValueConstants.ADORA_REPRESENTATION_ZOOM_CONTINUOUS }
				     			},
				    getFieldEditorParent()));
		
		addField(
				new RadioGroupFieldEditor(
					IPreferenceNameConstants.ADORA_REPRESENTATION_FILTER_LAYOUT_ALGORITHM,
					"&Filter layout algorithm",
					1,
					new String[][] {
									{"New Berner", IPreferenceValueConstants.ADORA_REPRESENTATION_ZOOM_BERNER },
									{"Continuous Zoom", IPreferenceValueConstants.ADORA_REPRESENTATION_ZOOM_CONTINUOUS }
				     			},
				    getFieldEditorParent()));
		
		addField(
				new BooleanFieldEditor(
					IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_INSERT_CENTERED,
					"Insert new nodes centered",
					getFieldEditorParent()));

	}
	
	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
	 */
	public void init(IWorkbench workbench) {
	}
}