package ch.unizh.ifi.adora.ui.views.syntaxerrors;

import org.eclipse.core.runtime.IAdapterFactory;

import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.syntaxerrors.SyntaxProblem;
import ch.unizh.ifi.adora.model.syntaxerrors.rules.JoinRelationContextMapSyntaxRule;
import ch.unizh.ifi.adora.model.syntaxerrors.rules.SpecificationSyntaxRule;
import ch.unizh.ifi.adora.properties.contextmap.ContextMapEditorInput;
import ch.unizh.ifi.adora.properties.contextmap.JoinRelationContextMapPropertyAccess;
import ch.unizh.ifi.adora.properties.longtextproperty.ILongTextEditorInput;
import ch.unizh.ifi.adora.properties.specification.AspectContainerSpecificationPropertyAccess;
import ch.unizh.ifi.adora.properties.specification.ComponentSpecificationPropertyAccess;
import ch.unizh.ifi.adora.properties.specification.SpecificationEditorInput;

/** 
 * This is an auxiliary class for {@link SyntaxErrorsView}.
 * 
 * It is an {@link IAdapterFactory} that build
 * {@link ILongTextEditorInput} from a {@link SyntaxProblem}
 * if this entry refers to a long text property (e.g. specification)
 * 
 * @author Cedric Jeanneret
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history   24.04.2008 CJ First Version (refactoring of syntax error view)
 * @version   $Date: 2008-04-24 18:22:06 +0200 (Thu, 24 Apr 2008) $, $Author: jeanneret $, $Revision: 3619 $
 *
 */
@SuppressWarnings("unchecked")
class EditorInputAdapterFactory implements IAdapterFactory {

	/**
	 * The list of {@link Class} this {@link IAdapterFactory}
	 * can build.
	 */
	private static final Class[] ADAPTER_LIST = {ILongTextEditorInput.class};
	
	/* (non-Javadoc)
	 * @see org.eclipse.core.runtime.IAdapterFactory#getAdapter(java.lang.Object, java.lang.Class)
	 */
	public Object getAdapter(Object adaptableObject, Class adapterType)
	{
		// TODO: It should be possible
		// to obtain an IPropertySource and ask
		// it for the access property
		
		Object adaptedObject = null;
		if(adapterType.isAssignableFrom(SpecificationEditorInput.class))
		{
			if (adaptableObject instanceof SyntaxProblem) {
				SyntaxProblem entry = (SyntaxProblem) adaptableObject;
				if(SpecificationSyntaxRule.RULE_NAME.equals(entry.getSyntaxRuleName()))
				{
					if(entry.getModelElement() instanceof Component)
					{
						adaptedObject = new SpecificationEditorInput(new ComponentSpecificationPropertyAccess((Component) entry.getModelElement()));
					}
					else if(entry.getModelElement() instanceof AspectContainer)
					{
						adaptedObject = new SpecificationEditorInput(new AspectContainerSpecificationPropertyAccess((AspectContainer) entry.getModelElement()));
					}
				}
				if(JoinRelationContextMapSyntaxRule.RULE_NAME.equals(entry.getSyntaxRuleName()))
				{
					if(entry.getModelElement() instanceof JoinRelation)
					{
						adaptedObject = new ContextMapEditorInput(new JoinRelationContextMapPropertyAccess((JoinRelation) entry.getModelElement()));
					}
				}
			}
		}
		return adaptedObject;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.runtime.IAdapterFactory#getAdapterList()
	 */
	public Class[] getAdapterList()
	{
		return ADAPTER_LIST;
	}
}
