package ch.unizh.ifi.adora.ui.views.syntaxerrors;

import ch.unizh.ifi.adora.model.ModelElement;

/**
 * This is a container tree node which can also contain a model object. In the current implementation
 * each node denoting a separate problem (e.g., in a transition, functional specification, etc.) contains
 * a model object.
 *      
 *
 * @author				smeier
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				21.09.2007 SM First Version due to the heavy refactoring.
 * 						24.01.2008 CR total refactoring.
 * @version				$Date: 2008-01-24 13:08:40 +0100 (Thu, 24 Jan 2008) $, $Author: cramer $, $Revision: 3498 $
 *
 */
public class ModelContainerTreeNode extends ContainerTreeNode {

	/**
	 * The model object
	 */
	private Object model;
	
	/**
	 * Constant for the relative location of the transition icon.
	 */
	public static final String TRANSITION_IMAGE="icons/transition16.gif";
	
	/**
	 * Constant for the relative location of the rectangle icon.
	 */
	public static final String COMPONENT_IMAGE="icons/rectangle16.gif";
	
	

	/**
	 * Constant for the relative location an ellipse icon.
	 */
	public static final String SCENARIO_IMAGE="icons/ellipse16.gif";
	
	/**
	 * Constant for the relative location an aspectcontainericon.
	 */
	public static final String ASPECTCONTAINER_IMAGE="icons/AspectContainer.gif";

	public static final String JOINTRELATION_IMAGE = "icons/JoinRelation16.gif";
	
	/**
	 * Creates a new model container tree node which has a parent, a description and object and an image.
	 * If the parent is <i>null</i> the object is a root node.
	 * 
	 * @param parent The parent node.
	 * @param description The description of this tree node.
	 * @param model The associated model element.
	 * @param imageName The name of the image which should be displayed.
	 */
	public ModelContainerTreeNode(ContainerTreeNode parent, String description, Object model, String imageName) {
		super(parent, description, imageName);
		setModel(model);
		// TODO Auto-generated constructor stub
	}

	/**
	 * Returns the associated model element.
	 * @return The associated model element.
	 */
	public Object getModel() {
		return model;
	}
	
	public Object getModelElement() {
		return getModel();
	}

	/**
	 * Sets the associated model element.
	 * @param model The model element to set.
	 */
	public void setModel(Object model) {
		this.model = model;
	}

	/**
	 * Overrides the equals method and enables the comparison with various different types of objects.
	 * This method adheres to the following rules:
	 * <ol>
	 *   <li>If the given argument object is of the type {@link ch.unizh.ifi.adora.ui.views.syntaxerrors.ModelContainerTreeNode},
	 *       the model object of the given node and the model object this node are compared by reference, i.e., it is 
	 *       returned true if they are identical.</li>
	 *   <li>If the given argument object is of the type {@link ch.unizh.ifi.adora.model.ModelElement},
	 *       the model object and the model object this node are compared by reference, i.e., it is 
	 *       returned true if they are identical.</li>
	 *   <li>If the given argument object is of the type {@link java.lang.String},
	 *       the description of this node is compred with the given string and returned as result.</li>
	 *   <li>In all other cases, false is returned.</li>    
	 * </ol>
	 */
	@Override
	public boolean equals(Object obj) {
		boolean retVal = false;
		if (obj instanceof ModelContainerTreeNode) {
			retVal = ((ModelContainerTreeNode) obj).getModel() == this.getModel();
		} else if (obj instanceof ModelElement) {
			retVal = obj == this.getModel();
		} else if (obj instanceof String) {
			retVal = obj.equals(this.getDescription());
		} else {
			retVal = false;
		}
		return retVal;	
	} 
}
