package ch.unizh.ifi.adora.ui.views.syntaxerrors;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;

import ch.unizh.ifi.adora.model.IContainer;
import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.properties.ComponentPropNames;
import ch.unizh.ifi.adora.model.properties.ConnectableElementPropNames;
import ch.unizh.ifi.adora.model.properties.ScenarioPropNames;
import ch.unizh.ifi.adora.model.properties.VisualModElemPropNames;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.ObservableModelElement;
import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.model.representation.State;
import ch.unizh.ifi.adora.model.representation.Transition;
import ch.unizh.ifi.adora.model.representation.TransitionDescription;

/**
 * 
 *      
 *
 * @author				smeier
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			05.11.2007 SM First Version
 * 					07.12.2007 CR changes so SyntaxChecker acts as a listener. 
 * 					11.12.2007 CR some refactoring. Nothing essential. 
 * @version			$Date: 2008-01-23 11:17:41 +0100 (Wed, 23 Jan 2008) $, $Author: cramer $, $Revision: 3480 $
 *
 */
public class SyntaxChecker implements PropertyChangeListener {

	private SyntaxProblemsView view;

	private Hashtable<ModelElement,ModelContainerTreeNode> elementsToTreeNodes;
	
	/**
	 * PropertyNames of changes that should cause a rechecking of the modelelement.
	 */
	private final String[] listeningChangesArray={JoinRelation.CONTEXT_MAP_PROP,AspectContainer.ASPECT_SPECIFICATION_PROP,ScenarioPropNames.TRANSFORMATION_PROP, ScenarioPropNames.GUARD_PROP,ScenarioPropNames.ITERATOR_PROP,ComponentPropNames.ADORA_SPECIFICATION_PROP,VisualModElemPropNames.NAME_PROP};
	private Vector<String> listeningChanges;
	
	public SyntaxChecker(SyntaxProblemsView view, AdoraModel model) {
		listeningChanges=new Vector<String>();
		for(String s:listeningChangesArray){
			listeningChanges.add(s);
		}
		this.view = view;
		
	}
	
	/**
	 * Calls the right methods depending on what property of a model element changed. 
	 * If a new element was created, SyntaxChecker is added as a listener. 
	 * If a model element was deleted, SyntaxProblemsView is informed. 
	 * If name of element changed, SyntaxProblemsView is informed. 
	 */
	public void propertyChange(PropertyChangeEvent evt) {	
		if(evt.getPropertyName().equals(VisualModElemPropNames.NAME_PROP)&&this.elementsToTreeNodes.get((ModelElement)evt.getSource())!=null){
			view.changeName(this.elementsToTreeNodes.get((ModelElement)evt.getSource()), evt.getNewValue());
		}
		//new element, just add (at the beginning, it is definitely correct!
		if(evt.getPropertyName().equals(ConnectableElementPropNames.TARGETCONNECTION_NEW_PROP)||evt.getPropertyName().equals(IContainer.CHILD_ADDED_PROP)){
			ModelElement me=(ModelElement)evt.getNewValue();
			this.subscribeListener(me);
		}
		//exists, was deleted --> remove
		else if(evt.getPropertyName().equals(VisualModElemPropNames.WAS_DELETED_PROP)&&this.elementsToTreeNodes.get((ModelElement)evt.getSource())!=null){
			view.informDeleted(evt.getSource(),this.elementsToTreeNodes.get((ModelElement)evt.getSource()));
			
		}
		//exists, was changed
		else if(listeningChanges.contains(evt.getPropertyName())&&this.elementsToTreeNodes.get((ModelElement)evt.getSource())!=null){
			TreeNode newNode= view.checkForSyntaxErrors((ModelElement)evt.getSource()); //checks if exisiting node was deleted (because was false, is true now
		if(newNode==null){
				this.elementsToTreeNodes.remove((ModelElement)evt.getSource());}
		}
		else if(listeningChanges.contains(evt.getPropertyName())&&this.elementsToTreeNodes.get((ModelElement)evt.getSource())==null){//no node yet, create it
			ModelContainerTreeNode newNode= view.checkForNewSyntaxErrors((ModelElement)evt.getSource());
			if(newNode!=null){//add the new node to the listening things
				this.elementsToTreeNodes.put((ModelElement)evt.getSource(),newNode);
			}
				
			}
		}
		
	

	
	/**
	 * Initializing the listening process as well as sending all relevant model elements for a Syntax Check. 
	 * @param model
	 */
	public void CheckAndInitSyntaxViewForModel(AdoraModel model) {	
		elementsToTreeNodes=new Hashtable<ModelElement,ModelContainerTreeNode>();
		ModelContainerTreeNode newNode=null;
		model.addPropertyChangeListener(this);
		Iterator<ModelElement> modelelements = model.getRecursiveChildrenIterator();
		while (modelelements.hasNext()) {
			ModelElement me = modelelements.next();
			subscribeListener(me);
			if (me instanceof Component) {							
				newNode=view.checkForNewSyntaxErrors(me);
				
			}
			if (me instanceof Scenario) {
				newNode=view.checkForNewSyntaxErrors(me);
			}
			if (me instanceof AspectContainer) {
				newNode=view.checkForNewSyntaxErrors(me);
			}
			if(newNode!=null){
				elementsToTreeNodes.put(me, newNode);
			}
			
			if (me instanceof State || me instanceof AspectContainer ||
				me instanceof Component) {
				
				ConnectableElement ce = (ConnectableElement)me;
				for (Connection c : ce.getSourceConnections()) {
					if (c instanceof Transition) {
						TransitionDescription label = ((Transition)c).getLabel();
						subscribeListener(c);
						newNode=view.checkForNewSyntaxErrors(label);
						if(newNode!=null){
							elementsToTreeNodes.put(label, newNode);
						}
					}
					else if(c instanceof JoinRelation){
						subscribeListener(c);
						newNode=view.checkForNewSyntaxErrors(c);
						if(newNode!=null){
							elementsToTreeNodes.put(c, newNode);
						}
					}
				}
			}			
		}				
	}	
	
	/**
	 * Subscribes a listener for this syntax checker for the nodes that can potentially 
	 * contain elements that have to be checked.
	 * @param me The model element to subscribe at.
	 */
	protected void subscribeListener(ModelElement me) {
		if ((me instanceof State) ||
			(me instanceof Component) || 
			 (me instanceof Scenario) || 
			 (me instanceof JoinRelation) || 
			(me instanceof AspectContainer)) {
			((ObservableModelElement)me).addPropertyChangeListener(this);
		}
		if ((me instanceof Transition)) {
			Transition t = ((Transition)me);
			t.getLabel().addPropertyChangeListener(this);
			t.addPropertyChangeListener(this);
		}
	}
	
	
	/**
	 * Returns the TreeNode for that model element. 
	 * @param e model element
	 * @return ModelContainerTreeNode
	 */
	public ModelContainerTreeNode getTreeNode(ModelElement e){
		return elementsToTreeNodes.get(e);
	}

	
	
	
}
