package ch.unizh.ifi.adora.ui.views.syntaxerrors;

import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.ui.PartInitException;

import ch.unizh.ifi.adora.model.syntaxerrors.SyntaxProblem;
import ch.unizh.ifi.adora.properties.longtextproperty.ILongTextEditorInput;

/**
 * This class is an auxiliary class for {@link SyntaxErrorsView}.
 * It provides {@link MouseListener} services to it.
 * 
 * Upon double-click on an {@link SyntaxProblem}, it opens a suitable
 * editor (e.g. specification).
 * 
 * @author Cedric Jeanneret
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history   24.04.2008 CJ Refactoring of syntax error view
 * @version   $Date: 2008-04-24 18:22:06 +0200 (Thu, 24 Apr 2008) $, $Author: jeanneret $, $Revision: 3619 $
 */
class SyntaxErrorsMouseListener implements MouseListener {

	/**
	 * The {@link SyntaxErrorsView} this object assists
	 */
	private SyntaxErrorsView view;

	/**
	 * Constructor for this class.
	 * Should be called by {@link SyntaxErrorsView} only
	 * @param view this object has to assist
	 */
	SyntaxErrorsMouseListener(SyntaxErrorsView view)
	{
		this.view = view;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.MouseListener#mouseDoubleClick(org.eclipse.swt.events.MouseEvent)
	 */
	public void mouseDoubleClick(MouseEvent e) {
		Object selectedElement = (view.getSelection()).getFirstElement();
		if(selectedElement != null && selectedElement instanceof SyntaxProblem)
		{
			SyntaxProblem entry = (SyntaxProblem) selectedElement;
			ILongTextEditorInput editorInput = (ILongTextEditorInput) entry.getAdapter(ILongTextEditorInput.class);
			if(editorInput != null)
			{
    			try
    			{
        			view.getCurrentEditor().getSite().getPage().openEditor(editorInput, editorInput.getLongTextEditorID());
        		}
    			catch (PartInitException e1) {
        			e1.printStackTrace();
        		}
			}
		}		
	}
	
	/**
	 * Unimplemented methods of {@link MouseListener}
	 */

	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.MouseListener#mouseDown(org.eclipse.swt.events.MouseEvent)
	 */
	public void mouseDown(MouseEvent e) {}

	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.MouseListener#mouseUp(org.eclipse.swt.events.MouseEvent)
	 */
	public void mouseUp(MouseEvent e) {}
}
