package ch.unizh.ifi.adora.ui.views.syntaxerrors;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.IWorkbenchPart;

import ch.unizh.ifi.adora.ui.AdoraModelEditor;

/**
 * This class is an auxiliary class for {@link SyntaxErrorsView}.
 * It provides {@link IPartListener} services for it.
 * 
 * Once an {@link AdoraModelEditor} becomes active, this
 * listener update the content of {@link SyntaxErrorsView}.
 * 
 * @author Cedric Jeanneret
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history   24.04.2008 CJ Refactoring of syntax error view
 * @version   $Date: 2008-04-24 18:22:06 +0200 (Thu, 24 Apr 2008) $, $Author: jeanneret $, $Revision: 3619 $
 */
class SyntaxErrorsPartListener implements IPartListener {

	/**
	 * The {@link SyntaxErrorsView} this object assists
	 */
	private SyntaxErrorsView view;

	/**
	 * Constructor for this class.
	 * Should be called by {@link SyntaxErrorsView} only
	 * @param view this object has to assist
	 */
	SyntaxErrorsPartListener(SyntaxErrorsView view)
	{
		this.view = view;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IPartListener#partActivated(org.eclipse.ui.IWorkbenchPart)
	 */
	public void partActivated(IWorkbenchPart part)
	{
		if(part instanceof AdoraModelEditor)
		{
			if(part != view.getCurrentEditor())
			view.setCurrentEditor((AdoraModelEditor)part);
		}
		else if(part instanceof IEditorPart)
		{
			// TODO: Should we clean the view if an external editor is activated ?
			// view.setCurrentEditor(null);
		}		
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.IPartListener#partClosed(org.eclipse.ui.IWorkbenchPart)
	 */
	public void partClosed(IWorkbenchPart part)
	{
		if(part == view.getCurrentEditor())
		{
			view.setCurrentEditor(null);
		}
	}
	
	/**
	 * Unimplemented methods of {@link IPartListener}
	 */
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.IPartListener#partDeactivated(org.eclipse.ui.IWorkbenchPart)
	 */
	public void partDeactivated(IWorkbenchPart part) {}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IPartListener#partBroughtToTop(org.eclipse.ui.IWorkbenchPart)
	 */
	public void partBroughtToTop(IWorkbenchPart part) {}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IPartListener#partOpened(org.eclipse.ui.IWorkbenchPart)
	 */
	public void partOpened(IWorkbenchPart part) {}
}
