package ch.unizh.ifi.adora.ui.views.syntaxerrors;

import java.util.ArrayList;
import java.util.Iterator;

import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;

import ch.unizh.ifi.adora.model.syntaxerrors.SyntaxProblem;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;

/**
 * This class is an auxiliary class for {@link SyntaxErrorsView}.
 * It provides {@link ISelectionChangedListener} services, and is therefore
 * responsible to forward the selection from the {@link SyntaxErrorsView}
 * to {@link AdoraModelEditor}.
 * 
 * @author Cedric Jeanneret
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history   24.04.2008 CJ Refactoring of syntax error view
 * @version   $Date: 2008-04-24 18:22:06 +0200 (Thu, 24 Apr 2008) $, $Author: jeanneret $, $Revision: 3619 $
 */
class SyntaxErrorsSelectionListener implements ISelectionChangedListener {

	/**
	 * The {@link SyntaxErrorsView} this object assists
	 */
	private SyntaxErrorsView view;

	/**
	 * Constructor for this class.
	 * Should be called by {@link SyntaxErrorsView} only
	 * @param view this object has to assist
	 */
	SyntaxErrorsSelectionListener(SyntaxErrorsView view)
	{
		this.view = view;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ISelectionChangedListener#selectionChanged(org.eclipse.jface.viewers.SelectionChangedEvent)
	 */
	@SuppressWarnings("unchecked")
	public void selectionChanged(SelectionChangedEvent event)
	{
		if(!event.getSelection().isEmpty())
		{
    		ArrayList<Object> selectedModelElements = new ArrayList<Object>();
    		
    		if (event.getSelection() instanceof IStructuredSelection) {
    			IStructuredSelection selection = (IStructuredSelection) event.getSelection();
    			for(Iterator it = selection.iterator(); it.hasNext();)
    			{
    				Object selectedElement = it.next();
    				if (selectedElement instanceof SyntaxProblem) {
    					SyntaxProblem entry = (SyntaxProblem) selectedElement;
    					selectedModelElements.add(entry.getModelElement());
    				}
    			}
    		}
    		AdoraModelEditor editor = view.getCurrentEditor();
    		if(editor != null)
    		{
    			editor.selectElements(selectedModelElements);
    			// bring the editor to front
    			editor.getSite().getPage().activate(editor);
    		}
		}
	}
}
