package ch.unizh.ifi.adora.ui.views.syntaxerrors;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;

import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeViewerListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeExpansionEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.ui.IStartup;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.model.representation.TransitionDescription;
import ch.unizh.ifi.adora.properties.longtextproperty.ILongTextPropertyAccess;
import ch.unizh.ifi.adora.properties.specification.AspectContainerSpecificationPropertyAccess;
import ch.unizh.ifi.adora.properties.specification.ComponentSpecificationPropertyAccess;
import ch.unizh.ifi.adora.properties.specification.SpecificationEditorInput;
import ch.unizh.ifi.adora.properties.specification.SpecificationTextEditor;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;

/**
 * Treeview showing syntax validation problems.
 * 
 * @author Pascal Schoeni
 * @author smeier$
 * @copyright Requirements Engineering sResearch Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 21.09.2007 SM Heavy refactoring 02.10.2007 SM Bug #824 processed.
 *          05.10.2007 SM Bug #840. 07.12.2007 CR changes to decouple from
 *          Properties (ch.unizh.ifi.adora.properties).
 *          11.12.2007 CR lots of refactoring. 
 *          24.01.2008 CR fixing some errors. 
 *          09.04.2008 CJ Fix bug 855
 * @version $Date: 2009-05-06 10:51:39 +0200 (Wed, 06 May 2009) $, $Author: treinhard $, $Revision: 4182 $
 */
public class SyntaxProblemsView extends ViewPart implements IStartup {
	/**
	 * The eclipse id of this view.
	 */
	public static final String VIEW_ID = "ch.unizh.ifi.adora.editor.views.SyntaxProblemsView";

	/**
	 * root node of tree. 
	 */
	ContainerTreeNode root = new ContainerTreeNode("ADORA Parseerrors");

	/**
	 * Represents the default, empty editor.
	 */
	public static final Object NO_EDITOR_CHOSEN = "DEFAULT";

	/**
	 * The initialization value for associating the tree view with an object. It
	 * is needed in the case that the syntax problem tree view is opened without
	 * an active ADORA editor.
	 */
	private Object currentEditor = NO_EDITOR_CHOSEN;

	/**
	 * Mapping from editor to the Hashtable which contains the information about tree items. 
	 */
	private Hashtable<Object, Hashtable> adoraEditorToProblemViewMapping = new Hashtable<Object, Hashtable>();

	/**
	 * Mapping from editor to SyntaxChecker. As the SyntaxChecker also contains information about
	 * the model elements that currently have a syntax problem, each editor has an own SyntaxChecker. 
	 */
	private Hashtable<Object, SyntaxChecker> adoraEditorToSyntaxChecker = new Hashtable<Object, SyntaxChecker>();

	/**
	 * The TreeViewer object used to visualize the syntax errors in the model.
	 */
	private TreeViewer treeViewer;

	/**
	 * Listener for changes in the pages. Reference needs to be stored to undo
	 * the the changes.
	 */
	private SyntaxProblemsViewChangeListener listener;

	/**
	 * The SyntaxChecker associated with this view. 
	 */
	private SyntaxChecker sc;

	/**
	 * Creates a new SyntaxProblemsView
	 */
	public SyntaxProblemsView() {

	}

	/**
	 * Intializes the listener for newly selected parts or selection in the
	 * active editor.
	 */
	void initListener() {
		listener = new SyntaxProblemsViewChangeListener(this);
		getSite().getPage().addPostSelectionListener(listener);
	}

	@Override
	/**
	 * creates the view of the tree. first tree has no elements adds a
	 * SelectionChangeListener to the tree
	 * 
	 * @param parent
	 *            The parent composite which is used to register this parts ui
	 *            controls.
	 */
	public void createPartControl(Composite parent) {
		treeViewer = new TreeViewer(parent);
		treeViewer.setContentProvider(new TreeContentProvider());
		treeViewer.setLabelProvider(new TreeLabelProvider());
		final Menu menu=new Menu(parent);
		treeViewer.getTree().setMenu(menu);
		MenuItem menuItem=new MenuItem(menu,SWT.NONE);
		menuItem.setText("SHOW ERROR");
		menu.setVisible(false);
		menuItem.addSelectionListener(new SelectionListener(){

			public void widgetDefaultSelected(SelectionEvent e) {
				
			}

			public void widgetSelected(SelectionEvent e) {
				
			}});
		// Not that nice this embedded anonymous classes
		// bloat the code, should be refactored.....

		// listens for tree viewer events.
		treeViewer.addTreeListener(new ITreeViewerListener() {

			public void treeCollapsed(TreeExpansionEvent event) {
				// 
			}

			public void treeExpanded(TreeExpansionEvent event) {
				TreeNode element = (TreeNode) event.getElement();
				treeViewer.expandToLevel(element, -1);
			}

		});
		treeViewer.getTree().addMouseListener(new MouseListener(){

			public void mouseDoubleClick(MouseEvent e) {
				// TODO Auto-generated method stub
				
			}

			public void mouseDown(MouseEvent e) {
			menu.setVisible(false);
			Object selection = ((IStructuredSelection)treeViewer.getSelection()).getFirstElement();
			if(selection instanceof LeafTreeNode&&!(selection instanceof ContainerTreeNode)){
				Object model=((LeafTreeNode)selection).getAncestorNodeModel();
				if(model instanceof Component || model instanceof AspectContainer){
					openFunctionalSpeficication(model);
					
									}
			}
			}

			

			public void mouseUp(MouseEvent e) {
				// TODO Auto-generated method stub
				
			}
			});
		// listens for selection events and scrolls to the corresponding
		// element.
		treeViewer.addSelectionChangedListener(new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent event) {

				if (event.getSelection().isEmpty()) {
					return;
				}

				if (!(event.getSelection() instanceof IStructuredSelection)) {
					return;
				}

				ArrayList<Object> selectedModelElements = new ArrayList<Object>();
				IStructuredSelection selection = (IStructuredSelection) event
						.getSelection();
				// LeafTreeNode treenode = (LeafTreeNode)
				// selection.getFirstElement();

				Iterator i = selection.iterator();
				while (i.hasNext()) {
					Object model = ((LeafTreeNode) (i.next()))
							.getModelElement();
					if (model != null)
						selectedModelElements.add(model);
				}

				AdoraModelEditor ame = AdoraPlugin.getActiveEditor();
				if (ame != null) {
					//ame.setFocus();
					ame.selectElements(selectedModelElements);

				}
			}
		});
		
		// register the change part listner.
		initListener();
	}

	/**
	 * This method checks a given model element that already has syntax errors for syntax errors. This method
	 * is called from the locations where the changes in the corresponding model
	 * elements are noted.
	 * 
	 * @param model The model element which contains the string to parse.
	 * @return TreeNode if errors were found, null otherwize. 
	 */
	public TreeNode checkForSyntaxErrors(ModelElement model) {// called when
		// already
		// exists
		TreeNode node = null;
		if (model instanceof TransitionDescription) {
			node = checkTransition((TransitionDescription) model, sc.getTreeNode(model));
		}
		if (model instanceof Scenario) {
			node = checkScenario((Scenario) model, sc.getTreeNode(model));
		}
		if (model instanceof Component) {
			node = checkComponent((Component) model, sc.getTreeNode(model));
		}
		if (model instanceof JoinRelation) {
			node = checkJoinRelation((JoinRelation) model, sc.getTreeNode(model));
		}
		if(model instanceof AspectContainer){
			node=this.checkAspectContainer((AspectContainer)model, sc.getTreeNode(model));
		}
			

		return node;

	}

	/**
	 * Opens the Functional Specification of a model element. 
	 * 
	 * @param model
	 */
	private void openFunctionalSpeficication(Object model) {
		ILongTextPropertyAccess specificationProperty = null;
		if(model instanceof Component){
		Component comp = (Component)model;
		specificationProperty = new ComponentSpecificationPropertyAccess(comp);
		}
		else if(model instanceof AspectContainer){
		AspectContainer aspect =(AspectContainer)model;
		specificationProperty = new AspectContainerSpecificationPropertyAccess(aspect);
		}
		try {
		SpecificationEditorInput input = new SpecificationEditorInput(specificationProperty);
		IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
		page.openEditor(input, SpecificationTextEditor.ID);
		
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}          
		}
	
	
	/**
	 * Updated the Tree in the view.
	 */
	public void updateTree() {
		// if the syntax problem view is closed, the
		// tree viewer is not initialized.
		// as there may be calls by the listener
		// we need to check if there is a content provider
		// as otherwise an exception occurs.
		if (treeViewer.getContentProvider() != null) {
			treeViewer.setInput(this.getRoot());
			treeViewer.expandAll();
			treeViewer.refresh();
		}
		this.hasWeavingPreventingErrors();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.WorkbenchPart#setFocus()
	 */
	@Override
	public void setFocus() {
	}

	/**
	 * Returns the root node of the syntax problem viewer tree.
	 * Attached to the root are all first element nodes that have children. 
	 * If a node has not children, it won't be attached. 
	 * 
	 * @return Returns the root node of syntax problem viewer tree.
	 */
	public ContainerTreeNode getRoot() {
		root.removeChildren();
		if (this.getNodes().get(TransitionDescription.class).hasChildNodes()) {
			root.addNode(this.getNodes().get(TransitionDescription.class));
		}

		if (this.getNodes().get(Component.class).hasChildNodes()) {
			root.addNode(this.getNodes().get(Component.class));
		}

		if (this.getNodes().get(Scenario.class).hasChildNodes()) {
			root.addNode(this.getNodes().get(Scenario.class));
		}
		if (this.getNodes().get(AspectContainer.class).hasChildNodes()) {
			root.addNode(this.getNodes().get(AspectContainer.class));
		}
		if (this.getNodes().get(JoinRelation.class).hasChildNodes()) {
			root.addNode(this.getNodes().get(JoinRelation.class));
		}

		return root;
	}

	/**
	 * Returns the instance of this syntax problem viewer.
	 * 
	 * @return Returns the singleton instance of this syntax problem viewer.
	 */
	public static SyntaxProblemsView getInstance() {
		IViewPart view = null;

		try {
			view = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().showView(SyntaxProblemsView.VIEW_ID);
		} catch (PartInitException e) {
			e.printStackTrace();
		}

		return (SyntaxProblemsView) view;
	}

	/**
	 * Returns an object which identifies the currently used ADORA editor
	 * uniquely.
	 * 
	 * @return Returns a unique identification object which identifies the ADORA
	 *         editor object uniquely.
	 */
	public Object getActiveEditor() {
		if (currentEditor == null) {
			return null;
		}

		return currentEditor;
	}

	/**
	 * Sets the currently active ADORA editor part. It is used to find the
	 * corresponding syntax problem view.
	 * 
	 * @param activePart
	 *            The active ADORA to set.
	 */
	public void setActivePart(Object editor) {
		if (editor instanceof AdoraModelEditor) {
			AdoraModelEditor ame = (AdoraModelEditor) editor;
			if (!ame.equals(getActiveEditor())) {
				this.currentEditor = ame;
				if (this.adoraEditorToProblemViewMapping.get(editor) != null) {// editor has been checked alredy

					this.sc = this.adoraEditorToSyntaxChecker.get(editor);
					getNodes();
					getRoot();
				} else {// editor has not been checked yet
					getRoot();
					sc = new SyntaxChecker(this, ame.getModel());
					this.adoraEditorToSyntaxChecker.put(editor, sc);
					sc.CheckAndInitSyntaxViewForModel(ame.getModel());
					this.adoraEditorToProblemViewMapping
							.put(editor, getNodes());
					getRoot();
				}
			}
		}
		updateTree();
	}

	/**
	 * Returns the node for the currently active ADORA editor.
	 * 
	 * @return Returns the node for the currently active ADORA editor.
	 */
	public Hashtable<Class, ContainerTreeNode> getNodes() {
		// if the current editor hasn't been initialized
		if (currentEditor instanceof String) {
			AdoraModelEditor editor = AdoraPlugin.getActiveEditor();
			if (editor != null)
				setActivePart(editor);
		}

		if (!this.adoraEditorToProblemViewMapping.containsKey(currentEditor)) {

			Hashtable<Class, ContainerTreeNode> initialTable = new Hashtable<Class, ContainerTreeNode>();
			initialTable.put(TransitionDescription.class,
					new ContainerTreeNode("Transition Labels"));
			initialTable.put(AspectContainer.class, new ContainerTreeNode(
					"Aspect Container"));
			initialTable.put(Component.class,
					new ContainerTreeNode("Component"));
			initialTable
					.put(Scenario.class, new ContainerTreeNode("Scenarios"));
			initialTable.put(JoinRelation.class, new ContainerTreeNode(
					"Join Relation"));

			this.adoraEditorToProblemViewMapping.put(currentEditor,
					initialTable);
		}

		return this.adoraEditorToProblemViewMapping.get(this.getActiveEditor());
	}

	/**
	 * Clean up the used resources. Delete the listener from the workbench page.
	 */
	@Override
	public void dispose() {
		if (listener != null) {
			getSite().getPage().removePostSelectionListener(listener);
		}
		super.dispose();
	}

	/**
	 * Initializes the syntax viewer after early start up. This is necessary, as
	 * the part listener can be registered not before the ui startup finished,
	 * as otherwise a null pointer exception may occur.
	 */
	public void earlyStartup() {
	}

	/**
	 * Removing a deleted node/connection from the view.
	 * 
	 * @param source
	 * @param element
	 */
	public void informDeleted(Object element, ModelContainerTreeNode source) {
		if(getNodes().get(element.getClass())!=null){
			getNodes().get(element.getClass()).removeNode(source);
			getRoot();
			updateTree();
		}
		
	}

	/**
	 * Checking a model element that currently has no Syntax Errors for syntax errors. 
	 * @param model
	 * @return ModelContainerTreeNode if errors were found, null otherwize. 
	 */
	public ModelContainerTreeNode checkForNewSyntaxErrors(ModelElement model) {
		if (model instanceof TransitionDescription) {
			return checkTransition((TransitionDescription) model, null);
		}
		if (model instanceof Scenario) {
			return checkScenario((Scenario) model, null);
		}
		if (model instanceof Component) {
			return checkComponent((Component) model, null);
		}
		if (model instanceof AspectContainer) {
			return checkAspectContainer((AspectContainer) model, null);
		}
		if (model instanceof JoinRelation) {
			return checkJoinRelation((JoinRelation) model, null);
		}
		return null;
	}

	/**
	 * Checks a JoinRelation for parseerrors. 
	 * @param model
	 * @param oldNode
	 * @return ModelContainerTreeNode if errors, null otherwize. 
	 */
	private ModelContainerTreeNode checkJoinRelation(JoinRelation model,
			ModelContainerTreeNode oldNode) {
		ModelContainerTreeNode newNode = null;
		String result = "";
		String content = "";
		result = model.parseContextMap();
		content = model.getContextMap().substring(0,Math.min(model.getContextMap().length(), 50)); //show only first 50 letters of context map 
		if (oldNode == null) {
			if (!result.equals("")) {
				newNode = new ModelContainerTreeNode(this.getNodes().get(JoinRelation.class), content, model,ModelContainerTreeNode.JOINTRELATION_IMAGE);
				this.getNodes().get(JoinRelation.class).addNode(newNode);
				newNode = getJoinRelationErrors(model, newNode);
			}
		} else {// oldnode exists
			if (result.equals("")) {// result is correct
				this.getNodes().get(JoinRelation.class).removeNode(oldNode);
			} else {
				newNode = getJoinRelationErrors(model, oldNode);
				newNode.setDescription(content);
			}
		}
		updateTree();
		return newNode;
	}

	/**
	 * Checking of TransitionDescriptions. At the same time, TreeNodes are
	 * created or removed.
	 */
	private ModelContainerTreeNode checkTransition(TransitionDescription model,
			ModelContainerTreeNode oldNode) {
		ModelContainerTreeNode newNode = null;
		String result = "";
		String content = "";
		result = model.parseTransitionDescription();
		content = model.getName();
		if (oldNode == null) {
			if (!result.equals("")) {
				newNode = new ModelContainerTreeNode(this.getNodes().get(TransitionDescription.class), content, model,ModelContainerTreeNode.TRANSITION_IMAGE);
				this.getNodes().get(TransitionDescription.class).addNode(newNode);
				newNode = getTransitionErrors(model, newNode);
			}
		} else {// oldnode exists
			if (result.equals("")) {// result is correct
				this.getNodes().get(TransitionDescription.class).removeNode(oldNode);
			} else {
				newNode = getTransitionErrors(model, oldNode);
				newNode.setDescription(content);
			}
		}
		updateTree();
		return newNode;
	}

	/**
	 * Checks a component for parseerorrs. 
	 * @param model
	 * @param oldNode
	 * @return ModelContainerTreeNode if errors found, null if correct. 
	 */
	private ModelContainerTreeNode checkComponent(Component model,
			ModelContainerTreeNode oldNode) {
		ModelContainerTreeNode newNode = null;
		String result = "";
		String content = model.getName()
				+ ":   "
				+ model.getSpecification().substring(0,Math.min(model.getSpecification().length(), 50));
		result = model.parseFunctionalSpecification();
		if (oldNode == null) {
			if (!result.equals("")) {
				newNode = new ModelContainerTreeNode(this.getNodes().get(Component.class), content, model,ModelContainerTreeNode.COMPONENT_IMAGE);
				this.getNodes().get(Component.class).addNode(newNode);
				newNode = getComponentErrors(model, newNode);
			}
		} else {
			if (result.equals("")) {
				this.getNodes().get(Component.class).removeNode(oldNode);
			} else {
				newNode = getComponentErrors(model, oldNode);
				newNode.setDescription(content);
			}
		}
		updateTree();
		return newNode;
	}

	/**
	 * Checks an aspectContainer for parseerrors. 
	 * @param model
	 * @param oldNode
	 * @return ModelContainerTreeNode if there were errors, null if correct. 
	 */
	private ModelContainerTreeNode checkAspectContainer(AspectContainer model,ModelContainerTreeNode oldNode) {
		ModelContainerTreeNode newNode = null;
		String result = "";
		String content = model.getName()+ ":   "+ model.getSpecification().substring(0,Math.min(model.getSpecification().length(), 50));
		result = model.parseFunctionalSpecification();
		if (oldNode == null) {
			if (!result.equals("")) {
				newNode = new ModelContainerTreeNode(this.getNodes().get(AspectContainer.class), content, model,ModelContainerTreeNode.ASPECTCONTAINER_IMAGE);
				this.getNodes().get(AspectContainer.class).addNode(newNode);
				newNode = getAspectContainerErrors(model, newNode);
			}
		} else {
			if (result.equals("")) {
				this.getNodes().get(AspectContainer.class).removeNode(oldNode);
			} else {
				newNode = getAspectContainerErrors(model, oldNode);
				newNode.setDescription(content);
			}
		}
		updateTree();
		return newNode;
	}

	/**
	 * Checks aspect container (functional specification) for parseerrors. 
	 * If there are errors, a node is created and errormessages are attached to it. 
	 * @param model
	 * @param newNode
	 * @return node if there are errors, null if correct. 
	 */
	private ModelContainerTreeNode getAspectContainerErrors(AspectContainer model, ModelContainerTreeNode newNode) {
		String result = "";
		newNode.removeChildren();
		result = model.parseFunctionalSpecification();
		if (!result.equals("")) {
			LeafTreeNode ltn = new LeafTreeNode(newNode, result,LeafTreeNode.ERROR_IMAGE);
			newNode.addNode(ltn);
		}
		return newNode;
	}

	/**
	 * Checks component (functional specification) for parseerrors. 
	 * If there are errors, a node is created and errormessages are attached to it. 
	 * @param model
	 * @param newNode
	 * @return node if there are errors, null if correct. 
	 */
	private ModelContainerTreeNode getComponentErrors(Component model, ModelContainerTreeNode newNode) {
		String result = "";
		newNode.removeChildren();
		result = model.parseFunctionalSpecification();
		if (!result.equals("")) {
			LeafTreeNode ltn = new LeafTreeNode(newNode, result, LeafTreeNode.ERROR_IMAGE);
			newNode.addNode(ltn);
		}
		return newNode;
	}

	/**
	 * Calls the relevant checks for a scenario.
	 * 
	 * @param model
	 * @param oldNode
	 * @return null if no errors, TreeNode if errors.
	 */
	private ModelContainerTreeNode checkScenario(Scenario model, ModelContainerTreeNode oldNode) {
		boolean allCorrect = true;
		ModelContainerTreeNode newNode = null;
		String result = "";
		result = model.parseGuardExpression();
		if (!result.equals("")) {
			allCorrect = false;
		}
		result = model.parseTransformExpression();
		if (!result.equals("")) {
			allCorrect = false;
		}
		if (oldNode == null) {// new Scenario
			if (allCorrect == false) {

				newNode = new ModelContainerTreeNode(this.getNodes().get(Scenario.class), model.getName(), model, ModelContainerTreeNode.SCENARIO_IMAGE);
				this.getNodes().get(Scenario.class).addNode(newNode);
				newNode = getScenarioErrors(model, newNode);

			}
		} else {// oldNode exists already
			if (allCorrect) {// all checks correct --> remove the node
				this.getNodes().get(Scenario.class).removeNode(oldNode);
			} else {
				newNode = getScenarioErrors(model, oldNode);

			}
		}
		updateTree();
		return newNode;
	}

	/**
	 * Returns all errors in scenario model and attaches errormessages to
	 * newNode.
	 * 
	 * @param model
	 * @param newNode
	 * @return newNode with attached errorMessages:
	 */
	private ModelContainerTreeNode getScenarioErrors(Scenario model, ModelContainerTreeNode newNode) {
		String result = "";
		String content = "";
		newNode.removeChildren();
		result = model.parseGuardExpression();
		if (!result.equals("")) {
			content = model.getGuard();
			ContainerTreeNode guardNode = new ContainerTreeNode(newNode,"GUARD: " + content);
			LeafTreeNode ltn = new LeafTreeNode(guardNode, result, LeafTreeNode.ERROR_IMAGE);
			newNode.addNode(guardNode);
			guardNode.addNode(ltn);
		}
		result = model.parseTransformExpression();
		if (!result.equals("")) {
			content = model.getTransformation();
			ContainerTreeNode transformNode = new ContainerTreeNode(newNode, "TRANSFORM: " + content);
			LeafTreeNode ltn = new LeafTreeNode(transformNode, result, LeafTreeNode.ERROR_IMAGE);
			newNode.addNode(transformNode);
			transformNode.addNode(ltn);

		}
		return newNode;
	}

	/**
	 * Returns all errors in transition model and attaches errormessages to
	 * newNode.
	 * 
	 * @param model
	 * @param newNode
	 * @return newNode with attached errorMessages:
	 */
	private ModelContainerTreeNode getTransitionErrors(TransitionDescription model, ModelContainerTreeNode newNode) {
		String result = "";
		newNode.removeChildren();
		result = model.parseTransitionDescription();
		if (!result.equals("")) {
			LeafTreeNode ltn = new LeafTreeNode(newNode, result, LeafTreeNode.ERROR_IMAGE);
			newNode.addNode(ltn);
		}
		return newNode;
	}

	/**
	 * Returns all erors in Join Relation (contextMap). Creates a new node if there are errors. 
	 * Errormessages are attached to new node. 
	 * @param model
	 * @param newNode
	 * @return
	 */
	private ModelContainerTreeNode getJoinRelationErrors(JoinRelation model, ModelContainerTreeNode newNode) {
		String result = "";
		newNode.removeChildren();
		result = model.parseContextMap();
		if (!result.equals("")) {
			LeafTreeNode ltn = new LeafTreeNode(newNode, result, LeafTreeNode.ERROR_IMAGE);
			newNode.addNode(ltn);
		}
		return newNode;
	}

	/**
	 * Changes the Name of a Model Element in the tree. 
	 * @param modelContainerTreeNode 
	 * @param newValue new name
	 */
	public void changeName(ModelContainerTreeNode modelContainerTreeNode, Object newValue) {
		modelContainerTreeNode.setDescription((String) newValue);

	}
	
	
	/**
	 * Returns true if there are errors that must be fixed before weaving. 
	 * Errors in functional specification (Component/AspectContainer), TransitionDescription, JoinRelation will inhibit weaving. 
	 * @return
	 */
	public boolean hasWeavingPreventingErrors(){
		
	return root.hasChildNodes();
	}


}
