package ch.unizh.ifi.adora.ui.views.syntaxerrors;

import org.eclipse.jface.viewers.ISelection;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchPart;

import ch.unizh.ifi.adora.properties.specification.SpecificationTextEditor;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;

/**
 * Syntax Tree Part Listener, which listens to changes of the part. If the current editor part
 * changes, the {@link ch.unizh.ifi.adora.ui.views.syntaxerrors.SyntaxProblemsView}
 * will be changed to the actual part.
 * 
 * @author 				Pascal Schoeni
 * @author				smeier
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				21.09.2007 SM First Version
 * 						05.11.2007 SM Renamed class.
 * @version				$Date: 2009-05-06 10:51:39 +0200 (Wed, 06 May 2009) $, $Author: treinhard $, $Revision: 4182 $
 *
 */
public class SyntaxProblemsViewChangeListener implements ISelectionListener {
	/**
	 * Reference for the syntax problem view.
	 */
	private SyntaxProblemsView view;
	
	/**
	 * Reference for the current part of the environment.
	 */
	private Object currentPart;
	
	/**
	 * Constructor initializeing the listener for part change
	 * @param view The syntax view which check for changes in the active editor.
	 */
	public SyntaxProblemsViewChangeListener(SyntaxProblemsView view) {
		this.view = view;
		this.currentPart = null;
	}

	/**
	 * Changes the selected part.
	 */
	public void selectionChanged(IWorkbenchPart part, ISelection selection) {
		Object newPart = part;
		if (part instanceof SpecificationTextEditor) {							
			newPart = ((SpecificationTextEditor)part).getAdoraEditor();											
		}
		
		if (newPart != null && newPart != currentPart) {			
			if ((newPart instanceof AdoraModelEditor)) {
				this.currentPart = newPart;
				this.view.setActivePart(newPart);
			} 
			if (!(newPart instanceof AdoraModelEditor)) {
				this.currentPart = null;
				this.view.setActivePart(SyntaxProblemsView.NO_EDITOR_CHOSEN);
			}
		}
	}

}
