package ch.unizh.ifi.adora.ui.views.syntaxerrors;

import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;

/**
 * Implements the tree content provider interface and maps the 
 * tree node data structure for the tree viewer.
 *      
 * @author				smeier
 * @author              Pascal Schoeni
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				21.09.2007 SM Comments
 * @version				$Date: 2007-09-21 11:10:06 +0200 (Fri, 21 Sep 2007) $, $Author: smeier $, $Revision: 2993 $
 *
 */
public class TreeContentProvider implements ITreeContentProvider {
	/**
	 * Returns the children of a given element as an array.
	 * @param parentElement The object of which the children should be returned.
	 * @return Returns an array of objects, representing the children.
	 */
	public Object[] getChildren(Object parentElement) {
		if (parentElement instanceof ContainerTreeNode) {
			return ((ContainerTreeNode) parentElement).getChildNodes().toArray();
		}

		return new Object[0];
	}

	/**
	 * Returns the parent of a node. If no parent is given, <i>null</i> is returned.
	 * @param element The node of which the parent should be returned.
	 * @return Returns the parent of a given node or null if it is a root node.
	 */
	public Object getParent(Object element) {
		if (element instanceof LeafTreeNode) {
			return ((LeafTreeNode) element).getParent();
		}

		return null;
	}

	/**
	 * Returns true, if the given element has children.
	 * @param element Should be a tree node object.
	 * @return Returns true whether the given object has children.
	 */
	public boolean hasChildren(Object element) {
		if (element instanceof ContainerTreeNode) {
			return ((ContainerTreeNode) element).getChildNodes().size() > 0;
		}

		return false;
	}

	/**
	 * Returns the child elements of the given element.
	 * @param inputElement The tree node of which the elements should be returned.
	 * @return Returns an array of the children of the given element.
	 */
	public Object[] getElements(Object inputElement) {
		return getChildren(inputElement);
	}

	/**
	 * Disposes this object.
	 */
	public void dispose() {
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
	 */
	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
	}
}