package ch.unizh.ifi.adora.ui.views.syntaxerrors;

import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.swt.graphics.Image;

/**
 * Provides the label and the image to the tree viewer. It converts the internal data model
 * consisting of {@link ch.unizh.ifi.adora.ui.views.syntaxerrors.ContainerTreeNode} objects.
 *      
 *
 * @author				smeier
 * 						Pascal Schöni
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				June 2007 PS First Version
 * 						19.07.2007 SM Comments
 * @version				$Date: 2007-09-21 11:10:06 +0200 (Fri, 21 Sep 2007) $, $Author: smeier $, $Revision: 2993 $
 *
 */
public class TreeLabelProvider implements ILabelProvider {

	/**
	 * Returns the image for the element.
	 * @param the element for which the image should be returned.
	 * @return An image object representing the icon of the element.
	 */
	public Image getImage(Object element) {
		if (element instanceof TreeNode) {
			return ((TreeNode) element).getImage();
		}

		return null;
	}

	/**
	 * Returns the textusl description for the object. It removes any new line 
	 * separator from the text.
	 * @param element The node for which the text should be returned.
	 * @param Returns the text associated with the given object.
	 * 
	 */
	public String getText(Object element) {
		String retVal = null;
		if (element instanceof TreeNode) {
			retVal =  ((TreeNode) element).getDescription();
			retVal = retVal.replaceAll(System.getProperty("line.separator"), "");
			retVal = retVal.replaceAll("\n", "");
		}

		return retVal;
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#addListener(org.eclipse.jface.viewers.ILabelProviderListener)
	 */
	public void addListener(ILabelProviderListener listener) {
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#dispose()
	 */
	public void dispose() {
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#isLabelProperty(java.lang.Object, java.lang.String)
	 */
	public boolean isLabelProperty(Object element, String property) {
		return false;
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IBaseLabelProvider#removeListener(org.eclipse.jface.viewers.ILabelProviderListener)
	 */
	public void removeListener(ILabelProviderListener listener) {
	}

}
