package ch.unizh.ifi.adora.utils;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.representation.Actor;
import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.ExitPoint;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.model.representation.State;
/**
 * The ModelElementIconFactory returns the icon of a node.
 * It differs if a node is visible or not.
 *   
 *      
 *
 * @author             Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             24.03.2006 SM First Version
 * @version             $Date: 2006-05-08 09:27:57 +0200 (Mon, 08 May 2006) $, $Author: vigan $, $Revision: 1312 $
 */
public class ModelElementIconFactory {

    /**
     * Stores the ModelElementIconFactory.
     */
	public static final ModelElementIconFactory INSTANCE = new ModelElementIconFactory();
		
	/** A 16x16 pictogram of a sexangle. */
	private static final Image SEXANGLE_ACTIVE_ICON = createImage("icons/sexangle16.gif");

	/** A 16x16 pictogram of a stack. */
	private static final Image STACK_ACTIVE_ICON = createImage("icons/stack16.gif");

	/** A 16x16 pictogram of an elliptical shape. */
	private static final Image ELLIPSE_ACTIVE_ICON = createImage("icons/ellipse16.gif");

	/** A 16x16 pictogram of a rounded rectangle. */
	private static final Image ROUNDEDRECTANGLE_ACTIVE_ICON = createImage("icons/roundedrectangle16.gif");

    /** A 16x16 pictogram of a rectangle. */
    private static final Image RECTANGLE_ACTIVE_ICON = createImage("icons/rectangle16.gif");
    
    /** A 16x16 pictogram of an beveled rectangle. */
    private static final Image BEVELED_RECTANGLE_ACTIVE_ICON = createImage("icons/beveledRectangle16.gif");

    /** A 16x16 pictogram of an exit point. */
    private static final Image EXITPOINT_ACTIVE_ICON = createImage("icons/exitPoint16.gif");

    
    /** A 16x16 pictogram of an inactive sexangle. */
    private static final Image SEXANGLE_INACTIVE_ICON = createImage("icons/sexangleInactive16.gif");

    /** A 16x16 pictogram of an inactive stack. */
    private static final Image STACK_INACTIVE_ICON = createImage("icons/stackInactive16.gif");

    /** A 16x16 pictogram of an inactive elliptical shape. */
    private static final Image ELLIPSE_INACTIVE_ICON = createImage("icons/ellipseInactive16.gif");

    /** A 16x16 pictogram of an inactive rounded rectangle. */
    private static final Image ROUNDEDRECTANGLE_INACTIVE_ICON = createImage("icons/roundedrectangleInactive16.gif");
    
    /** A 16x16 pictogram of an inactive rectangle. */
    private static final Image RECTANGLE_INACTIVE_ICON = createImage("icons/rectangleInactive16.gif");
    
    /** A 16x16 pictogram of an inactive beveled rectangle. */
    private static final Image BEVELED_RECTANGLE_INACTIVE_ICON = createImage("icons/beveledRectangleInactive.gif");

    /** A 16x16 pictogram of an inactive exit point. */
    private static final Image EXITPOINT_INACTIVE_ICON = createImage("icons/exitPointInactive16.gif");

    
	private ModelElementIconFactory() {
		
	}
	
	/**
	 * Creates an image from a filename.
	 * 
	 * @param name The name of the file.
	 * @return The image.
	 */
	private static Image createImage(String name) {
		return ImageDescriptor
		.createFromFile(AdoraPlugin.class,
				name).createImage();
	}
	

	/**
	 * Return a pictogram (small icon) describing this model element. Children
	 * should override this method and return an appropriate Image.
	 * 
	 * @return a 16x16 Image or null
	 */
	public Image getIcon(Node node, boolean isVisible) {
        if(isVisible)
        return getActiveIcon(node);
        return getInactiveIcon(node);
	}

    /**
     * Returns the Icon of an active node.
     * 
     * @pre       true
     * @post      true
     * @obligation 
     * @param node
     * @return
     */
    private Image getActiveIcon(Node node) {
        if (node instanceof Actor){
            return SEXANGLE_ACTIVE_ICON;
        } if (node instanceof Component) {
            if(!((Component)node).getCardinality().isDefaultCard()){
                return STACK_ACTIVE_ICON;
            }
            return RECTANGLE_ACTIVE_ICON;
        } if (node instanceof Scenario) {
            return  ELLIPSE_ACTIVE_ICON;
        } if (node instanceof State) {
            return ROUNDEDRECTANGLE_ACTIVE_ICON;
        } if (node instanceof AspectContainer) {
            return BEVELED_RECTANGLE_ACTIVE_ICON;
        } if (node instanceof ExitPoint) {
            return EXITPOINT_ACTIVE_ICON;
        }
        return null;
    }
    
    /**
     * Returns the Icon of an inactive node.
     * 
     * @pre       true
     * @post      true
     * @obligation 
     * @param node
     * @return
     */
    private Image getInactiveIcon(Node node) {
        if (node instanceof Actor){
            return SEXANGLE_INACTIVE_ICON;
        } if (node instanceof Component) {
            if(!((Component)node).getCardinality().isDefaultCard()){
                return STACK_INACTIVE_ICON;
            }
            return RECTANGLE_INACTIVE_ICON;
        } if (node instanceof Scenario) {
            return  ELLIPSE_INACTIVE_ICON;
        } if (node instanceof State) {
            return ROUNDEDRECTANGLE_INACTIVE_ICON;
        } if (node instanceof AspectContainer) {
            return BEVELED_RECTANGLE_INACTIVE_ICON;
        } if (node instanceof ExitPoint) {
            return EXITPOINT_INACTIVE_ICON;
        }
        return null;
    }
}
