package ch.unizh.ifi.adora.view.anchors;

import java.util.logging.Level;
import java.util.logging.Logger;

import org.eclipse.draw2d.AbstractConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;

import ch.unizh.ifi.adora.view.figures.geometry.GeometryUtils;

/**
 * This {@link ch.unizh.ifi.adora.view.anchors.ClickPointAnchor} 
 * is used in if {@link org.eclipse.draw2d.Connection}s are placed 
 * on an {@link ch.unizh.ifi.adora.view.figures.ActorFigure}.
 * 
 * @author		        Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             24.07.2006 IV First Version
 * @version             $Date: 2006-07-24 12:29:04 +0200 (Mon, 24 Jul 2006) $, $Author: vigan $, $Revision: 1891 $
 */
public class ActorFigureAnchor extends AbstractConnectionAnchor {
	
	/**
	 * Stores the logger for this class
	 */
	private static final Logger logger = Logger
			.getLogger(ActorFigureAnchor.class.getName());

	
	/**
	 * Instantiates a new Anchor for the given <li>owner</li>.
	 * 
	 * @param owner The figure for which this Anchor is been 
	 * Instantiated. 
	 */
	public ActorFigureAnchor(IFigure owner) {
		super(owner);
		
	}
	
	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.draw2d.ConnectionAnchor#getLocation(org.eclipse.draw2d.geometry.Point)
	 */
	public Point getLocation(Point outsidePoint) {
		Rectangle bounds = Rectangle.SINGLETON;
		
		bounds.setBounds(getOwner().getBounds());
		//bounds.translate(-1, -1);
		//bounds.resize(1, 1);
		
		Point insidePoint = new Point(bounds.width / 2 + bounds.x,
		                            bounds.height / 2 + bounds.y);
		if (logger.isLoggable(Level.FINE)) {
			logger.fine("Point - Point insidePoint=" + insidePoint);
		}
		if (logger.isLoggable(Level.FINE)) {
			logger.fine("Point - Point outsidePoint=" + outsidePoint);
		}
		
		Point p=null;
		//try {
		p = GeometryUtils.intersectSextangle(bounds, insidePoint, outsidePoint);
		if (p == null) 	p = insidePoint;  //This avoids a null pointer exception
		
		if (logger.isLoggable(Level.FINE)) {
			logger.fine("Point - Point p=" + p);
		}
		
		return p;
	}
}