package ch.unizh.ifi.adora.view.anchors;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;

import ch.unizh.ifi.adora.view.figures.geometry.GeometryUtils;

/**
 * This {@link ch.unizh.ifi.adora.view.anchors.ClickPointAnchor} 
 * is used in if {@link org.eclipse.draw2d.Connection}s are placed 
 * on an {@link ch.unizh.ifi.adora.view.figures.AspectContainers}.
 *      
 * @author              Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             02.05.2006 SM First Version
 *                      13.07.2006 JS changed getLocation(Point) to use 
 *                                    absolute coordinates, as done in 
 *                                    SextangleClickPointAnchor
 *                      28.10.2006 SM Scaling bug, i.e. order of Figure.translate(..)
 *                                    and Figure.translateToAbsolute(..) changed.
 * @version             $Date: 2007-07-27 15:05:52 +0200 (Fri, 27 Jul 2007) $, $Author: smeier $, $Revision: 2875 $
 */
public class AspectContainerClickPointAnchor extends ClickPointAnchor {
	
	/**
	 * Instantiates a new Anchor for the given <li>owner</li>.
	 * 
	 * @param owner The figure for which this Anchor is been 
	 * Instantiated. 
	 */
	public AspectContainerClickPointAnchor(IFigure owner) {
        super(owner);
    }
    
    /*
     *  (non-Javadoc)
     * @see org.eclipse.draw2d.ConnectionAnchor#getLocation(org.eclipse.draw2d.geometry.Point)
     */
    public Point getLocation(Point outsidePoint) {
        Rectangle bounds = Rectangle.SINGLETON;
        
        bounds.setBounds(getOwner().getBounds());
        
        
        Point inside = getClickPoint().translate(bounds.getLocation());
        getOwner().translateToAbsolute(inside);
        getOwner().translateToAbsolute(bounds);
        
        Point p2 = GeometryUtils.intersectBeveledRectangle(bounds, inside, outsidePoint);
        
        if(p2==null){

            return inside;
        }
		
        return p2;
    }
    
	/**
	 * Returns the absolute and unscaled coordinates of the point on the
	 * figure's border where the connection should start or end. The 
	 * point is defined by the interscetion between the line segment formed
	 * by the click point inside the node and the point given as argument and
	 * the border of the figure.
	 * The returned coordinates are unscaled (i.e. not adjsuted to the
	 * viewport and the scale factor).
	 * 
	 * @param reference the reference point outside the figure's border
	 * @return the point on the figure's border
	 */
    public Point getUnscaledLocation(Point reference) {
    	Rectangle bounds = Rectangle.SINGLETON;
        
        bounds.setBounds(getOwner().getBounds());
        
        
        Point inside = getClickPoint().translate(bounds.getLocation());
        
        Point p2 = GeometryUtils.intersectBeveledRectangle(bounds, inside, reference);
        if(p2==null){

            return inside;
        }
		
        return p2;
    }
    
}