package ch.unizh.ifi.adora.view.anchors;

import org.eclipse.draw2d.AbstractConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Point;

/**
 * Superclass for all ClickPointAnchors.
 *
 * @author		        Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             24.07.2006 IV First Version
 *                      28.10.2006 SM Scaling bug, i.e. order of Figure.translate(..)
 *                                    and Figure.translateToAbsolute(..) changed.
 * @version             $Date: 2007-04-20 09:42:41 +0200 (Fri, 20 Apr 2007) $, $Author: treinhard $, $Revision: 2607 $
 */
public abstract class ClickPointAnchor extends AbstractConnectionAnchor {

	/**
	 * Stores the xRatio.
	 */
	protected float xRatio;
	
	/**
	 * Stores the yRatio.
	 */
	protected float yRatio;
	
	/**
	 * Instantiates a new Anchor for the given <li>owner</li>.
	 * 
	 * @param owner The figure for which this Anchor is been 
	 * Instantiated. 
	 */
	public ClickPointAnchor(IFigure owner) {
		super(owner);
		xRatio = 0.5f;
		yRatio = 0.5f;
	}
		
	/**
	 * @param p The ClickPoint.
	 */
	public void setClickPoint(Point p) {
		xRatio = p.x / (float)getOwner().getBounds().width;
		yRatio = p.y / (float)getOwner().getBounds().height;
	}
	
	/**
	 * @param xRatio The xRatio.
	 * @param yRatio The yRatio.
	 */
	public void setClickPoint(float xRatio, float yRatio) {
		this.xRatio = xRatio;
		this.yRatio = yRatio;
	}
	
	/**
	 * @return The ClickPoint.
	 */
	public Point getClickPoint() {
		
		int x = Math.round(getOwner().getBounds().width * xRatio);
		int y = Math.round(getOwner().getBounds().height * yRatio);
		
		return new Point(x, y);
	}

	/**
	 * Returns the anchor's reference point in absolute coordinates. The reference
	 * point of the click point anchor is the click point itself.
	 * 
	 * @return The reference point
	 * 
	 */
	public Point getReferencePoint() {
		
		Point ref = getUnscaledReferencePoint();
		// Adjust the coordinates to the viewport and the scale factor
		getOwner().translateToAbsolute(ref);
		return ref;
	}
	
	/**
	 * Returns the anchor's reference point in absolue unscaled (i.e. not
	 * adjusted to the viewport and the scale factor) coordinates.
	 * 
	 * @return the anchor's reference point in absolute unscaled coordinates
	 */
	public Point getUnscaledReferencePoint() {
		
		Point ref = getClickPoint();
		// Make the click point's coordinates absolute
		ref.translate(getOwner().getBounds().getLocation());
		return ref;
		
	}
	
	/**
	 * Returns the absolute and unscaled coordinates of the point on the
	 * figure's border where the connection should start or end.
	 * The returned coordinates are unscaled (i.e. not adjsuted to the
	 * viewport and the scale factor).
	 * 
	 * @param reference the reference point outside the figure's border
	 * @return the point on the figure's border
	 */
	public abstract Point getUnscaledLocation(Point reference);
	
}