package ch.unizh.ifi.adora.view.anchors;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;

import ch.unizh.ifi.adora.view.figures.IZoomableFigure;
import ch.unizh.ifi.adora.view.figures.StartStateIndicatorFigure;

/**
 * This {@link ch.unizh.ifi.adora.view.anchors.ClickPointAnchor} is used in if
 * {@link org.eclipse.draw2d.Connection}s are placed on an
 * {@link ch.unizh.ifi.adora.view.figures.ComponentFigure}.
 * 
 * It Determines the endpoint-location of a connections, which connects
 * rectangular figures.
 * 
 * @author Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 28.04.2006 SM First Version
 * @version $Date: 2007-09-03 08:53:23 +0200 (Mon, 03 Sep 2007) $, $Author: treinhard $, $Revision: 2933 $
 */
public class ContainerNodeClickPointAnchor extends RectangleClickPointAnchor {

	/**
     * Instantiates a new Anchor for the given
     * <li>owner</li>.
     * 
     * @param owner
     *            The figure for which this Anchor is been Instantiated.
     */
	public ContainerNodeClickPointAnchor(IFigure owner) {
		super(owner);
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.draw2d.ConnectionAnchor#getLocation(org.eclipse.draw2d.geometry.Point)
     */
	public Point getLocation(Point reference) {
		Point p = super.getLocation(reference);

		this.adaptStartStatePoint(p);

		return p;
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.draw2d.ConnectionAnchor#getUnscaledLocation(org.eclipse.draw2d.geometry.Point)
     */
	public Point getUnscaledLocation(Point reference) {
		Point p = super.getUnscaledLocation(reference);

		this.adaptStartStatePoint(p);

		return p;
	}

	/**
	 * Adapts the point to the border if the owner is a start state.
	 * @param p
	 */
	private void adaptStartStatePoint(Point p) {
		Rectangle pos = getOwner().getBounds().getCopy();

		// Adds the offset due to the startstate
		if (((IZoomableFigure) getOwner()).isStartState()
				&& p.x <= pos.x + StartStateIndicatorFigure.WIDTH) {
			p.x += StartStateIndicatorFigure.WIDTH;
		}
	}
}