package ch.unizh.ifi.adora.view.anchors;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;

import ch.unizh.ifi.adora.view.figures.geometry.GeometryUtils;

/**
 * This {@link ch.unizh.ifi.adora.view.anchors.ClickPointAnchor} 
 * is used in if {@link org.eclipse.draw2d.Connection}s are placed 
 * on an {@link ch.unizh.ifi.adora.view.figures.ScenarioFigure}.
 *      
 * @author		        Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             24.07.2006 IV First Version
 *                      26.07.2006 JS Adapted getLocation(Point) to behave
 *                                    more like AspectContainerClickPointAnchor
 *                      28.10.2006 SM Scaling bug, i.e. order of Figure.translate(..)
 *                                    and Figure.translateToAbsolute(..) changed.                                    
 * @version             $Date: 2006-11-02 09:53:47 +0100 (Thu, 02 Nov 2006) $, $Author: treinhard $, $Revision: 2350 $
 */
public class EllipseClickPointAnchor extends ClickPointAnchor {

	/**
	 * Instantiates a new Anchor for the given <li>owner</li>.
	 * 
	 * @param owner The figure for which this Anchor is been 
	 * Instantiated. 
	 */
	public EllipseClickPointAnchor(IFigure owner) {
		super(owner);
	}

	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.draw2d.ConnectionAnchor#getLocation(org.eclipse.draw2d.geometry.Point)
	 */
	public Point getLocation(Point reference) {
		
		Rectangle r = Rectangle.SINGLETON;
		r.setBounds(getOwner().getBounds());

		if(getOwner().getParent() == null){
			return reference;
		}
		
		Point inside = getClickPoint().getTranslated(r.getLocation());
		
		getOwner().getParent().translateToAbsolute(inside);
		getOwner().getParent().translateToAbsolute(r);	
		
		
		Point p = GeometryUtils.intersectEllipse(r, inside, reference);
		
		if(p == null) {
			p = inside;
		}
		
		return p;
	}
	
	
	/**
	 * Returns the absolute and unscaled coordinates of the point on the
	 * figure's border where the connection should start or end. The 
	 * point is defined by the interscetion between the line segment formed
	 * by the click point inside the node and the point given as argument and
	 * the border of the figure.
	 * The returned coordinates are unscaled (i.e. not adjsuted to the
	 * viewport and the scale factor).
	 * 
	 * @param reference the reference point outside the figure's border
	 * @return the point on the figure's border
	 */
	public Point getUnscaledLocation(Point reference) {
		
		Rectangle r = Rectangle.SINGLETON;
		r.setBounds(getOwner().getBounds());

		if(getOwner().getParent() == null){
			return reference;
		}
		
		Point inside = getClickPoint().getTranslated(r.getLocation());		
		
		Point p = GeometryUtils.intersectEllipse(r, inside, reference);
		
		if(p == null) {
			p = inside;
		}
		
		return p;
		
	}
}