
package ch.unizh.ifi.adora.view.anchors;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;

import ch.unizh.ifi.adora.view.figures.geometry.GeometryUtils;

/**
 * This {@link ch.unizh.ifi.adora.view.anchors.ClickPointAnchor} 
 * is used if {@link org.eclipse.draw2d.Connection}s are placed 
 * on an {@link ch.unizh.ifi.adora.view.figures.ComponentFigure}.
 * 
 * It Determines the endpoint-location of a connections.
 *
 * @author              Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             03.04.2006 SM First Version
 *                      28.10.2006 SM Scaling bug, i.e. order of Figure.translate(..)
 *                                    and Figure.translateToAbsolute(..) changed.
 * @version             $Date: 2006-11-02 09:53:47 +0100 (Thu, 02 Nov 2006) $, $Author: treinhard $, $Revision: 2350 $
 */
public class RectangleClickPointAnchor extends ClickPointAnchor {

	/**
	 * Instantiates a new Anchor for the given <li>owner</li>.
	 * 
	 * @param owner The figure for which this Anchor is been 
	 * Instantiated. 
	 */
	public RectangleClickPointAnchor(IFigure owner) {
		super(owner);
	}
	
    /**
     * Calculates the point where a connection should end.
     */
	public Point getLocation(Point reference) {
		Rectangle r = Rectangle.SINGLETON;
		r.setBounds(getBox());

		Point inside = getClickPoint().translate(r.getLocation());
		
		// Adjust the point inside the node and the rectangle to the
		/// viewport and the scale factor
        getOwner().translateToAbsolute(inside);
        getOwner().translateToAbsolute(r);
        		
        return GeometryUtils.intersectRectangle(r, inside, reference);
	}
	
	/**
	 * Returns the absolute and unscaled coordinates of the point on the
	 * figure's border where the connection should start or end. The 
	 * point is defined by the interscetion between the line segment formed
	 * by the click point inside the node and the point given as argument and
	 * the border of the figure.
	 * The returned coordinates are unscaled (i.e. not adjsuted to the
	 * viewport and the scale factor).
	 * 
	 * @param reference the reference point outside the figure's border
	 * @return the point on the figure's border
	 */
	public Point getUnscaledLocation(Point reference) {
		
		Rectangle r = Rectangle.SINGLETON;
		r.setBounds(getBox());
	
		Point inside = getClickPoint().translate(r.getLocation());
        		
        return GeometryUtils.intersectRectangle(r, inside, reference);
		
	}
	
	/**
	 * @return the bounds of the owner.
	 */
    protected Rectangle getBox() {
		return getOwner().getBounds();
	}
}