package ch.unizh.ifi.adora.view.figures;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.MarginBorder;
import org.eclipse.draw2d.Polygon;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PointList;
import org.eclipse.draw2d.geometry.Rectangle;

import ch.unizh.ifi.adora.parts.figureinterfaces.INodeContainerProperties;
import ch.unizh.ifi.adora.view.figures.geometry.GeometryUtils;
import ch.unizh.ifi.adora.view.figures.label.NodeLabel;


/**
 * This class displays the AspectContainer.
 *      
 * @author              Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             27.04.2006 SM First Version
 *                      04.08.2006 JS This Figure will be filled now
 * @version             $Date: 2008-06-17 16:01:29 +0200 (Tue, 17 Jun 2008) $, $Author: treinhard $, $Revision: 3639 $
 */
public class AspectContainerFigure extends CommentableNodeFigure implements 
    INodeContainerProperties, IZoomableFigure {

   /**
     * Stores whether this node is zoomed in (true) or zoomed out (false).
     */
    private boolean zoomed;

    /**
     * The label that shows the name of this component
     */
    private NodeLabel nameLabel;
    
    private boolean calculatedPartial;
	
	/**
	 * Fires a PropertyChange message, if the have insets changed.
	 */
	protected PropertyChangeSupport pcsDelegate;
	
	private int minBorder = 5;
	
	private Insets border = new Insets(0, 0, 0, 0);

    /**
     * Intializes the AspectContainer object figure with an xy layout.
     * 
     * @param name The text shown on this figure.
     * @param isPartial True, if the AspectContainer is pratial.
     */
    public AspectContainerFigure(String name, boolean isPartial) {
    	
        setLayoutManager(new XYLayout());
        
        pcsDelegate = new PropertyChangeSupport(this);
        
        nameLabel = new NodeLabel();
        nameLabel.setVerticalAlignment(PositionConstants.MIDDLE);
        nameLabel.setHorizontalAligment(PositionConstants.CENTER);
        nameLabel.setBorder(new MarginBorder(5, 5, 5, 5));
        add(nameLabel);
        
        setText(name);
        setPartial(isPartial);
        
    }
 
    
    /*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.parts.figureinterfaces.INodeContainerProperties#setZoom(boolean
     *      z)
     */
    public void setZoomed(boolean z) {
        zoomed = z;
    }
    
    
    /*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.parts.figureinterfaces.INodeContainerProperties#isZoomed()
     */
    public boolean isZoomed() {
        return zoomed;
    }
    
    
	public void setCalculatedPartial(boolean isPartial) {
		calculatedPartial = isPartial;
	}
	
	
	public boolean isCalculatedPartial() {
		return calculatedPartial;
	}
    
       /*
     *  (non-Javadoc)
     * @see org.eclipse.draw2d.Figure#layout()
     */
    public void layout() {
    
		super.layout();
		
		String s = getText();
		if(!isZoomed() || isPartial() || isCalculatedPartial()) {
		    s = s.concat("...");
		}
		
		Insets newBorder = new Insets(0, 0, 0, 0);
		
		int x = getBounds().x + lineWidth / 2;
		int y = getBounds().y + lineWidth / 2;
		int w = getBounds().width - Math.max(1, lineWidth);
		int h = getBounds().height - Math.max(1, lineWidth);
		Rectangle r = new Rectangle(x, y, w, h);
		
		
		int[] points = GeometryUtils.getPolylineBeveledRectanglePoints(
                r.getLocation(), r.getSize());
                
		// The border
		newBorder.right = points[0] - getLocation().x;
		newBorder.left = points[0] - getLocation().x;
		
		// Add the name label
		Point labelLoc = getLocation().getTranslated(newBorder.left, newBorder.top);
		nameLabel.setLocation(labelLoc);
		
		int width = getSize().width - newBorder.left - newBorder.right;
		int height = getSize().height - newBorder.top - newBorder.bottom;
		
		nameLabel.setText(s);
		if(getChildren().size() > 1) {
			
			nameLabel.setSize(width, nameLabel.getMinimalHeight(width));
			nameLabel.setHorizontalAligment(PositionConstants.LEFT);
			nameLabel.setVerticalAlignment(PositionConstants.TOP);
			
			if(!nameLabel.getText().equals("")) {
				newBorder.top = nameLabel.getBounds().height - minBorder - 10;
			}
		}
		else {
			nameLabel.setSize(width, height);
			nameLabel.setHorizontalAligment(PositionConstants.CENTER);
			nameLabel.setVerticalAlignment(PositionConstants.MIDDLE);
		}
		
		
		newBorder.top += minBorder;
		newBorder.left += minBorder;
		newBorder.bottom += minBorder;
		newBorder.right += minBorder;
		
		if(!newBorder.equals(border)) {

			border = newBorder;
			pcsDelegate.firePropertyChange(INSETS_CHANGED_PROP, null, newBorder);
			
		}
		

    }

    public IFigure getNameLabel() {
    	return nameLabel;
    }


    /*
     *  (non-Javadoc)
     * @see ch.unizh.ifi.adora.view.figures.IZoomableFigure#setStartState(boolean)
     */
    public void setStartState(boolean startState) {
    }

    /*
     *  (non-Javadoc)
     * @see ch.unizh.ifi.adora.view.figures.IZoomableFigure#isStartState()
     */
    public boolean isStartState() {
        return false;
    }

    /*
     *  (non-Javadoc)
     * @see ch.unizh.ifi.adora.view.figures.IZoomableFigure#relayout()
     */
    public void relayout() {
    }

    @Override
    protected void fillShape(Graphics graphics) {
    	
    	super.fillShape(graphics);
    	
		int x = getBounds().x + lineWidth / 2;
		int y = getBounds().y + lineWidth / 2;
		int w = getBounds().width - Math.max(1, lineWidth);
		int h = getBounds().height - Math.max(1, lineWidth);
		Rectangle r = new Rectangle(x, y, w, h);
    	
        graphics.fillPolygon(
                GeometryUtils.getPolylineBeveledRectanglePoints(
                        r.getLocation(), r.getSize()));    
    }

    @Override
    protected void outlineShape(Graphics graphics) {
    	
		int x = getBounds().x + lineWidth / 2;
		int y = getBounds().y + lineWidth / 2;
		int w = getBounds().width - Math.max(1, lineWidth);
		int h = getBounds().height - Math.max(1, lineWidth);
		Rectangle r = new Rectangle(x, y, w, h);
    	
        graphics.drawPolyline(
                GeometryUtils.getPolylineBeveledRectanglePoints(
                        r.getLocation(), r.getSize()));
    } 
    
    /**
     * Returns <code>true</code> if the given point (x,y) is contained within this Polygon.
     * @param x the x coordinate
     * @param y the y coordinate
     * @return <code>true</code>if the given point is contained
     */
    public boolean containsPoint(int x, int y) {
    	
		int bx = getBounds().x + lineWidth / 2;
		int by = getBounds().y + lineWidth / 2;
		int w = getBounds().width - Math.max(1, lineWidth);
		int h = getBounds().height - Math.max(1, lineWidth);
		Rectangle r = new Rectangle(bx, by, w, h);
    	
        Polygon poly = new Polygon();
        poly.setPoints(new PointList(
                GeometryUtils.getPolylineBeveledRectanglePoints(
                        r.getLocation(), r.getSize())));
        
        return poly.containsPoint(x,y); 
    }
    
    /**
     * Returns the name of the aspect container.
     * 
     * @return name of the aspect container
     */
    public String toString() {
        return getText();
    }
	
	
	/**
	 * Adds a PropertyChangeListener to this Border. This is used to update the Border bounds
	 * eachtime the paint method is called. 
	 */
	public synchronized void addPropertyChangeListener(PropertyChangeListener l) {
		if (l == null) {
			throw new IllegalArgumentException();
		}
		pcsDelegate.addPropertyChangeListener(l);
	}

	/**
	 * Remove a PropertyChangeListener from this component.
	 * 
	 * @param l a PropertyChangeListener instance
	 */
	public synchronized void removePropertyChangeListener(
			PropertyChangeListener l) {
		if (l != null) {
			pcsDelegate.removePropertyChangeListener(l);
		}
	}

}