package ch.unizh.ifi.adora.view.figures;

import org.eclipse.draw2d.Connection;
import org.eclipse.draw2d.ConnectionLocator;

import ch.unizh.ifi.adora.view.figures.label.associationlabel.ArrowDecoration;
import ch.unizh.ifi.adora.view.figures.label.associationlabel.MovableArrowLocator;

/**
 * This class is responsible for creating a visual connection between two
 * objects, connected via an association. 
 * 
 * @author vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history Feb 15, 2006 SM First Version
 * @version $Date: 2006-09-26 10:52:33 +0200 (Tue, 26 Sep 2006) $, $Author: vigan $, $Revision: 2267 $
 * @responsibilities
 * 
 */
public class AssociationFigure extends LabeledConnection implements IAbstractableConnectionFigure{

	/**
	 * Locator, used to rotate the target arrow parallel to the connection.
	 */
	private MovableArrowLocator targetArrowLocator;
	
	/**
	 * Locator, used to rotate the source arrow parallel to the connection.
	 */
	private MovableArrowLocator sourceArrowLocator;
	
	/**
     * True if the underlying association is abstracted, false otherwise.
     */
	private boolean abstracted = false;

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.view.figures.IAbstractableConnectionFigure#setAbstracted(boolean)
	 */
	public void setAbstracted(boolean abstracted) {
		this.abstracted = abstracted;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.view.figures.IAbstractableConnectionFigure#isAbstracted()
	 */
	public boolean isAbstracted() {
		return abstracted;
	}
	
	/**
	 * Sets the decoration to be used at the start of the {@link Connection}.
	 * @param dec the new source decoration
	 */
	public void setSourceDecoration(ArrowDecoration dec) {
		sourceArrowLocator = new MovableArrowLocator(this, ConnectionLocator.SOURCE, dec);
	}
	
	/**
	 * Sets the decoration to be used at the start of the {@link Connection}.
	 * @param dec the new source decoration
	 */
	public void setTargetDecoration(ArrowDecoration dec) {
		targetArrowLocator = new MovableArrowLocator(this, ConnectionLocator.TARGET, dec);
	}
	
	/**
	 * This method is overwritten, 
	 * because the {@link MovableArrowLocator} have to recalcualte
	 * the orientation of the arrows each time the association is been relayouted.
	 * Therefore the {@link MovableArrowLocator#reAdjust()} method is been called in
	 * this method.
	 */
	public void layout(){
		super.layout();
		if(targetArrowLocator != null)
			targetArrowLocator.reAdjust();
		if(sourceArrowLocator != null)
			sourceArrowLocator.reAdjust();
	}
}