package ch.unizh.ifi.adora.view.figures;

/**
 * Base class for those node figures that can have a comment attached.
 *
 * @author		        Tobias Reinhard
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             2007-12-21 TR First Version
 * @version             $Date: 2007-12-21 13:53:37 +0100 (Fri, 21 Dec 2007) $, $Author: treinhard $, $Revision: 3435 $
 */
public abstract class CommentableNodeFigure extends NodeFigure implements ICommentableFigure {

	/**
	 * Stores the comment for this Figure.
	 */
	private String comment = "";

	/**
	 * Returns the comment that is attached to this figure.
	 * 
	 * @return the comment
	 */
	public String getComment() {
		return comment;
	}

	
	/**
	 * Sets the comment that is attached to this figure.
	 * 
	 * @param comment the comment
	 */
	public void setComment(String comment) {
		this.comment = comment;
		updateToolTip();
	}


	/**
	 * Updates the tool tip so that the name and the comment are shown in it.
	 */
	protected void updateToolTip() {
		String text = "";
		String name = getText();
		String comment = getComment();
		if (name.equals("") && comment.equals("")) {
			setToolTip(null);
		} else {
			if (!name.equals("") && !comment.equals(""))
				text = name + ": " + comment;
			else
				text = name + comment;
			setToolTip(toolTipLabel);
			toolTipLabel.setText(text);
		}
	}

}
