package ch.unizh.ifi.adora.view.figures;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;

/**
 * Interface that can be used to get the relative bounds of a node inside the 
 * implementing node and to get the absolute (GEF) coordinates of the implementing
 * node. This interface is used on the figures to convert their absolute 
 * coordinates to the relative coordinates of the line router and vice-versa.
 *
 * @author		        Tobias Reinhard
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             2008-06-13 TR First Version
 * @version             $Date: 2008-06-13 15:23:55 +0200 (Fri, 13 Jun 2008) $, $Author: treinhard $, $Revision: 3638 $
 */
public interface INestedNode {
	
	/**
	 * Returns the relative coordinates of the node given as argument with
	 * respect to this node.
	 * 
	 * @param child the node to get the relative coordinates for
	 * @return the relative coordinates of the child
	 */
	public Rectangle getRelativeBounds(IFigure child);
	
	
	/**
	 * Returns the absolute (GEF) coordinates of this node.
	 * 
	 * @return the absolute coordinates.
	 */
	public Point getAbsoluteLocation();

}
