package ch.unizh.ifi.adora.view.figures;

import org.eclipse.draw2d.IFigure;

/**
 * This interface is used to treat all zoomable-figures the same.
 *      
 * @author              Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             27.03.2006 SM First Version
 * @version             $Date: 2008-04-29 11:08:17 +0200 (Tue, 29 Apr 2008) $, $Author: treinhard $, $Revision: 3625 $
 */
public interface IZoomableFigure extends IFigure{

	/**
	 * Key for the INSETS_CHANGED_PROP.
	 */
	public static final String INSETS_CHANGED_PROP = "LabelBorder.InsetsChanged";
	
   /**
     * Sets the Partial state of the figure. This determins, if 3 dots are
     * displayed after the text.
     * 
     * @param partial  <li>true</li>, if this {@link IZoomableFigure}
     * is partial.
     */
    public void setPartial(boolean partial);
    
    /**
     * Sets the start state of the figure. This determins, if the start-state-indicator
     * is displayed left of the figure (0->)
     * 
     * @param startState The startState
     */
    public void setStartState(boolean startState);
    
    /*
     * (non-Javadoc)
     * 
     * @see ch.unizh.ifi.adora.parts.figureinterfaces.INodeProperties#setName(java.lang.String)
     */
    public void setText(String name);
    
    /**
     * @return Returns <li>true</li>, if this {@link IZoomableFigure}
     * is partial.
     */
    public boolean isPartial();
    
    /**
     * @return Returns <li>true</li>, if this {@link IZoomableFigure}
     * is a startState.
     */
    public boolean isStartState();
    
    /**
     * Used to mark the figure as partial if one of its children is hidden by
     * the view mechanism.
     * 
     * @param isPartial
     */
    public void setCalculatedPartial(boolean isPartial);
	
	
    /**
     * Returns true if the figure is marked as partial because one of its
     * children is hidden by the view mechanism.
     * 
     * @return
     */
	public boolean isCalculatedPartial();
    
    
    /**
     * Updates the border and the figure.
     */
    public void relayout();
    
    /**
     * Relayouts the figure.
     */
    public void layout();
    
    /**
     * Returns true, if a Point is situated inside a figures bounds.
     */
    public boolean containsPoint(int x, int y);
}