package ch.unizh.ifi.adora.view.figures;

import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.PolylineDecoration;

/**
 * This class is responsible for creating a visual connection between two
 * objects, connected via a JoinRelation. 
 *      
 * @author		        Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             24.07.2006 IV First Version
 * @version             $Date: 2009-05-06 10:51:39 +0200 (Wed, 06 May 2009) $, $Author: treinhard $, $Revision: 4182 $
 */
public class JoinRelationFigure extends LabeledConnection implements IAbstractableConnectionFigure {

	/**
	 * True if the underlying association is abstracted, false otherwise.
	 */
	private boolean abstracted = false;

	/**
	 * Stores the size of the DECORATION_SIZE.
	 */
    private static final int DECORATION_SIZE = 6;
    
    /**
     * Instantiates a new JoinRelationFigure.
     */
    public JoinRelationFigure() {
        this.setLineStyle(Graphics.LINE_DASH);
        
        PolylineDecoration decoration = new PolylineDecoration();
        decoration.setScale(DECORATION_SIZE, DECORATION_SIZE / 2);
        setTargetDecoration(decoration); 
    }
    
    /*
     *  (non-Javadoc)
     * @see ch.unizh.ifi.adora.view.figures.IAbstractableConnectionFigure#setAbstracted(boolean)
     */
	public void setAbstracted(boolean abstracted) {
		this.abstracted = abstracted;
	}
	
	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.view.figures.IAbstractableConnectionFigure#isAbstracted()
	 */
	public boolean isAbstracted() {
		return abstracted;
	}
}