package ch.unizh.ifi.adora.view.figures;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.Shape;
import org.eclipse.draw2d.geometry.PointList;

/**
 * Figure that shows the start state indicator.
 *
 * @author		        Tobias Reinhard
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             2007-09-03 TR First Version
 * @version             $Date: 2007-09-03 08:53:23 +0200 (Mon, 03 Sep 2007) $, $Author: treinhard $, $Revision: 2933 $
 */
public class StartStateIndicatorFigure extends Shape {
	
	/**
	 * Width of the figure
	 */
	public static final int WIDTH = 22;
	
	/**
	 * Height of the figure
	 */
	public static final int HEIGHT = 11;
	
	
	/**
	 * Constructor.
	 *
	 */
	public StartStateIndicatorFigure() {
				
		setSize(WIDTH, HEIGHT);
		
	}


	/**
	 * Doesn't do anything
	 * 
	 * @see org.eclipse.draw2d.Shape#fillShape(org.eclipse.draw2d.Graphics)
	 */
	protected void fillShape(Graphics graphics) {
				
	}


	/**
	 * Draws the start state indicator. 
	 * 
	 * @see org.eclipse.draw2d.Shape#outlineShape(org.eclipse.draw2d.Graphics)
	 */
	protected void outlineShape(Graphics graphics) {
		
		int x = getLocation().x;
		int y = getLocation().y;
		
		graphics.setForegroundColor(ColorConstants.black);
		graphics.setBackgroundColor(ColorConstants.black);
		graphics.fillOval(x, y, 11, 11);
		
		graphics.drawLine(x + 5, y + 5, x + 21, y + 5);
		
		PointList points = new PointList();
		points.addPoint(x + 22, y + 5);
		points.addPoint(x + 14, y);
		points.addPoint(x + 14, y + 11);
		
		graphics.fillPolygon(points);
		
	}

}
