package ch.unizh.ifi.adora.view.figures.label;

import org.eclipse.core.runtime.Preferences;
import org.eclipse.core.runtime.Preferences.IPropertyChangeListener;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferenceConverter;
import org.eclipse.swt.graphics.Font;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.ui.preferences.IPreferenceNameConstants;

/**
 * Represents the label of a node, i.e. abstract object, 
 * object set, actor, etc.
 * 
 * @author				smeier
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			Feb 28, 2006 SM First Version
 *						Mar 28, 2006 SM Bug #538 fixed
 *						Jul 19, 2006 SM Comments reformatted, 
 *										corrected and checked
 * @version			$Date$, $Author$, $Revision$
 * @responsibilities	Represents the label of a node.
 *
 */
public class NodeLabel extends WordWrappingLabel implements IPropertyChangeListener {
	
	private Font labelFont;
	
	 /**
     * Constructor which initializes the label for 
     * listening for font changes.
     */
	public NodeLabel() {
		super();
		
		AdoraPlugin.getDefault().getPluginPreferences().addPropertyChangeListener(this);
		refreshFont();
	}
	
	/**
	 * Creates a new Label with a given text
	 * 
	 * @param text
	 *            Text wich will be displayed.
	 * @param showCardinality
	 *            Decides if the cardinality should be shown.
	 */
	public NodeLabel(String text) {
		this();
        setText(text);
	}
	
	/**
	 * Handler method for preference changes. This 
	 * method listens for font property changes in the 
	 * preferences.
	 */
	public void propertyChange(Preferences.PropertyChangeEvent evt) {
		if(evt.getProperty().equals(IPreferenceNameConstants.ADORA_VIEW_NODE_LABEL_FONT)) {		
			refreshFont();		
		}
	}
	
	/**
	 * Get the font from the preference store and update the visuals accordingly.
	 */
	private void refreshFont() {
		IPreferenceStore store  = AdoraPlugin.getDefault().getPreferenceStore();
		Font font = new Font(null, PreferenceConverter.getFontData(store, IPreferenceNameConstants.ADORA_VIEW_NODE_LABEL_FONT));
		
		setFont(font);
		
		if(labelFont != null && !labelFont.isDisposed()) {
			labelFont.dispose();
		}
		
		labelFont = font;
	}

	@Override
	public void removeNotify() {
		AdoraPlugin.getDefault().getPluginPreferences().removePropertyChangeListener(this);
		super.removeNotify();
	}
}