package ch.unizh.ifi.adora.view.figures.label;

import java.util.Iterator;

import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.geometry.Point;



/**
 * Responsible for displaying the text in a Tooltip.
 * The tool tip occurs if one moves the mousecursor over a figure. 
 * 
 * @author		        Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             07.09.2006 IV First Version
 *                       27.09.2006 JS [Bug 774] Initial value of lineHeight is
 *                                     calculated now
 * @version             $Date: 2009-05-06 10:51:39 +0200 (Wed, 06 May 2009) $, $Author: treinhard $, $Revision: 4182 $
 */
public class ToolTipLabel extends WordWrappingLabel{
	
	/**
	 * Stores the maximal width of the tool tip. 
	 */
	private final int maximalWidth  = 250;

	/**
	 * Stores the lineHeight.
	 */
	private int lineHeight = getGC().getFontMetrics().getHeight() + 4;
	
	/**
	 * Creates a new Label with a given text and a tool tip.
	 * 
	 * @param text Text wich will be displayed.
	 * @param showCardinality Decides if the cardinality should be shown.
	 */
	public ToolTipLabel(String text) {
		super();
		setSize(20,20);
		setText(text);
	}
	
	/**
	 * Creates a new Label with a tool tip.
	 */
	public ToolTipLabel() {
		super();
	}
	
	/**
	 * Draws the text of the label.
	 * 
	 * @param graphics The graphics canvas to draw the text.
	 */
	protected void outlineShape(Graphics graphics) {
		lineHeight  = graphics.getFontMetrics().getHeight() + 4;
		setLocation(new Point(5, 0));
		super.outlineShape(graphics);
	}

	/**
	 * Sets the text of the label.
	 * @param text The text to draw.
	 */
	public void setText(String text) {
		super.setText(text);
		drawToolTip(null) ;
	}
	
	/**
	 *  Draw the tool tip label.
	 *  
	 * @param graphics The graphics on which will be drawn.
	 */
	private void drawToolTip(Graphics graphics) {
		if (getText().equals("")) {
			return;
		}
	
		setSize(maximalWidth, 100);
		int textHeight = generateWrappedText(maximalWidth).size() * lineHeight;
		setSize(getLongestLineWidth() + 20, textHeight + 5);
	}
	
	/**
     * Returns the width of the longest line.
     * 
     * @return The width of the longest line or 0 if no line exists.
     */
    private int getLongestLineWidth() {
        int maxWidth = 0;
        final Iterator<String> iterator = generateWrappedText(maximalWidth).iterator();
        while (iterator.hasNext()) {
            final String element = iterator.next();
            final int length = getGC().stringExtent(element).x;
            maxWidth = Math.max(maxWidth, length);
        }
        return maxWidth;
    }
}