package ch.unizh.ifi.adora.view.figures.label.algorithm.placing;

import org.eclipse.draw2d.AbstractLocator;
import org.eclipse.draw2d.Connection;
import org.eclipse.draw2d.geometry.Point;

import ch.unizh.ifi.adora.view.figures.geometry.PointListUtilities;
import ch.unizh.ifi.adora.view.figures.geometry.LineSeg.KeyPoint;

/**
 * This {@link org.eclipse.draw2d.Locator} is capable of positioning 
 * a {@link org.eclipse.draw2d.IFigure} at the middle of a {@link Connection} 
 * 	regardless of the {@link Connection}'s routing type.
 * 
 * GEF offers a similar {@link org.eclipse.draw2d.Locator} called
 * {@link org.eclipse.draw2d.MidpointLocator} but the problem with it is, 
 * that it is only capable of locating {@link org.eclipse.draw2d.IFigure}s on
 *  {@link Connection}s which are routed using the
 *   {@link org.eclipse.draw2d.ShortestPathConnectionRouter}.
 * 
 * @author		        Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             20.07.2006 IV First Version
 * @version             $Date: 2006-08-31 16:46:01 +0200 (Thu, 31 Aug 2006) $, $Author: vigan $, $Revision: 2156 $
 */
public class ConnectionMidpointLocator extends AbstractLocator {

	/**
	 * Stores the {@link Connection} on which a {@link org.eclipse.draw2d.IFigure}
	 * will be appended.
	 */
	private Connection connection;
	
	/**
	 * Instantiates a new DelegatingMidpointLocator.
	 * 
	 * @param connection The {@link Connection} on which a {@link org.eclipse.draw2d.IFigure}
	 * will be appended.
	 */
	public ConnectionMidpointLocator(Connection connection){
		this.connection = connection;
	}
	
	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.draw2d.AbstractLocator#getReferencePoint()
	 */
	protected Point getReferencePoint() {
		Point midpoint = new Point();
        PointListUtilities.pointOn(connection.getPoints(), 0, 
        		KeyPoint.MIDPOINT, midpoint);
        
        connection.translateToAbsolute(midpoint);
	
        return midpoint;
	}
}