package ch.unizh.ifi.adora.view.figures.label.algorithm.placing;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Locator;
import org.eclipse.draw2d.geometry.Point;

import ch.unizh.ifi.adora.view.figures.LabeledConnection;
import ch.unizh.ifi.adora.view.figures.label.ConnectionLabel;

/**
 * This class is responsible for placing a label at the 
 * It is implemented using the <link rel="Delegating-Pattern" 
 * href="http://de.wikipedia.org/wiki/Dekorierer"/>.
 *         
 * @author		        Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             26.07.2006 IV First Version
 * @version             $Date: 2009-10-30 14:58:38 +0100 (Fri, 30 Oct 2009) $, $Author: treinhard $, $Revision: 4917 $
 */
public class MovableLocator implements Locator{

	/**
	 * Stores the distance between the label and the connection.
	 */
	private Point locationOffset = new Point();

	/**
	 * The Connection for which this {@link Locator}  used.
	 */
	private LabeledConnection con;

	/**
	 * Stores the {@link Locator} which gets decorated by this MovableLocator.
	 */
	private Locator locator;
	
	/**
	 * Instantiates a new MovableLocator.
	 * 
	 * @param con The Connection for which this {@link Locator}  used.
	 * @param locator The {@link Locator} 
	 * 				which gets decorated by this MovableLocator.
	 */
	public MovableLocator(LabeledConnection con, Locator locator) {
		this.con = con;
		this.locator = locator;
	}

	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.draw2d.Locator#relocate(org.eclipse.draw2d.IFigure)
	 */
	public void relocate(IFigure target) {
		locator.relocate(target);
		
		// Try to get the offset set by the label placement algorithm ...
		Point offset = ((ConnectionLabel)target).getMidpointOffset();
		// ... or the one set by the user if none was defined by the algorithm.
		if(offset == null) {
			offset = locationOffset;
		}

		Point location = target.getBounds().getLocation();
		location.x += offset.x;
		location.y += offset.y;
		target.setLocation(location);
		
	}

	/**
	 * Specifies the distance between the label and the connection
	 * end point. This method is called, each time the user drags the 
	 * connection label.
	 * 
	 * @param p The the delta between old and the new position.
	 */
	public void setLocationDelta(Point p) {
		locationOffset.x = p.x;
		locationOffset.y = p.y;
		if( con.getParent()!= null){
			con.refresh();
		}
	}

	/**
	 * Used to determine, whether {@link #setLocationDelta(Point)} 
	 * has to be called or not.
	 * 
	 * @return The distance between the label and the connection.
	 */
	public Point getLocationDelta() {
		return locationOffset;
	}
}