package ch.unizh.ifi.adora.view.figures.label.algorithm.wrapping;

import java.util.ArrayList;
import java.util.List;

/**
 * This class defines a string layouter which uses all types of bracket characters as delimiters.
 * If such a delimiter is found in the searched string, the string will be partitioned
 * at the position of the found delimiter.
 * If this layouter cannot handle a string, it is passed to the next layouter,
 * which searches for a weaker delimiter in the text.
 *      
 *
 * @author		        Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             18.08.2006 IV First Version
 * @version             $Date: 2007-01-25 19:51:22 +0100 (Thu, 25 Jan 2007) $, $Author: smeier $, $Revision: 2409 $
 */
public class BracketDelimiter extends StringDelimiter {

	/**
     * Stores all delimiting characters.
     */
    private final List<Character> delimiters;

    public BracketDelimiter(){
    	delimiters = new ArrayList<Character>();
    	delimiters.add('{');
    	delimiters.add('[');
    	//delimiters.add('(');
    }
    /*
     *  (non-Javadoc)
     * @see ch.unizh.ifi.adora.view.figures.label.algorithm.wrapping.StringDelimiter#nextIndexOfDelimiter(java.lang.String, int)
     */
    public int nextIndexOfDelimiter(String fullString, int index) {
        if (index == 0)
            index = 1;
 
        final int stringLength = fullString.length();
        
        for (int i = index;
             i < stringLength;
             i++) {
            if (delimiters.contains((fullString.charAt(i)))) {
                return i;
            }
        }
        return CONTAINS_NO_DELIMITER;
    }
}
