package ch.unizh.ifi.adora.view.figures.label.algorithm.wrapping;

/**
 * This class defines a string layouter which uses a underscore character as delimiter.
 * If such a delimiter is found in the scanned string, the string will be partitioned
 * at the position of the found delimiter.
 * If this layouter cannot handle a string, it is passed to the next layouter,
 * which searches for a weaker delimiter in the text.
 *
 * @author					Silvio Meier
 * @history				2002-02-13  SM  First version
 *							2002-08-19  SM  Corrected a problem, when starting the string with
 *											a capital character a inifinite loop in the
 *											{@link WordWrappingStringLayouter} was caused which
 *											caused a OutOfMemoryException
 *							2004-02-19  PM  Refactoring
 *							2004-02-20  SM  Comments corrected/Refactoring
 *							2004-10-20  AG  Simplified algorithm
 *							2006-02-15  IV  Moved to Eclipse implementation
 *							2006-07-19  SM  Comments checked, reformatted and corrected
 * @version $Date: 2006-07-27 09:25:04 +0200 (Thu, 27 Jul 2006) $, $Author: vigan $, $Revision: 1911 $
 */
public final class CapitalCharStringDelimiter extends StringDelimiter {

    /**
     * Returns the index of the next upper case character.
     * @pre index > 0
     * @return Retunrs the index of the next upper case character.
     */
    public int nextIndexOfDelimiter(final String fullString, int index) {
        // first character should not be used for delimiting,
        // so begin at index 1, at least one charcter must be contained
        // in the delimited substring
        if (index == 0)
            index = 1;

        final int stringLength = fullString.length();
        for (int i = index;
             i < stringLength;
             i++) {
            if (Character.isUpperCase(fullString.charAt(i))) {
                return i;
            }
        }

        return CONTAINS_NO_DELIMITER;
    }


    /**
     * Creates a new instance of AdoraStringBySpaceLayouter.
     */
    public CapitalCharStringDelimiter() {
        //		super(layouter);
        // inner class which handles the delimiter
        //stringDelimiter = new CapitalStringDelimiter();
        super(-1);
    }
}
