package ch.unizh.ifi.adora.view.figures.label.algorithm.wrapping;

/**
 * This class is a base class for defining a delimitersearch. It is abstract and
 * has to be implemented in a concrete strategy classes which define a method
 * for finding a next delimiter in a given text.
 *
 * @author			Silvio Meier
 * @history		2002-08-13  SM  First version
 *					2006-02-15  IV  Moved to Eclipse implementation
 *					2006-07-19  SM  Comments checked, reformatted and corrected
 * @version $Date: 2006-07-27 09:25:04 +0200 (Thu, 27 Jul 2006) $, $Author: vigan $, $Revision: 1911 $
 *
 */
public abstract class StringDelimiter {

    /**
     * This constant defines the value if no delimiter is found in the string.
     */
    public static final int CONTAINS_NO_DELIMITER = -1;

    /**
     * This shift vector is used to adjust the position, where the string to layout
     * should be cut.
     * By default this shift vector is zero, i.e. the cut is done at the position of
     * the delimiter.
     */
    private final int shiftVector;


    /**
     * Initializes the shiftVector with 0.
     */
    protected StringDelimiter() {
        shiftVector = 0;
    }
    
    /**
     * Initializes the shiftVector with the given value.
     * 
     * @param shiftVector Value for the shiftVector.
     */
    protected StringDelimiter(final int shiftVector){
        this.shiftVector = shiftVector;
    }


    /**
     * This is an abstract method, which should be implemented for
     * a finding a specific delimiter in a given string.
     * @param index index of the character to start the search for
     *        the next delimiter character in the string fullString
     * @param fullString string in which it is searched for a delimiter.
     */
    public abstract int nextIndexOfDelimiter(String fullString, int index);


    /**
     * This method provides a shift vector, which can be used if the delimiter
     * character and the character at which position the text is separated for
     * the layouting differ from each other. With the shift Vector, it can be
     * defined a position which is <i>shiftVector</i> characters away.
     *
     * @return This method returns the #shiftVector attribute.
     */
    public int getShiftVector() {
        return shiftVector;
    }

}
