package ch.unizh.ifi.adora.view.figures.label.algorithm.wrapping;

/**
 * This class defines a string layouter which uses a underscore character as delimiter.
 * If such a delimiter is found in the searched string, the string will be partitioned
 * at the position of the found delimiter.
 * If this layouter cannot handle a string, it is passed to the next layouter,
 * which searches for a weaker delimiter in the text.
 *
 * @author 		Silvio Meier
 * @history    	2002-08-13  SM  First version
 *					2004-02-19  PM  Refactoring
 *					2004-02-20  SM  Comments corrected/Refactoring
 *					2006-02-15  IV  Moved to Eclipse implementation
 *					2006-07-19  SM  Comments checked, reformatted and corrected
 * @version $Date: 2006-07-27 09:25:04 +0200 (Thu, 27 Jul 2006) $, $Author: vigan $, $Revision: 1911 $
 *
 */
public final class UnderlineCharStringDelimiter extends StringDelimiter {

    /**
     * This delimiter character.
     */
    private static final char DELIMITER = '_';


    /**
     * This is an abstract method, which should be implemented for
     * a finding a specific delimiter in a given string.
     * @param index index of the character to start the search for
     *        the next delimiter character in the string fullString
     * @param fullString string in which it is searched for a delimiter.
     */
    public int nextIndexOfDelimiter(String fullString, int index) {
        int pos = fullString.indexOf(DELIMITER, index);
        if (pos == -1) {
            return CONTAINS_NO_DELIMITER;
        } else {
            return (pos + 1);
        }
    }
}
