package ch.unizh.ifi.adora.view.figures.label.associationlabel;

import org.eclipse.draw2d.PolygonDecoration;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PointList;


/**
 * Since our {@link org.eclipse.draw2d.PolygonDecoration} can be moved 
 * by the user, the rotation angle has to be calculated by taking to 
 * points from the connection.     
 *
 * @author				Ivo Vigan
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			17.07.2006 SM First Version<br>
 * 						19.07.2006 SM Comments completed, corrected and 
 *									  checked.<br>
 * @version			$Date: 2006-07-27 14:50:45 +0200 (Thu, 27 Jul 2006) $, $Author: vigan $, $Revision: 1919 $
 */
public class ArrowDecoration extends PolygonDecoration {

	/**
	 * Constructs a new ArrowDecoration with a defined Template.
	 */
	public ArrowDecoration(){
		
		PointList decorationPointListSource = new PointList();
		
		decorationPointListSource.addPoint(-9, 21);
		decorationPointListSource.addPoint(-9, 9);
		decorationPointListSource.addPoint(-30, 15);
		setScale(0.6d, 0.6d);
		setTemplate(decorationPointListSource);
	}
	
	/**
	 * The points which represent the arrow have to be translated in a way, that the
	 * arrow is located in the center of the vertical axis.
	 * This is necessary, because draw2d only deals with absulute coordinates.
	 * 
	 * @return Returns a PointList containing the translated points of this 
	 */
	public PointList getPoints() {
		
		PointList pl = super.getPoints();
		
		if (pl.getBounds() != null && this.getParent() != null) {
			
			int transY = ((AssociationLabel)getParent()).getTextHeight() + 
								getParent().getBounds().getLocation().y - pl.getBounds().y;
			
			int transX = getParent().getBounds().getLocation().x - pl.getBounds().x;
		
			pl.translate(transX, transY);
		}
		
		return pl;
	}
	
	/**
	 * Calculates the rotation angle of the {@link ArrowDecoration}, 
	 * according to the gradient of the connection. 
	 * 
	 * @param pStart Nearest kinkpoint of the connection in relation to the endpoint, 
	 *					where this ArrowDecoration is been associated.
	 * @param pEnd Second nNearest kinkpoint of the connection in relation to the endpoint, 
	 * 					where this ArrowDecoration is been associated.
	 */
	public void setReferencePoint(Point pStart, Point pEnd) {
		
		Point pt = Point.SINGLETON;
		pt.setLocation(pEnd);
		pt.negate().translate(pStart);
		setRotation(Math.atan2(pt.y, pt.x));
	}
}