package ch.unizh.ifi.adora.view.figures.label.associationlabel;

import org.eclipse.draw2d.Connection;
import org.eclipse.draw2d.ConnectionLocator;
import org.eclipse.draw2d.geometry.PointList;


/**
 * This class is used to calculate the gradient of an 
 * {@link ArrowDecoration} by considering the 
 * {@link ch.unizh.ifi.adora.model.representation.Association}.  
 * 
 * @author				Ivo Vigan
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history			17.07.2006 IV First Version
 * 						19.07.2006 SM Comments reformatted, corrected and
 *									  checked.
 * @version			$Date: 2006-07-27 14:50:45 +0200 (Thu, 27 Jul 2006) $, $Author: vigan $, $Revision: 1919 $
 */
public class MovableArrowLocator{

	/**
	 * Stores the {@link ch.unizh.ifi.adora.view.figures.label.associationlabel.ArrowDecoration}, 
	 * which gets rotated by this Locator.
	 */
	private ArrowDecoration arrow;
	
	/**
	 *  Stores the {@link org.eclipse.draw2d.Connection}, 
	 *  to which the {@link ArrowDecoration} belongs.
	 */
	private Connection connection;
	
	/**
	 * Determine whether the 
	 * {@link ch.unizh.ifi.adora.view.figures.label.associationlabel.ArrowDecoration} 
	 * is associated with the source or 
	 * target {@link ch.unizh.ifi.adora.model.representation.Node}
	 */
	private int align;
	
	/**
	 * Instantiates a new MovableArrowLocator.
	 * 
	 * @param connection The {@link Connection}, to whom the {@link ArrowDecoration} belongs.
	 * @param align Determine whether the {@link ArrowDecoration} is associated with the source- or 
	 * 				target {@link ch.unizh.ifi.adora.model.representation.Node}
	 * @param arrow The {@link ArrowDecoration}, which gets rotated by this Locator.
	 */
	public MovableArrowLocator(Connection connection, int align, ArrowDecoration arrow)  {
		this.arrow = arrow;
		this.connection = connection; 
		this.align = align;
	}

	/**
	 * Calculates the gradient of an {@link ArrowDecoration} by considering the
	 * First 2 Points of the {@link ch.unizh.ifi.adora.model.representation.Association}, if the {@link ArrowDecoration} is associated
	 * with the source {@link ch.unizh.ifi.adora.model.representation.Node}, or the last 2 Points, if the {@link ArrowDecoration} is associated
	 * with the target {@link ch.unizh.ifi.adora.model.representation.Node}
	 */
	public void reAdjust(){
		PointList points = connection.getPoints();
		if (align == ConnectionLocator.SOURCE){
			arrow.setReferencePoint(points.getPoint(0), points.getPoint(1));
		}
		
		else if (align == ConnectionLocator.TARGET){
				arrow.setReferencePoint(points.getLastPoint(),
						points.getPoint(points.size()-2));
		}
	}
}
