package ch.unizh.ifi.adora.view.layers;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.eclipse.core.runtime.Preferences.IPropertyChangeListener;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.FreeformLayer;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.geometry.Point;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.linerouting.algorithm.OrthogonalTileMazeRouter;
import ch.unizh.ifi.adora.linerouting.labels.OrthogonalLabelPlacement;
import ch.unizh.ifi.adora.linerouting.tilestructure.TileStructure;
import ch.unizh.ifi.adora.representation.routing.algorithm.IRoutingEventConstants;
import ch.unizh.ifi.adora.ui.preferences.IPreferenceNameConstants;
import ch.unizh.ifi.adora.view.figures.INestedNode;

public class LabelPlacementVisualizationLayer extends FreeformLayer implements PropertyChangeListener, IPropertyChangeListener {
	
	public LabelPlacementVisualizationLayer() {
		
		setEnabled(false);
		setOpaque(false);
		AdoraPlugin.getDefault().getPluginPreferences().addPropertyChangeListener(this);
		
	}
	
	public void propertyChange(PropertyChangeEvent event) {
        String eventType = event.getPropertyName();

        boolean visible = AdoraPlugin.getDefault().getPreferenceStore().getBoolean(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_SHOW_LABEL_POSITIONS);
        
        if(eventType.equals(IRoutingEventConstants.TILESTRUCTURE) && visible) {
        	
        	updateLabelPlaces();
        	
        }
	}

	
	private void updateLabelPlaces() {
		
		removeAll();
		OrthogonalLabelPlacement olp = OrthogonalLabelPlacement.getInstance();
		TileStructure structure = OrthogonalTileMazeRouter.getInstance().getCurrentTileStructure();
		
		int index = 0;
		for(java.awt.Rectangle rect: olp.getAllLabelPlaces()) {
			
			Label label = new Label(String.format("%.1f", olp.getAllScores().get(index)));
			label.setForegroundColor(ColorConstants.darkGreen);
			label.setTextAlignment(PositionConstants.TOP);
			label.setLabelAlignment(PositionConstants.LEFT);
			label.setLocation(new Point(rect.x, rect.y));
			label.setSize(rect.width, rect.height);
			label.setBorder(new LineBorder(ColorConstants.darkGreen));
			
			// Make the coordinates of the figure absolute
			Point location = ((INestedNode)structure.getParent()).getAbsoluteLocation();
	        label.translate(location.x, location.y);
			
	        add(label);
	        index++;
			
		}
		
	}

	public void propertyChange(org.eclipse.core.runtime.Preferences.PropertyChangeEvent event) {
		String eventType = event.getProperty();
        if(eventType.equals(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_SHOW_LABEL_POSITIONS)) {
        	
        	boolean visible = AdoraPlugin.getDefault().getPreferenceStore().getBoolean(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_SHOW_LABEL_POSITIONS);
        	if(visible == false) {
        		removeAll();
        	} else {
        		updateLabelPlaces();
        	}
        }
		
	}
	
	@Override
	public void removeNotify() {
		AdoraPlugin.getDefault().getPluginPreferences().removePropertyChangeListener(this);
		super.removeNotify();
	}
	
}
