package ch.unizh.ifi.adora.view.layers;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Iterator;

import org.eclipse.core.runtime.Preferences.IPropertyChangeListener;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.FreeformLayer;
import org.eclipse.draw2d.geometry.Point;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.linerouting.algorithm.OrthogonalTileMazeRouter;
import ch.unizh.ifi.adora.linerouting.tilestructure.Tile;
import ch.unizh.ifi.adora.linerouting.tilestructure.TileStructure;
import ch.unizh.ifi.adora.representation.routing.algorithm.IRoutingEventConstants;
import ch.unizh.ifi.adora.ui.preferences.IPreferenceNameConstants;
import ch.unizh.ifi.adora.view.figures.INestedNode;

/**
 * Layer that shows auxiliary routing structures.
 *
 * @author		        Tobias Reinhard
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             2007-04-19 TR First Version
 * @version             $Date: 2009-12-23 14:25:55 +0100 (Wed, 23 Dec 2009) $, $Author: jeanneret $, $Revision: 5005 $
 * @responsibilities	Shows the auxiliary structures for the routing
 *
 */
public class RoutingVisualizationLayer extends FreeformLayer implements PropertyChangeListener, IPropertyChangeListener {
	
	
	/**
	 * Layer to show the tile structure on
	 */
	private FreeformLayer tileStructure;
	
	private FreeformLayer costPoints;
	
	
	public RoutingVisualizationLayer() {
		
    	setEnabled(false);
        AdoraPlugin.getDefault().getPluginPreferences().addPropertyChangeListener(this);

        tileStructure = new FreeformLayer();
        tileStructure.setForegroundColor(ColorConstants.gray);
        tileStructure.setVisible(AdoraPlugin.getDefault().getPreferenceStore().getBoolean(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_SHOW_TILES));
        tileStructure.setOpaque(false);
        add(tileStructure);
        
        costPoints = new FreeformLayer();
        costPoints.setForegroundColor(ColorConstants.red);
        costPoints.setVisible(AdoraPlugin.getDefault().getPreferenceStore().getBoolean(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_SHOW_TILES));
        costPoints.setOpaque(false);
        add(costPoints);
        
	}
	
	public void propertyChange(org.eclipse.core.runtime.Preferences.PropertyChangeEvent event) {
		
		String eventType = event.getProperty();
        if(eventType.equals(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_SHOW_TILES)) {
        	
        	boolean visible = AdoraPlugin.getDefault().getPreferenceStore().getBoolean(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_SHOW_TILES);
        	if(visible == false) {
        		tileStructure.removeAll();
        		costPoints.removeAll();
        	} else {
        		updateTileFigures();
            	updateCostPointFigures();
        	}
        }
        else if(eventType.equals(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_COLOR_TILES)) {
        	
        	updateTileFigures();
        	
        }
	}
	
	
	public void propertyChange(PropertyChangeEvent event) {
        String eventType = event.getPropertyName();

        boolean visible = AdoraPlugin.getDefault().getPreferenceStore().getBoolean(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_SHOW_TILES);
        
        if(eventType.equals(IRoutingEventConstants.TILESTRUCTURE) && visible) {
        	
        	updateTileFigures();
        	updateCostPointFigures();
        	
        }
	}
	
	
	private void updateTileFigures() {
		
		tileStructure.removeAll();
		
		TileStructure structure = OrthogonalTileMazeRouter.getInstance().getCurrentTileStructure();
		
		Iterator<Tile> iter = structure.getAllTiles();
    	while(iter.hasNext()) {
    		Tile t = iter.next();
    		
    		TileFigure label = new TileFigure(t);        
            tileStructure.add(label);
            
    	}
	}
	
	
	private void updateCostPointFigures() {
		
		costPoints.removeAll();
		
		TileStructure structure = OrthogonalTileMazeRouter.getInstance().getCurrentTileStructure();
		Iterator<Tile> iter = structure.getAllTiles();
    	while(iter.hasNext()) {
    		Tile t = iter.next();
    		
    		if (t.getCostPoint() != null) {
    			addCostPoint(t.getCostPoint());
    		}
    	}
    	
	}
	
	
	private void addCostPoint(java.awt.Point loc) {
		
		CostPointFigure point = new CostPointFigure(new Point(loc.x, loc.y));
		TileStructure structure = OrthogonalTileMazeRouter.getInstance().getCurrentTileStructure();
		
		// Make the coordinates absolute
		Point location = ((INestedNode)structure.getParent()).getAbsoluteLocation();
        point.translate(location.x, location.y);
		
		costPoints.add(point);
		
	}
	@Override
	public void removeNotify() {
		AdoraPlugin.getDefault().getPluginPreferences().removePropertyChangeListener(this);
		super.removeNotify();
	}
}
