package ch.unizh.ifi.adora.view.layers;

import java.util.Iterator;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Display;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.linerouting.algorithm.OrthogonalTileMazeRouter;
import ch.unizh.ifi.adora.linerouting.tilestructure.Tile;
import ch.unizh.ifi.adora.linerouting.tilestructure.TileStructure;
import ch.unizh.ifi.adora.ui.preferences.IPreferenceNameConstants;
import ch.unizh.ifi.adora.view.figures.INestedNode;


/**
 * Figure to visualize the tiles of the tile maze router.
 *
 * @author		        Tobias Reinhard
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             2008-02-19 TR First Version
 * @version             $Date: 2008-06-13 15:23:55 +0200 (Fri, 13 Jun 2008) $, $Author: treinhard $, $Revision: 3638 $
 */
public class TileFigure extends Label {
	
	public TileFigure(Tile t) {
		
		super(t.getName() + ": " + t.getCost());
		
		setLocation(new Point(t.getTopLeft().x, t.getTopLeft().y));
		setSize(t.getWidth(), t.getHeight());
		setLabelAlignment(PositionConstants.CENTER);
		setForegroundColor(ColorConstants.lightGray);
		setBorder(new LineBorder(ColorConstants.lightGray));
		
		int maxValue = 0;
		int minValue = Integer.MAX_VALUE;
		
		TileStructure structure = OrthogonalTileMazeRouter.getInstance().getCurrentTileStructure();
		Iterator<Tile> iter = structure.getAllTiles();
		while(iter.hasNext()) {
			int cost = iter.next().getCost();
			if(cost > maxValue) {
				maxValue = cost;
			}
			if(cost > 0 && cost < minValue) {
				minValue = cost;
			}
		}
		
		IPreferenceStore ps = AdoraPlugin.getDefault().getPreferenceStore();
		if(ps.getBoolean(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_COLOR_TILES) && t.getCost() > 0) {
			setCostColor(t.getCost(), minValue, maxValue);
			setOpaque(true);
		}
        
		// Make the coordinates of the figure absolute
		Point location = ((INestedNode)structure.getParent()).getAbsoluteLocation();
        translate(location.x, location.y);
		
		// Don't show the bottom boundary, only show the top boundary of the
        // next tile.
		if(t.getLb() != null) {
			setSize(getSize().width, getSize().height + 1);
		}
		
		// Don't show the right boundary, only show the left boundary of the
        // next tile.
		if(t.getTr() != null) {
			setSize(getSize().width + 1, getSize().height);
		}
		
	}
	
	
	private void setCostColor(int cost, int minCost, int maxCost) {
		
		int startR = 255;
		int startG = 246;
		int startB = 231;
		
		int endR = 224;
		int endG = 186;
		int endB = 111;
		
		int r = (int)((((float)cost - minCost) / (maxCost - minCost)) * (endR - startR) + startR);
		int g = (int)((((float)cost - minCost) / (maxCost - minCost)) * (endG - startG) + startG);
		int b = (int)((((float)cost - minCost) / (maxCost - minCost)) * (endB - startB) + startB);
		
		setBackgroundColor(new Color(Display.getDefault(), r, g, b));
		
	}
	
	
	/**
	 * Disposes the os resources (color).
	 */
	public void finalize() {
		
		Color color = getBackgroundColor();
		
		if(color != null && !color.isDisposed()) {
			color.dispose();
		}
		
	}
	
}
