package ch.unizh.ifi.adora.empiricism.adoraeditor.model;

import java.io.InputStream;

import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PlatformUI;

import ch.unizh.ifi.adora.ui.AdoraModelEditor;
import ch.unizh.ifi.adora.ui.storage.AdoraInput;

/**
 * Responsible for illustrating the model in the editor.
 * 
 * @author		        Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             26.06.2006 SM First Version
 * @version             $Date: 2009-05-06 10:51:39 +0200 (Wed, 06 May 2009) $, $Author: treinhard $, $Revision: 4182 $
 */
public class AdoraModelLoader {
	
	/**
	 * Shows the a model in the adora editor, if a model is associated with 
     * this question.
	 * @pre       true
	 * @post      true
	 * @obligation 
	 * @param stream
	 */
	public static void showModel(InputStream stream, String name){
		IWorkbenchPage page = null;
		if(stream == null){
			return;
		}

		try {
			page = PlatformUI.getWorkbench().
				getActiveWorkbenchWindow().getActivePage();
			//The page field is the active part of the Workbench.
	        page.openEditor(new AdoraInput(stream, name), "Adora Model Editor", true);
		}
		catch (Exception ex) {
            ex.printStackTrace();
        }
		AdoraModelEditor ame = (AdoraModelEditor)page.getActiveEditor();
        ame.getCommandStack().addCommandStackEventListener(AdoraCommandListener.getInstance());
	}
	
	/**
	 * Hides the model.
	 * 
	 * @pre       true
	 * @post      true
	 * @obligation
	 */
	public static void hideModel(){
		IWorkbenchPage page = PlatformUI.getWorkbench().
			getActiveWorkbenchWindow().getActivePage();
		page.closeAllEditors(false);
	}
}