package ch.unizh.ifi.adora.empiricism.model;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;


/**
 * Stores all PropertyName, PropertyValue pairs, obtained from db for a given question.
 *
 * @author		        Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             26.06.2006 SM First Version
 * @version             $Date: 2006-06-26 16:27:53 +0200 (Mon, 26 Jun 2006) $, $Author: vigan $, $Revision: 1737 $
 */
public class AdoraPropertySet extends DBObjectSet{

	/**
	 * The serial id for the serialization mechanism.
	 */
	private static final long serialVersionUID = 5988216635004366367L;

	/**
	 * The Question which is associated with the AdoraProperty set.
	 */
	private Question q;
	
	/**
	 * Instantiates a new set of questions for a given test suite in a given database.
	 * 
	 * @param q  The Question which is associated with the AdoraProperty set.
	 * @param db The database.
	 */
	public AdoraPropertySet(Question q, Database db) {
		super(db);
		this.q = q;
		this.refreshObjects();
	}

	@Override
	protected String getSelectStatement() {
		return DataBaseSQLStrings.SQL_ADORA_PROPERTY_OF_QUESTION;
	}

	@Override
	protected void prepareSelectStatement(PreparedStatement statement) {
		try {
			statement.setInt(1, q.getId());
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	@Override
	protected Object createThisObject(ResultSet resultSet) throws SQLException {
		AdoraProperty p = null;
		int id = resultSet.getInt(1);
		String propertyName = resultSet.getString(2);
		if (db.getAdoraPropertyCache().getDBObject(id + "" + propertyName) != null) {
			p = db.getAdoraPropertyCache().getDBObject(id + "" + propertyName);
		} else {
			p = new AdoraProperty(db, q, propertyName);
		}
		return p;
	}
}
