package ch.unizh.ifi.adora.empiricism.model;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * Used to determin if a user has allready answerd a userinfo.
 *
 * @author		        Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             13.06.2006 SM First Version
 * @version             $Date: 2007-01-21 13:14:36 +0100 (Sun, 21 Jan 2007) $, $Author: smeier $, $Revision: 2385 $
 */
public class AnsweredUserInfoSet extends DBObjectSet{

	/**
	 *
	 */
	private static final long s = -4781028403557236909L;

	/**
	 * User for which all allready given answers are stored.
	 */
	User user;

	/**
	 *
	 * @param db
	 * @param User for which all allready given answers are stored.
	 */
	public AnsweredUserInfoSet(Database db,User user) {
		super(db);
		this.user = user;
		refreshObjects();
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DBObjectSet#getSelectStatement()
	 */
	@Override
	protected String getSelectStatement() {
		return DataBaseSQLStrings.SQL_GET_ALL_ANSWERED_USER_INFOS_FOR_GIVE_USER;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DBObjectSet#prepareSelectStatement(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareSelectStatement(PreparedStatement statement) {
		try {
			statement.setInt(1, user.getID());
		} catch (SQLException ex) {
			ex.printStackTrace();
		}

	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DBObjectSet#createThisObject(java.sql.ResultSet)
	 */
	@Override
	protected Object createThisObject(ResultSet resultSet) throws SQLException {
		UserInfo t=null;
		int userInfoID = resultSet.getInt(1);

		if (db.getUserInfoCache().getDBObject(userInfoID) != null) {
			t = db.getUserInfoCache().getDBObject(userInfoID);
		} else {
			t = new UserInfo(userInfoID, db);
		}
		return t;
	}
}