package ch.unizh.ifi.adora.empiricism.model;

/**
 * This method contains all SQL statements for making the (database) object to relational
 * database mapping.
 *
 * @author		       smeier
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             May 22, 2006 SM First Version
 *                      May 29, 2006 SM Comments
 *                      Jan 21, 2007 SM Changes for loading more than one model per question.
 * @version             $Date: 2008-12-02 14:44:15 +0100 (Tue, 02 Dec 2008) $, $Author: vigan $, $Revision: 3931 $
 *
 */
public interface DataBaseSQLStrings {
	//--------------------------------------
	//-------- DatabaseObjects statements
	//--------------------------------------

	// ------ Table USER SQL statements
	public static final String SQL_CREATE_USER =
		"INSERT INTO USER (ID, AGE, CREATED) values (?, ?, CURRENT_TIMESTAMP)";

	public static final String SQL_GET_NEXT_USER_ID="SELECT NEXT VALUE FOR SEQ_USER_ID FROM INFORMATION_SCHEMA.SYSTEM_TABLES WHERE TABLE_NAME='USER'";

	public static final String SQL_UPDATE_USER="UPDATE USER SET age=?, created=? WHERE ID=?";

	public static final String SQL_SELECT_USER="SELECT ID, AGE, CREATED FROM USER WHERE ID=?";

	public static final String SQL_DELETE_USER="DELETE FROM USER WHERE ID=?";

	public static final String SQL_GET_ALL_NON_ANSWERED_TEST_SUITES =

		"SELECT  Distinct * " +
		"FROM TEST_SUITE " +
		"WHERE   TEST_SUITE.ID Not in ( SELECT  QUESTION.TEST_SUITE_ID " +
		"								FROM    USER_ANSWERS_OF_TEST_SUITE_QUESTION, QUESTION " +
		"								WHERE   USER_ANSWERS_OF_TEST_SUITE_QUESTION.QUESTION_ID = QUESTION.ID " +
		"								AND     USER_ANSWERS_OF_TEST_SUITE_QUESTION.USER_ID = ?)"+
        " Order by ID";


	// -------- Table TEST_SUITE SQL statements
	public static final String SQL_CREATE_TEST_SUITE =
		"INSERT INTO TEST_SUITE (ID, SUITE_NAME, NO_OF_EXECUTED_TESTS, START_UI_INFO, START_QUESTION_INFO, END_SUITE_INFO) values (?, ?,?,?,?,?)";

	public static final String SQL_GET_NEXT_TEST_SUITE_ID="SELECT NEXT VALUE FOR SEQ_TEST_SUITE_ID FROM INFORMATION_SCHEMA.SYSTEM_TABLES WHERE TABLE_NAME='USER'";

	public static final String SQL_SELECT_TEST_SUITE="SELECT ID, SUITE_NAME, NO_OF_EXECUTED_TESTS, START_UI_INFO, START_QUESTION_INFO, END_SUITE_INFO FROM TEST_SUITE WHERE ID=?";

	public static final String SQL_DELETE_TEST_SUITE="DELETE FROM TEST_SUITE WHERE ID=?";

	public static final String SQL_UPDATE_TEST_SUITE="UPDATE TEST_SUITE SET SUITE_NAME=?, NO_OF_EXECUTED_TESTS=?, START_UI_INFO=?, START_QUESTION_INFO=?, END_SUITE_INFO=? WHERE ID = ?";

	public static final String SQL_GET_ALL_TEST_SUITES="SELECT * FROM TEST_SUITE";

	public static final String SQL_GET_LEAST_EXECUTED_SUITES=" SELECT MIN(NO_OF_EXECUTED_TESTS) ID FROM TEST_SUITE";

	public static final String SQL_GET_ALL_ANSWERED_USER_INFOS_FOR_GIVE_USER ="SELECT USER_INFO_ID FROM USER_ANSWERS_OF_USER_INFO WHERE USER_ID = ?";

	// --------- Table QUESTIONS SQL statements
	public static final String SQL_CREATE_QUESTION =
		"INSERT INTO QUESTION (ID, QUESTION, QUESTION_TYPE, NON_MC_ITEM, TEST_SUITE_ID, START_INFO_TEXT) values (?, ?,?,?,?,?)";

	public static final String SQL_GET_NEXT_QUESTION_ID = "SELECT NEXT VALUE FOR SEQ_QUESTION_ID FROM INFORMATION_SCHEMA.SYSTEM_TABLES WHERE TABLE_NAME='USER'";

	public static final String SQL_SELECT_QUESTION      = "SELECT ID, QUESTION, QUESTION_TYPE, NON_MC_ITEM, TEST_SUITE_ID, START_INFO_TEXT FROM QUESTION WHERE ID=?";

	public static final String SQL_DELETE_QUESTION      = "DELETE FROM TEST_SUITE WHERE ID=?";

	public static final String SQL_UPDATE_QUESTION      = "UPDATE QUESTION SET QUESTION=?, QUESTION_TYPE=?, NON_MC_ITEM=?, TEST_SUITE_ID=?, START_INFO_TEXT = ? WHERE ID = ?";

	// --------- Table MODEL_FOR_QUESTIONS SQL statements
	public static final String SQL_CREATE_MODEL_FOR_QUESTION =
		"INSERT INTO MODEL_FOR_QUESTION (ID, NAME, QUESTION_ID ) values (?, ?,?)";

	public static final String SQL_GET_NEXT_MODEL_FOR_QUESTION_ID = "SELECT NEXT VALUE FOR SEQ_MODEL_FOR_QUESTION_ID FROM INFORMATION_SCHEMA.SYSTEM_TABLES WHERE TABLE_NAME='USER'";

	public static final String SQL_SELECT_MODEL_FOR_QUESTION      = "SELECT ID, NAME, QUESTION_ID, ADORA_MODEL FROM MODEL_FOR_QUESTION WHERE ID=?";

	public static final String SQL_DELETE_MODEL_FOR_QUESTION      = "DELETE FROM MODEL_FOR_QUESTION WHERE ID=?";

	public static final String SQL_UPDATE_MODEL_FOR_QUESTION      = "UPDATE MODEL_FOR_QUESTION SET NAME=?, QUESTION_ID=? WHERE ID = ?";


	//	 --------- Table ADORA_PROPERTY SQL statements
	public static final String SQL_CREATE_ADORA_PROPERTY 		= "INSERT INTO ADORA_PROPERTY (QUESTION_ID, PROPERTY_NAME, PROPERTY_VALUE) values (?,?,?)";

	public static final String SQL_SELECT_ADORA_PROPERTY 		 = "SELECT PROPERTY_NAME, PROPERTY_VALUE FROM ADORA_PROPERTY WHERE QUESTION_ID = ? AND PROPERTY_NAME = ?";

	public static final String SQL_GET_LAST_ADORA_PROPERTY_ID	 = "SELECT max(QUESTION_ID) from ADORA_PROPERTY";

	public static final String SQL_DELETE_ADORA_PROPERTY       = "DELETE FROM ADORA_PROPERTY WHERE QUESTION_ID = ? AND PROPERTY_NAME = ?";

	public static final String SQL_UPDATE_ADORA_PROPERTY     	 = "UPDATE ADORA_PROPERTY SET PROPERTY_VALUE = ? WHERE QUESTION_ID = ? AND PROPERTY_NAME = ?";



	//	 --------- Table MC_ITEM SQL statements
	public static final String SQL_CREATE_MC_ITEM =
		"INSERT INTO MC_ITEM (ID, ANSWER, QUESTION_ID, CORRECT) values (?,?,?,?)";

	public static final String SQL_GET_NEXT_MC_ITEM_ID = "SELECT NEXT VALUE FOR SEQ_MC_ITEM_ID FROM INFORMATION_SCHEMA.SYSTEM_TABLES WHERE TABLE_NAME='USER'";

	public static final String SQL_SELECT_MC_ITEM      = "SELECT ID, ANSWER, QUESTION_ID, CORRECT FROM MC_ITEM WHERE ID=?";

	public static final String SQL_DELETE_MC_ITEM      = "DELETE FROM MC_ITEM WHERE ID=?";

	public static final String SQL_UPDATE_MC_ITEM      = "UPDATE MC_ITEM SET ANSWER=?, QUESTION_ID=?, CORRECT=? WHERE ID = ?";


	//	 --------- Table USER_INFO SQL statements
	public static final String SQL_CREATE_USER_INFO =
		"INSERT INTO USER_INFO (ID, INFO_NAME, INFO_TYPE) values (?, ?,?)";

	public static final String SQL_GET_NEXT_USER_INFO_ID = "SELECT NEXT VALUE FOR SEQ_USER_INFO_ID FROM INFORMATION_SCHEMA.SYSTEM_TABLES WHERE TABLE_NAME='USER'";

	public static final String SQL_SELECT_USER_INFO      = "SELECT ID, INFO_NAME, INFO_TYPE FROM USER_INFO WHERE ID=?";

	public static final String SQL_DELETE_USER_INFO      = "DELETE FROM USER_INFO WHERE ID=?";

	public static final String SQL_UPDATE_USER_INFO      = "UPDATE USER_INFO SET INFO_NAME=?, INFO_TYPE=? WHERE ID = ?";


	//	 --------- Table USER_INFO_MC_ITEM SQL statements
	public static final String SQL_CREATE_USER_INFO_MC_ITEM =
		"INSERT INTO USER_INFO_MC_ITEM (ID, ANSWER, USER_INFO_ID) values (?, ?,?)";

	public static final String SQL_GET_NEXT_USER_INFO_MC_ITEM_ID = "SELECT NEXT VALUE FOR SEQ_USER_INFO_MC_ITEM_ID FROM INFORMATION_SCHEMA.SYSTEM_TABLES WHERE TABLE_NAME='USER'";

	public static final String SQL_SELECT_USER_INFO_MC_ITEM      = "SELECT ID, ANSWER, USER_INFO_ID FROM USER_INFO_MC_ITEM WHERE ID=?";

	public static final String SQL_DELETE_USER_INFO_MC_ITEM      = "DELETE FROM USER_INFO_MC_ITEM WHERE ID=?";

	public static final String SQL_UPDATE_USER_INFO_MC_ITEM      = "UPDATE USER_INFO_MC_ITEM SET ANSWER=?, USER_INFO_ID=? WHERE ID = ?";

	//	 --------- Table USER_ANSWERS_OF_USER_INFO_MC_ITEM SQL statements
	public static final String SQL_CREATE_USER_ANSWERS_OF_USER_INFO_MC_ITEM =
		"INSERT INTO USER_ANSWERS_OF_USER_INFO_MC_ITEM (USER_ID, USER_INFO_MC_ITEM_ID, CORRECT) values (?,?,?)";

	public static final String SQL_SELECT_USER_ANSWERS_OF_USER_INFO_MC_ITEM      = "SELECT USER_ID, USER_INFO_MC_ITEM_ID, CORRECT FROM USER_ANSWERS_OF_USER_INFO_MC_ITEM WHERE USER_ID=? AND USER_INFO_MC_ITEM_ID=?";

	public static final String SQL_DELETE_USER_ANSWERS_OF_USER_INFO_MC_ITEM      = "DELETE FROM USER_ANSWERS_OF_USER_INFO_MC_ITEM WHERE USER_ID=? AND USER_INFO_MC_ITEM_ID=?";

	public static final String SQL_UPDATE_USER_ANSWERS_OF_USER_INFO_MC_ITEM      = "UPDATE USER_ANSWERS_OF_USER_INFO_MC_ITEM SET USER_ID=?, USER_INFO_MC_ITEM_ID=?, CORRECT=? WHERE USER_ID = ? and USER_INFO_MC_ITEM_ID = ?";


	//	 --------- Table USER_ANSWERS_OF_TEST_SUITE_QUESTION SQL statements
	public static final String SQL_CREATE_USER_ANSWERS_OF_QUESTION =
			"INSERT INTO USER_ANSWERS_OF_TEST_SUITE_QUESTION (USER_ID, QUESTION_ID, TIME_USED, START_TIME, USER_ANSWER) values (?,?,?,?,?)";

	public static final String SQL_SELECT_USER_ANSWERS_OF_QUESTION      = "SELECT USER_ID, QUESTION_ID, TIME_USED, START_TIME, USER_ANSWER FROM USER_ANSWERS_OF_TEST_SUITE_QUESTION WHERE USER_ID=? AND QUESTION_ID=?";

	public static final String SQL_DELETE_USER_ANSWERS_OF_QUESTION      = "DELETE FROM USER_ANSWERS_OF_TEST_SUITE_QUESTION WHERE USER_ID=? AND QUESTION_ID=?";

	public static final String SQL_UPDATE_USER_ANSWERS_OF_QUESTION      = "UPDATE USER_ANSWERS_OF_TEST_SUITE_QUESTION SET USER_ID=?, QUESTION_ID=?, TIME_USED=?, START_TIME=?, USER_ANSWER=? WHERE USER_ID = ? and QUESTION_ID = ?";

	//	 --------- Table USER_ANSWERS_OF_MC_ITEMS SQL statements
	public static final String SQL_CREATE_USER_ANSWERS_OF_MC_ITEM =
			"INSERT INTO USER_ANSWERS_OF_MC_ITEMS (USER_ID, MC_ITEM_ID, CORRECT) values (?,?,?)";

	public static final String SQL_SELECT_USER_ANSWERS_OF_MC_ITEM      = "SELECT USER_ID, MC_ITEM_ID, CORRECT FROM USER_ANSWERS_OF_MC_ITEMS WHERE USER_ID=? AND MC_ITEM_ID=?";

	public static final String SQL_DELETE_USER_ANSWERS_OF_MC_ITEM      = "DELETE FROM USER_ANSWERS_OF_MC_ITEMS WHERE USER_ID=? AND MC_ITEM_ID=?";

	public static final String SQL_UPDATE_USER_ANSWERS_OF_MC_ITEM      = "UPDATE USER_ANSWERS_OF_MC_ITEMS SET USER_ID=?, MC_ITEM_ID=?, CORRECT=? WHERE USER_ID = ? and MC_ITEM_ID = ?";


	//	 --------- Table USER_ANSWERS_OF_USER_INFO SQL statements
	public static final String SQL_CREATE_USER_ANSWERS_OF_USER_INFO =
			"INSERT INTO USER_ANSWERS_OF_USER_INFO (USER_ID, USER_INFO_ID, ANSWER) values (?,?,?)";

	public static final String SQL_SELECT_USER_ANSWERS_OF_USER_INFO      = "SELECT USER_ID, USER_INFO_ID, ANSWER FROM USER_ANSWERS_OF_USER_INFO WHERE USER_ID=? AND USER_INFO_ID=?";

	public static final String SQL_DELETE_USER_ANSWERS_OF_USER_INFO      = "DELETE FROM USER_ANSWERS_OF_USER_INFO WHERE USER_ID=? AND USER_INFO_ID=?";

	public static final String SQL_UPDATE_USER_ANSWERS_OF_USER_INFO      = "UPDATE USER_ANSWERS_OF_USER_INFO SET USER_ID=?, USER_INFO_ID=?, ANSWER=? WHERE USER_ID = ? AND USER_INFO_ID = ?";


	//	 --------- Table TEST_SUITE_USER_INFO SQL statements
	public static final String SQL_CREATE_TEST_SUITE_USER_INFO 		=	"INSERT INTO TEST_SUITE_USER_INFO (TEST_SUITE_ID, USER_INFO_ID) values (?,?)";

	public static final String SQL_SELECT_TEST_SUITE_USER_INFO     = 	"SELECT TEST_SUITE_ID, USER_INFO_ID FROM TEST_SUITE_USER_INFO WHERE TEST_SUITE_ID=? AND USER_INFO_ID=?";

	public static final String SQL_DELETE_TEST_SUITE_USER_INFO     	= 	"DELETE FROM TEST_SUITE_USER_INFO WHERE WHERE TEST_SUITE_ID=? AND USER_INFO_ID=?";

	public static final String SQL_UPDATE_TEST_SUITE_USER_INFO      = 	"UPDATE TEST_SUITE_USER_INFO SET TEST_SUITE_ID=?, USER_INFO_ID=? WHERE TEST_SUITE_ID=? AND USER_INFO_ID=?";

	// --------- Table LOGGED_COMMANDS SQL statements
	public static final String SQL_CREATE_LOGGED_COMMANDS 		=	"INSERT INTO LOGGED_COMMANDS (USER_ID, COMMAND_NAME, EXECUTED) values (?, ?, CURRENT_TIMESTAMP)";
	
	//--------------------------------------
	//-------- SET STATEMENTS
	//--------------------------------------
	//--- Question set of a suite
	public static final String SQL_QUESTION_SET_OF_SUITE_SELECT = "SELECT ID FROM QUESTION WHERE TEST_SUITE_ID=?";

	//	--- Model for Question set of a suite
	public static final String SQL_MODEL_FOR_QUESTION_SET_OF_SUITE_SELECT = "SELECT ID FROM MODEL_FOR_QUESTION WHERE QUESTION_ID=?";

	//	--- USER INFO set of suite
	public static final String SQL_USER_INFO_SET_OF_TEST_SUITE_SELECT = "SELECT USER_INFO_ID FROM TEST_SUITE_USER_INFO WHERE TEST_SUITE_ID=?";

	//	--- MC Answer of Question
	public static final String SQL_MC_ITEM_OF_QUESTION_SELECT = "SELECT ID FROM MC_ITEM WHERE QUESTION_ID=?";

	//	--- MC Answer of user info
	public static final String SQL_USER_INFO_MC_ITEM_OF_USER_INFO_SELECT = "SELECT ID FROM USER_INFO_MC_ITEM WHERE USER_INFO_ID=?";

	//	--- Adora Properties of Question
	public static final String SQL_ADORA_PROPERTY_OF_QUESTION 	= "SELECT QUESTION_ID, PROPERTY_NAME, PROPERTY_VALUE FROM ADORA_PROPERTY WHERE QUESTION_ID = ?";



}
