package ch.unizh.ifi.adora.empiricism.model;

/**
 * Represents a multiple choice answer or optional answer of a 
 * {@link ch.unizh.ifi.adora.empiricism.model.Question}. A multiple choice {@link #answer}
 * consists of an answer string which gives a possible answer to a  
 * {@link ch.unizh.ifi.adora.empiricism.model.Question}. The answer can be {@link #correct}
 * or not. A mutliple choice answer has a unique {@link #id} which identifies it whithin all
 * exisiting multiple choice answers. There is the {@link #question} to which the answer
 * belongs to.<br>
 * This multiple can be answered be answered by a {@link ch.unizh.ifi.adora.empiricism.model.User}
 * and the user answer is stored in a {@link ch.unizh.ifi.adora.empiricism.model.UserAnswersOfMcItems}.
 * 
 *      
 * @author		        smeier
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             May 26, 2006 SM First Version
 *                      May 29, 2006 SM Comments
 * @version             $Date: 2006-06-30 12:15:37 +0200 (Fri, 30 Jun 2006) $, $Author: vigan $, $Revision: 1752 $
 *
 */
public abstract class McItem extends DatabaseObject{

	/**
	 * The unique id of the database tuple of this user info mutliple choice item.
	 */
	protected int id = -1;
	
	/**
	 * The text of the question.
	 */
	protected String description="";
	
	
	/**
	 * Creates a new multiple choice answer for a given question for a given database,
	 * by creating a new record for the multiple choice answer.
	 * 
	 * @param question The question for which a new multiple choice answer should be
	 *                 created.
	 * @param db       The database in which the multiple choice will be created.
	 */
	public McItem(Database db) {
		super(db);
	}
	
	/**
	 * Returns the unique id of the user info multiple choice item.
	 * @return Returns the id of the multiple choice item. 
	 */
	public int getId() {
		return id;
	}
	
	/**
	 * Returns the statement of this multiple choice item, which has the user to mark as 
	 * true or false.
	 * 
	 * @return Returns the statement of this multiple choice item.
	 */
	public String getDescription() {
		return description;
	}
	
	/**
	 * Sets the statement of this multiple choice item, which the user has to mark as true
	 * or false.
	 * @param desc The description of the statement.
	 */
	public void setDescription(String desc) {
		description = desc;
		changed = true;
	}
}
