package ch.unizh.ifi.adora.empiricism.model;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * This instances of this class represent a set of {@link ch.unizh.ifi.adora.empiricism.model.Question}
 * objects.
 *
 * @author		       smeier
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             May 26, 2006 SM First Version
 *                      May 28, 2006 SM Some refactoring.
 *                      May 29, 2006 SM Comments
 *                      Jan 21, 2007 SM Changes for loading more than one model per question.
 * @version             $Date: 2007-01-21 13:14:36 +0100 (Sun, 21 Jan 2007) $, $Author: smeier $, $Revision: 2385 $
 *
 */
public class ModelForQuestionSet extends DBObjectSet {



	/**
	 * The serial id for the serialization mechanism.
	 */
	private static final long serialVersionUID = 5643372849262217739L;

	/**
	 * The test suite which is associated with this set of questions.
	- */
	private Question question;

	/**
	 * Instantiates a new set of questions for a given test suite in a given database.
	 *
	 * @param suite The test suite which is associated with the question set.
	 * @param db The database with which the questions and the test suite are associated.
	 */
	protected ModelForQuestionSet(Question suite, Database db) {
		super(db);
		this.question = suite;

		refreshObjects();
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DBObjectSet#getSelectStatement()
	 */
	@Override
	protected String getSelectStatement() {
		return DataBaseSQLStrings.SQL_MODEL_FOR_QUESTION_SET_OF_SUITE_SELECT;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DBObjectSet#prepareSelectStatement(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareSelectStatement(PreparedStatement statement) {
		try {
			statement.setInt(1, question.getId());
		} catch (SQLException ex) {
			ex.printStackTrace();
		}
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DBObjectSet#createThisObject(java.sql.ResultSet)
	 */
	@Override
	protected Object createThisObject(ResultSet resultSet) throws SQLException {
		ModelForQuestion q=null;
		int id = resultSet.getInt(1);
		if (db.getModelForQuestionCache().getDBObject(id) != null) {
			q = db.getModelForQuestionCache().getDBObject(id);
		} else {
			q = new ModelForQuestion(id, db, question);
		}
		return q;
	}
}
