package ch.unizh.ifi.adora.empiricism.model;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * Represents a set of a specific type of multiple choice answer database objects
 * (see {@link ch.unizh.ifi.adora.empiricism.model.QuestionMcItem}).
 *
 * @author		       smeier
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             May 29, 2006 SM First Version,
 *                      May 28, 2006 SM Comments
 *
 * @version             $Date: 2007-01-21 13:14:36 +0100 (Sun, 21 Jan 2007) $, $Author: smeier $, $Revision: 2385 $
 */
public class QuestionMcItemSet extends DBObjectSet {

		/**
	 * The serial id for the serializing mechanism.
	 */
	private static final long serialVersionUID = 5643372849262217739L;

	/**
	 * The question to which this multiple choice answer set belongs.
	 */
	private Question question;

	/**
	 * Creates a new multiple choice answer set for a given question.
	 *
	 * @pre       true
	 * @post      true
	 *
	 * @param question The question for which the multiple choice answers should be retrieved.
	 * @param db The database to in which the multiple choice questions are stored.
	 */
	protected QuestionMcItemSet(Question question, Database db) {
		super(db);
		this.question = question;

		refreshObjects();
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DBObjectSet#getSelectStatement()
	 */
	@Override
	protected String getSelectStatement() {
		return DataBaseSQLStrings.SQL_MC_ITEM_OF_QUESTION_SELECT;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DBObjectSet#prepareSelectStatement(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareSelectStatement(PreparedStatement statement) {
		try {
			statement.setInt(1, question.getId());
		} catch (SQLException ex) {
			ex.printStackTrace();
		}
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DBObjectSet#createThisObject(java.sql.ResultSet)
	 */
	@Override
	protected Object createThisObject(ResultSet resultSet) throws SQLException {
		QuestionMcItem ma=null;
		int id = resultSet.getInt(1);
		if (db.getQuestionMcItemCache().getDBObject(id) != null) {
			ma = db.getQuestionMcItemCache().getDBObject(id);
		} else {
			ma = new QuestionMcItem(id, db);
		}
		return ma;
	}
}