package ch.unizh.ifi.adora.empiricism.model;

/**
 * Stores all types of topic answers. These types are used to determine  
 * the datatype of an answers.
 *  
 * @author		        smeier
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             30.06.2006 SM First Version
 * @version             $Date: 2006-06-30 12:15:37 +0200 (Fri, 30 Jun 2006) $, $Author: vigan $, $Revision: 1752 $
 */
public interface TopicDataTypes {
	/**
	 * Describes no question type. 
	 */
	public static final int T_NO     = 0;
	
	/**
	 * Describes a question which requires a float compatible answer.
	 */
	public static final int T_FLOAT  = 1;
	
	/**
	 * Describes a question which requires a integer compatible answer.
	 */
	public static final int T_INT    = 2;
	
	/**
	 * Describes a question which requires a string compativle answer.
	 */
	public static final int T_STRING = 3;

	/**
	 * Describes a question which requires a multiple choice answer with one possible answer, 
	 * ie. for the question with this type there are {@link ch.unizh.ifi.adora.empiricism.model.QuestionMcItem} 
	 * objects available, describing the multiple choice answer in detail.
	 */
	public static final int T_MC     = 4;

	/**
	 * Describes a question which requires a multiple choice answer with one or more possible answers, 
	 * ie. for the question with this type there are {@link ch.unizh.ifi.adora.empiricism.model.QuestionMcItem} 
	 * objects available, describing the multiple choice answer in detail.
	 */
	public static final int T_OPTION = 5;
}
