package ch.unizh.ifi.adora.empiricism.model;

import java.sql.PreparedStatement;
import java.sql.SQLException;

/**
 * Stores all TestSuites which are unanswered by a given user.
 *
 * @author		        Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             13.06.2006 SM First Version
 * @version             $Date: 2007-01-21 13:14:36 +0100 (Sun, 21 Jan 2007) $, $Author: smeier $, $Revision: 2385 $
 */
public class UnansweredTestSuiteSet extends TestSuiteSet{


	/**
	 * Serial id.
	 */
	private static final long serialVersionUID = 8793909823731381395L;

	/**
	 * Stores the user, for whom all unanswered TestSuites are Stores.
	 */
	User user;

	/**
	 * Initialises the UnansweredTestSuiteSet.
	 *
	 * @param db The database from which the TestSuits are gained.
	 * @param user user for whom all unanswered TestSuites are Stores.
	 */
	public UnansweredTestSuiteSet(Database db, User user) {
		super(db);
		this.user = user;
		this.refreshObjects();
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DBObjectSet#getSelectStatement()
	 */
	@Override
	protected String getSelectStatement() {
		return DataBaseSQLStrings.SQL_GET_ALL_NON_ANSWERED_TEST_SUITES;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DBObjectSet#prepareSelectStatement(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareSelectStatement(PreparedStatement statement) {
		try {
			statement.setInt(1, user.getID());
		} catch (SQLException ex) {
			ex.printStackTrace();
		}
	}
}
