package ch.unizh.ifi.adora.empiricism.model;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * Instances of this class represent a set of user info multiple choice items that are
 * associated with a given user info object.
 *
 * @author		       smeier
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             May 29, 2006 SM First Version
 *                      May 31, 2006 SM Comments.
 * @version             $Date: 2007-01-21 13:14:36 +0100 (Sun, 21 Jan 2007) $, $Author: smeier $, $Revision: 2385 $
 */
public class UserInfoMcItemSet extends DBObjectSet {

	/**
	 * The serial id for the identification of the object version when using the serialization
	 * mechanism.
	 */
	private static final long serialVersionUID = -7203032435387698497L;

	/**
	 * The user info object to which the multiple choice items in this set are associated.
	 */
	private UserInfo userInfo;

	/**
	 * This method creates a new set of user (already existing) info multiple choice
	 * items that are associated with a given user info.
	 *
	 * @param userInfo The user info for which the associated multiple choice items are
	 *        returned in this set.
	 * @param db The database which holds the set of multiple choice items.
	 */
	protected UserInfoMcItemSet(UserInfo userInfo, Database db) {
		super(db);
		this.userInfo = userInfo;
		this.refreshObjects();
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DBObjectSet#getSelectStatement()
	 */
	@Override
	protected String getSelectStatement() {
		return DataBaseSQLStrings.SQL_USER_INFO_MC_ITEM_OF_USER_INFO_SELECT;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DBObjectSet#prepareSelectStatement(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareSelectStatement(PreparedStatement statement) {
		try {
			statement.setInt(1, userInfo.getId());
		} catch (SQLException ex) {
			ex.printStackTrace();
		}
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DBObjectSet#createThisObject(java.sql.ResultSet)
	 */
	@Override
	protected Object createThisObject(ResultSet resultSet) throws SQLException {
		UserInfoMcItem u;
		int id = resultSet.getInt(1);
		if (db.getUserInfoMcItemCache().getDBObject(id) != null) {
			u = db.getUserInfoMcItemCache().getDBObject(id);
		} else {
			u = new UserInfoMcItem(id, db);
		}
		return u;
	}
}