package ch.unizh.ifi.adora.empiricism.model;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * Encapsulates a set of user info objects that are associated with a given
 * {@link ch.unizh.ifi.adora.empiricism.model.UserInfo}.
 *
 *
 * @author		       smeier
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             May 29, 2006 SM First Version
 *                      May 31, 2006 SM Comments
 * @version             $Date: 2007-01-21 13:14:36 +0100 (Sun, 21 Jan 2007) $, $Author: smeier $, $Revision: 2385 $
 */
public class UserInfoSet extends DBObjectSet {

	/**
	 * The serial id for the identification of the object version when using the serialization
	 * mechanism.
	 */
	private static final long serialVersionUID = -7203032435387698497L;

	/**
	 * The test suite with which the user info set is associated.
	 */
	private TestSuite suite;

	/**
	 * Creates a set of all user {@link ch.unizh.ifi.adora.empiricism.model.UserInfo} objects
	 * that are associated with a given test.
	 *
	 * @param suite The test suite of which all the associated user info object should
	 *        be contained in the set.
	 * @param db The database in which the user info tupels and the test suite are stored.
	 */
	protected UserInfoSet(TestSuite suite, Database db) {
		super(db);
		this.suite = suite;
		this.refreshObjects();
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DBObjectSet#getSelectStatement()
	 */
	@Override
	protected String getSelectStatement() {
		return DataBaseSQLStrings.SQL_USER_INFO_SET_OF_TEST_SUITE_SELECT;
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DBObjectSet#prepareSelectStatement(java.sql.PreparedStatement)
	 */
	@Override
	protected void prepareSelectStatement(PreparedStatement statement) {
		try {
			statement.setInt(1, suite.getId());
		} catch (SQLException ex) {
			ex.printStackTrace();
		}
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.model.DBObjectSet#createThisObject(java.sql.ResultSet)
	 */
	@Override
	protected Object createThisObject(ResultSet resultSet) throws SQLException {
		UserInfo u;
		int id = resultSet.getInt(1);
		if (db.getUserInfoCache().getDBObject(id) != null) {
			u = db.getUserInfoCache().getDBObject(id);
		} else {
			u = new UserInfo(id, db);
		}
		return u;
	}
}