package ch.unizh.ifi.adora.empiricism.model;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * Represents a database object with a weak identity, i.d. a database object
 * whose primary key is composed from key of other objects.
 *
 * @author		        @author smeier
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             05.06.2006 SM First Version
 * @version             $Date: 2007-01-21 13:14:36 +0100 (Sun, 21 Jan 2007) $, $Author: smeier $, $Revision: 2385 $
 *
 */
public abstract class WeakIdentityDatabaseObject extends DatabaseObject {
	/**
	 * Constructor for the weak identity database object.
	 * @param db The database to which the object belongs to.
	 */
	public WeakIdentityDatabaseObject(Database db) {
		super(db);
	}

	/**
	 * Returns if the database contains already a tuple for a given
	 * set of attribute values that represent the key for a sub class object
	 * of this class.
	 *
	 * @return Returns true, if in the database is already a tuple existing.
	 */
	protected boolean checkIfDatabaseTupleExists() {
		boolean result = false;
		try {
			PreparedStatement statement = db.getConnection().prepareStatement(this.getSelectStatement());
			this.prepareSelectString(statement);
			ResultSet rs = statement.executeQuery();
			if (rs.next()) {
				result = true;
			}
			rs.close();
			statement.close();
		} catch (SQLException ex) {
			ex.printStackTrace();
		}

		return result;
	}

}
