package ch.unizh.ifi.adora.empiricism.model.cache;

import java.util.HashMap;

/**
 * Represents a global storage cache for {@link ch.unizh.ifi.adora.empiricism.model.DatabaseObject}
 * objects. This global storage is used to avoid several objects representing the same 
 * data tuple in the database.
 *      
 * @author		       smeier
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             May 26, 2006 SM First Version
 *                      May 31, 2006 SM Comments
 * @version             $Date: 2006-05-31 12:30:37 +0200 (Wed, 31 May 2006) $, $Author: smeier $, $Revision: 1599 $
 */
public abstract class DataBaseObjectCache  {

	/**
	 * The hash map which stores the database objects.
	 */
	private HashMap map = new HashMap();

	/**
	 * Stores the database object <i>o</i> with a given key <i>key</i>.
	 *  
	 * @param key The key for accessing the object in the storage.
	 * @param o The object which is stored. 
	 */
	public void setDBObject(Object key, Object o) {
		map.put(key, o);
	}
	
	/**
	 * Returns the object which is stored under a given access key.
	 * @param key The key of the object.
	 * @return Returns the object found under the entry with the key <i>key></i>.
	 */
	public Object getDBObject(Object key) {
		return map.get(key);
	}

	/**
	 * Removes the object which is stored under a given key.
	 * If the key value pair does not exist, nothing is removed.
	 * @param key The key under which the object is stored.
	 */
	public void removeDBObject(Object key) {
		map.remove(key);
	}
}
