package ch.unizh.ifi.adora.empiricism.preferences;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;

import org.eclipse.jface.preference.FieldEditor;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.preference.StringFieldEditor;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

import ch.unizh.ifi.adora.empiricism.Activator;

/**
 * Creates the preferences dialog page for the empirical test plugin.
 *      
 * @author		        smeier
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             May 31, 2006 SM First Version
 *                      Oct 13, 2006 SM Implemented the access for the content of an inquery
 *                      from a http server.
 * @version             $Date: 2008-11-28 11:41:23 +0100 (Fri, 28 Nov 2008) $, $Author: vigan $, $Revision: 3907 $
 *
 */
public class EmpiricalTestPreferences extends FieldEditorPreferencePage implements
		IWorkbenchPreferencePage {

	private StringFieldEditor dbDriverName;
	
	private StringFieldEditor dbConnectionString;
	
	private StringFieldEditor dbUser;
	
	private StringFieldEditor dbPassword;
	
	private StringFieldEditor contentDirectory;
	
	private StringFieldEditor contentSqlFile;
	
	private StringFieldEditor commandFilter;
	
	public EmpiricalTestPreferences() {
		super(GRID);
		setPreferenceStore(Activator.getDefault().getPreferenceStore());
		setDescription("ADORA empirical tests");
	}

	public void init(IWorkbench workbench) {

	}

	@Override
	protected void createFieldEditors() {
		dbDriverName = new StringFieldEditor(
				PreferencesConstants.DB_DRIVER_NAME,
				"Database driver name:",
				getFieldEditorParent());
		addField(dbDriverName);
	
		
		dbConnectionString = new StringFieldEditor(
				PreferencesConstants.DB_CONNECTION_PARAM,
				"Database connection string:",
				getFieldEditorParent());
		addField(dbConnectionString);
	
		dbUser = new StringFieldEditor(
				PreferencesConstants.DB_USER_NAME,
				"Database user name:",
				getFieldEditorParent());
		addField(dbUser);
		

		dbPassword = new StringFieldEditor(
				PreferencesConstants.DB_USER_PASSWORD,
				"Database user name password:",
				getFieldEditorParent());
		addField(dbPassword);
		Text t = dbPassword.getTextControl(getFieldEditorParent());
		t.setEchoChar('*');

		this.contentSqlFile = new StringFieldEditor(
				PreferencesConstants.DB_CONTENT_FILE_NAME,
				"Initial content file:",
				getFieldEditorParent());
		addField(contentSqlFile);

		this.contentDirectory = new StringFieldEditor(
				PreferencesConstants.DB_CONTENT_DIRECTORY,
				"Initial content directory:",
				getFieldEditorParent());
		addField(contentDirectory);
		
		this.commandFilter = new StringFieldEditor(
				PreferencesConstants.LOGGED_COMMANDS,
				"Logged Commands:",
				getFieldEditorParent());
		addField(commandFilter);
	}
	
	/**
	 * Validates the entred paramters. 
	 */
	protected void checkState() {
		super.checkState();
		if(!isValid()) {
			return;
		}
		try {
			try {
				Class.forName(this.dbDriverName.getStringValue()).newInstance();
			} catch (Exception ex) {
				setErrorMessage("Plugin is not installed properly! The database connection driver could not be found!");
			}
			Connection conn = DriverManager.getConnection(
					           dbConnectionString.getStringValue(),
					           dbUser.getStringValue(),
					           dbPassword.getStringValue());
			if (conn == null) {
				setErrorMessage("DB Connection string, the user name or the password is/are not valid!");
				setValid(false);
				return;
			} else {
				conn.close();
			}
		} catch (SQLException e) {
			setErrorMessage("DB Connection is not valid: " + e.getMessage() + " / " + e.getErrorCode());
			setValid(false);
			return;
		}
		
		setErrorMessage(null);
		setValid(true);
	}

	/**
	 * Calls the validation
	 */
	public void propertyChange(PropertyChangeEvent evt) {
		super.propertyChange(evt);
		if(evt.getProperty().equals(FieldEditor.VALUE)) {
			if( evt.getSource() == this.dbConnectionString ||
				evt.getSource() == this.dbDriverName ||
				evt.getSource() == this.dbPassword ||
				evt.getSource() == this.dbUser) {
				checkState();
			}

		}
	}	
}
