package ch.unizh.ifi.adora.empiricism.views.wizard.pages;

import org.eclipse.jface.wizard.IWizard;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;

import ch.unizh.ifi.adora.empiricism.wizard.container.EmpiricismWizardContainer;

/**
 * Supplies methods used by any EmpericismPage.
 * 
 * @author Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 12.06.2006 SM First Version
 * @version $Date: 2007-06-04 14:28:18 +0200 (Mon, 04 Jun 2007) $, $Author: schoeni $, $Revision: 2748 $
 */
public abstract class AbstractEmpericismPage extends WizardPage implements
		Listener {

	/**
	 * @param wizard
	 *            Wizard, to whom this page belongs.
	 */
	protected AbstractEmpericismPage(IWizard wizard) {
		super("");

		setWizard(wizard);
	}

	/**
	 * Updates the wizard's buttons.
	 */
	protected void updateWizardButtons() {
		setPageComplete(isPageComplete());
		getWizard().getContainer().updateButtons();
	}

	/**
	 * Checks if the value, contained in the String is a Float.
	 * 
	 * @return true, if value is a float.
	 */
	protected boolean isFloat(String value) {
		try {
			Float.parseFloat((String) value);
		} catch (NumberFormatException exc) {
			return false;
		}

		return true;
	}

	/**
	 * Checks if the value, contained in the String is not empty.
	 * 
	 * @return true, if the String is not empty
	 */
	protected boolean isNotEmpty(String value) {
		return !"".equals(value);
	}

	/**
	 * Checks if the value, contained in the String is an int.
	 * 
	 * @return true, if value is a Number.
	 */
	protected boolean isNumber(String value) {
		try {
			Integer.parseInt((String) value);
		} catch (NumberFormatException exc) {
			return false;
		}

		return true;
	}

	/**
	 * Draws a horizontal line onto the {@link WizardPage}.
	 */
	protected void createLine(Composite parent, int ncol) {
		Label line = new Label(parent, SWT.SEPARATOR | SWT.HORIZONTAL
				| SWT.BOLD);
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = ncol;
		line.setLayoutData(gridData);
	}

	/**
	 * Shows the errormessage on the {@link EmpiricismWizardContainer}.
	 */
	protected void conditionalShowStatusMessage(String message, boolean show) {
		if (show) {
			((EmpiricismWizardContainer) getWizard().getContainer())
					.setErrorMessage(message);
		} else {
			((EmpiricismWizardContainer) getWizard().getContainer())
					.setErrorMessage("");
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.wizard.IWizardPage#canFlipToNextPage()
	 */
	public boolean canFlipToNextPage() {
		return isPageComplete();
	}

	/**
	 * Determines, if one can navigate to the previous page.
	 * 
	 * @return true, if a previous page is accassible.
	 */
	public abstract boolean canFlipToPreviousPage();
}