package ch.unizh.ifi.adora.empiricism.views.wizard.pages.loginpages;

import org.eclipse.jface.wizard.IWizard;
import org.eclipse.jface.wizard.IWizardPage;

import ch.unizh.ifi.adora.empiricism.model.Database;
import ch.unizh.ifi.adora.empiricism.model.User;
import ch.unizh.ifi.adora.empiricism.views.wizard.pagemanager.PageManager;
import ch.unizh.ifi.adora.empiricism.views.wizard.pages.textpages.TestSuiteFinishPage;
import ch.unizh.ifi.adora.empiricism.views.wizard.pages.textpages.TestSuiteQuestionStartPage;
import ch.unizh.ifi.adora.empiricism.views.wizard.pages.textpages.TestSuiteStartPage;
import ch.unizh.ifi.adora.empiricism.views.wizard.pages.topicpages.multiplechoice.MultiChoiceUserInfoPage;
import ch.unizh.ifi.adora.empiricism.views.wizard.pages.topicpages.textual.UserInfoPage;
import ch.unizh.ifi.adora.empiricism.views.wizard.uimessages.UIMessages;

/**
 * Page which gets displayed if the user is an existing user. He is prompted to
 * enter his user id.
 * 
 * @author Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 12.06.2006 SM First Version
 * @version $Date: 2007-06-04 14:28:18 +0200 (Mon, 04 Jun 2007) $, $Author: schoeni $, $Revision: 2748 $
 */
public class ExistingUserPage extends AbstractLoginPage {

	/**
	 * Determins if a user exists in the db; This variable is used to display a
	 * "wrong id" message.
	 */
	private boolean userExists = false;

	/**
	 * Constructs a new ExistingUserPage with a label "ID".
	 * 
	 * @param wizard
	 *            Wizard, which contains this Page.
	 * @param wrongID
	 *            Used to determine if this the ExistingUserPage is displayed,
	 *            because the user entered a wrong id.
	 */
	public ExistingUserPage(IWizard wizard, boolean wrongID) {
		super("ID", wizard);
		conditionalShowStatusMessage(UIMessages.WRONG_USER_ID_MSG, wrongID);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.wizard.IWizardPage#getNextPage()
	 */
	public IWizardPage getNextPage() {
		conditionalShowStatusMessage(UIMessages.NOT_NEG_ERR_MSG,
				!isNumber(getTextField().getText()));

		boolean testSuiteExists = saveDataToModel();

		if (!userExists)
			return new ExistingUserPage(getWizard(), true);

		PageManager pm = getPageManager();
		
		boolean questionExist = pm.getUser().getLeastUsedTestSuit()
				.getAllQuestions().size() > 0;

		if (!testSuiteExists || (!pm.hasNextPage() && !questionExist)) {
			return new TestSuiteFinishPage(UIMessages.ALL_SUITES_ANSWERED_MSG,
					getWizard(), pm.getSuite());
		}

		if (!(pm.getSuite().getDescriptionStartUserInfo() == null || "".equals(pm
				.getSuite().getDescriptionStartUserInfo()))
				&& pm.hasNextPage())
			return new TestSuiteStartPage(pm, getWizard());

		if (pm.isNextMultipleChoice())
			return new MultiChoiceUserInfoPage(pm, getWizard());

		if (pm.hasNextPage())
			return new UserInfoPage(pm, getWizard());

		return new TestSuiteQuestionStartPage(pm.getSuite()
				.getDescriptionStartQuestion(), pm.getUser(), getWizard());
	}

	/**
	 * Creates a new user, chooses a testsuite to answer with the corresponding
	 * UserInfos.
	 * 
	 * @return true if a testSuite exists
	 */
	protected boolean saveDataToModel() {
		boolean testSuiteExists = true;
		User user = new User(Integer.parseInt(getTextField().getText()),
				new Database());
		if (user.doesTupelExists()) {
			testSuiteExists = createPageManager(user);
			userExists = true;
		}
		return testSuiteExists;
	}

	@Override
	public boolean canFlipToPreviousPage() {
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.wizard.IWizardPage#getPreviousPage()
	 */
	public IWizardPage getPreviousPage() {
		return new ReceptionPage(this.getWizard());
	}

	@Override
	protected String getDescriptionText() {
		return UIMessages.ENTER_ID_MSG;
	}
}