package ch.unizh.ifi.adora.empiricism.views.wizard.pages.loginpages;

import org.eclipse.jface.wizard.IWizard;
import org.eclipse.jface.wizard.IWizardPage;

import ch.unizh.ifi.adora.empiricism.model.Database;
import ch.unizh.ifi.adora.empiricism.model.User;
import ch.unizh.ifi.adora.empiricism.views.wizard.pages.textpages.TestSuiteFinishPage;
import ch.unizh.ifi.adora.empiricism.views.wizard.pages.textpages.UserCreatedPage;
import ch.unizh.ifi.adora.empiricism.views.wizard.uimessages.UIMessages;

/**
 * Page which gets displayed if a non-existing user registers himself. He is
 * prompted to enter his age.
 * 
 * @author Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 19.06.2006 SM First Version
 * @version $Date: 2007-06-04 14:28:18 +0200 (Mon, 04 Jun 2007) $, $Author: schoeni $, $Revision: 2748 $
 */
public class NewUserPage extends AbstractLoginPage {
	/**
	 * Creates a NewUserPage with a label "Age".
	 * 
	 * @param wizard
	 *            izard, which contains this Page.
	 */
	public NewUserPage(IWizard wizard) {
		super("Age", wizard);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.wizard.IWizardPage#getNextPage()
	 */
	public IWizardPage getNextPage() {

		boolean testSuiteExists;

		testSuiteExists = saveDataToModel();

		if (!testSuiteExists) {
			return new TestSuiteFinishPage(UIMessages.NO_SUITES_EXIST_MSG,
					getWizard(), this.getPageManager().getSuite());
		}

		conditionalShowStatusMessage(UIMessages.NOT_NEG_ERR_MSG,
				!isNumber(getTextField().getText()));

		return new UserCreatedPage(this.getPageManager(), getWizard());
	}

	/**
	 * Creates a new user and choses a testsuite to answer and the corresponding
	 * UserInfos.
	 * 
	 * @return true if a testSuite exists
	 */
	protected boolean saveDataToModel() {
		User user = new User(new Database());
		user.setAge(Integer.parseInt(getTextField().getText()));
		user.writeChangedData();

		return createPageManager(user);
	}

	@Override
	public boolean canFlipToPreviousPage() {
		return true;
	}

	@Override
	protected String getDescriptionText() {
		return UIMessages.ENTER_AGE_MSG;
	}
}