package ch.unizh.ifi.adora.empiricism.views.wizard.pages.loginpages;

import org.eclipse.jface.dialogs.IDialogPage;
import org.eclipse.jface.wizard.IWizard;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;

import ch.unizh.ifi.adora.empiricism.views.wizard.pages.AbstractEmpericismPage;
import ch.unizh.ifi.adora.empiricism.views.wizard.uimessages.UIMessages;

/**
 * Start page of the wizard. Here a user can choose, if he is an existing user
 * or if he wants to create a new account.
 * 
 * @author Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 12.06.2006 SM First Version
 * @version $Date: 2007-06-04 14:28:18 +0200 (Mon, 04 Jun 2007) $, $Author: schoeni $, $Revision: 2748 $
 */
public class ReceptionPage extends AbstractEmpericismPage implements Listener {

	/**
	 * New-User Button.
	 */
	private Button newButton;

	/**
	 * Log-in Button.
	 */
	private Button existingButton;

	/**
	 * ReceptionPage constructor.
	 * 
	 * @param wizard
	 *            Wizard, which contains this Page.
	 */
	public ReceptionPage(IWizard wizard) {
		super(wizard);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.wizard.IWizardPage#getNextPage()
	 */
	public IWizardPage getNextPage() {
		if (newButton.getSelection()) {
			return new NewUserPage(getWizard());
		}

		return new ExistingUserPage(getWizard(), false);
	}

	/**
	 * @see IDialogPage#createControl(Composite)
	 */
	public void createControl(Composite parent) {
		GridData gd;
		Composite composite = new Composite(parent, SWT.NULL);

		// create the desired layout for this wizard page
		GridLayout gl = new GridLayout();
		int ncol = 2;
		gl.numColumns = ncol;
		composite.setLayout(gl);

		gd = new GridData();
		gd.horizontalAlignment = GridData.HORIZONTAL_ALIGN_BEGINNING;
		gd.grabExcessHorizontalSpace = true;
		gd.horizontalSpan = ncol;

		// Date of travel
		Label lbl = new Label(composite, SWT.WRAP | SWT.LEFT);
		lbl.setText(UIMessages.LOGIN_CREATENEW_MSG);
		lbl.setLayoutData(gd);

		createLine(composite, ncol);

		newButton = new Button(composite, SWT.RADIO);
		newButton.setText(UIMessages.NEW_USR_BUTTON_MSG);

		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = ncol;

		newButton.setLayoutData(gd);
		newButton.setSelection(true);

		existingButton = new Button(composite, SWT.RADIO);
		existingButton.setText(UIMessages.EXISTING_USR_BUTTON_MSG);

		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = ncol;

		existingButton.setLayoutData(gd);

		// set the composite as the control for this page
		setControl(composite);
		addListeners();
	}

	/**
	 * Adds a listener to the {@link Button}. This is used to check if the page
	 * is completed.
	 */
	private void addListeners() {
		newButton.addListener(SWT.Selection, this);
		existingButton.addListener(SWT.Selection, this);
	}

	/**
	 * Process the events: when the user has entered all information the wizard
	 * can be finished
	 */
	public void handleEvent(Event e) {
		setPageComplete(isPageComplete());
		getWizard().getContainer().updateButtons();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see ch.unizh.ifi.adora.empiricism.views.wizardpages.AbstractEmpericismPage#isBackAllowed()
	 */
	public boolean canFlipToPreviousPage() {
		return false;
	}
}