package ch.unizh.ifi.adora.empiricism.views.wizard.pages.textpages;

import org.eclipse.jface.dialogs.IDialogPage;
import org.eclipse.jface.wizard.IWizard;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;

import ch.unizh.ifi.adora.empiricism.views.wizard.pages.AbstractEmpericismPage;

/**
 * Supplies methods used by all concrete Text Pages.
 * 
 * @author Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 15.06.2006 SM First Version
 * @version $Date: 2007-06-04 14:28:18 +0200 (Mon, 04 Jun 2007) $, $Author: schoeni $, $Revision: 2748 $
 */
public abstract class AbstractTextPage extends AbstractEmpericismPage {

	/**
	 * Text, which will be displayed in the concrete Textpage.
	 */
	private String labelText;

	/**
	 * AbstractTextPage constructor.
	 * 
	 * @param labelText
	 *            Text, which will be displayed in the concrete Textpage.
	 * @param wizard
	 *            Wizard, which contains this Page.
	 */
	public AbstractTextPage(String labelText, IWizard wizard) {
		super(wizard);

		this.labelText = labelText;
	}

	/**
	 * Appends a String to to label text. Used to show User ID after creation
	 * 
	 * @param text
	 *            String which gets appended.
	 */
	protected void appendString(String text) {
		labelText += text;
	}

	/**
	 * @see IDialogPage#createControl(Composite)
	 */
	public void createControl(Composite parent) {
		GridData gd;
		Composite composite = new Composite(parent, SWT.NULL);

		// create the desired layout for this wizard page
		GridLayout gl = new GridLayout();
		composite.setLayout(gl);

		gd = new GridData();
		gd.horizontalAlignment = GridData.FILL;
		gd.grabExcessHorizontalSpace = true;
		gd.grabExcessVerticalSpace = true;

		Label lbl = new Label(composite, SWT.WRAP | SWT.LEFT);
		lbl.setText(labelText);
		lbl.setLayoutData(gd);

		setControl(composite);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.swt.widgets.Listener#handleEvent(org.eclipse.swt.widgets.Event)
	 */
	public void handleEvent(Event event) {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.wizard.IWizardPage#canFlipToNextPage()
	 */
	public abstract boolean canFlipToNextPage();

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.wizard.IWizardPage#getNextPage()
	 */
	public abstract IWizardPage getNextPage();

	/*
	 * Sets the completed field on the wizard class when all the information is
	 * entered and the wizard can be completed
	 */
	public boolean isPageComplete() {
		return true;
	}
}