package ch.unizh.ifi.adora.empiricism.views.wizard.pages.textpages;

import org.eclipse.jface.wizard.IWizard;
import org.eclipse.jface.wizard.IWizardPage;

import ch.unizh.ifi.adora.empiricism.model.Question;
import ch.unizh.ifi.adora.empiricism.views.wizard.pagemanager.PageManager;
import ch.unizh.ifi.adora.empiricism.views.wizard.pages.topicpages.multiplechoice.MultiChoiceQuestionPage;
import ch.unizh.ifi.adora.empiricism.views.wizard.pages.topicpages.textual.QuestionPage;

/**
 * Page, which is displayed before a question, if a question description is
 * stored in the Question table.
 * 
 * @author Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 19.06.2006 SM First Version
 * @version $Date: 2007-06-04 14:28:18 +0200 (Mon, 04 Jun 2007) $, $Author: schoeni $, $Revision: 2748 $
 */
public class QuestionInterPage extends AbstractTextPage {

	/**
	 * {@link PageManager}
	 */
	private PageManager pm;

	/**
	 * QuestionInterPage Constructor.
	 * 
	 * @param pm
	 *            {@link PageManager} for this QuestionInterPage.
	 * @param wizard
	 *            Wizard, which contains this Page.
	 */
	public QuestionInterPage(PageManager pm, IWizard wizard) {
		super(((Question) pm.peekNextTopic()).getDescription(), wizard);
		this.pm = pm;
	}

	@Override
	public boolean canFlipToNextPage() {
		return true;
	}

	@Override
	public IWizardPage getNextPage() {
		if (pm.isNextMultipleChoice())
			return new MultiChoiceQuestionPage(pm, getWizard());

		return new QuestionPage(pm, getWizard());
	}

	@Override
	public boolean canFlipToPreviousPage() {
		return false;
	}
}