package ch.unizh.ifi.adora.empiricism.views.wizard.pages.textpages;

import org.eclipse.jface.wizard.IWizard;
import org.eclipse.jface.wizard.IWizardPage;

import ch.unizh.ifi.adora.empiricism.model.TestSuite;
import ch.unizh.ifi.adora.empiricism.views.wizard.uimessages.UIMessages;

/**
 * Page, which is displayed after the last question. If a question description
 * is stored in the TestSuite table, the description is displayed, else a
 * default message will appear ({@link UIMessages#FINISH_MSG}).
 * 
 * @author Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 14.06.2006 SM First Version
 * @version $Date: 2007-06-04 14:28:18 +0200 (Mon, 04 Jun 2007) $, $Author: schoeni $, $Revision: 2748 $
 */
public class TestSuiteFinishPage extends AbstractTextPage {

	/**
	 * Displayes the default finished messages.
	 * 
	 * @param wizard
	 *            Wizard, which contains this Page.
	 * @param suite
	 */
	public TestSuiteFinishPage(String msg, IWizard wizard, TestSuite suite) {
		super(msg == null || msg.equals("") ? UIMessages.FINISH_MSG : msg,
				wizard);
		suite.writeIncrementNrOfExeTest();
	}

	/**
	 * Displayes the finished messages obtained from the TestSuite. Furthermore,
	 * the TestSuite's nr of executed test field gets incremented.
	 * 
	 * @param wizard
	 *            Wizard, which contains this Page.
	 * @param suite
	 *            Suites which contains the finished text, and which nr of
	 *            executed test field gets incremented.
	 */
	public TestSuiteFinishPage(IWizard wizard, TestSuite suite) {
		this(suite.getDescriptionEndSuite(), wizard, suite);
	}

	@Override
	public boolean canFlipToNextPage() {
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see ch.unizh.ifi.adora.empiricism.views.wizardpages.AbstractEmpericismPage#isBackAllowed()
	 */
	public boolean canFlipToPreviousPage() {
		return false;
	}

	@Override
	public IWizardPage getNextPage() {
		return null;
	}
}