package ch.unizh.ifi.adora.empiricism.views.wizard.pages.textpages;

import org.eclipse.jface.wizard.IWizard;
import org.eclipse.jface.wizard.IWizardPage;

import ch.unizh.ifi.adora.empiricism.model.Question;
import ch.unizh.ifi.adora.empiricism.model.TestSuite;
import ch.unizh.ifi.adora.empiricism.model.User;
import ch.unizh.ifi.adora.empiricism.views.wizard.pagemanager.PageManager;
import ch.unizh.ifi.adora.empiricism.views.wizard.pages.topicpages.multiplechoice.MultiChoiceQuestionPage;
import ch.unizh.ifi.adora.empiricism.views.wizard.pages.topicpages.textual.QuestionPage;
import ch.unizh.ifi.adora.empiricism.views.wizard.uimessages.UIMessages;

/**
 * Page, shown before the first question appears. It instructs the user.
 * 
 * @author Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 14.06.2006 SM First Version
 * @version $Date: 2007-06-04 14:28:18 +0200 (Mon, 04 Jun 2007) $, $Author: schoeni $, $Revision: 2748 $
 */
public class TestSuiteQuestionStartPage extends AbstractTextPage {

	/**
	 * User, who answers the questions.
	 */
	private User user;

	/**
	 * Used, because the next page is a Question page.
	 */
	private PageManager pm;

	/**
	 * Constructs a new TestSuiteQuestionStartPage.
	 * 
	 * @param questionStartDesciption
	 *            Text to be displayed
	 * @param User,
	 *            who answers the questions.
	 * @param wizard
	 *            Wizard, which contains this Page.
	 */
	public TestSuiteQuestionStartPage(String questionStartDesciption,
			User user, IWizard wizard) {
		super(
				questionStartDesciption == null
						|| questionStartDesciption.equals("") ? UIMessages.QUESTION_START_MSG
						: questionStartDesciption, wizard);
		this.user = user;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.wizard.IWizardPage#getNextPage()
	 */
	public IWizardPage getNextPage() {
		saveDataToModel();

		if (!pm.hasNextPage())
			return new TestSuiteFinishPage(getWizard(), pm.getSuite());

		if (!((((Question) pm.peekNextTopic()).getDescription() == null) || ((Question) pm
				.peekNextTopic()).getDescription().equals(""))) {
			return new QuestionInterPage(pm, getWizard());
		}

		if (pm.isNextMultipleChoice())
			return new MultiChoiceQuestionPage(pm, getWizard());

		return new QuestionPage(pm, getWizard());
	}

	/**
	 * Associates a TestSuite to the user. The associated TestSuite is
	 * determined by the number of executed test field of the TestSuite.
	 * Furthermore, the PageManager gets initiated.
	 */
	private void saveDataToModel() {
		// which TestSuite to chose
		TestSuite ts = user.getLeastUsedTestSuit();

		if (ts == null) {
			// show label all test made
			conditionalShowStatusMessage(UIMessages.ALL_SUITES_ANSWERED_MSG,
					ts == null);
			return;
		}

		pm = new PageManager(ts.getAllQuestions(), user, ts);
	}

	@Override
	public boolean canFlipToNextPage() {
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see ch.unizh.ifi.adora.empiricism.views.wizardpages.AbstractEmpericismPage#isBackAllowed()
	 */
	public boolean canFlipToPreviousPage() {
		return false;
	}
}