package ch.unizh.ifi.adora.empiricism.views.wizard.pages.textpages;

import org.eclipse.jface.wizard.IWizard;
import org.eclipse.jface.wizard.IWizardPage;

import ch.unizh.ifi.adora.empiricism.views.wizard.pagemanager.PageManager;
import ch.unizh.ifi.adora.empiricism.views.wizard.pages.topicpages.multiplechoice.MultiChoiceUserInfoPage;
import ch.unizh.ifi.adora.empiricism.views.wizard.pages.topicpages.textual.UserInfoPage;

/**
 * Page, which is displayed before the first
 * {@link ch.unizh.ifi.adora.empiricism.model.UserInfo} is been asked, if a
 * start description is stored in the TestSuite Table.
 * 
 * @author Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 19.06.2006 SM First Version
 * @version $Date: 2007-06-04 14:28:18 +0200 (Mon, 04 Jun 2007) $, $Author: schoeni $, $Revision: 2748 $
 */
public class TestSuiteStartPage extends AbstractTextPage {

	/**
	 * PageManager, used to get the TestSuite
	 */
	private PageManager pm;

	/**
	 * Constructs a new TestSuiteStartPage.
	 * 
	 * @param pm
	 *            PageManager, used to get the TestSuite.
	 * @param wizard
	 *            Wizard, which contains this Page.
	 */
	public TestSuiteStartPage(PageManager pm, IWizard wizard) {
		super(pm.getSuite().getDescriptionStartUserInfo(), wizard);

		this.pm = pm;
	}

	@Override
	public boolean canFlipToNextPage() {
		return true;
	}

	@Override
	public IWizardPage getNextPage() {
		if (!pm.hasNextPage())
			return new TestSuiteQuestionStartPage(pm.getSuite()
					.getDescriptionStartQuestion(), pm.getUser(), getWizard());

		if (pm.isNextMultipleChoice())
			return new MultiChoiceUserInfoPage(pm, getWizard());

		return new UserInfoPage(pm, getWizard());
	}

	@Override
	public boolean canFlipToPreviousPage() {
		return true;
	}
}