package ch.unizh.ifi.adora.empiricism.views.wizard.pages.textpages;

import org.eclipse.jface.wizard.IWizard;
import org.eclipse.jface.wizard.IWizardPage;

import ch.unizh.ifi.adora.empiricism.views.wizard.pagemanager.PageManager;
import ch.unizh.ifi.adora.empiricism.views.wizard.pages.topicpages.multiplechoice.MultiChoiceUserInfoPage;
import ch.unizh.ifi.adora.empiricism.views.wizard.pages.topicpages.textual.UserInfoPage;
import ch.unizh.ifi.adora.empiricism.views.wizard.uimessages.UIMessages;

/**
 * Page, displayed after a user is successfully created.
 * 
 * @author Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 16.06.2006 SM First Version
 * @version $Date: 2007-06-04 14:28:18 +0200 (Mon, 04 Jun 2007) $, $Author: schoeni $, $Revision: 2748 $
 */
public class UserCreatedPage extends AbstractTextPage {

	/**
	 * Used to get the user id.
	 */
	private PageManager pm;

	/**
	 * Constructs a new UserCreatedPage.
	 * 
	 * @param pm
	 *            PageManager, used to get the TestSuite.
	 * @param wizard
	 *            Wizard, which contains this Page.
	 */
	public UserCreatedPage(PageManager pm, IWizard wizard) {
		super(UIMessages.REG_FINSIHED_I_MSG, wizard);
		this.pm = pm;
		appendString("\n\n\nID: " + String.valueOf(pm.getUser().getID())
				+ UIMessages.REG_FINSIHED_II_MSG);
	}

	@Override
	public IWizardPage getNextPage() {

		boolean questionExist = pm.getUser().getLeastUsedTestSuit()
				.getAllQuestions().size() > 0;

		if ((!pm.hasNextPage() && !questionExist)) {
			return new TestSuiteFinishPage(UIMessages.ALL_SUITES_ANSWERED_MSG,
					getWizard(), pm.getSuite());
		}

		if (!(pm.getSuite().getDescriptionStartUserInfo() == null || ""
				.equals(pm.getSuite().getDescriptionStartUserInfo())))
			return new TestSuiteStartPage(pm, getWizard());

		if (pm.isNextMultipleChoice())
			return new MultiChoiceUserInfoPage(pm, getWizard());

		if (pm.hasNextPage())
			return new UserInfoPage(pm, getWizard());

		return new TestSuiteQuestionStartPage(pm.getSuite()
				.getDescriptionStartQuestion(), pm.getUser(), getWizard());
	}

	@Override
	public boolean canFlipToNextPage() {
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see ch.unizh.ifi.adora.empiricism.views.wizardpages.AbstractEmpericismPage#isBackAllowed()
	 */
	public boolean canFlipToPreviousPage() {
		return false;
	}
}