package ch.unizh.ifi.adora.empiricism.views.wizard.pages.topicpages;

import org.eclipse.jface.dialogs.IDialogPage;
import org.eclipse.jface.wizard.IWizard;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;

import ch.unizh.ifi.adora.empiricism.model.AbstractTopic;
import ch.unizh.ifi.adora.empiricism.model.User;
import ch.unizh.ifi.adora.empiricism.views.wizard.pagemanager.PageManager;
import ch.unizh.ifi.adora.empiricism.views.wizard.pages.AbstractEmpericismPage;

/**
 * Supplies methods used by all Question- & UserInfo-Pages.
 * 
 * @author Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 13.06.2006 SM First Version
 * @version $Date: 2007-06-04 14:28:18 +0200 (Mon, 04 Jun 2007) $, $Author: schoeni $, $Revision: 2748 $
 */
public abstract class AbstractTopicPage extends AbstractEmpericismPage {

	/**
	 * Shows the question text;
	 */
	protected Label questionLabel;

	/**
	 * Stores the User.
	 */
	protected User user;

	/**
	 * Superconstruct of question and UserInfo
	 */
	protected AbstractTopic topic;

	/**
	 * Stores the PageManager. The PM is used as a model for the page
	 * management.
	 */
	protected PageManager pm;

	/**
	 * In here, the PageManager gets incremented.
	 * 
	 * @param pm
	 *            The PM is used as a model for the page management.
	 * @param wizard
	 *            Wizard, to whom this page belongs.
	 */
	public AbstractTopicPage(PageManager pm, IWizard wizard) {
		super(wizard);

		this.topic = pm.getNextTopic();
		this.user = pm.getUser();
		this.pm = pm;
	}

	/**
	 * @see IDialogPage#createControl(Composite)
	 */
	public void createControl(Composite parent) {
		GridData gd;
		Composite composite = new Composite(parent, SWT.NULL);

		// create the desired layout for this wizard page
		GridLayout gl = new GridLayout();
		int ncol = 2;
		gl.numColumns = ncol;
		composite.setLayout(gl);

		gd = new GridData();
		gd.horizontalAlignment = GridData.HORIZONTAL_ALIGN_BEGINNING;
		gd.grabExcessHorizontalSpace = true;
		gd.horizontalSpan = ncol;

		// Text of question.
		Label lbl = new Label(composite, SWT.WRAP | SWT.LEFT);
		lbl.setText(topic.getQuestion());
		lbl.setLayoutData(gd);

		gd = new GridData();
		gd.horizontalAlignment = GridData.HORIZONTAL_ALIGN_BEGINNING;
		gd.horizontalSpan = 1;

		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 1;

		createLine(composite, ncol);

		// set the composite as the control for this page
		setControl(composite);
		addSpecificControl(composite, gd);

		addListeners();
	}

	/**
	 * Adds specific control elements to the Composite. (Concret: Mulitple
	 * choise buttons or TextField)
	 * 
	 * @param parent
	 *            parent's composite.
	 * @param gd
	 *            GridData fro the Controll element
	 */
	protected abstract void addSpecificControl(Composite parent, GridData gd);

	/**
	 * Adds listeners to SWT components of this Page. These listener are used to
	 * check the entered data immediately after entering them.
	 */
	protected abstract void addListeners();

	/**
	 * Process the events: when the user has entered all information the wizard
	 * can be finished
	 */
	public abstract void handleEvent(Event event);

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.wizard.IWizardPage#getNextPage()
	 */
	public abstract IWizardPage getNextPage();

	/**
	 * Sets the completed field on the wizard class when all the information is
	 * entered and the wizard can be completed
	 */
	public abstract boolean isPageComplete();

	/**
	 * Interacts with the data base. Typique Operation: Write the given answer
	 * into the db.
	 */
	protected abstract void saveDataToModel();

	/**
	 * @return Returns the PageManager.
	 */
	public PageManager getPm() {
		return pm;
	}
}