package ch.unizh.ifi.adora.empiricism.views.wizard.pages.topicpages.multiplechoice;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.eclipse.jface.dialogs.IDialogPage;
import org.eclipse.jface.wizard.IWizard;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;

import ch.unizh.ifi.adora.empiricism.model.McItem;
import ch.unizh.ifi.adora.empiricism.model.TopicDataTypes;
import ch.unizh.ifi.adora.empiricism.views.wizard.pagemanager.PageManager;
import ch.unizh.ifi.adora.empiricism.views.wizard.pages.topicpages.AbstractTopicPage;

/**
 * Supplies methods used by all Question- & UserInfo MulitpleChoisePages.
 * 
 * @author Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 14.06.2006 SM First Version
 * @version $Date: 2007-06-04 14:28:18 +0200 (Mon, 04 Jun 2007) $, $Author: schoeni $, $Revision: 2748 $
 */
public abstract class AbstractMultipleChoicePage extends AbstractTopicPage {
	/**
	 * List containing all mc items.
	 */
	private HashMap<McItem, Button> mcItem_ButtonTable;

	/**
	 * Constructs a new AbstractMulitChoicePage.
	 * 
	 * @param pm
	 *            PageManager, used to get the TestSuite.
	 * @param wizard
	 *            Wizard, which contains this Page.
	 * 
	 */
	protected AbstractMultipleChoicePage(PageManager pm, IWizard wizard) {
		super(pm, wizard);
		mcItem_ButtonTable = new HashMap<McItem, Button>();
	}

	/**
	 * Return the mcItem_ButtonTable. This is used to check if a button is
	 * checked or not.
	 * 
	 * @return List containing all mc items.
	 */
	protected HashMap<McItem, Button> getMcItemTable() {
		return mcItem_ButtonTable;
	}

	/**
	 * @see IDialogPage#createControl(Composite)
	 */
	public void createControl(Composite parent) {
		GridData gd;

		int buttonStyle;

		Composite composite = new Composite(parent, SWT.NULL);
		gd = new GridData(GridData.FILL_BOTH);
		composite.setLayoutData(gd);

		// create the desired layout for this wizard page
		GridLayout gl = new GridLayout();
		int ncol = 2;
		gl.numColumns = ncol;
		composite.setLayout(gl);

		gd = new GridData();
		gd.horizontalAlignment = GridData.HORIZONTAL_ALIGN_BEGINNING;
		gd.grabExcessHorizontalSpace = true;
		gd.horizontalSpan = ncol;

		// Text of question.
		Label lbl = new Label(composite, SWT.WRAP | SWT.LEFT);
		lbl.setText(topic.getQuestion());
		lbl.setLayoutData(gd);

		createLine(composite, ncol);

		Button b;
		McItem currentItem;

		// Decide if the question can have multiple answers.
		if (topic.getAnswerType() == TopicDataTypes.T_MC) {
			buttonStyle = SWT.CHECK;
		} else {
			buttonStyle = SWT.RADIO;
		}

		Iterator<McItem> mcItemsIter = getMcItemsList().iterator();
		while (mcItemsIter.hasNext()) {

			currentItem = mcItemsIter.next();

			b = new Button(composite, buttonStyle | SWT.WRAP);

			gd = new GridData(GridData.FILL_HORIZONTAL);
			gd.horizontalAlignment = GridData.BEGINNING;
			gd.minimumWidth = 20;

			b.setLayoutData(gd);

			gd = new GridData();
			gd.horizontalAlignment = GridData.BEGINNING;
			gd.grabExcessHorizontalSpace = true;

			// Text of question.
			Label buttonlbl = new Label(composite, SWT.WRAP | SWT.LEFT);
			buttonlbl.setText(currentItem.getDescription());
			buttonlbl.setLayoutData(gd);

			b.setSelection(getStoredAnswer(currentItem));

			// put item / button pair into the hashtable
			mcItem_ButtonTable.put(currentItem, b);
		}

		// set the composite as the control for this page
		setControl(composite);
		addListeners();
	}

	/**
	 * Checks if an {@link McItem} was answered before. This if for example the
	 * case, if a user pressed the back button.
	 * 
	 * @param currentItem
	 *            {@link McItem} which is checked.
	 * @return True, if the {@link McItem} has allready been answered.
	 */
	protected abstract boolean getStoredAnswer(McItem currentItem);

	/**
	 * @return List of {@link McItem}, associated with this topic.
	 */
	protected abstract List getMcItemsList();

	@Override
	protected void addSpecificControl(Composite parent, GridData gd) {
		// TODO Auto-generated method stub
	}

	@Override
	protected void addListeners() {
		for (Button b : mcItem_ButtonTable.values())
			b.addListener(SWT.Selection, this);
	}

	@Override
	public void handleEvent(Event event) {
		updateWizardButtons();
	}

	/**
	 * Return true if at least one button is checked.
	 */
	public boolean isPageComplete() {
		for (Button b : mcItem_ButtonTable.values()) {
			if ((b.getStyle() & SWT.CHECK) == SWT.CHECK || b.getSelection()) {
				return true;
			}
		}
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.wizard.IWizardPage#canFlipToNextPage()
	 */
	public boolean canFlipToNextPage() {
		return isPageComplete();
	}
}