package ch.unizh.ifi.adora.empiricism.views.wizard.pages.topicpages.multiplechoice;

import java.sql.Timestamp;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.eclipse.jface.wizard.IWizard;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.swt.widgets.Button;

import ch.unizh.ifi.adora.empiricism.adoraeditor.model.AdoraModelLoader;
import ch.unizh.ifi.adora.empiricism.adoraeditor.preferences.PreferencesSetter;
import ch.unizh.ifi.adora.empiricism.model.AdoraPropertySet;
import ch.unizh.ifi.adora.empiricism.model.Database;
import ch.unizh.ifi.adora.empiricism.model.McItem;
import ch.unizh.ifi.adora.empiricism.model.ModelForQuestion;
import ch.unizh.ifi.adora.empiricism.model.ModelForQuestionSet;
import ch.unizh.ifi.adora.empiricism.model.Question;
import ch.unizh.ifi.adora.empiricism.model.QuestionMcItem;
import ch.unizh.ifi.adora.empiricism.model.QuestionMcItemSet;
import ch.unizh.ifi.adora.empiricism.model.UserAnswersOfMcItems;
import ch.unizh.ifi.adora.empiricism.model.UserAnswersOfQuestion;
import ch.unizh.ifi.adora.empiricism.views.wizard.pagemanager.PageManager;
import ch.unizh.ifi.adora.empiricism.views.wizard.pages.textpages.QuestionInterPage;
import ch.unizh.ifi.adora.empiricism.views.wizard.pages.textpages.TestSuiteFinishPage;
import ch.unizh.ifi.adora.empiricism.views.wizard.pages.topicpages.textual.QuestionPage;

/**
 * This class represents a WizarPage used for a mulitplechoice Question.
 * 
 * @author Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 14.06.2006 SM First Version 21.01.2007 SM Changes for loading more
 *          than one model per question.
 * @version $Date: 2008-12-08 14:59:58 +0100 (Mon, 08 Dec 2008) $, $Author: treinhard $, $Revision: 3951 $
 */
public class MultiChoiceQuestionPage extends AbstractMultipleChoicePage {

	/**
	 * Stores the timeStamp after creating this page. This is used to determin
	 * who long it took the user to answer this question.
	 */
	private long fromTime;

	/**
	 * Stores all McQuestions.
	 */
	private QuestionMcItemSet itemSet;

	/**
	 * Constructs a new MultiChoiceQuestionPage.
	 * 
	 * @param pm
	 *            PageManager, used to get the TestSuite.
	 * @param wizard
	 *            Wizard, which contains this Page.
	 */
	public MultiChoiceQuestionPage(PageManager pm, IWizard wizard) {
		super(pm, wizard);

		itemSet = ((Question) topic).getAllMcAnswer();

		Question q = (Question) pm.getCurrentTopic();
		ModelForQuestionSet mfqs = q.getModelForQuestionSet();
		int count = 0;
		while (count < mfqs.size()) {
			ModelForQuestion mfq = (ModelForQuestion) (mfqs.get(count));
			AdoraModelLoader
					.showModel(mfq.getAdoraModelStream(), mfq.getName());
			count++;
		}

		// Shows the model;

		PreferencesSetter ps = new PreferencesSetter(new AdoraPropertySet(q,
				new Database()));
		ps.setPreferences();

		// store TimeStamp
		fromTime = System.currentTimeMillis();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see ch.unizh.ifi.adora.empiricism.views.wizardpages.topicpages.mulitplechoise.AbstractMulitChoisePage#getMcItemsList()
	 */
	protected List getMcItemsList() {
		return itemSet;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.wizard.IWizardPage#getNextPage()
	 */
	public IWizardPage getNextPage() {
		saveDataToModel();

		if (!pm.hasNextPage()) {
			AdoraModelLoader.hideModel();
			return new TestSuiteFinishPage(getWizard(), pm.getSuite());
		}
		
		Question nextQuestion = (Question)pm.peekNextTopic();
		ModelForQuestionSet mfqs = nextQuestion.getModelForQuestionSet();
		
		boolean hideModels = true;
		if(mfqs.size() > 0 && ((ModelForQuestion)mfqs.get(0)).getAdoraModelStream() == null) {
			hideModels = false;
		}
		
		if(hideModels) {
			AdoraModelLoader.hideModel();
		}

		if (!((((Question) pm.peekNextTopic()).getDescription() == null) || ((Question) pm
				.peekNextTopic()).getDescription().equals(""))) {
			return new QuestionInterPage(pm, getWizard());
		}

		if (pm.isNextMultipleChoice())
			return new MultiChoiceQuestionPage(pm, getWizard());

		return new QuestionPage(pm, getWizard());
	}

	@Override
	protected void saveDataToModel() {

		Database db = new Database();

		UserAnswersOfMcItems mcAnswer = null;

		UserAnswersOfQuestion answer = new UserAnswersOfQuestion(db, user,
				(Question) topic);

		int deltaSec = (int) (System.currentTimeMillis() - fromTime) / 1000;

		Timestamp d = new Timestamp(fromTime);

		answer.setTime(d);
		answer.setUsedTime(deltaSec);
		answer.writeChangedData();

		HashMap<McItem, Button> map = getMcItemTable();
		Iterator<McItem> mcItemIter = map.keySet().iterator();
		QuestionMcItem mcItem;
		while (mcItemIter.hasNext()) {
			mcItem = (QuestionMcItem) mcItemIter.next();

			mcAnswer = new UserAnswersOfMcItems(db, user, mcItem);
			mcAnswer.setCorrect(map.get(mcItem).getSelection());
			// update table
			mcAnswer.writeChangedData();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see ch.unizh.ifi.adora.empiricism.views.wizardpages.AbstractEmpericismPage#isBackAllowed()
	 */
	public boolean canFlipToPreviousPage() {
		return false;
	}

	/**
	 * One can not navigate to previous pages while answering Questions.
	 */
	protected boolean getStoredAnswer(McItem currentItem) {
		return false;
	}
}