package ch.unizh.ifi.adora.empiricism.views.wizard.pages.topicpages.multiplechoice;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.eclipse.jface.wizard.IWizard;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.swt.widgets.Button;

import ch.unizh.ifi.adora.empiricism.model.Database;
import ch.unizh.ifi.adora.empiricism.model.McItem;
import ch.unizh.ifi.adora.empiricism.model.UserAnswersOfUserInfo;
import ch.unizh.ifi.adora.empiricism.model.UserAnswersOfUserInfoMcItems;
import ch.unizh.ifi.adora.empiricism.model.UserInfo;
import ch.unizh.ifi.adora.empiricism.model.UserInfoMcItem;
import ch.unizh.ifi.adora.empiricism.model.UserInfoMcItemSet;
import ch.unizh.ifi.adora.empiricism.views.wizard.pagemanager.PageManager;
import ch.unizh.ifi.adora.empiricism.views.wizard.pages.textpages.TestSuiteFinishPage;
import ch.unizh.ifi.adora.empiricism.views.wizard.pages.textpages.TestSuiteQuestionStartPage;
import ch.unizh.ifi.adora.empiricism.views.wizard.pages.topicpages.AbstractTopicPage;
import ch.unizh.ifi.adora.empiricism.views.wizard.pages.topicpages.textual.UserInfoPage;

/**
 * This class represents a WizarPage used for a mulitplechoise UserInfoQuestion.
 * 
 * @author Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 12.06.2006 SM First Version 21.01.2007 SM Changes for loading more
 *          than one model per question.
 * @version $Date: 2007-06-04 14:28:18 +0200 (Mon, 04 Jun 2007) $, $Author: schoeni $, $Revision: 2748 $
 */
public class MultiChoiceUserInfoPage extends AbstractMultipleChoicePage {

	/**
	 * Stores the {@link UserInfoMcItemSet} associated this this page. It
	 * contains all McItems associated with the question.
	 */
	private UserInfoMcItemSet itemSet;

	/**
	 * Constructs a new MultiChoiceUserInfoPage.
	 * 
	 * @param pm
	 *            PageManager, used to get the TestSuite.
	 * @param wizard
	 *            Wizard, which contains this Page.
	 */
	public MultiChoiceUserInfoPage(PageManager pm, IWizard wizard) {
		super(pm, wizard);

		itemSet = ((UserInfo) topic).getAllUserInfoMcAnswers();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see ch.unizh.ifi.adora.empiricism.views.wizardpages.topicpages.mulitplechoice.AbstractMulitChoicePage#getMcItemsList()
	 */
	protected List getMcItemsList() {
		return itemSet;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.wizard.IWizardPage#getNextPage()
	 */
	public IWizardPage getNextPage() {
		saveDataToModel();

		if (pm.isNextMultipleChoice())
			return new MultiChoiceUserInfoPage(pm, getWizard());
		if (pm.hasNextPage())
			return new UserInfoPage(pm, getWizard());

		boolean questionExist = pm.getUser().getLeastUsedTestSuit()
				.getAllQuestions().size() > 0;

		if (questionExist) {
			return new TestSuiteQuestionStartPage(pm.getSuite()
					.getDescriptionStartQuestion(), pm.getUser(), getWizard());
		}

		return new TestSuiteFinishPage("", getWizard(), pm.getSuite());
	}

	@Override
	protected void saveDataToModel() {
		Database db = new Database();

		UserAnswersOfUserInfoMcItems mcAnswer = null;

		UserAnswersOfUserInfo answer = new UserAnswersOfUserInfo(db, user,
				(UserInfo) topic);
		answer.writeChangedData();

		HashMap<McItem, Button> map = getMcItemTable();
		Iterator<McItem> mcItemIter = map.keySet().iterator();
		UserInfoMcItem mcItem;
		while (mcItemIter.hasNext()) {
			mcItem = (UserInfoMcItem) mcItemIter.next();

			mcAnswer = new UserAnswersOfUserInfoMcItems(db, user, mcItem);
			mcAnswer.setCorrect(map.get(mcItem).getSelection());
			// update table
			mcAnswer.writeChangedData();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see ch.unizh.ifi.adora.empiricism.views.wizardpages.AbstractEmpericismPage#isBackAllowed()
	 */
	public boolean canFlipToPreviousPage() {
		return getPreviousPage() instanceof AbstractTopicPage;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see ch.unizh.ifi.adora.empiricism.views.wizardpages.topicpages.mulitplechoice.AbstractMulitChoicePage#getStoredAnswer(ch.unizh.ifi.adora.empiricism.model.McItem)
	 */
	protected boolean getStoredAnswer(McItem currentItem) {
		UserAnswersOfUserInfoMcItems answer = new UserAnswersOfUserInfoMcItems(
				new Database(), user, (UserInfoMcItem) currentItem);
		return answer.isCorrect();
	}
}