package ch.unizh.ifi.adora.empiricism.views.wizard.pages.topicpages.textual;

import org.eclipse.jface.wizard.IWizard;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Text;

import ch.unizh.ifi.adora.empiricism.model.TopicDataTypes;
import ch.unizh.ifi.adora.empiricism.views.wizard.pagemanager.PageManager;
import ch.unizh.ifi.adora.empiricism.views.wizard.pages.topicpages.AbstractTopicPage;
import ch.unizh.ifi.adora.empiricism.views.wizard.uimessages.UIMessages;

/**
 * Supplies methods used by all textual Question- & UserInfo-Pages.
 * 
 * @author Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 13.06.2006 SM First Version
 * @version $Date: 2007-06-04 14:28:18 +0200 (Mon, 04 Jun 2007) $, $Author: schoeni $, $Revision: 2748 $
 */
public abstract class AbstractTextualTopicPage extends AbstractTopicPage {

	/**
	 * TextField, in which the user has to anser the question.
	 */
	private Text answerText;

	/**
	 * Constructs a new AbstractTextualTopicPage.
	 * 
	 * @param pm
	 *            PageManager, used to get the TestSuite.
	 * @param wizard
	 *            Wizard, which contains this Page.
	 */
	public AbstractTextualTopicPage(PageManager pm, IWizard wizard) {
		super(pm, wizard);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see ch.unizh.ifi.adora.empiricism.views.wizardpages.AbstractTopicPage#addSpecificControl(org.eclipse.swt.widgets.Composite)
	 */
	protected void addSpecificControl(Composite parent, GridData gd) {
		answerText = new Text(parent, SWT.BORDER);
		answerText.setFocus();

		if (!(getStoredAnswer() == null || getStoredAnswer().equals("")))
			answerText.setText(getStoredAnswer());
		answerText.setLayoutData(gd);

		// set the composite as the control for this page
		setControl(parent);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see ch.unizh.ifi.adora.empiricism.views.wizard.pages.topicpages.AbstractTopicPage#addListeners()
	 */
	protected void addListeners() {
		answerText.addListener(SWT.KeyUp, this);
	}

	/**
	 * Used to set the value of a field, if the back-button has been pressed.
	 * 
	 * @return StoredAnswer answer the user given to the last question.
	 */
	protected abstract String getStoredAnswer();

	/*
	 * Process the events: when the user has entered all information the wizard
	 * can be finished
	 */
	public void handleEvent(Event event) {

		conditionalShowStatusMessage(UIMessages.WRONG_DATA_TYPE_MSG,
				!isEnteredDataTypeOk());
		updateWizardButtons();
	}

	/**
	 * Checks if the datatype of the entered data matches the requested answer
	 * data type, stored in the UserInfoAnswer Tuppel
	 * 
	 * @return true if datatype is valid.
	 */
	private boolean isEnteredDataTypeOk() {
		switch (topic.getAnswerType()) {
		case TopicDataTypes.T_INT:
			return isNumber(answerText.getText());
		case TopicDataTypes.T_FLOAT:
			return isFloat(answerText.getText());
		case TopicDataTypes.T_STRING:
			return isNotEmpty(answerText.getText());
		}
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.wizard.IWizardPage#isPageComplete()
	 */
	public boolean isPageComplete() {
		return isEnteredDataTypeOk() && !answerText.getText().equals("");
	}

	/**
	 * @return Returns the answerText.
	 */
	public Text getAnswerText() {
		return answerText;
	}

	/**
	 * @param answerText
	 *            The answerText to set.
	 */
	public void setAnswerText(Text answerText) {
		this.answerText = answerText;
	}

}