package ch.unizh.ifi.adora.empiricism.views.wizard.pages.topicpages.textual;

import java.sql.Timestamp;

import org.eclipse.jface.wizard.IWizard;
import org.eclipse.jface.wizard.IWizardPage;

import ch.unizh.ifi.adora.empiricism.adoraeditor.model.AdoraModelLoader;
import ch.unizh.ifi.adora.empiricism.adoraeditor.preferences.PreferencesSetter;
import ch.unizh.ifi.adora.empiricism.model.AdoraPropertySet;
import ch.unizh.ifi.adora.empiricism.model.Database;
import ch.unizh.ifi.adora.empiricism.model.ModelForQuestion;
import ch.unizh.ifi.adora.empiricism.model.ModelForQuestionSet;
import ch.unizh.ifi.adora.empiricism.model.Question;
import ch.unizh.ifi.adora.empiricism.model.UserAnswersOfQuestion;
import ch.unizh.ifi.adora.empiricism.views.wizard.pagemanager.PageManager;
import ch.unizh.ifi.adora.empiricism.views.wizard.pages.textpages.QuestionInterPage;
import ch.unizh.ifi.adora.empiricism.views.wizard.pages.textpages.TestSuiteFinishPage;
import ch.unizh.ifi.adora.empiricism.views.wizard.pages.topicpages.multiplechoice.MultiChoiceQuestionPage;

/**
 * This class represents a WizarPage used for a textual Question.
 *
 * @author		        Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             12.06.2006 SM First Version
 * 						21.01.2007 SM Changes for loading more than one model per question.
 * @version             $Date: 2008-12-08 14:59:58 +0100 (Mon, 08 Dec 2008) $, $Author: treinhard $, $Revision: 3951 $
 */
public class QuestionPage extends AbstractTextualTopicPage{

	/**
	 * Stores the timeStamp after creating this page.
	 * This is used to determin who long it took the user to answer this question.
	 */
	private long fromTime;

	/**
	 * Constructs a QuestionPage.
	 *
	 * @param pm PageManager, used to get the TestSuite.
	 * @param wizard Wizard, which contains this Page.
     */
	public QuestionPage(PageManager pm, IWizard wizard) {
		super(pm, wizard);

		Question q = (Question)pm.getCurrentTopic();
		//Shows the model;
		ModelForQuestionSet mfqs = q.getModelForQuestionSet();
		int count = 0;
		while (count < mfqs.size()) {
			ModelForQuestion mfq = (ModelForQuestion)(mfqs.get(count));
			AdoraModelLoader.showModel(mfq.getAdoraModelStream(), mfq.getName());
			count++;
		}
		//AdoraModelLoader.showModel(q.getAdoraModelStream(), q.getQuestion());

		PreferencesSetter ps = new PreferencesSetter(new AdoraPropertySet(q, new Database()));
		ps.setPreferences();

		fromTime = System.currentTimeMillis();
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.wizard.IWizardPage#getNextPage()
	 */
	public IWizardPage getNextPage(){
		saveDataToModel();

		if (!pm.hasNextPage()){
			AdoraModelLoader.hideModel();
			return new TestSuiteFinishPage(getWizard(), pm.getSuite());
		}
		
		Question nextQuestion = (Question)pm.peekNextTopic();
		ModelForQuestionSet mfqs = nextQuestion.getModelForQuestionSet();
		
		boolean hideModels = true;
		if(mfqs.size() > 0 && ((ModelForQuestion)mfqs.get(0)).getAdoraModelStream() == null) {
			hideModels = false;
		}
		
		if(hideModels) {
			AdoraModelLoader.hideModel();
		}

		if (! ((((Question)pm.peekNextTopic()).getDescription() == null) ||
				((Question)pm.peekNextTopic()).getDescription().equals(""))){
			return new QuestionInterPage(pm, getWizard());
		}

		if (pm.isNextMultipleChoice())
			return new MultiChoiceQuestionPage(pm, getWizard());

		return new QuestionPage(pm, getWizard());
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.views.wizardpages.TextualTopic#saveDataToModel()
	 */
	protected void saveDataToModel(){

		UserAnswersOfQuestion answer =
			new UserAnswersOfQuestion(new Database(), user, (Question)topic);
		answer.setUserAnswer(getAnswerText().getText());

		int deltaSec = (int)(System.currentTimeMillis() - fromTime)/1000;
		Timestamp d = new Timestamp(fromTime);

		answer.setTime(d);
		answer.setUsedTime(deltaSec);

		//update table
		answer.writeChangedData();
	}

	/**
	 * One can not navigate to previous pages while answering Questions.
	 */
	protected String getStoredAnswer() {
		return "";
	}

	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.empiricism.views.wizardpages.AbstractEmpericismPage#isBackAllowed()
	 */
	public boolean canFlipToPreviousPage() {
		return false;
	}
}