package ch.unizh.ifi.adora.empiricism.views.wizard.pages.topicpages.textual;

import org.eclipse.jface.wizard.IWizard;
import org.eclipse.jface.wizard.IWizardPage;

import ch.unizh.ifi.adora.empiricism.model.Database;
import ch.unizh.ifi.adora.empiricism.model.UserAnswersOfUserInfo;
import ch.unizh.ifi.adora.empiricism.model.UserInfo;
import ch.unizh.ifi.adora.empiricism.views.wizard.pagemanager.PageManager;
import ch.unizh.ifi.adora.empiricism.views.wizard.pages.textpages.TestSuiteFinishPage;
import ch.unizh.ifi.adora.empiricism.views.wizard.pages.textpages.TestSuiteQuestionStartPage;
import ch.unizh.ifi.adora.empiricism.views.wizard.pages.topicpages.AbstractTopicPage;
import ch.unizh.ifi.adora.empiricism.views.wizard.pages.topicpages.multiplechoice.MultiChoiceUserInfoPage;

/**
 * This class represents a WizarPage used for a textual UserInfoQuestion.
 * 
 * @author Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 12.06.2006 SM First Version
 * @version $Date: 2007-06-04 14:28:18 +0200 (Mon, 04 Jun 2007) $, $Author: schoeni $, $Revision: 2748 $
 */
public class UserInfoPage extends AbstractTextualTopicPage {

	/**
	 * Constructs a UserInfoPage.
	 * 
	 * @param pm
	 *            PageManager, used to get the TestSuite.
	 * @param wizard
	 *            Wizard, which contains this Page.
	 */
	public UserInfoPage(PageManager pm, IWizard wizard) {
		super(pm, wizard);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see ch.unizh.ifi.adora.empiricism.views.wizardpages.topicpages.textual.AbstractTextualTopicPage#getStoredAnswer()
	 */
	protected String getStoredAnswer() {
		UserAnswersOfUserInfo answer = new UserAnswersOfUserInfo(
				new Database(), user, (UserInfo) topic);
		return answer.getAnswer();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.wizard.IWizardPage#getNextPage()
	 */
	public IWizardPage getNextPage() {
		saveDataToModel();

		if (pm.isNextMultipleChoice())
			return new MultiChoiceUserInfoPage(pm, getWizard());

		if (pm.hasNextPage())
			return new UserInfoPage(pm, getWizard());

		boolean questionExist = pm.getUser().getLeastUsedTestSuit()
				.getAllQuestions().size() > 0;

		if (questionExist) {
			return new TestSuiteQuestionStartPage(pm.getSuite()
					.getDescriptionStartQuestion(), pm.getUser(), getWizard());
		}

		return new TestSuiteFinishPage("", getWizard(), pm.getSuite());
	}

	/**
	 * Writes the given answer into the db.
	 */
	protected void saveDataToModel() {
		UserAnswersOfUserInfo answer = new UserAnswersOfUserInfo(
				new Database(), user, (UserInfo) topic);
		answer.setAnswer(getAnswerText().getText());

		// update table
		answer.writeChangedData();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see ch.unizh.ifi.adora.empiricism.views.wizardpages.AbstractEmpericismPage#isBackAllowed()
	 */
	public boolean canFlipToPreviousPage() {
		return getPreviousPage() instanceof AbstractTopicPage;
	}
}