package ch.unizh.ifi.adora.empiricism.wizard.container;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.util.Assert;
import org.eclipse.jface.wizard.IWizardContainer;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.BusyIndicator;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;

import ch.unizh.ifi.adora.empiricism.views.wizard.EmpiricismWizard;
import ch.unizh.ifi.adora.empiricism.views.wizard.pages.AbstractEmpericismPage;
import ch.unizh.ifi.adora.empiricism.views.wizard.pages.loginpages.ReceptionPage;
import ch.unizh.ifi.adora.empiricism.views.wizard.pages.topicpages.AbstractTopicPage;
import ch.unizh.ifi.adora.empiricism.wizard.container.layoutmanager.PageContainerFillLayout;

/**
 * This class encapsulates the whole WizardContainer functionality.
 * 
 * @author Ivo Vigan
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 13.06.2006 SM First Version
 * @version $Date: 2007-06-04 14:28:18 +0200 (Mon, 04 Jun 2007) $, $Author: schoeni $, $Revision: 2748 $
 */
public class EmpiricismWizardContainer extends Composite implements
		IWizardContainer {

	/**
	 * Stores if user is moving to a previous page.
	 */
	private boolean isMovingToPreviousPage = false;

	/**
	 * The currently displayed page.
	 */
	private IWizardPage currentPage = null;

	/**
	 * Button, used to navigate forward.
	 */
	private Button nextButton;

	/**
	 * Button, used to finish Wizard.
	 */
	private Button finishButton;

	/**
	 * Button, used to navigate backward.
	 */
	private Button backButton;

	/**
	 * Container, in which the wizard is embedded.
	 */
	private Composite pageContainer;

	/**
	 * Label, on which the errormessages appear.
	 */
	private Label errorMessageLabel;

	/**
	 * Used LayoutManager
	 */
	private PageContainerFillLayout pageContainerLayout = new PageContainerFillLayout(
			5, 5, 300, 800);

	/**
	 * Contains the actual empericism Wizard
	 */
	EmpiricismWizard wizard;

	/**
	 * Instantiates a new EmpiriWizardContainer for a given parent.
	 * 
	 * @param parent
	 *            of this Container
	 */
	public EmpiricismWizardContainer(Composite parent) {
		super(parent, SWT.NULL);
		init(parent);
	}

	/**
	 * Creates the Composite in which all Buttons are situated.
	 * 
	 * @param parentComposite
	 *            where the ButtonBar will be situated.
	 * @return Composite in which all Buttons are situated.
	 */
	protected Composite createButtonBar(Composite parent) {

		Composite composite = new Composite(parent, SWT.NONE);
		// create a layout with spacing and margins appropriate for the font
		// size.
		GridLayout layout = new GridLayout();
		layout.numColumns = 3;
		layout.makeColumnsEqualWidth = true;

		layout.marginWidth = 5;
		layout.marginHeight = 5;
		layout.horizontalSpacing = 5;
		layout.verticalSpacing = 5;

		composite.setLayout(layout);
		GridData data = new GridData();
		composite.setLayoutData(data);
		composite.setFont(parent.getFont());
		// Add the buttons to the button bar.
		createButtonsForButtonBar(composite);
		return composite;
	}

	/**
	 * Creates the Composite in which the message label is situated.
	 * 
	 * @param parentComposite
	 *            where the MessageBar will be situated.
	 * @return Composite in which the message label is situated.
	 */
	protected Composite createMessageBar(Composite parent) {

		GridData gd;
		Composite composite = new Composite(parent, SWT.NULL);

		// create the desired layout for this wizard page
		GridLayout gl = new GridLayout();
		composite.setLayout(gl);

		gd = new GridData();
		gd.horizontalAlignment = GridData.FILL;
		gd.grabExcessHorizontalSpace = true;
		gd.grabExcessVerticalSpace = true;

		errorMessageLabel = new Label(composite, SWT.WRAP | SWT.LEFT);
		errorMessageLabel.setLayoutData(gd);

		composite.setFont(parent.getFont());
		return composite;
	}

	/**
	 * Sets the errormessage
	 * 
	 * @param text
	 *            Contains the error message.
	 */
	public void setErrorMessage(String text) {
		errorMessageLabel.setText(text);
	}

	/**
	 * Instantiates all Buttons and adds them to the ButtonBar.
	 * 
	 * @param parent
	 *            Composite in which the buttons will be situated.
	 */
	private void createButtonsForButtonBar(Composite parent) {

		GridData gd;

		gd = new GridData();
		gd.horizontalAlignment = GridData.BEGINNING;
		gd.horizontalSpan = 1;
		gd.widthHint = 80;

		backButton = new Button(parent, SWT.PUSH);
		backButton.setLayoutData(gd);
		backButton.setSize(70, 20);
		backButton.setText(IDialogConstants.BACK_LABEL);
		backButton.setData(IDialogConstants.BACK_ID);
		addSelectionListener(backButton);

		gd = new GridData();
		gd.horizontalAlignment = GridData.CENTER;
		gd.horizontalSpan = 1;
		gd.widthHint = 80;

		nextButton = new Button(parent, SWT.PUSH);
		nextButton.setLayoutData(gd);
		nextButton.setSize(70, 20);
		nextButton.setText(IDialogConstants.NEXT_LABEL);
		nextButton.setData(IDialogConstants.NEXT_ID);
		addSelectionListener(nextButton);

		gd = new GridData();
		gd.horizontalAlignment = GridData.BEGINNING;
		gd.horizontalSpan = 1;
		gd.widthHint = 80;

		finishButton = new Button(parent, SWT.PUSH);
		finishButton.setLayoutData(gd);
		finishButton.setSize(70, 20);
		finishButton.setText(IDialogConstants.FINISH_LABEL);
		finishButton.setData(IDialogConstants.FINISH_ID);
		addSelectionListener(finishButton);

	}

	/**
	 * Adds a SelectionListener to a given Button.
	 * 
	 * @param b
	 *            Button
	 */
	private void addSelectionListener(Button b) {
		b.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				buttonPressed(((Integer) event.widget.getData()).intValue());
			}
		});
	}

	/**
	 * Initialises all GUI components used to display and navigate throug the
	 * wizard.
	 * 
	 * @param parent
	 *            Composite in which the EmpiricismWizardContainer will be situated.
	 */
	public void init(Composite parent) {

		GridLayout layout = new GridLayout();
		layout.marginHeight = 0;
		layout.marginWidth = 0;
		layout.verticalSpacing = 20;
		layout.numColumns = 1;
		parent.setLayout(layout);
		parent.setLayoutData(new GridData(GridData.END));

		pageContainer = createPageContainer(parent);
		GridData gd = new GridData(GridData.FILL_BOTH);
		gd.verticalAlignment = GridData.VERTICAL_ALIGN_FILL;
		pageContainer.setLayoutData(gd);

		gd = new GridData(GridData.FILL_BOTH);
		gd.minimumHeight = 40;
		Composite bb = createButtonBar(parent);
		bb.setLayoutData(gd);

		gd = new GridData(GridData.FILL_BOTH);
		gd.minimumHeight = 40;

		Composite mb = createMessageBar(parent);
		mb.setLayoutData(gd);

		wizard = new EmpiricismWizard();
		wizard.setContainer(this);
		showStartingPage(wizard.getStartingPage());
	}

	/**
	 * The Back button has been pressed.
	 */
	protected void backPressed() {
		setErrorMessage("");
		IWizardPage page = currentPage.getPreviousPage();

		// Sets the currentPage couter to the right position.
		if (page instanceof AbstractTopicPage) {
			((AbstractTopicPage) page).getPm().getPreviousTopic();
		}

		isMovingToPreviousPage = true;

		// show the page
		showPage(page);
	}

	/**
	 * The Next button has been pressed.
	 */
	protected void nextPressed() {
		setErrorMessage("");
		showPage(currentPage.getNextPage());
	}

	/*
	 * (non-Javadoc) Method declared on Dialog.
	 */
	protected void buttonPressed(int buttonId) {
		switch (buttonId) {
		case IDialogConstants.BACK_ID: {
			backPressed();
			break;
		}
		case IDialogConstants.NEXT_ID: {
			nextPressed();
			break;
		}
		case IDialogConstants.FINISH_ID: {
			finishPressed();
			break;
		}
		}
	}

	/**
	 * The Finish button has been pressed.
	 */
	protected void finishPressed() {
		setErrorMessage("");
		showPage(new ReceptionPage(wizard));
	}

	/*
	 * /* (non-Javadoc) Method declared on IWizardContainer.
	 */
	public void updateButtons() {

		boolean canFlipToNextPage = false;
		boolean canFinish = wizard.canFinish();

		if (backButton != null)
			backButton.setEnabled(((AbstractEmpericismPage) currentPage)
					.canFlipToPreviousPage());

		if (nextButton != null) {
			canFlipToNextPage = currentPage.canFlipToNextPage();
			nextButton.setEnabled(canFlipToNextPage);
		}

		finishButton.setEnabled(canFinish);

		// finish is default unless it is diabled and next is enabled
		if (canFlipToNextPage && !canFinish)
			getShell().setDefaultButton(nextButton);
		else
			getShell().setDefaultButton(finishButton);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.wizard.IWizardContainer#getCurrentPage()
	 */
	public IWizardPage getCurrentPage() {
		return currentPage;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.wizard.IWizardContainer#getShell()
	 */
	public Shell getShell() {
		return PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();
	}

	/**
	 * Shows the Starting Page.
	 * 
	 * @param page
	 *            StartPage
	 */
	public void showStartingPage(IWizardPage page) {
		if (page == null) {
			// something must have happend getting the page
			return;
		}
		// ensure the page control has been created
		if (page.getControl() == null) {
			page.createControl(pageContainer);
			// the page is responsible for ensuring the created control is
			// accessable
			// via getControl.
			Assert.isNotNull(page.getControl());
			// we do not need to update the size since the call
			// to initialize bounds has not been made yet.
		}
		// make the new page visible
		page.setVisible(true);
		// update the view part^s controls
		currentPage = page;
		updateButtons();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.wizard.IWizardContainer#showPage(org.eclipse.jface.wizard.IWizardPage)
	 */
	public void showPage(IWizardPage page) {
		if (page == null || page == currentPage) {
			return;
		}

		page.createControl(pageContainer);

		if (!isMovingToPreviousPage) {
			// remember my previous page.
			page.setPreviousPage(currentPage);
		} else {
			isMovingToPreviousPage = false;
		}

		if (page instanceof AbstractEmpericismPage
				&& !((AbstractEmpericismPage) page).canFlipToPreviousPage()) {
			this.disposeAllPrevious(currentPage);
		}

		// Update for the new page in a busy cursor if possible
		if (pageContainer != null) {
			updateForPage(page);
		} else {
			final IWizardPage finalPage = page;
			BusyIndicator.showWhile(pageContainer.getDisplay(), new Runnable() {
				public void run() {
					updateForPage(finalPage);
				}
			});
		}

		pageContainer.layout();
		updateButtons();
	}

	/**
	 * Disposes the given and all previous pages.
	 * 
	 * @param currentPage
	 */
	private void disposeAllPrevious(IWizardPage currentPage) {
		IWizardPage page = currentPage;

		while (page != null) {
			// System.out.println("disposing of " +
			// page.getClass().getSimpleName());

			page.dispose();

			page = page.getPreviousPage();
		}

		currentPage.setPreviousPage(null);
	}

	/**
	 * Updates the receiver for the new page.
	 * 
	 * @param page
	 *            Page, which will get updated.
	 */
	private void updateForPage(IWizardPage page) {
		// ensure this page belongs to the current wizard
		if (wizard != page.getWizard())
			if (page.getControl() == null) {
				page.createControl(pageContainer);
				Assert.isNotNull(page.getControl());
			}
		// make the new page visible
		IWizardPage oldPage = currentPage;
		currentPage = page;
		currentPage.setVisible(true);
		if (oldPage != null)
			oldPage.setVisible(false);
	}

	/**
	 * Creates a new PageContainer, which will contain the wizard.
	 * 
	 * @param parent
	 *            Composite in which the PageContainer will be situated.
	 * @return PageContainer
	 */
	private Composite createPageContainer(Composite parent) {
		Composite result = new Composite(parent, SWT.NULL);
		result.setLayout(pageContainerLayout);
		return result;
	}

	/**
	 * Not used.
	 */
	public void updateMessage() {
	}

	/**
	 * Not used.
	 */
	public void updateTitleBar() {
	}

	/**
	 * Not used.
	 */
	public void updateWindowTitle() {
	}

	/**
	 * Not used.
	 */
	public void run(boolean fork, boolean cancelable,
			IRunnableWithProgress runnable) throws InvocationTargetException,
			InterruptedException {
	}

	/**
	 * Not used.
	 */
	public Object getSelectedPage() {
		return null;
	}
}