package ch.unizh.ifi.adora.figureexporter.thesis;

import java.io.IOException;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.gef.EditPart;
import org.eclipse.ui.IEditorPart;

import ch.unizh.ifi.adora.figureexporter.FigureExporter;
import ch.unizh.ifi.adora.model.commands.FisheyeZoom;
import ch.unizh.ifi.adora.model.geometry.Rectangle;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;

public class DesiredZoomProperties extends FigureExporter {

	public void testCompactLayout() throws CoreException, IOException {
		
		// Create a simple model
		AdoraModel model = new AdoraModel();
		
		Component a = addComponent(model, new Rectangle(5, 5, 390, 220), "A");
		addComponent(model, new Rectangle(240, 250, 100, 40), "B");
		addComponent(model, new Rectangle(80, 250, 100, 40), "C");
		
		Component d = addComponent(a, new Rectangle(30, 35, 220, 170), "D");
		addComponent(a, new Rectangle(280, 60, 80, 40), "E");
		addComponent(a, new Rectangle(280, 150, 80, 40), "F");
		
		addComponent(d, new Rectangle(30, 50, 60, 90), "G");
		addComponent(d, new Rectangle(130, 50, 60, 90), "H");
		
		// Initialize the editor
		IFile file = storeModel(model);
		IEditorPart editor = openEditor(file);
		
		exportModel(editor, "CompactLayoutInitial.pdf");
		
		EditPart compEditPart = getEditPartByName(editor, "D");	
		ZoomableNodeContainer zoomedNode = (ZoomableNodeContainer)compEditPart.getModel();
		
		FisheyeZoom cmd = new FisheyeZoom();
		cmd.setTarget(zoomedNode);
		cmd.execute();
		
		exportModel(editor, "CompactLayoutZoomed.pdf");
		
		closeEditor(editor);
		
	}
	
	
	public void testStability() throws CoreException, IOException {
		
		// Create a simple model
		AdoraModel model = new AdoraModel();
		
		Component root = addComponent(model, new Rectangle(5, 5, 300, 240), "");
		Component a = addComponent(root, new Rectangle(20, 20, 120, 80), "A");
		Component b = addComponent(root, new Rectangle(180, 40, 100, 160), "B");
		Component c = addComponent(root, new Rectangle(40, 140, 120, 80), "C");
		
		addComponent(a, new Rectangle(30, 35, 60, 35), "D");
		addComponent(b, new Rectangle(20, 35, 60, 45), "E");
		addComponent(b, new Rectangle(20, 100, 60, 45), "F");
		addComponent(c, new Rectangle(30, 35, 60, 35), "G");
		
		// Initialize the editor
		IFile file = storeModel(model);
		IEditorPart editor = openEditor(file);
		
		exportModel(editor, "LayoutStabilityInitial.pdf");
		
		// Zoom node A out
		EditPart compEditPart = getEditPartByName(editor, "A");	
		ZoomableNodeContainer zoomedNode = (ZoomableNodeContainer)compEditPart.getModel();
		FisheyeZoom cmd = new FisheyeZoom();
		cmd.setTarget(zoomedNode);
		cmd.execute();
		
		exportModel(editor, "LayoutStabilityAZoomedOut.pdf");
		
		// Zoom node C out
		compEditPart = getEditPartByName(editor, "C");	
		zoomedNode = (ZoomableNodeContainer)compEditPart.getModel();
		cmd = new FisheyeZoom();
		cmd.setTarget(zoomedNode);
		cmd.execute();
		
		exportModel(editor, "LayoutStabilityCZoomedOut.pdf");
		
		// Zoom node A in
		compEditPart = getEditPartByName(editor, "A");	
		zoomedNode = (ZoomableNodeContainer)compEditPart.getModel();
		cmd = new FisheyeZoom();
		cmd.setTarget(zoomedNode);
		cmd.execute();
		
		exportModel(editor, "LayoutStabilityAZoomedIn.pdf");
		
		closeEditor(editor);
		
		
	}
	
}
