package ch.unizh.ifi.adora.figureexporter.thesis;

import java.io.IOException;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.gef.EditPart;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.ui.IEditorPart;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.figureexporter.FigureExporter;
import ch.unizh.ifi.adora.model.geometry.Rectangle;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.ui.preferences.IPreferenceNameConstants;
import ch.unizh.ifi.adora.zoom.adora.AdoraZoomAdapter;

public class DiscussionZoom extends FigureExporter {

	public void testShadowing() throws CoreException, IOException {
		
		IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_HORIZONTAL_INTERVALS, true);
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_VERTICAL_INTERVALS, true);
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_ZOOM_HOLES, true);
		
		store.setValue(IPreferenceNameConstants.ADORA_VIEW_SHADE_HIERARCHY, false);
		
		AdoraZoomAdapter zoom = AdoraZoomAdapter.getInstance();
		
		// Create a simple model
		AdoraModel model = new AdoraModel();
		
		Component root = addComponent(model, new Rectangle(10, 10, 800, 550), "");
		addComponent(root, new Rectangle(50, 100, 300, 350), "A");
		Component b = addComponent(root, new Rectangle(400, 50, 350, 200), "B");
		Component c = addComponent(root, new Rectangle(450, 300, 250, 200), "C");
		
		addComponent(b, new Rectangle(100, 50, 150, 100), "D");
		addComponent(c, new Rectangle(50, 50, 150, 100), "E");
		
		// Initialize the editor
		IFile file = storeModel(model);
		IEditorPart editor = openEditor(file);
		
		// Execute a layout operation to initialize the interval structure
		EditPart compEditPart = getEditPartByName(editor, "A");
		Component a = (Component)compEditPart.getModel();
		compEditPart = getEditPartByName(editor, "B");
		b = (Component)compEditPart.getModel();
		compEditPart = getEditPartByName(editor, "C");
		c = (Component)compEditPart.getModel();
		zoom.changeNodeBounds(a, a.getZoomBounds());
		
		// Export an image with the initial situation
		exportModel(editor, "ShadowingInitial.pdf");
		
		zoom.zoom(b, false);
		zoom.zoom(c, false);
		
		exportModel(editor, "ShadowingFinal.pdf");
		
		closeEditor(editor);
		
	}
	
	
	public void testNoOverlapps() throws CoreException, IOException {
		
		IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_HORIZONTAL_INTERVALS, true);
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_VERTICAL_INTERVALS, true);
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_ZOOM_HOLES, true);
		
		store.setValue(IPreferenceNameConstants.ADORA_VIEW_SHADE_HIERARCHY, false);
		
		AdoraZoomAdapter zoom = AdoraZoomAdapter.getInstance();
		
		// Create a simple model
		AdoraModel model = new AdoraModel();
		
		Component root = addComponent(model, new Rectangle(10, 10, 500, 370), "");
		addComponent(root, new Rectangle(50, 50, 150, 100), "A");
		addComponent(root, new Rectangle(300, 50, 150, 100), "B");
		addComponent(root, new Rectangle(100, 200, 300, 120), "C");
		
		// Initialize the editor
		IFile file = storeModel(model);
		IEditorPart editor = openEditor(file);
		
		// Execute a layout operation to initialize the interval structure
		EditPart compEditPart = getEditPartByName(editor, "A");
		Component a = (Component)compEditPart.getModel();
		zoom.changeNodeBounds(a, a.getZoomBounds());
		
		// Export an image with the initial situation
		exportModel(editor, "NoOverlaps.pdf");
		
		closeEditor(editor);
		
	}
	
	
	public void testOrthogonalOrdering() throws CoreException, IOException {
		
		IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_HORIZONTAL_INTERVALS, true);
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_VERTICAL_INTERVALS, true);
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_ZOOM_HOLES, true);
		
		store.setValue(IPreferenceNameConstants.ADORA_VIEW_SHADE_HIERARCHY, false);
		
		AdoraZoomAdapter zoom = AdoraZoomAdapter.getInstance();
		
		// Create a simple model
		AdoraModel model = new AdoraModel();
		
		Component root = addComponent(model, new Rectangle(10, 10, 390, 460), "");
		addComponent(root, new Rectangle(30, 30, 200, 400), "A");
		addComponent(root, new Rectangle(260, 30, 100, 100), "B");
		addComponent(root, new Rectangle(260, 160, 100, 110), "C");
		
		// Initialize the editor
		IFile file = storeModel(model);
		IEditorPart editor = openEditor(file);
		
		// Execute a layout operation to initialize the interval structure
		EditPart compEditPart = getEditPartByName(editor, "A");
		Component a = (Component)compEditPart.getModel();
		zoom.changeNodeBounds(a, a.getZoomBounds());
		
		// Export an image with the initial situation
		exportModel(editor, "OrthogonalOrderingInitial.pdf");
		
		zoom.zoom(a, false);
		
		exportModel(editor, "OrthogonalOrderingFinal.pdf");
		
		closeEditor(editor);
		
	}
	
	
	public void testProximity() throws CoreException, IOException {
		
		IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_HORIZONTAL_INTERVALS, true);
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_VERTICAL_INTERVALS, true);
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_ZOOM_HOLES, true);
		
		store.setValue(IPreferenceNameConstants.ADORA_VIEW_SHADE_HIERARCHY, false);
		
		AdoraZoomAdapter zoom = AdoraZoomAdapter.getInstance();
		
		// Create a simple model
		AdoraModel model = new AdoraModel();
		
		Component root = addComponent(model, new Rectangle(10, 10, 400, 340), "");
		addComponent(root, new Rectangle(20, 20, 300, 220), "A");
		addComponent(root, new Rectangle(60, 260, 80, 60), "B");
		addComponent(root, new Rectangle(200, 260, 80, 60), "C");
		addComponent(root, new Rectangle(300, 105, 80, 60), "D");
		
		// Initialize the editor
		IFile file = storeModel(model);
		IEditorPart editor = openEditor(file);
		
		// Execute a layout operation to initialize the interval structure
		EditPart compEditPart = getEditPartByName(editor, "A");
		Component a = (Component)compEditPart.getModel();
		
		zoom.changeNodeBounds(a, a.getZoomBounds());

		
		// Export an image with the initial situation
		exportModel(editor, "ProximityInitial.pdf");
		
		zoom.zoom(a, false);
		
		exportModel(editor, "ProximityFinal.pdf");
		
		closeEditor(editor);
	}
	
	
	public void testStability() throws CoreException, IOException {
		
		IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_HORIZONTAL_INTERVALS, true);
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_VERTICAL_INTERVALS, true);
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_ZOOM_HOLES, true);
		
		store.setValue(IPreferenceNameConstants.ADORA_VIEW_SHADE_HIERARCHY, false);
		
		AdoraZoomAdapter zoom = AdoraZoomAdapter.getInstance();
		
		// Create a simple model
		AdoraModel model = new AdoraModel();
		
		Component root = addComponent(model, new Rectangle(10, 10, 330, 330), "");
		addComponent(root, new Rectangle(30, 160, 200, 140), "A");
		addComponent(root, new Rectangle(100, 30, 200, 100), "B");
		
		// Initialize the editor
		IFile file = storeModel(model);
		IEditorPart editor = openEditor(file);
		
		// Execute a layout operation to initialize the interval structure
		EditPart compEditPart = getEditPartByName(editor, "A");
		Component a = (Component)compEditPart.getModel();
		compEditPart = getEditPartByName(editor, "B");
		Component b = (Component)compEditPart.getModel();
		
		zoom.changeNodeBounds(a, a.getZoomBounds());
		
		// Export an image with the initial situation
		exportModel(editor, "StabilityInitial.pdf");
		
		zoom.zoom(a, false);
		
		exportModel(editor, "StabilityAZoomed.pdf");
		
		zoom.zoom(b, false);
		
		exportModel(editor, "StabilityBZoomed.pdf");
		
		zoom.zoom(a, true);
		
		exportModel(editor, "StabilityFinal.pdf");
		
		closeEditor(editor);
		
	}
	
	
	public void testComplexity() throws CoreException, IOException {
		
		IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_HORIZONTAL_INTERVALS, true);
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_VERTICAL_INTERVALS, true);
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_ZOOM_HOLES, true);
		
		store.setValue(IPreferenceNameConstants.ADORA_VIEW_SHADE_HIERARCHY, false);
		
		AdoraZoomAdapter zoom = AdoraZoomAdapter.getInstance();
		
		// Create a simple model
		AdoraModel model = new AdoraModel();
		
		Component root = addComponent(model, new Rectangle(10, 10, 300, 220), " ");
		addComponent(root, new Rectangle(40, 40, 100, 70), "A");
		
		// Initialize the editor
		IFile file = storeModel(model);
		IEditorPart editor = openEditor(file);
		
		// Execute a layout operation to initialize the interval structure
		EditPart compEditPart = getEditPartByName(editor, "A");
		Component a = (Component)compEditPart.getModel();
		
		zoom.changeNodeBounds(a, a.getZoomBounds());
		
		// Export an image with the initial situation
		exportModel(editor, "ComplexityInitial.pdf");
		
		// Get the AdoraModelEditPart
		EditPart rootEditPart = getEditPartByName(editor, " ");
		
		addComponent(editor, rootEditPart, new org.eclipse.draw2d.geometry.Rectangle(90, 150, 170, 50), "B");
		
		exportModel(editor, "ComplexityFinal.pdf");
		
		closeEditor(editor);
		
	}
	
	
}
