package ch.unizh.ifi.adora.figureexporter.thesis;

import java.io.IOException;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.commands.Command;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.ui.IEditorPart;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.figureexporter.FigureExporter;
import ch.unizh.ifi.adora.model.commands.RemoveNodeVisually;
import ch.unizh.ifi.adora.model.geometry.Rectangle;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.parts.AdoraModelEditPart;
import ch.unizh.ifi.adora.parts.AdoraRootEditPart;
import ch.unizh.ifi.adora.ui.preferences.IPreferenceNameConstants;
import ch.unizh.ifi.adora.zoom.adora.AdoraZoomAdapter;

public class Editing extends FigureExporter {

	
	public void testInserting() throws CoreException, IOException {
		
		IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_HORIZONTAL_INTERVALS, true);
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_VERTICAL_INTERVALS, true);
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_ZOOM_HOLES, true);
		
		store.setValue(IPreferenceNameConstants.ADORA_VIEW_SHADE_HIERARCHY, false);
		
		AdoraZoomAdapter zoom = AdoraZoomAdapter.getInstance();
		
		// Create a simple model
		AdoraModel model = new AdoraModel();
		
		Component root = addComponent(model, new Rectangle(10, 10, 600, 500), " ");
		addComponent(root, new Rectangle(50, 50, 140, 350), "A");
		addComponent(root, new Rectangle(320, 100, 180, 120), "B");
		addComponent(root, new Rectangle(270, 300, 280, 150), "C");
		
		// Initialize the editor
		IFile file = storeModel(model);
		IEditorPart editor = openEditor(file);
		
		// Execute a layout operation to initialize the interval structure
		EditPart compEditPart = getEditPartByName(editor, "A");
		Component a = (Component)compEditPart.getModel();
		zoom.changeNodeBounds(a, a.getZoomBounds());
		
		// Export an image with the initial situation
		exportModel(editor, "InsertingInitial.pdf");
		
		// Get the AdoraModelEditPart
		EditPart rootEditPart = getEditPartByName(editor, " ");
		
		addComponent(editor, rootEditPart, new org.eclipse.draw2d.geometry.Rectangle(240, 90, 150, 250), "D");
		
		// Export an image with the final situation
		exportModel(editor, "InsertingFinal.pdf");
		
		closeEditor(editor);
		
	}
	
	
	public void testRemoving() throws CoreException, IOException {
		
		IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_HORIZONTAL_INTERVALS, true);
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_VERTICAL_INTERVALS, true);
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_ZOOM_HOLES, true);
		
		store.setValue(IPreferenceNameConstants.ADORA_VIEW_SHADE_HIERARCHY, false);
		
		AdoraZoomAdapter zoom = AdoraZoomAdapter.getInstance();
		
		// Create a simple model
		AdoraModel model = new AdoraModel();
		
		Component root = addComponent(model, new Rectangle(10, 10, 550, 350), "");
		addComponent(root, new Rectangle(50, 50, 150, 100), "A");
		addComponent(root, new Rectangle(250, 100, 200, 100), "B");
		addComponent(root, new Rectangle(350, 250, 150, 50), "C");
		
		// Initialize the editor
		IFile file = storeModel(model);
		IEditorPart editor = openEditor(file);
		
		// Execute a layout operation to initialize the interval structure
		EditPart compEditPart = getEditPartByName(editor, "B");
		Component b = (Component)compEditPart.getModel();
		zoom.changeNodeBounds(b, b.getZoomBounds());
		
		// Export an image with the initial situation
		exportModel(editor, "RemovingInitial.pdf");
		
		Command cmd = new RemoveNodeVisually(b);
		cmd.execute();
		
		exportModel(editor, "RemovingFinal.pdf");
		
		closeEditor(editor);
		
	}
	
	
	public void testEditingZoomHole() throws CoreException, IOException {
		
		IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_HORIZONTAL_INTERVALS, true);
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_VERTICAL_INTERVALS, true);
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_ZOOM_HOLES, true);
		
		AdoraZoomAdapter zoom = AdoraZoomAdapter.getInstance();
		
		// Create a simple model
		AdoraModel model = new AdoraModel();
		
		addComponent(model, new Rectangle(60, 10, 150, 90), "A");
		addComponent(model, new Rectangle(10, 130, 100, 70), "B");
		addComponent(model, new Rectangle(160, 130, 100, 70), "C");
	
		// Initialize the editor
		IFile file = storeModel(model);
		IEditorPart editor = openEditor(file);
		
		// Get the AdoraModelEditPart
		GraphicalViewer viewer = (GraphicalViewer)editor.getAdapter(GraphicalViewer.class);
		AdoraRootEditPart rootEditPart = (AdoraRootEditPart)viewer.getRootEditPart();
		AdoraModelEditPart modelEditPart = (AdoraModelEditPart)rootEditPart.getChildren().get(0);
		
		// Execute a layout operation to initialize the interval structure
		EditPart compEditPart = getEditPartByName(editor, "A");
		Component a = (Component)compEditPart.getModel();
		zoom.changeNodeBounds(a, new Rectangle(60, 10, 150, 80));
		
		// Export an image with the initial situation
		exportModel(editor, "InsertingZoomHoleInitial.pdf");
		
		addComponent(editor, modelEditPart, new org.eclipse.draw2d.geometry.Rectangle(85, 120, 100, 90), "D");
		
		// Export an image with the final situation
		exportModel(editor, "InsertingZoomHoleFinal.pdf");
		
		closeEditor(editor);
	}
	
	
	public void testEditZoomedNode() throws CoreException, IOException {
		
		IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
		//	Don't shade the hierarchy
		store.setValue(IPreferenceNameConstants.ADORA_VIEW_SHADE_HIERARCHY, false);
		
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_HORIZONTAL_INTERVALS, true);
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_VERTICAL_INTERVALS, true);
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_ZOOM_HOLES, false);
		
		AdoraZoomAdapter zoom = AdoraZoomAdapter.getInstance();
		
		// Create a simple model
		AdoraModel model = new AdoraModel();
		
		Component root = addComponent(model, new Rectangle(10, 10, 300, 290), "");
		addComponent(root, new Rectangle(70, 40, 160, 70), "A");
		Component b = addComponent(root, new Rectangle(30, 100, 240, 160), "B");
		addComponent(b, new Rectangle(40, 40, 100, 50), "C");
		
		// Initialize the editor
		IFile file = storeModel(model);
		IEditorPart editor = openEditor(file);
		
		// Execute a layout operation to initialize the interval structure
		EditPart compEditPart = getEditPartByName(editor, "B");
		b = (Component)compEditPart.getModel();
		zoom.changeNodeBounds(b, b.getZoomBounds());
		
		// Export an image with the initial situation
		exportModel(editor, "EditingZoomedNodeInitial.pdf");
		
		zoom.zoom(b, false);
		
		exportModel(editor, "EditingZoomedNodeZoomed.pdf");
		
		zoom.changeNodeBounds(b, new Rectangle(b.getZoomBounds().x, 140, b.getZoomBounds().width, b.getZoomBounds().height));
		
		exportModel(editor, "EditingZoomedNodeEdited.pdf");
		
		zoom.zoom(b, true);
		
		exportModel(editor, "EditingZoomedNodeFinal.pdf");
		
		
		closeEditor(editor);
		
	}
	
	
	public void testUndoInsert() throws CoreException, IOException {
		
		IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
		//	Don't shade the hierarchy
		store.setValue(IPreferenceNameConstants.ADORA_VIEW_SHADE_HIERARCHY, false);
		
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_HORIZONTAL_INTERVALS, true);
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_VERTICAL_INTERVALS, true);
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_ZOOM_HOLES, false);
		
		AdoraZoomAdapter zoom = AdoraZoomAdapter.getInstance();
		
		// Create a simple model
		AdoraModel model = new AdoraModel();
		
		Component parent = addComponent(model, new Rectangle(10, 10, 230, 210), " ");
		addComponent(parent, new Rectangle(70, 100, 130, 80), "A");
		
		// Initialize the editor
		IFile file = storeModel(model);
		IEditorPart editor = openEditor(file);
		
		// Execute a layout operation to initialize the interval structure
		EditPart compEditPart = getEditPartByName(editor, "A");
		Component a = (Component)compEditPart.getModel();
		zoom.changeNodeBounds(a, a.getZoomBounds());
		
		exportModel(editor, "EditingUndoInsertInitial.pdf");
		
		compEditPart = addComponent(editor, getEditPartByName(editor, " "), new org.eclipse.draw2d.geometry.Rectangle(50, 30, 120, 130), "B");
		
		exportModel(editor, "EditingUndoInsertInserted.pdf");
		
		Command cmd = new RemoveNodeVisually((Component)compEditPart.getModel());
		cmd.execute();
		
		exportModel(editor, "EditingUndoInsertRemoved.pdf");
		
		closeEditor(editor);
		
	}
	
}
