package ch.unizh.ifi.adora.figureexporter.thesis;

import java.io.IOException;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.ui.IEditorPart;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.figureexporter.FigureExporter;
import ch.unizh.ifi.adora.model.commands.representation.RepresentationNodeBloat;
import ch.unizh.ifi.adora.model.commands.representation.RepresentationNodeShrink;
import ch.unizh.ifi.adora.model.geometry.Dimension;
import ch.unizh.ifi.adora.model.geometry.Rectangle;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.representation.management.ViewManager;
import ch.unizh.ifi.adora.ui.preferences.IPreferenceNameConstants;
import ch.unizh.ifi.adora.zoom.adora.AdoraZoomAdapter;

public class Filtering extends FigureExporter {
	
	
	protected void setUp() {
		
		super.setUp();
		
		IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_HORIZONTAL_INTERVALS, true);
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_VERTICAL_INTERVALS, true);
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_ZOOM_HOLES, true);
		
		store.setValue(IPreferenceNameConstants.ADORA_VIEW_SHADE_HIERARCHY, false);
		
	}
	

	public void testStabilityMinimalSize() throws CoreException, IOException {
		
		AdoraZoomAdapter zoom = AdoraZoomAdapter.getInstance();
		
		// Create a simple model
		AdoraModel model = new AdoraModel();
		
		Component a = addComponent(model, new Rectangle(10, 10, 320, 160), "A");
		addComponent(a, new Rectangle(30, 40, 260, 101), "B");
		
		// Initialize the editor
		IFile file = storeModel(model);
		IEditorPart editor = openEditor(file);
		
		// Execute a layout operation to initialize the interval structure
		EditPart compEditPart = getEditPartByName(editor, "B");
		Component b = (Component)compEditPart.getModel();
		zoom.changeNodeBounds(b, new Rectangle(30, 40, 260, 100));
		
		// Export an image with the initial situation
		exportModel(editor, "StabilityMinimalSizeInitial.pdf");
		
		// Filter (hide) node b
		Command cmd = new RepresentationNodeShrink(b);
		cmd.execute();
		b.updateVisibility();
		
		exportModel(editor, "StabilityMinimalSizeFiltered.pdf");
		
		// Show node b again
		b.updateVisibility();
		cmd = new RepresentationNodeBloat(b);
		cmd.execute();
		
		exportModel(editor, "StabilityMinimalSizeFinal.pdf");
		
		closeEditor(editor);
		
	}
	
	
	public void testHideNodes() throws CoreException, IOException {
		
		IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_ZOOM_HOLES, false);
		
		AdoraZoomAdapter zoom = AdoraZoomAdapter.getInstance();
		
		// Create a simple model
		AdoraModel model = new AdoraModel();
		
		Component a = addComponent(model, new Rectangle(5, 5, 700, 400), "A");
		Component b = addComponent(a, new Rectangle(50, 50, 300, 200), "B");
		
		Scenario c = new Scenario();
		c.setLocation(420, 100);
		c.setSize(new Dimension(200, 100));
		c.setName("C");
		a.addChild(c);
		zoom.insertNode(c);
		
		Scenario d = new Scenario();
		d.setLocation(230, 280);
		d.setSize(new Dimension(300, 90));
		d.setName("D");
		a.addChild(d);
		zoom.insertNode(d);
		
		Scenario e = new Scenario();
		e.setLocation(50, 70);
		e.setSize(new Dimension(200, 100));
		e.setName("E");
		b.addChild(e);
		zoom.insertNode(e);
		
		// Initialize the editor
		IFile file = storeModel(model);
		IEditorPart editor = openEditor(file);
		
		// Filter (hide) the scenarios
		ViewManager vm = ViewManager.getInstance();
		model.changeViewFilterState(Scenario.class);
		vm.applyViewFilter(Scenario.class);
		
		exportModel(editor, "FilterNodesHidden.pdf");
		
		// Show the scenarios
		model.changeViewFilterState(Scenario.class);
		vm.applyViewFilter(Scenario.class);
		
		exportModel(editor, "FilterNodesInitial.pdf");
		
		closeEditor(editor);
		
	}
	
}
