package ch.unizh.ifi.adora.figureexporter.thesis;

import java.io.IOException;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.ui.IEditorPart;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.figureexporter.FigureExporter;
import ch.unizh.ifi.adora.model.commands.TransitionCreate;
import ch.unizh.ifi.adora.model.geometry.Point;
import ch.unizh.ifi.adora.model.geometry.Rectangle;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.parts.ComponentEditPart;
import ch.unizh.ifi.adora.ui.preferences.IPreferenceNameConstants;
import ch.unizh.ifi.adora.zoom.adora.AdoraZoomAdapter;

public class LinkLabelProblem extends FigureExporter {

	public void testOverlappingAfterZooming() throws CoreException, IOException {		
		
		AdoraZoomAdapter zoom = AdoraZoomAdapter.getInstance();
		
		// Create a simple model
		AdoraModel model = new AdoraModel();
		
		Component a = addComponent(model, new Rectangle(10, 10, 350, 130), "A");
		addComponent(model, new Rectangle(10, 160, 100, 70), "B");
		addComponent(model, new Rectangle(260, 160, 100, 70), "C");
		addComponent(a, new Rectangle(120, 30, 110, 70), "D");
		
		// Initialize the editor
		IFile file = storeModel(model);
		IEditorPart editor = openEditor(file);
		
		ComponentEditPart compAEditPart = (ComponentEditPart)getEditPartByName(editor, "A");
		a = (Component)compAEditPart.getModel();
		ComponentEditPart compBEditPart = (ComponentEditPart)getEditPartByName(editor, "B");
		Component b = (Component)compBEditPart.getModel();
		EditPart compCEditPart = getEditPartByName(editor, "C");
		Component c = (Component)compCEditPart.getModel();
		
		// Create a transition between state 1 and state 2
		TransitionCreate cmd = new TransitionCreate(b);
		cmd.setTarget(c);
		cmd.setSourceClickPoint(new Point(90, 35));
		cmd.setTargetClickPoint(new Point(10, 35));
		cmd.setTransitionDescription("receive check() | send valid()");
		
		cmd.execute();
		
		// Flush all deferred updates of the view so that the line is routed.
		// That's necessary because the line router works on the view.
		GraphicalViewer viewer = (GraphicalViewer)editor.getAdapter(GraphicalViewer.class);
		viewer.flush();
		
		// Export an image with the initial situation
		exportModel(editor, "LabelProblemZoomingInitial.pdf");
		// TODO: Connection labels are only shown on the second export!
		exportModel(editor, "LabelProblemZoomingInitial.pdf");
		
		zoom.zoom(a, false);
		
		exportModel(editor, "LabelProblemZoomingZoomed.pdf");
		
		closeEditor(editor);
		
	}
	
	
	public void testOverlappingAfterZoomingSolution() throws CoreException, IOException {
		
		// Enable automatic label placement
		IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_PLACE_LABELS, true);
		
		AdoraZoomAdapter zoom = AdoraZoomAdapter.getInstance();
		
		// Create a simple model
		AdoraModel model = new AdoraModel();
		
		Component a = addComponent(model, new Rectangle(10, 10, 350, 130), "A");
		addComponent(model, new Rectangle(10, 160, 100, 70), "B");
		addComponent(model, new Rectangle(260, 160, 100, 70), "C");
		addComponent(a, new Rectangle(120, 30, 110, 70), "D");
		
		// Initialize the editor
		IFile file = storeModel(model);
		IEditorPart editor = openEditor(file);
		
		ComponentEditPart compAEditPart = (ComponentEditPart)getEditPartByName(editor, "A");
		a = (Component)compAEditPart.getModel();
		ComponentEditPart compBEditPart = (ComponentEditPart)getEditPartByName(editor, "B");
		Component b = (Component)compBEditPart.getModel();
		EditPart compCEditPart = getEditPartByName(editor, "C");
		Component c = (Component)compCEditPart.getModel();
		
		// Create a transition between state 1 and state 2
		TransitionCreate cmd = new TransitionCreate(b);
		cmd.setTarget(c);
		cmd.setSourceClickPoint(new Point(90, 35));
		cmd.setTargetClickPoint(new Point(10, 35));
		cmd.setTransitionDescription("receive check() | send valid()");
		
		cmd.execute();
		
		// Flush all deferred updates of the view so that the line is routed.
		// That's necessary because the line router works on the view.
		GraphicalViewer viewer = (GraphicalViewer)editor.getAdapter(GraphicalViewer.class);
		viewer.flush();
		
		zoom.zoom(a, false);
		
		exportModel(editor, "LabelProblemZoomingSolution.pdf");
		// TODO: Connection labels are only shown on the second export!
		exportModel(editor, "LabelProblemZoomingSolution.pdf");
		
		closeEditor(editor);
		
	}
	
	
	public void testLabelPlacementStructure() throws CoreException, IOException {
		
		// Enable automatic label placement
		IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_PLACE_LABELS, true);
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_SHOW_TILES, true);
		
		// Create a simple model
		AdoraModel model = new AdoraModel();
		
		Component root = addComponent(model, new Rectangle(5, 5, 540, 370), "");
		
		addComponent(root, new Rectangle(20, 20, 200, 90), "A");
		addComponent(root, new Rectangle(300, 260, 200, 90), "B");
		
		// Initialize the editor
		IFile file = storeModel(model);
		IEditorPart editor = openEditor(file);
		
		ComponentEditPart compAEditPart = (ComponentEditPart)getEditPartByName(editor, "A");
		Component a = (Component)compAEditPart.getModel();
		ComponentEditPart compBEditPart = (ComponentEditPart)getEditPartByName(editor, "B");
		Component b = (Component)compBEditPart.getModel();
		
		// Create a transition between state 1 and state 2
		TransitionCreate cmd = new TransitionCreate(a);
		cmd.setTarget(b);
		cmd.setSourceClickPoint(new Point(190, 45));
		cmd.setTargetClickPoint(new Point(100, 10));
		cmd.setTransitionDescription("receive check() | send valid()");
		
		cmd.execute();
		
		// Flush all deferred updates of the view so that the line is routed.
		// That's necessary because the line router works on the view.
		GraphicalViewer viewer = (GraphicalViewer)editor.getAdapter(GraphicalViewer.class);
		viewer.flush();
		
		exportModel(editor, "LabelPlacementStructure.pdf");
		
		closeEditor(editor);
		
	}
	
	
	public void testShowLabelPositions() throws CoreException, IOException {
		
		// Enable automatic label placement
		IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_PLACE_LABELS, true);
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_SHOW_LABEL_POSITIONS, true);
		
		// Create a simple model
		AdoraModel model = new AdoraModel();
		
		Component root = addComponent(model, new Rectangle(5, 5, 540, 370), "");
		
		addComponent(root, new Rectangle(20, 20, 200, 90), "A");
		addComponent(root, new Rectangle(300, 260, 200, 90), "B");
		
		// Initialize the editor
		IFile file = storeModel(model);
		IEditorPart editor = openEditor(file);
		
		ComponentEditPart compAEditPart = (ComponentEditPart)getEditPartByName(editor, "A");
		Component a = (Component)compAEditPart.getModel();
		ComponentEditPart compBEditPart = (ComponentEditPart)getEditPartByName(editor, "B");
		Component b = (Component)compBEditPart.getModel();
		
		// Create a transition between state 1 and state 2
		TransitionCreate cmd = new TransitionCreate(a);
		cmd.setTarget(b);
		cmd.setSourceClickPoint(new Point(190, 45));
		cmd.setTargetClickPoint(new Point(100, 10));
		cmd.setTransitionDescription("receive check() | send valid()");
		
		cmd.execute();
		
		// Flush all deferred updates of the view so that the line is routed.
		// That's necessary because the line router works on the view.
		GraphicalViewer viewer = (GraphicalViewer)editor.getAdapter(GraphicalViewer.class);
		viewer.flush();
		
		exportModel(editor, "LabelPositions.pdf");
		
		closeEditor(editor);
		
	}
	
	
}
