package ch.unizh.ifi.adora.figureexporter.thesis;

import java.io.IOException;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.ui.IEditorPart;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.figureexporter.FigureExporter;
import ch.unizh.ifi.adora.model.commands.AssociationCreate;
import ch.unizh.ifi.adora.model.geometry.Point;
import ch.unizh.ifi.adora.model.geometry.Rectangle;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.parts.ComponentEditPart;
import ch.unizh.ifi.adora.ui.preferences.IPreferenceNameConstants;

public class RoutingAlgorithm extends FigureExporter {

	public void testTileStructure() throws CoreException, IOException {
		
		IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_SHOW_TILES, true);
		store.setValue(IPreferenceNameConstants.ADORA_VIEW_SHADE_HIERARCHY, false);
		
		// Create a simple model
		AdoraModel model = new AdoraModel();
		
		Component root = addComponent(model, new Rectangle(5, 5, 520, 400), "");
		
		addComponent(root, new Rectangle(30, 60, 100, 170), "A");
		addComponent(root, new Rectangle(160, 30, 300, 100), "B");
		addComponent(root, new Rectangle(100, 260, 180, 110), "C");
		addComponent(root, new Rectangle(340, 180, 150, 150), "D");
		
		// Initialize the editor
		IFile file = storeModel(model);
		IEditorPart editor = openEditor(file);
		
		ComponentEditPart compEditPart = (ComponentEditPart)getEditPartByName(editor, "A");
		Component a = (Component)compEditPart.getModel();
		compEditPart = (ComponentEditPart)getEditPartByName(editor, "B");
		Component b = (Component)compEditPart.getModel();
		
		// We have to add a line so that the tile structure is constructed
		AssociationCreate cmd = new AssociationCreate(a);
		cmd.setTarget(b);
		cmd.setSourceClickPoint(new Point(50, 35));
		cmd.setTargetClickPoint(new Point(50, 35));
		
		cmd.execute();
		
		// Export an image with the initial situation
		exportModel(editor, "TileStructure.pdf");
		
		closeEditor(editor);
		
	}
	
	
	public void testPathFinding() throws CoreException, IOException {
		
		IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_SHOW_TILES, true);
		store.setValue(IPreferenceNameConstants.ADORA_VIEW_SHADE_HIERARCHY, false);
		
		// Create a simple model
		AdoraModel model = new AdoraModel();
		
		Component root = addComponent(model, new Rectangle(5, 5, 410, 270), "");
		
		addComponent(root, new Rectangle(50, 60, 100, 70), "A");
		addComponent(root, new Rectangle(200, 30, 150, 130), "B");
		addComponent(root, new Rectangle(50, 200, 140, 100), "C");
		addComponent(root, new Rectangle(250, 220, 100, 60), "D");
		
		// Initialize the editor
		IFile file = storeModel(model);
		IEditorPart editor = openEditor(file);
		
		ComponentEditPart compEditPart = (ComponentEditPart)getEditPartByName(editor, "A");
		Component a = (Component)compEditPart.getModel();
		compEditPart = (ComponentEditPart)getEditPartByName(editor, "D");
		Component d = (Component)compEditPart.getModel();
		
		// We have to add a line so that the tile structure is constructed
		AssociationCreate cmd = new AssociationCreate(a);
		cmd.setTarget(d);
		cmd.setSourceClickPoint(new Point(70, 70));
		cmd.setTargetClickPoint(new Point(50, 0));
		
		cmd.execute();
		
		// Export an image with the initial situation
		exportModel(editor, "PathFinding.pdf");
		
		closeEditor(editor);
		
	}
	
	
	public void testRouteOrthogonal() throws CoreException, IOException {
		
		IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_SHOW_TILES, true);
		store.setValue(IPreferenceNameConstants.ADORA_VIEW_SHADE_HIERARCHY, false);
		
		// Create a simple model
		AdoraModel model = new AdoraModel();
		
		Component root = addComponent(model, new Rectangle(5, 5, 410, 290), "");
		
		addComponent(root, new Rectangle(30, 30, 120, 30), "A");
		addComponent(root, new Rectangle(50, 90, 120, 120), "B");
		addComponent(root, new Rectangle(270, 20, 120, 100), "C");
		addComponent(root, new Rectangle(240, 150, 120, 60), "D");
		addComponent(root, new Rectangle(290, 240, 100, 30), "E");
		
		// Initialize the editor
		IFile file = storeModel(model);
		IEditorPart editor = openEditor(file);
		
		ComponentEditPart compEditPart = (ComponentEditPart)getEditPartByName(editor, "A");
		Component a = (Component)compEditPart.getModel();
		compEditPart = (ComponentEditPart)getEditPartByName(editor, "E");
		Component e = (Component)compEditPart.getModel();
		
		// We have to add a line so that the tile structure is constructed
		AssociationCreate cmd = new AssociationCreate(a);
		cmd.setTarget(e);
		cmd.setSourceClickPoint(new Point(60, 40));
		cmd.setTargetClickPoint(new Point(50, 0));
		
		cmd.execute();
		
		exportModel(editor, "OrthogonalRouting.pdf");
		
		closeEditor(editor);
		
	}
	
	
	public void testMultipleSolutions() throws CoreException, IOException {
		
		IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_SHOW_TILES, true);
		store.setValue(IPreferenceNameConstants.ADORA_VIEW_SHADE_HIERARCHY, false);
		
		// Create a simple model
		AdoraModel model = new AdoraModel();
		
		Component root = addComponent(model, new Rectangle(5, 5, 380, 260), "");
		
		addComponent(root, new Rectangle(30, 30, 100, 70), "A");
		addComponent(root, new Rectangle(90, 130, 100, 70), "B");
		addComponent(root, new Rectangle(250, 160, 100, 70), "C");
		
		// Initialize the editor
		IFile file = storeModel(model);
		IEditorPart editor = openEditor(file);
		
		ComponentEditPart compEditPart = (ComponentEditPart)getEditPartByName(editor, "A");
		Component a = (Component)compEditPart.getModel();
		compEditPart = (ComponentEditPart)getEditPartByName(editor, "C");
		Component c = (Component)compEditPart.getModel();
		
		// We have to add a line so that the tile structure is constructed
		AssociationCreate cmd = new AssociationCreate(a);
		cmd.setTarget(c);
		cmd.setSourceClickPoint(new Point(20, 70));
		cmd.setTargetClickPoint(new Point(0, 60));
		
		cmd.execute();
		
		exportModel(editor, "MultipleSolutions.pdf");
		
		closeEditor(editor);
		
	}
	
	
	public void testLineCrossing() throws CoreException, IOException {
		
		// Set the distToNode value in the LineTileStructure to 10 so that
		// the line tiles are visible.
		
		IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_SHOW_TILES, true);
		store.setValue(IPreferenceNameConstants.ADORA_VIEW_SHADE_HIERARCHY, false);
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_LINE_COSTFACTOR, 4);
		
		// Create a simple model
		AdoraModel model = new AdoraModel();
		
		Component root = addComponent(model, new Rectangle(5, 5, 530, 350), "");
		
		addComponent(root, new Rectangle(170, 30, 100, 60), "A");
		addComponent(root, new Rectangle(30, 120, 60, 90), "B");
		addComponent(root, new Rectangle(170, 260, 100, 60), "C");
		addComponent(root, new Rectangle(400, 150, 100, 80), "D");
		
		// Initialize the editor
		IFile file = storeModel(model);
		IEditorPart editor = openEditor(file);
		
		ComponentEditPart compEditPart = (ComponentEditPart)getEditPartByName(editor, "A");
		Component a = (Component)compEditPart.getModel();
		compEditPart = (ComponentEditPart)getEditPartByName(editor, "C");
		Component c = (Component)compEditPart.getModel();
		
		// Add the first (existing) line
		AssociationCreate cmd = new AssociationCreate(a);
		cmd.setTarget(c);
		cmd.setSourceClickPoint(new Point(50, 60));
		cmd.setTargetClickPoint(new Point(50, 0));
		
		cmd.execute();
		
		compEditPart = (ComponentEditPart)getEditPartByName(editor, "B");
		Component b = (Component)compEditPart.getModel();
		compEditPart = (ComponentEditPart)getEditPartByName(editor, "D");
		Component d = (Component)compEditPart.getModel();
		
		// Add the second (new) line
		cmd = new AssociationCreate(b);
		cmd.setTarget(d);
		cmd.setSourceClickPoint(new Point(60, 60));
		cmd.setTargetClickPoint(new Point(0, 30));
		
		cmd.execute();
		
		exportModel(editor, "LineCrossing.pdf");
		
		closeEditor(editor);
		
	}
	
	
	public void testStartEndPoint() throws CoreException, IOException {
		
		IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_SHOW_TILES, true);
		store.setValue(IPreferenceNameConstants.ADORA_VIEW_SHADE_HIERARCHY, false);
		
		// Create a simple model
		AdoraModel model = new AdoraModel();
		
		Component root = addComponent(model, new Rectangle(5, 5, 500, 350), "");
		
		addComponent(root, new Rectangle(60, 50, 150, 100), "A");
		addComponent(root, new Rectangle(290, 200, 150, 100), "B");
		
		// Initialize the editor
		IFile file = storeModel(model);
		IEditorPart editor = openEditor(file);
		
		ComponentEditPart compEditPart = (ComponentEditPart)getEditPartByName(editor, "A");
		Component a = (Component)compEditPart.getModel();
		compEditPart = (ComponentEditPart)getEditPartByName(editor, "B");
		Component b = (Component)compEditPart.getModel();
		
		// Add the first (existing) line
		AssociationCreate cmd = new AssociationCreate(a);
		cmd.setTarget(b);
		cmd.setSourceClickPoint(new Point(60, 70));
		cmd.setTargetClickPoint(new Point(70, 20));
		
		cmd.execute();
		
		exportModel(editor, "StartEndPoint.pdf");
		
		closeEditor(editor);
		
	}
	
	
	public void testReflexiveLine() throws CoreException, IOException {
		
		IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_SHOW_TILES, true);
		store.setValue(IPreferenceNameConstants.ADORA_VIEW_SHADE_HIERARCHY, false);
		
		// Create a simple model
		AdoraModel model = new AdoraModel();
		
		Component root = addComponent(model, new Rectangle(5, 5, 500, 350), "");
		
		addComponent(root, new Rectangle(100, 100, 300, 150), "A");
		
		// Initialize the editor
		IFile file = storeModel(model);
		IEditorPart editor = openEditor(file);
		
		ComponentEditPart compEditPart = (ComponentEditPart)getEditPartByName(editor, "A");
		Component a = (Component)compEditPart.getModel();
		
		// Add the first (existing) line
		AssociationCreate cmd = new AssociationCreate(a);
		cmd.setTarget(a);
		cmd.setSourceClickPoint(new Point(20, 60));
		cmd.setTargetClickPoint(new Point(180, 20));
		
		cmd.execute();
		
		exportModel(editor, "ReflexiveLine.pdf");
		
		closeEditor(editor);
		
	}
	
	
}
