package ch.unizh.ifi.adora.figureexporter.thesis;

import java.io.IOException;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.ui.IEditorPart;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.figureexporter.FigureExporter;
import ch.unizh.ifi.adora.model.commands.AssociationCreate;
import ch.unizh.ifi.adora.model.commands.FisheyeZoom;
import ch.unizh.ifi.adora.model.commands.TransitionCreate;
import ch.unizh.ifi.adora.model.geometry.Point;
import ch.unizh.ifi.adora.model.geometry.Rectangle;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.parts.ComponentEditPart;
import ch.unizh.ifi.adora.ui.preferences.IPreferenceNameConstants;
import ch.unizh.ifi.adora.ui.preferences.IPreferenceValueConstants;
import ch.unizh.ifi.adora.zoom.adora.AdoraZoomAdapter;

public class RoutingGeneral extends FigureExporter {
	
	
	public void testAffectedNodes() throws CoreException, IOException {
		
		IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTER_ASSOCIATION, IPreferenceValueConstants.ADORA_REPRESENTATION_ROUTER_DIRECTLINE);
		
		// Create a simple model
		AdoraModel model = new AdoraModel();
		
		Component root = addComponent(model, new Rectangle(10, 10, 470, 350), "A");
		
		Component b = addComponent(root, new Rectangle(30, 30, 140, 170), "B");
		addComponent(b, new Rectangle(30, 30, 80, 50), "F");
		addComponent(b, new Rectangle(30, 100, 80, 50), "G");
		
		Component c = addComponent(root, new Rectangle(230, 30, 200, 100), "C");
		addComponent(c, new Rectangle(60, 30, 80, 50), "H");
		
		addComponent(root, new Rectangle(190, 170, 80, 150), "D");
		
		Component e = addComponent(root, new Rectangle(300, 160, 140, 170), "E");
		addComponent(e, new Rectangle(30, 30, 80, 50), "I");
		addComponent(e, new Rectangle(30, 100, 80, 50), "J");
		
		// Initialize the editor
		IFile file = storeModel(model);
		IEditorPart editor = openEditor(file);
		
		ComponentEditPart compEditPart = (ComponentEditPart)getEditPartByName(editor, "F");
		Component f = (Component)compEditPart.getModel();
		compEditPart = (ComponentEditPart)getEditPartByName(editor, "J");
		Component j = (Component)compEditPart.getModel();
		compEditPart = (ComponentEditPart)getEditPartByName(editor, "B");
		b = (Component)compEditPart.getModel();
		
		// Add a link between nodes F and J
		AssociationCreate cmd = new AssociationCreate(f);
		cmd.setTarget(j);
		cmd.setSourceClickPoint(new Point(50, 35));
		cmd.setTargetClickPoint(new Point(50, 35));
		
		cmd.execute();
		
		// Export an image with the initial situation
		exportModel(editor, "RoutingAffectedNodes.pdf");
		
		// We have to use a command here so that the abstract association is
		// calculated correctly.
		FisheyeZoom command = new FisheyeZoom();
		command.setTarget(b);
		command.execute();
		
		exportModel(editor, "RoutingAbstractRelationship.pdf");
		
		closeEditor(editor);
		
	}
	
	
	public void testDynamicLayout() throws CoreException, IOException {
		
		AdoraZoomAdapter zoom = AdoraZoomAdapter.getInstance();
		
		IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_SHOW_TILES, false);
		store.setValue(IPreferenceNameConstants.ADORA_VIEW_SHADE_HIERARCHY, true);
		
		// Create a simple model
		AdoraModel model = new AdoraModel();
		
		Component a = addComponent(model, new Rectangle(10, 10, 340, 330), "");
		Component b = addComponent(a, new Rectangle(20, 20, 60, 70), "A");
		Component c = addComponent(a, new Rectangle(100, 70, 220, 240), "B");
		addComponent(c, new Rectangle(60, 60, 100, 110), "");
		Component e = addComponent(model, new Rectangle(380, 80, 110, 80), "");
		addComponent(e, new Rectangle(20, 20, 70, 40), "C");
		
		// Initialize the editor
		IFile file = storeModel(model);
		IEditorPart editor = openEditor(file);
		
		ComponentEditPart compEditPart = (ComponentEditPart)getEditPartByName(editor, "A");
		b = (Component)compEditPart.getModel();
		compEditPart = (ComponentEditPart)getEditPartByName(editor, "B");
		c = (Component)compEditPart.getModel();
		compEditPart = (ComponentEditPart)getEditPartByName(editor, "C");
		Component f = (Component)compEditPart.getModel();
		
		// Add the first (existing) line
		AssociationCreate cmd = new AssociationCreate(b);
		cmd.setTarget(f);
		cmd.setSourceClickPoint(new Point(50, 35));
		cmd.setTargetClickPoint(new Point(35, 20));
		
		cmd.execute();
		
		exportModel(editor, "DynamicLayoutInitial.pdf");
		
		zoom.zoom(c, false);
		
		exportModel(editor, "DynamicLayoutFinal.pdf");
		
		closeEditor(editor);
		
	}
	
	
	public void testStartEndPoint() throws CoreException, IOException {
		
		IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_SHOW_TILES, false);
		store.setValue(IPreferenceNameConstants.ADORA_VIEW_SHADE_HIERARCHY, true);
		
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_NODE_COSTFACTOR, 2.5);
		
		// Create a simple model
		AdoraModel model = new AdoraModel();
		
		Component root = addComponent(model, new Rectangle(10, 10, 300, 370), "");
		addComponent(root, new Rectangle(100, 50, 100, 70), "A");
		addComponent(root, new Rectangle(100, 150, 100, 70), "B");
		addComponent(root, new Rectangle(100, 250, 100, 70), "C");
		
		// Initialize the editor
		IFile file = storeModel(model);
		IEditorPart editor = openEditor(file);
		
		ComponentEditPart compEditPart = (ComponentEditPart)getEditPartByName(editor, "A");
		Component a = (Component)compEditPart.getModel();
		compEditPart = (ComponentEditPart)getEditPartByName(editor, "B");
		Component b = (Component)compEditPart.getModel();
		compEditPart = (ComponentEditPart)getEditPartByName(editor, "C");
		Component c = (Component)compEditPart.getModel();
		
		TransitionCreate cmd = new TransitionCreate(a);
		cmd.setTarget(b);
		cmd.setSourceClickPoint(new Point(50, 70));
		cmd.setTargetClickPoint(new Point(50, 0));
		cmd.execute();
		
		cmd = new TransitionCreate(b);
		cmd.setTarget(c);
		cmd.setSourceClickPoint(new Point(50, 70));
		cmd.setTargetClickPoint(new Point(50, 0));
		cmd.execute();
		
		cmd = new TransitionCreate(c);
		cmd.setTarget(a);
		cmd.setSourceClickPoint(new Point(50, 70));
		cmd.setTargetClickPoint(new Point(50, 0));
		cmd.execute();
		
		exportModel(editor, "SecondaryNotationLine.pdf");
		
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_ROUTING_NODE_COSTFACTOR, 0.9);
		
		exportModel(editor, "NoSecondaryNotationLine.pdf");
		
		closeEditor(editor);
		
	}
	

}
