package ch.unizh.ifi.adora.figureexporter.thesis;

import java.io.IOException;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.gef.EditPart;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.ui.IEditorPart;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.figureexporter.FigureExporter;
import ch.unizh.ifi.adora.model.commands.FisheyeZoom;
import ch.unizh.ifi.adora.model.geometry.Rectangle;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.ui.preferences.IPreferenceNameConstants;
import ch.unizh.ifi.adora.zoom.adora.AdoraZoomAdapter;

public class ZoomAlgorithm extends FigureExporter {

	public void testIntervalStructure() throws CoreException, IOException {
		
		IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_HORIZONTAL_INTERVALS, true);
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_VERTICAL_INTERVALS, true);
		
		store.setValue(IPreferenceNameConstants.ADORA_VIEW_SHADE_HIERARCHY, false);
		
		AdoraZoomAdapter zoom = AdoraZoomAdapter.getInstance();
		
		// Create a simple model
		AdoraModel model = new AdoraModel();
		
		Component root = addComponent(model, new Rectangle(5, 5, 700, 400), "");
		addComponent(root, new Rectangle(70, 200, 320, 160), "A");
		addComponent(root, new Rectangle(40, 40, 300, 100), "B");
		addComponent(root, new Rectangle(450, 230, 200, 80), "C");
		
		// Initialize the editor
		IFile file = storeModel(model);
		IEditorPart editor = openEditor(file);
		
		// Execute a layout operation to initialize the interval structure
		EditPart compEditPart = getEditPartByName(editor, "A");
		Component a = (Component)compEditPart.getModel();
		zoom.changeNodeBounds(a, new Rectangle(80, 200, 320, 160));
		
		exportModel(editor, "IntervalStructure.pdf");
		
		closeEditor(editor);
		
	}
	
	
	public void testZoomingIn() throws CoreException, IOException {
		
		IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_HORIZONTAL_INTERVALS, true);
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_VERTICAL_INTERVALS, true);
		
		store.setValue(IPreferenceNameConstants.ADORA_VIEW_SHADE_HIERARCHY, false);
		
		AdoraZoomAdapter zoom = AdoraZoomAdapter.getInstance();
		
		// Create a simple model
		AdoraModel model = new AdoraModel();
		
		Component root = addComponent(model, new Rectangle(5, 5, 650, 420), "");
		Component a = addComponent(root, new Rectangle(40, 200, 350, 200), "A");
		addComponent(root, new Rectangle(40, 40, 200, 70), "B");
		addComponent(root, new Rectangle(450, 70, 140, 70), "C");
		addComponent(root, new Rectangle(500, 260, 60, 120), "D");
		
		addComponent(a, new Rectangle(50, 50, 50, 50), "");
		
		// Initialize the editor
		IFile file = storeModel(model);
		IEditorPart editor = openEditor(file);
		
		EditPart compEditPart = getEditPartByName(editor, "A");	
		ZoomableNodeContainer zoomedNode = (ZoomableNodeContainer)compEditPart.getModel();
		
		FisheyeZoom cmd = new FisheyeZoom();
		cmd.setTarget(zoomedNode);
		cmd.execute();
		
		// Reset the interval structure
		zoom.changeNodeBounds(zoomedNode, new Rectangle(180, 180, 100, 70));
		
		exportModel(editor, "ZoomingInInitial.pdf");
		
		cmd = new FisheyeZoom();
		cmd.setTarget(zoomedNode);
		cmd.execute();
		
		exportModel(editor, "ZoomingInExpanded.pdf");
		
		closeEditor(editor);
		
	}
	
	
	public void testZoomingOut() throws CoreException, IOException {
		
		IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_HORIZONTAL_INTERVALS, true);
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_VERTICAL_INTERVALS, true);
		
		store.setValue(IPreferenceNameConstants.ADORA_VIEW_SHADE_HIERARCHY, false);
		
		AdoraZoomAdapter zoom = AdoraZoomAdapter.getInstance();
		
		// Create a simple model
		AdoraModel model = new AdoraModel();
		
		Component root = addComponent(model, new Rectangle(5, 5, 650, 420), "");
		Component a = addComponent(root, new Rectangle(40, 180, 350, 200), "A");
		addComponent(root, new Rectangle(110, 40, 200, 70), "B");
		addComponent(root, new Rectangle(450, 250, 140, 70), "C");
		
		addComponent(a, new Rectangle(50, 50, 50, 50), "");
		
		// Initialize the editor
		IFile file = storeModel(model);
		IEditorPart editor = openEditor(file);
		
		EditPart compEditPart = getEditPartByName(editor, "A");	
		ZoomableNodeContainer zoomedNode = (ZoomableNodeContainer)compEditPart.getModel();
		zoom.changeNodeBounds(zoomedNode, new Rectangle(40, 180, 350, 200));
		
		exportModel(editor, "ZoomingOutInitial.pdf");
		
		FisheyeZoom cmd = new FisheyeZoom();
		cmd.setTarget(zoomedNode);
		cmd.execute();
		
		exportModel(editor, "ZoomingOutCollapsed.pdf");
		
		closeEditor(editor);
	}
	
}
