package ch.unizh.ifi.adora.figureexporter.thesis;

import java.io.IOException;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.gef.EditPart;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.ui.IEditorPart;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.figureexporter.FigureExporter;
import ch.unizh.ifi.adora.model.commands.FisheyeZoom;
import ch.unizh.ifi.adora.model.geometry.Point;
import ch.unizh.ifi.adora.model.geometry.Rectangle;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.ui.preferences.IPreferenceNameConstants;

public class ZoomFundamentals extends FigureExporter {
	
	public void testExample() throws CoreException, IOException {
		
		// Create a simple model
		AdoraModel model = new AdoraModel();
		
		Component a = addComponent(model, new Rectangle(10, 10, 870, 400), "A");
		addComponent(a, new Rectangle(80, 100, 280, 180), "B");
		Component c = addComponent(a, new Rectangle(440, 100, 280, 180), "C");
		addComponent(c, new Rectangle(70, 60, 140, 80), "D");
	
		// Initialize the editor
		IFile file = storeModel(model);
		IEditorPart editor = openEditor(file);
		
		// Export an image with the initial situation
		exportModel(editor, "Test2.pdf");
		
		EditPart compEditPart = getEditPartByName(editor, "C");	
		ZoomableNodeContainer zoomedNode = (ZoomableNodeContainer)compEditPart.getModel();
		
		FisheyeZoom cmd = new FisheyeZoom();
		cmd.setTarget(zoomedNode);
		cmd.execute();
		
		IPreferenceStore store = AdoraPlugin.getDefault().getPreferenceStore();
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_HORIZONTAL_INTERVALS, true);
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_VERTICAL_INTERVALS, true);
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_ZOOM_HOLES, true);
		
		exportModel(editor, "Test3.pdf");
		
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_HORIZONTAL_INTERVALS, false);
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_VERTICAL_INTERVALS, false);
		store.setValue(IPreferenceNameConstants.ADORA_REPRESENTATION_LAYOUT_ZOOM_HOLES, false);
		
		closeEditor(editor);
		
	}
	
	
	public void testZoomExample() throws CoreException, IOException {
		
		AdoraModel model = new AdoraModel();
		
		// Add the nodes
		Component a = addComponent(model, new Rectangle(10, 10, 100, 70), "A");
		Component b = addComponent(model, new Rectangle(60, 130, 320, 310), "B");
		Component c = addComponent(model, new Rectangle(330, 10, 100, 70), "C");
		Component d = addComponent(b, new Rectangle(20, 50, 160, 230), "D");
		Component e = addComponent(b, new Rectangle(200, 60, 100, 70), "E");
		Component f = addComponent(b, new Rectangle(200, 200, 100, 70), "F");
		Component g = addComponent(d, new Rectangle(30, 40, 100, 70), "G");
		Component h = addComponent(d, new Rectangle(30, 140, 100, 70), "H");
		
		// Mark some of the nodes as abstracted
		a.setPartial(true);
		c.setPartial(true);
		e.setPartial(true);
		f.setPartial(true);
		
		// Add the associations
		addAssociation(a, new Point(95, 35), c, new Point(5, 35));
		addAssociation(a, new Point(20, 65), g, new Point(5, 35));
		addAssociation(c, new Point(80, 65), e, new Point(95, 35));
		addAssociation(e, new Point(50, 65), f, new Point(50, 5));
		addAssociation(h, new Point(95, 35), f, new Point(5, 35));
		addAssociation(g, new Point(50, 65), h, new Point(50, 5));
		
		
		// Initialize the editor
		IFile file = storeModel(model);
		IEditorPart editor = openEditor(file);
		
		exportModel(editor, "ZoomExpanded.pdf");
		
		EditPart compEditPart = getEditPartByName(editor, "B");	
		ZoomableNodeContainer zoomedNode = (ZoomableNodeContainer)compEditPart.getModel();
		
		FisheyeZoom cmd = new FisheyeZoom();
		cmd.setTarget(zoomedNode);
		cmd.execute();
		
		exportModel(editor, "ZoomAbstracted.pdf");
		
		closeEditor(editor);
		
	}

}
