package ch.unizh.ifi.adora.linerouting.algorithm;

/**
 * This class holds various constants for the line routing. The constants that
 * are not final can be changed via the eclipse preferences.
 *
 * @author				Tobias Reinhard
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				2007-12-20 TR First Version
 * @version				$Date: 2009-11-03 09:26:09 +0100 (Tue, 03 Nov 2009) $, $Author: treinhard $, $Revision: 4929 $
 *
 */
public class RoutingConstants {
	
	/**
	 * The cost factor inside a node. The distance by which the line passes
	 * through the node is multiplied by this factor.
	 */
	public static float ROUTING_NODE_COSTFACTOR = 1.8f;
	
	/**
	 * The cost factor inside an abstract node. The distance by which the line 
	 * passes through the abstracted node is multiplied by this factor.
	 */
	public static float ROUTING_ABSTRACT_NODE_COSTFACTOR = 1.0f;
	
	/**
	 * The line cost factor. The cost factor inside a node. The distance by 
	 * which the line passes through the line is multiplied by this factor.
	 * 
	 */
	public static float ROUTING_LINE_COSTFACTOR = 60.0f;
	
	/**
	 * Set to true if the routing solutions (paths) should be printed on the
	 * console.
	 */
	public static boolean SHOW_ROUTING_SOLUTIONS = false;
	
	
	/**
	 * Geometric constant that represents to the left of something.
	 */
	public static final int WEST = 0;

	
	/**
	 * Geometric constant that represents at the top of something.
	 */
    public static final int NORTH = 1;

    
    /**
	 * Geometric constant that represents to the right of something.
	 */
    public static final int EAST = 2;

    
    /**
	 * Geometric constant that represents at the bottom of something.
	 */
    public static final int SOUTH = 3;
	
	
	/**
	 * Maximal width or height to center a line if the line is routed around an
	 * obstacle
	 */
	public static final int MAX_SIZE_TO_CENTER = 100;

	
	/**
	 * Distance to the node if the line isn't centered anymore
	 */
	public static final int DIST_TO_NODE = 50;
	
	/**
	 * Minimal distance between a line and the border of one of its source's
	 * or target's ancestors a can pass by.
	 */
	public static final int DIST_TO_PARENT_BORDER = 3;
	
	/**
	 * Set to true if the router should additionally automatically place the
	 * line labels. 
	 */
	public static boolean PLACE_LABELS = false;

}
