package ch.unizh.ifi.adora.linerouting.factors;

import java.awt.Point;
import java.util.List;

/**
 * Criteria that calculates the total length of the line.
 *
 * @author				Tobias Reinhard
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				2008-01-11 TR First Version
 * @version				$Date: 2008-08-19 15:49:22 +0200 (Tue, 19 Aug 2008) $, $Author: treinhard $, $Revision: 3813 $
 * @responsibilities	Calculates the total length of the line
 *
 */
public class LengthFactor extends InfluenceFactor {

	/**
	 * Default Constructor
	 *
	 */
	public LengthFactor() {
		
		super("Length", 1.0);
		
	}
	
	/**
	 * Method that returns the calculated value for the routing given as argument
	 * <i>candidate</i>. A lower value stands for a better solution (i.e. the
	 * line is shorter.
	 * 
	 * @param candidate the line routing to calculate the value for
	 * @return the total length of the line multiplied by a constant factor
	 */
	public double getValue(List<Point> cand) {
		
		int cost = 0;

		Point v1, v2;
		v1 = cand.get(0);
		
		// Calculates the costs according to the distance of the segments
		for (int j = 0; j < cand.size(); j++) {

			v2 = cand.get(j);

			double distance = v1.distance(v2);
			cost += factor * distance;

			v1 = v2;

		}
		
		return cost;
		
	}

}
