package ch.unizh.ifi.adora.linerouting.factors;

import java.awt.Point;
import java.util.List;

/**
 * Criteria that evaluates the uniformity of the line's segments (i.e. ratio
 * between the longest and shortest segment of the line. 
 *
 * @author				Tobias Reinhard
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				2008-01-11 TR First Version
 * @version				$Date: 2008-01-11 15:52:32 +0100 (Fri, 11 Jan 2008) $, $Author: treinhard $, $Revision: 3449 $
 * @responsibilities	Criteria that evaluates the uniformity of the line segments' length
 *
 */
public class UniformityFactor extends InfluenceFactor {
	
	/**
	 * Constructor
	 *
	 */
	public UniformityFactor() {
		
		super("Uniformity", 1.0);
		
	}
	
	/**
	 * Returns the calculated value for the routing given as argument
	 * <i>candidate</i>. A lower return value means that the uniformity of
	 * the line is better.
	 * 
	 * @param candidate the line routing to calculate the value for
	 * @return the value indicating the uniformity of the line.
	 */
	public double getValue(List<Point> cand) {
		
		Point v1, v2;
		v1 = cand.get(0);

		double shortest = Double.MAX_VALUE;
		double longest = Double.MIN_VALUE;

		// Calc the costs according to the distance of the segments
		for (int j = 0; j < cand.size(); j++) {

			v2 = cand.get(j);

			double distance = v1.distance(v2);

			if (distance > 0.0 && distance < shortest) {
				shortest = distance;
			}
			if (distance > longest) {
				longest = distance;
			}

			v1 = v2;

		}

		return factor * (longest / shortest);
	}
	
}
