package ch.unizh.ifi.adora.linerouting.factors;

import java.awt.Point;
import java.util.List;

/**
 * Criteria that counts the number of bendpoints in the line.
 *
 * @author				Tobias Reinhard
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				2008-01-11 TR First Version
 * @version				$Date: 2008-01-11 15:52:32 +0100 (Fri, 11 Jan 2008) $, $Author: treinhard $, $Revision: 3449 $
 * @responsibilities	Counts the number of bendpoints in the line
 *
 */
public class VertexFactor extends InfluenceFactor {

	/**
	 * Default constructor
	 *
	 */
	public VertexFactor() {
		
		super("# Vertices", 100.0);
		
	}
	
	
	/**
	 * Method that returns the calculated value for the routing given as argument
	 * <i>candidate</i>. A lower value means a better solution (i.e. the less
	 * bendpoints the better).
	 * 
	 * @param candidate the line routing to calculate the value for
	 * @return the number of bendpoints multiplied by a constant factor
	 */
	public double getValue(List<Point> candidate) {
		
		// Additional costs for each vertex in the line
		return factor * candidate.size();
		
	}

}
