package ch.unizh.ifi.adora.linerouting.interfaces;

import java.awt.Point;
import java.util.List;

/**
 * Interface that should be implemented by the lines that have to be routed
 * by the tile maze router.
 *
 * @author				Tobias Reinhard
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				2007-03-21 TR First Version
 * @version				$Date: 2009-12-11 14:55:00 +0100 (Fri, 11 Dec 2009) $, $Author: treinhard $, $Revision: 4986 $
 *
 */
public interface IRoutingLine {

	/**
	 * Returns the node from which the line originates.
	 * 
	 * @return the source node
	 */
	public IRoutingNode getRoutingSource();
	
	
	/**
	 * Returns the node where the line ends.
	 * 
	 * @return the target node
	 */
	public IRoutingNode getRoutingTarget();
	
	
	/**
	 * Returns a list containing all the bendpoints inside the line. The
	 * coordinates of the bendpoints are relative the the first common
	 * ancestor of the source and target node.
	 * 
	 * @return the list containing all the bendpoints in the line
	 */
	public List<Point> getRoutingBendpoints();
	
	
	/**
	 * Used by the routing algorithm to set the new bendpoints after the line
	 * has been routed. The points have coordinates that are relative to the 
	 * first common ancestor of the source and the target node of this line.
	 * 
	 * @param bendpoints the new bendpoints
	 */
	public void setRoutingBendpoints(List<Point> bendpoints);
	
	
	/**
	 * Returns the start point of the line. The coordinates are relative to
	 * the source node.
	 * 
	 * @return the start point of the line
	 */
	public Point getRoutingStartPoint();
	
	
	/**
	 * Returns the end point of the line. The coordinates are relative to the
	 * target node.
	 * 
	 * @return the end point of the line
	 */
	public Point getRoutingEndPoint();
	
	
	/**
	 * Returns list containing all the line labels of this line
	 * 
	 * @return list containing the line labels
	 */
	public List<IRoutingLineLabel> getLineLabels();
	
}
