package ch.unizh.ifi.adora.linerouting.interfaces;

import java.awt.Rectangle;

/**
 * Interface that has to be implemented by a line label whose place and size
 * should be calculated by the routing mechanism.
 * 
 * @author				Tobias Reinhard
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				2009-10-27 TR First Version
 * @version				$Date: 2009-08-17 09:29:40 +0200 (Mon, 17 Aug 2009) $, $Author: treinhard $, $Revision: 4672 $
 *
 */
public interface IRoutingLineLabel {
	
	/**
	 * Returns the current bounds of this label. The coordinates
	 * must be relative to the the first common ancestor of the 
	 * source and target node of the line this label belongs to.
	 * 
	 * @return current bounds of the label
	 */
	public Rectangle getRoutingBounds();
	
	/**
	 * Sets the bounds (i.e., position and size) of this label to
	 * the values given as argument. The coordinates must be relative 
	 * to the the first common ancestor of the source and target node 
	 * of the line this label belongs to.
	 * 
	 * @param bounds the new bounds of the label
	 */
	public void setRoutingBounds(Rectangle bounds);

	/**
	 * Returns the real width this label has if the maximal width is the
	 * one given as argument. This is used by the router while looking for 
	 * a suitable place for the label.
	 * 
	 * @param maxWidth the maximal width that can be used
	 * @return the minimal width of the label
	 */
	public int getWidth(int maxWidth);
	
	/**
	 * Returns the minimal height this label has if its width is the one
	 * given as argument. The height is usually calculated by wrapping the
	 * text in the label accordingly.
	 * 
	 * @param width the width the minimal height should be given for
	 * @return minimal height with width as constraint
	 */
	public int getMinimalHeight(int width);
	
	/**
	 * Returns the desired position of the label on the line. Method must return
	 * one of the following values:
	 * 
	 * @see ch.ifi.unizh.ch.adora.linerouting.labels.LabelConstants#START
	 * @see ch.ifi.unizh.ch.adora.linerouting.labels.LabelConstants#MIDDLE
	 * @see ch.ifi.unizh.ch.adora.linerouting.labels.LabelConstants#END
	 * 
	 * @return desired position of the label
	 */
	public int getLabelPosition();
	
}
