package ch.unizh.ifi.adora.linerouting.interfaces;

import java.awt.Rectangle;
import java.util.Iterator;

/**
 * Interface that should be implemented by the nodes that are the source or
 * target of a line that has to be routed or potential obstacles during the
 * routing.
 *
 * @author				Tobias Reinhard
 * @copyright			Requirements Engineering Research Group<br>
 *						Department of Informatics<br>
 *						University of Zurich<br>
 * @history				2007-03-21 TR First Version
 * @version				$Date: 2009-10-30 14:58:38 +0100 (Fri, 30 Oct 2009) $, $Author: treinhard $, $Revision: 4917 $
 *
 */
public interface IRoutingNode {

	/**
	 * Returns the parent of this node or null if this is the root node.
	 * 
	 * @return the parent node
	 */
	public IRoutingNode getRoutingParent();
	
	/**
	 * Returns all the children of this node. The method returns an iterator
	 * over an empty collection if this node doesn't have any children.
	 * 
	 * @return the contained children
	 */
	public Iterator<IRoutingNode> getRoutingChildren();
	
	
	/**
	 * Returns the bounds of this node. The coordinates of the bounds have to
	 * be relative to the parent of this node.
	 * 
	 * @return the relative bounds of the node
	 */
	public Rectangle getRoutingBounds();
	
	
	/**
	 * Returns true if the node is currently zoomed in and false if it is
	 * zoomed out. This method is used to calculate a specific node cost factor
	 * for abstract nodes. The method should return true if zooming is not
	 * supported.
	 * 
	 * @return true if the node is zoomed in, false otherwise
	 */
	public boolean isZoomed();
	
}
