package ch.unizh.ifi.adora.linerouting.tilestructure;

import java.awt.Point;
import java.awt.Rectangle;
import java.util.Iterator;
import java.util.List;

import ch.unizh.ifi.adora.linerouting.interfaces.IRoutingLine;
import ch.unizh.ifi.adora.linerouting.interfaces.IRoutingNode;

/**
 * Simple rectangle figure that implements the IRoutingObstace marker interface
 * and is therefore a routing obstacle. The class is used for reflexive transitions
 * for which the start/end node is splitted and two new figures have to be inserted.
 * 
 * @author Tobias Reinhard
 * 
 * @version $Date: 2007-12-20 13:31:31 +0100 (Thu, 20 Dec 2007) $, $Author: treinhard $, $Revision: 3427 $
 */
public class DummyNode implements IRoutingNode {

	private Rectangle bounds;
	
	private IRoutingNode parent;
	
	public DummyNode(IRoutingNode parent) {
		bounds = new Rectangle();
		this.parent = parent;
	}
	
	public Rectangle getRoutingBounds() {
		return bounds;
	}

	public Iterator<IRoutingNode> getRoutingChildren() {
		return null;
	}

	public IRoutingNode getRoutingParent() {
		return parent;
	}

	public List<IRoutingLine> getRoutingSourceLines() {
		return null;
	}

	public List<IRoutingLine> getRoutingTargetLines() {
		return null;
	}
	
	public void setBounds(Point topLeft, Point bottomRight) {
		
		int width = bottomRight.x - topLeft.x;
		int height = bottomRight.y - topLeft.y;
		
		bounds.setBounds(topLeft.x, topLeft.y, width, height);
		
	}
	
	
	public String toString() {
		
		return bounds.toString();
		
	}

	public boolean isZoomed() {
		
		return true;
		
	}

}
