package ch.unizh.ifi.adora.linerouting.tilestructure;

/**
 * Exception to indicate an invalid layout (e.g. overlapping of two nodes). This
 * exception is thrown during the construction of the tile structure if the
 * structure can't be constructed due to an invalid layout.
 *
 * @author		       Tobias Reinhard
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             2006-01-03 TR First Version
 * @version             $Date: 2008-10-27 10:12:16 +0100 (Mon, 27 Oct 2008) $, $Author: jehle $, $Revision: 3853 $
 * @responsibilities	   Exception to indicate an invalid layout (e.g. overlapping nodes)
 *
 */
public class InvalidLayoutException extends Exception {

	/**
	 * Version id for the serialization
	 */
	private static final long serialVersionUID = -4207637550374085521L;
	
	/**
	 * Additional information why the problem occured
	 */
	private String message;

	
	/**
	 * Constructor for the exception.
	 * 
	 * @param tile the tile where the problem occured
	 * @param message additional informations why the problem occured
	 * 
	 * @pre true
	 * @post true
	 */
	public InvalidLayoutException(String message) {
		this.message = message;
	}
	
	
	/**
	 * Method to print a string representation of the exception
	 * 
	 * @return string representation of the exception
	 * 
	 * @pre true
	 * @post true
	 * 
	 */
	public String toString() {
		return message;
	}
	
}
