package ch.unizh.ifi.adora.linerouting.tilestructure;

import java.awt.Point;
import java.awt.Rectangle;
import java.util.Iterator;
import java.util.List;

import ch.unizh.ifi.adora.linerouting.interfaces.IRoutingLine;
import ch.unizh.ifi.adora.linerouting.interfaces.IRoutingLineLabel;
import ch.unizh.ifi.adora.linerouting.interfaces.IRoutingNode;
import ch.unizh.ifi.adora.linerouting.utils.HierarchyUtilities;

public class LabelTileStructure extends LineTileStructure {

	/**
	 * Constructor to build a new corner stitching structure for the node (i.e. for all
	 * the children of this node).
	 * 
	 * @param node the node the structure has to be constructed for
	 * 
	 */
	public LabelTileStructure(IRoutingNode node) throws InvalidLayoutException {	
		super(node);	
	}
	
	/**
	 * Constructor to build a new corner stitching structure for a connection (i.e. for the
	 * parent node of the connection).
	 * 
	 * @param line the line the structure has to be constructed for
     * @param lines the existing lines that are obstacles for the new line
	 * 
	 */
	public LabelTileStructure(IRoutingLine line, List<IRoutingLine> lines) throws InvalidLayoutException {	
		super(line);
        
        Iterator<IRoutingLine> iter = lines.iterator();
        while(iter.hasNext()) {
            IRoutingLine current = iter.next();
            if(current != line) {
            	insertLine(current);
            }
        }
        
	}
	
    /**
     * Method to insert the line given as parameter into the corner stitching structure.
     * 
     * @param line the line to insert into the structure
     * 
     */
	protected void insertLine(IRoutingLine line) throws InvalidLayoutException {
		
		if(!isOrthogonal(line)) {
			return;
		}
		
		super.insertLine(line);
		
		for(IRoutingLineLabel label: line.getLineLabels()) {
			Rectangle bounds = label.getRoutingBounds();
			
			// Check whether the line label lies within the parent node of the line
			// we're building the structure for.
			IRoutingNode lineParent = HierarchyUtilities.getInstance().commonAncestor(line.getRoutingSource(), line.getRoutingTarget());
			
			Point absLineParentLoc = HierarchyUtilities.getInstance().getAbsoluteLocation(lineParent);
			Point absStructureLoc = HierarchyUtilities.getInstance().getAbsoluteLocation(parentNode);
			Rectangle labelBounds = new Rectangle(absLineParentLoc.x + bounds.x, absLineParentLoc.y + bounds.y, bounds.width, bounds.height);
			
			//labelBounds.translate(absLineParentLoc.x, absLineParentLoc.y);
			labelBounds.translate(-absStructureLoc.x, -absStructureLoc.y);
			
			// Label overlaps with something else
			// TODO: Relax
			if(!isFreeSpace(labelBounds)) {
				continue;
			}
			
			Point p1 = labelBounds.getLocation();
			Point p2 = new Point(p1.x + labelBounds.width, p1.y + labelBounds.height);
			LabelTile tile = new LabelTile(p1, p2, null, null, null, null,
                    null, null, null, null);
			insert(tile);
		}
		
	}
	
}
