package ch.unizh.ifi.adora.linerouting.tilestructure;

import java.util.Iterator;

/**
 * This class is an implementation of an {@link java.util.Iterator} that iterates
 * over all the left neighbors of a tile.
 * 
 * @author              Tobias Reinhard (TR)
 * @copyright           Requirements Engineering Research Group
 *                      Department of Computer Science
 *                      University of Zurich
 * @history
 * 2005-03-04           TR First version
 * 
 * @version             $Date: 2008-10-27 10:12:16 +0100 (Mon, 27 Oct 2008) $, $Author: jehle $, $Revision: 3853 $
 * @responsibilities    Iterates over all the left neighbors of a tile.
 */
class LeftNeighborsIterator implements Iterator<Tile> {

    /**
     * The tile over whose neighbors this iterator iterates
     */
    private Tile tile;

    /**
     * The current tile (neighbor) of the iteration
     */
    private Tile current;

    
    /**
     * Constructor for an iterator that iterates over all the left neighbors
     * of the tile given as argument.
     * 
     * @pre tile != null
     * @post true
     * 
     * @param tile the tile over whose left neighbors the iterator iterates
     */
    public LeftNeighborsIterator(Tile tile) {
        this.tile = tile;
        current = tile.getTl();
    }

    
    /**
     * Returns true if the iteration has more elements. This method can return
     * false during the first invocation if the tile doesn't have any top
     * neighbors (i.e. the tile lies at the border of the tile structure).
     * 
     * @pre true
     * @post true
     * 
     * @return true if the iteration has more elements
     */
    public boolean hasNext() {
        return current != null;
    }

    
    /**
     * Returns the next element in the iteration and advances the iteration to
     * the next element. Returns null if the iteration doesn't have a next element.
     * 
     * @pre true
     * @post true
     * 
     * @return the next element in the iteration
     */
    public Tile next() {
        if (current != null) {
            Tile t = current;
            if (current == tile.getBl()) {
                current = null;
            } else {
                current = current.getRb();
            }
            return t;
        }
        return null;
    }

    
    /**
     * Standard iterator method to remove from the underlying collection the 
     * last element returned by the  iterator. This method doesn't do anything
     * in this implementation because the removal of a tile would destroy the
     * whole tile structure and doesn't make any sense.
     * 
     * @pre true
     * @post true
     * 
     */
    public void remove() {
    }
}
