package ch.unizh.ifi.adora.manualtest.actions;


import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowPulldownDelegate;

import ch.unizh.ifi.adora.manualtest.operations.AbstractOperations;
import ch.unizh.ifi.adora.manualtest.operations.StandardOperations;
import ch.unizh.ifi.adora.manualtest.testcases.AbstractTestCase;
import ch.unizh.ifi.adora.manualtest.testcases.ElevatorTestCase;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.parts.AdoraRootEditPart;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;


/**
 * Runs the test.     
 *      
 * @author		        Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             03.05.2006 SM First Version
 * @version             $Date: 2006-10-27 11:45:03 +0200 (Fri, 27 Oct 2006) $, $Author: smeier $, $Revision: 2336 $
 */
public class TestRunner implements IWorkbenchWindowPulldownDelegate, SelectionListener{

	MenuItem elevatorItem;
	
	private static AbstractOperations operations;
	
	/**
     * Window the Adora editor is part of.
     * Used to 
     */
	private IWorkbenchWindow window;

	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchWindowActionDelegate#dispose()
	 */
	public void dispose() {
		// TODO Auto-generated method stub
	}

    /*
     *  (non-Javadoc)
     * @see org.eclipse.ui.IWorkbenchWindowActionDelegate#init(org.eclipse.ui.IWorkbenchWindow)
     */
	public void init(IWorkbenchWindow window) {
		this.window = window;
		operations = null;
	}
	
	/**
	 * Runs the test.
	 */
	public void run(IAction action) {
		IWorkbenchPage page = window.getActivePage();
        try {
            IEditorPart editor = page.getActiveEditor();
            if (editor instanceof AdoraModelEditor) {
                AdoraModelEditor ame = (AdoraModelEditor)editor;
                AdoraRootEditPart arep = (AdoraRootEditPart)ame.getViewer().getRootEditPart();
                arep.getZoomManager().setZoom(0.5);
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
	}

	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action.IAction, org.eclipse.jface.viewers.ISelection)
	 */
	public void selectionChanged(IAction action, ISelection selection) {
		 
	}
	
    /**
     * Return the AdoraModelEditorPaletteFactory on which icons can be set hidden.
     * 
     * @pre       true
     * @post      check if returned value == null
     * @obligation 
     * @return AdoraModelEditorPaletteFactory
     */
    protected AdoraModel getAdoraModel(){
        IWorkbenchPage page = window.getActivePage();
        try {
            IEditorPart editor = page.getActiveEditor();
            if (editor instanceof AdoraModelEditor) {
                return ((AdoraModelEditor)editor).getModel();
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return null;
    }

	public Menu getMenu(Control parent) {
		Menu menu = new Menu(parent);
		
		elevatorItem = new MenuItem(menu, 0);
		elevatorItem.setText("Elevator");
		elevatorItem.addSelectionListener(this);
		return menu;
	}

	public void widgetSelected(SelectionEvent e) {
		AbstractTestCase test;
		if(e.getSource().equals(elevatorItem)){
			test = new ElevatorTestCase(getAdoraModel());
			test.run();
			operations = new StandardOperations(getAdoraModel());
		}
	}

	public void widgetDefaultSelected(SelectionEvent e) {
	}
	
	public static AbstractOperations getOperations(){
		return operations;
	}
}