package ch.unizh.ifi.adora.manualtest.operations;

import ch.unizh.ifi.adora.model.commands.representation.RepresentationStateChange;
import ch.unizh.ifi.adora.model.representation.Actor;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.Association;
import ch.unizh.ifi.adora.model.representation.Scenario;

/**
 * Executes operations on an {@link AdoraModel}.
 * 
 * @author		        Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of yInformatics<br>
 *                      University of Zurich<br>
 * @history             04.05.2006 SM First Version
 * @version             $Date: 2006-07-20 14:14:35 +0200 (Thu, 20 Jul 2006) $, $Author: vigan $, $Revision: 1881 $
 */
public class StandardOperations extends AbstractOperations{

	/**
	 * Stores the {@link AdoraModel} on which the operations will be applied. 
	 */
	private AdoraModel model;
	
	/**
	 * Constructs a new StandardOperations object.
	 * 
	 * @param model The {@link AdoraModel} on which the operations will be applied.
	 */
	public StandardOperations(AdoraModel model){
		super(model);
	}
	
	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.manualtest.operations.AbstractOperations#init()
	 */
	protected void init(){

		RepresentationStateChange repCmd = new RepresentationStateChange(Actor.class, model);
        commands.add(new CommandObject(repCmd,"Hides all Actors"));

        RepresentationStateChange repCmd2 = new RepresentationStateChange(Association.class, model);
        commands.add(new CommandObject(repCmd2,"Hides all Associations"));

        RepresentationStateChange repCmd3 = new RepresentationStateChange(Actor.class, model);
		commands.add(new CommandObject(repCmd3,"Shows all Actors"));

        RepresentationStateChange repCmd4 = new RepresentationStateChange(Scenario.class, model);
		commands.add(new CommandObject(repCmd4,"Hides all Scenarios"));
	}
	
	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.manualtest.operations.AbstractOperations#getLabelForNextOp()
	 */
	public String getLabelForNextOp(){
		if(currentOpperation+1<commands.size()){
			return commands.get(currentOpperation+1).label;
		}
		return "no more Commands";
	}
	
	/*
	 *  (non-Javadoc)
	 * @see ch.unizh.ifi.adora.manualtest.operations.AbstractOperations#apply()
	 */
	public void apply(){
		if(currentOpperation<commands.size()){
			commands.get(currentOpperation).cmd.execute();
		}
		currentOpperation++;
	}
}