package ch.unizh.ifi.adora.manualtest.testcases;


import ch.unizh.ifi.adora.model.commands.AssociationCreate;
import ch.unizh.ifi.adora.model.commands.NodeCreate;
import ch.unizh.ifi.adora.model.commands.ScenarioConnectionCreate;
import ch.unizh.ifi.adora.model.commands.TransitionCreate;
import ch.unizh.ifi.adora.model.geometry.Point;
import ch.unizh.ifi.adora.model.geometry.Rectangle;
import ch.unizh.ifi.adora.model.representation.Actor;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.Association;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.model.representation.State;
import ch.unizh.ifi.adora.model.representation.Transition;

/**
 * Build an elevator.      
 *
 * @author		        Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             03.05.2006 SM First Version
 * @version             $Date: 2006-05-05 09:31:50 +0200 (Fri, 05 May 2006) $, $Author: vigan $, $Revision: 1307 $
 */
public class ElevatorTestCase extends AbstractTestCase {

	
	//Global components
	Component elevator;
	
	Component keyPad;
	
	public ElevatorTestCase(AdoraModel model){
		super(model);
	}
	
	/**
	 * Builds the elevator.
	 * 
	 * @pre       true
	 * @post      true
	 * @obligation
	 */
	public void run(){
		buildUIElements();
		buildDoorManagement();
		buildElevatorMotor();
	}
	
	
	private void buildElevatorMotor(){
		NodeCreate nodeCreateCmd;
		
		//Components
		
		Component elevatorMotor = new Component();
		elevatorMotor.setName("ElevatorMotorManagement");
		nodeCreateCmd = new NodeCreate(elevatorMotor,elevator,new Rectangle(0,200,100,70));
		nodeCreateCmd.execute();
		
		createAssociation(keyPad,elevatorMotor,"floor n","");
		
		
		State idle = new State();
		idle.setName("Idle");
		nodeCreateCmd = new NodeCreate(idle,elevatorMotor,new Rectangle(0,0,100,70));
		nodeCreateCmd.execute();
		
		State moving = new State();
		moving.setName("Moving");
		nodeCreateCmd = new NodeCreate(moving,elevatorMotor,new Rectangle(0,160,100,70));
		nodeCreateCmd.execute();
		
		//Transitions

		createTransition(idle,moving,"receive floor n");
		createTransition(moving,idle,"revceive stopped");
		
		Actor motor = new Actor();
		motor.setName("ElevatorMotor");
		nodeCreateCmd = new NodeCreate(motor,model,new Rectangle(0,450,100,70));
		nodeCreateCmd.execute();
		
		//Associations

		createAssociation(motor, elevatorMotor,"stopped", "move n-actualFloor floors");

	
	}
	
	/**
	 * States
		1.Create an Abstract Object named DoorMangement inside Elevator.
		2.Create a State named DoorClosed
		3.DoorClosed = StartState
		4.Create a State named DoorOpend
		5.Create a State named OpeningDoor
		6.Create a State named ClosingDoor
		7.Create a Transition from DoorOpend to ClosingDoor, label = receive closeDoor
		8.Create a Transition from OpeningDoor to DoorOpend
		9.Create a Transition from ClosingDoor to OpeningDoor, label = receive openDoor
		10.Create a Transition from DoorClosed to OpeningDoor, label = receive openDoor
		11.Create a Transition from ClosingDoor to DoorClosed
		
	   Scenarios
		1.Create a Scenario named Poll Door
		2.Create a Scenario named Repeat, iterator = true
		3.Create a ScenarioConnection,  Parent =  Poll Door, Child = Repeat;
		4.Create a Scenario named Request Sensor Data,Type = Sequence, SeqNo = 1, 
		5.Create a ScenarioConnection,  Parent =  Repeat, Child = Request Sensor Data;
		6.Create a Scenario named getSensorData,Type = Sequence, SeqNo = 2
		7.Create a ScenarioConnection,  Parent =  Repeat, Child = getSensorData;
		8.Create a Scenario named doorCloseable,Type = Alternative, guard=space sufficient, Transform input closeDoor
		9.Create a ScenarioConnection,  Parent =  getSensorData, Child = doorCloseable
		10.Create a Scenario named doorBlocked,Type = Alternative, guard=space insufficient,Transform input openDoor
		11.Create a ScenarioConnection,  Parent =  getSensorData, Child = doorBlocked
		
		
	 * @pre       true
	 * @post      true
	 * @obligation
	 */
	private void buildDoorManagement(){
		NodeCreate nodeCreateCmd;
		
		//Components
		
		Component doorMangement = new Component();
		doorMangement.setName("DoorMangement");
		nodeCreateCmd = new NodeCreate(doorMangement,elevator,new Rectangle(0,0,100,70));
		nodeCreateCmd.execute();
		
		
		createAssociation(keyPad,doorMangement,"tryCloseDoor","");
		createAssociation(keyPad,doorMangement,"tryOpenDoor","");
		
		//States
		
		State doorOpend = new State();
		doorOpend.setName("DoorOpend");
		nodeCreateCmd = new NodeCreate(doorOpend,doorMangement,new Rectangle(0,0,100,70));
		nodeCreateCmd.execute();
		
		State doorClosed = new State();
		doorClosed.setName("DoorClosed");
		nodeCreateCmd = new NodeCreate(doorClosed,doorMangement,new Rectangle(200,0,100,70));
		nodeCreateCmd.execute();
		
		State openingDoor = new State();
		openingDoor.setName("OpeningDoor");
		nodeCreateCmd = new NodeCreate(openingDoor,doorMangement,new Rectangle(0,200,100,70));
		nodeCreateCmd.execute();
		
		State closingDoor = new State();
		closingDoor.setName("ClosingDoor");
		nodeCreateCmd = new NodeCreate(closingDoor,doorMangement,new Rectangle(200,200,100,70));
		nodeCreateCmd.execute();
		
		//Transitions

		createTransition(doorOpend,closingDoor,"receive closeDoor");
		createTransition(openingDoor,doorOpend,"");
		createTransition(closingDoor,openingDoor,"[EMM.isActualState(\"Idle\")] receive openDoor");
		createTransition(doorClosed,openingDoor,"[EMM.isActualState(\"Idle\")] receive openDoor");
		createTransition(closingDoor,doorClosed,"");
		
		//Scenarios
	
		Scenario pollDoorAction = new Scenario();
		pollDoorAction.setName("Poll Door");
		nodeCreateCmd = new NodeCreate(pollDoorAction,doorMangement,new Rectangle(400,0,100,70));
		nodeCreateCmd.execute();
		
		Scenario repeatAction = new Scenario();
		repeatAction.setName("Repeat Action");
		repeatAction.setIterator(true);
		repeatAction.setScenarioType(Scenario.SEQUENCE);	
		repeatAction.setSeqNr(1);
		nodeCreateCmd = new NodeCreate(repeatAction,doorMangement,new Rectangle(400,80,100,70));
		nodeCreateCmd.execute();
		
		createScenarioConnection(pollDoorAction,repeatAction);
		
		Scenario requestSensorDataAction = new Scenario();
		requestSensorDataAction.setName("Request Sensor Data");
		requestSensorDataAction.setScenarioType(Scenario.SEQUENCE);
		requestSensorDataAction.setSeqNr(1);
		nodeCreateCmd = new NodeCreate(requestSensorDataAction,doorMangement,new Rectangle(400,160,100,70));
		nodeCreateCmd.execute();
		
		createScenarioConnection(repeatAction,requestSensorDataAction);
		
		Scenario getSensorDataAction = new Scenario();
		getSensorDataAction.setName("Get Sensor Data");
		getSensorDataAction.setScenarioType(Scenario.SEQUENCE);
		getSensorDataAction.setSeqNr(2);
		nodeCreateCmd = new NodeCreate(getSensorDataAction,doorMangement,new Rectangle(510,160,100,70));
		nodeCreateCmd.execute();
		
		createScenarioConnection(repeatAction,getSensorDataAction);
		
		
		Scenario doorCloseableAction = new Scenario();
		doorCloseableAction.setName("DoorCloseable");
		doorCloseableAction.setScenarioType(Scenario.ALTERNATIV);
		doorCloseableAction.setGuard("space sufficient");
		doorCloseableAction.setTransformation("input closeDoor");
		
		nodeCreateCmd = new NodeCreate(doorCloseableAction,doorMangement,new Rectangle(510,240,100,70));
		nodeCreateCmd.execute();
		
		createScenarioConnection(getSensorDataAction,doorCloseableAction);
		
		Scenario doorBlocked = new Scenario();
		doorBlocked.setName("Door Blocked");
		doorBlocked.setScenarioType(Scenario.ALTERNATIV);
		doorBlocked.setGuard("space insufficient");
		doorBlocked.setTransformation("input openDoor");
		
		nodeCreateCmd = new NodeCreate(doorBlocked,doorMangement,new Rectangle(620,240,100,70));
		nodeCreateCmd.execute();

		createScenarioConnection(getSensorDataAction,doorBlocked);
		
		//Actor
		
		Actor sensor = new Actor();
		sensor.setName("Sensor");
		nodeCreateCmd = new NodeCreate(sensor,model,new Rectangle(0,200,100,70));
		nodeCreateCmd.execute();
		
		//Associations

		createAssociation(sensor, pollDoorAction,"request", "sensor Data");
		
	}
		
	/**
	 * 
	 	1.Create an Abstract Object named Elevator.
		2.Create an Abstract Object named ElevatorControlBoard, by copying the Elevator and pasting it into the Elevator.
		3.Create an Abstract Object named KeyPad in the ElevatorControlBoard.
		4.Create an Abstract Object named Close Button in the KeyPad
		5.Create an Abstract Object named Close Floor 0 in the KeyPad
		6.Create an Abstract Object named Close Floor 1 in the KeyPad
		7.Create an Abstract Object named Close Floor 2 in the KeyPad
		8.Create an Abstract Object named Close Floor 3 in the KeyPad
		9.Create a Scenario named CloseAction, in Close Button.
		10.Set Transform Property of CloseAction: transform input tryCloseDoor
		11.Copy Close Button, past it into  KeyPad.
		12.Replace all Close in the newly pasted structure by Open.
		13.Create a Scenario named Floor0Action, in Floor0 Button.
		14.Set Transform Property of CloseAction: transform input tryCloseDoor
		15.Create an Actor named User
		16.Connect User by an Association to CloseAction, SourceLabel=close
		17.Connect User by an Association to OpenAction, SourceLabel=open

	 * @pre       true
	 * @post      true
	 * @obligation
	 */
	private void buildUIElements(){
		NodeCreate nodeCreateCmd;
		
		//Components
		
		elevator = new Component();
		elevator.setName("Elevator");
		nodeCreateCmd = new NodeCreate(elevator,model,new Rectangle(250,250,100,70));
		nodeCreateCmd.execute();
		
		Component elevatorControlBoard = new Component();
		elevatorControlBoard.setName("ElevatorControlBoard");
		nodeCreateCmd = new NodeCreate(elevatorControlBoard,elevator,new Rectangle(0,0,100,70));
		nodeCreateCmd.execute();
		
		keyPad = new Component();
		keyPad.setName("KeyPad");
		nodeCreateCmd = new NodeCreate(keyPad,elevatorControlBoard,new Rectangle(0,0,100,70));
		nodeCreateCmd.execute();
		
		Component closeButton = new Component();
		closeButton.setName("CloseButton");
		nodeCreateCmd = new NodeCreate(closeButton,keyPad,new Rectangle(0,0,100,70));
		nodeCreateCmd.execute();

		Component openButton = new Component();
		openButton.setName("OpenButton");
		nodeCreateCmd = new NodeCreate(openButton,keyPad,new Rectangle(110,0,100,70));
		nodeCreateCmd.execute();

		Component floor0Button = new Component();
		floor0Button.setName("Floor0 Button");
		nodeCreateCmd = new NodeCreate(floor0Button,keyPad,new Rectangle(0,80,100,70));
		nodeCreateCmd.execute();

		Component floor1Button = new Component();
		floor1Button.setName("Floor1 Button");
		nodeCreateCmd = new NodeCreate(floor1Button,keyPad,new Rectangle(110,80,100,70));
		nodeCreateCmd.execute();

		Component floor2Button = new Component();
		floor2Button.setName("Floor2 Button");
		nodeCreateCmd = new NodeCreate(floor2Button,keyPad,new Rectangle(0,160,100,70));
		nodeCreateCmd.execute();
		
		Component floor3Button = new Component();
		floor3Button.setName("Floor3 Button");
		nodeCreateCmd = new NodeCreate(floor3Button,keyPad,new Rectangle(110,160,100,70));
		nodeCreateCmd.execute();
		
		//Scenarios
		
		Scenario closeAction = new Scenario();
		closeAction.setName("CloseAction");
		closeAction.setTransformation("transform input tryCloseDoor");
		nodeCreateCmd = new NodeCreate(closeAction,closeButton,new Rectangle(0,0,100,70));
		nodeCreateCmd.execute();

		Scenario openAction = new Scenario();
		openAction.setName("OpenAction");
		openAction.setTransformation("transform input tryOpenDoor");
		nodeCreateCmd = new NodeCreate(openAction,openButton,new Rectangle(0,0,100,70));
		nodeCreateCmd.execute();

		Scenario floor0Action = new Scenario();
		floor0Action.setName("Floor 0 Action");
		floor0Action.setTransformation("transform input floor0");
		nodeCreateCmd = new NodeCreate(floor0Action,floor0Button,new Rectangle(0,0,100,70));
		nodeCreateCmd.execute();

		Scenario floor1Action = new Scenario();
		floor1Action.setName("Floor 1 Action");
		floor1Action.setTransformation("transform input floor1");
		nodeCreateCmd = new NodeCreate(floor1Action,floor1Button,new Rectangle(0,0,100,70));
		nodeCreateCmd.execute();
		
		Scenario floor2Action = new Scenario();
		floor2Action.setName("Floor 2 Action");
		floor2Action.setTransformation("transform input floor2");
		nodeCreateCmd = new NodeCreate(floor2Action,floor2Button,new Rectangle(0,0,100,70));
		nodeCreateCmd.execute();
		
		Scenario floor3Action = new Scenario();
		floor3Action.setName("Floor 3 Action");
		floor3Action.setTransformation("transform input floor3");
		nodeCreateCmd = new NodeCreate(floor3Action,floor3Button,new Rectangle(0,0,100,70));
		nodeCreateCmd.execute();
		
		//Actors
		
		Actor user = new Actor();
		user.setName("User");
		nodeCreateCmd = new NodeCreate(user,model,new Rectangle(824,-28,100,70));
		nodeCreateCmd.execute();
		
		//Associations
/*
		createAssociation(user, openAction,"open", "");
		createAssociation(user, closeAction,"close", "");
		createAssociation(user, floor0Action,"floor 0", "");
		createAssociation(user, floor1Action,"floor 1", "");
		createAssociation(user, floor2Action,"floor 2", "");
		createAssociation(user, floor3Action,"floor 3", "");
*/
		createAssociation(user, openAction,"", "");
		createAssociation(user, closeAction,"", "");
		createAssociation(user, floor0Action,"", "");
		createAssociation(user, floor1Action,"", "");
		createAssociation(user, floor2Action,"", "");
		createAssociation(user, floor3Action,"", "");


	}
	
	
	private void createTransition(Node source, Node target, String transitionDesciptor){
		
		TransitionCreate transitionCreateCmd;
		transitionCreateCmd = new TransitionCreate(source); 
		transitionCreateCmd.setTarget(target);
		
		transitionCreateCmd.setTargetClickPoint(
				new Point(target.getLocation().x+target.getSize().width/2,
						target.getLocation().y));
		
		transitionCreateCmd.setSourceClickPoint(
				new Point(source.getLocation().x+source.getSize().width/2,
						source.getLocation().y));
		
		transitionCreateCmd.execute();
		
		((Transition)transitionCreateCmd.getConnection()).getLabel().setName(transitionDesciptor);
		
	}
	
	private void createAssociation(Node source, Node target, String roleFrom, String roleTo){
		
		AssociationCreate associationCreateCmd = new AssociationCreate(source);
		associationCreateCmd.setTarget(target);
		
		associationCreateCmd.setTargetClickPoint(
				new Point(target.getLocation().x+target.getSize().width/2,
						target.getLocation().y));
		
		associationCreateCmd.setSourceClickPoint(
				new Point(source.getLocation().x+source.getSize().width/2,
						source.getLocation().y));
		
		associationCreateCmd.execute();
		
		((Association)associationCreateCmd.getConnection()).getRoleFrom().setName(roleFrom);
		((Association)associationCreateCmd.getConnection()).getRoleTo().setName(roleTo);
	}

	private void createScenarioConnection(Scenario source, Scenario target){
		
		ScenarioConnectionCreate scenarioConnectionCmd = new ScenarioConnectionCreate(source);
		scenarioConnectionCmd.setTarget(target);
		
		scenarioConnectionCmd.setTargetClickPoint(
				new Point(target.getLocation().x+target.getSize().width/2,
						target.getLocation().y));
		
		scenarioConnectionCmd.setSourceClickPoint(
				new Point(source.getLocation().x+source.getSize().width/2,
						source.getLocation().y));
		
		scenarioConnectionCmd.execute();
	}	

}