package ch.unizh.ifi.adora.productline.ui.views.constraintstable;

import java.beans.PropertyChangeEvent;
import java.util.Observable;
import java.util.Observer;

import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;

import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;
import ch.unizh.ifi.adora.ui.views.ModelObserverMediator;

/**
 * This class is the {@link IStructuredContentProvider} of the
 * {@link ConstraintsViewPart}. It implements also the observer pattern, so
 * that it can listen to changes in the actual model and refresh the view.
 * 
 * @author Reinhard Stoiber
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 2008-10-16 RS First Version
 * @version 
 */
public class ConstraintsContentProvider implements IStructuredContentProvider, IPartListener, Observer {
	
	/**
     * The private class for handling the selection.
     * TODO: use this one to add constraints via context-menus, or similar..
     * 
     * @author Reinhard Stoiber
     * @copyright Requirements Engineering Research Group<br>
     *            Department of Informatics<br>
     *            University of Zurich<br>
     * @history 2008-10-16 RS First Version (preliminary)
     * @version 
     */
	private final class SelectionListener implements ISelectionListener {
		
		
		/*
         * (non-Javadoc)
         * 
         * @see org.eclipse.ui.ISelectionListener#selectionChanged(org.eclipse.ui.IWorkbenchPart,
         *      org.eclipse.jface.viewers.ISelection)
         */
		public void selectionChanged(IWorkbenchPart part, ISelection selection) {
			
			//TODO: Implementation to visually select (highlight) the corresponding join relations in the graphical model...
			
//			ArrayList<Object> toSelect = new ArrayList<Object>();
//			Iterator selectionIterator = ((IStructuredSelection) selection)
//					.iterator();
//
//			while (selectionIterator.hasNext()) {
//				Object selectedObject = selectionIterator.next();
//
//				if (selectedObject instanceof JoinRelationEditPart) {
//					JoinRelationEditPart editPart = (JoinRelationEditPart) selectedObject;
//					ProductLine decision = getProductLine(editPart
//							.getModel());
//					if (decision != null) {
//						toSelect.add(decision);
//					}
//				}
//			}
//
//			List actualList = ((IStructuredSelection) tableViewer
//					.getSelection()).toList();
//			if (alreadySelected(actualList, toSelect)) {
//				return;
//			}
//
//			tableViewer.setSelection(new StructuredSelection(toSelect));
		}

//		/**
//         * Checks, if the two lists contains the same elements.
//         * 
//         * @param actualList
//         * @param toSelect
//         * 
//         * @return true if the two lists have the same elements
//         */
//		private boolean alreadySelected(List actualList, List toSelect) {
//			if (actualList.size() != toSelect.size()) {
//				return false;
//			}
//
//			for (Object object : actualList) {
//				if (!toSelect.contains(object)) {
//					return false;
//				}
//			}
//
//			return true;
//		}
	}

	/**
     * The {@link TableViewer} of the {@link ConstraintsViewPart}.
     */
	private TableViewer tableViewer;

	/**
     * The constructor, which adds a listener to the page and sets the
     * {@link ConstraintsContentProvider#tableViewer}.
     * 
     * @param viewer
     *            The {@link TableViewer} used by the view.
     * @param page
     *            The {@link IWorkbenchPage} used.
     */
	public ConstraintsContentProvider(TableViewer viewer, IWorkbenchPage page) {
		
		ModelObserverMediator.getInstance().addObserver(this);
		page.addPartListener(this);
		this.tableViewer = viewer;

		if (page.getActivePart() instanceof AdoraModelEditor) {
			ModelObserverMediator.getInstance().setAdoraModelEditor((AdoraModelEditor) page.getActivePart());
		}
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.IContentProvider#dispose()
     */
	public void dispose() {
		ModelObserverMediator.getInstance().deleteObserver(this);
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
     */
	public Object[] getElements(Object parent) {
		try {
			return ModelObserverMediator.getInstance().getAdoraModelEditor()
					.getModel().getConstraints().toArray();
		} catch (NullPointerException e) {
			return new Object[0];
		}
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer,
     *      java.lang.Object, java.lang.Object)
     */
	public void inputChanged(Viewer v, Object oldInput, Object newInput) {
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.IPartListener#partActivated(org.eclipse.ui.IWorkbenchPart)
     */
	public void partActivated(IWorkbenchPart part) {
		if (!(part instanceof AdoraModelEditor)) {
			return;
		}

		processPart(part);

		ModelObserverMediator observer = ModelObserverMediator.getInstance();
		observer.setAdoraModelEditor((AdoraModelEditor) part);
	}

	/**
     * Saves the current page.
     */
	private IWorkbenchPage currentPage;

	/**
     * Saves a listener, which is used for the selections.
     */
	private SelectionListener listener = new SelectionListener();

	/**
     * Removes the selectionlistener from the one part and adds it to the passed
     * one.
     * 
     * @param part
     */
	private void processPart(IWorkbenchPart part) {
		if (currentPage != null) {
			currentPage.removeSelectionListener(listener);
		}

		currentPage = part.getSite().getPage();
		currentPage.addSelectionListener(listener);
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.IPartListener#partBroughtToTop(org.eclipse.ui.IWorkbenchPart)
     */
	public void partBroughtToTop(IWorkbenchPart part) {
		if (!(part instanceof AdoraModelEditor)) {
			return;
		}

		processPart(part);

		ModelObserverMediator observer = ModelObserverMediator.getInstance();
		observer.setAdoraModelEditor((AdoraModelEditor) part);
		
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.IPartListener#partClosed(org.eclipse.ui.IWorkbenchPart)
     */
	public void partClosed(IWorkbenchPart part) {
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.IPartListener#partDeactivated(org.eclipse.ui.IWorkbenchPart)
     */
	public void partDeactivated(IWorkbenchPart part) {
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.IPartListener#partOpened(org.eclipse.ui.IWorkbenchPart)
     */
	public void partOpened(IWorkbenchPart part) {
	}


	/*
     * (non-Javadoc)
     * 
     * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
     */
	public void update(Observable o, Object arg) {
		
		if (arg instanceof String
				&& ModelObserverMediator.MODEL_CHANGED.equals(arg)) {
			tableViewer.refresh();
			return;
		}

		if (!(arg instanceof PropertyChangeEvent)) {
			return;
		}

		PropertyChangeEvent event = (PropertyChangeEvent) arg;

		//TODO: update the constraints table when there were 'decisions' deleted (respectively aspects deleted, aspects changed not to be variability, ...)
		
		// Constraints
		if (AdoraModel.CONSTRAINT_ADDED.equals(event.getPropertyName())) {
			tableViewer.refresh();
		}

		if (AdoraModel.CONSTRAINT_REMOVED.equals(event.getPropertyName())) {
			tableViewer.refresh();
		}

		if (AdoraModel.CONSTRAINT_CHANGED.equals(event.getPropertyName())) {
			tableViewer.refresh();
		}
		
	}
	
}