package ch.unizh.ifi.adora.productline.ui.views.constraintstable.parser;

import org.sat4j.specs.ISolver;

import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.ConstraintConnection;
import ch.unizh.ifi.adora.model.representation.variability.Constraint;
import ch.unizh.ifi.adora.model.representation.variability.Decision;
import ch.unizh.ifi.adora.productline.ui.views.constraintstable.sat.qmcalgorithm.QMCalgorithm;

/**
 * This class - generated mostly by javaCC - is the main class for the parser
 * used to buil the {@link ConstraintConnection}s.
 * 
 *  
 * @author		        Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             2.12.2008 IV First Version
 */
public class ConstraintParser implements ConstraintsTreeConstants, ConstraintsConstants {/*@bgen(jjtree)*/

	/**
	 * The {@link Constraint} for which the {@link ConstraintConnection}s get generated
	 */
  private Constraint constraint;
  
  /**
   * The {@link AdoraModel} on which the {@link Constraint}s get checked.
   */
  private AdoraModel model;
	  
  protected JJTConstraintsState jjtree = new JJTConstraintsState();
  
  /**
   * This method is responsible for building the {@link ConstraintConnection}s
   * resulting from {@link Constraint}s on {@link Decision}s.
   * 
   * @return The root {@link ConstraintConnection}.
   */
  public ConnectableElement connectionParser() {
      
	     Infix infix = new Infix(constraint);
	     boolean run = true;
	     for (;run; jjtree.reset())
	          try {
	            switch (expr()) {
	            default:      jjtree.rootNode().jjtAccept(infix, null);
	            case 0:       return infix.getRootConnection();
	            case -1:      run = false;
	            }
	          } catch (Exception e) {
	        	  
	            e.printStackTrace(); System.exit(1);
	          }
	   return null;
  }
  
  /**
   * This method builds the {@link ISolver} of Sat4J used to 
   * determine whether the current {@link Constraint}s are satisfiable.
   * 
   * @return The {@link ISolver} of Sat4J. 
   */
  public ISolver satParser() {
      
	     SATvisitor vis = new SATvisitor(model);
	     boolean run = true;
	     for (;run; jjtree.reset())
	          try {
	        	int exprNr = expr();
	            switch (exprNr) {
	            default:      jjtree.rootNode().jjtAccept(vis, null);
	            case 0:       return vis.getSolver();
	            case -1:      run = false;
	            }
	          } catch (Exception e) {
	        	System.exit(1);
	          }
	   return null;
}
  
  
  /**
   * This method simplifies a given logical expression over decisions by considering
   * the already set {@link Decision}s and returns the simplified expression
   * 
   * @return The simplified expression
   */
   public String simplifyParser() {
      
     SimplifyVisitor vis = new SimplifyVisitor();
     boolean run = true;
     for (;run; jjtree.reset())
          try {
            switch (expr()) {
            default:      jjtree.rootNode().jjtAccept(vis, null);
            case 0:       return new QMCalgorithm(model).simplify(vis.getSimplyfiedExp());
            case -1:      run = false;
            }
          } catch (Exception e) {
        	  
            e.printStackTrace(); System.exit(1);
       }
	   return null;
  }
  
  //------------------
   
  //Automatically generated methods.
  //Don't edit!!!

  final public int expr() throws ParseException {
    try {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case CONSTANT:
      case FALSE:
      case TRUE:
      case 12:
      case 14:
        expr1();
        jj_consume_token(EOL);
                          {if (true) return 1;}
        break;
      case EOL:
        jj_consume_token(EOL);
                          {if (true) return 0;}
        break;
      case 0:
        jj_consume_token(0);
                          {if (true) return -1;}
        break;
      default:
        jj_la1[0] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
    } catch (ParseException pe) {
      System.err.println(pe);
      for (;;)
        switch (getNextToken().kind) {
        case EOF:       {if (true) return -1;}
        case EOL:       {if (true) return 0;}
        }
    }
    throw new Error("Missing return statement in function");
  }

  final public void expr1() throws ParseException {
    expr2();
    label_1:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case 9:
      case 10:
      case 11:
        ;
        break;
      default:
        jj_la1[1] = jj_gen;
        break label_1;
      }
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case 9:
        jj_consume_token(9);
            And jjtn001 = new And(JJTAND);
            boolean jjtc001 = true;
            jjtree.openNodeScope(jjtn001);
        try {
          expr2();
        } catch (Throwable jjte001) {
            if (jjtc001) {
              jjtree.clearNodeScope(jjtn001);
              jjtc001 = false;
            } else {
              jjtree.popNode();
            }
            if (jjte001 instanceof RuntimeException) {
              {if (true) throw (RuntimeException)jjte001;}
            }
            if (jjte001 instanceof ParseException) {
              {if (true) throw (ParseException)jjte001;}
            }
            {if (true) throw (Error)jjte001;}
        } finally {
            if (jjtc001) {
              jjtree.closeNodeScope(jjtn001,  2);
            }
        }
        break;
      case 10:
        jj_consume_token(10);
           Or jjtn002 = new Or(JJTOR);
           boolean jjtc002 = true;
           jjtree.openNodeScope(jjtn002);
        try {
          expr2();
        } catch (Throwable jjte002) {
           if (jjtc002) {
             jjtree.clearNodeScope(jjtn002);
             jjtc002 = false;
           } else {
             jjtree.popNode();
           }
           if (jjte002 instanceof RuntimeException) {
             {if (true) throw (RuntimeException)jjte002;}
           }
           if (jjte002 instanceof ParseException) {
             {if (true) throw (ParseException)jjte002;}
           }
           {if (true) throw (Error)jjte002;}
        } finally {
           if (jjtc002) {
             jjtree.closeNodeScope(jjtn002,  2);
           }
        }
        break;
      case 11:
        jj_consume_token(11);
            Xor jjtn003 = new Xor(JJTXOR);
            boolean jjtc003 = true;
            jjtree.openNodeScope(jjtn003);
        try {
          expr2();
        } catch (Throwable jjte003) {
            if (jjtc003) {
              jjtree.clearNodeScope(jjtn003);
              jjtc003 = false;
            } else {
              jjtree.popNode();
            }
            if (jjte003 instanceof RuntimeException) {
              {if (true) throw (RuntimeException)jjte003;}
            }
            if (jjte003 instanceof ParseException) {
              {if (true) throw (ParseException)jjte003;}
            }
            {if (true) throw (Error)jjte003;}
        } finally {
            if (jjtc003) {
              jjtree.closeNodeScope(jjtn003,  2);
            }
        }
        break;
      default:
        jj_la1[2] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
    }
  }

  final public void expr2() throws ParseException {
   Token t;
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case 12:
      jj_consume_token(12);
      expr1();
      jj_consume_token(13);
      break;
    case 14:
      jj_consume_token(14);
            Not jjtn001 = new Not(JJTNOT);
            boolean jjtc001 = true;
            jjtree.openNodeScope(jjtn001);
      try {
        expr2();
      } catch (Throwable jjte001) {
            if (jjtc001) {
              jjtree.clearNodeScope(jjtn001);
              jjtc001 = false;
            } else {
              jjtree.popNode();
            }
            if (jjte001 instanceof RuntimeException) {
              {if (true) throw (RuntimeException)jjte001;}
            }
            if (jjte001 instanceof ParseException) {
              {if (true) throw (ParseException)jjte001;}
            }
            {if (true) throw (Error)jjte001;}
      } finally {
            if (jjtc001) {
              jjtree.closeNodeScope(jjtn001,  1);
            }
      }
      break;
    case CONSTANT:
      Lit jjtn002 = new Lit(JJTLIT);
      boolean jjtc002 = true;
      jjtree.openNodeScope(jjtn002);
      try {
        t = jj_consume_token(CONSTANT);
                      jjtree.closeNodeScope(jjtn002, true);
                      jjtc002 = false;
                      jjtn002.val = new String(t.image);
      } finally {
      if (jjtc002) {
        jjtree.closeNodeScope(jjtn002, true);
      }
      }
      break;
    case FALSE:
      False jjtn003 = new False(JJTFALSE);
      boolean jjtc003 = true;
      jjtree.openNodeScope(jjtn003);
      try {
        jj_consume_token(FALSE);
      } finally {
      if (jjtc003) {
        jjtree.closeNodeScope(jjtn003, true);
      }
      }
      break;
    case TRUE:
      True jjtn004 = new True(JJTTRUE);
      boolean jjtc004 = true;
      jjtree.openNodeScope(jjtn004);
      try {
        jj_consume_token(TRUE);
      } finally {
      if (jjtc004) {
        jjtree.closeNodeScope(jjtn004, true);
      }
      }
      break;
    default:
      jj_la1[3] = jj_gen;
      jj_consume_token(-1);
      throw new ParseException();
    }
  }

  /** Generated Token Manager. */
  public ConstraintsTokenManager token_source;
  SimpleCharStream jj_input_stream;
  /** Current token. */
  public Token token;
  /** Next token. */
  public Token jj_nt;
  private int jj_ntk;
  private int jj_gen;
  final private int[] jj_la1 = new int[4];
  static private int[] jj_la1_0;
  static {
      jj_la1_init_0();
   }
   private static void jj_la1_init_0() {
      jj_la1_0 = new int[] {0x51b1,0xe00,0xe00,0x51a0,};
   }

  /** Constructor with InputStream. */
  public ConstraintParser(java.io.InputStream stream, Constraint constraint, boolean antecedent, AdoraModel model) {
	  this(stream, null);
	  this.constraint = constraint;
	  this.model = model;
  }
  /** Constructor with InputStream and supplied encoding */
  public ConstraintParser(java.io.InputStream stream, String encoding) {
    try { jj_input_stream = new SimpleCharStream(stream, encoding, 1, 1); } catch(java.io.UnsupportedEncodingException e) { throw new RuntimeException(e); }
    token_source = new ConstraintsTokenManager(jj_input_stream);
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 4; i++) jj_la1[i] = -1;
  }

  /** Reinitialise. */
  public void ReInit(java.io.InputStream stream) {
     ReInit(stream, null);
  }
  /** Reinitialise. */
  public void ReInit(java.io.InputStream stream, String encoding) {
    try { jj_input_stream.ReInit(stream, encoding, 1, 1); } catch(java.io.UnsupportedEncodingException e) { throw new RuntimeException(e); }
    token_source.ReInit(jj_input_stream);
    token = new Token();
    jj_ntk = -1;
    jjtree.reset();
    jj_gen = 0;
    for (int i = 0; i < 4; i++) jj_la1[i] = -1;
  }

  /** Constructor. */
  public ConstraintParser(java.io.Reader stream) {
    jj_input_stream = new SimpleCharStream(stream, 1, 1);
    token_source = new ConstraintsTokenManager(jj_input_stream);
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 4; i++) jj_la1[i] = -1;
  }

  /** Reinitialise. */
  public void ReInit(java.io.Reader stream) {
    jj_input_stream.ReInit(stream, 1, 1);
    token_source.ReInit(jj_input_stream);
    token = new Token();
    jj_ntk = -1;
    jjtree.reset();
    jj_gen = 0;
    for (int i = 0; i < 4; i++) jj_la1[i] = -1;
  }

  /** Constructor with generated Token Manager. */
  public ConstraintParser(ConstraintsTokenManager tm) {
    token_source = tm;
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 4; i++) jj_la1[i] = -1;
  }

  /** Reinitialise. */
  public void ReInit(ConstraintsTokenManager tm) {
    token_source = tm;
    token = new Token();
    jj_ntk = -1;
    jjtree.reset();
    jj_gen = 0;
    for (int i = 0; i < 4; i++) jj_la1[i] = -1;
  }

  private Token jj_consume_token(int kind) throws ParseException {
    Token oldToken;
    if ((oldToken = token).next != null) token = token.next;
    else token = token.next = token_source.getNextToken();
    jj_ntk = -1;
    if (token.kind == kind) {
      jj_gen++;
      return token;
    }
    token = oldToken;
    jj_kind = kind;
    throw generateParseException();
  }


/** Get the next Token. */
  final public Token getNextToken() {
    if (token.next != null) token = token.next;
    else token = token.next = token_source.getNextToken();
    jj_ntk = -1;
    jj_gen++;
    return token;
  }

/** Get the specific Token. */
  final public Token getToken(int index) {
    Token t = token;
    for (int i = 0; i < index; i++) {
      if (t.next != null) t = t.next;
      else t = t.next = token_source.getNextToken();
    }
    return t;
  }

  private int jj_ntk() {
    if ((jj_nt=token.next) == null)
      return (jj_ntk = (token.next=token_source.getNextToken()).kind);
    else
      return (jj_ntk = jj_nt.kind);
  }

  private java.util.List jj_expentries = new java.util.ArrayList();
  private int[] jj_expentry;
  private int jj_kind = -1;

  /** Generate ParseException. */
  public ParseException generateParseException() {
    jj_expentries.clear();
    boolean[] la1tokens = new boolean[15];
    if (jj_kind >= 0) {
      la1tokens[jj_kind] = true;
      jj_kind = -1;
    }
    for (int i = 0; i < 4; i++) {
      if (jj_la1[i] == jj_gen) {
        for (int j = 0; j < 32; j++) {
          if ((jj_la1_0[i] & (1<<j)) != 0) {
            la1tokens[j] = true;
          }
        }
      }
    }
    for (int i = 0; i < 15; i++) {
      if (la1tokens[i]) {
        jj_expentry = new int[1];
        jj_expentry[0] = i;
        jj_expentries.add(jj_expentry);
      }
    }
    int[][] exptokseq = new int[jj_expentries.size()][];
    for (int i = 0; i < jj_expentries.size(); i++) {
      exptokseq[i] = (int[])jj_expentries.get(i);
    }
    return new ParseException(token, exptokseq, tokenImage);
  }

  /** Enable tracing. */
  final public void enable_tracing() {
  }

  /** Disable tracing. */
  final public void disable_tracing() {
  }

  }
