package ch.unizh.ifi.adora.productline.ui.views.constraintstable.parser;

import java.util.HashMap;
import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.ConstraintConnection;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.variability.Constraint;
import ch.unizh.ifi.adora.model.representation.variability.Decision;


/**
 * Visitor Pattern for creating the syntax tree which builds the basis for
 * the creation of the {@link ConstraintConnection}s.  
 * 
 * @author		        Ivo Vigan
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             2.12.2008 IV First Version
 */
public class Infix implements ConstraintsVisitor{

	/**
	 * Stores the top level {@link Connection}.
	 */
	private ConnectableElement rootConnection = null;
	
	/**
	 * Stores the nr->decision relation.
	 */
	private HashMap<Integer, Decision> decMap;
	
	/**
	 * The {@link Constraint} for which the {@link ConstraintConnection}s get generated
	 */
 	private Constraint constraint;
	
	/**
	 * Constructs a new InfixVisitor
	 * @param constraint 
	 */
	public Infix(Constraint constraint) {
		this.constraint = constraint;
		decMap = AdoraPlugin.getActiveModel().getDecisionsMap();
	}
	
	/**
	 * Generic Method to generate the {@link ConstraintConnection}s. 
	 * 
	 * @param node The {@link Node}
	 * @param data not used
	 * @param opCode Determines whether the operation is an AND, OR, XOR
	 * @return new {@link ConstraintConnection}
	 */
	private Object operationVisitor(Node node, Object data, int opCode) {
		Node a = node.jjtGetChild(0);
		Node b = node.jjtGetChild(1);
		
		Object aVal = a.jjtAccept(this, data);
		Object bVal = b.jjtAccept(this, data);
		
		boolean negFrom = false;
		boolean negTo = false;
		
		if (a instanceof Not)
			negFrom = true;
		if (b instanceof Not)
			negTo = true;
		
		if (aVal instanceof String) {
			aVal = decMap.get(Integer.parseInt(((String)aVal).substring(1)));
	//		if (((Decision)aVal).getFamilyLines().size() == 0)
				((Decision)aVal).generateFamilyLines();
			//to get the 0-th JR is somehow arbitrarily, but someone should propose
			//a term which should be optimized regarding the routing
			aVal = ((Decision)aVal).getJoinRelations().get(0);
			if (!((JoinRelation)aVal).isVisible())
				aVal = ((JoinRelation)aVal).abstrectedIn();
		}
		if (bVal instanceof String) {
			bVal = decMap.get(Integer.parseInt(((String)bVal).substring(1)));
	//		if (((Decision)bVal).getFamilyLines().size() == 0)
				((Decision)bVal).generateFamilyLines();
			bVal = ((Decision)bVal).getJoinRelations().get(0);
			if (!((JoinRelation)bVal).isVisible())
				bVal = ((JoinRelation)bVal).abstrectedIn();
		}
		
		/*ConstraintConnection con = new ConstraintConnection((ConnectableElement)aVal, (ConnectableElement)bVal, opCode, negFrom, negTo);
		constraint.addConnection(con);*/
		
		Connection con = null;
		if (bVal instanceof JoinRelation &&
			aVal instanceof JoinRelation && 
			aVal == bVal) {
			
			con = ((JoinRelation)aVal);
			
		} else {
			try{
			con = new ConstraintConnection((ConnectableElement)aVal, (ConnectableElement)bVal, opCode, negFrom, negTo);
			constraint.addConnection((ConstraintConnection)con);
			} catch(Exception e){}
		}
		
		if (node.jjtGetParent() == null) {
			setRootConnection(con);
		}
		return con;
	}

	/**
	 * 
	 */
	public Object visit(SimpleNode node, Object data) {
		return node;
	}

	/**
	 * Called if a node in the AST is AND.
	 */
	public Object visit(And node, Object data) {
		return operationVisitor(node, data, ConstraintConnection.AND);
	}

	/**
	 * Called if a node in the AST is a OR.
	 */
	public Object visit(Or node, Object data) {
		return operationVisitor(node, data, ConstraintConnection.OR);
	}
	
	/**
	 * Called if a node in the AST is a XOR.
	 */
	public Object visit(Xor node, Object data) {
		return operationVisitor(node, data, ConstraintConnection.XOR);
	}

	public Object visit(Not node, Object data) {
		Node operand  = node.jjtGetChild(0);

		if (operand instanceof Not)
			return operand.jjtGetChild(0).jjtAccept(this, data);
		else 
			return operand.jjtAccept(this, data);
	}

	/**
	 * Called if a node in the AST is a Decision.
	 */
	public Object visit(Lit node, Object data) {
		if (node.jjtGetParent() == null) {
			JoinRelation jr = (decMap.get(
					Integer.parseInt(node.val.substring(1)))).getJoinRelations().get(0);
			if (jr.isVisible())
				setRootConnection(jr);
			else
				setRootConnection(jr.abstrectedIn());
		}
		
		
		//-------new
		
		if (node.jjtGetParent() instanceof Not && node.jjtGetParent().jjtGetParent() == null) {
			JoinRelation jr = (decMap.get(
					Integer.parseInt(node.val.substring(1)))).getJoinRelations().get(0);
			if (jr.isVisible())
				setRootConnection(jr);
			else
				setRootConnection(jr.abstrectedIn());
		}
			
		return node.val;
	}

	/**
	 * Called if a node in the AST is a Boolean Value.
	 */
	public Object visit(False node, Object data) {
		return node;
	}

	/**
	 * Called if a node in the AST is a Boolean Value.
	 */
	public Object visit(True node, Object data) {
		return node;
	}
	
	/**
	 * 
	 * @return
	 */
	public ConnectableElement getRootConnection() {
		return rootConnection;
	}

	/**
	 * 
	 * @param rootConnection
	 */
	public void setRootConnection(ConnectableElement rootConnection) {
		this.rootConnection = rootConnection;
	}
}