package ch.unizh.ifi.adora.productline.ui.views.decisiontable;

import java.beans.PropertyChangeEvent;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import org.eclipse.gef.commands.Command;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.IContainer;
import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.commands.representation.RepresentationNodeBloat;
import ch.unizh.ifi.adora.model.commands.representation.RepresentationNodeShrink;
import ch.unizh.ifi.adora.model.properties.AspectContainerPropNames;
import ch.unizh.ifi.adora.model.properties.ConnectableElementPropNames;
import ch.unizh.ifi.adora.model.properties.VisualModElemPropNames;
import ch.unizh.ifi.adora.model.properties.ZoomablePropNames;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.VisualizableModelElement;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.model.representation.variability.Constraint;
import ch.unizh.ifi.adora.model.representation.variability.Decision;
import ch.unizh.ifi.adora.parts.JoinRelationEditPart;
import ch.unizh.ifi.adora.productline.ui.views.constraintstable.UpdateConstraintConnections;
import ch.unizh.ifi.adora.productline.ui.views.constraintstable.sat.SatSolver;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;
import ch.unizh.ifi.adora.ui.views.ModelObserverMediator;

/**
 * This class is the {@link IStructuredContentProvider} of the
 * {@link DecisionTableViewPart}. It implements also the observer pattern, so
 * that it can listen to changes in the actual model and refresh the view.
 * 
 * @author Pascal Schoeni
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 2008-03-05 PS First Version
 * @version $Date: 2010-02-01 22:22:55 -0500 (Mon, 01 Feb 2010) $, $Author: vigan $, $Revision: 5105 $
 */
public class DecisionTableContentProvider implements IStructuredContentProvider,
		IPartListener, Observer {
	/**
     * The private class for handling the selection.
     * 
     * @author Pascal Schoeni
     * @copyright Requirements Engineering Research Group<br>
     *            Department of Informatics<br>
     *            University of Zurich<br>
     * @history 2008-03-11 PS First Version
     * @version $Date: 2010-02-01 22:22:55 -0500 (Mon, 01 Feb 2010) $, $Author: vigan $, $Revision: 5105 $
     */
	private final class SelectionListener implements ISelectionListener {
		/*
         * (non-Javadoc)
         * 
         * @see org.eclipse.ui.ISelectionListener#selectionChanged(org.eclipse.ui.IWorkbenchPart,
         *      org.eclipse.jface.viewers.ISelection)
         */
		public void selectionChanged(IWorkbenchPart part, ISelection selection) {
			if (!(selection instanceof IStructuredSelection))
				return;
			ArrayList<Object> toSelect = new ArrayList<Object>();
			Iterator selectionIterator = ((IStructuredSelection) selection)
					.iterator();

			while (selectionIterator.hasNext()) {
				Object selectedObject = selectionIterator.next();

				if (selectedObject instanceof JoinRelationEditPart) {
					JoinRelationEditPart editPart = (JoinRelationEditPart) selectedObject;
					Decision decision = getDecision(editPart
							.getModel());
					if (decision != null) {
						toSelect.add(decision);
					}
				}
			}

			List actualList = ((IStructuredSelection) tableViewer
					.getSelection()).toList();
			if (alreadySelected(actualList, toSelect) || toSelect.size()==0) {
				return;
			}
			
//		 	tableViewer.getTable().setFocus();
		//	tableViewer.setSelection(new StructuredSelection(toSelect), true);
		}

		/**
         * Checks, if the two lists contains the same elements.
         * 
         * @param actualList
         * @param toSelect
         * 
         * @return true if the two lists have the same elements
         */
		private boolean alreadySelected(List actualList, List toSelect) {
			if (actualList.size() != toSelect.size()) {
				return false;
			}

			for (Object object : actualList) {
				if (!toSelect.contains(object)) {
					return false;
				}
			}

			return true;
		}
	}

	/**
     * The {@link TableViewer} of the {@link DecisionTableViewPart}.
     */
	private TableViewer tableViewer;
	
	/**
     * The constructor, which adds a listener to the page and sets the
     * {@link DecisionTableContentProvider#tableViewer}.
     * 
     * @param viewer
     *            The {@link TableViewer} used by the view.
     * @param page
     *            The {@link IWorkbenchPage} used.
     */
	public DecisionTableContentProvider(TableViewer viewer, IWorkbenchPage page) {
		ModelObserverMediator.getInstance().addObserver(this);
		page.addPartListener(this);
		this.tableViewer = viewer;

		if (page.getActivePart() instanceof AdoraModelEditor) {
			ModelObserverMediator.getInstance().setAdoraModelEditor((AdoraModelEditor) page.getActivePart());
		}
	}
	
	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.IContentProvider#dispose()
     */
	public void dispose() {
		ModelObserverMediator.getInstance().deleteObserver(this);
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
     */
	public Object[] getElements(Object parent) {
		try {
			
			Decision[] ob = new Decision[ModelObserverMediator.getInstance().getAdoraModelEditor()
			     					.getModel().getDecisions().toArray().length]; 
			System.arraycopy(ModelObserverMediator.getInstance().getAdoraModelEditor()
					.getModel().getDecisions().toArray(), 0, ob, 0, ModelObserverMediator.getInstance().getAdoraModelEditor()
					.getModel().getDecisions().toArray().length);
			
			Arrays.sort(ob);
			return  ob;
			
		} catch (NullPointerException e) {
			return new Object[0];
		}
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer,
     *      java.lang.Object, java.lang.Object)
     */
	public void inputChanged(Viewer v, Object oldInput, Object newInput) {
	}
	
	DecisionTableViewPart decTableVP;
	public void setDecisionTableViewPart(DecisionTableViewPart decTableVP) {
		this.decTableVP = decTableVP;
	}
	

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.IPartListener#partActivated(org.eclipse.ui.IWorkbenchPart)
     */
	public void partActivated(IWorkbenchPart part) {
		if (!(part instanceof AdoraModelEditor)) {
			return;
		}
		
		//Set the initial content of the iftrue/iffalse columns
		//new SatSolver().solve();
		

		processPart(part);

		ModelObserverMediator observer = ModelObserverMediator.getInstance();
		observer.setAdoraModelEditor((AdoraModelEditor) part);
		
		decTableVP.updateProductDerivation(observer.getAdoraModelEditor().getModel().hasProductDerivationStarted());
	}

	/**
     * Saves the current page.
     */
	private IWorkbenchPage currentPage;

	/**
     * Saves a listener, which is used for the selections.
     */
	private SelectionListener listener = new SelectionListener();

	/**
     * Removes the selectionlistener from the one part and adds it to the passed
     * one.
     * 
     * @param part
     */
	private void processPart(IWorkbenchPart part) {
		if (currentPage != null) {
			currentPage.removeSelectionListener(listener);
		}

		currentPage = part.getSite().getPage();
		currentPage.addSelectionListener(listener);
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.IPartListener#partBroughtToTop(org.eclipse.ui.IWorkbenchPart)
     */
	public void partBroughtToTop(IWorkbenchPart part) {
		if (!(part instanceof AdoraModelEditor)) {
			return;
		}

		processPart(part);

		ModelObserverMediator observer = ModelObserverMediator.getInstance();
		observer.setAdoraModelEditor((AdoraModelEditor) part);
		
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.IPartListener#partClosed(org.eclipse.ui.IWorkbenchPart)
     */
	public void partClosed(IWorkbenchPart part) {
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.IPartListener#partDeactivated(org.eclipse.ui.IWorkbenchPart)
     */
	public void partDeactivated(IWorkbenchPart part) {
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.IPartListener#partOpened(org.eclipse.ui.IWorkbenchPart)
     */
	public void partOpened(IWorkbenchPart part) {
	}

	/**
     * This method is used to observe a {@link JoinRelation} for changes and to
     * add it to the list of elements and the tableViewer.
     * 
     * @param joinRelation
     *            The {@link JoinRelation} to process.
     */
	private void processJoinRelation(JoinRelation joinRelation) {
		processJoinRelation(joinRelation, isInVariability(joinRelation)); //ursprungsAspectContainer und boolean variability (true) vom aspect container holen
	}

	/**
     * This method is used to observe a {@link JoinRelation} for changes and to
     * add it to the list of elements and the tableViewer.
     * 
     * @param joinRelation
     *            The {@link JoinRelation} to process.
     * @param inVariability
     *            If the {@link JoinRelation} is in an {@link AspectContainer}
     *            which is a variability.
     */
	private void processJoinRelation(JoinRelation joinRelation,
			boolean inVariability) {
			Decision decision = getDecision(joinRelation);

		if (inVariability && decision == null) { // wenn decision nicht vorhanden
			List<JoinRelation> list = new ArrayList<JoinRelation>();
			list.add(joinRelation);
			Decision dec = new Decision(list);
			ModelObserverMediator.getInstance().getAdoraModelEditor()
								.getModel().addDecision(dec);
		} else if (!inVariability && decision != null) {// decision lschen, wenn variability auf false gesetzt und decision existiert
			ModelObserverMediator.getInstance().getAdoraModelEditor()
					.getModel().removeDecision(decision);
		}
	}

	private Decision getDecision(JoinRelation joinRelation) {
		return ModelObserverMediator.getInstance().getAdoraModelEditor()
				.getModel().getDecision(joinRelation);
	}

	/**
     * Checks, if a {@link JoinRelation} is inside an {@link AspectContainer}
     * which is a variability.
     * 
     * @param joinRelation
     *            The {@link JoinRelation} to check.
     * @return True, if it is inside a variability.
     */
	private boolean isInVariability(JoinRelation joinRelation) {
		Node node = joinRelation.getSource();

		IContainer parent = node.getParent();

		while (parent != null && !(parent instanceof AspectContainer)) {
			parent = parent.getParent();
		}

		boolean variable = false;
		if (parent instanceof AspectContainer ) {
			variable = variable || ((AspectContainer) parent).isVariability();
		}
		if (joinRelation.getSource() instanceof AspectContainer) 
			variable = variable || ((AspectContainer)joinRelation.getSource()).isVariability();

		return variable;
	}

	/*
	 * (non-Javadoc)
	 * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
	 */
	public void update(Observable o, Object arg) {
		
		//TODO: IV COMMENTED OUT TO avoid reentrant call
		tableViewer.refresh();
		
		if (arg instanceof PropertyChangeEvent &&  
				((PropertyChangeEvent)arg).getPropertyName().equals(ZoomablePropNames.ZOOM_OPERATION_DONE)) {
		
			
			AdoraModel adoraModel = AdoraPlugin.getActiveModel();
			
			List<Constraint> visibleConst = new ArrayList<Constraint>();
			for (Constraint con : adoraModel.getConstraints()) {
				if (con.isVisible()) {
					visibleConst.add(con);
				}
			}
			
			UpdateConstraintConnections ucc = new UpdateConstraintConnections(adoraModel, visibleConst);
			AdoraPlugin.getActiveEditor().getCommandStack().execute(ucc);
			
		}
		
		if (arg instanceof PropertyChangeEvent && 
				   ((PropertyChangeEvent)arg).getPropertyName().equals(AspectContainerPropNames.ASPECT_WEAVING_HIDDEN_PROP)) {
			
			
			if (((PropertyChangeEvent)arg).getSource() instanceof AspectContainer) {
				AspectContainer ac = (AspectContainer) ((PropertyChangeEvent)arg).getSource();
				boolean wasAChidden = (Boolean)((PropertyChangeEvent)arg).getOldValue();
				boolean isAChidden = (Boolean)((PropertyChangeEvent)arg).getNewValue();
				
				Command cmd;
			
				if (!wasAChidden && isAChidden) {
					
					cmd = new RepresentationNodeShrink(ac);
					executeCommandOnStack(cmd);
				}
					
				else if (wasAChidden && !isAChidden) {
				
					cmd = new RepresentationNodeBloat(ac);
					executeCommandOnStack(cmd);
				}
			}
		}	
	}
	
	 /**
     * Executes a given {@link org.eclipse.gef.commands.Command} on the {@link org.eclipse.gef.commands.CommandStack} 
     * of the editor.
     *
     * @param cmd The {@link org.eclipse.gef.commands.Command} which gets executed on the 
     * {@link org.eclipse.gef.commands.CommandStack}.
     */
    protected void executeCommandOnStack(Command cmd){
    	IWorkbenchPage page = PlatformUI.getWorkbench()
		.getActiveWorkbenchWindow().getActivePage();
		
    	try {
			IEditorPart editor = page.getActiveEditor();
			if (editor instanceof AdoraModelEditor) {
				((AdoraModelEditor)editor).getCommandStack().execute(cmd);
			}
		} catch (Exception ex) {
			ex.printStackTrace();
		}
    }
	
	/*
     * (non-Javadoc)
     * 
     * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
     */
	public void update1(Observable o, Object arg) {
		if (arg instanceof String
				&& ModelObserverMediator.MODEL_CHANGED.equals(arg)) {
			tableViewer.refresh();
			return;
		}

		if (!(arg instanceof PropertyChangeEvent)) {
			return;
		}

		PropertyChangeEvent event = (PropertyChangeEvent) arg;

		//if a variable join relationship gets deleted...
		if (VisualModElemPropNames.WAS_DELETED_PROP.equals(event
				.getPropertyName())
				&& event.getSource() instanceof JoinRelation) {
			Decision decision = getDecision((JoinRelation) event
					.getSource());
			
			if (decision != null) {
				decision.removeJoinRelation((JoinRelation)event.getSource());
				if (decision.getJoinRelations().size() == 0) 
					ModelObserverMediator.getInstance().getAdoraModelEditor()
						.getModel().removeDecision(decision);
			}
			tableViewer.refresh();
		}

		//if an aspect, which is a variability, is deleted, these decisions must also be deleted
		if (VisualModElemPropNames.WAS_DELETED_PROP.equals(event
				.getPropertyName())
				&& event.getSource() instanceof AspectContainer) {
			AspectContainer deletedAspect = ((AspectContainer) event.getSource());
			if (deletedAspect.isVariability()) {
				deletedAspect.setVariability(false);	//Variability is removed
				processAspectContainer(deletedAspect);	//And now the decisions will be removed
			}
			tableViewer.refresh();
		}
		
		
		//if an aspect, which is a variability, is deleted, these decisions must also be deleted
		if (VisualModElemPropNames.WAS_DELETED_PROP.equals(event
				.getPropertyName())
				&& event.getSource() instanceof ZoomableNodeContainer) {
			ZoomableNodeContainer znc = ((ZoomableNodeContainer) event.getSource());
			for (Connection con : znc.getTargetConnections()) {
				if (con instanceof JoinRelation && ((JoinRelation)con).getDecision() != null) {
					Decision dec = ((JoinRelation)con).getDecision(); 
					dec.removeJoinRelation((JoinRelation)con);
					/*if (dec.getJoinRelations().size() == 0) { 
						ModelObserverMediator.getInstance().getAdoraModelEditor()
						.getModel().removeDecision(((JoinRelation)con).getDecision());
					}*/

				}
			}
			tableViewer.refresh();
		}

		if (ConnectableElementPropNames.SOURCECONNECTION_NEW_PROP.equals(event
				.getPropertyName())
				&& event.getNewValue() instanceof JoinRelation) {
			processJoinRelation((JoinRelation) event.getNewValue());
		}

		if (AspectContainerPropNames.ASPECT_VARIBILITY_PROP.equals(event
				.getPropertyName())) {
			processAspectContainer((AspectContainer) event.getSource());
		}

		// Decisions
		if (AdoraModel.DECISION_ADDED.equals(event.getPropertyName())) {
			tableViewer.refresh();
		}

		if (AdoraModel.DECISION_REMOVED.equals(event.getPropertyName())) {
			tableViewer.refresh();
		}

		if (AdoraModel.DECISION_CHANGED.equals(event.getPropertyName())) {
			//tableViewer.refresh();
		}
		//jaro
		if (IContainer.CHILD_ADDED_PROP.equals(event.getPropertyName())) {
			//System.out.println("won!");
		}
		
		if (AdoraModel.VARIATION_POINT_CHANGED.equals(event.getPropertyName()) ||
				AdoraModel.VARIATION_POINT_ADDED.equals(event.getPropertyName()) ||
				AdoraModel.VARIATION_POINT_REMOVED.equals(event.getPropertyName())) {
			//then update the variation points table!
			tableViewer.refresh();
		}

		if (AdoraModel.CONSTRAINT_CHANGED.equals(event.getPropertyName()) ||
				AdoraModel.CONSTRAINT_ADDED.equals(event.getPropertyName()) ||
				AdoraModel.CONSTRAINT_REMOVED.equals(event.getPropertyName())) {
			//then update the constraints table!
			tableViewer.refresh();
		}
		
	}

	/**
     * Processes an {@link AspectContainer}. Iterates over all
     * {@link ModelElement} and seeks for {@link JoinRelation}'s to process.
     * 
     * @see #processJoinRelation(JoinRelation)
     * @param container
     *            The {@link AspectContainer} to process.
     */
	private void processAspectContainer(AspectContainer container) {
		Iterator<ModelElement> iterator = container
				.getRecursiveChildrenIterator();

		while (iterator.hasNext()) {
			ModelElement modelElement = iterator.next(); //modelElement rausholen

			if (modelElement instanceof ConnectableElement) {
				ConnectableElement node = (ConnectableElement) modelElement; //node herausholen
				for (Connection connection : node.getSourceConnections()) { //fr alle connections
					if (connection instanceof JoinRelation) { //joinRealation bearbeiten
					processJoinRelation((JoinRelation) connection,
						container.isVariability());
					}
				}
			}
		}
	}
}
