package ch.unizh.ifi.adora.productline.ui.views.decisiontable;

import java.util.List;

import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.graphics.Image;

import ch.unizh.ifi.adora.model.representation.variability.Decision;

/**
 * This is the {@link ITableLabelProvider} of the {@link DecisionTableViewPart}.
 * 
 * @author Pascal Schoeni
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 2008-03-05 PS First Version
 * @version $Date: 2010-01-21 22:35:41 -0500 (Thu, 21 Jan 2010) $, $Author: vigan $, $Revision: 5090 $
 */
public class DecisionTableLabelProvider extends LabelProvider implements
		ITableLabelProvider {

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnImage(java.lang.Object,
     *      int)
     */
	public Image getColumnImage(Object element, int columnIndex) {
		return null;
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnText(java.lang.Object,
     *      int)
     */
	public String getColumnText(Object element, int columnIndex) {
		Decision decision = (Decision) element;

		switch (columnIndex) {
		case 0:
			return decision.toString(); //Decision-Number
		case 1:
			return decision.getDescription(); //description
		case 2:
			return decision.getCardinality(); //cardinality
		case 3:
			return decision.getRecommendation();
		case 4: {
			if (decision.getFeatures() !=null)
				return decision.getFeatureNames();
			return "";
		}
		case 5:
			if (decision.getDecisionValue() == null) {
				return "undecided";
			}
			return decision.getDecisionValue() ? "true" : "false";
		case 6:
			//unicode "not" encoding
			String reqTstr = decision.getReqIfTrue().replace("\u00A8", "\u00AC");
			return reqTstr;
		case 7:
			String reqFstr = decision.getReqIfFalse().replace("\u00A8", "\u00AC");
			return reqFstr;
		case 8:
			return decision.wasDecisionSetAutomatically() ? "\u2713" : "";
		case 9: {
			return decision.isFixed() ? "\u2713" : "-";
		}
			
		}
		return null;
	}

	protected static String getString(List list) {
		if (list.size() == 0) {
			return "";
		}
		return list.toString().substring(1, list.toString().length() - 1);
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.LabelProvider#addListener(org.eclipse.jface.viewers.ILabelProviderListener)
     */
	@Override
	public void addListener(ILabelProviderListener listener) {
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.LabelProvider#dispose()
     */
	@Override
	public void dispose() {
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.LabelProvider#isLabelProperty(java.lang.Object,
     *      java.lang.String)
     */
	@Override
	public boolean isLabelProperty(Object element, String property) {
		return false;
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.LabelProvider#removeListener(org.eclipse.jface.viewers.ILabelProviderListener)
     */
	@Override
	public void removeListener(ILabelProviderListener listener) {
	}

}
