package ch.unizh.ifi.adora.productline.ui.views.decisiontable;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Path;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.RootEditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.CheckboxCellEditor;
import org.eclipse.jface.viewers.ColumnWeightData;
import org.eclipse.jface.viewers.ComboBoxCellEditor;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableLayout;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IPartService;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.commands.ConstraintRemove;
import ch.unizh.ifi.adora.model.commands.HideConstraintsAndVPs;
import ch.unizh.ifi.adora.model.commands.HideSelectedConstraints;
import ch.unizh.ifi.adora.model.commands.SetVariationPointListDecisions;
import ch.unizh.ifi.adora.model.commands.VariationPointAdd;
import ch.unizh.ifi.adora.model.commands.VariationPointConnectionCreate;
import ch.unizh.ifi.adora.model.commands.VariationPointRemove;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.ConstraintConnection;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.variability.Constraint;
import ch.unizh.ifi.adora.model.representation.variability.Decision;
import ch.unizh.ifi.adora.model.representation.variability.VariationPoint;
import ch.unizh.ifi.adora.parts.AdoraRootEditPart;
import ch.unizh.ifi.adora.productline.Activator;
import ch.unizh.ifi.adora.productline.ui.views.constraintstable.ConstraintsViewPart;
import ch.unizh.ifi.adora.productline.ui.views.constraintstable.UpdateConstraintConnections;
import ch.unizh.ifi.adora.productline.ui.views.constraintstable.sat.SatSolver;
import ch.unizh.ifi.adora.productline.ui.views.variationpointtable.VariationPointViewPart;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;
import ch.unizh.ifi.adora.ui.AdoraModelEditorPaletteFactory;
import ch.unizh.ifi.adora.ui.views.ModelObserverMediator;

/**
 * This class is the view, which can be used for creating so called "decisions".
 * 
 * @author Pascal Schoeni
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 2008-03-05 PS First Version
 * @version $Date: 2010-01-22 22:28:07 -0500 (Fri, 22 Jan 2010) $, $Author: vigan $, $Revision: 5093 $
 */
public class DecisionTableViewPart extends ViewPart {
	
	public static DecisionTableCellModifier  decisionTableCellModifier;
	
	public static DecisionTableCellModifier getDecisionTableCellModifier() {
		return decisionTableCellModifier;
	}

	public static void setDecisionTableCellModifier(
			DecisionTableCellModifier decisionTableCellModifier) {
		DecisionTableViewPart.decisionTableCellModifier = decisionTableCellModifier;
	}

	/**
	 * The ID of this view.
	 */
	public static final String VIEW_ID = "ch.unizh.ifi.adora.productline.views.VariabilityDecisions";

	public static final String VIEW_ID_DECISION_HISTORY = "ch.unizh.ifi.adora.weaver.view.DecisionHistoryTableViewPart";

	/********************************
	 * /** The BASIC COLUMNS
	 * 
	 * /** The identifier of the Nr Column of this tableview.
	 */
	protected static final String COL_NR = "NR";

	/**
	 * The identifier of the ID Column of this tableview. This column will not
	 * be visualized in the editor, since the automatic highlighting does the
	 * job already well.
	 */
	protected static final String COL_ID = "ID";

	/**
	 * The identifier of the Description Column of this tableview.
	 */
	protected static final String COL_DESCRIPTION = "Description";

	/********************************
	 * /** The VARIABILITY CONSTRAINTS
	 * 
	 * /** The identifier of the Dependency Column of this tableview.
	 */
	// protected static final String COL_DEPENDENCY = "Relevance";

	/*
	 * The identifier of the Cardinality Column of this tableview.
	 */
	protected static final String COL_CARDINALITY = "Constraints & VPs";

	/**
	 * The identifier of the IF-True Column of this tableview.
	 */
	protected static final String COL_IFTRUE = "If true";

	/**
	 * The identifier of the IF-False Column of this tableview.
	 */
	protected static final String COL_IFFALSE = "If false";

	/********************************
	 * /** The DECISION SUPPORT
	 * 
	 * /** The identifier of the Recommendation Column of this tableview.
	 */
	protected static final String COL_RECOMMENDATION = "Recommendation";

	/**
	 * The identifier of the Decision Column of this tableview.
	 */
	public static final String COL_DECISION = "Decision";
	
	/**
	 * The names of the features in which this {@link Decision} appears.
	 */
	public static final String COL_FEATURE = "Feature";
	
	/**
	 * The identifier of the Decision Column of this tableview.
	 */
	public static final String COL_SETAUTO = "Automatically Set";

	/**
	 * The identifier of the Decision Column of this tableview.
	 */
	public static final String COL_FIXED = "Fixed";

	
	/**
	 * Stores the name of the selected Preset.
	 */
	private String selectedPreset = "";

	/**
	 * Stores the name of the Preset to save
	 */
	private String savedName = "";

	/**
	 * The {@link TableViewer} used.
	 */
	private TableViewer viewer;

	/**
	 * The {@link DecisionTableContentProvider} used.
	 */
	private DecisionTableContentProvider contentProvider;

	/**
	 * Switch to decision history of dynamic weaving.
	 */

	private StartProduDerivAction startProductDerivationAction;

	/**
	 * Toolbar Actions
	 */
	private Action addVariationPointAction;

	/**
	 * The Action used to show the Variation Points associated with the selected
	 * Decisions.
	 */
	private Action showSelConstraints;

	/**
	 * The Action used to hide all Variation Points.
	 */
	private Action hideSelConstraints;

	/**
	 * The Action used to hide all Variation Points.
	 */
	private Action hideAllVPConnections;

	/**
	 * The Action used to load a preset.
	 */
	private Action presetManagement;

	/**
	 * The Action reset all {@link Decision}
	 */
	private Action resetDecisions;

	/**
	 * The Action used to count #sat assignments.
	 */
	private Action cntSat;

	/**
	 * The underlying model
	 */
	private AdoraModel adoraModel;

	/**
	 * Stores the selected elements in the Table
	 */
	private ArrayList<Decision> selectedDecisions = new ArrayList<Decision>();
	
	/**
	 * Added for Testability purposes:
	 * --- get current table when only the ViewPart is known 
	 */
	private Table currentTable;
	
	/**
	 * The constructor.
	 */
	public DecisionTableViewPart() {
	}

	public Table getCurrentTable(){
		return currentTable;
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.ui.part.WorkbenchPart#createPartControl(org.eclipse.swt.widgets
	 * .Composite)
	 */
	@Override
	public void createPartControl(Composite parent) {

		this.addChildControls(parent);

		Table table = new Table(parent, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
		currentTable = table;

		table.setLinesVisible(true);
		table.setHeaderVisible(true);

		TableLayout layout = new TableLayout();
		layout.addColumnData(new ColumnWeightData(8, true)); // nr
		layout.addColumnData(new ColumnWeightData(20, true)); // desc
		layout.addColumnData(new ColumnWeightData(10, true)); // rel
		layout.addColumnData(new ColumnWeightData(10, true)); // cardinality
		layout.addColumnData(new ColumnWeightData(10, true)); // recommendation
		layout.addColumnData(new ColumnWeightData(20, true)); // productline
		layout.addColumnData(new ColumnWeightData(10, true)); // decision
		layout.addColumnData(new ColumnWeightData(30, true)); // iftrue
		layout.addColumnData(new ColumnWeightData(30, true)); // iffalse
		layout.addColumnData(new ColumnWeightData(30, true)); // automatically set
		layout.addColumnData(new ColumnWeightData(30, true)); // fixed
		
		table.setLayout(layout);

		TableColumn nrCol = new TableColumn(table, SWT.NONE);
		nrCol.setText(COL_NR);
		nrCol.setResizable(true);

		TableColumn descriptionCol = new TableColumn(table, SWT.NONE);
		descriptionCol.setText(COL_DESCRIPTION);
		descriptionCol.setResizable(true);

		TableColumn cardinalityCol = new TableColumn(table, SWT.NONE);
		cardinalityCol.setText(COL_CARDINALITY);
		cardinalityCol.setResizable(true);

		TableColumn recommendationCol = new TableColumn(table, SWT.NONE);
		recommendationCol.setText(COL_RECOMMENDATION);
		recommendationCol.setResizable(true);

		TableColumn prodLineCol = new TableColumn(table, SWT.NONE);
		prodLineCol.setText(COL_FEATURE);
		prodLineCol.setResizable(true);
		
		TableColumn decisionCol = new TableColumn(table, SWT.NONE);
		decisionCol.setText(COL_DECISION);
		decisionCol.setResizable(true);
		
		TableColumn iftrueCol = new TableColumn(table, SWT.NONE);
		iftrueCol.setText(COL_IFTRUE);
		iftrueCol.setResizable(true);

		TableColumn iffalseCol = new TableColumn(table, SWT.NONE);
		iffalseCol.setText(COL_IFFALSE);
		iffalseCol.setResizable(true);
		
		TableColumn autoSet = new TableColumn(table, SWT.NONE);
		autoSet.setText(COL_SETAUTO);
		autoSet.setResizable(true);

		TableColumn fixedCol = new TableColumn(table, SWT.NONE);
		fixedCol.setText(COL_FIXED);
		fixedCol.setResizable(true);

		
		viewer = new TableViewer(table);
		contentProvider = new DecisionTableContentProvider(viewer, getSite()
				.getPage());
		
		contentProvider.setDecisionTableViewPart(this);
		
		viewer.setContentProvider(contentProvider);
		viewer.setLabelProvider(new DecisionTableLabelProvider());
		viewer.setInput(getViewSite());
		viewer.addSelectionChangedListener(new ISelectionChangedListener() {

			@SuppressWarnings("unchecked")
			public void selectionChanged(SelectionChangedEvent event) {
				if (event.getSelection().isEmpty()) {
					return;
				}

				if (!(event.getSelection() instanceof IStructuredSelection)) {
					return;
				}

				ArrayList selectedModelElements = new ArrayList();
				IStructuredSelection selection = (IStructuredSelection) event
						.getSelection();

				Iterator i = selection.iterator();

				selectedDecisions.clear();
				
				AdoraModelEditor ame = AdoraPlugin.getActiveEditor();
				
				while (i.hasNext()) {

					Decision d = (Decision) i.next();
					
					
					//if the decision is true, select the woven elements
					if (d.getDecisionValue() != null && d.getDecisionValue()) {
						if (d.getWovenElements() != null && d.getWovenElements().size() > 0) {
							for (Node n : d.getWovenElements()) {
								//ame.selectElement(n);
								selectedModelElements.add(n);
							}
						}
					} else {
						//reset list
						d.setWovenElements(new ArrayList<Node>());
					}

					for (JoinRelation jr : d.getJoinRelations()) {
						if (jr.abstrectedIn() != null && !jr.isVisible()) {
							if (!selectedModelElements.contains(jr
									.abstrectedIn()))
								selectedModelElements.add(jr.abstrectedIn());
						} else {
							if (!selectedModelElements.contains(jr))
								selectedModelElements.add(jr);
						}
					}
					selectedDecisions.add(d);
				}

				if (ame != null) {
					if (alreadySelected(ame, selectedModelElements)) {
						return;
					}
					ame.selectElements(selectedModelElements);
				}
			}

			/**
			 * Checks, if the list of selectedModelElements is already selected
			 * in the editor.
			 * 
			 * @param ame
			 * @param selectedModelElements
			 * 
			 * @return true if already selected.
			 */
			private boolean alreadySelected(AdoraModelEditor ame,
					List selectedModelElements) {
				if (ame.getViewer().getSelectedEditParts().size() != selectedModelElements
						.size()) {
					return false;
				}

				boolean equals = true;
				for (Object object : ame.getViewer().getSelectedEditParts()) {
					EditPart part = (EditPart) object;
					if (!selectedModelElements.contains(part.getModel())) {
						equals = false;
					}
				}

				return equals;
			}

		});

		CellEditor[] editors = new CellEditor[12];

		TextCellEditor nrEditor = new TextCellEditor(table);
		((Text) nrEditor.getControl()).setTextLimit(5);
		editors[0] = nrEditor;

		TextCellEditor descriptionEditor = new TextCellEditor(table);
		((Text) descriptionEditor.getControl()).setTextLimit(200);
		editors[1] = descriptionEditor;

		TextCellEditor cardinalityEditor = new TextCellEditor(table);
		((Text) cardinalityEditor.getControl()).setTextLimit(60);
		editors[2] = cardinalityEditor;

		TextCellEditor recommendationEditor = new TextCellEditor(table);
		((Text) recommendationEditor.getControl()).setTextLimit(60);
		editors[3] = recommendationEditor;
		
		TextCellEditor prodLineEditor = new TextCellEditor(table);
		((Text) prodLineEditor.getControl()).setTextLimit(60);
		editors[4] = prodLineEditor;
		

		String[] choices = { "true", "false", "undecided" };
		ComboBoxCellEditor decision = new ComboBoxCellEditor(table, choices);
		editors[5] = decision;

		TextCellEditor iftrueEditor = new TextCellEditor(table);
		((Text) iftrueEditor.getControl()).setTextLimit(60);
		editors[6] = iftrueEditor;

		TextCellEditor iffalseEditor = new TextCellEditor(table);
		((Text) iffalseEditor.getControl()).setTextLimit(60);
		editors[7] = iffalseEditor;
		
		TextCellEditor autoSetEditor = new TextCellEditor(table);
		((Text) autoSetEditor.getControl()).setTextLimit(60);
		editors[8] = autoSetEditor;
		
		CheckboxCellEditor fixedEditor = new CheckboxCellEditor(table);
		editors[9] = fixedEditor;
		

		String[] columnProperties = { COL_NR, COL_DESCRIPTION, //COL_DEPENDENCY, comment: i removed the dependency column for now (didn't work..)
				COL_CARDINALITY, COL_RECOMMENDATION, COL_FEATURE, COL_DECISION, COL_IFTRUE,
				COL_IFFALSE, COL_SETAUTO, COL_FIXED};

		viewer.setColumnProperties(columnProperties);
		viewer.setCellEditors(editors);
		DecisionTableCellModifier dtcm=new DecisionTableCellModifier();
		viewer.setCellModifier(dtcm);	
		
		setDecisionTableCellModifier(dtcm);

	}

	/**
	 * Returns the {@link DecisionTableContentProvider} of this view.
	 * 
	 * @return The {@link DecisionTableContentProvider} of this view.
	 */
	public DecisionTableContentProvider getContentProvider() {
		return contentProvider;
	}

	/**
	 * Passing the focus request to the viewer's control.
	 */
	@Override
	public void setFocus() {
		viewer.getControl().setFocus();
	}

	/**
	 * Creates the controls for the parent composite
	 * 
	 * @param Composite
	 *            parent
	 */
	private void addChildControls(Composite parent) {
		createToolbarActions(parent);
		createToolBar();
	}

	/**
	 * Creates the view toolbar and adds actions to it
	 */
	private void createToolBar() {
		IToolBarManager mgr = getViewSite().getActionBars().getToolBarManager();

		mgr.add(startProductDerivationAction);
		mgr.add(resetDecisions);
		
		mgr.add(new Separator());

		mgr.add(presetManagement);

		mgr.add(new Separator());

		mgr.add(addVariationPointAction);

		mgr.add(new Separator());

		mgr.add(showSelConstraints);
		mgr.add(hideSelConstraints);
		mgr.add(hideAllVPConnections);

		mgr.add(new Separator());

		mgr.add(cntSat);

	}

	/**
	 * creates the toolbar actions and sets the action image
	 */
	public void createToolbarActions(final Composite parent) {

		startProductDerivationAction = new StartProduDerivAction();
		ImageDescriptor id = ImageDescriptor.createFromURL(
					FileLocator.find(
								Activator.getDefault().getBundle(), 
								new Path("icons/run.gif"), null));
		startProductDerivationAction.setImageDescriptor(id);
		startProductDerivationAction.setToolTipText("Start product derivation");
		
		
		
		
		presetManagement = new Action("Manage Presets") {

			@Override
			public void run() {

				PresetDialog md = new PresetDialog(getSite()
						.getWorkbenchWindow().getShell());
				md.tmp();
			}
		};

		cntSat = new Action("Get #SAT-Assignments") {

			@Override
			public void run() {
						
				AdoraModel model = ModelObserverMediator.getInstance()
										.getAdoraModelEditor().getModel();

				int cnt = new SatSolver(model).cntSATassignments();

				if (cnt < 0) {
					MessageBox mb = new MessageBox(getSite()
							.getWorkbenchWindow().getShell(),
							SWT.ICON_INFORMATION | SWT.OK);
					mb.setText("#SAT-Assignments");
					mb
							.setMessage("There are too many Decision to count the number of Satisfying assignments");
					mb.open();

					return;
				} else {

					MessageBox mb = new MessageBox(getSite()
							.getWorkbenchWindow().getShell(),
							SWT.ICON_INFORMATION | SWT.OK);
					mb.setText("#SAT-Assignments");
					mb.setMessage(cnt + " potential products exist.");
					mb.open();

					return;
				}
			}
		};

		hideAllVPConnections = new Action("Hide all Constraints") {

			@Override
			public void run() {
				adoraModel = AdoraPlugin.getActiveModel();
				AdoraModelEditor ame = AdoraPlugin.getActiveEditor();

				HideConstraintsAndVPs hideConst = new HideConstraintsAndVPs(
						adoraModel);
				ame.getCommandStack().execute(hideConst);

			}
		};

		showSelConstraints = new Action("Show selected Constraints") {

			@Override
			public void run() {
				AdoraModelEditor ame = AdoraPlugin.getActiveEditor();
				
				VariationPointConnectionCreate showCommand = new VariationPointConnectionCreate(
						AdoraPlugin.getActiveModel(), selectedDecisions);
				ame.getCommandStack().execute(showCommand);
				
				
				List<Constraint> selectedConst = new ArrayList<Constraint>();
				for (Constraint con : AdoraPlugin.getActiveModel().getConstraints()) {
					for (Decision d : selectedDecisions) {
						if (con.containsDecision(d)) {
							selectedConst.add(con);
							break;
						}
					}
				}
				
				UpdateConstraintConnections constCreate = new UpdateConstraintConnections(
						AdoraPlugin.getActiveModel(), selectedConst);
				ame.getCommandStack().execute(constCreate);
				
			}
		};

		hideSelConstraints = new Action("Hide selected Constraints") {

			@Override
			public void run() {
			
				adoraModel = AdoraPlugin.getActiveModel();
				AdoraModelEditor ame = AdoraPlugin.getActiveEditor();
				
				HideSelectedConstraints hideCmd = new HideSelectedConstraints(adoraModel, selectedDecisions);
				ame.getCommandStack().execute(hideCmd);
				
			}
		};

		addVariationPointAction = new Action("Add VP") {

			@Override
			public void run() {

				adoraModel = AdoraPlugin.getActiveModel();
				AdoraModelEditor ame = AdoraPlugin.getActiveEditor();
				ArrayList<Decision> decisionsToAdd = new ArrayList<Decision>();

				for (Decision d : selectedDecisions)
					decisionsToAdd.add(d);

				if (decisionsToAdd != null && decisionsToAdd.size() > 1) {

					VariationPointAdd addCommand = new VariationPointAdd(
							adoraModel, decisionsToAdd);
					ame.getCommandStack().execute(addCommand);
				}

				VariationPointViewPart variationPointViewPart = (VariationPointViewPart) ame
						.getSite()
						.getPage()
						.findView(
								"ch.unizh.ifi.adora.editor.views.VariationPoints");
				// to activate the variation points view...
				if (variationPointViewPart != null) {
					variationPointViewPart.refresh();
					variationPointViewPart.setFocus();
				}

				viewer.refresh();

				AdoraModel model = ModelObserverMediator.getInstance()
										.getAdoraModelEditor().getModel();

				new SatSolver(model).solve();
			}
		};

		resetDecisions = new Action() {

			@Override
			public void run() {
				
				AdoraModel model = AdoraPlugin.getActiveModel();
				if (!model.hasProductDerivationStarted())
					return;
				
				AdoraModel modelBefore = model.getModelBeforeProdLine();

				for (Decision d : modelBefore.getDecisions()) {
					if (d.getDecisionValue() != null)
						d.resetDecision();
				}
				
				
				modelBefore.setProductDerivationStarted(false);
				AdoraPlugin.getActiveEditor().setModel(modelBefore);
				
				modelBefore.setModelBeforeProdLine(null);
				modelBefore.update();
				
				AdoraModel newmodel = ModelObserverMediator.getInstance()
				.getAdoraModelEditor().getModel();

				
				new SatSolver(newmodel).solve();
				refreshModel();
				AdoraPlugin.setFocus();
				setFocus();
				
				HideConstraintsAndVPs hideConst = new HideConstraintsAndVPs(
						modelBefore);
				AdoraPlugin.getActiveEditor().getCommandStack().execute(hideConst);
				
				
				
				selectedDecisions.clear();
				
				//now hide the entries in the palette factory
				updateProductLineVisisbility(false);

			}
			

            private void showAndCreateVPAndCP()
            {
               
        		Command command = null;
        		AdoraModelEditor ame = AdoraPlugin.getActiveEditor();
        		List<VariationPoint> selectedVPs = AdoraPlugin.getActiveModel()
        				.getVariationPoints();
        		List<VariationPoint>shortenedVPs=new ArrayList<VariationPoint>();
        		VariationPoint shortenVariationPoint;
        		List<Decision> shortenedVariationPointDecisions;
        		for (VariationPoint variationPoint : selectedVPs) {
        			String decs = ",";
        				for (Decision dec : variationPoint.getDecisions()) {
        					decs = decs.concat("D".concat(dec.getNr().toString())
        							.concat(","));
        				}
        			command = new SetVariationPointListDecisions(decs,
        					variationPoint);
        			command.execute();
        			
        			shortenedVariationPointDecisions=new ArrayList();
        			for (Decision dec : variationPoint.getDecisions()) {
        				if (dec.getDecisionValue() == null) {
        					shortenedVariationPointDecisions.add(dec);
        				}
        			}
        			
        			shortenVariationPoint=new VariationPoint(shortenedVariationPointDecisions,variationPoint.getNr(),Integer.parseInt(variationPoint.getMinCard()),Integer.parseInt(variationPoint.getMaxCard()),variationPoint.getDescription(),variationPoint.isVisible(),variationPoint.isRelevant());
        			shortenedVPs.add(shortenVariationPoint);
        		
        		}
        		VariationPointConnectionCreate showCommand = new VariationPointConnectionCreate(
        				AdoraPlugin.getActiveModel(), selectedVPs);
        		ame.getCommandStack().execute(showCommand);
        		// fetch constraint and show them
        		List<Constraint> selectedConst = new ArrayList<Constraint>();
        		for (Constraint con : AdoraPlugin.getActiveModel().getConstraints()) {
        			selectedConst.add(con);
        		}

        		UpdateConstraintConnections constCreate = new UpdateConstraintConnections(
        				AdoraPlugin.getActiveModel(), selectedConst);
        		AdoraPlugin.getActiveEditor().getCommandStack().execute(constCreate);
                    
                } 
            private void refreshModel() {
        		GraphicalViewer viewer = (GraphicalViewer) AdoraPlugin
        				.getActiveEditor().getAdapter(GraphicalViewer.class);
        		viewer.flush();

        		RootEditPart rep = AdoraPlugin.getActiveEditor().getViewer()
        				.getRootEditPart();
        		AdoraRootEditPart arep = (AdoraRootEditPart) rep;
        		
  //      		arep.getZoomManager().setZoom(
    //    				arep.getZoomManager().getZoom() + 0.000001);
        	}
		};
		
		
		
		ImageDescriptor id2 = ImageDescriptor.createFromURL(
				FileLocator.find(
							Activator.getDefault().getBundle(), 
							new Path("icons/stop.gif"), null));
		resetDecisions.setImageDescriptor(id2);
		resetDecisions.setText("Stop Product Derivation");
		resetDecisions.setEnabled(false);
		
		
	}

	/**
	 * disables actions in the toolbar
	 */
	public void disableActions() {
		addVariationPointAction.setEnabled(false);
		showSelConstraints.setEnabled(false);
		hideAllVPConnections.setEnabled(false);
		presetManagement.setEnabled(false);
		hideSelConstraints.setEnabled(false);
		resetDecisions.setEnabled(false);
	}

	/**
	 * enables actions in the toolbar if disabled
	 */
	private void enableActions() {
		addVariationPointAction.setEnabled(true);
		showSelConstraints.setEnabled(true);
		hideAllVPConnections.setEnabled(true);
		presetManagement.setEnabled(true);
		hideSelConstraints.setEnabled(true);
		resetDecisions.setEnabled(true);
	}

	/**
	 * Class which stores and manages the Presets
	 * 
	 * @author vigan
	 * 
	 */
	class PresetDialog {

		private Shell shell;

		private Combo myCombo;

		public PresetDialog(Shell shell) {
			this.shell = shell;
		}

		private void setValue(int index) {
			myCombo.select(index);
		}
		
		private void addPreset(String name) {
			myCombo.add(name);
			myCombo.select(myCombo.indexOf(name));
		}

		private void removePreset(String name) {
			if (contains(name)) {
				myCombo.remove(name);
				myCombo.deselectAll();
			}
		}

		public boolean contains(String name) {
			return myCombo.indexOf(name) > -1;
		}

		public void tmp() {
			final Shell dialog = new Shell(shell, SWT.DIALOG_TRIM
					| SWT.APPLICATION_MODAL);
			dialog.setLayout(new RowLayout());
			dialog.setText("Preset Management");

			myCombo = new Combo(dialog, SWT.NONE | SWT.DROP_DOWN); // |
			// SWT.READ_ONLY
			myCombo.setText("             ");
			Map<String, List<Decision>> map = AdoraPlugin.getActiveModel()
					.getSavedDecisionMap();
			for (String s : map.keySet()) {

				myCombo.add(s);
				myCombo.setTextLimit(30);
				myCombo.select(0);
			}

			myCombo.addModifyListener(new ModifyListener() {
				public void modifyText(final ModifyEvent e) {
					savedName = ((Combo) e.getSource()).getText();
				}
			});
			myCombo.addSelectionListener(new SelectionListener() {
				public void widgetDefaultSelected(SelectionEvent e) {
				}

				public void widgetSelected(SelectionEvent e) {
					selectedPreset = myCombo.getItem(myCombo
							.getSelectionIndex());
				}
			});

			Button saveButton = new Button(dialog, 0);
			saveButton.setText("Save Preset");
			saveButton.addSelectionListener(new SelectionListener() {
				public void widgetSelected(SelectionEvent e) {
					handleSave();
				}

				public void widgetDefaultSelected(SelectionEvent e) {
				}
			});

			Button loadButton = new Button(dialog, 0);
			loadButton.setText("Load Preset");
			loadButton.addSelectionListener(new SelectionListener() {
				public void widgetSelected(SelectionEvent e) {
					handleLoad();
				}

				public void widgetDefaultSelected(SelectionEvent e) {
				}
			});
			Button deleteButton = new Button(dialog, 0);
			deleteButton.setText("Delete Preset");
			deleteButton.addSelectionListener(new SelectionListener() {
				public void widgetSelected(SelectionEvent e) {
					handleDelete();
				}

				public void widgetDefaultSelected(SelectionEvent e) {
				}
			});

			dialog.pack();
			dialog.open();

			// Move the dialog to the center of the top level shell.
			org.eclipse.swt.graphics.Rectangle shellBounds = shell.getBounds();
			Point dialogSize = dialog.getSize();

			dialog.setLocation(shellBounds.x
					+ (shellBounds.width - dialogSize.x) / 2, shellBounds.y
					+ (shellBounds.height - dialogSize.y) / 2);

		}

		private void handleDelete() {
			if (selectedPreset.trim().equals("")) {
				MessageBox mb = new MessageBox(getSite().getWorkbenchWindow()
						.getShell(), SWT.ICON_INFORMATION | SWT.OK);
				mb.setText("Delete Decisions");
				mb.setMessage("Please select a Preset first");
				mb.open();

				return;
			}

			MessageBox mb = new MessageBox(getSite().getWorkbenchWindow()
					.getShell(), SWT.ICON_QUESTION | SWT.OK | SWT.CANCEL);
			mb.setText("Delete Decisions");
			mb.setMessage("Are you sure to delete Preset " + selectedPreset
					+ "?");

			if (mb.open() != SWT.OK)
				return;

			Map<String, List<Decision>> map = AdoraPlugin.getActiveModel()
					.getSavedDecisionMap();
			map.remove(selectedPreset);
			removePreset(selectedPreset);
			selectedPreset = "";

			myCombo.update();
		}

		private void handleSave() {

			if (savedName.trim().equals("")) {
				MessageBox mb = new MessageBox(getSite().getWorkbenchWindow()
						.getShell(), SWT.ICON_INFORMATION | SWT.OK);
				mb.setText("Save Decisions");
				mb.setMessage("Please enter a name for the Preset");
				mb.open();

				return;
			}

			adoraModel = AdoraPlugin.getActiveModel();

			List<Decision> decList = new ArrayList<Decision>();

			adoraModel = AdoraPlugin.getActiveModel();
			Decision sd;
			for (Decision d : adoraModel.getDecisions()) {
				sd = new Decision(d.getNr(), d.getDecisionValue(), d
						.getDescription());
				decList.add(sd);
			}
			adoraModel.addSavedDecisions(savedName, decList);

			if (!contains(savedName))
				addPreset(savedName);
		}

		private void handleLoad() {
			if (selectedPreset.trim().equals("")) {
				MessageBox mb = new MessageBox(getSite().getWorkbenchWindow()
						.getShell(), SWT.ICON_INFORMATION | SWT.OK);
				mb.setText("Load Decisions");
				mb.setMessage("Please enter a name for the Preset to load");
				mb.open();

				return;
			}

			MessageBox mb = new MessageBox(getSite().getWorkbenchWindow()
					.getShell(), SWT.ICON_QUESTION | SWT.OK | SWT.CANCEL);
			mb.setText("Load Decisions");
			mb.setMessage("Are you sure to load Preset " + selectedPreset
					+ "\nNote this overwrites your current configureation");

			if (mb.open() != SWT.OK || selectedPreset.equals(""))
				return;

			adoraModel = AdoraPlugin.getActiveModel();

			for (Decision savedDec : adoraModel
					.getSavedDecisions(selectedPreset)) {
				Decision d = adoraModel.getDecisionsMap().get(savedDec.getNr());
				if (d != null) {
					d.setDescription(savedDec.getDescription());
					d.setDecisionValue(savedDec.getDecisionValue());
				}
			}
		}
	}
	
	
	/**
	 * This {@link Action} is responsible for starting the 
	 * product derivation process.
	 * 
	 * @author ivovigan
	 */
	public class StartProduDerivAction extends Action {
		
		/*
		 * 	(non-Javadoc)
		 * @see org.eclipse.jface.action.Action#run()
		 */
		@Override
		public void run() {

			//copy the model for the product derivation.
			AdoraModel originalModel = AdoraPlugin.getActiveModel();
			//AdoraModel copiedModel = originalModel.backUpModel();
			AdoraModel copiedModel = originalModel.modelCopy();
			
			copiedModel.setModelBeforeProdLine(originalModel);
			AdoraPlugin.getActiveEditor().setModel(copiedModel);
			copiedModel.setProductDerivationStarted(true);
			
	  		try {
				PlatformUI.getWorkbench().getActiveWorkbenchWindow()
						.getActivePage().showView(VIEW_ID_DECISION_HISTORY);
			} catch (PartInitException e) {
				e.printStackTrace();
			}
			
			HideConstraintsAndVPs hideConst = new HideConstraintsAndVPs(
					copiedModel);
			AdoraPlugin.getActiveEditor().getCommandStack().execute(hideConst);
			
			selectedDecisions.clear();
			
			//now update the visibility of the buttons and the palette
			updateProductLineVisisbility(true);
		}
	}
	
	/**
	 * This method updates the start stop buttons when a new 
	 * {@link AdoraModel} has been loaded by the user.
	 * 
	 * @param derivationStarted Stores the information, whether the {@link AdoraModel} is 
	 * in a state where the product derivation has already been started.
	 */
	public void updateProductDerivation(boolean derivationStarted) {
		resetDecisions.setEnabled(derivationStarted);
		startProductDerivationAction.setEnabled(!derivationStarted);
	}
	
	/**
	 * This method is responsible for hiding / showing the 
	 * {@link AdoraModelEditorPaletteFactory}. This is important since
	 * during the product derivation, no new elements should be added because
	 * once the derivation is stopped, these elements are deleted.
	 * 
	 * @param pdStart  true  => palette-elements get hidden
	 * 				   false => palette-elements get shown 
	 */
	private void updateProductLineVisisbility(boolean pdStart) {
		AdoraModelEditorPaletteFactory palette = 
							AdoraPlugin.getActiveEditor().getPaletteFactory();
		if (palette != null) 
			palette.updatePaletteVisisbility(pdStart);
		
		resetDecisions.setEnabled(pdStart);
		startProductDerivationAction.setEnabled(!pdStart);
		addVariationPointAction.setEnabled(!pdStart);

		IWorkbenchPage workbench = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
		ConstraintsViewPart constViewPart = (ConstraintsViewPart)workbench.findView(ConstraintsViewPart.VIEW_ID);
		VariationPointViewPart vpViewPart = (VariationPointViewPart)workbench.findView(VariationPointViewPart.VIEW_ID);

		if (constViewPart != null)
			constViewPart.updateProductDerivation(pdStart);
		if (vpViewPart != null)
			vpViewPart.updateProductDerivation(pdStart);
	}
}