package ch.unizh.ifi.adora.productline.ui.views.variationpointtable;

import org.eclipse.gef.commands.Command;
import org.eclipse.jface.viewers.ICellModifier;
import org.eclipse.swt.widgets.TableItem;

import ch.unizh.ifi.adora.model.commands.SetProperty;
import ch.unizh.ifi.adora.model.commands.SetVariationPointListDecisions;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.variability.VariationPoint;
import ch.unizh.ifi.adora.ui.views.ModelObserverMediator;
import ch.unizh.ifi.adora.productline.ui.views.constraintstable.sat.SatSolver;
import ch.unizh.ifi.adora.productline.ui.views.variationpointtable.VariationPointLabelProvider;
import ch.unizh.ifi.adora.productline.ui.views.variationpointtable.VariationPointViewPart;

/**
 * This class is the {@link ICellModifier} for the {@link VariationPointViewPart}.
 * It returns, if a column can be modified, what the current value is and it
 * modifies the entry.
 * 
 * @author Reinhard Stoiber
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 2008-04-23 RS First Version
 * @version 
 */
public class VariationPointCellModifier implements ICellModifier {
	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.ICellModifier#canModify(java.lang.Object,
     *      java.lang.String)
     */
	public boolean canModify(Object element, String property) {
		
		if (property.equals(VariationPointViewPart.COL_DECISIONS)) {
			return true;
		}

		if (property.equals(VariationPointViewPart.COL_DESCRIPTION)) {
			return true;
		}

		if (property.equals(VariationPointViewPart.COL_MINCARD)
				|| property.equals(VariationPointViewPart.COL_MAXCARD)) {
			return true;
		}

		return false;
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.ICellModifier#getValue(java.lang.Object,
     *      java.lang.String)
     */
	public Object getValue(Object element, String property) {
		VariationPoint variationPoint = (VariationPoint) element;
		if (property.equals(VariationPointViewPart.COL_DESCRIPTION)) {
			return variationPoint.getDescription();
		}

		if (property.equals(VariationPointViewPart.COL_MINCARD)) {
			return variationPoint.getMinCard();
		}
		
		if (property.equals(VariationPointViewPart.COL_MAXCARD)) {
			return variationPoint.getMaxCard();
		}
		
		if (property.equals(VariationPointViewPart.COL_DECISIONS)) {
			return VariationPointLabelProvider.getString(variationPoint.getDecisions());
		}

		return null;
	}

	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.ICellModifier#modify(java.lang.Object,
     *      java.lang.String, java.lang.Object)
     */
	public void modify(Object element, String property, Object value) {
		Command command = null;
		
		Command undoCommand = null;
		
		VariationPoint variationPoint = (VariationPoint) ((TableItem) element).getData();

		if (property.equals(VariationPointViewPart.COL_DESCRIPTION)) {
			try {
				command = new SetProperty(variationPoint, value, variationPoint
						.getClass().getMethod("setDescription",
								new Class[] { String.class }), variationPoint
						.getClass().getMethod("getDescription", new Class[] {}));
				
				undoCommand = new SetProperty(variationPoint, variationPoint.getDescription(), variationPoint
						.getClass().getMethod("setDescription",
								new Class[] { String.class }), variationPoint
						.getClass().getMethod("getDescription", new Class[] {}));
				
				
				
			} catch (SecurityException e) {
				e.printStackTrace();
			} catch (NoSuchMethodException e) {
				e.printStackTrace();
			}
		}
		
		if (property.equals(VariationPointViewPart.COL_MINCARD)) {
			
			if (variationPoint.getMinCard().equals(value))
				return;
			
			try {
				command = new SetProperty(variationPoint, value, variationPoint
						.getClass().getMethod("setMinCard",
								new Class[] { String.class }), variationPoint
						.getClass().getMethod("getMinCard", new Class[] {}));
			
			
			} catch (SecurityException e) {
				e.printStackTrace();
			} catch (NoSuchMethodException e) {
				e.printStackTrace();
			}

			//new SatSolver().solve();
		}
		
		if (property.equals(VariationPointViewPart.COL_MAXCARD)) {
			
			if (variationPoint.getMaxCard().equals(value))
				return;
			
			try {
				command = new SetProperty(variationPoint, value, variationPoint
						.getClass().getMethod("setMaxCard",
								new Class[] { String.class }), variationPoint
						.getClass().getMethod("getMaxCard", new Class[] {}));
				
				undoCommand = new SetProperty(variationPoint, variationPoint.getMaxCard(), variationPoint
						.getClass().getMethod("setDescription",
								new Class[] { String.class }), variationPoint
						.getClass().getMethod("getDescription", new Class[] {}));
			
				
			} catch (SecurityException e) {
				e.printStackTrace();
			} catch (NoSuchMethodException e) {
				e.printStackTrace();
			}
			
			//new SatSolver().solve();
		}

		if (property.equals(VariationPointViewPart.COL_DECISIONS)) {
			command = new SetVariationPointListDecisions((String) value, variationPoint);
		}

		if (command != null) {
			ModelObserverMediator.getInstance().getAdoraModelEditor()
			.getCommandStack().execute(command);
		}
		
		// update truthvalues and check if unsatisfiable, 
		// ifso reset to old values 
		AdoraModel model = ModelObserverMediator.getInstance()
								.getAdoraModelEditor().getModel();

		
		if (!(new SatSolver(model).solve())) {
			ModelObserverMediator.getInstance().getAdoraModelEditor()
			.getCommandStack().execute(undoCommand);
		}
	}
}
