package ch.unizh.ifi.adora.productline.ui.views.variationpointtable;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.gef.EditPart;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ColumnWeightData;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableLayout;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.ViewPart;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.commands.VariationPointRemove;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.variability.Constraint;
import ch.unizh.ifi.adora.model.representation.variability.Decision;
import ch.unizh.ifi.adora.model.representation.variability.VariationPoint;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;
import ch.unizh.ifi.adora.ui.views.ModelObserverMediator;
import ch.unizh.ifi.adora.productline.ui.views.constraintstable.sat.SatSolver;
import ch.unizh.ifi.adora.productline.ui.views.variationpointtable.VariationPointCellModifier;
import ch.unizh.ifi.adora.productline.ui.views.variationpointtable.VariationPointContentProvider;
import ch.unizh.ifi.adora.productline.ui.views.variationpointtable.VariationPointLabelProvider;
import ch.unizh.ifi.adora.productline.ui.views.variationpointtable.VariationPointViewPart;

/**
 * This class is the view, which can be used for creating Variation Points.
 * 
 * @author Reinhard Stoiber
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 2008-04-22 RS First Version
 * @version 
 */
public class VariationPointViewPart extends ViewPart {
	
	/**
     * The ID of this view.
     */
	public static final String VIEW_ID = "ch.unizh.ifi.adora.productline.views.VariationPoints";

	/**
     * The identifier of the Nr Column of this tableview.
     */
	protected static final String COL_NR = "NR";

	/**
     * The identifier of the Description Column of this tableview.
     */
	protected static final String COL_DESCRIPTION = "Description";

	/**
     * The identifier of the Minimum Cardinality Column of this tableview.
     */
	protected static final String COL_MINCARD = "minCard";

	/**
     * The identifier of the Maximum Cardinality Column of this tableview.
     */
	protected static final String COL_MAXCARD = "maxCard";

	/**
     * The identifier of the "Decision Involved" Column of this tableview.
     */
	protected static final String COL_DECISIONS = "Decisions Involved";

	/**
     * The {@link TableViewer} used.
     */
	private TableViewer viewer;

	/**
     * The {@link VariationPointContentProvider} used.
     */
	private VariationPointContentProvider contentProvider;
	
	/**
	 * Toolbar Actions
	 */
	private Action deleteVariationPointAction;
	
	private AdoraModel adoraModel;

	/**
     * The constructor.
     */
	public VariationPointViewPart() {
	}
	
    /**
     * @see IViewPart.init(IViewSite)
     */
	@Override
	public void init(IViewSite site) throws PartInitException {
            super.init(site);
            // Normally we might do other stuff here.
    }


	/*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.part.WorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
     */
	@Override
	public void createPartControl(Composite parent) {
		
		this.addChildControls(parent);

		Table table = new Table(parent, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);

		table.setLinesVisible(true);
		table.setHeaderVisible(true);

		TableLayout layout = new TableLayout();
		layout.addColumnData(new ColumnWeightData(2, true)); // number
		layout.addColumnData(new ColumnWeightData(20, true)); // description
		layout.addColumnData(new ColumnWeightData(4, true)); // minCard
		layout.addColumnData(new ColumnWeightData(4, true)); // maxCard
		layout.addColumnData(new ColumnWeightData(15, true)); // DecisionsInvolved

		table.setLayout(layout);

		TableColumn nrCol = new TableColumn(table, SWT.NONE);
		nrCol.setText(COL_NR);
		nrCol.setResizable(true);

		TableColumn descriptionCol = new TableColumn(table, SWT.NONE);
		descriptionCol.setText(COL_DESCRIPTION);
		descriptionCol.setResizable(true);

		TableColumn minCardCol = new TableColumn(table, SWT.NONE);
		minCardCol.setText(COL_MINCARD);
		minCardCol.setResizable(true);

		TableColumn maxCardCol = new TableColumn(table, SWT.NONE);
		maxCardCol.setText(COL_MAXCARD);
		maxCardCol.setResizable(true);

		TableColumn decisionsCol = new TableColumn(table, SWT.NONE);
		decisionsCol.setText(COL_DECISIONS);
		decisionsCol.setResizable(true);

		viewer = new TableViewer(table);
		contentProvider = new VariationPointContentProvider(viewer, getSite()
				.getPage());
		viewer.setContentProvider(contentProvider);
		viewer.setLabelProvider(new VariationPointLabelProvider());
		viewer.setInput(getViewSite());
		viewer.addSelectionChangedListener(new ISelectionChangedListener() {

			@SuppressWarnings("unchecked")
			public void selectionChanged(SelectionChangedEvent event) {
				if (event.getSelection().isEmpty()) {
					return;
				}

				if (!(event.getSelection() instanceof IStructuredSelection)) {
					return;
				}

				ArrayList selectedModelElements = new ArrayList();
				IStructuredSelection selection = (IStructuredSelection) event
						.getSelection();

				Iterator i = selection.iterator();
				while (i.hasNext()) {
					//finds out the involved join relations... (from the involved decisions)
					for(Decision decision : ( (ArrayList<Decision>) ( (VariationPoint) i.next() ).getDecisions())) {
						/*if(!selectedModelElements.contains(decision.getJoinRelation())){
							selectedModelElements.add(decision.getJoinRelation());
						}*/
						//Updated code s.t. it works for Decisions containing several JR
						for (JoinRelation jr : decision.getJoinRelations()) {
							if(!selectedModelElements.contains(jr)) {
								selectedModelElements.add(jr);
							}
						}
					}
				}

				AdoraModelEditor ame = AdoraPlugin.getActiveEditor();
				if (ame != null) {
					if (alreadySelected(ame, selectedModelElements)) {
						return;
					}

					ame.selectElements(selectedModelElements);
				}
			}

			/**
             * Checks, if the list of selectedModelElements is already selected
             * in the editor.
             * 
             * @param ame
             * @param selectedModelElements
             * 
             * @return true if already selected.
             */
			private boolean alreadySelected(AdoraModelEditor ame, List selectedModelElements) {
				if (ame.getViewer().getSelectedEditParts().size() != selectedModelElements
						.size()) {
					return false;
				}

				boolean equals = true;
				for (Object object : ame.getViewer().getSelectedEditParts()) {
					EditPart part = (EditPart) object;
					if (!selectedModelElements.contains(part.getModel())) {
						equals = false;
					}
				}

				return equals;
			}

		});

		CellEditor[] editors = new CellEditor[5];

		TextCellEditor nrEditor = new TextCellEditor(table);
		((Text) nrEditor.getControl()).setTextLimit(5);
		editors[0] = nrEditor;

		TextCellEditor descriptionEditor = new TextCellEditor(table);
		((Text) descriptionEditor.getControl()).setTextLimit(200);
		editors[1] = descriptionEditor;

		TextCellEditor minCardEditor = new TextCellEditor(table);
		((Text) minCardEditor.getControl()).setTextLimit(2);
		editors[2] = minCardEditor;

		TextCellEditor maxCardEditor = new TextCellEditor(table);
		((Text) maxCardEditor.getControl()).setTextLimit(2);
		editors[3] = maxCardEditor;

		TextCellEditor decicionsInvolvedEditor = new TextCellEditor(table);
		((Text) decicionsInvolvedEditor.getControl()).setTextLimit(60);
		editors[4] = decicionsInvolvedEditor;

		String[] columnProperties = { COL_NR, COL_DESCRIPTION,
				COL_MINCARD, COL_MAXCARD, COL_DECISIONS };

		viewer.setColumnProperties(columnProperties);
		viewer.setCellEditors(editors);
		viewer.setCellModifier(new VariationPointCellModifier());
	}

	/**
     * Returns the {@link VariationPointContentProvider} of this view.
     * 
     * @return The {@link VariationPointContentProvider} of this view.
     */
	public VariationPointContentProvider getContentProvider() {
		return contentProvider;
	}

	/**
     * Passing the focus request to the viewer's control.
     */
	@Override
	public void setFocus() {
		viewer.getControl().setFocus();
	}
	
	/**
	 * Refreshes this viewer completely with information freshly obtained from this viewer's model.
	 */
	public void refresh() {
		viewer.refresh();
	}
	
	
	
	/**
	 * creates the controls for the parent composite
	 * @param Composite parent
	 */
	private void addChildControls(Composite parent){

		createToolbarActions(parent);
		createToolBar();
		
	}
	
	/**
	 * creates the view toolbar and adds actions to it
	 */
	private void createToolBar(){
		IToolBarManager mgr = getViewSite().getActionBars().getToolBarManager();
		mgr.add(deleteVariationPointAction);
	}
	
	/**
	 * creates the toolbar actions and sets the action image
	 */
	public void createToolbarActions (final Composite parent){		

		deleteVariationPointAction = new Action("DeleteVP") {
			
			@Override
			public void run() {
				
				adoraModel = AdoraPlugin.getActiveModel();
				AdoraModelEditor ame = AdoraPlugin.getActiveEditor();
								
				ArrayList<VariationPoint> variationPointsToRemove = new ArrayList<VariationPoint>();
				variationPointsToRemove.clear();

				TableItem[] selection = viewer.getTable().getSelection();
				
				for (TableItem item : selection) {
					if (item.getData() instanceof VariationPoint) {
						variationPointsToRemove.add((VariationPoint) item.getData());
					}
				}
				
				VariationPointRemove removeCommand = new VariationPointRemove(adoraModel, variationPointsToRemove);
				ame.getCommandStack().execute(removeCommand);
				
				VariationPointViewPart variationPointViewPart = (VariationPointViewPart) ame.getSite().getPage().findView(VIEW_ID);
			
				variationPointViewPart.refresh();
				variationPointViewPart.setFocus();
				
				AdoraModel model = ModelObserverMediator.getInstance()
										.getAdoraModelEditor().getModel();
				new SatSolver(model).solve();
				
				viewer.refresh();
				
			}
		};
	}
	
	/**
	 * This method is responsible for enabling/disabling the {@link VariationPoint} deletion button.
	 * 
	 * @param pdStart  true  => palette-elements get hidden
	 * 				   false => palette-elements get shown 
	 */
	public void updateProductDerivation(boolean derivationStarted) {
		deleteVariationPointAction.setEnabled(!derivationStarted);
	}
}