package ch.unizh.ifi.adora.requirements.commands;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.model.representation.requirements.ImplementationProposal;

/**
 * This class is the edit command for the implementation proposal. Whenever an
 * attribute of an implementation proposal is changed, this command will be
 * executed.
 * 
 * @author Jaro Habr
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 2008-08-11 JH First Version
 */
public class ImplPropEdit extends Command {
	
	/**
	 * Saves the column ID.
	 */
	private int column;
	
	/**
	 * Saves the new string to be set.
	 */
	private String newString;
	
	/**
	 * Saves the implementation proposal.
	 */
	private ImplementationProposal implProp;
	
	/**
	 * The constructor.
	 * 
	 * @param int column
	 * @param String newString
	 * @param ImplementationProposal implProp
	 */
	public ImplPropEdit(int column, String newString,
			ImplementationProposal implProp) {
		this.column = column;
		this.newString = newString;
		this.implProp = implProp;
	}
	
	/**
	 * Executes the command.
	 */
	public void execute() {
		if (column == 1) { // Title-column
			implProp.setTitle(newString);
		} else if (column == 2) { // RequirementID-column
			implProp.setRequirementIDs(newString);
		} else if (column == 4) { // Assumptions-column
			implProp.setAssumption(newString);
		} else if (column == 5) { // Impact-column
			implProp.setImpact(newString);
		} else if (column == 6) { // Dismissed Designs-column
			implProp.setDismissedDesigns(newString);
		} else if (column == 7) { // Open Issues-column
			implProp.setOpenIssues(newString);
		} else if (column == 8) { // Comment-column
			implProp.setComment(newString);
		}
	}

	
	public void redo() {
		//TODO
	}

	
	public void undo() {
		//TODO
	}

}
