package ch.unizh.ifi.adora.requirements.commands;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.requirements.Requirement;

/**
 * This Class represents the Add-Command for adding new requirements to the
 * adora model.
 * 
 * @author Jaro Habr
 *
 */
public class RequirementAdd extends Command {
	
	/**
	 * Holds the requirement.
	 */
	private Requirement requirement;
	
	/**
	 * Holds the current Adora Model.
	 */
	private AdoraModel adoraModel;
	
	/**
	 * The constructor.
	 * 
	 * @param AdoraModel adoraModel
	 */
	public RequirementAdd(AdoraModel adoraModel) {
		this.adoraModel = adoraModel;
		requirement = new Requirement(adoraModel);		
	}
	
	/**
	 * Executes the command by adding a new requirement to the adora model.
	 */
	public void execute() {
		adoraModel.addRequirement(requirement);	
	}
	
	/**
	 * Redoes the adding command.
	 */
	public void redo() {
		adoraModel.addRequirement(requirement);
	}

	/**
	 * Undoes the added requirement a returns the adora model to the previous
	 * state.
	 */
	public void undo() {
		adoraModel.removeRequirement(requirement);
	}
}
