package ch.unizh.ifi.adora.requirements.ui.views.DesignDecisionTable;

import java.util.Vector;

import org.eclipse.gef.commands.Command;
import org.eclipse.jface.viewers.ICellModifier;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.TableItem;

import ch.unizh.ifi.adora.model.representation.requirements.DesignDecision;
import ch.unizh.ifi.adora.requirements.commands.DesignDecisionEdit;

public class DesignDecisionCellModifier implements ICellModifier {

	private DesignDecisionViewPart designDecisionViewPart;
	private int columnIndex;
	private Vector<String> requirementIDs;
	private String[] typeChoices;
	private Command command;

	public DesignDecisionCellModifier(
			DesignDecisionViewPart designDecisionViewPart) {
		super();
		this.designDecisionViewPart = designDecisionViewPart;
	}

	public boolean canModify(Object element, String property) {
		return true;
	}

	public Object getValue(Object element, String property) {
		// Find the index of the column
		columnIndex = designDecisionViewPart.getColumnNames().indexOf(property);

		Object result = null;
		DesignDecision designDecision = (DesignDecision) element;

		switch (columnIndex) {
		case 0: //Id column
			result = designDecision.getId();
			break;

		case 1: // Description column
			result = designDecision.getDescription();
			break;

		case 2: // ModelElement column
			result = designDecision.getScope();
			break;

		case 3: // Type column		
			typeChoices = designDecisionViewPart.getTypeChoices();
			String type = designDecision.getType();
			for (int i = 0; i < typeChoices.length; i++) {
				if (type.equals(typeChoices[i])) {
					result = new Integer(i);
					break;
				}
			}
			break;

		case 4: // requirement ID column 
			result = designDecision.getRequirementIDs();
			break;
			
		case 5: // comment column 
			//TODO -> full text editor
//			AbstractLongTextEditorInput input = buildEditorInput();
//            IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
//			page.openEditor(input, input.getLongTextEditorID(), true);
			result = designDecision.getComment();
			break;

		default:
			result = "";
		}
		return result;
	}

	/**
	 * This method is responsible for modifying the content of the cells
	 */

	public void modify(Object element, String property, Object value) {
		// Find the index of the column 
		columnIndex = designDecisionViewPart.getColumnNames().indexOf(property);

		TableItem item = (TableItem) element;
		DesignDecision designDecision = (DesignDecision) item.getData();
		String valueString;

		switch (columnIndex) {
		
		case 1: // Description-column
			valueString = ((String) value).trim();
			command = new DesignDecisionEdit(1, valueString, designDecision);
			break;

		case 3: // Type-column
			valueString = typeChoices[((Integer) value).intValue()];
			if (!designDecision.getType().equals(valueString)) {
				command = new DesignDecisionEdit(3, valueString, designDecision);
			}
			break;

		case 4: // RequirementID-column
			valueString = ((String) value).trim();
			boolean isSet = false;
			MessageBox messageBox;
			String message;
			requirementIDs = designDecisionViewPart.getRequirementIDs();

			// sigle Requirement ID
			if (valueString.equals("") || valueString.equals("?")) {
				command = new DesignDecisionEdit(4, valueString, designDecision);
			} else if (!valueString.contains(";")) {
				while (isSet == false) {
					for (int i = 0; i < requirementIDs.size(); i++) {
						String requirementID = requirementIDs.elementAt(i);

						if (requirementID.equals(valueString)) {
							command = new DesignDecisionEdit(4, valueString,
									designDecision);
							isSet = true;
						}
					}

					if (isSet == false) {
						isSet = true;
						messageBox = new MessageBox(designDecisionViewPart
								.getSite().getShell(), SWT.ICON_WARNING
								| SWT.OK);
						messageBox.setText("Notification");
						message = "The Requirement ID " + value
								+ " is not valid!";
						messageBox.setMessage(message);
						messageBox.open();
					}
				}
			} else if (valueString.contains(";")) { // multiple Requirement IDs
				int start = 0;
				int end = 4;
				String result = "";
				while (isSet == false) {
					while (end <= valueString.length()) {
						String reqID = valueString.substring(start, end);

						for (int i = 0; i < requirementIDs.size(); i++) {
							String requirementID = requirementIDs.elementAt(i);

							if (requirementID.equals(reqID)) {
								result = result + requirementID + ";";
								break;
							}
						}
						start += 5;
						end += 5;
					}

					if (result.length() < 1) {
						messageBox = new MessageBox(designDecisionViewPart
								.getSite().getShell(), SWT.ICON_WARNING
								| SWT.OK);
						messageBox.setText("Notification");
						message = "The Requirement IDs " + value
								+ " are not valid!";
						messageBox.setMessage(message);
						messageBox.open();
					} else {
						command = new DesignDecisionEdit(4, result.substring(0,
								result.length() - 1), designDecision);
					}
					isSet = true;
				}
			}
			break;
			
		case 5: // Comment-column
			valueString = ((String) value).trim();
			command = new DesignDecisionEdit(5, valueString, designDecision);
			break;
		}
		designDecisionViewPart.getActiveEditor().getCommandStack().execute(command);
		designDecisionViewPart.getViewer().update(designDecision, null);
	}
}
