package ch.unizh.ifi.adora.requirements.ui.views.DesignDecisionTable;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.eclipse.gef.EditPart;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.jface.viewers.ComboBoxCellEditor;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.requirements.DesignDecision;
import ch.unizh.ifi.adora.model.representation.requirements.ImplementationProposal;
import ch.unizh.ifi.adora.model.representation.requirements.Requirement;
import ch.unizh.ifi.adora.requirements.RequirementsPlugIn;
import ch.unizh.ifi.adora.requirements.commands.DesignDecisionAdd;
import ch.unizh.ifi.adora.requirements.commands.DesignDecisionRemove;
import ch.unizh.ifi.adora.requirements.ui.views.ImplPropTable.ImplPropViewPart;
import ch.unizh.ifi.adora.requirements.ui.views.RequirementsTable.RequirementsViewPart;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;

/**
 * This class represents the requirements table.
 * 
 * @author 		Jaro Habr
 * @copyright 	Requirements Engineering Research Group
 *            	University of Zurich
 *            	Department of Informatics
 * @history 	2008-06-19 JH First Version
 * @version 	$Date: 2008-09-20 15:08:56 +0200 (Sat, 20 Sep 2008) $, $Author: jaro $
 */
public class DesignDecisionViewPart extends ViewPart {
	
	/**
     * The ID of this view.
     */
	public static final String DESIGN_DECISION_VIEW_ID = "ch.unizh.ifi.adora.requirements.DesignDecisionTable";
	
	/**
	 * The Description of the COL_ID column.
	 */
	private static final String COL_ID = "ID";
	
	/**
	 * The Description of the COL_DESCRIPTION column.
	 */
	private static final String COL_DESCRIPTION = "Description";
	
	/**
	 * The Description of the COL_MODEL_ELEMENTS column.
	 */
	private static final String COL_MODEL_ELEMENTS = "Scope";
	
	/**
	 * The Description of the COL_TYPE column.
	 */
	private static final String COL_TYPE = "Type";
	
	/**
	 * The Description of the COL_REQUIREMENT_ID column.
	 */
	private static final String COL_REQUIREMENT_ID = "Requirement ID";
	
	/**
	 * The Description of the COL_COMMENT column.
	 */
	private static final String COL_COMMENT = "Comment";
	
	/**
	 * Holds the references to the column descriptions.
	 */
	private String[] columnNames = new String[] {
			COL_ID, 
			COL_DESCRIPTION,
			COL_MODEL_ELEMENTS,
			COL_TYPE,
			COL_REQUIREMENT_ID,
			COL_COMMENT};
	
	/**
	 * The Design Decison table.
	 */
	private Table table;
	
	/**
	 * The viewer of the Design Decision table.
	 */
	private TableViewer viewer;
	
	/**
	 * The toolbar actions.
	 */
	private Action addDesignDecisionAction, deleteDesignDecisionAction,
			searchAction, filterAction, associateModelElementAction,
			dissociateModelElementAction, exportDesignDecisionsAction;
	
	/**
	 * The active adora editor.
	 */
	private AdoraModelEditor activeAdoraEditor;
	
	/**
	 * The active adora model.
	 */
	private AdoraModel adoraModel;
	
	/**
	 * Holds the designDecisions vector saved in the adora model.
	 */
	private Vector<DesignDecision> designDecisions = new Vector<DesignDecision>();
	
	/**
	 * Holds the selected requirements in the table.
	 */
	private Vector<DesignDecision> selection = new Vector<DesignDecision>();
	
	/**
	 * Holds the current requirement IDs list.
	 */
	private List<String> requirementIDs = new Vector<String>();
	
	/**
	 * Contains the choices for the type choices.
	 */
	private String[] typeChoices = { "?", "Existence-Functionality",
			"Existence-Parts", "Existence-Data", "Properties-Design Rule",
			"Properties-Technology Use", "Structure-Relationships",
			"Structure-Architectural Style", "Structure-Design Pattern",
			"Texture-Interface", "Texture-Metaphor" };
	
//	private List<EditPart> selectedModelElements = new Vector<EditPart>();
//	private Vector<ConnectableElement> modelElements = new Vector<ConnectableElement>();
	
	/**
	 * The content provider.
	 */
	private DesignDecisionContentProvider contentProvider;
	
	/**
	 * If implementation proposals are selected from the implementation proposal
	 * table viewer, selectedFromDesignDecision is set to true.
	 */
	private boolean selectedFromDesignDecision = false;
	
	/**
	 * IPartListener to listen to active adora model and to update the data from
	 * the adora model.
	 */
	private IPartListener listener = new IPartListener() {

		public void partActivated(IWorkbenchPart part) {
			if (part instanceof AdoraModelEditor) {
				activeAdoraEditor = (AdoraModelEditor) getSite().getPage().getActiveEditor();
				adoraModel = activeAdoraEditor.getModel();
				designDecisions = adoraModel.getDesignDecisions();
				requirementIDs = adoraModel.getRequirementsIDs();
				viewer.refresh();

//				if (!table.isDisposed()) {
//					table.setVisible(true);
//					enableActions();
//					
//				}
				
			}
		}

		public void partBroughtToTop(IWorkbenchPart part) {
		}
		
		//set table invisible, if last editor has been closed
		public void partClosed(IWorkbenchPart part) {
//			if (part instanceof AdoraModelEditor) {
//				if (getSite().getPage().getEditors().length == 0) {
//					if (!table.isDisposed()) {
//						try {
//							table.setVisible(false);
//							disableActions();
//						} catch (Exception e) {
//							e.printStackTrace();
//						}
//					}
//				}
//			}
		}

		public void partDeactivated(IWorkbenchPart part) {
			
		}

		public void partOpened(IWorkbenchPart part) {
		}
	};
	
	/**
	 * Filter for the filter action in the toolbar.
	 */
	final ViewerFilter filter = new ViewerFilter() {
	      public boolean select (
	        Viewer viewer,
	        Object parentElement,
	        Object element) {
	        if(((DesignDecision) element).isSelected()) {
	        	return true;
	        }
	        return false;
	      }
	    };

	/**
     * The DesignDecisionViewPart constructor.
     */
	public DesignDecisionViewPart() {
	}
 
	/**
	 * Creates the part control.
	 * 
	 * @see org.eclipse.ui.part.WorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createPartControl(Composite parent) {			
		this.addChildControls(parent);
//		if (getSite().getPage().getEditors().length == 0) {
//			table.setVisible(false);
//			disableActions();
//		} else { 
			if (getSite().getPage().getActivePart() instanceof AdoraModelEditor) {
//				table.setVisible(true);
				activeAdoraEditor = (AdoraModelEditor) getSite().getPage().getActiveEditor();
				adoraModel = activeAdoraEditor.getModel();
				designDecisions = adoraModel.getDesignDecisions();
				requirementIDs = adoraModel.getRequirementsIDs();
				viewer.refresh();
			}
//		}
		// add listener
		getSite().getPage().addPartListener(listener);
	}
	
	/**
	 * Creates the controls for the parent composite.
	 * 
	 * @param Composite parent
	 */
	private void addChildControls(Composite parent){
		GridLayout layout = new GridLayout(2, false);
		parent.setLayout(layout);
		
		createTable(parent);
		createToolbarActions(parent);
		createToolBar();
		createTableViewer();
		initTableViewer();
	}
	
	/**
	 * Creates the view table and adds the table columns.
	 * 
	 * @param Composite parent
	 */

	private void createTable(Composite parent){
		int style = SWT.CHECK | SWT.H_SCROLL | SWT.V_SCROLL
				| SWT.FULL_SELECTION | SWT.MULTI;

		table = new Table(parent, style);
		
		GridData gridData = new GridData(GridData.FILL_BOTH);
		gridData.grabExcessVerticalSpace = true;
		gridData.horizontalSpan = 3;
		table.setLayoutData(gridData);
		
		table.setHeaderVisible(true);
		table.setLinesVisible(true);
		
		//table check listener
		table.addListener(SWT.Selection, new Listener() {
		      public void handleEvent(Event event) {
		    	  if (event.detail == SWT.CHECK) {
		    		  DesignDecision designDecision = ((DesignDecision)event.item.getData());
		    		  boolean checked = ((TableItem)event.item).getChecked();
		    		  designDecision.setSelected(checked);    		  
		    		  
		    		  if (checked){
		    			  selection.add(designDecision);  
		    		  } else {
		    			  selection.remove(designDecision);
		    		  }
			      }
		      } 
		    });
		
		// ID column
		TableColumn column = new TableColumn(table, SWT.LEFT, 0);		
		column.setText(COL_ID);
		column.setWidth(60);
		column.setResizable(true);
		
		// Desription column
		column = new TableColumn(table, SWT.LEFT, 1);
		column.setText(COL_DESCRIPTION);
		column.setWidth(180);
		column.setResizable(true);
		
		// Model elements column
		column = new TableColumn(table, SWT.LEFT, 2);
		column.setText(COL_MODEL_ELEMENTS);
		column.setWidth(200);
		column.setResizable(true);
		
		// Type column
		column = new TableColumn(table, SWT.LEFT, 3);
		column.setText(COL_TYPE);
		column.setWidth(170);
		column.setResizable(true);
		
		// Requirement ID column
		column = new TableColumn(table, SWT.LEFT, 4);
		column.setText(COL_REQUIREMENT_ID);
		column.setWidth(100);
		column.setResizable(true);
		
		// Comment column
		column = new TableColumn(table, SWT.LEFT, 5);
		column.setText(COL_COMMENT);
		column.setWidth(100);
		column.setResizable(true);
	}
	
	/**
	 * Creates the table viewer and cell editors.
	 */
	private void createTableViewer(){
		viewer = new CheckboxTableViewer(table);
				
		viewer.setUseHashlookup(true);
		viewer.setColumnProperties(columnNames);
		
		// create cell editor
		CellEditor[] editors = new CellEditor[columnNames.length];
		
		// ID column (free text)
//		TextCellEditor textEditor = new TextCellEditor(table);
//		((Text) textEditor.getControl()).setTextLimit(50);
//		editors[0] = textEditor;
		
		//Description column (free text)
		TextCellEditor textEditor = new TextCellEditor(table);
		((Text) textEditor.getControl()).setTextLimit(300);
		editors[1] = textEditor;
		
		//Model element column (free text) -> no editor
		
		// Type column (combobox)
		editors[3] = new ComboBoxCellEditor(table, typeChoices, SWT.READ_ONLY);
		
		// Requirement ID column (combobox)
		textEditor = new TextCellEditor(table);
		((Text) textEditor.getControl()).setTextLimit(100);
		editors[4] = textEditor;
		
		//Comment column
		//TODO -> longTextCellEditor
		textEditor = new TextCellEditor(table);
		((Text) textEditor.getControl()).setTextLimit(500);
		editors[5] = textEditor;
		
		// Assign the cell editors to the viewer 
		viewer.setCellEditors(editors);
		viewer.setCellModifier(new DesignDecisionCellModifier(this));
		viewer.addSelectionChangedListener(new ISelectionChangedListener() {
			@SuppressWarnings("unchecked")
			public void selectionChanged(SelectionChangedEvent event) {
				
				if (event.getSelection().isEmpty()) {
					return;
				}

				if (!(event.getSelection() instanceof IStructuredSelection)) {
					return;
				}

				ArrayList selectedModelElements = new ArrayList();
				IStructuredSelection selection = (IStructuredSelection) event
						.getSelection();
				RequirementsViewPart requirementsViewPart = getRequirementsViewPart();
				ImplPropViewPart implPropViewPart = getImplPropViewPart();
				DesignDecision designDecision = null;
				Vector<Requirement> requirementsToSelect = new Vector<Requirement>();
				Vector<ImplementationProposal> implPropsToSelect = new Vector<ImplementationProposal>();
				
				if (requirementsViewPart != null && requirementsViewPart.getSelectedFromRequirement() == true) {
					requirementsViewPart.setSelectedFromRequirement(false);
				} else if (implPropViewPart != null && implPropViewPart.getSelectedFromImplProp() == true) {
					implPropViewPart.setSelectedFromImplProp(false);
				} else {
					Iterator selectionIterator = selection.iterator();
					while (selectionIterator.hasNext()) {
						designDecision = (DesignDecision) selectionIterator.next();
						if (designDecision.getModelElements().size() != 0) {
							for (int a = 0; a < designDecision.getModelElements().size(); a++) { //iterate through all modelElements of one designDecisions
								if (!selectedModelElements.contains(designDecision.getModelElements().get(a))) {
									selectedModelElements.add(designDecision.getModelElements().get(a)); //add one ConnectableElemente to selected Elements
								}
							}
						}
						
						if (implPropViewPart != null) {
							for (ImplementationProposal implProp : adoraModel.getImplProps()) {
								if (implProp.getDesignDecisions().contains(designDecision)) {
									if (!implPropsToSelect.contains(implProp)) {
										implPropsToSelect.add(implProp);
									}
								}
							}
						}
						
						//get requirements associated with the selected design decision in DD Table
						try {
							if (designDecision.getRequirements().size() != 0 && designDecision.getRequirements() != null) {
								for (Requirement requirement : designDecision.getRequirements()) {
									requirementsToSelect.add(requirement);
								}
							}
						} catch (NullPointerException e) {
							e.printStackTrace();
						}
					}
									
					//mark model elements in the adora model
					AdoraModelEditor adoraModelEditor = AdoraPlugin.getActiveEditor();
					if (adoraModelEditor != null) {
						if (alreadySelected(adoraModelEditor, selectedModelElements)) {
							return;
						}
						adoraModelEditor.selectElements(selectedModelElements);
					}
					
					//mark associated requirements in the requirements Table
					if (requirementsViewPart != null) {
						setSelectedFromDesignDecision(true);
						requirementsViewPart.getViewer().setSelection(new StructuredSelection(requirementsToSelect));
					} else {
						//TODO
						System.out.println("Please open the requirements table first!");
					}
					
					if (implPropViewPart != null) {
						setSelectedFromDesignDecision(true);
						implPropViewPart.getViewer().setSelection(new StructuredSelection(implPropsToSelect));
					} else {
						//TODO
						System.out.println("Please open the implPropTable first!");
					}
				}
			}

			/**
             * Checks, if the list of selectedModelElements is already selected
             * in the editor.
             * 
             * @param adoraModelEditor
             * @param selectedModelElements
             * 
             * @return true if already selected.
             */
			private boolean alreadySelected(AdoraModelEditor adoraModelEditor,
					List selectedModelElements) {
				if (adoraModelEditor.getViewer().getSelectedEditParts().size() != selectedModelElements
						.size()) {
					return false;
				}

				boolean equals = true;
				for (Object object : adoraModelEditor.getViewer().getSelectedEditParts()) {
					EditPart part = (EditPart) object;
					if (!selectedModelElements.contains(part.getModel())) {
						equals = false;
					}
				}

				return equals;
			}

		});
	}
	
	/**
	 * Initializes the tableViewer.
	 */
	private void initTableViewer() {
		contentProvider = new DesignDecisionContentProvider(viewer, getSite().getPage());
		viewer.setContentProvider(contentProvider);
		viewer.setLabelProvider(new DesignDecisionLabelProvider());
		viewer.setInput(designDecisions);
	}
	
	/**
	 * Creates the view toolbar and adds actions to it.
	 */
	private void createToolBar(){
		IToolBarManager mgr = getViewSite().getActionBars().getToolBarManager();
		mgr.add(addDesignDecisionAction);
		mgr.add(deleteDesignDecisionAction);
		mgr.add(new Separator());
		mgr.add(associateModelElementAction);
		mgr.add(dissociateModelElementAction);
		mgr.add(new Separator());
		mgr.add(filterAction);
		mgr.add(searchAction);
		mgr.add(new Separator());
		mgr.add(exportDesignDecisionsAction);
	}
	
	/**
	 * Creates the toolbar actions and sets the action image.
	 */
	public void createToolbarActions (final Composite parent){
		
		// add action
		addDesignDecisionAction = new Action("Add Design Decision") {
			public void run() {
				DesignDecisionAdd addCommand = new DesignDecisionAdd(adoraModel);
				activeAdoraEditor.getCommandStack().execute(addCommand);
				if (contentProvider.getSelectedModelElements().size() != 0) {
					DesignDecision designDecision = addCommand.getDesignDecision();
					designDecision.setModelElements(contentProvider.getSelectedModelElements());
					contentProvider.processScope(designDecision.getModelElements(), designDecision);
				}
				viewer.refresh();
			}
		};
		addDesignDecisionAction.setImageDescriptor(ImageDescriptor.createFromFile(RequirementsPlugIn.class, "icons/add.gif"));
		
		// delete action with message Box
		deleteDesignDecisionAction = new Action("Delete Design Decision") {
			public void run() {		        
				MessageBox messageBox;
		        String message = new String();
		        		        
		        if (selection.size() == 0){
		        	messageBox = new MessageBox(parent.getShell(), SWT.ICON_WARNING | SWT.OK);
		        	messageBox.setText("Notification");
		        	message = "Please check a Design Decision first!";
		        } else {
		        	messageBox = new MessageBox(parent.getShell(), SWT.ICON_QUESTION | SWT.YES | SWT.NO);
			        messageBox.setText("Confirmation");
			        message = "Are you sure to remove the ";
		        	
			        if (selection.size() == 1) {
			        	message += "selected Design Decision (" + selection.get(0).getId() + ")?";
			        } else {
			        	message += selection.size() + " selected Design Decisions?";
			        }
		        }
			    
		        messageBox.setMessage(message);
		        
		        if (messageBox.open() == SWT.YES) {
		          for(int i = 0; i < selection.size(); i++){
		        	  DesignDecisionRemove removeCommand = new DesignDecisionRemove(adoraModel, selection.elementAt(i));
		        	  activeAdoraEditor.getCommandStack().execute(removeCommand);
		        	  viewer.refresh();
					}
		          //clear the selection vector -> no elemets are selected
		          selection.removeAllElements();
		          }
		        }
		};
		ISharedImages sharedImages = PlatformUI.getWorkbench().getSharedImages();
		deleteDesignDecisionAction.setImageDescriptor(sharedImages.getImageDescriptor(ISharedImages.IMG_TOOL_DELETE));
		
		//search action
		searchAction = new Action("Search..."){
			public void run(){
				//TODO
			}
		};
		searchAction.setImageDescriptor(ImageDescriptor.createFromFile(RequirementsPlugIn.class, "icons/search.gif"));
		
		//filter action
		filterAction = new Action("Filter Design Decisions") {
			public void run() {
				// isChecked() == false
				if (isChecked()) { // isChecked() == true after klicking on action
					viewer.addFilter(filter);
				} else {
					viewer.removeFilter(filter);
				}
			}
		};
		filterAction.setChecked(false);
		filterAction.setImageDescriptor(ImageDescriptor.createFromFile(RequirementsPlugIn.class, "icons/filter.gif"));
		
		//associate requirement ID action
		associateModelElementAction = new Action("Associate Model Elements") {
			public void run() {
				MessageBox messageBox = new MessageBox(parent.getShell(), SWT.ICON_WARNING | SWT.OK);
				messageBox.setText("Notification");
				setFocus();
				
				if (selection.size() == 0) {
					messageBox.setMessage("Please check a Design Decision first!");
					messageBox.open();
				} else {
					if (contentProvider.getSelectedModelElements().size() == 0) {
						messageBox.setMessage("Please select a Model Element first!");
						messageBox.open();
					} else {
						for (DesignDecision designDecision : selection) {
							if (designDecision.getModelElements().containsAll(contentProvider.getSelectedModelElements())) {
								messageBox.setMessage("The selected Design Decision (" + designDecision.getId() + ") is already associated with the selected Model Elements!");
								messageBox.open();
							} else {
								//sets the selected model elements to the requirement
								designDecision.setModelElements((Vector<ConnectableElement>) contentProvider.getSelectedModelElements());
								contentProvider.processScope(designDecision.getModelElements(), designDecision);
							}
						}
					}
				}
				viewer.refresh();
			}
		};
		associateModelElementAction.setImageDescriptor(ImageDescriptor.createFromFile(RequirementsPlugIn.class, "icons/associate_2.gif"));
		
		//dissociate requirement ID action
		dissociateModelElementAction = new Action("Dissociate Model Elements") {
			public void run() {
				MessageBox messageBox = new MessageBox(parent.getShell(), SWT.ICON_WARNING | SWT.OK);
				messageBox.setText("Notification");
				setFocus();
				if (selection.size() == 0) {
					messageBox.setMessage("Please select a Design Decision first!");
					messageBox.open();
				} else {
					if (contentProvider.getSelectedModelElements().size() == 0) {
						messageBox.setMessage("Please select a Model Element first!");
						messageBox.open();
					} else {
						for (DesignDecision designDecision : selection) {
							for (ConnectableElement element : contentProvider.getSelectedModelElements()) {
								if (designDecision.getModelElements().contains(element)) {
									designDecision.getModelElements().remove(element);
								} else {
									String message = "The selected Design Decision (" + designDecision.getId() + ") is not associated with the selected Model Element";
									if (contentProvider.getSelectedModelElements().size() == 1) {
										messageBox.setMessage(message + "!");
									} else {
										messageBox.setMessage(message + "s!");
									}
									messageBox.open();
								}
							}
							contentProvider.processScope(designDecision.getModelElements(), designDecision);
						}
					}
				}
				viewer.refresh();			
			}
		};
		dissociateModelElementAction.setImageDescriptor(ImageDescriptor.createFromFile(RequirementsPlugIn.class, "icons/dissociate_2.gif"));
		
		//import action
		exportDesignDecisionsAction = new Action("Export Design Decisions to Excel") {
			public void run() {
				//TODO
			}
		};
		exportDesignDecisionsAction.setImageDescriptor(ImageDescriptor.createFromFile(RequirementsPlugIn.class, "icons/export.gif"));
	}
	
	/**
	 * Disables actions in the toolbar.
	 */
	public void disableActions() {
		addDesignDecisionAction.setEnabled(false);
		deleteDesignDecisionAction.setEnabled(false);
		searchAction.setEnabled(false);
		filterAction.setEnabled(false);
		associateModelElementAction.setEnabled(false);
		dissociateModelElementAction.setEnabled(false);
		exportDesignDecisionsAction.setEnabled(false);
	}
	
	/**
	 * Enables actions in the toolbar if disabled.
	 */
	private void enableActions() {
		addDesignDecisionAction.setEnabled(true);
		deleteDesignDecisionAction.setEnabled(true);
		searchAction.setEnabled(true);
		filterAction.setEnabled(true);
		associateModelElementAction.setEnabled(true);
		dissociateModelElementAction.setEnabled(true);
		exportDesignDecisionsAction.setEnabled(true);
	}
	
	/**
	 * Returns column names as an array.
	 * 
	 * @return List<String> columnNames
	 */
	public List<String> getColumnNames() {
		return Arrays.asList(columnNames);
	}
	
	
	/**
	 * Sets the keyboard focus.
	 * 
	 */
	public void setFocus() {
		table.getParent().setFocus();
	}
	
	/**
	 * Returns the table viewer.
	 * 
	 * @return TableViewer viewer
	 */
	public TableViewer getViewer(){
		return viewer;
	}
	
	/**
	 * Returns the design decision type choices.
	 * 
	 * @return String[] typeChoices
	 */
	public String[] getTypeChoices(){
		return typeChoices;
	}
	
	/**
	 * Returns the design decisions list.
	 * 
	 * @return Vector<DesignDecision> designDecisions
	 */
	public Vector<DesignDecision> getDesignDecisions() {
		return designDecisions;
	}
	
	/**
	 * Returns the requirement IDs list.
	 * 
	 * @return Vector<String> requirementIDs
	 */
	public Vector<String> getRequirementIDs() {
		return (Vector<String>) requirementIDs;
	}
	
	/**
	 * Returns the selectedFromDesignDecision boolean value. Used to identify
	 * the source of the selection event.
	 * 
	 * @return boolean selectedFromDesignDecision
	 */
	public boolean getSelectedFromDesignDecision() {
		return selectedFromDesignDecision;
	}
	
	/**
	 * Sets the boolean value of selectedFromImplProp. Used to identify the
	 * source of the selection event.
	 * 
	 * @param boolean b
	 */
	public void setSelectedFromDesignDecision(boolean b) {
		selectedFromDesignDecision = b;
	}
	
	/**
	 * Returns the filter action.
	 * 
	 * @return Action filterAction
	 */
	public Action getFilterAction() {
		return filterAction;
	}
	
	/**
	 * Returns the active adoraModelEditor.
	 * 
	 * @return AdoraModelEditor activeAdoraEditor
	 */
	public AdoraModelEditor getActiveEditor() {
		return activeAdoraEditor;
	}
	
	/**
	 * Returns the requirementViewPart.
	 * 
	 * @return RequirementsViewPart requirementsViewPart
	 */
	public RequirementsViewPart getRequirementsViewPart() {
		return (RequirementsViewPart) PlatformUI
		.getWorkbench().getActiveWorkbenchWindow()
		.getActivePage().findView(
				RequirementsViewPart.REQUIREMENT_VIEW_ID);
	}
	
	/**
	 * Returns the implPropViewPart.
	 * 
	 * @return ImplPropViewPart implPropViewPart
	 */
	public ImplPropViewPart getImplPropViewPart() {
		return (ImplPropViewPart) PlatformUI
		.getWorkbench().getActiveWorkbenchWindow()
		.getActivePage().findView(
				ImplPropViewPart.IMPL_PROP_VIEW_ID);
	}
}