package ch.unizh.ifi.adora.requirements.ui.views.ImplPropTable;

import java.util.Vector;

import org.eclipse.gef.commands.Command;
import org.eclipse.jface.viewers.ICellModifier;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.TableItem;

import ch.unizh.ifi.adora.model.representation.requirements.ImplementationProposal;
import ch.unizh.ifi.adora.requirements.commands.ImplPropEdit;

public class ImplPropCellModifier implements ICellModifier {

	private int columnIndex;
	private ImplPropViewPart implPropViewPart;
	private ImplPropContentProvider contentProvider;
	private Vector<String> requirementIDs;
	private Command command;
	
	public ImplPropCellModifier(ImplPropViewPart viewPart) {
		this.implPropViewPart = viewPart;
	}
	
	public boolean canModify(Object element, String property) {
		return true;
	}

	public Object getValue(Object element, String property) {
		columnIndex = implPropViewPart.getColumnNames().indexOf(property);

		Object result = null;
		ImplementationProposal implProp = (ImplementationProposal) element;

		switch (columnIndex) {
		case 0: //Id column
			result = implProp.getId();
			break;

		case 1: // Title column
			result = implProp.getTitle();
			break;

		case 2: // Requirement IDs column
			result = implProp.getRequirementIDs();
			break;
			
		case 3: // Design Decision IDs column
			result = implProp.getDesignDecisionIDs();
			break;
			
		case 4: // Assumption column
			result = implProp.getAssumption();
			break;
		
		case 5: // Impact column
			result = implProp.getImpact();
			break;
		
		case 6: // Dismissed Decisions column
			result = implProp.getDismissedDesigns();
			break;
		
		case 7: // Open Issues column
			result = implProp.getOpenIssues();
			break;

		case 8: // Comment column
			result = implProp.getComment();
			break;
			
		default:
			result = "";
		}
		return result;
	}

	public void modify(Object element, String property, Object value) {
		// Find the index of the column
		columnIndex = implPropViewPart.getColumnNames().indexOf(property);

		TableItem item = (TableItem) element;
		ImplementationProposal implProp = (ImplementationProposal) item
				.getData();
		String valueString;

		switch (columnIndex) {

		case 1: // Title-column
			valueString = ((String) value).trim();
			command = new ImplPropEdit(1, valueString, implProp);
			implPropViewPart.getActiveEditor().getCommandStack().execute(
					command);
			break;
			
		case 2: // RequirementID-column
			valueString = ((String) value).trim();
			boolean isSet = false;
			MessageBox messageBox;
			String message;
			requirementIDs = implPropViewPart.getRequirementIDs();

			// sigle Requirement ID
			if (valueString.equals("") || valueString.equals("?")) {
				command = new ImplPropEdit(2, valueString, implProp);
				implPropViewPart.getActiveEditor().getCommandStack().execute(
						command);
				implProp.setDesignDecisionIDs(valueString);
			} else if (!valueString.contains(";")) {
				while (isSet == false) {
					for (String requirementID : requirementIDs) {
						if (requirementID.equals(valueString)) {
							command = new ImplPropEdit(2, valueString, implProp);
							implPropViewPart.getActiveEditor()
									.getCommandStack().execute(command);
							contentProvider = implPropViewPart
									.getContentProvider();
							contentProvider.processDesignDecisionIDs(implProp);
							isSet = true;
						}
					}
					if (isSet == false) {
						isSet = true;
						messageBox = new MessageBox(implPropViewPart.getSite()
								.getShell(), SWT.ICON_WARNING | SWT.OK);
						messageBox.setText("Notification");
						message = "The Requirement ID " + value
								+ " is not valid!";
						messageBox.setMessage(message);
						messageBox.open();
					}
				}
			} else if (valueString.contains(";")) { // multiple Requirement IDs
				int start = 0;
				int end = 4;
				String result = "";
				while (isSet == false) {
					while (end <= valueString.length()) {
						String reqID = valueString.substring(start, end);
						for (String requirementID : requirementIDs) {
							if (requirementID.equals(reqID)) {
								result = result + requirementID + ";";
								break;
							}
						}
						start += 5;
						end += 5;
					}
					if (result.length() < 1) {
						messageBox = new MessageBox(implPropViewPart.getSite()
								.getShell(), SWT.ICON_WARNING | SWT.OK);
						messageBox.setText("Notification");
						message = "The Requirement IDs " + value
								+ " are not valid!";
						messageBox.setMessage(message);
						messageBox.open();
					} else {
						command = new ImplPropEdit(2, result.substring(0,
								result.length() - 1), implProp);
						implPropViewPart.getActiveEditor().getCommandStack()
								.execute(command);
						contentProvider = implPropViewPart.getContentProvider();
						contentProvider.processDesignDecisionIDs(implProp);
					}
					isSet = true;
				}
			}
			break;
			
		case 4: // Assumptions-column
			valueString = ((String) value).trim();
			command = new ImplPropEdit(4, valueString, implProp);
			implPropViewPart.getActiveEditor().getCommandStack().execute(command);
			break;
			
		case 5: // Impact-column
			valueString = ((String) value).trim();
			command = new ImplPropEdit(5, valueString, implProp);
			implPropViewPart.getActiveEditor().getCommandStack().execute(command);
			break;
		
		case 6: // Dismissed Designs-column
			valueString = ((String) value).trim();
			command = new ImplPropEdit(6, valueString, implProp);
			implPropViewPart.getActiveEditor().getCommandStack().execute(command);
			break;
		
		case 7: // Open Issues-column
			valueString = ((String) value).trim();
			command = new ImplPropEdit(7, valueString, implProp);
			implPropViewPart.getActiveEditor().getCommandStack().execute(command);
			break;
		
		case 8: // Comment-column
			valueString = ((String) value).trim();
			command = new ImplPropEdit(8, valueString, implProp);
			implPropViewPart.getActiveEditor().getCommandStack().execute(command);
			break;
		}
		implPropViewPart.getViewer().update(implProp, null);
	}
}
