package ch.unizh.ifi.adora.requirements.ui.views.ImplPropTable;

import java.beans.PropertyChangeEvent;
import java.util.List;
import java.util.Observable;
import java.util.Observer;
import java.util.Vector;

import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;

import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.requirements.DesignDecision;
import ch.unizh.ifi.adora.model.representation.requirements.ImplementationProposal;
import ch.unizh.ifi.adora.model.representation.requirements.Requirement;
import ch.unizh.ifi.adora.parts.AdoraModelEditPart;
import ch.unizh.ifi.adora.requirements.ui.views.ModelObserver;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;

public class ImplPropContentProvider implements IStructuredContentProvider, IPartListener, Observer {

	/**
	 * Holds the selected model elements from the adora model editor.
	 */
	private Vector<ConnectableElement> selectedModelElements = new Vector<ConnectableElement>();
	
	/**
	 * Generates connectableElements out of editParts.
	 * 
	 * @param List<AbstractGraphicalEditPart> selectedEditParts
	 */
	private void processEditParts(List<AbstractGraphicalEditPart> selectedEditParts) {
		selectedModelElements = new Vector<ConnectableElement>();
		for (AbstractGraphicalEditPart editPart : selectedEditParts) {
			ConnectableElement modelElement = (ConnectableElement) editPart.getModel();
			selectedModelElements.add(modelElement);	
		}
	}
	
	/**
	 * Returns the selectedModelElements vector.
	 * 
	 * @return Vector<ConnectableElement> selectedModelElements
	 */
	public Vector<ConnectableElement> getSelectedModelElements() {
		return selectedModelElements;
	}

	/**
	 * This class represents the selection listener and handels the selection in the
	 * designDecisionTableViewer.
	 * 
	 * @author 		Jaro Habr
	 *
	 */
	private final class SelectionListener implements ISelectionListener {
		//set model element selection from editor and table
		@SuppressWarnings("unchecked")
		public void selectionChanged(IWorkbenchPart part, ISelection selection) {
			if (part instanceof AdoraModelEditor) {
				Vector<Object> toSelect = new Vector<Object>();
				if (!(((StructuredSelection) selection).toList().get(0) instanceof AdoraModelEditPart)) {
					processEditParts(((StructuredSelection) selection).toList());
					for (ImplementationProposal implProp : getAdoraModel().getImplProps()) {
						if (implProp.getModelElements().size() == 1 && !toSelect.contains(implProp)) {
							for (ConnectableElement element : selectedModelElements) {
								if (implProp.getModelElements().contains(element)) {
									toSelect.add(implProp);
								}
							}
						}
						if (selectedModelElements.containsAll(implProp.getModelElements()) && implProp.getModelElements().containsAll(selectedModelElements) && !toSelect.contains(implProp) && implProp != null) {
							toSelect.add(implProp);
						}
					}
				} else {
					selectedModelElements.removeAllElements();
				}
				
				//selektierten elemente in der Tabelle
				List actualList = ((IStructuredSelection) tableViewer.getSelection()).toList();
				//vergleich zwischen selektierten Modelelementen im editor und in der Tabelle
				if (alreadySelected(actualList, toSelect)) {
					return;
				}
				
				//selektierten elemente in der Tabelle markieren
				tableViewer.setSelection(new StructuredSelection(toSelect));
			}
		}
			
		
		//vergleicht beide listen (1. modelelemente aus editor; 2. elemente aus tabelle)
		@SuppressWarnings("unchecked")
		private boolean alreadySelected(List actualList, List toSelect) {
			if (actualList.size() != toSelect.size()) {
				return false;
			}

			for (Object object : actualList) {
				if (!toSelect.contains(object)) {
					return false;
				}
			}

			return true;
		}
	}
	
	/**
	 * The table viewer.
	 */
	private TableViewer tableViewer;
	
	/**
	 * Saves the current page.
	 */
	private IWorkbenchPage currentPage;
	
	/**
	 * Saves a listener, which is used for the selections.
	 */
	SelectionListener listener = new SelectionListener();
	
	/**
	 * The Constructor.
	 * 
	 * @param TableViewer viewer
	 */
	public ImplPropContentProvider(TableViewer viewer, IWorkbenchPage page) {
		ModelObserver.getInstance().addObserver(this);
		page.addPartListener(this);
		this.tableViewer = viewer;
		try {
			if (page.getActivePart() instanceof AdoraModelEditor) {
				ModelObserver.getInstance().setAdoraModelEditor((AdoraModelEditor) page.getActivePart());
			}
		} catch (ClassCastException e) {
			e.printStackTrace();
		}
	}
	
	public Object[] getElements(Object inputElement) {
		try {
			return ModelObserver.getInstance().getAdoraModelEditor()
					.getModel().getImplProps().toArray();
		} catch (NullPointerException e) {
			return new Object[0];
		}
	}
	
	/**
	 * Returns the current adora Model.
	 * 
	 * @return AdoraModel adoraModel
	 */
	private AdoraModel getAdoraModel() {
		return ModelObserver.getInstance().getAdoraModelEditor().getModel();
	}

	public void dispose() {		
	}

	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		
	}
	
	/**
	 * Generates the designDecisionIDs of the requirement.
	 * 
	 * @param Vector<DesignDecision> designDecisions
	 * @param Requirement requirement
	 */
	private void processRequirementIDs(ImplementationProposal implProp) {
		String requirementIDs = "";
		if (implProp.getRequirements().size() == 0) {
			implProp.setRequirementIDs("?");
		} else {
			for (Requirement requirement : implProp.getRequirements()) {
				requirementIDs += requirement.getId() + ";";
			}
			implProp.setRequirementIDs(requirementIDs.substring(0, requirementIDs.length() - 1));
		}
		processDesignDecisionIDs(implProp);
	}
	
	
	public void processDesignDecisionIDs(ImplementationProposal implProp) {
		String designDecisionIDs = "";
		if (implProp.getDesignDecisions().size() == 0) {
			implProp.setDesignDecisionIDs("?");
		} else {
			for (DesignDecision designDecision : implProp.getDesignDecisions()) {
				designDecisionIDs += designDecision.getId() + ";";
			}
			implProp.setDesignDecisionIDs(designDecisionIDs.substring(0,
					designDecisionIDs.length() - 1));
		}
		tableViewer.refresh();
	}

	public void update(Observable o, Object arg) {
		if (!(arg instanceof PropertyChangeEvent)) {
			return;
		}
		
		PropertyChangeEvent event = (PropertyChangeEvent) arg;
		
		AdoraModel adoraModel = ModelObserver.getInstance().getAdoraModelEditor().getModel();
		Requirement requirement;
		
		//design decision added to requirement
		if (Requirement.DESIGN_DECISION_ASSOCIATED.equals(event.getPropertyName())) {
			requirement = (Requirement) event.getOldValue();
			for (ImplementationProposal implProp : adoraModel.getImplProps()) {
				if (implProp.getRequirements().contains(requirement)) {
					implProp.getRequirements().remove(requirement);
					implProp.getRequirements().add(requirement);
					for (DesignDecision designDecision : requirement.getDesignDecisions()) {
						if (!implProp.getDesignDecisions().contains(designDecision)) {
							implProp.getDesignDecisions().add(designDecision);
						}
					}
					processRequirementIDs(implProp);
				}
			}
			tableViewer.refresh();
		}
		
		// if all design decisions have been deleted from a requirement
		if (Requirement.DESIGN_DECISION_DISSOCIATED.equals(event.getPropertyName())) {
			requirement = (Requirement) event.getOldValue();
			for (ImplementationProposal implProp : adoraModel.getImplProps()) {
				if (implProp.getRequirements().contains(requirement)) {
					requirement.getDesignDecisions().removeAllElements();
					processRequirementIDs(implProp);
				}
			}
			if (!tableViewer.getTable().isDisposed()) {
				tableViewer.refresh();
			}
		}
		
//		if (AdoraModel.DECISION_REMOVED.equals(event.getPropertyName())) {
////			for (ImplementationProposal )
//			if (!tableViewer.getTable().isDisposed()) {
//				tableViewer.refresh();
//			}
//		}
		
		// deleting a requirement
		if (AdoraModel.REQUIREMENT_REMOVED.equals(event.getPropertyName())) {
			requirement = (Requirement) event.getOldValue();
			for (ImplementationProposal implProp : adoraModel.getImplProps()) {
				if (implProp.getRequirements() != null
						&& implProp.getRequirements().contains(requirement)) {
					implProp.getRequirements().remove(requirement);
					for (DesignDecision designDecision : requirement.getDesignDecisions()) {
						if (implProp.getDesignDecisions().contains(designDecision)) {
							implProp.getDesignDecisions().remove(designDecision);
						}
					}
					processRequirementIDs(implProp);
				}
			}
			tableViewer.refresh();
		}
	}

	public void partActivated(IWorkbenchPart part) {
		if (!(part instanceof AdoraModelEditor)) {
			return;
		}

		processPart(part);

		ModelObserver observer = ModelObserver.getInstance();
		observer.setAdoraModelEditor((AdoraModelEditor) part);
	}
	
	private void processPart(IWorkbenchPart part) {
		if (currentPage != null) {
			currentPage.removeSelectionListener(listener);
		}

		currentPage = part.getSite().getPage();
		currentPage.addSelectionListener(listener);
	}

	public void partBroughtToTop(IWorkbenchPart part) {
		// TODO Auto-generated method stub
		
	}

	public void partClosed(IWorkbenchPart part) {
		// TODO Auto-generated method stub
		
	}

	public void partDeactivated(IWorkbenchPart part) {
		// TODO Auto-generated method stub
		
	}

	public void partOpened(IWorkbenchPart part) {
		// TODO Auto-generated method stub
		
	}


//	public void update(Observable o, Object arg) {
//		
//		if (!(arg instanceof PropertyChangeEvent)) {
//			return;
//		}
//		
//		PropertyChangeEvent event = (PropertyChangeEvent) arg;
//		
//		AdoraModel adoraModel = ModelObserver.getInstance().getAdoraModelEditor().getModel();
//		
//		if (Requirement.DESIGN_DECISION_ADDED.equals(event.getPropertyName())) {
//			Requirement requirement = (Requirement) event.getOldValue();
//			for (ImplementationProposal implProp : getAdoraModel().getImplProps()) {
//				if (implProp.getRequirements().contains(requirement)) {
//					implProp.getRequirements().remove(requirement);
//					implProp.getRequirements().add(requirement);
//					processDesignDecisionIDs(implProp);
//				}
//			}
//			tableViewer.refresh();
//		}
//		
//		// deleting a requirement
//		if (AdoraModel.REQUIREMENT_REMOVED.equals(event.getPropertyName())) {
//			Requirement requirement = (Requirement) event.getOldValue();
//			for (ImplementationProposal implProp : adoraModel.getImplProps()) {
//				if (implProp.getRequirements() != null
//						&& implProp.getRequirements().contains(requirement)) {
//					implProp.getRequirements().remove(requirement);
//					processDesignDecisionIDs(implProp);
//				}
//			}
//			tableViewer.refresh();
//		}
//		
//		if (AdoraModel.CHILD_ADDED_PROP.equals(event.getPropertyName()) && event.getSource() instanceof AdoraModel) {
//			
//		}
//		
//	}
}
