package ch.unizh.ifi.adora.requirements.ui.views.ImplPropTable;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.eclipse.gef.EditPart;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.requirements.DesignDecision;
import ch.unizh.ifi.adora.model.representation.requirements.ImplementationProposal;
import ch.unizh.ifi.adora.model.representation.requirements.Requirement;
import ch.unizh.ifi.adora.requirements.RequirementsPlugIn;
import ch.unizh.ifi.adora.requirements.commands.ImplPropAdd;
import ch.unizh.ifi.adora.requirements.commands.ImplPropCompose;
import ch.unizh.ifi.adora.requirements.commands.ImplPropRemove;
import ch.unizh.ifi.adora.requirements.ui.views.DesignDecisionTable.DesignDecisionViewPart;
import ch.unizh.ifi.adora.requirements.ui.views.RequirementsTable.RequirementsViewPart;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;

public class ImplPropViewPart extends ViewPart {

	/**
	 * The ID of this view.
	 */
	public static final String IMPL_PROP_VIEW_ID = "ch.unizh.ifi.adora.requirements.ImplPropTable";
	
	/**
	 * The Description of the COL_ID column.
	 */
	private static final String COL_ID = "ID";
	
	/**
	 * The Description of the COL_TITLE column.
	 */
	private static final String COL_TITLE = "Title";
	
	/**
	 * The Description of the COL_REQUIREMENT_ID column.
	 */
	private static final String COL_REQUIREMENT_ID = "Requirement ID";
	
	/**
	 * The Description of the COL_REQUIREMENT_ID column.
	 */
	private static final String COL_DESIGN_DECISION_ID = "Design Decision ID";
	
	/**
	 * The Description of the COL_ASSUMPTION column.
	 */
	private static final String COL_ASSUMPTION = "Assumptions";
	
	/**
	 * The Description of the COL_IMPACT column.
	 */
	private static final String COL_IMPACT = "Impact";
	
	/**
	 * The Description of the COL_DISMISSED column.
	 */
	private static final String COL_DISMISSED = "Dismissed Designs";
	
	/**
	 * The Description of the COL_ISSUES column.
	 */
	private static final String COL_ISSUES = "Open Issues";
	
	/**
	 * The Description of the COL_COMMENT column.
	 */
	private static final String COL_COMMENT = "Comment";
	
	/**
	 * Holds the references to the column descriptions.
	 */
	private String[] columnNames = new String[] {
			COL_ID, 
			COL_TITLE,
			COL_REQUIREMENT_ID,
			COL_DESIGN_DECISION_ID,
			COL_ASSUMPTION,
			COL_IMPACT,
			COL_DISMISSED,
			COL_ISSUES,
			COL_COMMENT};
	
	/**
	 * The toolbar actions.
	 */
	private Action addImplPropAction, deleteImplPropAction, associateReqsAction, composeImplPropAction;
	
	/**
	 * The content provider.
	 */
	private ImplPropContentProvider contentProvider;
	
	/**
	 * The active adora editor.
	 */
	private AdoraModelEditor activeAdoraEditor;
	
	/**
	 * The active adora model.
	 */
	private AdoraModel adoraModel;
	
	/**
	 * The Implementation Proposal table.
	 */
	private Table table;
	
	/**
	 * The viewer of the Implementation Proposal table.
	 */
	private TableViewer viewer;
	
	/**
	 * The requirements view part.
	 */
	private RequirementsViewPart requirementsViewPart;
	
	/**
	 * Holds the implProps vector saved in the adora model.
	 */
	private Vector<ImplementationProposal> implProps = new Vector<ImplementationProposal>();
	
	/**
	 * Holds the current requirement IDs list.
	 */
	private Vector<String> requirementIDs = new Vector<String>();
	
	/**
	 * Holds the selected requirements in the table.
	 */
	private Vector<ImplementationProposal> selection = new Vector<ImplementationProposal>();
	
	/**
	 * If implementation proposals are selected from the implementation proposal
	 * table viewer, selectedFromImplProp is set to true.
	 */
	private Boolean selectedFromImplProp = false;
	
	/**
	 * IPartListener to listen to active adora model and to update the data from
	 * the adora model.
	 */
	private IPartListener listener = new IPartListener() {

		public void partActivated(IWorkbenchPart part) {
			if (part instanceof AdoraModelEditor) {
				activeAdoraEditor = (AdoraModelEditor) getSite().getPage().getActiveEditor();
				adoraModel = activeAdoraEditor.getModel();
				implProps = adoraModel.getImplProps();
				requirementIDs = adoraModel.getRequirementsIDs();
				if (!viewer.getTable().isDisposed()) {
					try {
						viewer.refresh();
					} catch (Exception e) {
						e.printStackTrace();
					}
				}
			}
		}

		public void partBroughtToTop(IWorkbenchPart part) {
		}
		
		//set table invisible, if last editor has been closed
		public void partClosed(IWorkbenchPart part) {
//			if (part instanceof AdoraModelEditor) {
//				if (getSite().getPage().getEditors().length == 0) {
//					if (!table.isDisposed()) {
//						try {
//							table.setVisible(false);
//							disableActions();
//						} catch (Exception e) {
//							e.printStackTrace();
//						}
//					}
//				}
//			}
		}

		public void partDeactivated(IWorkbenchPart part) {
		}

		public void partOpened(IWorkbenchPart part) {
		}
	};

	/**
	 * Creates the part control.
	 * 
	 * @see org.eclipse.ui.part.WorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createPartControl(Composite parent) {
		this.addChildControls(parent);
//		if (getSite().getPage().getEditors().length == 0) {
//			table.setVisible(false);
//			disableActions();
//		} else { 
			if (getSite().getPage().getActivePart() instanceof AdoraModelEditor) {
//				table.setVisible(true);
//				activeAdoraEditor = AdoraPlugin.getActiveEditor();
				activeAdoraEditor = (AdoraModelEditor) getSite().getPage().getActiveEditor();
				adoraModel = activeAdoraEditor.getModel();
				implProps = adoraModel.getImplProps();
				requirementIDs = adoraModel.getRequirementsIDs();
				viewer.refresh();
			}
			getSite().getPage().addPartListener(listener);
//		}
	}

	/**
	 * Adds the child controls.
	 * 
	 * @param Composite parent
	 */
	private void addChildControls(Composite parent) {
		GridLayout layout = new GridLayout(2, false);
		parent.setLayout(layout);
		
		createTable(parent);
		createToolbarActions(parent);
		createToolBar();
		createTableViewer();
		initTableViewer();
	}

	/**
	 * Initializes the tableViewer.
	 */
	private void initTableViewer() {
		contentProvider = new ImplPropContentProvider(viewer, getSite().getPage());
		viewer.setContentProvider(contentProvider);
		viewer.setLabelProvider(new ImplPropLabelProvider());
		viewer.setInput(implProps);
		
	}
	
	/**
	 * Creates the Implementation Proposal table.
	 * 
	 * @param Composite parent
	 */
	private void createTable(Composite parent) {
		int style = SWT.CHECK | SWT.H_SCROLL | SWT.V_SCROLL
				| SWT.FULL_SELECTION | SWT.MULTI;
		
		table = new Table(parent, style);
		
		GridData gridData = new GridData(GridData.FILL_BOTH);
		gridData.grabExcessVerticalSpace = true;
		gridData.horizontalSpan = 3;
		table.setLayoutData(gridData);
		
		table.setHeaderVisible(true);
		table.setLinesVisible(true);
		
		//table check listener
		table.addListener(SWT.Selection, new Listener() {
		      public void handleEvent(Event event) {
		    	  if (event.detail == SWT.CHECK) {
		    		  ImplementationProposal implProp = ((ImplementationProposal)event.item.getData());
		    		  boolean checked = ((TableItem)event.item).getChecked();		  
		    		  
		    		  if (checked){
		    			  selection.add(implProp);  
		    		  } else {
		    			  selection.remove(implProp);
		    		  }
			      }
		      } 
		    });
		
		// ID column
		TableColumn column = new TableColumn(table, SWT.LEFT, 0);		
		column.setText(COL_ID);
		column.setWidth(60);
		column.setResizable(true);
		
		// Title column
		column = new TableColumn(table, SWT.LEFT, 1);
		column.setText(COL_TITLE);
		column.setWidth(150);
		column.setResizable(true);
		
		// Requirement IDs column
		column = new TableColumn(table, SWT.LEFT, 2);
		column.setText(COL_REQUIREMENT_ID);
		column.setWidth(200);
		column.setResizable(true);
		
		// Design Decision column
		column = new TableColumn(table, SWT.LEFT, 3);
		column.setText(COL_DESIGN_DECISION_ID);
		column.setWidth(170);
		column.setResizable(true);
		
		// Assumption column
		column = new TableColumn(table, SWT.LEFT, 4);
		column.setText(COL_ASSUMPTION);
		column.setWidth(100);
		column.setResizable(true);
		
		// Impact column
		column = new TableColumn(table, SWT.LEFT, 5);
		column.setText(COL_IMPACT);
		column.setWidth(100);
		column.setResizable(true);
		
		// Dismissed Designs column
		column = new TableColumn(table, SWT.LEFT, 6);
		column.setText(COL_DISMISSED);
		column.setWidth(150);
		column.setResizable(true);
		
		// Open Issues column
		column = new TableColumn(table, SWT.LEFT, 7);
		column.setText(COL_ISSUES);
		column.setWidth(100);
		column.setResizable(true);
		
		// Comment column
		column = new TableColumn(table, SWT.LEFT, 8);
		column.setText(COL_COMMENT);
		column.setWidth(100);
		column.setResizable(true);
	}

	/**
	 * Creates the table viewer for the table widget.
	 */
	private void createTableViewer() {
		viewer = new CheckboxTableViewer(table);
		
		viewer.setUseHashlookup(true);
		viewer.setColumnProperties(columnNames);
		viewer.addSelectionChangedListener(new ISelectionChangedListener() {
			@SuppressWarnings("unchecked")
			public void selectionChanged(SelectionChangedEvent event) {
				
				if (event.getSelection().isEmpty()) {
					return;
				}

				if (!(event.getSelection() instanceof IStructuredSelection)) {
					return;
				}

				ArrayList selectedModelElements = new ArrayList();
				IStructuredSelection selection = (IStructuredSelection) event
						.getSelection();
				Iterator selectionIterator = selection.iterator();
				ImplementationProposal implProp = null;
				DesignDecisionViewPart designDecisionViewPart = getDesignDecisionViewPart();
				RequirementsViewPart requirementsViewPart = getRequirementsViewPart();
				
				Vector<DesignDecision> designDecisionsToSelect = new Vector<DesignDecision>();
				Vector<Requirement> requirementsToSelect = new Vector<Requirement>();
				
				//selected from DD Table
				if (designDecisionViewPart != null && designDecisionViewPart.getSelectedFromDesignDecision() == true) {
					designDecisionViewPart.setSelectedFromDesignDecision(false);
				} else if (requirementsViewPart != null && requirementsViewPart.getSelectedFromRequirement() == true) {
					requirementsViewPart.setSelectedFromRequirement(false);
				} else {
					//select model elements from selected requirement
					while (selectionIterator.hasNext()) {
						implProp = (ImplementationProposal) selectionIterator.next();
						if (implProp.getModelElements().size() != 0) {
							for (int a = 0; a < implProp.getModelElements().size(); a++) { //iterate through all modelElements of one designDecisions
								if (!selectedModelElements.contains(implProp.getModelElements().get(a))) {
									selectedModelElements.add(implProp.getModelElements().get(a)); //add one ConnectableElemente to selected Elements
								}
							}
						}
						
						//select design decision from Req Table
						try {
							if (implProp.getRequirements() != null && implProp.getRequirements().size() != 0) {
								for (Requirement req : implProp.getRequirements()) {
									requirementsToSelect.add(req);
								}
							}
						} catch (NullPointerException e) {
							e.printStackTrace();
						}
						
						//select design decision from Req Table
						try {
							if (implProp.getDesignDecisions() != null && implProp.getDesignDecisions().size() != 0) {
								for (DesignDecision decision : implProp.getDesignDecisions()) {
									designDecisionsToSelect.add(decision);
								}
							}
						} catch (NullPointerException e) {
							e.printStackTrace();
						}
					}
					
					//select model elements associated with the implProp in the adora editor
					AdoraModelEditor adoraModelEditor = AdoraPlugin.getActiveEditor();
					if (adoraModelEditor != null) {
						if (alreadySelected(adoraModelEditor, selectedModelElements)) {
							return;
						}
						adoraModelEditor.selectElements(selectedModelElements);
					}
					
					setSelectedFromImplProp(true);
					//mark associated requirements in the requirements Table
					if (requirementsViewPart != null) {
						requirementsViewPart.getViewer().setSelection(new StructuredSelection(requirementsToSelect));
					} else {
						System.out.println("Please open the requirements table first!");
					}
					
					// select designDecisions in the DD Table with the associated requirements selected from Req Table
					if (designDecisionViewPart != null) {
						designDecisionViewPart.getViewer().setSelection(new StructuredSelection(designDecisionsToSelect));
					} else {
						System.out.println("Please open the requirements table first!");
					}
				}				
			}

			/**
             * Checks, if the list of selectedModelElements is already selected
             * in the editor.
             * 
             * @param AdoraModelEditor adoraModelEditor
             * @param List selectedModelElements
             * 
             * @return true if already selected.
             */
			private boolean alreadySelected(AdoraModelEditor adoraModelEditor,
					List selectedModelElements) {
				if (adoraModelEditor.getViewer().getSelectedEditParts().size() != selectedModelElements
						.size()) {
					return false;
				}

				boolean equals = true;
				for (Object object : adoraModelEditor.getViewer().getSelectedEditParts()) {
					EditPart part = (EditPart) object;
					if (!selectedModelElements.contains(part.getModel())) {
						equals = false;
					}
				}
				
				return equals;
			}
		});
		
		// create sell editor
		CellEditor[] editors = new CellEditor[columnNames.length];
		
		// ID column (free text)
//		TextCellEditor textEditor = new TextCellEditor(table);
//		((Text) textEditor.getControl()).setTextLimit(50);
//		editors[0] = textEditor;
		
		//Title column (free text)
		TextCellEditor textEditor = new TextCellEditor(table);
		((Text) textEditor.getControl()).setTextLimit(300);
		editors[1] = textEditor;
		
		//Requirement IDs column (free text) -> no editor
		textEditor = new TextCellEditor(table);
		((Text) textEditor.getControl()).setTextLimit(100);
		editors[2] = textEditor;
		
		//Design Decision IDs column (free text) -> no editor
//		textEditor = new TextCellEditor(table);
//		((Text) textEditor.getControl()).setTextLimit(100);
//		editors[3] = textEditor;
		
		//Assumptions column (free text) -> no editor
		textEditor = new TextCellEditor(table);
		editors[4] = textEditor;
		
		// Impact column (combobox)
		textEditor = new TextCellEditor(table);
		((Text) textEditor.getControl()).setTextLimit(100);
		editors[5] = textEditor;
		
		// Dismissed Designs column (combobox)
		textEditor = new TextCellEditor(table);
		((Text) textEditor.getControl()).setTextLimit(100);
		editors[6] = textEditor;
		
		// Open Issues column (combobox)
		textEditor = new TextCellEditor(table);
		editors[7] = textEditor;
		
		//Comment column
		//TODO -> longTextCellEditor
		textEditor = new TextCellEditor(table);
		editors[8] = textEditor;
		
		// Assign the cell editors to the viewer 
		viewer.setCellEditors(editors);
		viewer.setCellModifier(new ImplPropCellModifier(this));
		
	}

	/**
	 * Creates the toolbar of the table viewer.
	 */
	private void createToolBar() {
		IToolBarManager mgr = getViewSite().getActionBars().getToolBarManager();
		mgr.add(addImplPropAction);
		mgr.add(deleteImplPropAction);
		mgr.add(new Separator());
		mgr.add(associateReqsAction);
		mgr.add(new Separator());
		mgr.add(composeImplPropAction);
	}

	/**
	 * Creates the toolbar actions in the toolbar of the table viewer.
	 * 
	 * @param Composite parent
	 */
	private void createToolbarActions(final Composite parent) {
		// add action
		addImplPropAction = new Action("Add Implementation Proposal") {
			public void run() {
				ImplPropAdd addCommand = new ImplPropAdd(adoraModel);
				activeAdoraEditor.getCommandStack().execute(addCommand);
				viewer.refresh();
//				viewer.add(addCommand.getRequirement());
			}
		};
		addImplPropAction.setImageDescriptor(ImageDescriptor.createFromFile(RequirementsPlugIn.class, "icons/add.gif"));
		
		// delete action with message Box
		deleteImplPropAction = new Action("Delete Implementation Proposal") {
			MessageBox messageBox;
	        String message = new String();
			
			public void run() {
		        if (selection.size() == 0){
		        	messageBox = new MessageBox(parent.getShell(), SWT.ICON_WARNING | SWT.OK);
		        	messageBox.setText("Notification");
		        	message = "Please select a Implementation Proposal first!";
		        } else {
		        	messageBox = new MessageBox(parent.getShell(), SWT.ICON_QUESTION | SWT.YES | SWT.NO);
			        messageBox.setText("Confirmation");
			        message = "Are you sure to remove the ";
		        	
			        if (selection.size() == 1) {
			        	message += "selected Implementation Proposal (" + selection.get(0).getId() + ")?";
			        } else {
			        	message += selection.size() + " selected Implementation Proposals?";
			        }
		        }
			    
		        messageBox.setMessage(message);
		        
		        if (messageBox.open() == SWT.YES) {
		          for(int i = 0; i < selection.size(); i++){
		        	  ImplPropRemove removeCommand = new ImplPropRemove(adoraModel, selection.elementAt(i));
		        	  activeAdoraEditor.getCommandStack().execute(removeCommand);
					}
		          //clear the selection vector -> no elemets are selected
		          selection.removeAllElements();
		          }
		        viewer.refresh();
		        }
		};
		ISharedImages sharedImages = PlatformUI.getWorkbench().getSharedImages();
		deleteImplPropAction.setImageDescriptor(sharedImages.getImageDescriptor(ISharedImages.IMG_TOOL_DELETE));
		
		//associate requirements selection action
		associateReqsAction = new Action("Associate Requirements") {
			MessageBox messageBox;
	        String message = new String();
	        
			public void run() {
				requirementsViewPart = getRequirementsViewPart();
				if (selection.size() == 0) {
					messageBox = new MessageBox(parent.getShell(), SWT.ICON_WARNING | SWT.OK);
		        	messageBox.setText("Notification");
		        	message = "Please check an Implementation Proposal first!";
		        	messageBox.setMessage(message);
		        	messageBox.open();
				} else if (requirementsViewPart.getRequirementSelection().size() == 0) {
					messageBox = new MessageBox(parent.getShell(), SWT.ICON_WARNING | SWT.OK);
		        	messageBox.setText("Notification");
		        	message = "Please check a Requirement first!";
		        	messageBox.setMessage(message);
		        	messageBox.open();
				} else {
					processRequirementIDs(requirementsViewPart.getRequirementSelection(), selection);
				}
				viewer.refresh();
			}
		};
		associateReqsAction.setImageDescriptor(ImageDescriptor.createFromFile(RequirementsPlugIn.class, "icons/associate_2.gif"));
		
		//compose implementation proposal action
		composeImplPropAction = new Action("Compose Implementation Proposal") {
			MessageBox messageBox;
	        String message = new String();
			
			public void run() {
		        if (selection.size() == 0) {
		        	messageBox = new MessageBox(parent.getShell(), SWT.ICON_WARNING | SWT.OK);
		        	messageBox.setText("Notification");
		        	message = "Please check an Implementation Proposal first!";
		        	messageBox.setMessage(message);
		        	messageBox.open();
		        	setChecked(false);
		        } else if (selection.size() > 1) {
		        	messageBox = new MessageBox(parent.getShell(), SWT.ICON_WARNING | SWT.OK);
		        	messageBox.setText("Notification");
		        	message = "Please check only one Implementation Proposal!";
		        	messageBox.setMessage(message);
		        	messageBox.open();
		        	setChecked(false);
		        } else if (selection.size() == 1){
		        	ImplPropCompose compose = new ImplPropCompose(selection.get(0).getModelElements(), isChecked());
		        	Action reqFilterAction = getRequirementsViewPart().getFilterAction();
		        	Action decisionFilterAction = getDesignDecisionViewPart().getFilterAction();
		        	
			        if (isChecked()) {
						activeAdoraEditor.getCommandStack().execute(compose);
						setChecked(isChecked());
						for (Requirement requirement : selection.get(0).getRequirements()) {
							requirement.setSelected(true);
			        		for (DesignDecision designDecision : requirement.getDesignDecisions()) {
			        			designDecision.setSelected(true);
			        		}
			        	}
						reqFilterAction.setChecked(true);
						reqFilterAction.run();
						decisionFilterAction.setChecked(true);
						decisionFilterAction.run();
					} else {
						activeAdoraEditor.getCommandStack().execute(compose);
						for (Requirement requirement : selection.get(0).getRequirements()) {
			        		requirement.setSelected(false);
			        		for (DesignDecision designDecision : requirement.getDesignDecisions()) {
			        			designDecision.setSelected(false);
			        		}
			        	}
						reqFilterAction.setChecked(false);
						reqFilterAction.run();
						decisionFilterAction.setChecked(false);
						decisionFilterAction.run();
						setChecked(isChecked());
					}
		        }
			}
		};
		composeImplPropAction.setChecked(false);
		composeImplPropAction.setImageDescriptor(ImageDescriptor.createFromFile(RequirementsPlugIn.class, "icons/run.gif"));
	}
	
	/**
	 * Disables the toolbar actions.
	 */
	private void disableActions() {
		addImplPropAction.setEnabled(false);
		deleteImplPropAction.setEnabled(false);
		associateReqsAction.setEnabled(false);
		composeImplPropAction.setEnabled(false);
	}
	
	/**
	 * Enables the toolbar actions.
	 */
	private void enableActions() {
		addImplPropAction.setEnabled(true);
		deleteImplPropAction.setEnabled(true);
		associateReqsAction.setEnabled(true);
		composeImplPropAction.setEnabled(true);
	}

	/**
	 * Sets the keyboard focus.
	 */
	public void setFocus() {
		table.getParent().setFocus();
	}
	
	/**
	 * Returns the columnNames array as list.
	 * 
	 * @return List columnNames
	 */
	public List<String> getColumnNames() {
		return Arrays.asList(columnNames);
	}
	
	/**
	 * Returns the table viewer.
	 * 
	 * @return TableViewer viewer
	 */
	public TableViewer getViewer(){
		return viewer;
	}
	
	/**
	 * Returns the requirement IDs list.
	 * 
	 * @return Vector<String> requirementIDs
	 */
	public Vector<String> getRequirementIDs() {
		return (Vector<String>) requirementIDs;
	}
	
	/**
	 * Returns the ImplPropContentProvider.
	 * 
	 * @return ImplPropContentProvider contentProvider
	 */
	public  ImplPropContentProvider getContentProvider() {
		return contentProvider;
	}
	
	/**
	 * Returns the requirementViewPart.
	 * 
	 * @return RequirementsViewPart requirementsViewPart
	 */
	public RequirementsViewPart getRequirementsViewPart() {
		return (RequirementsViewPart) PlatformUI
		.getWorkbench().getActiveWorkbenchWindow()
		.getActivePage().findView(
				RequirementsViewPart.REQUIREMENT_VIEW_ID);
	}
	
	/**
	 * Returns the designDecisionViewPart.
	 * 
	 * @return DesignDecisionViewPart designDecisionViewPart
	 */
	public DesignDecisionViewPart getDesignDecisionViewPart() {
		return (DesignDecisionViewPart) PlatformUI
		.getWorkbench().getActiveWorkbenchWindow()
		.getActivePage().findView(
				DesignDecisionViewPart.DESIGN_DECISION_VIEW_ID);
	}
	
	/**
	 * Generates a requirementIDs string out of checked requirements and
	 * sets it to all selected implementation proposals.
	 * 
	 * @param Vector<Requirement> requirements
	 * @param Vector<ImplementationProposal> implProps
	 */
	private void processRequirementIDs(Vector<Requirement> requirements, Vector<ImplementationProposal> implProps) {
		String requirementIDs = "";
		
		for (Requirement requirement : requirements) {
			requirementIDs += requirement.getId() + ";";
		}
		
		requirementIDs = requirementIDs.substring(0, requirementIDs.length() - 1);
		
		for (ImplementationProposal implProp : implProps) {
			if (implProp.getRequirements().size() != 0) {
				requirementIDs = implProp.getRequirementIDs() + ";" + requirementIDs;
			}
			implProp.setRequirementIDs(requirementIDs);
			contentProvider.processDesignDecisionIDs(implProp);
		}
	}
	
	/**
	 * Returns the active adoraModelEditor.
	 * 
	 * @return AdoraModelEditor activeAdoraEditor
	 */
	public AdoraModelEditor getActiveEditor() {
		return activeAdoraEditor;
	}
	
	/**
	 * Returns the selectedFromImplProp boolean value. Used to identify the
	 * source of the selection event.
	 * 
	 * @return boolean selectedFromImplProp
	 */
	public boolean getSelectedFromImplProp() {
		return selectedFromImplProp;
	}
	
	/**
	 * Sets the boolean value of selectedFromImplProp. Used to identify the
	 * source of the selection event.
	 * 
	 * @param boolean b
	 */
	public void setSelectedFromImplProp(boolean b) {
		this.selectedFromImplProp = b;
	}
	
	
//
//	/**
//	 * Adds a listener.
//	 * 
//	 * @param PropertyChangeListener listener
//	 */
//	public synchronized void addPropertyChangeListener(PropertyChangeListener listener) {
//		if (listener == null) {
//			throw new IllegalArgumentException();
//		}
//		propertyChangeSupport.addPropertyChangeListener(listener);
//	}
//	
//	/**
//	 * Removes the listener.
//	 * 
//	 * @param PropertyChangeListener listener
//	 */
//	public synchronized void removePropertyChangeListener(PropertyChangeListener listener) {
//		if (listener != null) {
//			propertyChangeSupport.removePropertyChangeListener(listener);
//		}
//	}
//	
//	/**
//	 * Report a property change to registered listeners (for example edit
//	 * parts).
//	 * 
//	 * @param String property
//	 * @param Object oldValue
//	 * @param Object newValue
//	 */
//	public  void firePropertyChange(String property, Object oldValue,
//			Object newValue) {
//		if (propertyChangeSupport.hasListeners(property)) {
//			propertyChangeSupport.firePropertyChange(property, oldValue, newValue);
//		}
//	}
}
