package ch.unizh.ifi.adora.requirements.ui.views.RequirementsTable;

import java.util.Vector;

import org.eclipse.gef.commands.Command;
import org.eclipse.jface.viewers.ICellModifier;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.TableItem;

import ch.unizh.ifi.adora.model.representation.requirements.Requirement;
import ch.unizh.ifi.adora.requirements.commands.RequirementEdit;

public class RequirementCellModifier implements ICellModifier {

	private RequirementsViewPart requirementsViewPart;
	private int columnIndex;
	private String[] priorityChoices;
	private String[] abstractionLevelChoices;
	private Vector<String> requirementIDs;
	private Command command;

	/**
	 * The constructor.
	 * 
	 * @param requirementsViewPart
	 */
	public RequirementCellModifier(RequirementsViewPart requirementsViewPart) {
		super();
		this.requirementsViewPart = requirementsViewPart;
	}

	public boolean canModify(Object element, String property) {
		return true;
	}

	public Object getValue(Object element, String property) {
		// Find the index of the column
		columnIndex = requirementsViewPart.getColumnNames().indexOf(property);

		Object result = null;
		Requirement requirement = (Requirement) element;

		switch (columnIndex) {
		case 0: //Id column
			result = requirement.getId();
			System.out.println("CellModifier: Requirement ID: " + result);
			break;

		case 1: // Title column
			result = requirement.getTitle();
			break;

		case 2: // Description column
			result = requirement.getDescription();
			break;

		case 3: // abstraction Level column		
			abstractionLevelChoices = requirementsViewPart
					.getAbstractionLevelChoices();
			String abstractionLevel = requirement.getPriority();
			for (int i = 0; i < abstractionLevelChoices.length; i++) {
				if (abstractionLevel.equals(abstractionLevelChoices[i])) {
					result = new Integer(i);
					break;
				}
			}
			break;

		case 4: // priority column 
			priorityChoices = requirementsViewPart.getPriorityChoices();
			String priority = requirement.getPriority();
			for (int i = 0; i < priorityChoices.length; i++) {
				if (priority.equals(priorityChoices[i])) {
					result = new Integer(i);
					break;
				}
			}
			break;

		case 5: //link to
			result = requirement.getLinkTo();
			break;
			
//		case 6: // scope column
//			result = requirement.getScope();
//			break;

		case 6: // Design Decision column
			result = requirement.getDesignDecisionID();
			break;

		default:
			result = "";
		}
		return result;
	}

	/**
	 * This method is responsible for modifying the content of the cells
	 */
	public void modify(Object element, String property, Object value) {
		// Find the index of the column 
		columnIndex = requirementsViewPart.getColumnNames().indexOf(property);

		TableItem item = (TableItem) element;
		Requirement requirement = (Requirement) item.getData();
		String valueString;

		switch (columnIndex) {

		case 1: // Title-column
			valueString = ((String) value).trim();
			command = new RequirementEdit(1, requirement.getTitle(), valueString, requirement);
			break;

		case 2: // Description-column
			valueString = ((String) value).trim();
			command = new RequirementEdit(2, requirement.getDescription(), valueString, requirement);
			break;

		case 3: // AbstactionLevel-column
			valueString = abstractionLevelChoices[((Integer) value).intValue()];
			if (!requirement.getAbstractionLevel().equals(valueString)) {
				command = new RequirementEdit(3, requirement.getAbstractionLevel(), valueString, requirement);
			}
			break;

		case 4: // Priority-column
			valueString = priorityChoices[((Integer) value).intValue()];
			if (!requirement.getPriority().equals(valueString)) {
				command = new RequirementEdit(4, requirement.getPriority(), valueString, requirement);
			}
			break;

		case 5: // LinkTo-column				
			valueString = ((String) value).trim();
			boolean isSet = false;
			MessageBox messageBox;
			String message;
			requirementIDs = requirementsViewPart.getRequirementIDs();

			//Entry = one Requirement ID
			if (valueString.equals("") || valueString.equals("?")) {
				requirement.setLinkTo(valueString);
			} else if (!valueString.contains(";")) {
				while (isSet == false) {
					for (int i = 0; i < requirementIDs.size(); i++) {
						String requirementID = requirementIDs.elementAt(i);

						if (requirementID.equals(valueString)) {
							command = new RequirementEdit(5, requirement.getLinkTo(), valueString, requirement);
							isSet = true;
						}
					}

					if (isSet == false) {
						isSet = true;
						messageBox = new MessageBox(requirementsViewPart
								.getSite().getShell(), SWT.ICON_WARNING
								| SWT.OK);
						messageBox.setText("Notification");
						message = "The Requirement ID " + value
								+ " is not valid!";
						messageBox.setMessage(message);
						messageBox.open();
					}
				}
			} else { //Entry = more Requirement IDs
				int start = 0;
				int end = 4;
				String result = "";
				while (isSet == false) {
					while (end <= valueString.length()) {
						String reqID = valueString.substring(start, end);

						for (int i = 0; i < requirementIDs.size(); i++) {
							String requirementID = requirementIDs
									.elementAt(i);

							if (requirementID.equals(reqID)) {
								result = result + requirementID + ";";
								break;
							}
						}
						start += 5;
						end += 5;
					}

					if (result.length() < 1) {
						messageBox = new MessageBox(requirementsViewPart
								.getSite().getShell(), SWT.ICON_WARNING
								| SWT.OK);
						messageBox.setText("Notification");
						message = "The Requirement IDs " + value
								+ " are not valid!";
						messageBox.setMessage(message);
						messageBox.open();
					} else {
						command = new RequirementEdit(5, requirement.getLinkTo(), result.substring(0, result
								.length() - 1), requirement);
					}
					isSet = true;
				}
			}
			break;

		case 6: //Design Desicion column
			valueString = ((String) value).trim();
			boolean decisionIsSet = false;
			MessageBox messageBoxDecision;
			String messageDecision;
			requirementIDs = requirementsViewPart.getDesignDecisionIDs();

			//Entry = one Requirement ID
			if (valueString.equals("") || valueString.equals("?")) {
				command = new RequirementEdit(6, requirement.getDesignDecisionID(), valueString, requirement);
			} else if (!valueString.contains(";")) {
				while (decisionIsSet == false) {
					for (int i = 0; i < requirementIDs.size(); i++) {
						String designDecisionID = requirementIDs
								.elementAt(i);

						if (designDecisionID.equals(valueString)) {
							command = new RequirementEdit(6, requirement.getDesignDecisionID(), valueString, requirement);
							decisionIsSet = true;
							break;
						}
					}
					if (decisionIsSet == false) {
						decisionIsSet = true;
						messageBoxDecision = new MessageBox(
								requirementsViewPart.getSite().getShell(),
								SWT.ICON_WARNING | SWT.OK);
						messageBoxDecision.setText("Notification");
						messageDecision = "The Design Decision ID " + value
								+ " is not valid!";
						messageBoxDecision.setMessage(messageDecision);
						messageBoxDecision.open();
					}
				}
			} else { //Entry = more Requirement IDs
				int start = 0;
				int end = 4;
				String result = "";
				while (decisionIsSet == false) {
					while (end <= valueString.length()) {
						String reqID = valueString.substring(start, end);

						for (int i = 0; i < requirementIDs.size(); i++) {
							String designDecisionID = requirementIDs
									.elementAt(i);

							if (designDecisionID.equals(reqID)) {
								result = result + designDecisionID + ";";
								break;
							}
						}
						start += 5;
						end += 5;
					}
					command = new RequirementEdit(6, requirement.getDesignDecisionID(), result.substring(0, result
							.length() - 1), requirement);
					//FirePropertyChange
					decisionIsSet = true;
				}

				if (decisionIsSet == false) {
					decisionIsSet = true;
					messageBoxDecision = new MessageBox(requirementsViewPart
							.getSite().getShell(), SWT.ICON_WARNING | SWT.OK);
					messageBoxDecision.setText("Notification");
					messageDecision = "The Design Decision IDs " + value
							+ " are not valid!";
					messageBoxDecision.setMessage(messageDecision);
					messageBoxDecision.open();
				}
			}
			break;
		}
		requirementsViewPart.getActiveEditor().getCommandStack().execute(command);
		requirementsViewPart.getViewer().update(requirement, null);
	}
}
