package ch.unizh.ifi.adora.unweaver.dialog;

import java.awt.Font;
import java.awt.MouseInfo;
import java.awt.Point;
import java.awt.PointerInfo;
import java.util.List;

import javax.swing.JFrame;
import javax.swing.JTextArea;

/**
 * This class provides the dialog which is shown to the user, when 
 * an invalid selection for an unweaving is made.
 * It displays the model elements that can not be unweaved and indicates 
 * the reason for it.
 * After a certain amount of time, the dialog is disposed automatically.
 * That time can be configured by changing the field DIALOG_DISPLAY_TIME of
 * this class.
 * 
 * @author Michael Jehle
 */
public class SelectionInfoDialog
{
    /**
     * Specify the display time of the dialog here.
     * (Indicated in milliseconds).
     */
    private static final long DIALOG_DISPLAY_TIME = 8000;
    
    /**
     * Default constructor.
     * 
     * @param messages List<String>, the messages generated by the selection analyzer.
     */
    public SelectionInfoDialog(List<String> messages)
    {
        this.init(messages);
    }

    /**
     * Initializes and displays the JFrame which shows the messages.
     * 
     * @param messages List<String>
     */
    private void init(List<String> messages)
    {
        PointerInfo pointerInfo = MouseInfo.getPointerInfo();
        Point mousePosition = pointerInfo.getLocation();
        
        JFrame f = new JFrame();
        f.setLocation(mousePosition);
        f.setTitle("Unweaver: Invalid Selection");
        
        JTextArea textArea = new JTextArea();
        textArea.setEditable(false);
        textArea.setFont(new Font("Arial", Font.PLAIN, 10));
        
        for (String s : messages)
        {
            textArea.setText(textArea.getText() + "\n" + s);
        }
        
        f.getContentPane().add(textArea);
        f.setSize(400, 45 + (messages.size() * 17));
        
        f.setVisible(true);
        
        SelectionInfoDialogTimer timer = new SelectionInfoDialogTimer(f, SelectionInfoDialog.DIALOG_DISPLAY_TIME);
        timer.start();
    }
}