package ch.unizh.ifi.adora.unweaver.jobManager;

import java.util.List;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.geometry.Point;
import ch.unizh.ifi.adora.model.representation.VisualizableModelElement;
import ch.unizh.ifi.adora.unweaver.dialog.SelectionInfoDialog;
import ch.unizh.ifi.adora.unweaver.modelModification.ModelModifier;
import ch.unizh.ifi.adora.unweaver.selection.SelectionAnalyzer;
import ch.unizh.ifi.adora.unweaver.selection.UserSelection;

/**
 * This is the class that receives the selected model elements and the
 * coordinates for the new variant.
 * As soon as the coordinates for a new variant arrive, the unweaver is
 * triggered.
 * 
 * @author Michael Jehle
 */
public class JobManager
{
    private static JobManager INSTANCE;
    private UserSelection selection;
    
    /**
     * Private constructor (singleton).
     */
    private JobManager()
    {
        
    }
    
    public static JobManager getInstance()
    {
        if (JobManager.INSTANCE == null)
        {
            JobManager.INSTANCE = new JobManager();
        }
        
        return JobManager.INSTANCE;
    }

    /**
     * If the user triggers an unweaving job, it will arrive here.
     *  
     * @param job List<Object>
     */
    public void addJob(List<VisualizableModelElement> job)
    {
        UserSelection selection = new UserSelection(job);
        selection.detectRequiredDecisions();
        
        this.selection = selection;
        
        SelectionAnalyzer analyzer = new SelectionAnalyzer(this.selection);

        if (analyzer.getErrorMessages().size() > 0)
        {
            AdoraPlugin.getActiveEditor().resetUnweaveAction();
            new SelectionInfoDialog(analyzer.getErrorMessages());
        }
    }
    
    /**
     * This method is called, when the user has defined, where to
     * put the extracted elements.
     * Here the actual extraction is triggered.
     * 
     * @param coords int[2] {xCoord, yCoord}
     */
    public void addCoordinates(int[] coords)
    {
        ModelModifier mf = new ModelModifier(this.selection, new Point(coords[0], coords[1]));
        mf.unweave();
    }
}