package ch.unizh.ifi.adora.unweaver.modelModification;

import java.util.ArrayList;
import java.util.List;

import ch.unizh.ifi.adora.model.geometry.Point;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.model.representation.State;
import ch.unizh.ifi.adora.model.representation.VisualizableModelElement;


/**
 * This class is responsible for the handling of the layout in the new
 * variant.
 * It analyzes the model elements that are moved and decides, where they are placed.
 * 
 * @author Michael Jehle
 */
public class LayoutHandler
{
    /**
     * Constant point which indicates the top- and leftmost position
     * of an element inside its container.
     */
    private final Point topLeft = new Point(60, 10);
    
    private List<Point> placesUsed = new ArrayList<Point>();
    
    private List<Component> componentsPlaced = new ArrayList<Component>();
    private List<State> statesPlaced = new ArrayList<State>();
    
    private int lowestScenarioLevelUsed = 10;
    
    /**
     * The singleton's instance.
     */
    private static LayoutHandler INSTANCE;
    
    /**
     * Private constructor.
     */
    private LayoutHandler()
    {
        
    }
    
    /**
     * Returns the singleton's instance.
     * 
     * @return LayoutHadler
     */
    public static LayoutHandler getInstance()
    {
        if (LayoutHandler.INSTANCE == null)
        {
            LayoutHandler.INSTANCE = new LayoutHandler();
        }
        
        return LayoutHandler.INSTANCE;
    }
    
    public void reset()
    {
        this.componentsPlaced = new ArrayList<Component>();
        this.lowestScenarioLevelUsed = 10;
        this.placesUsed = new ArrayList<Point>();
        this.statesPlaced = new ArrayList<State>();
    }
    
    public Point getInsertionPointFor(VisualizableModelElement e)
    {
        if (e instanceof Component)
        {
            Point result = this.getNextComponentInsertionPoint();
            this.componentsPlaced.add((Component) e);
            this.placesUsed.add(result);
            return result;
        }
        
        if (e instanceof Scenario)
        {
            Point result = this.getScenarioInsertionPoint((Scenario) e);
            this.placesUsed.add(result);
            return result;
        }
        
        return new Point(0, 0);
    }
    
    private Point getNextComponentInsertionPoint()
    {
        Point result = new Point(this.topLeft.x, this.topLeft.y);
        
        int c = this.componentsPlaced.size();
        
        result.x += (c * 120);
        
        return result;
    }
    
    private Point getScenarioInsertionPoint(Scenario s)
    {
        if (!s.getScenarioParent().getParent().equals(s.getParent()))
        {
            Point result = new Point(this.topLeft.x, this.lowestScenarioLevelUsed + 90);
            
            this.lowestScenarioLevelUsed += 90;
            
            return result;
        }
        else
        {
            int rootLevel = s.getScenarioParent().getLocation().y;
            int childLevel = rootLevel + 90;
            
            int x = this.topLeft.x;
            
            for (Point used : this.placesUsed)
            {
                if (used.y == childLevel)
                {
                    if (used.x >= x)
                    {
                        x = used.x + 120;
                    }
                }
            }
            
            Point result = new Point(x, childLevel);
            return result;
        }
    }
}