package ch.unizh.ifi.adora.unweaver.observer;

import java.util.List;
import java.util.Observable;
import java.util.Observer;

import ch.unizh.ifi.adora.model.representation.VisualizableModelElement;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;
import ch.unizh.ifi.adora.unweaver.blackboard.board.Blackboard;
import ch.unizh.ifi.adora.unweaver.jobManager.JobManager;

public class BlackboardObserver implements Observer
{
    private Blackboard blackboard;
   
    /**
     * Registers this class as an observer of the blackboard.
     * This is called every time, a new adora model editor is activated.
     * 
     * @param editor AdoraModelEditor
     */
    public void injectObserver(AdoraModelEditor editor)
    {
        this.blackboard = (Blackboard) editor.getUnweaverBlackboard();
        this.blackboard.addObserver(this);
    }
    
    /**
     * Removes itself as an observer of a blackboard. this is used, when
     * the active editor is changed (a model is closed or another model is opened).
     * 
     * @param editor AdoraModelEditor
     */
    public void removeObserver(AdoraModelEditor editor)
    {
        if (this.blackboard != null)
        {
            this.blackboard.removeObserver(this);
            this.blackboard = null;
        }
    }
    
    
    public void update(Observable observeable, Object value)
    {
        if (value instanceof int[])
        {
            //got new extraction coordinates
            JobManager.getInstance().addCoordinates((int[]) value);
            
        }
        else
        {
            //got new selection
            List<VisualizableModelElement> job = (List<VisualizableModelElement>) value;
            JobManager.getInstance().addJob(job);            
        }
    }   
}