package ch.unizh.ch.ifi.adora.weaver.layout;

import java.awt.Rectangle;
import java.util.ArrayList;
import java.util.List;

import ch.unizh.ifi.adora.model.geometry.Dimension;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.model.representation.ScenarioConnection;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.model.util.NodeGroup;

/**
 * Layouts scenario group optical appealing.
 * 
 * @author Anil Kandrical
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 13.03.2009 AK First Version
 * @version $Date: 2009-04-16 10:09:32 +0200 (Do, 16 Apr 2009) $, $Author: anil
 *          $, $Revision: 4102 $
 */
public class ScenarioLayouter {
	public ScenarioLayouter(ZoomableNodeContainer comp, Rectangle bound) {
		ArrayList<NodeGroup> nodeGroup = comp.getNodeGroups()
				.getScenarioGroups();
		ArrayList<Node> nodeOrdering = new ArrayList<Node>();
		setUniqueStateNodeSize(nodeGroup, nodeOrdering);
		// place the nodes
		placeScenarioNode(nodeOrdering, bound);
	}

	private List<Scenario> scenariosPerHeight = new ArrayList<Scenario>();

	private void setUniqueStateNodeSize(ArrayList<NodeGroup> nodeGroup,
			ArrayList<Node> nodeOrdering) {
		for (int i = 0; i < nodeGroup.size(); i++) {
			for (Node node : nodeGroup.get(i).getNodes()) {
				nodeOrdering.add(node);
				// set Size of node
				node.setSize(new Dimension(100, 70));
			}
		}
	}

	private void placeScenarioNode(ArrayList<Node> nodeOrdering,
			Rectangle nodeParentBound) {
		int nodeHeight = (int) nodeOrdering.get(0).getBounds().getHeight();
		Scenario root = fetchScenarioRoot(nodeOrdering.get(0));
		int nHeight = calculateScenarioTreeHeight(root);

		int nodeHeightDistance = (int) (Math.round(nodeParentBound.getHeight()
				- nHeight * nodeHeight) / (nHeight + 1));
		// calculate nodes per tree layer
		buildTree(root, nodeParentBound.width, nodeHeightDistance,
				calculateScenarioTreeHeight(root));

	}

	private void buildTree(Scenario root, int width, int height, int layerHeight) {
		for (int i = 1; i <= layerHeight; i++) {
			List<Scenario> scenariosPerLayer = getScenarioNodesPerHeight(root,
					i);
			int layerSpecificWidth = (width - scenariosPerLayer.size() * 100)
					/ (scenariosPerLayer.size() + 1);
			// int layerSpecificWidth=(width-())/(+);
			for (int j = 0; j < scenariosPerLayer.size(); j++) {
				scenariosPerLayer.get(j).setLocation(
						layerSpecificWidth + (j * layerSpecificWidth)
								+ (j * 100), i * height + (i - 1) * 70);
				// optimize connections
				for (Connection con : scenariosPerLayer.get(j).getConnections()) {
					if (con instanceof ScenarioConnection) {
						con.disconnect();
						con.reconnect();
					}
				}

			}
		}
	}

	private List<Scenario> getScenarioNodesPerHeight(Scenario root, int height) {
		scenariosPerHeight = new ArrayList<Scenario>();
		getScenarioNodesPerHeightHelper(root, height);
		return scenariosPerHeight;
	}

	private void getScenarioNodesPerHeightHelper(Scenario root, int height) {
		if (height == 1) {
			scenariosPerHeight.add(root);
			return;
		}
		if (root.getScenarioChildren().size() != 0) {
			for (int i = 0; i < root.getScenarioChildren().size(); i++) {
				getScenarioNodesPerHeightHelper(root.getScenarioChildren().get(
						i), height - 1);
			}
		}
	}


	private int calculateScenarioTreeHeight(Scenario scenarioNode) {
		if (scenarioNode.getScenarioChildren().size() == 0) {
			return 1;
		}
		int height = 0;
		for (int i = 0; i < scenarioNode.getScenarioChildren().size(); i++) {
			int branchHeight = calculateScenarioTreeHeight(scenarioNode
					.getScenarioChildren().get(i));
			if (height < branchHeight) {
				height = branchHeight;
			}

		}
		return 1 + height;

	}

	private Scenario fetchScenarioRoot(Node node) {
		Scenario scenarioNode = (Scenario) node;
		if (scenarioNode.getScenarioParent() == null) {
			return scenarioNode;
		}
		return fetchScenarioRoot(scenarioNode.getScenarioParent());
	}
}
