package ch.unizh.ch.ifi.adora.weaver.layout;

import java.awt.Rectangle;
import java.util.ArrayList;

import ch.unizh.ifi.adora.model.geometry.Dimension;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.Transition;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.model.util.NodeGroup;
/**
 * Layouts state group optical appealing.
 * 
 * @author Anil Kandrical
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 13.03.2009 AK First Version
 * @version $Date: 2009-04-16 10:09:32 +0200 (Do, 16 Apr 2009) $, $Author: anil
 *          $, $Revision: 4102 $
 */
public class StateGroupLayouter {
	public StateGroupLayouter(ZoomableNodeContainer comp, Rectangle bound) {
		ArrayList<NodeGroup> nodeGroup = comp.getNodeGroups().getStateGroups();
		ArrayList<Node> nodeOrdering = new ArrayList<Node>();
		setUniqueStateNodeSize(nodeGroup, nodeOrdering);
		// place the nodes
		placeStateNode(nodeOrdering, bound);
		optimizeConnections(nodeOrdering);
	}
	/**
	 * Place nodes within its parent node. 
	 * @param nodeOrdering
	 * @param nodeParentBound
	 */

	private void placeStateNode(ArrayList<Node> nodeOrdering,
			Rectangle nodeParentBound) {
		int nodeWidth = (int) nodeOrdering.get(0).getBounds().getWidth();
		int nodeHeight = (int) nodeOrdering.get(0).getBounds().getHeight();
		int nWidth = Math.round(((nodeParentBound.width) / (nodeOrdering.get(0)
				.getBounds().width)));
		int nHeight;
		// handles case where there are fewer nodes than width e.g. 2 nodes but
		// a nWidth of 3
		if (nWidth > nodeOrdering.size()) {
			nHeight = 1;
			nWidth = nodeOrdering.size();
		} else {
			nHeight = Math.round(nodeOrdering.size() / nWidth);
		}
		int nodeWidthDistance = (int) Math
				.round((nodeParentBound.getWidth() - nWidth * nodeWidth)
						/ (nWidth + 3));
		int nodeHeightDistance = (int) (Math.round(nodeParentBound.getHeight()
				- nHeight * nodeHeight) / (nHeight + 3));
		// distance too small, recalculate node distances
		while (nodeWidthDistance < 10) {
			nWidth = nWidth - 1;
			if (0 != (nodeOrdering.size() % nWidth)) {
				nHeight = nHeight + 1;
			}
			// nHeight=Math.round(nodeOrdering.size()/nWidth);
			nodeWidthDistance = (int) Math
					.round((nodeParentBound.getWidth() - nWidth * nodeWidth)
							/ (nWidth + 3));
			nodeHeightDistance = (int) (Math.round(nodeParentBound.getHeight()
					- nHeight * nodeHeight) / (nHeight + 3));
			if (nodeWidthDistance >= 10)
				break;
		}
		for (int i = 0; i < nodeOrdering.size(); i++) {
			nodeOrdering.get(i).setLocation(
					2 * nodeWidthDistance + i % nWidth
							* (nodeWidth + nodeWidthDistance),
					2 * nodeHeightDistance + (int) (i / nWidth)
							* (nodeHeight + nodeHeightDistance));
		}

	}

	private void setUniqueStateNodeSize(ArrayList<NodeGroup> nodeGroup,
			ArrayList<Node> nodeOrdering) {
		for (int i = 0; i < nodeGroup.size(); i++) {
			for (Node node : nodeGroup.get(i).getNodes()) {
				nodeOrdering.add(node);
				// set Size of node
				node.setSize(new Dimension(100, 70));
			}
		}
	}

	private void optimizeConnections(ArrayList<Node> nodeOrdering) {
		for (Node n : nodeOrdering) {
			for (Connection con : n.getConnections()) {
				if (con instanceof Transition) {
					con.disconnect();
					con.reconnect();
				}
			}
		}

	}
}
