package ch.unizh.ifi.adora.weaver.actions;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.RootEditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.ui.IEditorActionDelegate;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.commands.LayoutOrderingCommand;
import ch.unizh.ifi.adora.model.commands.SetVariationPointListDecisions;
import ch.unizh.ifi.adora.model.commands.VariationPointConnectionCreate;
import ch.unizh.ifi.adora.model.representation.variability.Constraint;
import ch.unizh.ifi.adora.model.representation.variability.Decision;
import ch.unizh.ifi.adora.model.representation.variability.VariationPoint;
import ch.unizh.ifi.adora.parts.AdoraRootEditPart;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;
import ch.unizh.ifi.adora.productline.ui.views.constraintstable.UpdateConstraintConnections;
import ch.unizh.ifi.adora.productline.ui.views.decisiontable.DecisionTableViewPart;
import ch.unizh.ifi.adora.weaver.engine.DynamicWeaver;

/**
 * This class handles the action, which is generated after the weaving button is
 * pressed.
 * 
 * @author Pascal Schoeni
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 18.06.2007 SM First Version<br>
 *          10.12.2007 PS Comments improved.
 * @version $Date: 2010-02-11 20:47:01 +0100 (Thu, 11 Feb 2010) $, $Author: jehle
 *          $, $Revision: 5117 $
 */
public class WeaveAction implements IEditorActionDelegate
{
    /**
     * A reference to the Adora Model Editor.
     */
    private AdoraModelEditor editor;

    /**
     * Variable to store if the model is woven.
     */
    private boolean woven;

    /**
     * Determines if weaver button has to be toggled or not after changing the
     * active editor.
     */
    public void setActiveEditor(IAction action, IEditorPart targetEditor)
    {
        editor = (AdoraModelEditor) targetEditor;

        if (editor != null)
        {
            if (!editor.isWeaved())
            {
                if (woven)
                {// needed, otherwise first weaving is not going to
                    // happen
                    action.setChecked(false);
                }
            }
            // editor has an original model -->model is woven -->button needs to
            // be in
            else
            {
                action.setChecked(true);
            }
        }
    }

    /**
     * Method which is called after a push on the button.
     */
    public void run(IAction action)
    {
        DynamicWeaver dynamicWeaver = DynamicWeaver.getInstance();

        woven = action.isChecked();

        // TODO open decision history to avoid nullpointer
        if (!dynamicWeaver.isDecisionHistoryActivated())
        {
            try
            {
                PlatformUI.getWorkbench().getActiveWorkbenchWindow()
                        .getActivePage().showView(
                                DecisionTableViewPart.VIEW_ID_DECISION_HISTORY);
            } catch (PartInitException e)
            {
                e.printStackTrace();
            }

            DynamicWeaver.setBaseAdoraModel(AdoraPlugin.getActiveModel());
        }

        if (woven)
        {
            // triger

            DynamicWeaver.mustWeaveCrosscuttingConcerns = true;

            // Tell the dynamicWeaver to weave the whole model again
            // including the ccs.

            editor.setWeaved(true);
        } else
        {
            // untriger
            DynamicWeaver.mustWeaveCrosscuttingConcerns = false;
            editor.setWeaved(false);
        }

        dynamicWeaver.activateWeaving((ArrayList<Decision>) DynamicWeaver
                .getDecisionForWeaving());
        createAndShowVPandCP();
        Command command = new LayoutOrderingCommand();
        command.execute();
        
        GraphicalViewer viewer = (GraphicalViewer) AdoraPlugin.getActiveEditor().getAdapter(GraphicalViewer.class);
        viewer.flush();
        
        RootEditPart rep = AdoraPlugin.getActiveEditor().getViewer().getRootEditPart();
        AdoraRootEditPart arep = (AdoraRootEditPart) rep;
        
        arep.getZoomManager().setZoom(arep.getZoomManager().getZoom() + 0.000001);        
    }

    /**
     * Notifies this action delegate that the selection in the workbench has
     * changed.
     */
    public void selectionChanged(IAction action, ISelection selection)
    {
    }

    private void createAndShowVPandCP()
    {
        Command command = null;
        AdoraModelEditor ame = AdoraPlugin.getActiveEditor();
        List<VariationPoint> selectedVPs = AdoraPlugin.getActiveModel()
                .getVariationPoints();
        for (VariationPoint variationPoint : selectedVPs)
        {
            String decs = ",";
            for (Decision dec : variationPoint.getDecisions())
            {
                decs = decs.concat("D".concat(dec.getNr().toString()).concat(
                        ","));
            }
            command = new SetVariationPointListDecisions((String) decs,
                    variationPoint);
            command.execute();
        }
        VariationPointConnectionCreate showCommand = new VariationPointConnectionCreate(
                AdoraPlugin.getActiveModel(), selectedVPs);
        ame.getCommandStack().execute(showCommand);
        // fetch constraint and show them
        List<Constraint> selectedConst = new ArrayList<Constraint>();
        for (Constraint con : AdoraPlugin.getActiveModel().getConstraints())
        {
            selectedConst.add(con);
        }

        UpdateConstraintConnections constCreate = new UpdateConstraintConnections(
                AdoraPlugin.getActiveModel(), selectedConst);
        AdoraPlugin.getActiveEditor().getCommandStack().execute(constCreate);
    }
}
