package ch.unizh.ifi.adora.weaver.commands;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Logger;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.commands.ComponentScenarioConnectionCopyCommand;
import ch.unizh.ifi.adora.model.commands.InsertNodeVisually;
import ch.unizh.ifi.adora.model.commands.InsertNodesVisually;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.ZoomableNode;
import ch.unizh.ifi.adora.model.representation.variability.Decision;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;
import ch.unizh.ifi.adora.ui.views.ModelObserverMediator;
import ch.unizh.ifi.adora.weaver.engine.analyzer.CrosscuttingNodeGroupList;
import ch.unizh.ifi.adora.weaver.engine.analyzer.ModelAnalyzer;
import ch.unizh.ifi.adora.weaver.engine.analyzer.WeavingHistory;
import ch.unizh.ifi.adora.weaver.engine.analyzer.WeavingTarget;
import ch.unizh.ifi.adora.weaver.engine.analyzer.WeavingTargetList;
import ch.unizh.ifi.adora.weaver.engine.generator.CommandController;

/**
 * Dynamic Weaving Command for the product lines. Weaves single aspect.
 * 
 * @author Anil Kandrical
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 13.03.2009 AK First Version
 * @version $Date: 2009-04-16 10:09:32 +0200 (Do, 16 Apr 2009) $, $Author: anil
 *          $, $Revision: 4102 $
 */
public class DynamicWeaveCommand extends Command {
	private static final Logger logger = Logger
			.getLogger(DynamicWeaveCommand.class.getName());
	
	Decision dec = null;

	
	public DynamicWeaveCommand(ArrayList<AspectContainer> aspList) {
		this.performDynamicWeaving(aspList);
	}
	
	public DynamicWeaveCommand(ArrayList<AspectContainer> aspList, Decision dec) {
		this.dec = dec;
		this.performDynamicWeaving(aspList);
	}
	
	public DynamicWeaveCommand(ArrayList<AspectContainer> aspList, AdoraModelEditor editor) {
		this.performDynamicWeaving(aspList, editor);
	}


	/**
	 * Aspect weaving is performed for a single decision.
	 * 
	 * @param aspList
	 * @param routingConnections
	 */
	private void performDynamicWeaving(ArrayList<AspectContainer> aspList) {
		try {
			Command c=new ComponentScenarioConnectionCopyCommand(aspList);
			
			List<Node> wovenNodes = new ArrayList<Node>();
			
			// /Layout testing
			/// Command layoutOrderingCommand=new LayoutOrderingCommand();
			 //	layoutOrderingCommand.execute();
			// /
			AdoraModel model = ModelObserverMediator.getInstance()
					.getAdoraModelEditor().getModel();
			WeavingHistory.reset();
			CrosscuttingNodeGroupList.reset();
			model.updateParents();
			ModelAnalyzer modelAnalyzer = new ModelAnalyzer(model, aspList);
			while (WeavingTargetList.getInstance().getWeavables()
					.hasMoreElements()) {
				/*
				 * The CommandController creates commands for each weaving
				 * component. These commands are not automatically executed.
				 */
				CommandController commandController = new CommandController(
						modelAnalyzer);

				// Execution of the generated commands
				for (Command command : commandController.getCommands()) {
					ModelObserverMediator.getInstance().getAdoraModelEditor()
							.getCommandStack().execute(command);
					
					if (command instanceof InsertNodesVisually) {
						wovenNodes.addAll(((InsertNodesVisually)command).getNodesToInsert());
					}
				}
				model.updateParents();
				modelAnalyzer = new ModelAnalyzer(model, aspList);
				
				//add this node to the list of woven nodes of the corresponding decision.
			}
			
			//add the elements which got woven to the decisions
			
			//dec.setWovenElements(wovenNodes);
			if (dec != null) {
				Decision decCloned = AdoraPlugin.getActiveModel().getDecisionsMap().get(dec.getNr());
				decCloned.setWovenElements(wovenNodes);
			}
			
			for (int i = 0; i < aspList.size(); i++) {
				this.deleteAspectContainers(aspList, model);
			}
			c.execute();
			// /Layout testing
			//layoutOrderingCommand.execute();
			// /
		} catch (Exception e) {
			logger.warning(e.getMessage());
			e.printStackTrace();
		}
	}
	
	
	/**
	 * Aspect weaving is performed for a single decision.
	 * 
	 * @param aspList, model
	 * @param routingConnections
	 */
	private void performDynamicWeaving(ArrayList<AspectContainer> aspList, AdoraModelEditor editor) {
		try {
			Command c=new ComponentScenarioConnectionCopyCommand(aspList);
			// /Layout testing
			/// Command layoutOrderingCommand=new LayoutOrderingCommand();
			 //	layoutOrderingCommand.execute();
			// /
			AdoraModel model = editor.getModel();
			WeavingHistory.reset();
			CrosscuttingNodeGroupList.reset();
			model.updateParents();
			ModelAnalyzer modelAnalyzer = new ModelAnalyzer(model, aspList);
			while (WeavingTargetList.getInstance().getWeavables()
					.hasMoreElements()) {
				/*
				 * The CommandController creates commands for each weaving
				 * component. These commands are not automatically executed.
				 */
				CommandController commandController = new CommandController(
						modelAnalyzer);

				// Execution of the generated commands
				for (Command command : commandController.getCommands()) {
					editor.getCommandStack().execute(command);
				}
				model.updateParents();
				modelAnalyzer = new ModelAnalyzer(model, aspList);
			}
			for (int i = 0; i < aspList.size(); i++) {
				this.deleteAspectContainers(aspList, model);
			}
			c.execute();
			// /Layout testing
			//layoutOrderingCommand.execute();
			// /
		} catch (Exception e) {
			logger.warning(e.getMessage());
			e.printStackTrace();
		}
	}



	/**
	 * Unused aspect containers, which have no connections to a module, will be
	 * deleted.
	 * 
	 * @param aspList
	 * @param model
	 */
	private void deleteAspectContainers(ArrayList<AspectContainer> aspList,
			AdoraModel model) {
		Iterator<ModelElement> iterator = model.getRecursiveChildrenIterator();

		while (iterator.hasNext()) {
			ModelElement modelElement = iterator.next();
			
			if (modelElement instanceof AspectContainer) {
				AspectContainer aspInSearch = (AspectContainer) modelElement;
				for (AspectContainer asp : aspList) {
					// weaved empty containers are set hidden
					if (asp.equals(aspInSearch) && 
							!hasOutgoingUndecidedDecision(aspInSearch)) {
						asp.setManuallyHidden(true);
						deleteHierarchicalAspects(asp);
					}
				}
			}
		}
	}
	
	/**
	 * This helper method is used to check whether or not an {@link AspectContainer} can be deleted from the {@link AdoraModel}.
	 * A {@link ConnectableElement} can be deleted, if it and non of its children have no outgoing {@link Decision} which are undecided an
	 * 
	 * @param ac The {@link AspectContainer} which gets tested.
	 * @return true, if the {@link AspectContainer} has an undecided outgoing {@link Decision}.
	 */
	private boolean hasOutgoingUndecidedDecision(AspectContainer ac) {
		for (JoinRelation jr : ac.getAllSourceJRs()) {
			if (jr.getDecision() != null && jr.getDecision().getDecisionValue() == null)
				return true;
		}
		return false;
	}

	private void deleteHierarchicalAspects(AspectContainer childAspectContainer) {
		if (childAspectContainer.getParent() instanceof Component
				|| childAspectContainer.getParent() instanceof AdoraModel) {
			childAspectContainer.setManuallyHidden(true);
			return;
		}
		if (childAspectContainer.getParent() instanceof AspectContainer) {
			AspectContainer parentContainer = (AspectContainer) childAspectContainer
					.getParent();
			if (checkChildrenVisibility(parentContainer)
					&& checkConnectionVisibility(parentContainer)
					&& parentContainer.getParent() instanceof AdoraModel) {
				parentContainer.setManuallyHidden(true);
				return;
			}
		}
	}

	private boolean checkChildrenVisibility(AspectContainer parentContainer) {
		for (ModelElement modelElement : parentContainer.getChildren()) {
			ZoomableNode childNode = (ZoomableNode) modelElement;
			if (childNode.isVisible()) {
				return false;
			}
		}
		return true;
	}

	private boolean checkConnectionVisibility(AspectContainer parentContainer) {
		for (ModelElement modelElement : parentContainer.getConnections()) {
			ConnectableElement aspectConnection = (ConnectableElement) modelElement;
			if (aspectConnection.isVisible()) {
				return false;
			}
		}
		return true;
	}
}
