package ch.unizh.ifi.adora.weaver.commands;

import java.awt.Rectangle;
import java.util.ArrayList;
import java.util.Iterator;

import org.eclipse.gef.commands.Command;

import ch.unizh.ch.ifi.adora.weaver.layout.ScenarioLayouter;
import ch.unizh.ch.ifi.adora.weaver.layout.StateGroupLayouter;
import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.geometry.Dimension;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.ui.views.ModelObserverMediator;

/**
 * Produce optical appealing layout of woven models.
 * 
 * @author Anil Kandrical
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 13.03.2009 AK First Version
 * @version $Date: 2009-04-16 10:09:32 +0200 (Do, 16 Apr 2009) $, $Author: anil
 *          $, $Revision: 4102 $
 */
public class LayoutOrderingCommand extends Command {
	public void execute() {
		AdoraModel model = ModelObserverMediator.getInstance()
				.getAdoraModelEditor().getModel();
		for (ModelElement modelElement : getModelElement(model)) {
			if (getModelElement(model) != null) {
				orderLayout(modelElement);
			}
		}
	}

	private ArrayList<ModelElement> getModelElement(AdoraModel model) {
		ArrayList<ModelElement> modelElementList = new ArrayList<ModelElement>();
		Iterator<ModelElement> iterator = model.getRecursiveChildrenIterator();

		while (iterator.hasNext()) {
			ModelElement modelElement = iterator.next();
			if (modelElement instanceof AspectContainer) {
				modelElementList.add(modelElement);

			}
			if (modelElement instanceof Component) {
				modelElementList.add(modelElement);

			}

		}
		return modelElementList;
	}

	private void orderLayout(ModelElement modelElement) {
		if (modelElement instanceof AspectContainer||modelElement instanceof Component) {
			ZoomableNodeContainer zoomableNodeCOntainer = (ZoomableNodeContainer) modelElement;
			Rectangle bound = zoomableNodeCOntainer.getBounds();
			handleZoomableNodeContainerGroups(zoomableNodeCOntainer, bound);
		}
	}

	private void handleZoomableNodeContainerGroups(ZoomableNodeContainer znc,
			Rectangle bound) {
		if(znc.getNodeGroups().getNodeGroups().size()!=0){
		if (znc.getNodeGroups().getStateGroups().size() != 0) {
			new StateGroupLayouter(znc, bound);
		}
		if (znc.getNodeGroups().getScenarioGroups().size() != 0) {
			new ScenarioLayouter(znc, bound);
		}
		}
		else{
			// all componente and aspects without children are set to this size 
			znc.setSize(new Dimension(100,70));
		}
	}
}
