package ch.unizh.ifi.adora.weaver.commands;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.gef.ui.actions.Clipboard;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.ui.IWorkbenchPart;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.actions.AdoraCopyAction;
import ch.unizh.ifi.adora.actions.AdoraPasteAction;
import ch.unizh.ifi.adora.model.IContainer;
import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.commands.ConnectionDelete;
import ch.unizh.ifi.adora.model.commands.InsertNodeVisually;
import ch.unizh.ifi.adora.model.commands.JoinRelationCreate;
import ch.unizh.ifi.adora.model.commands.NodeCreate;
import ch.unizh.ifi.adora.model.commands.NodeDelete;
import ch.unizh.ifi.adora.model.factories.ModelObjectMapper;
import ch.unizh.ifi.adora.model.geometry.Point;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.INodeContainer;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.ZoomableNode;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.model.representation.variability.Decision;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;
import ch.unizh.ifi.adora.ui.views.ModelObserverMediator;
import ch.unizh.ifi.adora.weaver.engine.DynamicWeaver;
import ch.unizh.ifi.adora.weaver.engine.analyzer.GroupCloner;

/**
 * Set visibility of the decisions and aspect visibility for the product lines.
 * 
 * @author Anil Kandrical
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 13.03.2009 AK First Version
 * @version $Date: 2009-04-16 10:09:32 +0200 (Do, 16 Apr 2009) $, $Author: anil
 *          $, $Revision: 4102 $
 */
public class SetDecisionVisible extends Command {
	
	
	/**
	 * IV: This is a temporary solution for the bug in which  
	 * {@link JoinRelation}s  get connected to the wrong Node.
	 * 
	 * To really correct it the sequence of actions taken in this command
	 * has to be altered. 
	 */
	private List<Object[]> reconnectList = new ArrayList<Object[]>(); 
	
	
	/**
	 * A {@link List} of {@link Decision}s which have to be checked 
	 * since some of their {@link JoinRelation} sources were  wrongly set to invisible.
	 */
    List<Decision> resetVisibilityList = new ArrayList<Decision>();;
	

	private ArrayList<AspectContainer> weavableAspectContainers;
	HashMap<Node, Connection> routingConnections;

	public SetDecisionVisible(Decision dec) {
		weavableAspectContainers = new ArrayList<AspectContainer>();
		
		//concurrent modification error
		boolean handlededAll = handleDecisions2ACs(dec);
		if (!handlededAll)
			checkDecisionValueForWeavability(dec, false);

	}

	/**
	 * @return a List of [{@link JoinRelation}, {@link Node}] tuples
	 * with {@link JoinRelation}s which need to be reconnected to the
	 * {@link Node}. 
	 */
	public List<Object[]> getJRsToChange() {
		return reconnectList;
	}
	
	
	/**
     * Returns a {@link List} of {@link Decision}s which have to be checked 
     * since some of their {@link JoinRelation} sources were  wrongly set to invisible. 
     */
	public List<Decision> getResetVisibilityList() {
		return resetVisibilityList;
	}

	/**
	 * Decision is checked and hidden or shown according to its decision value.
	 * Aspect Container are then also hidden or shown.
	 * 
	 * @param dec
	 */

	private void checkDecisionValueForWeavability(Decision dec,
			boolean incomingDecision) {
		// no incoming joinrelation in this aspect
		if (dec.getDecisionValue() != null && incomingDecision == false) {
			// decision="false" is treated here
			if (dec.getDecisionValue() == false) {
				//setVisibilityOfAspectsOfDecision(false, dec);
			} else {
				// decision="true" is treated as decision visibility=true
				setVisibilityOfAspectsOfDecision(true, dec);
				DynamicWeaveCommand command = new DynamicWeaveCommand(
						weavableAspectContainers, dec);
				command.execute();
				
			}
		}
		// decision="undecided" is treated as decision visibility=true
		if (dec.getDecisionValue() == null && incomingDecision == false) {
			setVisibilityOfAspectsOfDecision(true, dec);
		}
		// there are incoming joinrelation in this aspect
		if (incomingDecision) {
			//if (isDecisionVisible(dec)) {
			if (isDecisionVisible(dec)) {
				// decision="true" is treated as decision visibility=true
				setVisibilityOfAspectsOfDecision(true, dec);
			} else {
				// decision="false" is treated here
				if (hasDecisionOtherTruePath(dec)) {
					
					//TODO: IV here we fix the bug that too many ACs get hidden
					resetVisibilityList.add(dec);
				}
				setVisibilityOfAspectsOfDecision(false, dec);
			}

		}
	}

	/**
	 * @param dec The {@link Decision} to be checked.
	 * @return True <=> if some {@link JoinRelation} of this {@link Decision} 
	 * has a true-path to a commonality. This is needed to reset the visibility of wrongly hidden {@link Nodes}s
     * and there parents. 
     */
	private boolean hasDecisionOtherTruePath(Decision dec) {
		for (JoinRelation jr : dec.getJoinRelations()) {
			
			if (dec.hasJRtruePathToCommonality(jr))
				return true;
		}
			
		return false;
	}

	/**
	 * This method handles a aspect, which is attached to the decision which is
	 * set to true, false or not defined. According to this decision the
	 * following aspects and joinrelations are set visible/invisible.
	 * 
	 * @param decVisible
	 * @param dec
	 */

	private void setVisibilityOfAspectsOfDecision(boolean decVisible,
			Decision dec) {

		// Gets Aspects of decision, where the decision joinrelation are coming
		// from
		List<AspectContainer> decAspects = fetchSourceAspectOfDecision(dec);
		// Check whether any other outgoing join relations are set true. If this
		// is the case, do not delete the aspect.
		for (AspectContainer decAspect : decAspects) {
			if (checkForOtherOutgoingJoinRelations(decAspect, dec)
					&& decVisible == false) {
				// there are outgoing joinrelations therefore it aspect must be
				// visible.
				decAspect.setVisible(true);
				checkWeavabilityOfAspect(dec, decAspect);
				// this return is needed , since we do not want incoming join
				// relations to be set to false
				return;
			}
			if (checkForOtherOutgoingJoinRelations(decAspect, dec) == false
					&& isDecisionVisible(dec) == false) {
				decAspect.setVisible(isDecisionVisible(dec));
			}
			if (decVisible == true) {
				decAspect.setVisible(isDecisionVisible(dec));
				checkWeavabilityOfAspect(dec, decAspect);
			}
			handleIncomingJoinRelations(decVisible, dec, decAspect);
			if (decVisible) {
				adjustDecisionVisibility();
			}
		}
	}

	private void handleIncomingJoinRelations(boolean decVisible, Decision dec,
			AspectContainer decAspect) {

		// handle incoming joinrelations of an aspect
		List<Decision> incomingJoinRelations = getIncomingJoinRelations(
				decAspect, dec);

		if (incomingJoinRelations.size() > 0) {
			// Repeat iteratively with each join relation
			for (int i = 0; i < incomingJoinRelations.size(); i++) {
				// set Incoming Joinrelation visibility to them of the
				// aspect
				// container

				setVisibilityOfDecision(incomingJoinRelations.get(i),
						decVisible);
				if (decVisible == false) {
					checkDecisionValueForWeavability(incomingJoinRelations
							.get(i), true);
				}
				cutOffIncomingDecision(dec, incomingJoinRelations.get(i)
						.getJoinRelations().get(0));
				if (dec.getDecisionValue() != null) {
					if (dec.getDecisionValue() == true) {
						handleJoinRelationReRouting(dec, incomingJoinRelations,
								i);

					}
				}
			}
		}

		// set incoming joinrelations invisible, if aspect is invisible
		if (incomingJoinRelations.size() == 0 && decVisible == false
				&& checkForOtherOutgoingJoinRelations(decAspect, dec) == false) {
			AdoraPlugin.getActiveModel().updateParents();
			// Jehles Version to hide an aspect
			decAspect.setManuallyHidden(true);
			// /set cascaded aspectContainer invisible
			if (decAspect.getParent() instanceof AspectContainer) {

				AspectContainer parent = ((AspectContainer) decAspect
						.getParent());
				try {
					parent.setManuallyHidden(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}
		// /
	}

	private void handleJoinRelationReRouting(Decision dec,
			List<Decision> incomingJoinRelations, int i) {
		// reroute joinrelationship which were pointing at this
		// aspect to the new location
		ConnectableElement weavableJoinTarget;
		
		if (incomingJoinRelations.get(i).getJoinRelations().get(0).getTarget() instanceof AspectContainer) {
			weavableJoinTarget = (ConnectableElement) dec.getJoinRelations()
					.get(0).getTarget().getParent();
		} else {
			//
			//a jr gets connected for example to a scenario
			weavableJoinTarget = dec.getJoinRelations().get(0).getTarget();
			reconnectList.add(new Object[] {incomingJoinRelations.get(i)
					.getJoinRelations().get(0), 
											incomingJoinRelations.get(i).getJoinRelations().get(0).getTarget()});
			
		}

		ConnectableElement weavableJoinSource = incomingJoinRelations.get(i)
				.getJoinRelations().get(0).getSource();
		rerouteJoinRelation(weavableJoinSource, weavableJoinTarget,
				(incomingJoinRelations.get(i).getJoinRelations().get(0)));
	}

	/**
	 * Reroute a join relation, which is not directly involved in weaving
	 * activity. But needs rerouting.
	 * 
	 * @param weavableJoinSource
	 * @param weavableJoinTarget
	 * @param joinRelation
	 */
	private void rerouteJoinRelation(ConnectableElement weavableJoinSource,
			ConnectableElement weavableJoinTarget, JoinRelation joinRelation) {
		joinRelation.disconnect();
		joinRelation.reconnect(weavableJoinSource, weavableJoinTarget);

	}

	/**
	 * Deletion of incoming joinrelations, if decision is set true. Meaning
	 * aspect will be weaved!
	 * 
	 * @param dec
	 * @param joinRelation
	 */
	private void cutOffIncomingDecision(Decision dec, JoinRelation joinRelation) {
		if (dec.getDecisionValue() != null) {
			if (dec.getDecisionValue()) {
				ConnectionDelete command = new ConnectionDelete(joinRelation,
						true);
				command.execute();

			}
		}
	}

	/**
	 * Returns the visibility of the joinrelation of a decision.
	 * 
	 * @param dec
	 * @return
	 */
	private boolean isDecisionVisible(Decision dec) {
		for (JoinRelation joinRelation : dec.getJoinRelations()) {
			return joinRelation.isVisible();
		}
		return false;
	}

	/**
	 * Sets the visibility of the joinrelation of a decision.
	 * 
	 * @param dec
	 * @param v
	 */
	private void setVisibilityOfDecision(Decision dec, boolean v) {
		for (JoinRelation joinRelation : dec.getJoinRelations()) {
			joinRelation.setVisibility(v);
		}
	}

	/**
	 * Incoming joinrelations are searched of an aspect, from which we set the
	 * decision true, false or not defined.
	 * 
	 * @param decAspect
	 * @param dec
	 * @return
	 */
	private List<Decision> getIncomingJoinRelations(AspectContainer decAspect,
			Decision dec) {
		List<Decision> l = new ArrayList<Decision>();
		for (Decision decisionTargetAspect : AdoraPlugin.getActiveModel()
				.getDecisions()) {
			if (!DynamicWeaver.getWeavedDecisionHistory(decisionTargetAspect
					.getNr())) {
				// all decisions which are not the analyted decision
				if (!decisionTargetAspect.equals(dec)
						&& isTargetAspectOfDecision(decisionTargetAspect,
								decAspect)) {
					l.add(decisionTargetAspect);
				}
			}
		}
		return l;
	}

	/**
	 * Here we fetch the target aspect of a decision if it is a aspectcontainer.
	 * 
	 * @param dec
	 * @param asp
	 * @return
	 */
	private Boolean isTargetAspectOfDecision(Decision dec, AspectContainer asp) {
		for (JoinRelation joinRelation : dec.getJoinRelations()) {
			AspectContainer sourceAspect = getAspectContainerOfDecision((ZoomableNode) joinRelation
					.getSource());
			// Target of a joinrelation(or decision) can be a node or a
			// connection(think of a states and aspects).
			if (joinRelation.getTarget() instanceof ZoomableNode
					&& !(sourceAspect.equals(asp))) {
				return ((getAspectContainerOfDecision((ZoomableNode) joinRelation
						.getTarget())) != null && asp
						.equals(getAspectContainerOfDecision((ZoomableNode) joinRelation
								.getTarget())));
			}
			if (joinRelation.getTarget() instanceof Connection) {
				return ((getAspectContainerOfDecision((Connection) joinRelation
						.getTarget())) != null && asp
						.equals(getAspectContainerOfDecision((Connection) joinRelation
								.getTarget())));
			}
		}
		return false;
	}

	/**
	 * Fetch the Source AspectContainer from which the JoinRelation of the
	 * decision originates.
	 * 
	 * @param dec
	 * @return
	 */
	private List<AspectContainer> fetchSourceAspectOfDecision(Decision dec) {
		List<AspectContainer> aspList = new ArrayList<AspectContainer>();
		for (JoinRelation joinRelation : dec.getJoinRelations()) {
			aspList
					.add((getAspectContainerOfDecision((ZoomableNode) joinRelation
							.getSource())));
		}
		return aspList;
	}

	/**
	 * This method returns the aspect for a connection (Scenario or transition
	 * Connection).
	 * 
	 * @param target
	 * @return
	 */
	private AspectContainer getAspectContainerOfDecision(Connection target) {
		ZoomableNode zoomableNode = (ZoomableNode) target.getSource();
		return getAspectContainerOfDecision(zoomableNode);
	}

	/**
	 * Check if this aspect has other outgoing JoinRelations (better Decisions)
	 * beside the one which is already known.
	 * 
	 * @param aspectContainerOfDecision
	 * @return
	 */

	private Boolean checkForOtherOutgoingJoinRelations(
			AspectContainer aspectContainerOfDecision, Decision decision1) {
		// Search for all outgoing decisions of the AspectContainer.
		for (Decision dec : AdoraPlugin.getActiveModel().getDecisions()) {
			// this check assures, that already weaved aspects are not
			// considered anymore
			if (!DynamicWeaver.getWeavedDecisionHistory(dec.getNr())) {
				// search for decision with same aspect as source
				if (fetchSourceAspectOfDecision(dec) != null) {
					for (AspectContainer aspectContainer : fetchSourceAspectOfDecision(dec)) {
						if (aspectContainer.equals(aspectContainerOfDecision)
								&& !(dec.equals(decision1))
								&& isDecisionVisible(dec) == true) {
							return true;
						}
					}
				}
			}

		}
		return false;
	}

	/**
	 * This method searches for AspectContainer which is source of a decisions
	 * join relation.
	 * 
	 * @param zoomableNode
	 * @return
	 */
	private AspectContainer getAspectContainerOfDecision(
			ZoomableNode zoomableNode) {

		if (zoomableNode == null) {
			return null;
		}

		if (zoomableNode instanceof AspectContainer) {
			return (AspectContainer) zoomableNode;
		} else {
			if (zoomableNode.getParent() instanceof AdoraModel) {
				return null;
			}
			return getAspectContainerOfDecision((ZoomableNode) zoomableNode
					.getParent());
		}
	}

	/**
	 * This method is needed to overcome the setChildren Visibility Problem.
	 * AspectContainer also have outgoing joinrelations as there children. But
	 * when we set aspectcontainer visible, we do not always want all outgoing
	 * joinrelations to be set visible. This method resets all outgoing
	 * joinrelations, which are should not be visible but are visible back.
	 */
	private void adjustDecisionVisibility() {
		for (Decision dec : AdoraPlugin.getActiveModel().getDecisions()) {
			if (dec.getDecisionValue() != null
					&& dec.getDecisionValue() == false) {
				setVisibilityOfDecision(dec, dec.getDecisionValue());
				this.setVisibilityOfAspectsOfDecision(dec.getDecisionValue(),
						dec);
			}
		}

	}

	/**
	 * Check if a aspect is weavable according to a true set decision. Mark
	 * Aspect as weavable.
	 * 
	 * @param dec
	 * @param asp
	 */
	private void checkWeavabilityOfAspect(Decision dec, AspectContainer asp) {
		if (dec.getDecisionValue() != null) {
			if (dec.getDecisionValue() == true) {
				this.weavableAspectContainers.add(asp);
			}
		}

	}

	
	
	/**
	 * This method extends the existing weaving semantics by handling {@link Decision}s
	 * between {@link AspectContainer} and {@link ZoomableNodeContainer}. 
	 * The weaving semantic specifies that in such a case the content of the 
	 * {@link AspectContainer} simply gets copied into the target {@link ZoomableNodeContainer}.
	 * 
	 * The copying is implemented using the  
	 * 
	 * @param dec
	 * @return true if the {@link Decision} got woven.
	 */
	private boolean handleDecisions2ACs(Decision dec){
		
		boolean handledDec = false;
		
		for (JoinRelation jr : dec.getJoinRelations()) {
			
			if (!(jr.getSource() instanceof AspectContainer) || 
				!(jr.getTarget() instanceof ZoomableNodeContainer)) { 
				continue;
			}
			
			handledDec = true;
		
			if (dec.getDecisionValue() != null && 
				dec.getDecisionValue()) {
				
				decToACweave(jr);
			
			} else if (dec.getDecisionValue() == null){
				decToACunweave(jr);
			} 
		}			
		return handledDec;
	}	
	
	/**
	 * Used to Weave an {@link AspectContainer} which has a {@link Decision} directly
	 * attached to it.
	 * 
	 * @param jr The {@link JoinRelation} with the {@link Decision}.
	 */
	private void decToACweave(JoinRelation jr) {

		AspectContainer sourceAC;
		ZoomableNodeContainer target;
		
		Decision dec = jr.getDecision();
		
		sourceAC = (AspectContainer)jr.getSource();
		target = (ZoomableNodeContainer)jr.getTarget();
		
		if (dec.getDecisionValue() != null && 
			dec.getDecisionValue()) {
		
			IWorkbenchPart part = AdoraPlugin.getActiveEditor();
			AdoraCopyAction copyAction = new AdoraCopyAction(part);
			copyAction.weaveContent(sourceAC);
			
			//store woven elements in 
			AdoraPasteAction pasteAction = new AdoraPasteAction(part);
			List<Node> wovenElements = pasteAction.weaveContent(target);
			dec.setWovenElements(wovenElements);
			
			//----------
			
			//check if AC needs to be deleted
			Iterator<ModelElement> itr = sourceAC.getRecursiveChildrenIterator();
			boolean del;
			
			while (itr.hasNext()) {
				
				del = true;
				
				Node n = (Node)itr.next();
				if ((n instanceof ZoomableNodeContainer) && 
				  !((ZoomableNodeContainer)n).getAllSourceJRs().isEmpty()) {
					
					continue;
					
				}
				
				for (Connection c : n.getSourceConnections()) {
					if (c instanceof JoinRelation && 
						((JoinRelation)c).getDecision() != null) {
						del = false;
						break;
					}
				}			
				
				//check if parents have Decisions other than the current =>
				//we cant delete this node
				IContainer parent = n.getParent();
				while (!(parent instanceof AdoraModel) && del) {
				
					for (Connection c : ((ZoomableNodeContainer)parent).getSourceConnections()) {
						if (c instanceof JoinRelation && 
							((JoinRelation)c).getDecision() != null) {
							del = false;
							break;
						}
					}			
					parent = parent.getParent();
				}
				n.setVisible(!del);
			}
			
			if (sourceAC.getAllSourceJRs().size() == 1) {
				sourceAC.setVisible(false);
			}
		}
	}
		
	
	/**
	 * Used to unweave an {@link AspectContainer} which has a {@link Decision} directly
	 * attached to it.
	 * 
	 * @param jr The {@link JoinRelation} with the {@link Decision}.
	 */
	private void decToACunweave(JoinRelation jr) {
		AspectContainer sourceAC;
		
		sourceAC = (AspectContainer)jr.getSource();
		
		//check if AC needs to be deleted
		Iterator<ModelElement> itr = sourceAC.getRecursiveChildrenIterator();
		
		while (itr.hasNext()) {
			
			Node n = (Node)itr.next();
			n.setVisible(true);	

		}		
	}
}