package ch.unizh.ifi.adora.weaver.engine;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.commands.LayoutOrderingCommand;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;
import ch.unizh.ifi.adora.weaver.engine.analyzer.CrosscuttingNodeGroupList;
import ch.unizh.ifi.adora.weaver.engine.analyzer.WeavingHistory;

/**
 * This class is responsible of weaving the corrcutting concerns.<br>
 * Implemented as a singleton.
 * 
 * @author  Michael Jehle
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 29.06.2008 MJ First Version<br>
 */
public class CrosscuttingConcernWeaver
{
	/**
	 * Weaver Helper contains all Helper Method of this class.
	 */
	private WeaverHelper weaverHelper;
	
	private PluginActivatorHelper pluginActivatorHelper;

	/**
	 * The instance of the weaver.
	 */
	private static CrosscuttingConcernWeaver INSTANCE = null;

	/**
	 * Returns the weaver.
	 * 
	 * @return The instance of the weaver.
	 */
	public static CrosscuttingConcernWeaver getInstance()
	{
		if (INSTANCE == null)
		{
			INSTANCE = new CrosscuttingConcernWeaver();
		}

		return INSTANCE;
	}

	/**
	 * Returns the weaver.
	 * 
	 * @param disableLayoutCorrections
	 *            Parameter which you can set to true if you dont want to have
	 *            layout corrections.
	 * 
	 * @return The instance of the weaver.
	 */
	/**
	 * This aspectContainers consists all AspectContainer, which are useless
	 * after the weaving process and need deletion
	 */
	public static CrosscuttingConcernWeaver getInstance(boolean disableLayoutCorrections)
	{
		if (INSTANCE == null)
		{
			INSTANCE = new CrosscuttingConcernWeaver(disableLayoutCorrections);
		}

		return INSTANCE;
	}

	/**
	 * Singleton.
	 * 
	 * @see Weaver.getInstance()
	 */
	private CrosscuttingConcernWeaver()
	{
		this(false);
	}

	/**
	 * Singleton.
	 * 
	 * @param disableLayoutCorrections
	 *            Parameter which you can set to true if you dont want to have
	 *            layout corrections.
	 * 
	 * @see Weaver.getInstance()
	 */
	private CrosscuttingConcernWeaver(boolean disableLayoutCorrections)
	{
		this.weaverHelper = new WeaverHelper();
		this.pluginActivatorHelper=new PluginActivatorHelper();
		weaverHelper.setDisableLayoutCorrections(disableLayoutCorrections);
	}
	
	/**
	 * Transforms the AdoraModel so that the aspects are weaved into the model.
	 * 
	 * @param AdoraModelEditor
	 *            The AdoraModelEditor with the current model.
	 */
	public boolean transformModel(AdoraModel model)
	{
		try 
		{
		/*	if (pluginActivatorHelper.activateConstraintPluginValidation())
			{
				return false;
			}*/
			
//			if (pluginActivatorHelper.activateSyntaxPluginValidation(editor))
//			{
//				return false;
//			}
			
//			weaverHelper.removeFilters(editor);
			
//			weaverHelper.removeJoinRelations(model);
			weaverHelper.analyzesScenarioInsteads(model, true);

			// Reset the singletons
			WeavingHistory.reset();
			CrosscuttingNodeGroupList.reset();
			weaverHelper.analyzeModel(model, true);
//			weaverHelper.removeActors(model);
			weaverHelper.checkAndConnectServerComponent(model);

			/*
			 * After the weaving process, all the aspect containers are obsolete
			 * and can be deleted.
			 */
			weaverHelper.removeAspectContainers(model, true);
			weaverHelper.duplicateNameCheck(model);

			if (!weaverHelper.isDisableLayoutCorrections())
			{
				FilterHelper.getInstance().applyFilters(model);
			}
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}

		return true;
	}
}