package ch.unizh.ifi.adora.weaver.engine;

import java.beans.PropertyChangeEvent;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Observable;
import java.util.Observer;
import java.util.Stack;
import java.util.logging.Logger;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.jface.preference.IPreferenceStore;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.commands.RerouteChildConnections;
import ch.unizh.ifi.adora.model.commands.SetVariationPointListDecisions;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.INodeContainer;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.representation.Transition;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.model.representation.variability.Decision;
import ch.unizh.ifi.adora.model.representation.variability.VariationPoint;
import ch.unizh.ifi.adora.ui.views.ModelObserverMediator;
import ch.unizh.ifi.adora.productline.ui.views.decisiontable.DecisionTableViewPart;
import ch.unizh.ifi.adora.weaver.commands.SetDecisionVisible;
import ch.unizh.ifi.adora.weaver.engine.analyzer.GroupCloner;

/**
 * 
 * @author Anil Kandrical
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 26.03.2009 AKAN First Version
 * @version $Date: 2010-03-27 23:41:29 +0100 (Sat, 27 Mar 2010) $, $Author: anil
 *          $, $Revision: 5180 $
 */
public class DynamicWeaver extends Observable implements Observer
{
    /**
     * Sets only latest decision per decision number or all decisions
     */
    private boolean oneDecisionPerNumber = true;
    
    /**
     * Indicates whether crosscutting concerns should be woven or not.
     */
    public static boolean mustWeaveCrosscuttingConcerns = false;
    
    private static WeaverHelper weaverHelper;

    public void setOneDecisionPerNumber(boolean oneDecisionPerNumber)
    {
        this.oneDecisionPerNumber = oneDecisionPerNumber;
        notifyDecisionHistoryTable();
    }
    
    public DynamicWeaver() {
    	ModelObserverMediator.getInstance().addObserver(this);
    }

    /**
     * All decisions of the whole weaving process.
     */
    private static List<Decision> allDecisionsForWeaving;
    /**
     * Latest decisions for each decision number
     */
    private static List<Decision> decisionForWeaving;
    
    public static List<Decision> getDecisionForWeaving(){
        return decisionForWeaving;
    }
    // /
    /**
     * decisions with weaving state are stored
     */
    private static HashMap<Integer, Boolean> weavedDecisionHistory;
    // /
    /**
     * For logging error messages and replacing system outs.
     */
    private static final Logger logger = Logger.getLogger(DynamicWeaver.class
            .getName());
    /**
     * Base model for all weaving operation. This is a model with all decision
     * set to "not defined" meaning undecided.
     */
    private static AdoraModel baseAdoraModel;

    public static AdoraModel getBaseAdoraModel()
    {
        return baseAdoraModel;
    }
    public static void  performWeaving(){
        if(mustWeaveCrosscuttingConcerns){
        ModelObserverMediator.getInstance().getAdoraModelEditor().setModel(
                (AdoraModel) baseAdoraModel.modelCopy());
        
        }
        for (int i = 0; i < decisionForWeaving.size(); i++)
        {
            new SetDecisionVisible(decisionForWeaving.get(i));
        }
    }
        
    /**
     * The instance of the dynamic weaver.
     */
    private static DynamicWeaver INSTANCE = null;

    public static void setBaseAdoraModel(AdoraModel model)
    {
        if (isValidBaseModel(model))
        {
            //baseAdoraModel = model.modelCopy();
            decisionForWeaving = new ArrayList<Decision>();
            allDecisionsForWeaving = new ArrayList<Decision>();
            weavedDecisionHistory = new HashMap<Integer, Boolean>();
            intializeWeavedDecisionHistory();
            updateVP();
            logger.info("Adora Base Model copied for Weaving");
        }
    }

    private static void updateVP()
    {
        Command command;
        List<VariationPoint> selectedVPs = AdoraPlugin.getActiveModel()
                .getVariationPoints();
        for (VariationPoint variationPoint : selectedVPs)
        {
            String decs = ",";
            for (Decision dec : variationPoint.getDecisions())
            {
                decs = decs.concat("D".concat(dec.getNr().toString()).concat(
                        ","));
            }
            command = new SetVariationPointListDecisions((String) decs,
                    variationPoint);
            command.execute();
        }

    }

    /**
     * This initializes the weaved decision history to see, which decision is
     * weaved or currently not weaved.
     */
    private static void intializeWeavedDecisionHistory()
    {
        for (Decision d : ModelObserverMediator.getInstance()
                .getAdoraModelEditor().getModel().getDecisions())
        {
            weavedDecisionHistory.put(d.getNr(), false);
        }

    }

    /**
     * Returns the dynamic weaver.
     * 
     * @return The instance of the weaver.
     */
    public static DynamicWeaver getInstance()
    {
        if (INSTANCE == null)
        {
            INSTANCE = new DynamicWeaver();
            weaverHelper = new WeaverHelper();
        }
        if (DecisionTableViewPart.getDecisionTableCellModifier() != null) {
        	DecisionTableViewPart.getDecisionTableCellModifier().addObserver(INSTANCE);
        }
        return INSTANCE;
    }

    /**
     * Fetch all decision, execute them on base model and visualize result in
     * editor page. This also enables easy unweaving.
     */
    public void activateWeaving(ArrayList<Decision> productLineDec)
    {
        try
        {
            weaverHelper.removeFilters(ModelObserverMediator.getInstance()
                    .getAdoraModelEditor());
            DynamicWeaver.intializeWeavedDecisionHistory();
            // weaving intermediate is base model for a start
            
            AdoraModel copiedModel =  (AdoraModel) baseAdoraModel.modelCopy();
            
            ModelObserverMediator.getInstance().getAdoraModelEditor().setModel(copiedModel);
            

            weaverHelper.removeFilters(ModelObserverMediator.getInstance()
                    .getAdoraModelEditor());
           
            
            if (DynamicWeaver.mustWeaveCrosscuttingConcerns)
            {
                //weave cc
                CrosscuttingConcernWeaver ccw = CrosscuttingConcernWeaver.getInstance();
                ccw.transformModel(ModelObserverMediator.getInstance().getAdoraModelEditor().getModel());
            }
            
            //match the copied decisions to their originals
            fetchCopiedDecisions(productLineDec);
            
            
            // store decisions from product line table locally and into
            for (int i = 0; i < productLineDec.size(); i++)
            {
                this.storeProductLineDecision(productLineDec.get(i));
                allDecisionsForWeaving.add(productLineDec.get(i));
            }
            // execute setdecisionvisible command according to a list of
            // decision
            
            //TODO: IV 25.3 commented this out and seems to work well
            ModelObserverMediator.getInstance().getAdoraModelEditor()
                    .getModel().update();
            
            for (int i = 0; i < decisionForWeaving.size(); i++)
            {
                AdoraModel m = ModelObserverMediator.getInstance()
                        .getAdoraModelEditor().getModel();
                for (Decision d : m.getDecisions())
                {
                    if (d.getNr() == decisionForWeaving.get(i).getNr())
                    {
                        d.setDecisionValue(decisionForWeaving.get(i)
                                .getDecisionValue());
                        storeProductLineDecision(d);
                    }
                }
                // update the joinrelations and decisions in the model
                
                //TODO: this update is still needed...why???????
                ModelObserverMediator.getInstance().getAdoraModelEditor()
                       .getModel().update();
                
               SetDecisionVisible setDecVisCmd = new SetDecisionVisible(decisionForWeaving.get(i));

                // weaving state of decision is stored
                if (decisionForWeaving.get(i).getDecisionValue() != null)
                {
                    if (decisionForWeaving.get(i).getDecisionValue())
                    {
                        setWeavingDecisionHistory(decisionForWeaving.get(i),
                                decisionForWeaving.get(i).getDecisionValue());
                    }
                } else
                {
                    setWeavingDecisionHistory(decisionForWeaving.get(i), false);

                }
                
                //IV reconnectWrongJRs
                changeJRtargets(setDecVisCmd.getJRsToChange());
                
                resetACvisibility(setDecVisCmd.getResetVisibilityList());
                
                //recalculate the commonality paths
                AdoraPlugin.getActiveModel().updateCommonlityPahts();
                
                
                FilterHelper.getInstance().applyFilters(
                        AdoraPlugin.getActiveModel());
            }
        } catch (Exception e)
        {
            logger.warning(e.getMessage() + " Dynamic Weaving failed.");
            e.printStackTrace();
        }
        rerouteChildConnection(decisionForWeaving);
    	GroupCloner.original2cloneMap.clear();
    }
    
    /**
     * Matches the copied {@link Decision}s to their originals.
     * 
     * @param productLineDec The {@link Decision}s which get weaved.
     */
    private void fetchCopiedDecisions(ArrayList<Decision> productLineDec) {
    	 //since we copied the model we need to use the new decision object with the same number
        for (int i = 0; i < productLineDec.size(); i++)
        {
        	
            for (Decision dec : ModelObserverMediator.getInstance()
                    .getAdoraModelEditor().getModel().getDecisions())
            {
                if (dec.getNr() == productLineDec.get(i).getNr())
                {
                    if (productLineDec.get(i).getDecisionValue() != null)
                    {
                    	Decision prodLineDec = productLineDec.get(i);
                    	boolean b = productLineDec.get(i)
                                .getDecisionValue();
                        //match the decision from the old adora model to its copy
                        //by searching for the one decision with the same decisionNr.
                        productLineDec.set(i, dec);   
                        if (prodLineDec.wasDecisionSetAutomatically()) {
                            productLineDec.get(i).setDecisionValueAutomatically(b);
                            productLineDec.get(i).setReqIfTrue("");
                            productLineDec.get(i).setReqIfFalse("");
                            
                        } else {
                        	productLineDec.get(i).setDecisionValueManually(b);
                        }
                    } else
                    {
                         productLineDec.set(i, dec);
                         productLineDec.get(i).setDecisionValueManually(null);
                     }
                }
            }
        }
	}

	/**
     * This bugfix-method resets the visibility of wrongly hidden {@link Nodes}s
     * and there parents. 
     * 
     * @param resetVisibilityList A {@link List} of {@link Decision}s which have to be checked 
     * since some of their {@link JoinRelation} sources were  wrongly set to invisible. 
     */
	private void resetACvisibility(List<Decision> resetVisibilityList) {
		for (Decision dec : resetVisibilityList) {
			
			for (JoinRelation jr : dec.getJoinRelations()) {
				
				if (dec.hasJRtruePathToCommonality(jr)) {
					
					Node source = jr.getSource();
					INodeContainer parent = source.getParent();
					Stack<Node> visStack= new Stack<Node>();
					visStack.push(source);
					while (parent != null && (parent instanceof ZoomableNodeContainer)) {
						visStack.push((Node)parent);
						parent = ((ZoomableNodeContainer)parent).getParent();
					}
					
					while (!visStack.empty()) {
						Node n = visStack.pop();
						n.setManuallyHidden(false);
						n.setVisible(true);
					}
						
					jr.updateVisibility();
					
				}
				
			}
		}
	}
	/**
	 * IV: This is a temporary solution for the bug in which  
	 * {@link JoinRelation}s  get connected to the wrong Node.
	 * 
	 * To really correct it the sequence of actions taken in this command
	 * has to be altered. 
	 * @param reconnectList the {@link JoinRelation}s to be reconnected.
	 */
	private void changeJRtargets(List<Object[]> reconnectList) {
		
		for (Object[] o : reconnectList) {
			JoinRelation jr = (JoinRelation)o[0];
			ConnectableElement ce = (ConnectableElement)o[1];
			
			ConnectableElement ceWeaved = GroupCloner.original2cloneMap.get(ce);
			if (ceWeaved == null)
				continue;
			
			//now rereout  jr(x -> ce)  to jr(x -> weaved(ce)) 
			Node jrSource = jr.getSource();
			
			jr.disconnect();
			jr.reconnect(jrSource, ceWeaved);
		}
		reconnectList.clear();
	}
	
  	/**
     * This method serves to store decision and its weaving state. It's needed
     * to avoid accidently analysis of already weaved material.
     * 
     * @param decision
     * @param b
     */
    private void setWeavingDecisionHistory(Decision decision, boolean b)
    {
        // itemnumber is found
        if (weavedDecisionHistory.get(decision.getNr()) != null)
        {
            weavedDecisionHistory.remove(decision.getNr());
            weavedDecisionHistory.put(decision.getNr(), b);
        }
        // item is not found
        else
        {
            weavedDecisionHistory.put(decision.getNr(), b);
        }
    }

    /**
     * Observer Pattern for Dynamic Decision History of Weaving. This assures,
     * that table is updated properly in this view.
     */
    private void notifyDecisionHistoryTable()
    {
        setChanged();
        if (oneDecisionPerNumber)
        {
            super.notifyObservers(DynamicWeaver.decisionForWeaving);
        } else
        {
            super.notifyObservers(DynamicWeaver.allDecisionsForWeaving);
        }
    }

    public boolean isOneDecisionPerNumber()
    {
        return oneDecisionPerNumber;
    }

    /**
     * Decisions of a product line are stored to a seperate table, since the
     * model decisions are set to null in the base model.
     * 
     * @param productLineDec
     */
    private void storeProductLineDecision(Decision productLineDec)
    {
        boolean decisionNotStored = true;
        for (Decision d : DynamicWeaver.decisionForWeaving)
        {
            if (d.getNr() == productLineDec.getNr())
            {
                DynamicWeaver.decisionForWeaving.set(
                        DynamicWeaver.decisionForWeaving.indexOf(d),
                        productLineDec);
                decisionNotStored = false;
            }
        }
        if (decisionNotStored)
        {
            DynamicWeaver.decisionForWeaving.add(productLineDec);
        }
        notifyDecisionHistoryTable();
    }

    /**
     * Check, whether a model can be used as base model. All decisions must be
     * undecided, otherwise it is impossible to use the dynamic weaver
     * appropriately
     * 
     * @param model
     * @return
     */
    private static boolean isValidBaseModel(AdoraModel model)
    {
        for (Decision dec : model.getDecisions())
        {
            if (dec.getDecisionValue() != null)
            {
                return false;
            }
        }
        return true;
    }

    /**
     * For dynamic weaving process we need to check, whether a decision is
     * weaved and irrelevant or not weaved and still relevant. Decision number
     * and weaving state are stored.
     * 
     * @param decNr
     * @return
     */
    public static boolean getWeavedDecisionHistory(int decNr)
    {
        return weavedDecisionHistory.get(decNr);
    }

    /**
     * Decision, which are set in the table from the product line view are
     * updated here.
     */
    @SuppressWarnings("unchecked")
	public void update(Observable o, Object arg) {
    	
    if (arg instanceof PropertyChangeEvent && 
    		((PropertyChangeEvent)arg).getPropertyName().equals(AdoraModel.PRODUCT_DERIVATTION_STARTED)) {
    	
    	if (((Boolean)((PropertyChangeEvent)arg).getNewValue())) {
    	
    		baseAdoraModel = AdoraPlugin.getActiveModel();

    	}
    }
    	
    if (arg instanceof ArrayList<?>) {
	    	
	       ArrayList d = (ArrayList<Decision>) arg;
	        if (d == null)
	        {
	            AdoraPlugin.getActiveEditor().setModel(baseAdoraModel);
	            mustWeaveCrosscuttingConcerns = false;
	            updateVP();
	            decisionForWeaving = new ArrayList<Decision>();
	            allDecisionsForWeaving = new ArrayList<Decision>();
	            weavedDecisionHistory = new HashMap<Integer, Boolean>();
	            intializeWeavedDecisionHistory();
	        }
	        else {
	        	ArrayList<Decision> decisionList = (ArrayList<Decision>) arg;
	         	this.activateWeaving(decisionList);
	        }
	        
	        //refresh the view of the editor  
			AdoraPlugin.setFocus();
    	}
    }
    
    /**
     * Updates the routing of the connections of the involved ModelElements
     * 
     * @param decs The {@link Decision}s which were set.
     */
    private void rerouteChildConnection(List<Decision> decs) {
    	
    	List<ZoomableNodeContainer> zoomParents = new ArrayList<ZoomableNodeContainer>();
    	for (Decision d : decs) {
    		for (JoinRelation jr : d.getJoinRelations()) {
    			
    			Node source = jr.getSource();
				INodeContainer sourceParent = source.getParent();
				if (sourceParent instanceof ZoomableNodeContainer && !zoomParents.contains(sourceParent))  {
					zoomParents.add((ZoomableNodeContainer)sourceParent);
				}
    			
    			ConnectableElement target = jr.getTarget();
    			INodeContainer targetParent = null;
    			if (target instanceof Node) {
    				targetParent = ((Node)target).getParent();
    			} else if  (target instanceof Transition) {
    				targetParent = ((Transition)target).getSource().getParent();
    			}
    			
				if (targetParent != null && 
						targetParent instanceof ZoomableNodeContainer && 
						!zoomParents.contains(targetParent))  {
					zoomParents.add((ZoomableNodeContainer)targetParent);
				}
    		}
    	}
    	
    	CommandStack cmdStack =  ModelObserverMediator.getInstance().
    								getAdoraModelEditor().getCommandStack();

    	for (ZoomableNodeContainer zoomParent :zoomParents) {
    		
    		RerouteChildConnections routeCmd = new RerouteChildConnections(zoomParent);
    		try {
    			cmdStack.execute(routeCmd);
    		} catch(NullPointerException e){} 
    	}
    }
     
  
    /**
     * Tells if the decision history is activated or not.
     * 
     * @return boolean
     */
    public boolean isDecisionHistoryActivated()
    {
        if (DynamicWeaver.baseAdoraModel == null)
        {
            return false;
        }
        
        return true;
    }
}
