package ch.unizh.ifi.adora.weaver.engine;

import java.util.ArrayList;

import ch.unizh.ifi.adora.model.commands.representation.RepresentationStateChange;
import ch.unizh.ifi.adora.model.representation.Actor;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.Association;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.model.representation.State;

/**
 * This class provides the methods for removing and applying filters.
 * 
 * @author Pascal Schoeni
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 22.01.2008 SM First Version
 * @version $Date: 2009-05-07 10:30:42 +0200 (Thu, 07 May 2009) $, $Author: anil $, $Revision: 4195 $
 */
public class FilterHelper {
	/**
	 * Singleton Instance of this class.
	 */
	private static FilterHelper INSTANCE;

	/**
	 * Returns the instance of this class.
	 * 
	 * @return The only instance of this class.
	 */
	public static FilterHelper getInstance() {
		if (INSTANCE == null) {
			INSTANCE = new FilterHelper();
		}

		return INSTANCE;
	}

	/**
	 * Returns the instance of this class.
	 * 
	 * @param reset
	 *            True if the instance is resetted.
	 * @return The only instance of this class.
	 */
	public static FilterHelper getInstance(boolean reset) {
		if (reset || INSTANCE == null) {
			INSTANCE = new FilterHelper();
		}

		return INSTANCE;
	}

	/**
	 * Private constructor because of the singleton.
	 */
	private FilterHelper() {

	}

	/**
	 * This variable stores the filters, which were applied in the origin model.
	 */
	private ArrayList<Class<?>> filters = new ArrayList<Class<?>>();

	/**
	 * This method removes the filters, which are applied in the model.
	 * 
	 * @param model
	 *            The model where the filters are removed.
	 */
	public void removeFilters(AdoraModel model) {
		filters.clear();

		removeFilter(model, AspectContainer.class);
		removeFilter(model, State.class);
		removeFilter(model, Actor.class);
		removeFilter(model, JoinRelation.class);
		removeFilter(model, Association.class);
		removeFilter(model, Scenario.class);
	}

	/**
	 * Removes a filter, if set, from the model and stores a reference in the
	 * filters array.
	 * 
	 * @param model
	 *            The model where the filter will be removed.
	 * @param type
	 *            The filtertype.
	 */
	public void removeFilter(AdoraModel model, Class<?> type) {
		if (model.getViewFilterState(type)) {
			new RepresentationStateChange(type, model).execute();
			// Remember the class which was filtered for applyFilter.
			filters.add(type);
		}
	}

	/**
	 * Applies all the stored filters to the model.
	 * 
	 * @param model
	 *            The model, where the filter will be applied.
	 */
	public void applyFilters(AdoraModel model) {
		for (Class<?> type : filters) {
			new RepresentationStateChange(type, model).execute();
		}
	}
}
