package ch.unizh.ifi.adora.weaver.engine;

import java.util.logging.Logger;

import org.eclipse.ui.PlatformUI;

import ch.unizh.ifi.adora.constraints.ui.ConstraintManageView;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;
import ch.unizh.ifi.adora.ui.views.syntaxerrors.SyntaxChecker;
import ch.unizh.ifi.adora.ui.views.syntaxerrors.SyntaxProblemsView;
/**
* PlugnActivatorHelper class is containing all constraint and syntax plugin. 
* 
* @author Anil Kandrical
* @copyright Requirements Engineering Research Group<br>
*            Department of Informatics<br>
*            University of Zurich<br>
* @history 23.03.2007 AK First Version<br>
* @version $Date: 2009-03-25 11:23:29 +0100 (Wed, 25 Mar 2009) $, $Author: anil $, $Revision: 4089 $
*/
public class PluginActivatorHelper {
	/**
	 * private logger instance.
	 */
	private static final Logger logger = Logger.getLogger(PluginActivatorHelper.class
			.getName());
	
	/**
	 * Try to validate the model with the constraints plugin
	 */
	public boolean activateConstraintPluginValidation(){
		ConstraintManageView constraintManageView = null;
		try {
			constraintManageView = (ConstraintManageView) PlatformUI
					.getWorkbench().getActiveWorkbenchWindow()
					.getActivePage().showView(ConstraintManageView.VIEW_ID);
		} catch (Exception e) {
			logger.warning(e.getMessage());
		}

		if (constraintManageView != null
				&& !constraintManageView
						.setConstraints(ConstraintManageView.CATEGORY_WEAVING)) {
			constraintManageView.setFocus();
		}
		return constraintManageView != null
		&& !constraintManageView
		.setConstraints(ConstraintManageView.CATEGORY_WEAVING);
	}
	
	/**
	 * Before we start weaving, the SyntaxProblemsViews checks if there
	 * are any errors within transitions and/or Functionals
	 * Specifications. If so, the weaving process gets aborted and the
	 * focus is set on the SyntaxProblemsViews.
	 * @param editor 
	 */
	public boolean activateSyntaxPluginValidation(AdoraModelEditor editor) {
		SyntaxProblemsView syntaxProblemsView = null;
		SyntaxChecker checker = null;

		try {

			syntaxProblemsView = (SyntaxProblemsView) PlatformUI
					.getWorkbench().getActiveWorkbenchWindow()
					.getActivePage().showView(SyntaxProblemsView.VIEW_ID);
			checker = new SyntaxChecker(syntaxProblemsView, editor
					.getModel());
			checker.CheckAndInitSyntaxViewForModel(editor.getModel());

		} catch (Exception e) {
			logger.warning(e.getMessage());
		}

		if (syntaxProblemsView != null
				&& syntaxProblemsView.hasWeavingPreventingErrors()) {
			syntaxProblemsView.setFocus();
		}
		return syntaxProblemsView != null&& syntaxProblemsView.hasWeavingPreventingErrors(); 
	}
}
