package ch.unizh.ifi.adora.weaver.engine;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.gef.commands.Command;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.commands.ConnectionDelete;
import ch.unizh.ifi.adora.model.commands.NodeDelete;
import ch.unizh.ifi.adora.model.commands.RemoveNodeVisually;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.variability.Decision;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;
import ch.unizh.ifi.adora.ui.AdoraModelEditorPaletteFactory;
import ch.unizh.ifi.adora.weaver.engine.analyzer.ActorAssimilationAnalyzer;
import ch.unizh.ifi.adora.weaver.engine.analyzer.ModelAnalyzer;
import ch.unizh.ifi.adora.weaver.engine.analyzer.ScenarioInsteadAnalyzer;
import ch.unizh.ifi.adora.weaver.engine.analyzer.ServerComponentAnalyzer;
import ch.unizh.ifi.adora.weaver.engine.analyzer.WeavingTargetList;
import ch.unizh.ifi.adora.weaver.engine.generator.ActorAssimilationCommandGenerator;
import ch.unizh.ifi.adora.weaver.engine.generator.ActorRemoverCommandGenerator;
import ch.unizh.ifi.adora.weaver.engine.generator.CommandController;
import ch.unizh.ifi.adora.weaver.engine.generator.ModelElementRenamer;
import ch.unizh.ifi.adora.weaver.engine.generator.ScenarioInsteadCommandGenerator;
import ch.unizh.ifi.adora.weaver.engine.generator.ServerComponentCommandGenerator;
import ch.unizh.ifi.adora.weaver.engine.generator.ServerComponentReconnectionCommandGenerator;
/**
* WeavingHelper class is containing all methods needed for weaving. 
* 
* @author Anil Kandrical
* @copyright Requirements Engineering Research Group<br>
*            Department of Informatics<br>
*            University of Zurich<br>
* @history 23.03.2007 AK First Version<br>
* @version $Date: 2010-03-27 23:41:29 +0100 (Sat, 27 Mar 2010) $, $Author: vigan $, $Revision: 5180 $
*/
public class WeaverHelper {
	/**
	 * This variable stores the value, if the layout corrections (filter / zoom
	 * adaptions) should be used or not. This is especially for the tests, they
	 * do not need these corrections.
	 */
	private boolean disableLayoutCorrections;

	
	public boolean isDisableLayoutCorrections() {
		return disableLayoutCorrections;
	}

	public void setDisableLayoutCorrections(boolean disableLayoutCorrections) {
		this.disableLayoutCorrections = disableLayoutCorrections;
	}
	
	public void removeAspectContainers(AdoraModel model, boolean onlyCrosscuttingConcerns)
	{
		ArrayList<AspectContainer> aspectContainers = new ArrayList<AspectContainer>();
		
		for (ModelElement element : model.getChildren())
		{
			fetchAspectContainer(element, aspectContainers, onlyCrosscuttingConcerns);
		}
		
		deleteAspectContainers(aspectContainers);
	}
	
	/**
	 * It adds aspectcontainers of the adoramodel, which are empty, to a arraylist for further deletion
	 * @param element
	 * @param aspectContainers 
	 */
	private void fetchAspectContainer(ModelElement element, ArrayList<AspectContainer> aspectContainers, boolean onlyCrosscuttingConcerns)
	{
		//check if modelelement is a component with subcomponents
		if(element.getClass().toString().equals(Component.class.toString()))
		{
			for (ModelElement element2 : ((Component)element).getChildren())
			{
				if (element2 instanceof AspectContainer)
				{
					if (!onlyCrosscuttingConcerns)
					{
						aspectContainers.add((AspectContainer) element2);
					}
					else
					{
						if (!((AspectContainer) element2).isVariability())
						{
							aspectContainers.add((AspectContainer) element2);
						}
					}
					continue;
				}
				else
				{
					//if modelelement is component and has subelements call this method recursively
					fetchAspectContainer(element2, aspectContainers, onlyCrosscuttingConcerns);
				}
			}
		}
		// Check if modelelement is a aspectcontainer
		if (element instanceof AspectContainer)
		{
			if (!onlyCrosscuttingConcerns)
			{
				aspectContainers.add((AspectContainer) element);
			}
			else
			{
				if (!((AspectContainer) element).isVariability())
				{
					aspectContainers.add((AspectContainer) element);
				}
			}
			return;
		}
		
		return;
	}
	/**
	 * Delete empty aspectContainers in the model and visible from the GUI.
	 * @param aspectContainers 
	 */
	public void deleteAspectContainers(ArrayList<AspectContainer> aspectContainers){
		for (AspectContainer aspectContainer : aspectContainers) {
			new RemoveNodeVisually(aspectContainer).execute();
			new NodeDelete(aspectContainer.getParent(), aspectContainer)
					.execute();
		}
	}
	/**
	 * Executes a list of commands.
	 * 
	 * @param commands
	 *            the list of commands to execute
	 */
	public void executeCommands(List<Command> commands) {
		for (Command command : commands) {
		command.execute();
		}
	}
	/**
	 * Checking for duplicate names
	 * @param model
	 */
	public void duplicateNameCheck(AdoraModel model){

		ModelElementRenamer renamer = new ModelElementRenamer();
		List<Command> cmds = renamer.rename(model);

		for (Command cmd : cmds) {
			cmd.execute();
		}

	}
	/**
	 * The Filters will be removed before the model is cloned, because
	 * of the fact that there is not yet a clone for IntervalStructure.
	 */
	public void removeFilters(AdoraModelEditor editor) {
		/*
		 * The Filters will be removed before the model is cloned, because
		 * of the fact that there is not yet a clone for IntervalStructure.
		 */
		if (!disableLayoutCorrections) {
			/*
			 * The Filters will be removed before the model is cloned,
			 * because of the fact that there is not yet a clone for
			 * IntervalStructure.
			 */
			FilterHelper.getInstance(true).removeFilters(editor.getModel());

			/*
			 * The model has to be zoomed out before we can weave.
			 */
			ZoomHelper.getInstance(true).openUp(editor.getModel());
			
			AdoraModelEditorPaletteFactory palette = AdoraPlugin.getActiveEditor().getPaletteFactory();
			if (palette != null) 
				palette.updatePaletteVisisbility(true);


			
		}
	}
	/**
	 * Remove JoinRelations, which are not necessary (not used product
	 * lines).
	 */
	public void removeJoinRelations(AdoraModel model) {
		/*
		 * Remove JoinRelations, which are not necessary (not used product
		 * lines).
		 */
		for (Decision dec : model.getDecisions()) {
			if (dec.getDecisionValue() != null
					&& dec.getDecisionValue() == false) {
				for (JoinRelation jr : dec.getJoinRelations()) {
					new ConnectionDelete(jr).execute();
				}
			}
		}
	}
	
	/**
	 * The model analyzer finds every aspect of an adora model,
	 * subdivides these aspects into their groups and returns finally
	 * WEAVING COMPONENTS which are used by the command generators
	 */
	public void analyzeModel(AdoraModel model, boolean onlyCrosscuttingConcerns)
	{
		ModelAnalyzer modelAnalyzer = new ModelAnalyzer(model, onlyCrosscuttingConcerns);

		while (WeavingTargetList.getInstance().getWeavables()
				.hasMoreElements()) {
			/*
			 * The CommandController creates commands for each weaving
			 * component. These commands are not automatically executed.
			 */
			CommandController commandController = new CommandController(
					modelAnalyzer);

			// Execution of the generated commands
			executeCommands(commandController.getCommands());

			modelAnalyzer = new ModelAnalyzer(model, onlyCrosscuttingConcerns);
		}
	};
	
	/**
	 * Analyzes the model for actors with outgoing joinrelations and removes all actors without connection
	 * @param model
	 */
	public void removeActors(AdoraModel model)
	{
		/*
		 * Analyzes the model for actors with outgoing joinrelations.
		 */
		ActorAssimilationAnalyzer actorAssimilationAnalyzer = new ActorAssimilationAnalyzer(
				model);

		while (actorAssimilationAnalyzer.hasMoreActorsToAssimilate()) {
			ActorAssimilationCommandGenerator aacg = new ActorAssimilationCommandGenerator(
					actorAssimilationAnalyzer.getActorsToAssimilate());
			executeCommands(aacg.getWeaveCommands());
		}

		/*
		 * Now remove all actors, which have no connections.
		 */
		executeCommands(new ActorRemoverCommandGenerator(
				actorAssimilationAnalyzer.getActors()).getWeaveCommands());

	}

	/**
	 * ScenarioInsteadAnalyzer
	 */
	
	public void analyzesScenarioInsteads(AdoraModel model, boolean onlyCrosscuttingConerns)
	{
		/*
		 * ScenarioInsteadAnalyzer
		 */
		ScenarioInsteadAnalyzer scenarioInsteadAnalyzer = new ScenarioInsteadAnalyzer(model, onlyCrosscuttingConerns);

		executeCommands(new ScenarioInsteadCommandGenerator(
				scenarioInsteadAnalyzer).getWeaveCommands());
		
	}
	
	public void checkAndConnectServerComponent(AdoraModel model)
	{
		/*
		 * Server components may have associations to aspect containers.
		 * these associations must be analyzed and maybe there are some
		 * renamings.
		 */
		ServerComponentAnalyzer serverComponentAnalyzer = new ServerComponentAnalyzer(model);
		/*
		 * Make connections from the server components to the components.
		 */
		ServerComponentReconnectionCommandGenerator scrcg = new ServerComponentReconnectionCommandGenerator(
				serverComponentAnalyzer);
		executeCommands(scrcg.getWeaveCommands());

		/*
		 * Now it is time to check if there were some server components with
		 * associations.
		 */
		ServerComponentCommandGenerator serverComponentGenerator = new ServerComponentCommandGenerator(
				serverComponentAnalyzer);
		executeCommands(serverComponentGenerator.getWeaveCommands());	
	}
}
