package ch.unizh.ifi.adora.weaver.engine;

import ch.unizh.ifi.adora.model.ModelElement;
import ch.unizh.ifi.adora.model.commands.RecursiveFisheyeZoom;
import ch.unizh.ifi.adora.model.representation.AdoraModel;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;

/**
 * This class provides the helper method which zooms out an adora model.
 * 
 * @author Pascal Schoeni
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 22.01.2008 SM First Version
 * @version $Date: 2008-01-23 14:09:06 +0100 (Wed, 23 Jan 2008) $, $Author: schoeni $, $Revision: 3487 $
 */
public class ZoomHelper {
	/**
	 * Singleton Instance of this class.
	 */
	private static ZoomHelper INSTANCE;

	/**
	 * Returns the instance of this class.
	 * 
	 * @return The only instance of this class.
	 */
	public static ZoomHelper getInstance() {
		if (INSTANCE == null) {
			INSTANCE = new ZoomHelper();
		}

		return INSTANCE;
	}

	/**
	 * Returns the instance of this class.
	 * 
	 * @param reset
	 *            True if the instance is resetted.
	 * @return The only instance of this class.
	 */
	public static ZoomHelper getInstance(boolean reset) {
		if (reset || INSTANCE == null) {
			INSTANCE = new ZoomHelper();
		}

		return INSTANCE;
	}

	/**
	 * Private constructor because of the singleton.
	 */
	private ZoomHelper() {

	}

	/**
	 * This method zooms the whole model out.
	 * 
	 * @param model
	 *            The model which will be zoomed out.
	 */
	public void openUp(AdoraModel model) {
		for (ModelElement modelElement : model.getChildren()) {
			if (modelElement instanceof ZoomableNodeContainer) {
				RecursiveFisheyeZoom zoom = new RecursiveFisheyeZoom(true);
				zoom.setTarget((ZoomableNodeContainer) modelElement);
				zoom.execute();
			}
		}
	}
}
