package ch.unizh.ifi.adora.weaver.engine.analyzer;

import ch.unizh.ifi.adora.model.representation.Actor;
import ch.unizh.ifi.adora.model.representation.Association;
import ch.unizh.ifi.adora.model.representation.Scenario;

/**
 * This class is a container for the information which is needed to reconnect an
 * actor with the association to the corrent cloned one.
 * 
 * @author Pascal Schoeni
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 21.01.2008 PS First Version
 * @version $Date: 2008-01-21 13:11:17 +0100 (Mon, 21 Jan 2008) $, $Author: schoeni $, $Revision: 3456 $
 */
public class ActorScenarioAssociationContainer {
	/**
	 * The scenario.
	 */
	private Scenario scenario;

	/**
	 * The association which connects the scenario and the actor.
	 */
	private Association association;

	/**
	 * The actor.
	 */
	private Actor actor;

	/**
	 * Creates a new container for the actor scenario connection.
	 * 
	 * @param scenario
	 *            The scenario.
	 * @param association
	 *            The association.
	 * @param actor
	 *            The actor.
	 */
	public ActorScenarioAssociationContainer(Scenario scenario, Association association, Actor actor) {
		this.scenario = scenario;
		this.association = association;
		this.actor = actor;
	}

	/**
	 * @return The actor.
	 */
	public Actor getActor() {
		return actor;
	}

	/**
	 * @return The association.
	 */
	public Association getAssociation() {
		return association;
	}

	/**
	 * @return The scenario.
	 */
	public Scenario getScenario() {
		return scenario;
	}

}
