package ch.unizh.ifi.adora.weaver.engine.analyzer;

import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.util.NodeGroup;

/**
 * A specialized {@link GroupCloner} which has a {@link JoinRelation}.
 * 
 * @author Pascal Schoeni
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 11.12.2007 PS First Version
 * @version $Date: 2008-01-21 10:43:52 +0100 (Mon, 21 Jan 2008) $, $Author: schoeni $, $Revision: 3454 $
 */
public class ChunkCloner extends GroupCloner {
	/**
	 * The {@link JoinRelation} which connects the chunk.
	 */
	private JoinRelation joinRelation;

	/**
	 * This is the entry point of a {@link NodeGroup} with a
	 * {@link JoinRelation}.
	 */
	private Node entryPoint;

	/**
	 * Creates a clone of the given {@link NodeGroup}.
	 * 
	 * @param nodeGroup
	 * @param joinRelation
	 */
	public ChunkCloner(NodeGroup nodeGroup, JoinRelation joinRelation) {
		super(nodeGroup);

		this.joinRelation = joinRelation;

		this.determineEntryPoint();
	}

	/**
	 * calcs the start node
	 */
	private void determineEntryPoint() {
		for (int i = 0; i < getNodeGroup().getNodes().size(); i++) {
			if (getNodeGroup().getNodes().get(i).equals(this.getJoinRelation().getSource())) {
				this.entryPoint = this.getClonedNodes().get(i);
				return;
			}
		}

		throw new IllegalStateException("start node not found");
	}

	/**
	 * Returns the {@link AspectContainer}, which is the parentnode of the
	 * source of the {@link JoinRelation}.
	 * 
	 * @return AspectContainer The parent of the source of the JoinRelation.
	 */
	public AspectContainer getAspectContainer() {
		return (AspectContainer) this.joinRelation.getSource().getParent();
	}

	/**
	 * Returns the {@link JoinRelation}.
	 * 
	 * @return {@link JoinRelation}
	 */
	public JoinRelation getJoinRelation() {
		return joinRelation;
	}

	/**
	 * Returns the entry point of a nodegroup, i.e. the node to which the
	 * joinrelation targets.
	 * 
	 * @return The {@link Node} which is the entry point of a nodegroup.
	 */
	public Node getEntryPoint() {
		return entryPoint;
	}
}
