package ch.unizh.ifi.adora.weaver.engine.analyzer;

import java.util.Comparator;

/**
 * Comparator which compares two {@link GroupCloner} objects based on the
 * priority of the joinrelation.
 * 
 * @author Pascal Schoeni
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 27.11.2007 PS First Version<br>
 *          10.12.2007 PS Comments improved.
 * @version $Date: 2009-02-28 21:33:21 +0100 (Sat, 28 Feb 2009) $, $Author: anil $, $Revision: 4037 $
 */
public class ChunkClonerComparator implements Comparator<ChunkCloner> {
	
	/**
	 * The two order possiblities, ASC and DESC order.
	 */
	public static final int ASC = 0, DESC = 1;
	
	/**
	 * The current order used in this comparator.
	 */
	private int order;

	/**
	 * Creates a ChunkClonerComparator with either ASC or DESC ordering.
	 * @param order ASC or DESC
	 */
	public ChunkClonerComparator(int order) {
		if (order == ASC || order == DESC)
			this.order = order;
		else
			this.order = ASC;
	}
	
	/**
	 * Comparates the two GroupClones.
	 */
	public int compare(ChunkCloner o1, ChunkCloner o2) {
		if (o1.getJoinRelation().getPriority() < o2.getJoinRelation().getPriority()) {
			return (this.order == ASC) ? -1 : 1;
		} else {
			return (this.order == ASC) ? 1 : -1;
		}
	}
}
