package ch.unizh.ifi.adora.weaver.engine.analyzer;

import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.model.util.NodeGroup;

/**
 * Defines a group of nodes which belong to a crosscutting statechart or a
 * crosscutting scenariochart in an aspect group which has to get woven into the
 * target module. The behaviour chunks may not have any join relation leaving
 * the aspect group (by definition).
 * 
 * @author Pascal Schoeni
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 03.10.2007 MV First Version 08.10.2007 MV Comments added and
 *          reformatted 09.10.2007 MV Changed to handle multiple beaviour chunks
 * @version $Date: 2007-12-10 11:19:21 +0100 (Mon, 10 Dec 2007) $, $Author: schoeni $, $Revision: 3373 $
 * 
 */
public class CrosscuttingNodeGroup extends NodeGroup {
	/**
	 * The source container of the nodes
	 */
	private ZoomableNodeContainer sourceContainer;

	/**
	 * Creates a {@link CrosscuttingNodeGroup}.
	 * 
	 * @param nodeGroup
	 *            The base {@link NodeGroup}.
	 * @param sourceContainer
	 *            The container where this {@link NodeGroup} was placed.
	 */
	public CrosscuttingNodeGroup(NodeGroup nodeGroup, ZoomableNodeContainer sourceContainer) {
		super(nodeGroup.getNodes());

		this.sourceContainer = sourceContainer;
	}

	/**
	 * Returns the source container.
	 * 
	 * @return ZoomableNodeContainer The container where this
	 *         {@link CrosscuttingNodeGroup} was initially placed.
	 */
	public ZoomableNodeContainer getSourceContainer() {
		return this.sourceContainer;
	}
}
