package ch.unizh.ifi.adora.weaver.engine.analyzer;

import java.util.ArrayList;

/**
 * A list of crosscutting node groups to get weaved later on.
 * 
 * @author Pascal Schoeni
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 03.10.2007 PS First Version 08.10.2007 PS comments added 27.11.2007
 *          MV comments refactored
 * @version $Date: 2007-12-10 11:19:21 +0100 (Mon, 10 Dec 2007) $, $Author: schoeni $, $Revision: 3373 $
 * 
 */
public class CrosscuttingNodeGroupList {
	/**
	 * Instance of this class, stores the {@link CrosscuttingNodeGroup}'s.
	 */
	private static CrosscuttingNodeGroupList INSTANCE;

	/**
	 * Returns the instance of this class.
	 * 
	 * @return Instance
	 */
	public static CrosscuttingNodeGroupList getInstance() {
		if (INSTANCE == null) {
			INSTANCE = new CrosscuttingNodeGroupList();
		}

		return INSTANCE;
	}

	/**
	 * Resets the instance. All data within the list of components will get
	 * lost.
	 */
	public static void reset() {
		INSTANCE = new CrosscuttingNodeGroupList();
	}

	/**
	 * List uf {@link CrosscuttingNodeGroup}'s.
	 */
	private ArrayList<CrosscuttingNodeGroup> crosscuttingNodeGroups = new ArrayList<CrosscuttingNodeGroup>();

	/**
	 * Private constructor because of the singleton.
	 */
	private CrosscuttingNodeGroupList() {
	}

	/**
	 * Adds a {@link CrosscuttingNodeGroup} to the list.
	 * 
	 * @param crosscuttingNodeGroup
	 */
	public void addCrosscuttingNodeGroup(CrosscuttingNodeGroup crosscuttingNodeGroup) {
		this.crosscuttingNodeGroups.add(crosscuttingNodeGroup);
	}

	/**
	 * Returns the list of all stored {@link CrosscuttingNodeGroup}'s.
	 * 
	 * @return ArrayList<CrosscuttingNodeGroup> List of all stored
	 *         {@link ArrayList<CrosscuttingNodeGroup>}'s.
	 */
	public ArrayList<CrosscuttingNodeGroup> getCrosscuttingNodeGroups() {
		return this.crosscuttingNodeGroups;
	}
}
