package ch.unizh.ifi.adora.weaver.engine.analyzer;

import java.util.ArrayList;
import java.util.HashMap;

import ch.unizh.ifi.adora.model.factories.ModelObjectMapper;
import ch.unizh.ifi.adora.model.representation.Node;
import ch.unizh.ifi.adora.model.util.NodeGroup;

/**
 * Creates a clone of a group of nodes based on the target of a join relation.
 * 
 * @author Pascal Schoeni
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 29.06.2007 PS First Version<br>
 *          19.09.2007 PS Comments reformatted and completed.<br>
 *          08.10.2007 MV Added the functionality to use a group clone without a
 *          join relation
 * @version $Date: 2010-01-04 02:08:38 +0100 (Mon, 04 Jan 2010) $, $Author: vigan $, $Revision: 5039 $
 * 
 */
public class GroupCloner {
	/**
	 * The source {@link NodeGroup}.
	 */
	private NodeGroup nodeGroup;

	/**
	 * A list of the cloned nodes.
	 */
	private ArrayList<Node> clonedNodes = new ArrayList<Node>();
	
	/**
	 * Realizes a mapping from original nodes to cloned nodes.
	 * This is a (temporary) bugfix in order to reconnect wrong connections 
	 * after weaving.
	 */
	public static HashMap<Node, Node> original2cloneMap = new HashMap<Node, Node>();

	/**
	 * Creates a clone of the given {@link NodeGroup}.
	 * 
	 * @param nodeGroup
	 */
	public GroupCloner(NodeGroup nodeGroup) {
		this.nodeGroup = nodeGroup;

		this.cloneGroup();
	}
	
	/**
	 * Clones the nodegroup into the array of the cloned nodes.
	 */
	private void cloneGroup() {
		ModelObjectMapper mop = ModelObjectMapper.getInstance();
		mop.reinitialize();
		mop.setSelectedObjectsList(nodeGroup.getNodes());

		for (Node node : nodeGroup.getNodes()) {
			try {
				Node clonedNode = (Node) mop.getClone((Node) node);
				original2cloneMap.put((Node)node, clonedNode);
				clonedNodes.add(clonedNode);
			} catch (CloneNotSupportedException e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * @return the nodes
	 */
	public ArrayList<Node> getClonedNodes() {
		return clonedNodes;
	}

	/**
	 * Returns the source {@link NodeGroup}.
	 * 
	 * @return The source {@link NodeGroup}
	 */
	public NodeGroup getNodeGroup() {
		return nodeGroup;
	}
}
