package ch.unizh.ifi.adora.weaver.engine.analyzer;

import java.util.Comparator;

import ch.unizh.ifi.adora.model.geometry.Point;
import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.weaver.engine.generator.AbstractCommandGenerator;
import ch.unizh.ifi.adora.weaver.engine.generator.ScenarioChunkWeavingCommandGenerator;

/**
 * This class encapsulated the target, i.e. a scenario node, of one or more
 * scenario chunks. The scenario chunks are grouped into three groups, denoting
 * the before, instead and after weaving order.
 * 
 * @author Marc Vontobel
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 29.06.2007 PS First Version<br>
 *          19.09.2007 MV Comments reformatted and completed.<br>
 *          27.11.2007 MV Removed redundant code fragments (original vs. target
 *          nodes)<br>
 *          27.11.2007 MV Comments added<br>
 *          17.12.2007 PS Review changes
 * @version $Date: 2008-01-22 11:27:29 +0100 (Tue, 22 Jan 2008) $, $Author: vontobel $, $Revision: 3467 $
 * 
 */
public class ScenarioWeavingTarget extends WeavingTarget {
	/**
	 * The destination of all the join relations in the groupclones of this
	 * class.
	 */
	private Scenario targetNode;

	/**
	 * Makes a new {@link ScenarioWeavingTarget}.
	 * 
	 * @param scenario
	 *            The target scenario.
	 */
	public ScenarioWeavingTarget(Scenario scenario) {
		this.setTargetNode(scenario);
		this.setParent(scenario.getParent());
	}

	/**
	 * Returns the destination of all the join relations.
	 * 
	 * @return The destination of all the join relations in the groupclones of
	 *         this class.
	 */
	public Scenario getTargetNode() {
		return targetNode;
	}

	/**
	 * Sets the destination of all the join relations.
	 * 
	 * @param target
	 *            The destination of all the join relations in the groupclones
	 *            of this class.
	 */
	public void setTargetNode(Scenario target) {
		this.targetNode = target;
	}

	/**
	 * Returns the absolute locations of the target node.
	 * 
	 * @return The point with the coords.
	 */
	public Point getPosition() {
		return this.getTargetNode().getAbsoluteLocation();
	}

	/**
	 * Returns the scenario type of the target {@link Scenario}.
	 * 
	 * @return Scenario type of the target {@link Scenario}.
	 */
	public int getTargetScenarioType() {
		return this.getTargetNode().getScenarioType();
	}

	/**
	 * Creates a new CommandGenerator for ScenarioWeavingComponents
	 * 
	 * @return CommandGenerator ScenarioCommandGenerator
	 */
	@Override
	public AbstractCommandGenerator getCommandGenerator() {
		return new ScenarioChunkWeavingCommandGenerator(this);
	}

	@Override
	protected Comparator<ChunkCloner> getComparator() {
		return new ChunkClonerComparator(ChunkClonerComparator.DESC);
	}
}
