package ch.unizh.ifi.adora.weaver.engine.analyzer;

import java.util.Comparator;

import ch.unizh.ifi.adora.model.representation.Transition;
import ch.unizh.ifi.adora.weaver.engine.generator.AbstractCommandGenerator;
import ch.unizh.ifi.adora.weaver.engine.generator.StateCommandGenerator;

/**
 * This class encapsulates the target, i.e. a transition of one or more behavior
 * chunks. The behavior chunks are grouped into three groups denoting the
 * before, instead and after weaving order.
 * 
 * @author Pascal Schoeni
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 20.11.2007 SM Renamed from StateWeaveComponent to StateWeavingTarget<br>
 *          21.11.2007 PS Removed unused attributes<br>
 *          17.12.2007 PS Review changes
 * @version $Date: 2008-01-22 11:27:29 +0100 (Tue, 22 Jan 2008) $, $Author: vontobel $, $Revision: 3467 $
 * 
 */
public class StateWeavingTarget extends WeavingTarget {
	/**
	 * The transition, which is the "weaving target" of state group weaving.
	 */
	private Transition transition;

	/**
	 * Constructs a new {@link StateWeavingTarget} which is used to weave a
	 * group of behavior chunks.
	 * 
	 * @param transition
	 *            The target transition.
	 */
	public StateWeavingTarget(Transition transition) {
		this.setTransition(transition);
		this.setParent(transition.getSource().getParent());
	}

	/**
	 * Return the transition, on which all join relations point for these
	 * behavior chunks.
	 * 
	 * @return the target {@link Transition}.
	 */
	public Transition getTransition() {
		return transition;
	}

	/**
	 * Sets the transition, which is the target of the join relations.
	 * 
	 * @param transition
	 *            The target {@link Transition}.
	 */
	public void setTransition(Transition transition) {
		this.transition = transition;
	}

	/**
	 * Creates a new CommandGenerator for StateWeavingComponents
	 * 
	 * @return CommandGenerator StateCommandGenerator
	 */
	@Override
	public AbstractCommandGenerator getCommandGenerator() {
		return new StateCommandGenerator(this);
	}
	
	@Override
	protected Comparator<ChunkCloner> getComparator() {
		return new ChunkClonerComparator(ChunkClonerComparator.ASC);
	}
}
