package ch.unizh.ifi.adora.weaver.engine.analyzer;

import java.util.Enumeration;
import java.util.Hashtable;

import ch.unizh.ifi.adora.model.representation.ConnectableElement;
import ch.unizh.ifi.adora.model.representation.JoinRelation;

/**
 * This class stores all {@link WeavingTarget}'s.<br>
 * Implemented as singleton.
 * 
 * @author Pascal Schoeni
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 10.12.2007 PS First Version
 * @version $Date: 2007-12-10 13:46:26 +0100 (Mon, 10 Dec 2007) $, $Author: schoeni $, $Revision: 3379 $
 */
public class WeavingTargetList {
	/**
	 * The singleton instance.
	 */
	private static WeavingTargetList INSTANCE;

	/**
	 * Returns the only instance of this class.
	 * 
	 * @return WeavingTargetList The singleton.
	 */
	public static WeavingTargetList getInstance() {
		if (INSTANCE == null) {
			INSTANCE = new WeavingTargetList();
		}

		return INSTANCE;
	}

	/**
	 * Resets the singleton.
	 */
	public static void reset() {
		INSTANCE = null;
	}

	/**
	 * Hashtable which saves the {@link WeavingTarget}'s.
	 */
	private Hashtable<ConnectableElement, WeavingTarget> weavingTarget = new Hashtable<ConnectableElement, WeavingTarget>();

	/**
	 * Returns all the weavable components from every aspect of the model
	 * 
	 * @return
	 */
	public Enumeration<WeavingTarget> getWeavables() {
		return weavingTarget.elements();
	}

	/**
	 * Returns the {@link WeavingTarget} for a specified
	 * {@link ConnectableElement}.
	 * 
	 * @param element
	 *            The {@link ConnectableElement} which is the target of the
	 *            {@link JoinRelation}.
	 * @return WeavingTarget The {@link WeavingTarget}.
	 */
	public WeavingTarget getWeavingTarget(ConnectableElement element) {
		return weavingTarget.get(element);
	}

	/**
	 * Adds a {@link WeavingTarget} to the list.
	 * 
	 * @param element
	 *            The {@link ConnectableElement} which is the target of the
	 *            {@link JoinRelation}.
	 * @param weavingTarget
	 *            The {@link weavingTarget} to add.
	 */
	public void addWeavingTarget(ConnectableElement element, WeavingTarget weavingTarget) {
		this.weavingTarget.put(element, weavingTarget);
	}
}
