package ch.unizh.ifi.adora.weaver.engine.generator;

import java.util.ArrayList;

import ch.unizh.ifi.adora.model.commands.NodeDelete;
import ch.unizh.ifi.adora.model.commands.RemoveNodeVisually;
import ch.unizh.ifi.adora.model.representation.Actor;

/**
 * This command generator generates commands for all the actor which can be
 * deleted because they dont have a connection.
 * 
 * @author Pascal Schoeni
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 21.11.2007 PS First Version 10.12.2007 MV Changes from Meeting
 * @version $Date: 2007-12-17 11:26:15 +0100 (Mon, 17 Dec 2007) $, $Author: schoeni $, $Revision: 3417 $
 */
public class ActorRemoverCommandGenerator extends AbstractCommandGenerator {
	/**
	 * Sets the list of actors to the one provided as parameter.
	 * 
	 * @param actors
	 *            All actors of a model which can be deleted.
	 */
	public ActorRemoverCommandGenerator(ArrayList<Actor> actors) {
		for (Actor actor : actors) {
			addDeleteCommand(new RemoveNodeVisually(actor));
			addDeleteCommand(new NodeDelete(actor.getParent(), actor));
		}
	}
}
