package ch.unizh.ifi.adora.weaver.engine.generator;

import ch.unizh.ifi.adora.model.commands.AssociationCreate;
import ch.unizh.ifi.adora.model.commands.AssociationDelete;
import ch.unizh.ifi.adora.model.factories.ModelObjectMapper;
import ch.unizh.ifi.adora.model.representation.Actor;
import ch.unizh.ifi.adora.model.representation.Role;
import ch.unizh.ifi.adora.weaver.engine.analyzer.ActorScenarioAssociationContainer;
import ch.unizh.ifi.adora.weaver.engine.analyzer.ActorScenarioAssociationList;

/**
 * This class generates the commands for weaving an associated actor to the
 * node.
 * 
 * @author Pascal Schoeni
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 17.12.2007 PS First Version
 * @version $Date: 2008-01-21 13:11:17 +0100 (Mon, 21 Jan 2008) $, $Author: schoeni $, $Revision: 3456 $
 */
public class ActorScenarioAssociationCommandGenerator extends AbstractCommandGenerator {
	public ActorScenarioAssociationCommandGenerator(ActorScenarioAssociationList asal) {
		ModelObjectMapper.getInstance().reinitialize();

		for (ActorScenarioAssociationContainer asac : asal.getContainers()) {
			AssociationCreate create;

			if (asac.getAssociation().getSource() instanceof Actor) {
				create = new AssociationCreate(asac.getActor());
				create.setTarget(asac.getScenario());
			} else {
				create = new AssociationCreate(asac.getScenario());
				create.setTarget(asac.getActor());
			}

			try {
				if (!"".equals(asac.getAssociation().getRoleFrom().getName())) {
					create.setRoleFrom((Role) asac.getAssociation().getRoleFrom().deepClone());
				}
				if (!"".equals(asac.getAssociation().getRoleTo().getName())) {
					create.setRoleTo((Role) asac.getAssociation().getRoleTo().deepClone());
				}
			} catch (CloneNotSupportedException e) {
				e.printStackTrace();
			}

			GeneratorHelper.setDefaultClickPoints(create);
			addWeaveCommand(create);

			AssociationDelete adcmd = new AssociationDelete(asac.getAssociation());
			addDeleteCommand(adcmd);
		}
	}

}
