package ch.unizh.ifi.adora.weaver.engine.generator;

import java.util.ArrayList;

import ch.unizh.ifi.adora.model.representation.AspectContainer;
import ch.unizh.ifi.adora.model.representation.Association;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.Role;

/**
 * Groups several associations to a group. provides a method to check, if a
 * certain association is one of this "group". associations are grouped with the
 * help of the role names which have to be equal.
 * 
 * @author Pascal Schoeni
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 21.01.2008 PS Review changes
 * @version $Date: 2008-01-22 09:26:31 +0100 (Tue, 22 Jan 2008) $, $Author: schoeni $, $Revision: 3464 $
 */
class AssociationGroup {
	private ArrayList<Association> associations = new ArrayList<Association>();

	private Role roleFromServerComponent;

	private Role roleFromAspect;

	public AssociationGroup(Component component, Association association) {
		add(association);

		if (component.equals(association.getSource())) {
			roleFromServerComponent = association.getRoleFrom();
			roleFromAspect = association.getRoleTo();
		} else {
			roleFromServerComponent = association.getRoleTo();
			roleFromAspect = association.getRoleFrom();
		}
	}

	/**
	 * Checks, if the name of the roles on the {@link Association}'s matches
	 * each.
	 * 
	 * @param association
	 * @return
	 */
	public boolean matches(Association association) {
		return (roleFromServerComponent.getName().equals(association.getRoleFrom().getName()) && roleFromAspect
				.getName().equals(association.getRoleTo().getName()))
				|| (roleFromAspect.getName().equals(association.getRoleFrom().getName()) && roleFromServerComponent
						.getName().equals(association.getRoleTo().getName()));
	}

	/**
	 * Adds a new {@link Association}.
	 * 
	 * @param association
	 *            The {@link Association} to add.
	 */
	public void add(Association association) {
		associations.add(association);
	}

	/**
	 * Returns all {@link Association}'s saved.
	 * 
	 * @return the {@link Association}.
	 */
	public ArrayList<Association> getAssociations() {
		return associations;
	}

	/**
	 * Returns the {@link Role} from the {@link AspectContainer}.
	 * 
	 * @return the {@link Role} from the {@link AspectContainer}.
	 */
	public Role getRoleFromAspect() {
		return roleFromAspect;
	}

	/**
	 * @return the roleFromServerComponent
	 */
	public Role getRoleFromServerComponent() {
		return roleFromServerComponent;
	}

}