package ch.unizh.ifi.adora.weaver.engine.generator;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.gef.commands.Command;
/**
 * This class stores commands of a command generator. There are two possible 
 * command lists: WeaveCommands and DeleteCommands which only differ in the
 * order they are getting executed. Any command added by addWeaveCommand will
 * be executed before any other command added by addDeleteCommand.
 * 
 * @author 		Marc Vontobel
 * @copyright 	Requirements Engineering Research Group<br>
 *            	Department of Informatics<br>
 *            	University of Zurich<br>
 * @history 	10.12.2007 MV First Version
 * @version 	$Date: 2007-12-10 10:05:40 +0100 (Mon, 10 Dec 2007) $, $Author: vontobel $, $Revision: 3367 $
 */
public class CommandList {
	/**
	 * A list of commands which delete nodes. this list is processed after all
	 * weaveCommands.
	 */
	private List<Command> deleteCommands = new ArrayList<Command>();

	/**
	 * A list of commands used for the weaving.
	 */
	private List<Command> weavingCommands = new ArrayList<Command>();

	/**
	 * Adds a delete command to the stack of delete commands. Any
	 * command which is added by this method will be executed after
	 * any other command added by the addWeaveCommand method.
	 * 
	 * @param command Any command used for the weaving process.
	 */
	public void addDeleteCommand(Command command) {
		deleteCommands.add(command);
	}

	/**
	 * Adds a weaving command to the stack of weave commands. Any
	 * command which is added by this method will be executed before
	 * any other command added by the addDeleteCommand method.
	 * 
	 * @param command Any command used for the weaving process.
	 */
	public void addWeaveCommand(Command command) {
		weavingCommands.add(command);
	}
	/**
	 * Returns a list of all commands which are used for the weaving. The order
	 * of the commands is roughly explained as all weavingCommands followed by
	 * all deleteCommands.
	 * 
	 * @return list of commands for the weaving, first the "weaving" commands,
	 *         then the delete commands.
	 */
	public List<Command> getWeaveCommands() {
		List<Command> commands = new ArrayList<Command>(weavingCommands);

		commands.addAll(deleteCommands);

		return commands;
	}
}
