package ch.unizh.ifi.adora.weaver.engine.generator;

import java.lang.reflect.Method;
import java.util.TreeSet;

import ch.unizh.ifi.adora.model.commands.InsertNodesByComponent;
import ch.unizh.ifi.adora.model.commands.InsertNodesVisuallyByOrientation;
import ch.unizh.ifi.adora.model.commands.NodeDelete;
import ch.unizh.ifi.adora.model.commands.SetProperty;
import ch.unizh.ifi.adora.model.representation.Component;
import ch.unizh.ifi.adora.model.representation.ISpecificationContainer;
import ch.unizh.ifi.adora.model.representation.VisualizableModelElement;
import ch.unizh.ifi.adora.model.representation.ZoomableNodeContainer;
import ch.unizh.ifi.adora.model.util.ParseTool;
import ch.unizh.ifi.adora.weaver.engine.analyzer.ChunkCloner;
import ch.unizh.ifi.adora.weaver.engine.analyzer.ComponentWeavingTarget;

/**
 * 
 * @author 		Anil Kandrical
 * @copyright 	Requirements Engineering Research Group<br>
 *            	Department of Informatics<br>
 *            	University of Zurich<br>
 * @history 	27.02.2009 AKAN
 */
public class ComponentCommandGenerator extends AbstractCommandGenerator {
	private ComponentWeavingTarget weavingTarget;
	public ComponentCommandGenerator(ComponentWeavingTarget weavingTarget){
		this.weavingTarget=weavingTarget;
		handleBefore();
		handleInstead();
		handleAfter();
	}
	private void handleFunctionalSpecification(ChunkCloner clone) {
		
		String fs1Unparsed = ((ISpecificationContainer) weavingTarget.getTargetNode()).getSpecification();
		String fs2Unparsed = clone.getAspectContainer().getSpecification();
		
		String combinedParsedFS = ParseTool.getCombinedFunctionalSpecification(fs1Unparsed, fs2Unparsed, clone.getAspectContainer().getName());
		
		Method setMethod;
		Method getMethod;

		try {
			setMethod = ISpecificationContainer.class.getMethod("setSpecification", String.class);
			getMethod = ISpecificationContainer.class.getMethod("getSpecification");
		} catch (Exception e) {
			return;
		}

		SetProperty setProperty = new SetProperty((VisualizableModelElement) weavingTarget.getTargetNode(), combinedParsedFS, setMethod, getMethod);
		addWeaveCommand(setProperty);
		
	}
	private void handleAfter() {
		TreeSet<ChunkCloner> chunkCloners = this.weavingTarget.getAfter();
		for(ChunkCloner chunkCloner : chunkCloners){
			handleFunctionalSpecification(chunkCloner);
			addWeaveCommand(new InsertNodesByComponent((ZoomableNodeContainer)weavingTarget.getTargetNode() ,chunkCloner.getClonedNodes(),(Component)chunkCloner.getJoinRelation().getTarget()));
		}
	}

	private void handleInstead() {
		TreeSet<ChunkCloner> chunkCloners = this.weavingTarget.getInstead();
			for(ChunkCloner chunkCloner : chunkCloners){
				addWeaveCommand(new InsertNodesByComponent((ZoomableNodeContainer)weavingTarget.getTargetNode().getParent() ,chunkCloner.getClonedNodes(),(Component)chunkCloner.getJoinRelation().getTarget()));
				addWeaveCommand(new NodeDelete((ZoomableNodeContainer)this.weavingTarget.getTargetNode().getParent(), this.weavingTarget.getTargetNode()));
			}
		
	}

	private void handleBefore() {
		TreeSet<ChunkCloner> chunkCloners = this.weavingTarget.getBefore();
		for(ChunkCloner chunkCloner : chunkCloners){
			addWeaveCommand(new InsertNodesByComponent((ZoomableNodeContainer)weavingTarget.getTargetNode() ,chunkCloner.getClonedNodes(),(Component)chunkCloner.getJoinRelation().getTarget()));
				
		}
	}
}

