package ch.unizh.ifi.adora.weaver.engine.generator;

import java.util.Comparator;
import java.util.Iterator;
import java.util.List;
import java.util.TreeSet;

import ch.unizh.ifi.adora.model.commands.ConnectionDelete;
import ch.unizh.ifi.adora.model.representation.Connection;
import ch.unizh.ifi.adora.model.representation.JoinRelation;
import ch.unizh.ifi.adora.model.representation.Scenario;
import ch.unizh.ifi.adora.weaver.engine.analyzer.ScenarioInsteadAnalyzer;

/**
 * This command generator generates the command to delete all joinrelations
 * except the one which is used for an instead connection between scenarios.
 * 
 * @author Pascal Schoeni
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 28.02.2008 PS First Version.<br>
 * @version $Date: 2009-04-16 10:09:32 +0200 (Thu, 16 Apr 2009) $, $Author: anil $, $Revision: 4102 $
 */
public class ScenarioInsteadCommandGenerator extends AbstractCommandGenerator {
	/**
	 * Creates a new instance of this class.
	 * 
	 * @param scenarioInsteadAnalyzer
	 */
	public ScenarioInsteadCommandGenerator(
			ScenarioInsteadAnalyzer scenarioInsteadAnalyzer) {
		generateCommands(scenarioInsteadAnalyzer.getScenariosToHandle());
	}

	/**
	 * Generates the commands for deleting all the {@link JoinRelation}'s which
	 * are not necessary.
	 * 
	 * @param scenariosToHandle
	 */
	private void generateCommands(List<Scenario> scenariosToHandle) {
		for (Scenario scenario : scenariosToHandle) {
			TreeSet<JoinRelation> joinRelations = getInsteadJoinRelations(scenario);
			Iterator<JoinRelation> iterator = joinRelations.iterator();
			iterator.next();
			while (iterator.hasNext()) {
				JoinRelation joinRelation = iterator.next();
				addDeleteCommand(new ConnectionDelete(joinRelation, true));
			}
		}
	}

	/**
	 * Returns a {@link TreeSet} of {@link JoinRelation}'s of the given
	 * {@link Scenario} node. There are sorted by priority.
	 * 
	 * @param scenario
	 *            The source node.
	 * @return A {@link TreeSet} of {@link JoinRelation}'s, sorted by priority.
	 */
	private TreeSet<JoinRelation> getInsteadJoinRelations(Scenario scenario) {
		TreeSet<JoinRelation> joinRelations = new TreeSet<JoinRelation>(
				new Comparator<JoinRelation>() {

					public int compare(JoinRelation o1, JoinRelation o2) {
						return o1.getPriority() > o2.getPriority() ? -1 : 1;
					}

				});

		for (Connection con : scenario.getTargetConnections()) {
			if (con instanceof JoinRelation
					&& ((JoinRelation) con).getWeavingType() == JoinRelation.INSTEAD) {
				joinRelations.add((JoinRelation) con);
			}
		}

		return joinRelations;
	}
}
