package ch.unizh.ifi.adora.weaver.view;

import java.util.List;
import java.util.Observable;
import java.util.Observer;
import java.util.logging.Logger;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.viewers.ColumnWeightData;
import org.eclipse.jface.viewers.TableLayout;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import ch.unizh.ifi.adora.AdoraPlugin;
import ch.unizh.ifi.adora.model.representation.variability.Decision;
import ch.unizh.ifi.adora.ui.AdoraModelEditor;
import ch.unizh.ifi.adora.ui.views.ModelObserverMediator;
import ch.unizh.ifi.adora.weaver.engine.DynamicWeaver;

/**
 * This class is the view, which can be used for showing decision history of
 * dynamic weaving.
 * 
 * @author Anil Kandrical
 * @copyright Requirements Engineering Research Group<br>
 *            Department of Informatics<br>
 *            University of Zurich<br>
 * @history 2009-03-25 AKAN First Version
 * @version $Date: 2010-03-26 18:24:52 +0100 (Fri, 26 Mar 2010) $, $Author: anil
 *          $, $Revision: 5175 $
 */
public class DecisionHistoryTableViewPart extends ViewPart implements
		IPartListener, Observer {
	public static final String VIEW_ID_PRODUCT_LINES = "ch.unizh.ifi.adora.editor.views.VariabilityDecisions";

	public static final String VIEW_ID = "ch.unizh.ifi.adora.weaver.view.DecisionHistoryTableViewPart";

	private static final Logger logger = Logger
			.getLogger(DecisionHistoryTableViewPart.class.getName());
	private Action switchToProductLineViewAction;
	private Action switchToFullDecisionAction;
	private Table decisionTable;
	private TableViewer tableViewer;
	private String[] columnNames = new String[] { "Decision Number", "Decision" };
	private int[] columnWidths = new int[] { 120, 120 };
	private int[] columnWeights = new int[] { 25, 75 };
	private static final String DECISIONNUMBER = "Decision Number";
	private static final String DECISIONVALUE = "Decision";

	/**
	 * Empty constructor which initializes the view.
	 */
	public DecisionHistoryTableViewPart() {
		super();
		DynamicWeaver.getInstance().addObserver(this);
	}

	@SuppressWarnings("static-access")
	public void createPartControl(Composite parent) {
		initTable(parent);
		this.getSite().getPage().addPartListener(this);
		if (!(getSite().getPage().getEditorReferences().length == 0 && getSite()
				.getPage().getEditorReferences() != null)) {
			if (ModelObserverMediator.getInstance().getAdoraModelEditor() != null && 
					ModelObserverMediator.getInstance().getAdoraModelEditor()
					.getModel() != null) {
				for (Decision dec : (ModelObserverMediator.getInstance()
						.getAdoraModelEditor().getModel()).getDecisions()) {
					dec.setDecisionValue(null);
				}
				DynamicWeaver.getInstance().setBaseAdoraModel(
						ModelObserverMediator.getInstance()
								.getAdoraModelEditor().getModel());
			}
		} else {
			decisionTable.setVisible(false);
		}
	}

	/**
	 * Setup of view layout.
	 * 
	 * @param parent
	 */
	private void initTable(Composite parent) {
		this.addChildControl(parent);
		this.decisionTable = new Table(parent, SWT.H_SCROLL | SWT.V_SCROLL
				| SWT.FULL_SELECTION);
		this.decisionTable.setHeaderVisible(true);
		this.decisionTable.setLinesVisible(true);

		TableLayout layout = new TableLayout();
		for (int c = 0; c < this.columnNames.length; c++) {
			layout.addColumnData(new ColumnWeightData(this.columnWeights[c],
					this.columnWidths[c], true));

			TableColumn modelElementcolumn = new TableColumn(decisionTable,
					SWT.LEFT, c);
			modelElementcolumn.setText(this.columnNames[c]);
			modelElementcolumn.setWidth(this.columnWidths[c]);
			modelElementcolumn.setResizable(true);
		}

		decisionTable.setLayout(layout);

		tableViewer = new TableViewer(decisionTable);
		tableViewer.setUseHashlookup(true);
		tableViewer.setColumnProperties(new String[] { DECISIONNUMBER,
				DECISIONVALUE });
		tableViewer.setLabelProvider(new DecisionHistoryTableLabelProvider());
	}

	/**
	 * Creation of toolbar in layout.
	 * 
	 * @param parent
	 */

	private void addChildControl(Composite parent) {
		createToolbarActions(parent);
		createToolBar();
	}

	private void createToolBar() {
		IToolBarManager mgr = getViewSite().getActionBars().getToolBarManager();
		mgr.add(switchToProductLineViewAction);
		mgr.add(switchToFullDecisionAction);
	}

	/**
	 * Button in toolbar which allows to switch to the product lines decision
	 * table.
	 * 
	 * @param parent
	 */
	private void createToolbarActions(Composite parent) {
		switchToProductLineViewAction = new Action(
				"Show Product Lines Decision View") {
			public void run() {
				try {
					PlatformUI.getWorkbench().getActiveWorkbenchWindow()
							.getActivePage().showView(VIEW_ID_PRODUCT_LINES);
				} catch (PartInitException e) {
					logger.warning(e.getMessage());
				}
			}
		};
		switchToFullDecisionAction = new Action("Show All Decision View") {
			public void run() {
				try {
					DynamicWeaver.getInstance().setOneDecisionPerNumber(
							!DynamicWeaver.getInstance()
									.isOneDecisionPerNumber());
				} catch (Exception e) {
					logger.warning(e.getMessage());
				}
			}
		};
	}

	/**
	 * This method sets focus on the current table entry selected.
	 */

	public void setFocus() {
		tableViewer.getControl().setFocus();
	}

	@SuppressWarnings("static-access")
	public void partActivated(IWorkbenchPart part) {
		logger.info("PARTACTIVATED");
		if (part instanceof AdoraModelEditor && !decisionTable.isDisposed()) {
			if (DynamicWeaver.getBaseAdoraModel() == null) {
				DynamicWeaver.getInstance().setBaseAdoraModel(
						((AdoraModelEditor) part).getModel());
			}
			// base model should have always all decision undecided
			
			//IV remember to commented out // causes errors
			//for (Decision dec : DynamicWeaver.getBaseAdoraModel()
			//					.getDecisions()) {
			//	dec.setDecisionValue(null);
			//}
			DynamicWeaver.getInstance().setBaseAdoraModel(
					((AdoraModelEditor) part).getModel());
			
			clearTableWithDecisionCheck();
		}
	}

	private void clearTableWithDecisionCheck() {
		boolean noDecision=true;
		for(Decision decision:AdoraPlugin.getActiveModel().getDecisions()){
			if(decision.getDecisionValue()!=null){
				noDecision=false;
			}
		}
		if(noDecision){
		clearTable();
		}
	}

	/**
	 * Not used!
	 */

	public void partBroughtToTop(IWorkbenchPart part) {
	}

	/**
	 * Table is not shown, if there are no models in the editor.
	 */

	public void partClosed(IWorkbenchPart part) {
		if (part instanceof AdoraModelEditor && !decisionTable.isDisposed()) {

			IEditorReference[] editors = getSite().getPage()
					.getEditorReferences();

			if (editors != null && editors.length == 0) {
				decisionTable.setVisible(false);
			}
		}
	}

	/**
	 * Not used
	 */

	public void partDeactivated(IWorkbenchPart part) {
	}

	/**
	 * Sets table visible, if a model is opened in the editor.
	 */

	public void partOpened(IWorkbenchPart part) {
		if (part instanceof AdoraModelEditor && !decisionTable.isDisposed()) {
			decisionTable.setVisible(true);
		}
	}

	@SuppressWarnings("unchecked")
	public void update(Observable o, Object arg) {

		List<Decision> l = (List<Decision>) arg;
		clearTable();
		for (int i = 0; i < l.size(); i++) {
			tableViewer.add((Decision) l.get(i));
		}

	}

	/**
	 * remove all elements from the table
	 */
	public void clearTable() {
		int tableSize = tableViewer.getTable().getItemCount();
		for (int i = 0; i < tableSize; i++) {
			tableViewer.remove(tableViewer.getElementAt(0));
		}
	}

}
