package ch.unizh.ifi.adora.zoom;

/**
 * Constant definitions for the layout visualization.
 * <p>   
 *
 * @author		         schlaefli
 * @copyright           Requirements Engineering Research Group<br>
 *                       Department of Informatics<br>
 *                       University of Zurich<br>
 * @history             May 17, 2006 JS First Version
 *                      May 31, 2006 JS added CHILD_POSITION_CHANGE
 * @version             $Date: 2008-01-09 11:39:13 +0100 (Wed, 09 Jan 2008) $, $Author: treinhard $, $Revision: 3440 $
 * @responsibilities	 Container for the constants for the layout visualization.
 *
 */
public interface ZoomEventConstants {

    /**
     * Fired when a layout algorithm finds an occlusion between two elements.
     * If this event is fired, getNewValue() returns the occluding 
     * Rectangle.
     */
    public static final String OCCLUSION = "Occlusion";
    /**
     * Fired when a layout algorithm moves a single element.
     * If this event is fired, getNewValue() returns the new position 
     * as a Rectangle, and getOldValue() returns the original position.
     */
    public static final String POSITION_CHANGE = "PositionChange";
    /**
     * Fired when a layout algorithm moves all children of a ZoomableNodeContainer.
     * If this event is fired, getNewValue() returns the new position 
     * as a Rectangle, and getOldValue() returns the original position. The event
     * is fired once for each child.
     */
    public static final String CHILD_POSITION_CHANGE = "ChildPositionChange";
    /**
     * Fired when a layout algorithm gets called from the outside, so that
     * the current auxiliary figures can be discarded.
     */
    public static final String NEW_LAYOUT_OPERATION = "NewLayoutOperation";
    /**
     * Fired when the horizontal intervals of the continous zoom change.
     */
    public static final String INTERVALS_HORIZONTAL = "IntervalHorizontal";
    /**
     * Fired when the vertical intervals of the continuous zoom change.
     */
    public static final String INTERVALS_VERTICAL = "IntervalVertical";
    /**
     * Fired when the zoom holes of the continuous zoom change
     */
    public static final String ZOOM_HOLES = "ZoomHoles";
    
    public static final String START_ZOOM = "StartZoom";
    
    public static final String END_ZOOM = "EndZoom";
}
