package ch.unizh.ifi.adora.zoom.berner.geometry;

import ch.unizh.ifi.adora.model.geometry.Rectangle;




/**
 * This BufferZoneRectangle represents the buffer zone that is needed
 * by some of the layout algorithms to simplify the handling with minimal
 * distance between ConstructionRectangles. 
 * <p>
 *
 * <pre>
 * History:
 * 12.08.02  MM  Class created
 *
 * </pre>
 *
 * @author  $Author: treinhard $
 * @version $Date: 2007-03-14 14:11:05 +0100 (Wed, 14 Mar 2007) $ $Revision: 2527 $
 */
public class BufferZoneRectangle extends ConstructionRectangle {

  /**
	 * 
	 */
	private static final long serialVersionUID = -2766393714914949090L;

	/**
   * The base rectangle of this buffer zone.
   */
  private Rectangle baseRect = null;
  /**
   * The size of the buffer between the base rectangle and the BufferZoneRectangle
   */
  private int delta = 0;

  /**
   * Creates a new BufferZoneRectangle based on a java.awt.Rectangle.
   * @param rect The specified Rectangle.
   */
  public BufferZoneRectangle(Rectangle rect) {
    super(rect);
   
  }
  /**
   * Creates a new BufferZoneRectangle based on a java.awt.Rectangle. The delta
   * value defines the distance between one side of the base rectangle and its
   * corresponding side of the buffer zone.
   * @param rect The specified Rectangle.
   * @param delta The size of the buffer zone.
   */
  public BufferZoneRectangle(Rectangle rect, int delta) {
    super(rect);
    baseRect = rect;
    setBuffer(delta);
  }
  
  /**
   * Sets the size of the buffer zone.
   * @param delta The size of the buffer zone.
   */
  public void setBuffer(int delta) {
    this.setBounds(baseRect);
    this.delta = delta;
    //this.grow(delta, delta);
    this.x = this.x - delta;
    this.y = this.y - delta;
    this.width = this.width  + 2 * delta;
    this.height= this.height + 2 * delta;
  }
  /**
   * Gets the size of the buffer zone.
   * @return The size of the buffer zone.
   */
  public int getBuffer() {
    return delta;
  }
  
}
