package ch.unizh.ifi.adora.zoom.berner.geometry;

import ch.unizh.ifi.adora.model.geometry.Point;
import ch.unizh.ifi.adora.model.geometry.Rectangle;





/**
 * This ConstructionObject represents a rectangle. The class extends the
 * java.awt.Rectangle class by the methods defined in the ConstructionObject
 * interface.
 * <p>
 *
 * <pre>
 * History:
 * 18.07.02  MM  Class created
 * 25.07.02  MM  Added methods for visibility handling
 * 06.08.02  MM  Added methods getCenter, getCorners
 * 19.08.02  MM  Added getOcclusion getExtendedOcclusion
 * 24.08.02  MM  Solved problem with endless loop (no recursion in getExtendedOcclusion)
 *               Changed getExtendedOcclusion to work in any case -> conntainsCorners
 *               not used anymore.
 *
 * </pre>
 *
 *
 * @author  $Author: treinhard $
 * @version $Date: 2007-03-14 14:11:05 +0100 (Wed, 14 Mar 2007) $ $Revision: 2527 $
 */
public class ConstructionRectangle extends Rectangle {

	private static final long serialVersionUID = 8078513057063448934L;

  /**
   * Creates a new ConstructionRectangle.
   * @param rect The specified Rectangle.
   */
  public ConstructionRectangle(Rectangle rect) {
    super(rect);
  }

  /**
   * Returns the four corner Points of this Rectangle.
   * @return An array containing the four corner Points. Position 0 contains the
   * location Point of this rectangle. The other positions contains the last three
   * corner Points of this rectangle ordered clockwise.
   */
  public Point[] getCorners() {
    Point p1 = new Point(getLocation());
    Point p2 = new Point(x+width, y);
    Point p3 = new Point(x+width, y+height);
    Point p4 = new Point(x, y+height);
    Point[] corners = {p1, p2, p3, p4};
    return corners;
  }
  /**
   * Returns the center Point of this Rectangle.
   * @return The center Point.
   */
  public Point getCenter() {
    Point center = new Point((x + width/2), (y + height/2));
    return center;
  }
  /**
   * Returns the OcclusionRectangle between this and the specified ConstructionRectangle
   * if they overlap, null otherwise.
   * @param rect The specified ConstructionRectangle
   * @return The OcllusionRectangle if it exists, null otherwise.
   */
  public ConstructionRectangle getOcclusion(ConstructionRectangle rect) {
	  ConstructionRectangle occlusion = null;
    if (intersects(rect)) {
      occlusion = new ConstructionRectangle(intersection(rect));
    }
    return occlusion;
  }
  /**
   * Returns the extended OcclusionRectangle between this and the specified ConstructionRectangle
   * if they overlap, null otherwise. The returned OcclusionRectangle is the intersection
   * between the two rectangle enhanced by two of the rectangle's corner. Such a
   * extended occlusion is needed to find out the shortest shift vector to solve the occlusion.
   * @param rect The specified ConstructionRectangle
   * @return The extended OcllusionRectangle if it exists, null otherwise.
   */
  public ConstructionRectangle getExtendedOcclusion(ConstructionRectangle rect) {
	  ConstructionRectangle occlusion = getOcclusion(rect);
    if (occlusion != null) {
      Point center = getCenter();
      Point otherCenter = rect.getCenter();
      Point[] corners =  getCorners();
      Point[] otherCorners = rect.getCorners();
      if (otherCenter.y > center.y) {
        if (otherCenter.x > center.x) {
          occlusion.add(corners[2]);
          occlusion.add(otherCorners[0]);
        }
        else {
          occlusion.add(corners[3]);
          occlusion.add(otherCorners[1]);
        }
      }
      else {
        if (otherCenter.x > center.x) {
          occlusion.add(corners[1]);
          occlusion.add(otherCorners[3]);
        }
        else {
          occlusion.add(corners[0]);
          occlusion.add(otherCorners[2]);
        }
      }
    }
    return occlusion;
    
  }
  
  public void add(Rectangle r) {
	  int x1 = Math.min(x, r.x);
	  int x2 = Math.max(x, r.x);
	  int y1 = Math.min(y, r.y);
	  int y2 = Math.max(y + height, r.y + height);
	  x = x1;
	  y = y1;
	  width = x2 - x1;
	  height = y2 - y1;
  }
  
  public void add(int newx, int newy) {
	  int x1 = Math.min(x, newx);
	  int x2 = Math.max(x + width, newx);
	  int y1 = Math.min(y, newy);
	  int y2 = Math.max(y + height, newy);
	  x = x1;
	  y = y1;
	  width = x2 - x1;
	  height = y2 - y1; 
  }
  
  
  public void add(Point p) {
	  add(p.x,p.y);
  }
  
  /**
   * Translate the ConstructionRectangle by the values specified in the Point.
   * Implements ConstructionObject.
   * @param vector The specified Point.
   */
  public void translateObject(Point vector) {
    this.translate(vector.x, vector.y);
  }
}
