package ch.unizh.ifi.adora.zoom.berner.geometry;

import ch.unizh.ifi.adora.model.geometry.Dimension;
import ch.unizh.ifi.adora.model.geometry.Point;



/**
 * Helper methods for calculating the layout.
 *      
 * @author		       smeier
 * @copyright           Requirements Engineering Research Group<br>
 *                      Department of Informatics<br>
 *                      University of Zurich<br>
 * @history             Oct 30, 2005 SM First version.
 *                      Dec  2, 2005 SM Additional method for calculating the 
 *                                      shrunk size.
 *                      Aug 11, 2006 JS Added default size parameter to
 *                                      calcShrunkSize(...)
 * @version             $Date: 2007-03-14 14:11:05 +0100 (Wed, 14 Mar 2007) $, $Author: treinhard $, $Revision: 2527 $
 * @responsibilities    Provides several helper methods for calculating the 
 *                      layout.	
 *
 */
public class LayoutUtils {

	public static final Dimension SHRUNK_SIZE = new Dimension(5,5); 
	
	/**
     * Calculates a new location for this node, which size will be changed from
     * oldSize to newSize, so that the center remains the same.
     *
     * @pre location != null
     * @pre oldSize != null
     * @pre newSize != null
     * @post return != null
     */
    public static Point calcCenteredLocation(Point location, Dimension oldSize, Dimension newSize) {
    		Point p = new Point(location.x + (oldSize.width  - newSize.width)  / 2,
			    location.y + (oldSize.height - newSize.height) / 2);
    		return p;
    }
    
    /**
     * Resizes the given dimension in a proportional way to the shrunk size.
     * 
     * @param d The dimension to resize
     * @return Returns the shrunk size of the given dimension.
     */
    public static Dimension calcShrunkSize(Dimension d, Dimension defaultSize) {
    		int width  = d.width;
    		int height = d.height;
    		//int temp;
            double ratio;
    		Dimension ret;
           if (height > width) {
                ratio=(double)(((double)height)/((double)defaultSize.height));
            }
            else{
                ratio=(double)(((double)width)/((double)defaultSize.width));
            }
            ret = new Dimension((int)(width/ratio),(int)(height/ratio));

            /*         if (height > width) {
    			temp = (height * SHRUNK_SIZE.height) / height;
    			ret = new Dimension(SHRUNK_SIZE.width, temp);
    		} else {
    			temp = (width * SHRUNK_SIZE.width) / width;
    			ret = new Dimension(temp, SHRUNK_SIZE.height);
    		}*/
    		return ret;
    }
    
    
    
}
